#!/bin/sh
# file: build.sh
# build targets including server installer and other binaries

# version file
VERSION_FILE=version
VERSION_MAJOR=`awk -F "=" '/major/{print $2}' $VERSION_FILE | xargs`
VERSION_MINOR=`awk -F "=" '/minor/{print $2}' $VERSION_FILE | xargs`

# name for server installer
TC_RELEASE_NAME=TC_${VERSION_MAJOR}_Release
TC_RELEASE_TARBALL=${TC_RELEASE_NAME}.tar.gz
TCSERVER_INSTALLER_TARBALL=server_installer.tar

TOP_DIR=`pwd`
DL_DIR=$TOP_DIR/3rdParty
MODULE_DIR=$TOP_DIR/module
MODULE_CONFIG_DIR=$TOP_DIR/mod_config
OUTPUT_DIR=$TOP_DIR/output
BUILD_DIR=$OUTPUT_DIR/build
TARGET_DIR=$OUTPUT_DIR/target
SCRIPT_DIR=$TOP_DIR/script
DOC_DIR=$TOP_DIR/doc

# current configration
MODULE_ATTR_FILE=$MODULE_CONFIG_DIR/tc.pro
MODULE_CONFIG_FILE=$MODULE_CONFIG_DIR/mod.config
# Makefile which needed to be adjusted dynamically
MAKEFILE_INC=$MODULE_CONFIG_DIR/Makefile.inc

# commands
CP='/bin/cp -f'
RM='/bin/rm -f'
MV='/bin/mv -f'
MKDIR='/bin/mkdir -p'

# set log info
LOG_FILE='build.log'
LOG_MSG_TYPE='3'
LOG_STEP=
#!/bin/sh
# =============================== tc log (begin) ===============================
# functions: control how messages are print on screen
# tcmsg_init()
# tcmsg_finish()
# tcmsg_exit()
# tcinfo()
# tcerr()
# tcwarn()
# step()
#
# a filename log would be written into
#TCMSG_LOG_FILE=$1
TCMSG_LOG_FILE=$LOG_FILE
# msg type
# 0: none
# 1: steps only
# 2: classified log messages only
# 3: both
#TCMSG_MSG_TYPE=$2
TCMSG_MSG_TYPE=$LOG_MSG_TYPE
# total steps of your task
#TCMSG_STEP=$3
TCMSG_STEP=$LOG_STEP

TCMSG_STEPN=0

tcmsg_init(){
    echo -e "\e[32m[INFO]\e[0m set log file to $TCMSG_LOG_FILE"
    echo -e "\e[32m[INFO]\e[0m set log file to $TCMSG_LOG_FILE" >$TCMSG_LOG_FILE
    exec 3>&1 4>&2
    exec >>$TCMSG_LOG_FILE 2>&1
}

tcmsg_finish(){
    echo -e "\e[32m[INFO]\e[0m save logs to file $TCMSG_LOG_FILE" >&3
    echo -e "\e[32m[INFO]\e[0m save logs to file $TCMSG_LOG_FILE"
    exec 1>&3 2>&4
    exec 3>&- 4>&-
}

#alias tcexit='tcmsg_finish && exit'
tcexit(){
    tcmsg_finish
    exit $1
}

tcinfo(){ # green
    if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -ge 2 ] ; then
        echo -e "\e[32m[INFO]\e[0m $*" >&3
    fi
    echo -e "\e[32m[INFO]\e[0m $*" 
}

tcerr(){ # red
    if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -ge 2 ] ; then
        echo -e "\e[31m[ERROR]\e[0m $*" >&3
    fi
    echo -e "\e[31m[ERROR]\e[0m $*"
}

tcwarn(){ # magenta
    if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -ge 2 ] ; then
        echo -e "\e[35m[WARNING]\e[0m $*" >&3
    fi
    echo -e "\e[35m[WARNING]\e[0m $*"
}

step(){
    local s
    TCMSG_STEPN=$(($TCMSG_STEPN + 1))
    if [ -z "${TCMSG_STEP}" ] || [ -n "${TCMSG_STEP//[0-9]/}" -a ! -z "$TCMSG_STEP" ] ; then
        s=$(printf "%02u" $TCMSG_STEPN)
        if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -eq 1 -o $TCMSG_MSG_TYPE -eq 3 ] ; then
            echo -e "[$s] $*" >&3
        fi
        echo -e "[$s] $*"
    else
        s=$(echo "$TCMSG_STEPN % (1 + $TCMSG_STEP)" | bc)
        s=$(printf "%0${#TCMSG_STEP}u" $s)
        if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -eq 1 -o $TCMSG_MSG_TYPE -eq 3 ] ; then
            echo -e "[$s/$TCMSG_STEP] $*" >&3
        fi
        echo -e "[$s/$TCMSG_STEP] $*"
    fi
}
# ================================ tc log (end) ================================

# tcmsg init
tcmsg_init
tcinfo "init message"

# set PATH
step "check PATH"
CROSS_TOOL_PATH=/root/TC/buildroot-2016.08.1/output/host/usr/bin
echo "$PATH" | grep -q "$CROSS_TOOL_PATH"
if [ $? -eq 0 ] ; then
    tcinfo "adjust \"$CROSS_TOOL_PATH\" to be in front of \$PATH"
    tmp_PATH=`echo "$PATH" | sed -e "s,$CROSS_TOOL_PATH,,g" -e "s,^:,," -e "s,:$,," -e "s,::,:,g"`
else
    tcwarn "\"$CROSS_TOOL_PATH\" is not in \$PATH, add it in the front of \$PATH"
    tmp_PATH=$PATH
fi
export PATH="$CROSS_TOOL_PATH:$tmp_PATH"
unset CROSS_TOOL_PATH
unset tmp_PATH
tcinfo "PATH=$PATH"

step "clean up stuff in build directory, if any"
# empty all stuff in build and target directory
$RM -r $BUILD_DIR/{.[^.],}*
$RM -r $TARGET_DIR/{.[^.],}*

# create directory to save stuff to release
TC_RELEASE_DIR=$TARGET_DIR/$TC_RELEASE_NAME
TC_RELEASE_CLT_INST_DIR=$TC_RELEASE_DIR/client_installer
TC_RELEASE_CLT_WIN_DIR=$TC_RELEASE_CLT_INST_DIR/windows
TC_RELEASE_CLT_LINUX_DIR=$TC_RELEASE_CLT_INST_DIR/linux
TC_RELEASE_SRV_INST_DIR=$TC_RELEASE_DIR/server_installer
TC_RELEASE_TOOL_DIR=$TC_RELEASE_DIR/tool
TC_RELEASE_IMAGE_CREATOR_DIR=$TC_RELEASE_TOOL_DIR/image_export_tool
TC_RELEASE_INITIALIZER_DIR=$TC_RELEASE_TOOL_DIR/initializer
TC_RELEASE_DOC_DIR=$TC_RELEASE_DIR/doc
cd $TARGET_DIR
$MKDIR $TC_RELEASE_DIR
$MKDIR $TC_RELEASE_CLT_INST_DIR 
$MKDIR $TC_RELEASE_CLT_WIN_DIR
$MKDIR $TC_RELEASE_CLT_LINUX_DIR
$MKDIR $TC_RELEASE_SRV_INST_DIR
$MKDIR $TC_RELEASE_TOOL_DIR
$MKDIR $TC_RELEASE_IMAGE_CREATOR_DIR
$MKDIR $TC_RELEASE_INITIALIZER_DIR
$MKDIR $TC_RELEASE_DOC_DIR

# fake target
FAKE_TARGET_NAME=tc_fake_
FAKE_TOP_DIR=$OUTPUT_DIR/${FAKE_TARGET_NAME}

step "clean up stuff in `basename $OUTPUT_DIR`/`basename $FAKE_TOP_DIR` directory, if any"
# empty all generated stuff in fake root directory
$RM -r $FAKE_TOP_DIR
# fill in fake root directory with content in the current top directory
cd $TOP_DIR/.. && $CP -a `basename $TOP_DIR` $FAKE_TARGET_NAME
cd $TOP_DIR/.. && $MV -v $FAKE_TARGET_NAME $OUTPUT_DIR/
tcinfo "create fake target directory `basename $OUTPUT_DIR`/`basename $FAKE_TOP_DIR`"

step "set variables for fake root directory"
# set variable for fake target
FAKE_DL_DIR=$FAKE_TOP_DIR/`basename $DL_DIR`
FAKE_MODULE_DIR=$FAKE_TOP_DIR/`basename $MODULE_DIR`
FAKE_MODULE_CONFIG_DIR=$FAKE_TOP_DIR/`basename $MODULE_CONFIG_DIR`
FAKE_OUTPUT_DIR=$FAKE_TOP_DIR/`basename $OUTPUT_DIR`
FAKE_BUILD_DIR=$FAKE_OUTPUT_DIR/`basename $BUILD_DIR`
FAKE_TARGET_DIR=$FAKE_OUTPUT_DIR/`basename $TARGET_DIR`
FAKE_DOC_DIR=$FAKE_TOP_DIR/`basename $DOC_DIR`
# fake target configrations
FAKE_MODULE_ATTR_FILE=$FAKE_MODULE_CONFIG_DIR/`basename $MODULE_ATTR_FILE`
FAKE_MODULE_CONFIG_FILE=$FAKE_MODULE_CONFIG_DIR/`basename $MODULE_CONFIG_FILE`
FAKE_MAKEFILE_INC=$FAKE_MODULE_CONFIG_DIR/`basename $MAKEFILE_INC`

# remove files from fake target
clean_fake_target(){
    cd $FAKE_TOP_DIR

    tcinfo "remove log files"
    $RM *.log

    tcinfo "remove script build.sh"
    $RM build.sh

    tcinfo "move script install.sh"
    $MV `basename $SCRIPT_DIR`/install.sh .

    #tcinfo "move script loaddata.sh"
    #$MV `basename $SCRIPT_DIR`/loaddata.sh .

    tcinfo "remove directory `basename $SCRIPT_DIR`"
    $RM -r `basename $SCRIPT_DIR`

    tcinfo "remove directory `basename $FAKE_MODULE_CONFIG_DIR`"
    $RM -r $FAKE_MODULE_CONFIG_DIR

    tcinfo "remove directory `basename $FAKE_OUTPUT_DIR`"
    $RM -r $FAKE_OUTPUT_DIR

    tcinfo "remove directory `basename $DOC_DIR`"
    $RM -r $FAKE_DOC_DIR

    tcinfo "remove stuff in fake target directory `basename $FAKE_MODULE_DIR`"
    $RM -r $FAKE_MODULE_DIR/{.[^.],}*
}

step "clean up stuff in fake root"
clean_fake_target


################################################################################
# install binaries
################################################################################
install_binaries()
{
    set -e

    cur_module=bootLoader
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        if ! [ -f $BUILD_DIR/${cur_module}/app/TcBootLoader.bin ] ; then
            tcerr "$BUILD_DIR/${cur_module}/app/TcBootLoader.bin: no such file"
            tcexit 1
        fi
        install -p -m 644 $BUILD_DIR/${cur_module}/app/TcBootLoader.bin $BIN_DIR/BootLoad.bin
    fi

    cur_module=mbr
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 644 $BUILD_DIR/${cur_module}/MBRCode.bin $BIN_DIR/
    fi

    cur_module=kloader
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 644 $BUILD_DIR/${cur_module}/kloader $BIN_DIR/
    fi

    cur_module=tcentry
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tcentry $BIN_DIR/
        install -p $BUILD_DIR/${cur_module}/translations/tcentry_zh_CN.qm $BIN_DIR/
    fi

    cur_module=tcnotifier
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tcnotifier $BIN_DIR/
    fi

    cur_module=gendtag
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/gendtag $BIN_DIR/
    fi

    cur_module=clientControl2
    #b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    #b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    #if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/TcService/tcdclt.lite $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/TcService/shutdown-notify $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/TcService/tcctl.static.and $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/TcService/tcctl.static.linux $BIN_DIR/
    #fi

   cur_module=diskDriverForWindows
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=diskDriverForLinux
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/app/tc-client $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/driver/tcc.ko $BIN_DIR/
    fi

    cur_module=license
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tc-perm $BIN_DIR/
    fi

    cur_module=license2
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/verifylicense $BIN_DIR/
    fi

    cur_module=msock
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tcdctl $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tcdsrv $BIN_DIR/
    fi

    cur_module=syncClient
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tc-sync-client $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tc-sync-client.x86_64 $BIN_DIR/
    fi

    cur_module=syncServer
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/createMdfUdf $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/createMdfUdf.x86_64 $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/merge $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/merge.x86_64 $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tc-disk-ar.x86_64 $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/tc-sync-server $BIN_DIR/
    fi

    cur_module=webconsole
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi
 
    cur_module=tcserver
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=ps
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=imageExportTool
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      #install -p -m 755 -s $BUILD_DIR/${cur_module}/tcu.share $BIN_DIR/
      # install nothing
      :
    fi

    cur_module=ramdisk
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=tcc_setup
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=LiteP2P
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-delivery-core-import $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-delivery-core-server $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-delivery-core-tracker $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-delivery-static-tracker $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-createbitmap $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-fastcat $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-fastcopy $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-wipemudf $BIN_DIR/
        install -p -m 755 -s $BUILD_DIR/${cur_module}/executable/gcc/tc-mudf-core-server $BIN_DIR/

        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/CreateCache $BIN_DIR/CreateCache.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/FastCat $BIN_DIR/FastCat.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/FastCopy $BIN_DIR/FastCopy.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/p2p-create-seed $BIN_DIR/p2p-create-seed.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/p2p-node-lite $BIN_DIR/p2p-node-lite.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/p2p-tracker-server $BIN_DIR/p2p-tracker-server.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/p2p-verify-target $BIN_DIR/p2p-verify-target.client
        install -p -m 755 $BUILD_DIR/${cur_module}/executable/i686-linux-gcc/tc-mudf-client $BIN_DIR/tc-mudf-client.client
    fi

    cur_module=initializer
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      #install -p -m 755 -s $BUILD_DIR/${cur_module}/initializer $BIN_DIR/
      # install nothing
      :
    fi

    cur_module=TCUpdate
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=TcVfs
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=TcVrs
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    cur_module=NtFileUtils
    b_copy=`awk -v mod=$cur_module '$1 == mod {print $3}' $curr_config_file`
    b_comp=`awk -v mod=$cur_module '$1 == mod {print $4}' $curr_config_file`
    if [ "x$b_copy" == "x1" -a "x$b_comp" == "x1" ] ; then
      # install nothing
      :
    fi

    set +e
}



################################################################################
# build tc modules
make_tc_modules(){
    tcinfo "copy modules to `basename $BUILD_DIR`"
    $CP -a $MODULE_DIR/* $BUILD_DIR
    
    # modify Makefile.inc
    tcinfo "Modify Makefiles accroding to `basename $MODULE_ATTR_FILE`"
    SRCMOD=`awk '$3 == 1 {print $1}' $MODULE_ATTR_FILE | tr '\n' ' '`
    sed -i "s@^\(SRCMOD\)\s*:*=.*@\1\ :=\ ${SRCMOD}@" $MAKEFILE_INC

    # copy makefiles
    tcinfo "copy Makefiles from `basename ${MODULE_CONFIG_DIR}`"
    cd $MODULE_CONFIG_DIR
    find . -name 'Makefile*' -exec $CP --parents --remove-destination -t $BUILD_DIR/ '{}' \;
    
    # make all binaries, run `make all'
    tcinfo "make modules"
    cd $BUILD_DIR
    make all
    if ! [ $? -eq 0 ] ; then
        tcerr "command: make all"
        tcexit 1
    fi

:<<BLOCK
    # make install
    make install
    if ! [ $? -eq 0 ] ; then
        tcerr "command: make install"
        tcexit 1
    fi
BLOCK


# ==============================================================================
# ========================== rearrange target (begin) ==========================
# ==============================================================================
    BIN_DIR_NAME=output_bin
    BIN_DIR=$BUILD_DIR/$BIN_DIR_NAME
    curr_config_file=$MODULE_ATTR_FILE

    cd $TOP_DIR

    # copy binaries to output_bin
    # check bulit modules ($3 == 1 && $4 == 1 only)
    install_binaries

set -e
    # copy binaries to destination
    MODULE_RAMDISK_DIR=$BUILD_DIR/ramdisk
    RAMDISK_BIN_DIR=$MODULE_RAMDISK_DIR/bin

    MODULE_TCSERVER_DIR=$BUILD_DIR/tcserver

    cd $BIN_DIR
    #bootLoader
    $CP BootLoad.bin $RAMDISK_BIN_DIR/usr/sbin/tc/
    #mbr
    $CP MBRCode.bin  $RAMDISK_BIN_DIR/usr/sbin/tc/
    #kloader
    $CP kloader $RAMDISK_BIN_DIR/usr/sbin/tc/
    #tcctl_ui
#    $CP tcdclt.lite $RAMDISK_BIN_DIR/usr/sbin/tc/
#    $CP shutdown-notify $RAMDISK_BIN_DIR/usr/sbin/tc/
    #clientControl2
    $CP tcdclt.lite $RAMDISK_BIN_DIR/usr/sbin/tc/
    $CP shutdown-notify $RAMDISK_BIN_DIR/usr/sbin/tc/
    #tcentry
    $CP tcentry $RAMDISK_BIN_DIR/usr/sbin/tc/
    cpio -pdm --quiet $RAMDISK_BIN_DIR/usr/share/l10n/meegotouch/ <<<"tcentry_zh_CN.qm"
    #tcnotifier
    $CP tcnotifier $RAMDISK_BIN_DIR/usr/sbin/tc/
    #gendtag
    $CP gendtag $RAMDISK_BIN_DIR/usr/sbin/tc/
    #license
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-perm
    #license2
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/tc-license | xargs -n 1 $CP verifylicense
    #msock
    # also put another copy to /home/tc/bin
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/tc-client-control | xargs -n 1 $CP tcdctl
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/tc-client-control-server | xargs -n 1 $CP tcdsrv
    #syncClient
    $CP tc-sync-client $RAMDISK_BIN_DIR/usr/sbin/tc/
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/tc-sync-client | xargs -n 1 $CP tc-sync-client.x86_64
    #syncServer
    $CP createMdfUdf $RAMDISK_BIN_DIR/usr/sbin/tc/
    $CP merge $RAMDISK_BIN_DIR/usr/sbin/tc/
    # also put another copy to /home/tc/bin
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/merge | xargs -n 1 $CP merge.x86_64
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/tc-disk-init | xargs -n 1 $CP createMdfUdf.x86_64
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-sync-server
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/tc-disk-ar | xargs -n 1 $CP tc-disk-ar.x86_64
    #imageExportTool
    if [ -f tcu.share ] ; then
      $CP tcu.share $RAMDISK_BIN_DIR/usr/local/bin/
    else
      tcwarn "tcu.share: no such file"
    fi
    #LiteP2P
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-delivery-core-import
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-delivery-core-server
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-delivery-core-tracker
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-delivery-static-tracker
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-createbitmap
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-fastcat
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-fastcopy
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-wipemudf
    echo $MODULE_TCSERVER_DIR/centos7/tcroot/home/tc/bin/ | xargs -n 1 $CP tc-mudf-core-server
    $CP CreateCache.client $RAMDISK_BIN_DIR/usr/sbin/tc/CreateCache
    $CP FastCat.client $RAMDISK_BIN_DIR/usr/sbin/tc/FastCat
    $CP FastCopy.client $RAMDISK_BIN_DIR/usr/sbin/tc/FastCopy
    $CP p2p-create-seed.client $RAMDISK_BIN_DIR/usr/sbin/tc/p2p-create-seed
    $CP p2p-node-lite.client $RAMDISK_BIN_DIR/usr/sbin/tc/p2p-node-lite
    $CP p2p-tracker-server.client $RAMDISK_BIN_DIR/usr/sbin/tc/p2p-tracker-server
    $CP p2p-verify-target.client $RAMDISK_BIN_DIR/usr/sbin/tc/p2p-verify-target
    $CP tc-mudf-client.client $RAMDISK_BIN_DIR/usr/sbin/tc/tc-mudf-client
    #initializer
    if [ -f initializer ] ; then
      $CP initializer $RAMDISK_BIN_DIR/usr/local/bin/
    else
      tcwarn "initializer: no such file"
    fi

set +e
# ==============================================================================
# =========================== rearrange target (end) ===========================
# ==============================================================================
}

#step "build and installall modules"
step "build and install all modules"
make_tc_modules

# build tc firmware
# prerequisite: meego-themes.tar.gz
build_ramdisk(){
    DL_RAMDISK_DIR=$DL_DIR/ramdisk
    MEEGOTOUCH_THEME_PKG=$DL_RAMDISK_DIR/meegotouchtheme_0.21.14-1+0m6.tar.gz
    if ! [ -f "$MEEGOTOUCH_THEME_PKG" ] ; then
        tcerr "`basename $DL_DIR`/`basename $DL_RAMDISK_DIR`/`basename $MEEGOTOUCH_THEME_PKG`: missing file"
        tcexit 1
    fi

    MODULE_RAMDISK_DIR=$MODULE_DIR/ramdisk
    MODULE_RAMDISK_DL_DIR=$MODULE_RAMDISK_DIR/3rdParty
    BUILD_RAMDISK_DIR=$BUILD_DIR/`basename $MODULE_RAMDISK_DIR`
    BUILD_RAMDISK_DL_DIR=$BUILD_RAMDISK_DIR/`basename $MODULE_RAMDISK_DL_DIR`
    tcinfo "copy file `basename $DL_DIR`/`basename $DL_RAMDISK_DIR`/`basename $MEEGOTOUCH_THEME_PKG` to directory `basename $OUTPUT_DIR`/`basename $BUILD_DIR`/`basename $MODULE_RAMDISK_DIR`/`basename $BUILD_RAMDISK_DL_DIR`"
    $CP $MEEGOTOUCH_THEME_PKG $BUILD_RAMDISK_DL_DIR

    tcinfo "enter directory `basename $OUTPUT_DIR`/`basename $BUILD_DIR`/`basename $MODULE_RAMDISK_DIR` and run script build-ramdisk.sh"
    # set version environment
    export TC_MAJOR=${VERSION_MAJOR}
    export TC_MINOR=${VERSION_MINOR}
    cd $BUILD_RAMDISK_DIR && ./build-ramdisk.sh
    unset TC_MAJOR
    unset TC_MINOR
    if ! [ $? -eq 0 ] ; then
        tcerr "abort: build-ramdisk.sh"
        tcexit 1
    else
        tcinfo "success: build-ramdisk.sh"
    fi
}

step "build ramdisk"
build_ramdisk


# install tc_firmware
install_firmware(){
    TC_FIRMWARE="tc_firmware"
    TC_FIRMWARE_PATH=$BUILD_RAMDISK_DIR/out/$TC_FIRMWARE
    if ! [ -f "$TC_FIRMWARE_PATH" ] ; then
        tcerr "$TC_FIRMWARE_PATH : no such file"
        tcexit 1
    fi

    RAMDISK_GZ="rootfs.cpio.gz"
    MODULE_TCSERVER_DIR=$MODULE_DIR/tcserver
    BUILD_TCSERVER_DIR=$BUILD_DIR/`basename $MODULE_TCSERVER_DIR`
    FIRMWARE_INSTALL_DIR=$BUILD_TCSERVER_DIR/client

    tcinfo "install $TC_FIRMWARE"
    $MV -v $TC_FIRMWARE_PATH $FIRMWARE_INSTALL_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to install $TC_FIRMWARE"
        tcexit 1
    fi
}

step "install firmware"
install_firmware

# install tcboot.iso
install_iso(){
    RAMDISK_ISO="tcboot.iso"
    RAMDISK_ISO_PATH=$BUILD_RAMDISK_DIR/out/$RAMDISK_ISO
    if ! [ -f "$RAMDISK_ISO_PATH" ] ; then
        tcerr "$RAMDISK_ISO : no such file"
        tcexit 1
    fi

    tcinfo "install $RAMDISK_ISO"
    RAMDISK_ISO_INSTALL_DIR=$TC_RELEASE_IMAGE_CREATOR_DIR
    tcinfo "install $RAMDISK_ISO"
    $MV -v $RAMDISK_ISO_PATH $RAMDISK_ISO_INSTALL_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "install $RAMDISK_ISO"
        tcexit 1
    fi
}

step "install tcboot.iso"
install_iso

# install initializer
install_initializer(){
    INITIALIZER_ISO="tc-initializer.iso"
    INITIALIZER_ISO_PATH=$BUILD_RAMDISK_DIR/out/$INITIALIZER_ISO
    if ! [ -f "$INITIALIZER_ISO_PATH" ] ; then
        tcerr "$INITIALIZER_ISO : no such file"
        tcexit 1
    fi

    tcinfo "install $INITIALIZER_ISO "
    INITIALIZER_ISO_INSTALL_DIR=$TC_RELEASE_INITIALIZER_DIR
    tcinfo "install $INITIALIZER_ISO"
    $MV -v $INITIALIZER_ISO_PATH $INITIALIZER_ISO_INSTALL_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "install $INITIALIZER_ISO"
        tcexit 1
    fi
}

step "install tc-initializer.iso"
install_initializer

add_binary_for_tcserver(){
    ## centos7
    MODULE_TCSERVER_DIR=$MODULE_DIR/tcserver
    MODULE_TCSERVER_CENT7_DIR=$MODULE_TCSERVER_DIR/centos7
    MODULE_TCSERVER_TCROOT7_DIR=$MODULE_TCSERVER_DIR/tcroot

    BUILD_TCSERVER_DIR=$BUILD_DIR/`basename $MODULE_TCSERVER_DIR`
    BUILD_TCSERVER_CENT7_DIR=$BUILD_TCSERVER_DIR/`basename $MODULE_TCSERVER_CENT7_DIR`
    BUILD_TCSERVER_TCROOT7_DIR=$BUILD_TCSERVER_DIR/`basename $MODULE_TCSERVER_TCROOT7_DIR`

    tcinfo "MODULE_TCSERVER_DIR         = $MODULE_TCSERVER_DIR"
    tcinfo "MODULE_TCSERVER_CENT7_DIR   = $MODULE_TCSERVER_CENT7_DIR"
    tcinfo "MODULE_TCSERVER_TCROOT7_DIR = $MODULE_TCSERVER_TCROOT7_DIR"
    tcinfo "BUILD_TCSERVER_DIR          = $BUILD_TCSERVER_DIR"
    tcinfo "BUILD_TCSERVER_CENT7_DIR    = $BUILD_TCSERVER_CENT7_DIR"
    tcinfo "BUILD_TCSERVER_TCROOT7_DIR  = $BUILD_TCSERVER_TCROOT7_DIR"

    tcinfo "install webconsole for centos 7"
    MODULE_WEB_DIR=$MODULE_DIR/webconsole
    BUILD_WEB_DIR=$BUILD_DIR/`basename $MODULE_WEB_DIR`
    WEB_INSTALL_DIR=$BUILD_TCSERVER_TCROOT7_DIR/opt/lampp
    $CP -a $BUILD_WEB_DIR $WEB_INSTALL_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to install webconsole"
        tcexit 1
    fi

    tcinfo "copy cert files from top level directory"
    $CP --verbose $TOP_DIR/../../../cert/https/tcs.ca.certificate.crt $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/tcs.ca.certificate.crt
    $CP --verbose $TOP_DIR/../../../cert/https/console.tc.certificate.crt $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/console.tc.certificate.crt
    $CP --verbose $TOP_DIR/../../../cert/https/console.tc.private.pem.key $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/console.tc.private.pem.key
    $CP --verbose $TOP_DIR/../../../cert/syncSrv_cert.pem $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/syncSrv_cert.pem
    $CP --verbose $TOP_DIR/../../../cert/syncSrv_key.pem $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/syncSrv_key.pem
    $CP --verbose $TOP_DIR/../../../cert/p2p_cacert.pem $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/p2p_cacert.pem
    $CP --verbose $TOP_DIR/../../../cert/p2p_servkey.pem $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/p2p_servkey.pem
    $CP --verbose $TOP_DIR/../../../cert/p2p_servcert.pem $BUILD_TCSERVER_TCROOT7_DIR/home/tc/etc/cert/p2p_servcert.pem

    tcinfo "install pxelinux.0 into tftpboot directory"
    PXELINUX_ZERO=/usr/share/syslinux/pxelinux.0
    FIRMWARE_INSTALL_DIR=$BUILD_TCSERVER_TCROOT7_DIR/home/tc/tftpboot
    if ! [ -f $PXELINUX_ZERO ] ; then
        tcerr "{$PXELINUX_ZERO}: no such file, check if syslinux was installed correctly"
        tcexit 1
    fi
    $CP -a $PXELINUX_ZERO $FIRMWARE_INSTALL_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to copy file $PXELINUX_ZERO to directory $FIRMWARE_INSTALL_DIR"
        tcexit 1
    fi

    tcinfo "create tarball(tcroot.tar.gz) for content of tcroot directory, and remove the directory tcroot afterwards"
    dirtmp=/tmp/dir_`date "+%F%H%M%S"`
    $MKDIR $dirtmp
    $RM -r $dirtmp/*
    cd $BUILD_TCSERVER_CENT7_DIR/../tcroot && $CP -a * $dirtmp/
    cd $BUILD_TCSERVER_CENT7_DIR/tcroot && $CP -a * $dirtmp/
    cd $dirtmp && tar czf $BUILD_TCSERVER_CENT7_DIR/tcroot.tar.gz *
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to make tarball for tcroot directory"
        tcexit 1
    fi

    $RM -r $dirtmp
    cd $BUILD_TCSERVER_CENT7_DIR && $RM -r tcroot
    cd $BUILD_TCSERVER_CENT7_DIR/.. && $RM -r tcroot
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to remove tcroot directory"
        tcexit 1
    fi
}

step "add binary for tcserver"
add_binary_for_tcserver

# install modules: tcserver
install_module_tcserver_to_fake_target(){
    # tcserver
    tcinfo "save module tcserver to fake target"
    CUR_INST_MOD=tcserver
    if ! [ -d "$BUILD_DIR" ] ; then
        tcerr "$BUILD_DIR: no such directory"
        tcexit 1
    fi

    cd $BUILD_DIR
    $CP -a $CUR_INST_MOD $FAKE_MODULE_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to install module $CUR_INST_MOD into fake target"
        tcexit 1
    fi
}

step "install module tcserver into fake target"
install_module_tcserver_to_fake_target


install_module_ps_to_fake_target(){
    # ps
    tcinfo "load module ps to fake target"
    CUR_INST_MOD=ps

    if ! [ -d "$BUILD_DIR" ] ; then
        tcerr "$BUILD_DIR: no such directory"
        tcexit 1
    fi

    cd $BUILD_DIR
    $CP -a $CUR_INST_MOD $FAKE_MODULE_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to install module $CUR_INST_MOD into fake target"
        tcexit 1
    fi
}

tcinfo "install_module_ps_to_fake_target"
install_module_ps_to_fake_target

# generate server installer
generate_server_installer(){
    # make tarball
    cd $OUTPUT_DIR
    $MV -v `basename $FAKE_TOP_DIR` ${TCSERVER_INSTALLER_TARBALL%.tar}
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to move directory `basename $FAKE_TOP_DIR`"
        tcexit 1
    fi

    tar cf ${TCSERVER_INSTALLER_TARBALL} ${TCSERVER_INSTALLER_TARBALL%.tar}
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to create ${TCSERVER_INSTALLER_TARBALL}"
        tcexit 1
    fi

    tcinfo "save ${TCSERVER_INSTALLER_TARBALL} to directory `basename $OUTPUT_DIR`/`basename $TARGET_DIR`/`basename $TC_RELEASE_DIR`"
    $MV ${TCSERVER_INSTALLER_TARBALL} $TC_RELEASE_SRV_INST_DIR
    $RM -r ${TCSERVER_INSTALLER_TARBALL%.tar}
}

step "generate server installer"
generate_server_installer


# install modules: tcc_setup clientControl2 diskDriverForLinux
install_extra_modules(){
    cd $BUILD_DIR

    # tcc_setup
    CUR_INST_MOD=tcc_setup
    tcinfo "install module: $CUR_INST_MOD"
    $CP -a $CUR_INST_MOD $TC_RELEASE_CLT_WIN_DIR
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to install module $CUR_INST_MOD"
        tcexit 1
    fi

    # clientControl2
    CUR_INST_MOD=clientControl2
    tcinfo "install module: $CUR_INST_MOD"
#    $CP -a --parents $CUR_INST_MOD"/bin/tcdclt.lite" $TC_RELEASE_CLT_LINUX_DIR/tcctl.static.linux
#    $CP -a --parents $CUR_INST_MOD"/bin/tcctl.static.and" $TC_RELEASE_CLT_LINUX_DIR
    $CP -a $CUR_INST_MOD"/TcService/tcctl.static.linux" $TC_RELEASE_CLT_LINUX_DIR/
    $CP -a $CUR_INST_MOD"/TcService/tcctl.static.and" $TC_RELEASE_CLT_LINUX_DIR/
    if ! [ $? -eq 0 ] ; then
        tcerr "fail to install module $CUR_INST_MOD"
        tcexit 1
    fi

    # diskDriverForLinux
#    CUR_INST_MOD=diskDriverForLinux
#    tcinfo "install module: $CUR_INST_MOD"
#    $CP -a $CUR_INST_MOD $TC_RELEASE_CLT_LINUX_DIR
#    if ! [ $? -eq 0 ] ; then
#        tcerr "fail to install module $CUR_INST_MOD"
#        tcexit 1
#    fi

    # doc
    $CP -a $DOC_DIR/* $TC_RELEASE_DOC_DIR
}

step "install module: tcc_setup clientControl2 diskDriverForLinux"
install_extra_modules

step "generate release package"
cd $TC_RELEASE_DIR/..
#
# remove two iso
#
$RM -r $TC_RELEASE_NAME/tool/image_export_tool/
$RM -r $TC_RELEASE_NAME/tool/initializer/

tar czf $TC_RELEASE_TARBALL $TC_RELEASE_NAME
if [ $? -eq 0 ] ; then
    tcinfo "save output to output/target/$TC_RELEASE_TARBALL"
else
    tcerr "fail to create file output/target/$TC_RELEASE_TARBALL"
    tcexit 1
fi
$RM -r $TC_RELEASE_NAME


step "clean up stuff in directory output/build"
$RM -r $BUILD_DIR/{.[^.],}*
if ! [ $? -eq 0 ] ; then
    tcerr "fail to clean up stuff in directory output/build"
    tcexit 1
fi


# tcmsg
tcinfo "close messages"
tcmsg_finish

