#!/bin/sh
# export stuff into external file

SCRIPT_PATH=$0
OUTPUT_DIR=output

# you may adjust the list as you want
mod_list="\
buildroot \
busybox \
grub \
libmeegotouch \
linux \
qt \
seabios \
qemu \
"

# function: help
help(){
    cat <<EOT
SYNOPSIS
    $SCRIPT_PATH <html-filename>

DESCRIPTION
    export patch files from HTML <html-filename> to directory  \`output' 

EXAMPLE
    $SCRIPT_PATH tc.html
EOT
}


# function: find a str within a given file
# retval: return line number for the first occurence of str within file
find_str(){
    # str, a string to find
    # filename, the name of a file to search
    # return 0 if successfully find the string
    # also print the line number within the file for the first time occurrence
    local str=$1
    local file=$2
    if ! [ $# -eq 2 ] ; then
        return 1
    elif ! [ -f $file ] ; then
        return 2
    fi

    awk "/${str}/ {print NR}" ${file}
}


# function: export stuff to external file from specified position within target file
export_to(){
    # $1, the file to save exported content
    # $2, the target file to read
    # $3, the flag to specify a starting position within target file to read
    # $4, the flag to specify an ending position within target file to read
    local data_file=$1
    local file=$2
    local anchor_str_start=$3
    local anchor_str_end=$4
    local anchor_start_line_number
    local anchor_end_line_number
    local read_start_line_number
    local read_end_line_number

    if ! [ $# -eq 4 ] ; then
        return 1
    elif ! [ -f "$file" ] ; then
        return 3
    fi

    anchor_start_line_number=`find_str $anchor_str_start $file`
    anchor_end_line_number=`find_str $anchor_str_end $file`

    read_start_line_number=$(($anchor_start_line_number + 1))
    read_end_line_number=$(($anchor_end_line_number - 1))

    if [ $read_start_line_number -le $read_end_line_number ] ; then
        vim -e $file <<-EOT
        ${read_start_line_number},${read_end_line_number} write! ${data_file}
        wq
EOT
    fi
}

# entry
if ! [ -z "$1" ] && [ -f "$1" ] ; then
    mkdir -p $OUTPUT_DIR
    for mod in $mod_list ; do
        OUT_FILE=$OUTPUT_DIR/${mod}.patch
        BEGIN=anchor_start_${mod}
        END=anchor_end_${mod}
        export_to $OUT_FILE $1 $BEGIN $END
    done
else
    help
fi

