/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "BufferItem.h"


BufferItem::BufferItem()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    Init();
}

BufferItem::BufferItem(UINT size)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    Init();

    Alloc(size);
}

BufferItem::~BufferItem()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    delete []_head;
}

void BufferItem::Alloc(UINT size)
{
    UINT  nOldSize;

    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, size));

    nOldSize = room();
    if ( nOldSize < size)
    {
        if ( nOldSize )
        {
            DBG(D_PERF, (L"%s: %d -> %d\n", FULL_FUNC_NAME, nOldSize, size));
        }

        delete []_head;
        _head = new BYTE[size];
//        memset(_head, 0, size);  // only for valgrind
        _end = _head + size;
    }

    Reset();
}

void BufferItem::Expand(UINT len)
{
    PBYTE  p;
    UINT  size;

    size = room() + len;
    DBG(D_PERF, (L"%s: %d -> %d\n", FULL_FUNC_NAME, room(), size));

    p = new BYTE[size];
//    memset(p, 0, size);  // only for valgrind

    memcpy(p, _head, room());

    _data = p + head_size();
    _tail = _data + _data_len;
    _end = p + size;

    delete []_head;
    _head = p;
}

void BufferItem::Reset(UINT len)
{
    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    _data = _head + len;
    _tail = _data;
    _data_len = 0;

    assert(_tail <= _end);
}

void BufferItem::Reserve(UINT len)
{
    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    assert(!_data_len);

    _data += len;
    _tail = _data;

    assert(_tail <= _end);
}

void BufferItem::Retain(UINT len)
{
    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    assert(!_data_len);

    _data = _end - len;
    _tail = _data;

    assert(_tail <= _end);
}

PBYTE BufferItem::Put(UINT len)
{
    PBYTE  p;

    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    assert(_head);

    p = _tail;
    _tail += len;
    _data_len += len;

    assert(_tail <= _end);

    return p;
}

PBYTE BufferItem::Push(UINT len)
{
    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    assert(_head);

    _data -= len;
    _data_len += len;

    assert(_data >= _head);

    return _data;
}

PBYTE BufferItem::Pull(UINT len)
{
    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    assert(_head);

    _data += len;
    _data_len -= len;

    assert(_data <= _tail);

    return _data;
}

void BufferItem::Save(UINT len)
{
    DBG(D_FUNC, (L"%s: %d\n", FULL_FUNC_NAME, len));

    assert(_head);

    assert(_data_len_saved == (UINT)-1);
    _data_len_saved = _data_len;
    _data_len = len;

    _tail = _data + _data_len;

    assert(_tail <= _end);
}

void BufferItem::Restore()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    assert(_head);

    assert(_data_len_saved != (UINT)-1);
    _data_len = _data_len_saved;
    _data_len_saved = (UINT)-1;

    _tail = _data + _data_len;

    assert(_tail <= _end);
}

UINT BufferItem::room() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return (UINT)(_end - _head);
}

PBYTE BufferItem::data() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return _data;
}

UINT BufferItem::data_size() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return _data_len;
}

PBYTE BufferItem::head() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return _head;
}

UINT BufferItem::head_size() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return (UINT)(_data - _head);
}

PBYTE BufferItem::tail() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return _tail;
}

UINT BufferItem::tail_size() const
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    return (UINT)(_end - _tail);
}

void BufferItem::Init()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    _head = _data = _tail = _end = NULL;
    _data_len = 0;

    _data_len_saved = (UINT)-1;
}
