/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "MeasElapseTime.h"

MeasElapseTime::MeasElapseTime()
    : _qwCount(0)
    , _qwBytes(0)
    , _qwPreTicks(0)
    , _qwTicks(0)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _qwFreq = GetPerformanceFrequency();
}

MeasElapseTime::~MeasElapseTime()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

void MeasElapseTime::Reset()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    _qwCount = 0;
    _qwBytes = 0;
    _qwPreTicks = 0;
    _qwTicks = 0;
}

void MeasElapseTime::Start()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    assert(!_qwPreTicks);
    _qwPreTicks = GetPerformanceCounter();
}

void MeasElapseTime::Update(QWORD qwBytes)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    ++_qwCount;
    _qwBytes += qwBytes;
}

void MeasElapseTime::Finish(QWORD qwBytes)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    Update(qwBytes);
    Finish();
}

void MeasElapseTime::Finish()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    assert(_qwPreTicks);
    _qwTicks += GetPerformanceCounter() - _qwPreTicks;
    _qwPreTicks = 0;
}

void MeasElapseTime::Print(const char *szPrompt)
{
    QWORD  qwSize, qwTime, qwSpeed;

    if ( _qwCount )
    {
        qwSize = _qwBytes / (1024 * 1024);
        qwTime = _qwTicks * 1000 / _qwFreq;
        qwSpeed = qwSize * _qwFreq / _qwTicks;

        DBG(D_PERF, (L"%hs: count: %lld, size: %lldMB, time: %lld, speed: %lldMB/s\n",
            szPrompt, _qwCount, qwSize, qwTime, qwSpeed));
    }
}
