/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "ProducerCounter.h"


ProducerCounter::ProducerCounter()
    : _qwProduceNum(0)
    , _qwConsumeNum(0)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

ProducerCounter::~ProducerCounter()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

bool ProducerCounter::Produce(UINT nCount)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    // see issues_sendprod.txt
    assert(_qwProduceNum >= _qwConsumeNum);

    _qwProduceNum += nCount;

    MemoryBarrier();

    return ((_qwProduceNum - _qwConsumeNum) == (QWORD)nCount);
}

void ProducerCounter::Consume(UINT nCount)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _qwConsumeNum += nCount;

    MemoryBarrier();

    // see issues_sendprod.txt
    assert(_qwConsumeNum <= _qwProduceNum);
}

void ProducerCounter::Reset(UINT nProduct)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _qwConsumeNum = 0;
    _qwProduceNum = nProduct;
}

bool ProducerCounter::IsEmpty() const
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(_qwProduceNum >= _qwConsumeNum);

    // see issues_pcounter.txt
//    return (*(volatile QWORD*)&_qwProduceNum == *(volatile QWORD*)&_qwConsumeNum);
    return (_qwProduceNum == _qwConsumeNum);
}

UINT ProducerCounter::GetProductNum() const
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(_qwProduceNum >= _qwConsumeNum);

    return (UINT)(_qwProduceNum - _qwConsumeNum);
}

