/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "VectorMate.h"


VectorMate::VectorMate(bool bEnableLock)
    : _bEnableLock(bEnableLock)
    , _pFreeList(NULL)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    if ( _bEnableLock )
    {
        pthread_mutex_init(&_hLock, NULL);
    }
}

VectorMate::~VectorMate()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    delete _pFreeList;

    if ( _bEnableLock )
    {
        pthread_mutex_destroy(&_hLock);
    }
}

DWORD VectorMate::Init(UINT nSize, UINT nBase)
{
    UINT  nEnd;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(!_pFreeList);

    _pFreeList = new(std::nothrow) RingBuffer<UINT>(nSize, false);
    if ( !_pFreeList )
    {
        DBG(D_WARN, (L"%s: alloc failed\n", FULL_FUNC_NAME));
        return ERROR_OUTOFMEMORY;
    }

    nEnd = nBase + nSize;
    for (UINT i = nBase; i < nEnd; ++i )
    {
        _pFreeList->PushAtBack(i);
    }

    return ERROR_SUCCESS;
}

void VectorMate::Fini()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    delete _pFreeList;
    _pFreeList = NULL;
}

bool VectorMate::IsEmpty() const
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pFreeList->IsEmpty();
}

bool VectorMate::IsFull() const
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pFreeList->IsFull();
}

DWORD VectorMate::GetFreeIndex(UINT &nIndex)
{
    DWORD  dwRet;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(_pFreeList);

    if ( _bEnableLock )
    {
        pthread_mutex_lock(&_hLock);
    }

    if ( _pFreeList->IsEmpty() )
    {
        dwRet = ERROR_NOT_FOUND;
    }
    else
    {
        nIndex = _pFreeList->PopAtFront();
        dwRet = ERROR_SUCCESS;
    }

    if ( _bEnableLock )
    {
        pthread_mutex_unlock(&_hLock);
    }

    return dwRet;
}

void VectorMate::PutFreeIndex(UINT nIndex)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(_pFreeList);

    if ( _bEnableLock )
    {
        pthread_mutex_lock(&_hLock);
    }

    assert(!_pFreeList->IsFull());
    _pFreeList->PushAtBack(nIndex);

    if ( _bEnableLock )
    {
        pthread_mutex_unlock(&_hLock);
    }
}

