/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _BIT_ARRAY_H
#define _BIT_ARRAY_H


#include "FileMapping.h"


class BitArray
{
public:
    BitArray();
    ~BitArray();

    DWORD Init(UINT nBitsCnt, bool bSetAll, bool bCalcMask);
    DWORD Init(UINT nBitsCnt, const PBYTE pByteData, UINT nSize, bool bCalcMask);
    DWORD Init(UINT nBitsCnt, const char *szCacheFile, bool bCalcMask);
    void ReInit(bool bSetAll);
    void Fini();

    UINT GetBitsCount() const;
    UINT GetBytesCount() const;

    void SetBit(UINT nPos);
    void SetBit(UINT nPos, bool &rbMaskUpd);
    void ClearBit(UINT nPos);
    void ClearBit(UINT nPos, bool &rbMaskUpd);
    bool TestAndSetBit(UINT nPos);
    bool TestAndClearBit(UINT nPos);
    bool IsBitSet(UINT nPos) const;

    UINT GetSetsCount() const;
    bool IsAllSet() const;
    bool IsAllClear() const;

    UINT GetDegree() const;

    void GetCompactMask(QWORD &rqwMaskAnd, QWORD &rqwMaskOr) const;
    DWORD GetFullMask(PBYTE pByteBuf, UINT &rnSize) const;

private:
    DWORD DoInit(UINT nBitsCnt, bool bAlloc, bool bSetAll, bool bCalcMask);

    void SETBIT(UINT nIdx, BYTE bMask);
    void CLEARBIT(UINT nIdx, BYTE bMask);
    bool ISBITSET(UINT nIdx, BYTE bMask) const;

    void OnSetBit(UINT nPos);
    void OnClearBit(UINT nPos);

    void Pos2ArrayMask(UINT nPos, UINT &rnIdx, BYTE &rbMask) const;

    void InitMatrixParam(bool bSetAll);
    void Pos2MatrixMask(UINT nPos, UINT &rnRow, QWORD &rqwMask);

private:
    UINT                        _nBitsCnt;
    UINT                        _nBytesCnt;
    PBYTE                       _pByteArray;

    FILE                      * _hCacheFile;
    FileMapping               * _pCacheMapping;

    UINT                        _nSetsCnt;

    bool                        _bCalcMask;
    vector<QWORD>               _MaskArray;
    QWORD                       _qwMaskAnd;
    QWORD                       _qwMaskOr;
    UINT                        _nRemain;
    UINT                        _nRightH;
    UINT                        _nLeftH;
    UINT                        _nDivide;
};


#endif

