/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _DEFLATE_CODEC_H
#define _DEFLATE_CODEC_H


#ifdef CONFIG_USE_LZ4
#include <lz4frame.h>
#endif

#ifdef CONFIG_USE_ZLIB_NG
#include "zlib-ng.h"
#define ZLIB_PREFIX(x)          zng_ ## x
#define ZLIB_PREFIX2(x)         zng_ ## x
#endif

#ifdef CONFIG_USE_ZLIB
#include <zlib.h>
#define ZLIB_PREFIX(x)          x
#define ZLIB_PREFIX2(x)         z_ ## x
#endif

#include "BufferItem.h"
#include "MeasElapseTime.h"


typedef enum _COMPRESS_LEVEL
{
#if defined(CONFIG_USE_LZ4) || defined(CONFIG_DEFLATE_USE_ZNULL)
    COMPRESS_FASTEST = 1,
    COMPRESS_FAST = 3,
    COMPRESS_NORMAL = 9,
    COMPRESS_GOOD = 10,
    COMPRESS_BEST = 12
#endif
#if defined(CONFIG_USE_ZLIB_NG) || defined(CONFIG_USE_ZLIB)
    COMPRESS_FASTEST = 1,
    COMPRESS_FAST = 3,
    COMPRESS_NORMAL = 5,
    COMPRESS_GOOD = 7,
    COMPRESS_BEST = 9
#endif
}
COMPRESS_LEVEL;


class DeflateAlg
{
public:
    DeflateAlg();
    ~DeflateAlg();

    DWORD Init(COMPRESS_LEVEL level);
    void Fini();

    void Reset();

    UINT GetBound(UINT nSrc);

    DWORD ProcessBlock(const PBYTE pSrc, UINT nSrc, PBYTE pDst, UINT &rnDst, bool bReset);
    DWORD ProcessBlock(const BufferItem *pbiSrc, BufferItem *pbiDst, bool bReset);

    DWORD ProcessStream(PBYTE &rpSrc, UINT &rnSrc, PBYTE pDst, UINT &rnDst, bool &rbAgain, bool bTrunc);
    DWORD ProcessStream(BufferItem *pbiSrc, BufferItem *pbiDst, bool &rbAgain, bool bTrunc);

private:
#ifdef CONFIG_USE_LZ4
    DWORD DoLZ4FBegin(PBYTE &rpOut, UINT &rnLeft);
    DWORD DoLZ4FEnd(PBYTE &rpOut, UINT &rnLeft);
    DWORD DoLZ4FProcess(const PBYTE pIn, UINT nIn, PBYTE &rpOut, UINT &rnLeft, bool bAutoFlush, bool bDoEnd);
#endif

private:
    MeasElapseTime              _MesaComp;

    bool                        _bInited;
#ifdef CONFIG_USE_LZ4
    bool                        _bNotBegin;
    LZ4F_cctx                 * _lz4fStrm;
    LZ4F_preferences_t          _lz4fPrefs;
    LZ4F_compressOptions_t      _lz4fOpts;
#endif
#if defined(CONFIG_USE_ZLIB_NG) || defined(CONFIG_USE_ZLIB)
    ZLIB_PREFIX2(stream)        _zlibStrm;
#endif
};


class InflateAlg
{
public:
    InflateAlg();
    ~InflateAlg();

    DWORD Init();
    void Fini();

    void Reset();

    DWORD Process(PBYTE &rpSrc, UINT &rnSrc, PBYTE pDst, UINT &rnDst, bool &rbAgain, bool bLast);
    DWORD Process(PBYTE &rpSrc, UINT &rnSrc, BufferItem *pbiDst, bool &rbAgain, bool bLast);
    DWORD Process(BufferItem *pbiSrc, BufferItem *pbiDst, bool &rbAgain, bool bLast);

private:
    MeasElapseTime              _MesaDecomp;

    bool                        _bInited;
#ifdef CONFIG_USE_LZ4
    LZ4F_dctx                 * _lz4fStrm;
    LZ4F_decompressOptions_t    _lz4fOpts;
#endif
#if defined(CONFIG_USE_ZLIB_NG) || defined(CONFIG_USE_ZLIB)
    ZLIB_PREFIX2(stream)        _zlibStrm;
#endif
};


#endif
