/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _ENDPOINT_H
#define _ENDPOINT_H


#include "PCondEvent.h"
#include "VectorMate.h"
#include "ProducerQueue.h"
#include "EpProtocolHandler.h"

class Connection;

typedef enum _CONN_TYPE
{
    T_CONN_LISTENER = 1,
    T_CONN_CONNECT_ACCEPT,
    T_CONN_CONNECT_IN,
    T_CONN_CONNECT_OUT
}
CONN_TYPE;

typedef struct _CONN_REC
{
#ifndef NDEBUG
    volatile bool               bUsed;
#endif

    volatile bool               bValid;
    volatile WORD               wMagic;

    CONN_TYPE                   ConnType;

    bool                        bBounded;

    bool                        bCmdDisconn;

    Connection                * pConnection;
    EpProtocolHandler         * pHandler;
    DWORD                       dwCmdMask;
}
CONN_REC, *PCONN_REC;


class Endpoint
{
public:
    Endpoint();
    virtual ~Endpoint();

    DWORD Load(UINT nMaxConnNum, UINT nExTaskNum, UINT nEpSigNum);
    virtual void Unload();

protected:
    DWORD AddListener(PSOCK_ADDR pLocalAddr, bool bOneShot, DWORD &rhListener, UINT nBacklogNum = DEFAULT_BACKLOG_NUM);
    void RemoveListener(DWORD hListener);
    virtual void OnListenerEnded(DWORD hListener) = 0;

    DWORD ConnectIn(PSOCK_ADDR pLocalAddr, EpProtocolHandler *pHandler, DWORD &rdwConnId);
    DWORD ConnectOut(PSOCK_ADDR pLocalAddr, const PSOCK_ADDR pRemoteAddr, EpProtocolHandler *pHandler, DWORD &rdwConnId, const PSOCK_ADDR pProxyAddr = NULL);

    virtual EpProtocolHandler *OnNewHandler(DWORD hListener, Connection *pConnection, DWORD dwConnId) = 0;

    virtual void OnAsyncConnected(DWORD dwConnId, EpProtocolHandler *pHandler) = 0;
    virtual void OnDisconnected(DWORD dwConnId, EpProtocolHandler *pHandler) = 0;

    virtual void OnConnEvent(DWORD dwConnId, EpProtocolHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody) = 0;

    DWORD QueueConnCmd(DWORD dwConnId, DWORD dwCmdMask, BufferItem *pbiCmd);
    DWORD DisconnectConn(DWORD dwConnId);

    void RequestEpWork(UINT nSigIdx);
    virtual void ProcessEpWork(UINT nSigIdx) = 0;

private:
    void UnloadThreads();

    DWORD MakeConnId(UINT nConnIdx);
    void AddToConnPool(CONN_TYPE ConnType, Connection *pConnection, EpProtocolHandler *pHandler, DWORD dwConnId);

    void OnNewConnection(DWORD hListener, Connection *pListener);

    void ReleaseConn(DWORD dwConnId, PCONN_REC pConnRec);

    bool ProcessConnWork(UINT nSigIdx, DWORD dwConnId, PCONN_REC pConnRec);

    DWORD WorkThread(UINT nThreadIdx);
    static void *WorkThreadProc(void *arg);

private:
    UINT                        _nEpSigBase;
    UINT                        _nEpSigNum;

    PCondEvent                * _pEvent;

    volatile bool               _bQuit;
    vector<pthread_t>           _hWorkThreads;

    pthread_rwlock_t            _hConnValidLock;
    vector<CONN_REC>            _ConnList;
    VectorMate                  _ConnListMate;
    DWORD                       _dwConnMagic;

    // all conns share the same cmd lock
    pthread_mutex_t             _hConnCmdLock;
};


#endif
