/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _EP_PROTOCOL_HANDLER_H
#define _EP_PROTOCOL_HANDLER_H


#include "Connection.h"
#include "ProtocolHandler.h"
#include "PCondEvent.h"
#include "ProducerQueue.h"
#include "ProducerCounter.h"
#include "MeasElapseTime.h"

typedef enum _EPH_FLAG_BITS
{
    F_EPH_NONE                = 0x00,
    F_EPH_SESSIONEND          = 0x01,
    F_EPH_NEWCONNECTION       = 0x02,
    F_EPH_ASYNCCONNECTED      = 0x04,
    F_EPH_RECV                = 0x08,
    F_EPH_SEND                = 0x10,
    F_EPH_EVENT               = 0x20
}
EPH_FLAG_BITS;


void EpBuildMsg(UINT nCode, DWORD dwCookie, DWORD dwParam, BufferItem *pbiBody);


class EpProtocolHandler : public ProtocolHandler
{
public:
    EpProtocolHandler
        (
            UINT nRecvQSize, UINT nRecvBSize,
            UINT nSendQSize, UINT nSendBSize,
            UINT nEvtQSize, UINT nEvtBSize, bool bEvtLock = false
        );
    virtual ~EpProtocolHandler();

    virtual DWORD Bind(Connection *pConnection, PCondEvent *pEvent, UINT nSigIdx);
    virtual void Unbind();

    virtual void OnAsyncConnected(Connection *pConnection);
    virtual void OnDisconnectedOrError(Connection *pConnection);
    virtual UINT OnRecvReady(Connection *pConnection);
    virtual UINT OnSendReady(Connection *pConnection);

    // serialization and synchronization, but not under conn's work thread
    virtual void QueueCmd(DWORD dwCmdMask, BufferItem *pbiCmd);
    // under conn's work thread
    virtual void ProcessCmd(DWORD dwCmdMask);

    virtual DWORD GetFlags();

    void ProcessRecv();

    virtual void PrepareSend() = 0;

    // only called in conn thread @ endpoint, so no need bSync
    BufferItem *GetEvent();
    virtual void ReleaseEvent(BufferItem *pbiEvt);

protected:
    void SignalEp();

    void ReqEndSession();
    void ReqSendProc(bool bSetReq);

    // may multi-thread to alloc&put event, so need bSync
    BufferItem *AllocEvent(bool bSync = false);
    void PutEvent(BufferItem *pbiEvt, bool bSync = false);

/*-----------------------------------------------------------------*/
    /* not support reentry,
       should be called in conn's work thread,
       or in OnXXXXReady callback */
    void PutRecvData(BufferItem *pbiData);
    virtual bool ProcessRecvData(BufferItem *pbiData) = 0;
    virtual void ReleaseRecvData(BufferItem *pbiData);

    BufferItem *GetSendBuf();
    void PutSendData(BufferItem *pbiData, bool bLastOne);
    // if return true then need call ReqSendProc
    virtual bool ReleaseSendData(BufferItem *pbiData);
/*-----------------------------------------------------------------*/

protected:
    Connection                * _pConnection;

private:
    MeasElapseTime              _MesaRecv;

    PCondEvent                * _pEvent;
    UINT                        _nSigIdx;

    bool                        _bSessionEnd;
    bool                        _bAsyncConnected;

    bool                        _bReqSendProc;

    ProducerCounter             _RecvCounter;
    ProducerCounter             _SendCounter;

    ProducerQueue               _RecvQueue;
    ProducerQueue               _SendQueue;

    ProducerQueue               _EventQueue;
};


#endif
