/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _EXECUTOR_POOL_H
#define _EXECUTOR_POOL_H


#include "PCondEvent.h"
#include "VectorMate.h"
#include "Runnable.h"

typedef enum _RUNN_REQ
{
    RUNN_REQ_NONE,
    RUNN_REQ_DONE = RUNN_REQ_NONE,
    RUNN_REQ_REMOVE
}
RUNN_REQ;

typedef struct _RUNNABLE_REC
{
#ifndef NDEBUG
    volatile bool               bUsed;
#endif

    volatile bool               bValid;
    volatile WORD               wMagic;

    volatile RUNN_REQ           CtlReq;
    volatile UINT               nSlotIdx;

    Runnable                  * pRunnable;
}
RUNNABLE_REC, *PRUNNABLE_REC;


class ExecutorPool
{
public:
    ExecutorPool();
    ~ExecutorPool();

    DWORD Load(UINT nMaxRunnNum = MAX_RUNNABLE_NUM, UINT nExTaskNum = 0);
    void Unload();

    DWORD RegisterRunnable(Runnable *pRunnable, DWORD &rdwRunnId);
    DWORD UnregisterRunnable(DWORD dwRunnId);

private:
    DWORD WorkThread(UINT nThreadIdx);
    static void *WorkThreadProc(void *arg);

private:
    PCondEvent                * _pEvent;

    volatile bool               _bQuit;
    vector<pthread_t>           _hWorkThreads;

    vector<RUNNABLE_REC>        _RunnList;
    VectorMate                  _RunnListMate;
    DWORD                       _dwRunnMagic;
};


class ExecutorCenter : public ExecutorPool
{
public:
    static ExecutorCenter *Instance();
    static void Destroy();

private:
    static ExecutorCenter     * _instance;

protected:
    ExecutorCenter();
    ~ExecutorCenter();
};


#endif
