/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _LP2P_DOWNLOAD_HANDLER_H
#define _LP2P_DOWNLOAD_HANDLER_H


#include "EpPDUHandler.h"
#include "DeflateCodec.h"
#include "SeedTarget.h"

typedef enum _LP2P_DH_STATE
{
    DH_STATE_NONE,

    DH_STATE_REQ_OPEN,
    DH_STATE_OPEN_READY,

    DH_STATE_REQ_GETMASK,
    DH_STATE_GETMASK_READY,

    DH_STATE_REQ_READ,

    DH_STATE_ERROR
}
LP2P_DH_STATE;


class LP2PDownloadHandler : public EpPDUHandler
{
public:
    LP2PDownloadHandler(SeedTarget *pSeedTarget);
    virtual ~LP2PDownloadHandler();

    virtual DWORD Bind(Connection *pConnection, PCondEvent *pEvent, UINT nSigIdx);
    virtual void Unbind();

    virtual void OnAsyncConnected(Connection *pConnection);

    virtual void PrepareSend();

    SOCK_ADDR GetPeerAddr() const;
    void SetPeerAddr(const SOCK_ADDR *pAddr);

protected:
    virtual void ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, BufferItem *pbiBody);
    virtual void ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, const PBYTE pData, UINT nDataBytes, UINT nLeftBytes);

private:
    void OpenSeedTargetReq();
    DWORD OnOpenSeedTargetRep(DWORD dwResult);

    void GetSeedTargetBitmaskReq();
    DWORD OnGetSeedTargetBitmaskRep(DWORD dwResult, BufferItem *pbiBody);

    void ReadSeedTargetBlockReq();

#ifdef CONFIG_ST_BLK_COMPACT
    DWORD InflateBlock(const PBYTE pData, UINT nDataBytes, bool bLast, bool &rbEndOfBlk);
#endif

private:
    SeedTarget                * _pSeedTarget;
    const BitArray            * _pLocalBitMask;

    SOCK_ADDR                   _PeerAddr;

    LP2P_DH_STATE               _State;

    DWORD                       _dwSessionId;
    UINT                        _nBlkNum;
    UINT                        _nBlkIdx;
    DWORD                       _dwBlkId;

    BitArray                    _PeerBitMask;

#ifdef CONFIG_ST_BLK_COMPACT
    InflateAlg                  _DecompAlg;
    BufferItem                  _DecompBuf;
#endif
};


#endif
