/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _LP2P_ENDPOINT_H
#define _LP2P_ENDPOINT_H


#include "RingBuffer.h"
#include "VectorMate.h"
#include "Endpoint.h"
#include "LP2PSession.h"

class SeedTarget;
class SeedAgentHandler;
class LP2PUploadHandler;


typedef enum _LP2P_SESSION_REQ
{
    SESSION_REQ_NONE,
    SESSION_REQ_DONE = SESSION_REQ_NONE,
    SESSION_REQ_ACTIVE,
    SESSION_REQ_REMOVE
}
LP2P_SESSION_REQ;

typedef struct _LP2P_SEEDTARGET_REC
{
#ifndef NDEBUG
    volatile bool               bUsed;
#endif

    volatile bool               bValid;
    volatile WORD               wMagic;

    SeedTarget                * pSeedTarget;
    DWORD                       dwCookie;

    DWORD                       dwSASTId;

    list<DWORD>                 UpHIdList;

    volatile LP2P_SESSION_REQ   SessionReq;
    volatile bool               bSsValid;
    LP2PSession               * pSession;
    DWORD                       dwErrCode;
}
LP2P_SEEDTARGET_REC, *PLP2P_SEEDTARGET_REC;


class LP2PEpObserver
{
    // serialization and synchronization
public:
    virtual void OnConnectFailed(DWORD dwErrCode) = 0;
    virtual void OnConnecting() = 0;
    virtual void OnConnected() = 0;
    virtual void OnDisconnected() = 0;

    virtual void OnNodeRegistered(DWORD dwResult) = 0;

    virtual void OnSeedTargetFinished(DWORD dwCookie, DWORD dwResult) = 0;
};


class LP2PEndpoint : protected Endpoint, public LP2PSessionObserver
{
public:
    static LP2PEndpoint *Instance();
    static void Destroy();

private:
    static LP2PEndpoint       * _instance;

protected:
    LP2PEndpoint();
    virtual ~LP2PEndpoint();

public:
    DWORD Load(const char *szTrackerIP, const char* szNodeName, UINT nMaxSTNum, UINT nMaxUploadNum, UINT nMaxSessionNum, LP2PEpObserver *pObserver);
    void Unload();

    DWORD AddSeedTarget(SeedTarget *pSeedTarget, DWORD dwCookie, DWORD &rdwSTId);
    DWORD RemoveSeedTarget(DWORD dwSTId);

    virtual void OnFinished(DWORD dwCookie, DWORD dwErrCode);
    virtual void OnEnumNodes4Seed(DWORD dwCookie);

protected:
    virtual void OnListenerEnded(DWORD hListener);

    virtual EpProtocolHandler *OnNewHandler(DWORD hListener, Connection *pConnection, DWORD dwConnId);
    virtual void OnAsyncConnected(DWORD dwConnId, EpProtocolHandler *pHandler);
    virtual void OnDisconnected(DWORD dwConnId, EpProtocolHandler *pHandler);
    virtual void OnConnEvent(DWORD dwConnId, EpProtocolHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody);

    virtual void ProcessEpWork(UINT nSigIdx);

private:
    void OnSAHConnect();
    void OnSessionRequest();
    void OnSessionFree();
    bool ProcessSession(UINT nSTIdx, PLP2P_SEEDTARGET_REC pSTRec);

    void OnSeedAgentDisconnected();
    void OnUploadDisconnected(DWORD dwConnId, LP2PUploadHandler *pHandler);

    void OnSeedAgentConnEvent(PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody);

    void OnUploadConnEvent(DWORD dwConnId, LP2PUploadHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody);
    void OnOpenSeedTarget(DWORD dwConnId, LP2PUploadHandler *pHandler, const SHA1_VAL &SHA1ValSeed);

    static void TimerFunc(HANDLE hArg);

private:
    LP2PEpObserver            * _pObserver;

    SOCK_ADDR                   _TrackerAddr;

    DWORD                       _dwTimerId;

    DWORD                       _hListener;

    SeedAgentHandler          * _pSAHandler;
    DWORD                       _dwSAHConnId;

    RingBuffer<LP2PUploadHandler*>  *_pHandlerList;

    RingBuffer<LP2PSession*>  * _pSessionList;
    PCondEvent                * _pSessionEvent;

    pthread_rwlock_t            _hSTValidLock;
    vector<LP2P_SEEDTARGET_REC> _STList;
    VectorMate                  _STListMate;
    DWORD                       _dwSTMagic;

    // all uphs share the same lock
    pthread_mutex_t             _hUpHIdsLock;

    // all sessions share the same lock
    pthread_rwlock_t            _hSessionLock;
};


#endif
