/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _LP2P_SESSION_H
#define _LP2P_SESSION_H


#include "RingBuffer.h"
#include "BitArray.h"
#include "Endpoint.h"

class SeedTarget;
class LP2PMCastRecver;


class LP2PSessionObserver
{
    // serialization and synchronization
public:
    virtual void OnFinished(DWORD dwCookie, DWORD dwErrCode) = 0;

    virtual void OnEnumNodes4Seed(DWORD dwCookie) = 0;
};


class LP2PSession : protected Endpoint
{
public:
    LP2PSession();
    virtual ~LP2PSession();

    DWORD Load(SeedTarget *pSeedTarget, UINT nMaxPeerNum, LP2PSessionObserver *pObserver, DWORD dwCookie);
    virtual void Unload();

    void AddNodes4Seed(const PNODE4SEED_INFO pN4SArray, UINT nN4SCount, const PSOCK_ADDR pLocalAddr);

protected:
    virtual void OnListenerEnded(DWORD hListener);

    virtual EpProtocolHandler *OnNewHandler(DWORD hListener, Connection *pConnection, DWORD dwConnId);
    virtual void OnAsyncConnected(DWORD dwConnId, EpProtocolHandler *pHandler);
    virtual void OnDisconnected(DWORD dwConnId, EpProtocolHandler *pHandler);
    virtual void OnConnEvent(DWORD dwConnId, EpProtocolHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody);

    virtual void ProcessEpWork(UINT nSigIdx);

private:
    void ProcessTimer();

    void ProcessConnect();
    void ConnectToPeers();

    static void TimerFunc(HANDLE hArg);

private:
    LP2PSessionObserver       * _pObserver;
    DWORD                       _dwCookie;

    DWORD                       _dwTimerId;

    volatile UINT               _nReqN4SCnt;

    bool                        _bReported;
    DWORD                       _dwErrorCode;

    SeedTarget                * _pSeedTarget;
    const BitArray            * _pBitMask;

    pthread_mutex_t             _hAddrUsedLock;
    list<SOCK_ADDR>             _AddrUsedList;

    RingBuffer<SOCK_ADDR>     * _pPeerList;
    RingBuffer<EpProtocolHandler*>  * _pHandlerList;

#ifdef CONFIG_MCAST_SUPPORT
    bool                        _bMCastValid;
    SOCK_ADDR                   _MCastAddr;
    DWORD                       _dwMCastConnId;
    LP2PMCastRecver           * _pMCastHandler;
#endif
};


#endif
