/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _MISC_UTILS_H
#define _MISC_UTILS_H


#define  D_ERROR                1
#define  D_WARN                 1
#define  D_LOG                  1
#define  D_INFO                 1
#define  D_VERB                 0
#define  D_FUNC                 0
#define  D_PERF                 1

#if defined(_WIN32_) || defined(_LINUX_)
#define  err_wprintf(...)         fwprintf(stderr, __VA_ARGS__)
#define  log_wprintf(...)         do { fwprintf(stdout, __VA_ARGS__); fflush(stdout); } while(false)
#endif

#if defined(_WINCE_)
#define  DBG(cond,printf_exp)   ((void)((cond)?(NKDbgPrintfW printf_exp),1:0))
#endif

#if defined(_ANDROID_)
#define  TAG                    "Native"
void LOGI(const wchar_t *fmt, ...);
#define  DBG(cond,printf_exp)   ((void)((cond)?(LOGI printf_exp),1:0))
#endif

#if defined(_LINUX_)
#define  DBG(cond,printf_exp)   ((void)((cond)?(err_wprintf printf_exp),1:0))
#endif

#if defined(_WIN32_)
#ifndef _NO_STD_DBG_
#define  DBG(cond,printf_exp)   ((void)((cond)?(err_wprintf printf_exp),1:0))
#else
void TraceW(const wchar_t *fmt, ...);
#define  DBG(cond,printf_exp)   ((void)((cond)?(TraceW printf_exp),1:0))
#endif
#endif

#ifndef NDEBUG
#define DO_IF_DBG(expression)   ((void)(expression)) 
#else
#define DO_IF_DBG(expression)   ((void)0)
#endif

#if defined(_ANDROID_) || defined(_LINUX_)
#define  SET_PTH_INVALID(pth)   ((pth) = 0)
#define  IS_PTH_VALID(pth)      (pth)
#define  IS_PTH_EQUAL(pth1, pth2)  (pth1 == pth2)
#else
#define  SET_PTH_INVALID(pth)   ((pth).p = NULL)
#define  IS_PTH_VALID(pth)      ((pth).p)
#define  IS_PTH_EQUAL(pth1, pth2)  ((pth1).p == (pth2).p)
#endif

#define  bitsizeof(t)           (sizeof(t) * 8)

#define  ROUND_1K(n)            ((n + 1023) & (~1023))
#define  ROUND_4K(n)            ((n + 4095) & (~4095))

#define  CreateUniId()          (GetTickCount() + rand())

#define  MAKE_DWID(n, m)        MAKELONG(n, m)
#define  ADJUST_WMAGICNUM(m)    LOWORD(m)
#define  WIDX_FROM_DWID(x)      LOWORD(x)
#define  WMAGICNUM_FROM_DWID(x) HIWORD(x)

DWORD Round2Power(DWORD dwVal);

bool IsMCastAddr(const PSOCK_ADDR pAddr);

#define NTOHS(x)                MAKEWORD(HIBYTE(x), LOBYTE(x))
void addr2str(const PSOCK_ADDR pAddr, char *szBuf, UINT nSize);
bool operator == (const SOCK_ADDR &lhs, const SOCK_ADDR &rhs);
bool operator != (const SOCK_ADDR &lhs, const SOCK_ADDR &rhs);

bool operator == (const SHA1_VAL &lhs, const SHA1_VAL &rhs);
bool operator != (const SHA1_VAL &lhs, const SHA1_VAL &rhs);

bool PathHasFileName(const char *path);
void GetFileName(const char *path, char *buf, int size);
void GetFilePath(const char *path, char *buf, int size);

DWORD IsRegularFile(FILE *hFile, bool &rbRegular);

DWORD GetFileLength(FILE *hFile, QWORD &rqwSize);
DWORD SetFileLength(FILE *hFile, QWORD qwSize);

// Retrieves the number of seconds that have elapsed since the system was started
DWORD GetSecondCount();

QWORD GetPerformanceCounter();
QWORD GetPerformanceFrequency();

// -1: ts1<ts2, 0: ts1=ts2, 2: ts1>ts2
int TimeCompare(const timespec &ts1, const timespec &ts2);
void TimeAdd(timespec &ts, DWORD dwMilliseconds);

void InitStdIn();
DWORD ReadStdIn(char *pBuffer, UINT nBufSize, DWORD dwMilliseconds);
DWORD WaitStdIn(DWORD dwMilliseconds);

void InitRand();
// [nMin, nMax)
#ifdef _WIN32_
// see issues_rand.txt
UINT RAND_BETWEEN(UINT nMin, UINT nMax);
#else
#define  RAND_BETWEEN(MIN, MAX) (UINT)((double)rand()/(RAND_MAX+1.0)*(MAX-MIN)+MIN)
#endif

#endif
