/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _PRODUCER_QUEUE_H
#define _PRODUCER_QUEUE_H


#include "BufferItem.h"
#include "RingBuffer.h"


class ProducerQueue
{
public:
    ProducerQueue(UINT nQueueSize, UINT nBufferSize, bool bEnableLock);
    ~ProducerQueue();

    UINT GetQueueSize() const;
    UINT GetBufferSize() const;

    bool HasFreeItem() const;
    bool HasDataItem() const;

    BufferItem *FreeItem(UINT nReserveSize = 0);
    void PopFreeItem();
    BufferItem *GetFreeItem(UINT nReserveSize = 0, bool bLock = false);
    void PutFreeItem(BufferItem *pbi, bool bLock = false);

    BufferItem *DataItem();
    void PopDataItem();
    BufferItem *GetDataItem(bool bLock = false);
    void PutDataItem(BufferItem *pbi, bool bLock = false);

private:
    UINT                        _nQueueSize;
    UINT                        _nBufferSize;

    bool                        _bEnableLock;
    pthread_mutex_t             _hLock;

    RingBuffer<BufferItem*>     _FreeList;
    RingBuffer<BufferItem*>     _DataList;
};


#endif
