/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _REACTOR_EPOLL_H
#define _REACTOR_EPOLL_H


#include "VectorMate.h"
#include "RingBuffer.h"

class SocketConnection;

typedef enum _EPOLL_REQ
{
    EPOLL_REQ_NONE,
    EPOLL_REQ_DONE = EPOLL_REQ_NONE,
    EPOLL_REQ_REMOVE
}
EPOLL_REQ;

typedef struct _EPOLL_REC
{
#ifndef NDEBUG
    volatile bool               bUsed;
#endif

    volatile bool               bValid;
    WORD                        wMagic;

    SocketConnection          * pSocketConn;

    volatile EPOLL_REQ          CtlReq;

    volatile bool               bMsgSend;

    bool                        bRemoved;

    bool                        bRecvInd;
    bool                        bRecvReady;

    bool                        bSendInd;
    bool                        bSendReady;
}
EPOLL_REC, *PEPOLL_REC;

typedef struct _EPOLL_STATS
{
    LONGLONG                    llPollCnt;
    LONGLONG                    llModifyCnt;

    LONGLONG                    llYieldCnt;

    LONGLONG                    llMsgCallCnt;
    LONGLONG                    llMsgSendCnt;
}
EPOLL_STATS, *PEPOLL_STATS;


class ReactorEPoll
{
public:
    static ReactorEPoll *Instance();
    static void Destroy();

private:
    static ReactorEPoll       * _instance;

protected:
    ReactorEPoll();
    ~ReactorEPoll();

public:
    DWORD Load(UINT nMaxEPollNum = MAX_EPOLL_NUM);
    void Unload();

    DWORD Register(SocketConnection *pSocketConn, DWORD &rdwMuxId);
    DWORD Unregister(DWORD dwMuxId);

    void ResumePolling(DWORD dwMuxId);

private:
    DWORD AddEvent(SOCKET s, DWORD dwMagic, bool bRecvInd, bool bSendInd, bool bEdgeTrigger);
    DWORD ModifyEvent(SOCKET s, DWORD dwMagic, bool bRecvInd, bool bSendInd, bool bEdgeTrigger);
    void RemoveEvent(SOCKET s);

    DWORD SendMuxMsg(DWORD dwMuxId, PEPOLL_REC pEPRec);

    void ProcessMuxMsg(DWORD dwMuxId);

    void ProcessPipeEvent(DWORD dwEvents);
    void ProcessMuxEvent(DWORD dwMuxId, DWORD dwEvents);

    DWORD MuxThread();
    static void *MuxThreadProc(void *arg);

private:
    int                         _hEPoll;

    SOCKET                      _hPipeSend;
    SOCKET                      _hPipeRecv;

    volatile bool               _bQuit;
    pthread_t                   _hMuxThread;

    vector<EPOLL_REC>           _MuxList;
    VectorMate                  _MuxListMate;
    DWORD                       _dwMuxMagic;

    RingBuffer<DWORD>           _YieldList;

#ifndef NDEBUG
    EPOLL_STATS                 _stats;
#endif
};


#endif
