/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _REACTOR_SELECT_H
#define _REACTOR_SELECT_H


#include "RingBuffer.h"

class SocketConnection;

typedef struct _SELECT_STATS
{
    LONGLONG                    llPollCnt;

    LONGLONG                    llMsgCallCnt;
    LONGLONG                    llMsgSendCnt;
}
SELECT_STATS, *PSELECT_STATS;


class ReactorSelect
{
public:
    static ReactorSelect *Instance();
    static void Destroy();

private:
    static ReactorSelect      * _instance;

protected:
    ReactorSelect();
    ~ReactorSelect();

public:
    DWORD Load(UINT nMaxSelectNum = REACTOR_SELECT_LIMIT);
    void Unload();

    DWORD Register(SocketConnection *pSocketConn);
    void Unregister(SocketConnection *pSocketConn);

    void ResumePolling();

private:
    DWORD MuxThread();
    static void *MuxThreadProc(void *arg);

private:
    SOCKET                      _hPipeSend;
    SOCKET                      _hPipeRecv;

    volatile bool               _bMuxQuit;
    pthread_t                   _hMuxThread;

    volatile bool               _bPolling;
    volatile UINT               _nPollCnt;
    pthread_mutex_t             _hMuxLock;
    list<SocketConnection*>     _MuxList;

#ifndef NDEBUG
    SELECT_STATS                _stats;
#endif
};


#endif
