/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _SECURE_FILTER_AGENT_H
#define _SECURE_FILTER_AGENT_H


#include <openssl/ssl.h>
#include "FilterProtocol.h"


class SecureFilterAgent : public FilterProtocol
{
public:
    SecureFilterAgent(Connection *pConnection, bool bServer);
    virtual ~SecureFilterAgent();

    virtual DWORD Attach(ProtocolHandler *pHandler, UINT nRecvBufCnt, UINT nSendItemCnt);
    virtual void Detach();

    virtual void ResumeRecv(UINT nBufCnt);
    virtual void ResumeSend(UINT nItemCnt);

    virtual DWORD Recv(BufferItem *pBufItem);
    virtual DWORD Send(BufferItem *pBufItem);

public:
    virtual void OnAsyncConnected(Connection *pConnection);

    virtual UINT OnRecvReady(Connection *pConnection);
    virtual UINT OnSendReady(Connection *pConnection);

private:
    DWORD InitSSLEnv();
    void FiniSSLEnv();

    void ContinueHandShake();
    void EnterSecureState();

    int ReadBIOSend();

private:
    bool                        _bServer;

    SSL_CTX                   * _ctx;
    SSL                       * _ssl;
    BIO                       * _BIORecv;
    BIO                       * _BIOSend;

    pthread_mutex_t             _hStateLock;
    bool                        _bSecureReady;
    UINT                        _nRecvBufCnt;
    UINT                        _nSendItemCnt;

    BufferItem                  _biSendBuf;
    BufferItem                  _biRecvBuf;
};


#endif
