/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _SEED_AGENT_HANDLER_H
#define _SEED_AGENT_HANDLER_H


#include "VectorMate.h"
#include "EpPDUHandler.h"
#include "SeedTarget.h"

typedef struct _SA_SEEDTARGET_REC
{
#ifndef NDEBUG
    volatile bool               bUsed;
#endif

    volatile bool               bValid;
    volatile WORD               wMagic;

    bool                        bError;
    volatile DWORD              dwFlags;

    SeedTarget                * pSeedTarget;
    DWORD                       dwCookie;

    DWORD                       dwN4SId;
    SEED_STATE                  SeedState;
}
SA_SEEDTARGET_REC, *PSA_SEEDTARGET_REC;


class SeedAgentHandler : public EpPDUHandler, public SeedTargetObserver
{
public:
    SeedAgentHandler(const char *szNodeName, const PSOCK_ADDR pNodeAddr, UINT nMaxSTNum);
    virtual ~SeedAgentHandler();

    virtual DWORD Bind(Connection *pConnection, PCondEvent *pEvent, UINT nSigIdx);
    virtual void Unbind();

    virtual void OnAsyncConnected(Connection *pConnection);

    virtual void PrepareSend();

    virtual void ReleaseEvent(BufferItem *pbiEvt);

    DWORD AddSeedTarget(SeedTarget *pSeedTarget, DWORD dwCookie, DWORD &rdwSTId);
    DWORD RemoveSeedTarget(DWORD dwSTId);

    DWORD EnumNodes4Seed(DWORD dwSTId);

    virtual void OnStateUpdated(SeedTarget *pSeedTarget, DWORD dwCookie, UINT nDegree, QWORD qwMaskAnd, QWORD qwMaskOr);

protected:
    virtual void ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, BufferItem *pbiBody);
    virtual void ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, const PBYTE pData, UINT nDataBytes, UINT nLeftBytes);

private:
    bool OnRetRegisterNode(DWORD dwResult, PRET_REG_NODE pRetRegNode);
    bool OnRetAddSeed(DWORD dwResult, PRET_ADD_NODE4SEED pRetAddSeed);
    void OnRetRemoveSeed(DWORD dwResult);
    bool OnRetUpdateSeedState(DWORD dwResult);
    bool OnRetEnumNodes4Seed(DWORD dwResult, PRET_ENUM_NODE4SEED pRetEnumN4S);

    void ReqTrackProc(UINT nIdx);

    void DoTrackWork();
    void TrackNode();
    bool TrackSeed(PSA_SEEDTARGET_REC pSTRec, bool &bRemove);

private:
    pthread_mutex_t             _hLock;

    volatile bool               _bBound;
    volatile bool               _bBusy;

    NODE_INFO                   _NodeInfo;
    DWORD                       _dwNodeId;

    UINT                        _nCurST;
    PCondEvent                  _STEvent;
    vector<SA_SEEDTARGET_REC>   _STList;
    VectorMate                  _STListMate;
    DWORD                       _dwSTMagic;
};


#endif
