/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _SEED_CENTER_H
#define _SEED_CENTER_H


#include "SeedTrackObserver.h"

typedef struct _NODE_INFO_REC
{
    volatile bool               bUsed;
    DWORD                       dwMagic;

    NODE_INFO                   NodeInfo;

    UINT                        nSeedCnt;
    DWORD                       dwActiveSecs;
}
NODE_INFO_REC, *PNODE_INFO_REC;

typedef struct _SEED_INFO_REC
{
    UINT                        nRefCnt;

    SEED_INFO                   SeedInfo;
}
SEED_INFO_REC, *PSEED_INFO_REC;

typedef struct _N4S_STATE_REC
{
    volatile bool               bUsed;
    DWORD                       dwMagic;

    BYTE                        bPreDegree;
    SEED_STATE                  SeedState;
}
N4S_STATE_REC, *PN4S_STATE_REC;


class SeedCenter
{
public:
    static SeedCenter *Instance();
    static void Destroy();

private:
    static SeedCenter         * _instance;

protected:
    SeedCenter();
    virtual ~SeedCenter();

public:
    DWORD Load(UINT nMaxNodeNum, UINT nMaxSeedNum);
    void Unload();

    void RegisterObserver(SeedTrackObserver *pObserver);
    void UnregisterObserver(SeedTrackObserver *pObserver);

    DWORD RegisterNode(const PNODE_INFO pNodeInfo, DWORD &rdwNodeId);
    DWORD UnregisterNode(DWORD dwNodeId);

    DWORD AddNode4Seed(DWORD dwNodeId, const PSEED_INFO pSeedInfo, DWORD &rdwN4SId);
    DWORD RemoveNode4Seed(DWORD dwN4SId);
    DWORD UpdateNode4SeedState(DWORD dwN4SId, const PSEED_STATE pSeedState);

    DWORD EnumNodes4Seed(DWORD dwN4SId, UINT &nStartIdx, PNODE4SEED_INFO pN4SInfoArray, UINT &nCount);

private:
    DWORD GetFreeNode(const PSOCK_ADDR pAddr, UINT &rnIdx);
    DWORD GetFreeSeed(const PSHA1_VAL pSHA1, UINT &rnIdx);
    void PutSeed(UINT nIdx, const PSEED_INFO pSeedInfo);
    void ReleaseSeed(UINT nIdx);

private:
    pthread_rwlock_t            _hNSLock;
    pthread_mutex_t             _hRdLock;

    DWORD                       _dwNodeMagic;
    UINT                        _nNodeCnt;
    vector<NODE_INFO_REC>       _NodeList;

    UINT                        _nSeedCnt;
    vector<SEED_INFO_REC>       _SeedList;

    DWORD                       _dwStateMagic;
    vector<vector<N4S_STATE_REC> >  _StateMatrix;

    list<SeedTrackObserver*>    _ObsList;
};


#endif
