/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _SOCKET_CONNECTION_H
#define _SOCKET_CONNECTION_H


#include "Connection.h"
#include "ProducerCounter.h"


class SocketConnection : public Connection
{
    friend class ReactorSelect;
    friend class ReactorEPoll;
    friend class ConnectCenter;

public:
    SocketConnection(SOCKET s, bool bListener, bool bAsyncConnect);
    virtual ~SocketConnection();

    virtual bool ReqRecvInd() const;
    virtual bool ReqSendInd() const;

    virtual void OnAsyncConnected();

    virtual void OnDisconnectedOrError();

    virtual void OnRecvReady(bool *pbEAgain = NULL, bool *pbYield = NULL);
    virtual void OnSendReady(bool *pbEAgain = NULL, bool *pbYield = NULL);

    virtual DWORD Attach(ProtocolHandler *pHandler, UINT nRecvBufCnt, UINT nSendItemCnt);
    virtual void Detach();

    virtual DWORD EnableKeepAlive(UINT nInterval);

    // not support reentry
    virtual void ResetRecv(UINT nBufCnt);
    virtual void ResetSend(UINT nItemCnt);
    virtual void ResumeRecv(UINT nBufCnt);
    virtual void ResumeSend(UINT nItemCnt);

    virtual DWORD GetLocalAddr(PSOCK_ADDR pAddr);
    virtual DWORD GetPeerAddr(PSOCK_ADDR pAddr);

    virtual DWORD SetRecvBufSize(DWORD dwSize);
    virtual DWORD SetSendBufSize(DWORD dwSize);

    virtual DWORD PollRecv(DWORD dwMilliseconds);
    virtual DWORD PollSend(DWORD dwMilliseconds);

    virtual DWORD Recv(BufferItem *pBufItem);
    virtual DWORD Send(BufferItem *pBufItem);

protected:
    SOCKET                      _s;
    bool                        _bIgnore;

    // init by SocketConnection, used by ReactorSelect/EPoll
    bool                        _bListener;
    bool                        _bAsyncConnect;

#ifdef CONFIG_REACTOR_SELECT
    // used by ReactorSelect, for fast and avoid conflict:
    // when select@MuxThread return, the socket in rd_fds maybe closed by thread A and reopened by thread B.
    // They will have the same socket number, so thread B do Recv@OnRecvReady, no data arrived, so MuxThread will stuck.
    bool                        _bPollRecv;
    bool                        _bPollSend;
#endif

#ifdef CONFIG_REACTOR_EPOLL
    DWORD                       _dwMuxId;
    UINT                        _nYieldNum;
#endif

    ProtocolHandler           * _pHandler;

    ProducerCounter             _RecvCounter;
    ProducerCounter             _SendCounter;
};


#endif
