/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _TIMER_CENTER_H
#define _TIMER_CENTER_H


#include "RingBuffer.h"
#include "VectorMate.h"

typedef void (*TIMERFUNC)(HANDLE);

typedef struct _TIMER_REC
{
#ifndef NDEBUG
    volatile bool               bUsed;
#endif
    volatile bool               bReqDel;
    volatile bool               bBusy;

    volatile WORD               wMagic;
    volatile DWORD              dwEntryMagic;

    TIMERFUNC                   pTimerFunc;
    HANDLE                      hTimerArg;
}
TIMER_REC, *PTIMER_REC;

typedef struct _TIMER_ENTRY
{
    UINT                        nTimerIdx;
    DWORD                       dwMagic;

    timespec                    tsTime;
    
    bool operator < (const _TIMER_ENTRY &rhs) const
    {
        if ( tsTime.tv_sec == rhs.tsTime.tv_sec )
        {
            return (tsTime.tv_nsec > rhs.tsTime.tv_nsec);
        }

        return (tsTime.tv_sec > rhs.tsTime.tv_sec);
    }
}
TIMER_ENTRY, *PTIMER_ENTRY;

struct pless : public binary_function<TIMER_ENTRY *, TIMER_ENTRY *, bool>
{
    bool operator()(const TIMER_ENTRY *x, const TIMER_ENTRY *y) const
    {
        return less<TIMER_ENTRY>()(*x, *y);
    }
};

class TimerCenter
{
public:
    static TimerCenter *Instance();
    static void Destroy();

private:
    static TimerCenter        * _instance;

protected:
    TimerCenter();
    ~TimerCenter();

public:
    DWORD Load(UINT nMaxTimerNum = MAX_TIMER_NUM);
    void Unload();

    DWORD CreateTimer(TIMERFUNC pTimerFunc, HANDLE hTimerArg, DWORD &rdwTimerId);
    DWORD DestroyTimer(DWORD dwTimerId);

    DWORD SetTimer(DWORD dwTimerId, DWORD dwMilliseconds);

private:
    void DoTimerEntry(PTIMER_ENTRY pTimerEntry);

    DWORD TimerThread();
    static void *TimerThreadProc(void *arg);

private:
    volatile bool               _bQuit;
    sem_t                       _hEvent;
    pthread_t                   _hTimerThread;

    pthread_mutex_t             _hTimerDelLock;
    vector<TIMER_REC>           _TimerList;
    VectorMate                  _TimerListMate;

    pthread_mutex_t             _hEntryLock;
    DWORD                       _dwMagicBase;
    RingBuffer<PTIMER_ENTRY>  * _pEntryPool;
    priority_queue<TIMER_ENTRY*, vector<TIMER_ENTRY*>, pless>   _EntryQueue;
};


#endif
