/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "EpPDUHandler.h"


EpPDUHandler::EpPDUHandler
        (
            UINT nRecvQSize, UINT nRecvBSize,
            UINT nSendQSize, UINT nSendBSize,
            UINT nEvtQSize, UINT nEvtBSize, bool bEvtLock
        )
    : EpProtocolHandler(nRecvQSize, nRecvBSize,
                        nSendQSize, nSendBSize,
                        nEvtQSize, nEvtBSize, bEvtLock)
    , _biHdr(sizeof(PDU_COMM_HDR))
    , _pHdr(NULL)
    , _nCurBodySize(0)
    , _biBody(MAX_PDU_SIZE)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _pHdr = (PPDU_COMM_HDR)_biHdr.data();
}

EpPDUHandler::~EpPDUHandler()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

void EpPDUHandler::Unbind()
{
    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));

    EpProtocolHandler::Unbind();

    _biHdr.Reset();
}

bool EpPDUHandler::ProcessRecvData(BufferItem *pbiData)
{
    UINT  nBodySize, nCopySize, nLeftSize;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    while ( pbiData->data_size() )
    {
        if ( _biHdr.data_size() < sizeof(PDU_COMM_HDR) )
        {
            nCopySize = min(_biHdr.tail_size(), pbiData->data_size());
            memcpy(_biHdr.Put(nCopySize), pbiData->data(), nCopySize);
            pbiData->Pull(nCopySize);

            if ( _biHdr.data_size() < sizeof(PDU_COMM_HDR) )
            {
                return true;
            }

            if ( _pHdr->dwMagic != htonl(PDU_MAGIC_NUM) )
            {
                DBG(D_WARN, (L"%s: invalid magic number\n", FULL_FUNC_NAME));
                assert(0);
                ReqEndSession();
                return true;
            }

            _pHdr->wBodySize = ntohs(_pHdr->wBodySize);
            _pHdr->dwParam = ntohl(_pHdr->dwParam);

            _nCurBodySize = 0;
            _biBody.Reset(PDU_HEAD_RESERVE);
        }


        // wBodySize maybe zero
        nBodySize = (UINT)_pHdr->wBodySize;

        nLeftSize = nBodySize - _nCurBodySize;
        nCopySize = min(nLeftSize, pbiData->data_size());

        nLeftSize -= nCopySize;
        _nCurBodySize += nCopySize;

        if ( _pHdr->bBodyPDU )
        {
            memcpy(_biBody.Put(nCopySize), pbiData->data(), nCopySize);
            pbiData->Pull(nCopySize);

            if ( nLeftSize )
            {
                return true;
            }

            ProcessRecvPDU(_pHdr->bCode,_pHdr->bReq, _pHdr->dwParam, &_biBody);
        }
        else
        {
            ProcessRecvPDU(_pHdr->bCode,_pHdr->bReq, _pHdr->dwParam, pbiData->data(), nCopySize, nLeftSize);
            pbiData->Pull(nCopySize);

            if ( nLeftSize )
            {
                return true;
            }
        }

        _biHdr.Reset();
    }

    return true;
}

void EpPDUHandler::BuildPDU(BYTE bCode, bool bReq, bool bBodyPDU, DWORD dwParam, BufferItem *pbiBody)
{
    WORD  wBodySize;
    PPDU_COMM_HDR  pHdr;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(pbiBody->data_size() <= MAX_PDU_SIZE);
    wBodySize = (WORD)pbiBody->data_size();

    assert(pbiBody->head_size() >= sizeof(PDU_COMM_HDR));
    pHdr = (PPDU_COMM_HDR)pbiBody->Push(sizeof(PDU_COMM_HDR));

    pHdr->dwMagic = htonl(PDU_MAGIC_NUM);
    pHdr->wBodySize = htons(wBodySize);
    pHdr->bCode = bCode;
    pHdr->bReq = bReq;
    pHdr->bBodyPDU = bBodyPDU;
    pHdr->dwParam = htonl(dwParam);
}
