/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "FilterProtocol.h"


FilterProtocol::FilterProtocol(Connection *pConnection)
    : _pConnection(pConnection)
    , _pHandler(NULL)
{
    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));
}

FilterProtocol::~FilterProtocol()
{
    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));

    // FIXME, may in stack not heap
//    ConnectCenter::Instance()->FinishConnection(_pConnection);
    delete _pConnection;
}

DWORD FilterProtocol::Attach(ProtocolHandler *pHandler, UINT nRecvBufCnt, UINT nSendItemCnt)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _pHandler = pHandler;

    return _pConnection->Attach(this, nRecvBufCnt, nSendItemCnt);
}

void FilterProtocol::Detach()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _pConnection->Detach();

    _pHandler = NULL;
}

DWORD FilterProtocol::EnableKeepAlive(UINT nInterval)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->EnableKeepAlive(nInterval);
}

void FilterProtocol::ResetRecv(UINT nBufCnt)
{
    DBG(D_INFO, (L"%s: %d\n", FULL_FUNC_NAME, nBufCnt));

    _pConnection->ResetRecv(nBufCnt);
}

void FilterProtocol::ResetSend(UINT nItemCnt)
{
    DBG(D_INFO, (L"%s: %d\n", FULL_FUNC_NAME, nItemCnt));

    _pConnection->ResetSend(nItemCnt);
}

void FilterProtocol::ResumeRecv(UINT nBufCnt)
{
    DBG(D_VERB, (L"%s: %d\n", FULL_FUNC_NAME, nBufCnt));

    _pConnection->ResumeRecv(nBufCnt);
}

void FilterProtocol::ResumeSend(UINT nItemCnt)
{
    DBG(D_VERB, (L"%s: %d\n", FULL_FUNC_NAME, nItemCnt));

    _pConnection->ResumeSend(nItemCnt);
}

DWORD FilterProtocol::GetLocalAddr(PSOCK_ADDR pAddr)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->GetLocalAddr(pAddr);
}

DWORD FilterProtocol::GetPeerAddr(PSOCK_ADDR pAddr)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->GetPeerAddr(pAddr);
}

DWORD FilterProtocol::SetRecvBufSize(DWORD dwSize)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->SetRecvBufSize(dwSize);
}

DWORD FilterProtocol::SetSendBufSize(DWORD dwSize)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->SetSendBufSize(dwSize);
}

DWORD FilterProtocol::PollRecv(DWORD dwMilliseconds)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->PollRecv(dwMilliseconds);
}

DWORD FilterProtocol::PollSend(DWORD dwMilliseconds)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->PollSend(dwMilliseconds);
}

DWORD FilterProtocol::Recv(BufferItem *pBufItem)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->Recv(pBufItem);
}

DWORD FilterProtocol::Send(BufferItem *pBufItem)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pConnection->Send(pBufItem);
}

void FilterProtocol::OnAsyncConnected(Connection *pConnection)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _pHandler->OnAsyncConnected(this);
}

void FilterProtocol::OnDisconnectedOrError(Connection *pConnection)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    _pHandler->OnDisconnectedOrError(this);
}

UINT FilterProtocol::OnRecvReady(Connection *pConnection)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pHandler->OnRecvReady(this);
}

UINT FilterProtocol::OnSendReady(Connection *pConnection)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    return _pHandler->OnSendReady(this);
}
