/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include <openssl/err.h>
#include "SecureCenter.h"


SecureCenter *SecureCenter::_instance = NULL;

SecureCenter *SecureCenter::Instance()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    if ( !_instance )
    {
        DBG(D_VERB, (L"%s: %ls\n", FULL_FUNC_NAME, L"CreateInstance"));
        _instance = new SecureCenter();
    }
    return _instance;
}

void SecureCenter::Destroy()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    delete _instance;
    _instance = NULL;
}

SecureCenter::SecureCenter()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));
}

SecureCenter::~SecureCenter()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));
}

DWORD SecureCenter::Load(const char *szCAPath, const char *szCertPath, const char *szKeyPath)
{
    DBG(D_LOG, (L"%s: CA(%hs), Cert(%hs), Key(%hs)\n", FULL_FUNC_NAME, szCAPath, szCertPath, szKeyPath));

    _CAPath = szCAPath ? szCAPath : (getenv(ENV_CA_PATH) ? getenv(ENV_CA_PATH) : "");
    _CertPath = szCertPath ? szCertPath : (getenv(ENV_CERT_PATH) ? getenv(ENV_CERT_PATH) : "");
    _KeyPath = szKeyPath ? szKeyPath : (getenv(ENV_KEY_PATH) ? getenv(ENV_KEY_PATH) : "");

#ifdef CONFIG_TLS_SUPPORT
    SSL_library_init();
#endif

    return ERROR_SUCCESS;
}

void SecureCenter::Unload()
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));
}

DWORD SecureCenter::InstallRootCert(SSL_CTX *ctx)
{
    DBG(D_INFO, (L"%s: CA(%hs)\n", FULL_FUNC_NAME, _CAPath.c_str()));

    if ( SSL_CTX_load_verify_locations(ctx, _CAPath.c_str(), NULL) != 1 )
    {
        DBG(D_ERROR, (L"%s: SSL_CTX_load_verify_locations(%hs) failed: %lu\n", FULL_FUNC_NAME,  _CAPath.c_str(), ERR_get_error()));
        goto err;
    }

    return ERROR_SUCCESS;

err:
    return ERROR_NOT_READY;

}

DWORD SecureCenter::InstallCertAndKey(SSL_CTX *ctx)
{
    DBG(D_INFO, (L"%s: cert(%hs), key(%hs)\n", FULL_FUNC_NAME, _CertPath.c_str(), _KeyPath.c_str()));

    if ( SSL_CTX_use_certificate_file(ctx, _CertPath.c_str(), SSL_FILETYPE_PEM) != 1 )
    {
        DBG(D_ERROR, (L"%s: SSL_CTX_use_certificate_file(%hs) failed: %lu\n", FULL_FUNC_NAME,  _CertPath.c_str(), ERR_get_error()));
        goto err;
    }

    if ( SSL_CTX_use_PrivateKey_file(ctx, _KeyPath.c_str(), SSL_FILETYPE_PEM) != 1 )
    {
        DBG(D_ERROR, (L"%s: SSL_CTX_use_PrivateKey_file(%hs) failed: %lu\n", FULL_FUNC_NAME,  _KeyPath.c_str(), ERR_get_error()));
        goto err;
    }

    if ( SSL_CTX_check_private_key(ctx) != 1 )
    {
        DBG(D_ERROR, (L"%s: SSL_CTX_check_private_key failed: %lu\n", FULL_FUNC_NAME,  ERR_get_error()));
        goto err;
    }

    return ERROR_SUCCESS;

err:
    return ERROR_NOT_READY;
}
