/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "UdpConnectionIn.h"


UdpConnectionIn::UdpConnectionIn(SOCKET s)
    : SocketConnection(s, false, false)
    , _bBounded(false)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

UdpConnectionIn::~UdpConnectionIn()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

DWORD UdpConnectionIn::Recv(BufferItem *pBufItem)
{
    DWORD  dwRet;
    int  len;
    socklen_t  addrlen;
    struct sockaddr_in  addr;

    DBG(D_FUNC, (L"%s: %d @ %p\n", FULL_FUNC_NAME, pBufItem->tail_size(), pBufItem->tail()));

    if ( _bBounded )
    {
        return SocketConnection::Recv(pBufItem);
    }

    addrlen = sizeof(addr);
    len = recvfrom(_s, (char*)pBufItem->tail(), pBufItem->tail_size(), 0, (struct sockaddr*)&addr, &addrlen);
    if ( len <= 0 )
    {
        dwRet = (len < 0) ? WSAGetLastError() : ERROR_NOT_CONNECTED;
        DBG(D_WARN, (L"%s: recvfrom failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }

    if ( connect(_s, (struct sockaddr*)&addr, sizeof(addr)) == SOCKET_ERROR )
    {
        dwRet = WSAGetLastError();
        DBG(D_WARN, (L"%s: connect failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }

    _bBounded = true;

    pBufItem->Put(len);

    return ERROR_SUCCESS;
}

