/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "SeedMonCliHandler.h"


SeedMonCliHandler::SeedMonCliHandler(SeedTrackObserver *pObserver, UINT nRecvQSize)
    : EpPDUHandler(nRecvQSize, SEED_MON_NETBSIZE, 0, 0, 0, 0)
    , _pObserver(pObserver)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

SeedMonCliHandler::~SeedMonCliHandler()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

void SeedMonCliHandler::PrepareSend()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));
}

void SeedMonCliHandler::ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, BufferItem *pbiBody)
{
    DBG(D_VERB, (L"%s: bCode=%d, dwParam=%d\n", FULL_FUNC_NAME, bCode, dwParam));

    assert(!bReq);

    switch ( bCode )
    {
        case C_SM_ON_NODE_REG:
            assert(pbiBody->data_size() == sizeof(NODE_INFO));
            _pObserver->OnNodeRegistered((UINT)dwParam, (PNODE_INFO)pbiBody->data());
            break;

        case C_SM_ON_NODE_UNREG:
            assert(!pbiBody->data_size());
            _pObserver->OnNodeUnregistered((UINT)dwParam);
            break;

        case C_SM_ON_SEED_ADD:
            assert(pbiBody->data_size() == sizeof(SEED_INFO));
            _pObserver->OnSeedAdded((UINT)dwParam, (PSEED_INFO)pbiBody->data());
            break;

        case C_SM_ON_SEED_REMOVE:
            assert(!pbiBody->data_size());
            _pObserver->OnSeedRemoved((UINT)dwParam);
            break;

        case C_SM_ON_N4S_ADD:
        {
            assert(pbiBody->data_size() == sizeof(EVT_NODE4SEED_OP));
            PEVT_NODE4SEED_OP  pN4SAdd = (PEVT_NODE4SEED_OP)pbiBody->data();
            _pObserver->OnNode4SeedAdded((UINT)pN4SAdd->dwNodeIdx, (UINT)pN4SAdd->dwSeedIdx);
            break;
        }

        case C_SM_ON_N4S_REMOVE:
        {
            assert(pbiBody->data_size() == sizeof(EVT_NODE4SEED_OP));
            PEVT_NODE4SEED_OP  pN4SRemove = (PEVT_NODE4SEED_OP)pbiBody->data();
            _pObserver->OnNode4SeedRemoved((UINT)pN4SRemove->dwNodeIdx, (UINT)pN4SRemove->dwSeedIdx);
            break;
        }

        case C_SM_ON_N4S_STATE_UPDATE:
        {
            assert(pbiBody->data_size() == sizeof(EVT_NODE4SEED_STATE_UP));
            PEVT_NODE4SEED_STATE_UP  pN4SStateUp = (PEVT_NODE4SEED_STATE_UP)pbiBody->data();
            _pObserver->OnNode4SeedStateUpdated((UINT)pN4SStateUp->dwNodeIdx, (UINT)pN4SStateUp->dwSeedIdx, &pN4SStateUp->SeedState);
            break;
        }

        default:
            DBG(D_WARN, (L"%s: unknown code: %d\n", FULL_FUNC_NAME, bCode));
            assert(0);
            break;
    }
}

void SeedMonCliHandler::ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, const PBYTE pData, UINT nDataBytes, UINT nLeftBytes)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    assert(0);
}

