/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "SeedMonCliHandler.h"
#include "SeedMonClient.h"


SeedMonClient *SeedMonClient::_instance = NULL;

SeedMonClient *SeedMonClient::Instance()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    if ( !_instance )
    {
        DBG(D_VERB, (L"%s: %ls\n", FULL_FUNC_NAME, L"CreateInstance"));
        _instance = new SeedMonClient();
    }
    return _instance;
}

void SeedMonClient::Destroy()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    delete _instance;
    _instance = NULL;
}

SeedMonClient::SeedMonClient()
    : Endpoint()
    , _bQuit(false)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

SeedMonClient::~SeedMonClient()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

DWORD SeedMonClient::Load(const char *szIP, SeedTrackObserver *pObserver)
{
    DWORD  dwRet;
    SeedMonCliHandler  *pHandler;

    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

    dwRet = Endpoint::Load(2, 0, 0);
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_WARN, (L"%s: Endpoint::Load() failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err1;
    }

    _bQuit = false;

    _TrackerAddr.bType = ADDR_TYPE_IPV4;
    _TrackerAddr.IPv4Addr.bStream = 1;
    _TrackerAddr.IPv4Addr.dwIP = inet_addr(szIP);
    _TrackerAddr.IPv4Addr.wPort = htons(SEED_MON_PORT);

    pHandler = new(std::nothrow) SeedMonCliHandler(pObserver);
    if ( !pHandler )
    {
        dwRet = ERROR_OUTOFMEMORY;
        DBG(D_WARN, (L"%s: alloc SeedMonCliHandler failed\n", FULL_FUNC_NAME));
        goto err2;
    }

    dwRet = DoConnect(pHandler);
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_WARN, (L"%s: ConnectOut failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err3;
    }

    return ERROR_SUCCESS;

err3:
    delete pHandler;

err2:
    Endpoint::Unload();

err1:
    return dwRet;
}

void SeedMonClient::Unload()
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

    _bQuit = true;

    Endpoint::Unload();
}

void SeedMonClient::OnListenerEnded(DWORD hListener)
{
    DBG(D_LOG, (L"%s: %x\n", FULL_FUNC_NAME, hListener));

    assert(0);
}

EpProtocolHandler *SeedMonClient::OnNewHandler(DWORD hListener, Connection *pConnection, DWORD dwConnId)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    assert(0);

    return NULL;
}

void SeedMonClient::OnAsyncConnected(DWORD dwConnId, EpProtocolHandler *pHandler)
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));
}

void SeedMonClient::OnDisconnected(DWORD dwConnId, EpProtocolHandler *pHandler)
{
    DWORD  dwRet;

    DBG(D_INFO, (L"%s: id(%x)\n", FULL_FUNC_NAME, dwConnId));

    if ( _bQuit )
    {
        delete pHandler;
        return;
    }

    DBG(D_LOG, (L"%s: wait and try again\n", FULL_FUNC_NAME));
    Sleep(1000);

    dwRet = DoConnect(pHandler);
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_WARN, (L"%s: ConnectOut failed: %d\n", FULL_FUNC_NAME, dwRet));
        delete pHandler;
    }
}

void SeedMonClient::OnConnEvent(DWORD dwConnId, EpProtocolHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody)
{
    DBG(D_INFO, (L"%s: id(%#.8x), code(%d)\n", FULL_FUNC_NAME, dwConnId, pEvtHdr->nCode));

    assert(0);
}

void SeedMonClient::ProcessEpWork(UINT nSigIdx)
{
    DBG(D_INFO, (L"%s: %d\n", FULL_FUNC_NAME, nSigIdx));

    assert(0);
}

DWORD SeedMonClient::DoConnect(EpProtocolHandler *pHandler)
{
    DWORD  dwConnId;
    SOCK_ADDR  LocalAddr;

    DBG(D_INFO, (L"%s: -> %#.8x/%d\n", FULL_FUNC_NAME, _TrackerAddr.IPv4Addr.dwIP, ntohs(_TrackerAddr.IPv4Addr.wPort)));

    LocalAddr.bType = ADDR_TYPE_IPV4;
    LocalAddr.IPv4Addr.bStream = 1;
    LocalAddr.IPv4Addr.dwIP = INADDR_ANY;
    LocalAddr.IPv4Addr.wPort = htons(0);

    return ConnectOut(&LocalAddr, &_TrackerAddr, pHandler, dwConnId);
}
