/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "SeedMonSvrHandler.h"


SeedMonSvrHandler::SeedMonSvrHandler(UINT nSendQSize)
    : EpPDUHandler(0, 0, nSendQSize, SEED_MON_NETBSIZE, 0, 0)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

SeedMonSvrHandler::~SeedMonSvrHandler()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

DWORD SeedMonSvrHandler::Bind(Connection *pConnection, PCondEvent *pEvent, UINT nSigIdx)
{
    DWORD  dwRet;

    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));

    dwRet = EpPDUHandler::Bind(pConnection, pEvent, nSigIdx);
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: Bind failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }

    SeedCenter::Instance()->RegisterObserver(this);

    return ERROR_SUCCESS;
}

void SeedMonSvrHandler::Unbind()
{
    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));

    SeedCenter::Instance()->UnregisterObserver(this);

    EpPDUHandler::Unbind();
}

void SeedMonSvrHandler::PrepareSend()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));
}

void SeedMonSvrHandler::OnNodeRegistered(UINT nNodeIdx, const PNODE_INFO pNodeInfo)
{
    BufferItem  *pbi;

    DBG(D_INFO, (L"%s: idx(%d), name(%hs)\n", FULL_FUNC_NAME, nNodeIdx, pNodeInfo->name));

    pbi = AllocSendBuf(true);
    if ( pbi )
    {
        memcpy(pbi->Put(sizeof(NODE_INFO)), pNodeInfo, sizeof(NODE_INFO));

        BuildPDU(C_SM_ON_NODE_REG, false, true, (DWORD)nNodeIdx, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::OnNodeUnregistered(UINT nNodeIdx)
{
    BufferItem  *pbi;

    DBG(D_INFO, (L"%s: idx(%d)\n", FULL_FUNC_NAME, nNodeIdx));

    pbi = AllocSendBuf(true);
    if ( pbi )
    {
        BuildPDU(C_SM_ON_NODE_UNREG, false, true, (DWORD)nNodeIdx, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::OnSeedAdded(UINT nSeedIdx, const PSEED_INFO pSeedInfo)
{
    BufferItem  *pbi;

    DBG(D_INFO, (L"%s: idx(%d), name(%hs)\n", FULL_FUNC_NAME, nSeedIdx, pSeedInfo->name));

    pbi = AllocSendBuf(true);
    if ( pbi )
    {
        memcpy(pbi->Put(sizeof(SEED_INFO)), pSeedInfo, sizeof(SEED_INFO));

        BuildPDU(C_SM_ON_SEED_ADD, false, true, (DWORD)nSeedIdx, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::OnSeedRemoved(UINT nSeedIdx)
{
    BufferItem  *pbi;

    DBG(D_INFO, (L"%s: idx(%d)\n", FULL_FUNC_NAME, nSeedIdx));

    pbi = AllocSendBuf(true);
    if ( pbi )
    {
        BuildPDU(C_SM_ON_SEED_REMOVE, false, true, (DWORD)nSeedIdx, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::OnNode4SeedAdded(UINT nNodeIdx, UINT nSeedIdx)
{
    BufferItem  *pbi;
    PEVT_NODE4SEED_OP  pN4SOp;

    DBG(D_INFO, (L"%s: nidx(%d), sidx(%d)\n", FULL_FUNC_NAME, nNodeIdx, nSeedIdx));

    pbi = AllocSendBuf(true);
    if ( pbi )
    {
        pN4SOp = (PEVT_NODE4SEED_OP)pbi->Put(sizeof(EVT_NODE4SEED_OP));
        pN4SOp->dwNodeIdx = (DWORD)nNodeIdx;
        pN4SOp->dwSeedIdx = (DWORD)nSeedIdx;

        BuildPDU(C_SM_ON_N4S_ADD, false, true, 0, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::OnNode4SeedRemoved(UINT nNodeIdx, UINT nSeedIdx)
{
    BufferItem  *pbi;
    PEVT_NODE4SEED_OP  pN4SOp;

    DBG(D_INFO, (L"%s: nidx(%d), sidx(%d)\n", FULL_FUNC_NAME, nNodeIdx, nSeedIdx));

    pbi = AllocSendBuf(true);
    if ( pbi )
    {
        pN4SOp = (PEVT_NODE4SEED_OP)pbi->Put(sizeof(EVT_NODE4SEED_OP));
        pN4SOp->dwNodeIdx = (DWORD)nNodeIdx;
        pN4SOp->dwSeedIdx = (DWORD)nSeedIdx;

        BuildPDU(C_SM_ON_N4S_REMOVE, false, true, 0, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::OnNode4SeedStateUpdated(UINT nNodeIdx, UINT nSeedIdx, const PSEED_STATE pSeedState)
{
    BufferItem  *pbi;
    PEVT_NODE4SEED_STATE_UP  pN4SStateUp;

    DBG(D_INFO, (L"%s: nidx(%d), sidx(%d): %d\n", FULL_FUNC_NAME, nNodeIdx, nSeedIdx, pSeedState->bProgressDegree));

    pbi = AllocSendBuf((pSeedState->bProgressDegree == 100));
    if ( pbi )
    {
        pN4SStateUp = (PEVT_NODE4SEED_STATE_UP)pbi->Put(sizeof(EVT_NODE4SEED_STATE_UP));
        pN4SStateUp->dwNodeIdx = (DWORD)nNodeIdx;
        pN4SStateUp->dwSeedIdx = (DWORD)nSeedIdx;
        memcpy(&pN4SStateUp->SeedState, pSeedState, sizeof(SEED_STATE));

        BuildPDU(C_SM_ON_N4S_STATE_UPDATE, false, true, 0, pbi);

        PutSendData(pbi, true);
    }
}

void SeedMonSvrHandler::ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, BufferItem *pbiBody)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    assert(0);
}

void SeedMonSvrHandler::ProcessRecvPDU(BYTE bCode, bool bReq, DWORD dwParam, const PBYTE pData, UINT nDataBytes, UINT nLeftBytes)
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    assert(0);
}

BufferItem *SeedMonSvrHandler::AllocSendBuf(bool bRetry)
{
    int  cnt;
    BufferItem  *pbi;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    cnt = bRetry ? 3 : 1;
    while ( true )
    {
        pbi = GetSendBuf();
        --cnt;

        if ( pbi || (!cnt) )
        {
            break;
        }

        DBG(D_PERF, (L"%s: wait for free send buf...\n", FULL_FUNC_NAME));
        Sleep(200);
    }

    if ( (!pbi) && bRetry )
    {
        DBG(D_WARN, (L"%s: no free send buf\n", FULL_FUNC_NAME));
        assert(0);
        ReqEndSession();
    }

    return pbi;
}
