/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "SeedTrackerHandler.h"
#include "SeedMonSvrHandler.h"
#include "TrackerServer.h"


TrackerServer *TrackerServer::_instance = NULL;

TrackerServer *TrackerServer::Instance()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    if ( !_instance )
    {
        DBG(D_VERB, (L"%s: %ls\n", FULL_FUNC_NAME, L"CreateInstance"));
        _instance = new TrackerServer();
    }
    return _instance;
}

void TrackerServer::Destroy()
{
    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    delete _instance;
    _instance = NULL;
}

TrackerServer::TrackerServer()
    : Endpoint()
    , _hSTListener(INVALID_LISTENER_ID)
    , _hSMListener(INVALID_LISTENER_ID)
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

TrackerServer::~TrackerServer()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

DWORD TrackerServer::Load(UINT nMaxNodeNum)
{
    SOCK_ADDR  addr;

    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

    Endpoint::Load(nMaxNodeNum, 1, 0);

    addr.bType = ADDR_TYPE_IPV4;
    addr.IPv4Addr.bStream = 1;
    addr.IPv4Addr.dwIP = 0;
    addr.IPv4Addr.wPort = htons(SEED_TRACK_PORT);
    AddListener(&addr, false, _hSTListener);

    addr.bType = ADDR_TYPE_IPV4;
    addr.IPv4Addr.bStream = 1;
    addr.IPv4Addr.dwIP = 0;
    addr.IPv4Addr.wPort = htons(SEED_MON_PORT);
    AddListener(&addr, false, _hSMListener);

    return ERROR_SUCCESS;
}

void TrackerServer::Unload()
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

    if ( _hSMListener != INVALID_LISTENER_ID )
    {
        RemoveListener(_hSMListener);
        _hSMListener = INVALID_LISTENER_ID;
    }

    if ( _hSTListener != INVALID_LISTENER_ID )
    {
        RemoveListener(_hSTListener);
        _hSTListener = INVALID_LISTENER_ID;
    }

    Endpoint::Unload();
}

void TrackerServer::OnListenerEnded(DWORD hListener)
{
    DBG(D_LOG, (L"%s: %x\n", FULL_FUNC_NAME, hListener));
}

EpProtocolHandler *TrackerServer::OnNewHandler(DWORD hListener, Connection *pConnection, DWORD dwConnId)
{
    EpPDUHandler  *pHandler;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    if ( hListener == _hSTListener )
    {
        DBG(D_INFO, (L"%s: SeedTrackerHandler\n", FULL_FUNC_NAME));
        pHandler = new(std::nothrow) SeedTrackerHandler;
        return pHandler;
    }

    if ( hListener == _hSMListener )
    {
        DBG(D_INFO, (L"%s: SeedMonSvrHandler\n", FULL_FUNC_NAME));
        pHandler = new(std::nothrow) SeedMonSvrHandler;
        return pHandler;
    }

    DBG(D_ERROR, (L"%s: not match\n", FULL_FUNC_NAME));
    assert(0);

    return NULL;
}

void TrackerServer::OnAsyncConnected(DWORD dwConnId, EpProtocolHandler *pHandler)
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

    assert(0);
}

void TrackerServer::OnDisconnected(DWORD dwConnId, EpProtocolHandler *pHandler)
{
    DBG(D_INFO, (L"%s: id(%x)\n", FULL_FUNC_NAME, dwConnId));

    delete pHandler;
}

void TrackerServer::OnConnEvent(DWORD dwConnId, EpProtocolHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody)
{
    DBG(D_INFO, (L"%s: id(%#.8x), code(%d)\n", FULL_FUNC_NAME, dwConnId, pEvtHdr->nCode));

    assert(0);
}

void TrackerServer::ProcessEpWork(UINT nSigIdx)
{
    DBG(D_INFO, (L"%s: %d\n", FULL_FUNC_NAME, nSigIdx));

    assert(0);
}
