/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include "SHA1Hash.h"


#if defined(_WINCE_) || defined(_WIN32_)
HCRYPTPROV  SHA1Hash::_hProv = NULL;
#endif

DWORD SHA1Hash::Startup()
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    DWORD  dwRet;

    if ( !CryptAcquireContext(&_hProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: CryptAcquireContext failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

    return ERROR_SUCCESS;
}

void SHA1Hash::Cleanup()
{
    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    CryptReleaseContext(_hProv, 0);
    _hProv = NULL;
#endif
}

SHA1Hash::SHA1Hash()
#if defined(_WINCE_) || defined(_WIN32_)
    : _hHash(NULL)
#endif
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));
}

SHA1Hash::~SHA1Hash()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    assert(!_hHash);
#endif
}

DWORD SHA1Hash::BeginHash()
{
    DWORD  dwRet;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    if ( !_hProv )
    {
        DBG(D_ERROR, (L"%s: _hProv is NULL\n", FULL_FUNC_NAME));
        assert(0);
        return ERROR_NOT_READY;
    }
    if ( !CryptCreateHash(_hProv, CALG_SHA1, 0, 0, &_hHash) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: CryptCreateHash failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

#if defined(_ANDROID_) || defined(_LINUX_)
    if ( !SHA1_Init(&_hHash) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: SHA1_Init failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

    return ERROR_SUCCESS;
}

DWORD SHA1Hash::DoHash(PBYTE pData, UINT nSize)
{
    DWORD  dwRet;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    if ( !CryptHashData(_hHash, pData, nSize, 0) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: CryptHashData failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

#if defined(_ANDROID_) || defined(_LINUX_)
    if ( !SHA1_Update(&_hHash, pData, nSize) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: SHA1_Update failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

    return ERROR_SUCCESS;
}

DWORD SHA1Hash::GetHashVal(PSHA1_VAL pValSHA1)
{
    DWORD  dwRet;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    DWORD  dwLen;

    dwLen = SHA1_VAL_LEN;
    // After CryptGetHashParam has been called, no more data can be added to the hash.
    // Additional calls to CryptHashData or CryptHashSessionKey fail.
    // After the application is done with the hash,
    // CryptDestroyHash should be called to destroy the hash object.
    if ( !CryptGetHashParam(_hHash, HP_HASHVAL, pValSHA1->bVal, &dwLen, 0) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: CryptGetHashParam failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

#if defined(_ANDROID_) || defined(_LINUX_)
    // then call SHA1_Init() to issue a new hash procedure
    if ( !SHA1_Final(pValSHA1->bVal, &_hHash) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: SHA1_Final failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }
#endif

    return ERROR_SUCCESS;
}

void SHA1Hash::EndHash()
{
    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

#if defined(_WINCE_) || defined(_WIN32_)
    CryptDestroyHash(_hHash);
    _hHash = NULL;
#endif
}
