/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#if defined(_WIN32_) || defined(_WINCE_)
#include <vld.h>
#endif
#ifdef CONFIG_TLS_SUPPORT
#include "SecureCenter.h"
#endif
#include "ReactorSelect.h"
#include "ReactorEPoll.h"
#include "ConnectCenter.h"
#include "MUdfTarget.h"
#include "MUdfClientEp.h"


static bool                     gbUpload;

static DWORD                    gdwStartTicks;
static DWORD                    gdwPreTicks;
static QWORD                    gqwPreBytes;


void PrintSpeed(MUdfTarget *pMUdfTarget)
{
    UINT  nBlkCnt, nDegree;
    QWORD  qwRdBytes, qwWrBytes, qwBytes;
    DWORD  dwCurTicks, dwDelta, dwSpeed;

    DBG(D_FUNC, (L"%s\n", FULL_FUNC_NAME));

    dwCurTicks = GetTickCount();
    dwDelta = dwCurTicks - gdwPreTicks;

    pMUdfTarget->GetStatInfo(nBlkCnt, qwRdBytes, qwWrBytes);
    qwBytes = gbUpload ? qwRdBytes : qwWrBytes;

    if ( qwBytes == 0 )
    {
        return;
    }

    nDegree = nBlkCnt * 100 / pMUdfTarget->GetBlockCount();
    dwSpeed = (DWORD)((qwBytes - gqwPreBytes) / 1024 / 1024 * 1000 / dwDelta);
    log_wprintf(L"(d[%3dMB/s]), %3d%%\r", dwSpeed, nDegree);

    if ( dwSpeed != 0 )
    {
        gqwPreBytes = qwBytes;
        gdwPreTicks = dwCurTicks;
    }
}

int main(int argc, char *argv[])
{
    DWORD  dwRet;
    bool  bNtMode;
    MUdfTarget  muTarget;
    MU_TARGET_TYPE  mutType;
#ifdef _WIN32_
    HANDLE  hmuCtrl, hmuFile;
#endif

#ifdef _WIN32_
    if ( (argc != 8) && (argc != 5) )
#else
    if ( argc != 8 )
#endif
    {
        log_wprintf(L"MUdfClient\n");
        log_wprintf(L"\t<server_ip> <remote_mdf> <remote_udf> <local_mdf> <local_udf> <1=PSUP/2=OSUP/3=PSDW/4=OSDW> <1=update_mode>\n");
#ifdef _WIN32_
        log_wprintf(L"\t<server_ip> <remote_mdf> <remote_udf> <mudf_ntname>\n");
#endif
        return ERROR_INVALID_PARAMETER;
    }

#if defined(_WIN32_) || defined(_WINCE_)
    WSADATA  WSAData;
    WSAStartup(MAKEWORD(2, 2), &WSAData);
#endif

    bNtMode = (argc == 5);
    gbUpload = bNtMode ? true : (argv[6][0] < '3');

    if ( bNtMode )
    {
#ifdef _WIN32_
        dwRet = ERROR_SUCCESS;
        hmuCtrl = CreateFile(L"\\\\.\\DTccDisk1", GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
        hmuFile = CreateFile(L"\\\\.\\DTccDisk1", GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
        if ( (hmuCtrl == INVALID_HANDLE_VALUE) || (hmuFile == INVALID_HANDLE_VALUE) )
        {
            dwRet = GetLastError();
        }
#else
        dwRet = ERROR_NOT_SUPPORTED;
#endif
    }
    else
    {
        dwRet = gbUpload ? muTarget.Init(argv[4], argv[5], M_MUT_READONLY, 0) : ERROR_SUCCESS;
    }
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"init local mudf failed: %d\n", dwRet);
        goto err1;
    }

#ifdef CONFIG_TLS_SUPPORT
    dwRet = SecureCenter::Instance()->Load(NULL, NULL, NULL);
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"SecureCenter init failed: %d\n", dwRet);
        goto err1_2;
    }
#endif
    ReactorSocket::Instance()->Load();
    ConnectCenter::Instance()->Load();
    dwRet = MUdfClientEp::Instance()->Load();
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"MUdfClientEp init failed: %d\n", dwRet);
        goto err2;
    }

    if ( bNtMode )
    {
#ifdef _WIN32_
        dwRet = MUdfClientEp::Instance()->StartUpload(argv[1], hmuCtrl, hmuFile, argv[2], argv[3]);
#else
        dwRet = ERROR_NOT_SUPPORTED;
#endif
    }
    else
    {
        if ( gbUpload )
        {
			mutType = (argv[6][0] == '1') ? T_MUT_PS_UPLOAD : T_MUT_OS_UPLOAD;
            dwRet = MUdfClientEp::Instance()->StartUpload(argv[1], &muTarget, mutType, argv[2], argv[3], (argv[7][0] == '1'));
        }
        else
        {
            mutType = (argv[6][0] == '3') ? T_MUT_PS_DOWNLOAD: T_MUT_OS_DOWNLOAD;
            dwRet = MUdfClientEp::Instance()->StartDownload(argv[1], &muTarget, mutType, argv[2], argv[3], argv[4], argv[5]);
        }
    }
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"start upload failed: %d\n", dwRet);
        goto err3;
    }

    gdwStartTicks = GetTickCount();
    gdwPreTicks = gdwStartTicks;
    gqwPreBytes = 0;
    while ( true )
    {
        dwRet = MUdfClientEp::Instance()->WaitForFinished(bNtMode ? 0 : 2);
        if ( dwRet != WAIT_TIMEOUT )
        {
            break;
        }

        PrintSpeed(&muTarget);
    }
    if ( dwRet == ERROR_SUCCESS )
    {
        log_wprintf(L"\nup/dw 100%% done\n");
    }
    else
    {
        log_wprintf(L"\nup/dw failed: %d\n", dwRet);
    }

err3:
    MUdfClientEp::Instance()->Unload();

err2:
    if ( bNtMode )
    {
#ifdef _WIN32_
        CloseHandle(hmuCtrl);
        CloseHandle(hmuFile);
#endif
    }
    else if ( gbUpload )
    {
        muTarget.Fini();
    }

    ConnectCenter::Instance()->Unload();
    ReactorSocket::Instance()->Unload();
#ifdef CONFIG_TLS_SUPPORT
    SecureCenter::Instance()->Unload();
#endif

    MUdfClientEp::Destroy();
    ConnectCenter::Destroy();
    ReactorSocket::Destroy();
#ifdef CONFIG_TLS_SUPPORT
err1_2:
    SecureCenter::Destroy();
#endif

err1:
#if defined(_WIN32_) || defined(_WINCE_)
    WSACleanup();
#endif

    return dwRet;
}
