/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _MUDF_CLIENT_EP_H
#define _MUDF_CLIENT_EP_H


#include "Endpoint.h"
class MUdfTarget;


typedef enum _MU_TARGET_TYPE
{
    T_MUT_PS_UPLOAD           = 1,
    T_MUT_OS_UPLOAD,
    T_MUT_PS_DOWNLOAD,
    T_MUT_OS_DOWNLOAD
}
MU_TARGET_TYPE;


class MUdfClientEp : protected Endpoint
{
public:
    static MUdfClientEp *Instance();
    static void Destroy();

private:
    static MUdfClientEp       * _instance;

protected:
    MUdfClientEp();
    virtual ~MUdfClientEp();

public:
    DWORD Load();
    virtual void Unload();

    DWORD StartUpload(const char *szServerIP, MUdfTarget *pMUdfTarget, MU_TARGET_TYPE mutType, const char *szRemoteMdf, const char *szRemoteUdf, bool bUpdateMode);
#ifdef _WIN32_
    DWORD StartUpload(const char *szServerIP, HANDLE hPsMUdfCtrl, HANDLE hPsMUdfFile, const char *szRemoteMdf, const char *szRemoteUdf);
#endif
    DWORD StartDownload(const char *szServerIP, MUdfTarget *pMUdfTarget, MU_TARGET_TYPE mutType, const char *szRemoteMdf, const char *szRemoteUdf, const char *szLocalMdf, const char *szLocalUdf);

    void CancelUpload();

    DWORD WaitForFinished(DWORD dwSeconds);

protected:
    virtual void OnListenerEnded(DWORD hListener);

    virtual EpProtocolHandler *OnNewHandler(DWORD hListener, Connection *pConnection, DWORD dwConnId);
    virtual void OnAsyncConnected(DWORD dwConnId, EpProtocolHandler *pHandler);
    virtual void OnDisconnected(DWORD dwConnId, EpProtocolHandler *pHandler);
    virtual void OnConnEvent(DWORD dwConnId, EpProtocolHandler *pHandler, PEP_MSG_HDR pEvtHdr, BufferItem *pEvtBody);

    virtual void ProcessEpWork(UINT nSigIdx);

    DWORD DoConnect(const char *szServerIP, MU_TARGET_TYPE mutType, EpProtocolHandler *pHandler, DWORD &rdwConnId);
    void OnFinished(DWORD dwCode);

private:
    sem_t                       _hDoneEvt;

    DWORD                       _dwDoneRet;
    DWORD                       _dwConnId;
};


#endif
