/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#include <wininet.h>
#ifdef CONFIG_TLS_SUPPORT
#include "SecureCenter.h"
#endif
#include "NtFileLib.h"
#include "ReactorSelect.h"
#include "ReactorEPoll.h"
#include "ConnectCenter.h"
#include "MUdfClientEp.h"


#ifdef USE_ACTIVE_MODE
typedef NTSTATUS (WINAPI *fnRtlSetProcessIsCritical)
    (
        BOOLEAN  bNew,
        BOOLEAN  *pbOld,
        BOOLEAN  bNeedScb
    );
#endif


#define IOCTL_TC_GET_BLOCK_CNT_LEFT   CTL_CODE(FILE_DEVICE_DISK, 0x807, METHOD_BUFFERED, FILE_READ_DATA)
#define IOCTL_TC_GET_IMG_PATH         CTL_CODE(FILE_DEVICE_DISK, 0x808, METHOD_BUFFERED, FILE_READ_DATA)


#define  SVCNAME                L"MUdfClientService"

#define  SVC_ERROR              ((DWORD)0xC0020001L)

#define  TCC_DISK_OBJECT        L"\\\\.\\DTccDisk1"
#define  TCC_DISK_MNTPOINT      L"D:\\"

#define  DEFAULT_SERVER_IP      "192.168.0.1"
#define  SERVER_IP_FILE         L"\\Device\\Harddisk0\\Partition3\\tc-serv.addr"
//#define  SERVER_IP_FILE         L"\\DosDevices\\D:\\tc.addr"

#define  USER_ID_FILE           L"\\Device\\Harddisk0\\Partition1\\user.id"
#define  IMG_VER_PATH           L"\\Device\\Harddisk0\\Partition1\\%hs.ver"

#define  CLEANUP_TIME           2
#ifdef USE_ACTIVE_MODE
#define  WAIT_FINISH_LIMIT      5
#define  WAIT_FINISH_INTERVAL   2
#define  MERGE_REQ_TIME         3
#define  STOP_WAIT_TIME         (CLEANUP_TIME + MERGE_REQ_TIME)
#define  SHUTDOWN_WAIT_TIME     (WAIT_FINISH_INTERVAL * WAIT_FINISH_LIMIT + STOP_WAIT_TIME)
#else
#define  STOP_WAIT_TIME         CLEANUP_TIME
#endif

#define  RE_UPLOAD_WAIT_TIME    (10*60*1000)


static SERVICE_STATUS            gTcServStatus;
static SERVICE_STATUS_HANDLE     gTcServStatusHandle;
static HANDLE                    ghTcServStopEvent;

static char                      gszServerIP[20];
static HANDLE                    ghmuCtrl;
static HANDLE                    ghmuFile;

static char                      gszUserId[10];

static char                      gszImgVer[10];
static char                      gszImgName[MAX_PATH];
static char                      gszMdfName[MAX_PATH];
static char                      gszUdfName[MAX_PATH];

#ifdef USE_ACTIVE_MODE
static bool                      gbMergeImg = false;
#endif


DWORD ntReadLine(const wchar_t *szFileName, char *szBufData, ULONG ulBufSize)
{
    DWORD  dwRet;
    HANDLE  hFile;
    int  nPos;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    dwRet = ntOpenFileW(&hFile, szFileName, "rb");
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_WARN, (L"%s: ntOpenFile(%s) failed: %d\n", FULL_FUNC_NAME, szFileName, dwRet));
        goto err;
    }

    --ulBufSize;
    dwRet = ntReadFile(hFile, (PBYTE)szBufData, &ulBufSize);
    ntCloseFile(hFile);

    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_WARN, (L"%s: ntReadFile(%s) failed: %d\n", FULL_FUNC_NAME, szFileName, dwRet));
        goto err;
    }

    nPos = ulBufSize - 1;
    while ( ulBufSize > 0 )
    {
        if ( (szBufData[nPos] != '\r') && (szBufData[nPos] != '\n') )
        {
            break;
        }

        --nPos;
        --ulBufSize;
    }

    if ( ulBufSize == 0 )
    {
        DBG(D_WARN, (L"%s: zero string\n", FULL_FUNC_NAME));
        goto err;
    }

    szBufData[ulBufSize] = 0;
    DBG(D_INFO, (L"%s: file(%s), str(%hs)\n", FULL_FUNC_NAME, szFileName, szBufData));

err:
    return dwRet;
}

#ifdef USE_ACTIVE_MODE
BOOL EnableDebugPriv()
{
    BOOL  bRet;
    HANDLE  hToken;
    LUID  luid;
    TOKEN_PRIVILEGES  tkprivs;

    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));

    bRet = OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES|TOKEN_QUERY, &hToken);
    if ( !bRet )
    {
        DBG(D_WARN, (L"%s: OpenProcessToken failed\n", FULL_FUNC_NAME));
        goto err1;
    }

    bRet = LookupPrivilegeValue(NULL, SE_DEBUG_NAME, &luid);
    if ( !bRet )
    {
        DBG(D_WARN, (L"%s: LookupPrivilegeValue failed\n", FULL_FUNC_NAME));
        goto err2;
    }

    tkprivs.PrivilegeCount = 1;
    tkprivs.Privileges[0].Luid = luid;
    tkprivs.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
    bRet = AdjustTokenPrivileges(hToken, FALSE, &tkprivs, sizeof(tkprivs), NULL, NULL);
    if ( !bRet )
    {
        DBG(D_WARN, (L"%s: AdjustTokenPrivileges failed\n", FULL_FUNC_NAME));
        goto err2;
    }

    DBG(D_INFO, (L"%s: adjust debug privilege OK\n", FULL_FUNC_NAME));

err2:
    CloseHandle(hToken);

err1:
    return bRet;
}

void SetCriticalProcess(BOOLEAN bCritical)
{
    DBG(D_INFO, (L"%s: %d\n", FULL_FUNC_NAME, bCritical));

//    ULONG  ulVal = 1;
//    NtSetInformationProcess(GetCurrentProcess(), 29/*ProcessBreakOnTermination*/, &ulVal, sizeof(ulVal));

    HMODULE  hModule = LoadLibraryW(L"ntdll.dll");
    if ( hModule != NULL )
    {
        fnRtlSetProcessIsCritical  SetProcessIsCritical = (fnRtlSetProcessIsCritical)GetProcAddress(hModule, "RtlSetProcessIsCritical");
        if ( SetProcessIsCritical != NULL )
        {
            DBG(D_INFO, (L"%s: will set/unset critical process\n", FULL_FUNC_NAME));
            SetProcessIsCritical(bCritical, NULL, FALSE);
        }
    }
}
#endif

void RefreshServerIP()
{
    DWORD  dwRet;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    dwRet = ntReadLine(SERVER_IP_FILE, gszServerIP, sizeof(gszServerIP));
    if ( dwRet != ERROR_SUCCESS )
    {
        strncpy(gszServerIP, DEFAULT_SERVER_IP, sizeof(gszServerIP));
    }

    DBG(D_INFO, (L"%s: server ip: %hs\n", FULL_FUNC_NAME, gszServerIP));
}

DWORD OpenMUdfFile()
{
    DWORD  dwRet;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    ghmuCtrl = CreateFile(TCC_DISK_OBJECT, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if ( ghmuCtrl == INVALID_HANDLE_VALUE )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: open ctrl file failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err1;
    }

    ghmuFile = CreateFile(TCC_DISK_OBJECT, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if ( ghmuFile == INVALID_HANDLE_VALUE )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: open mudf file failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err2;
    }

    return ERROR_SUCCESS;

err2:
    CloseHandle(ghmuCtrl);

err1:
    return dwRet;
}

DWORD GetMUdfName()
{
    DWORD  dwRet;
    DWORD  dwBytes;
    BufferItem  biBuf;
    char  *szImgPath, *szMdfPath, *szUdfPath;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    biBuf.Alloc(4096);
    if ( !DeviceIoControl(ghmuCtrl,
                    IOCTL_TC_GET_IMG_PATH,
                    NULL,
                    0,
                    biBuf.tail(),
                    biBuf.tail_size(),
                    &dwBytes,
                    NULL) )
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: IOCTL_TC_GET_IMG_PATH failed: %d\n", FULL_FUNC_NAME, dwRet));
        return dwRet;
    }

    DBG(D_INFO, (L"%s: IOCTL_TC_GET_IMG_PATH: bytes=%d\n", FULL_FUNC_NAME, dwBytes));
    biBuf.Put(dwBytes);

    szImgPath = (char*)biBuf.data();
    biBuf.Pull((UINT)strlen(szImgPath) + 1);
    szMdfPath = (char*)biBuf.data();
    biBuf.Pull((UINT)strlen(szMdfPath) + 1);
    szUdfPath = (char*)biBuf.data();

    GetFileName(szImgPath, gszImgName, _countof(gszImgName));
    GetFileName(szMdfPath, gszMdfName, _countof(gszMdfName));
    GetFileName(szUdfPath, gszUdfName, _countof(gszUdfName));
    DBG(D_INFO, (L"%s: img(%hs), mdf(%hs), udf(%hs)\n", FULL_FUNC_NAME, gszImgName, gszMdfName, gszUdfName));

    return ERROR_SUCCESS;
}

DWORD PrepareRunEnv()
{
    DWORD  dwRet;
    wchar_t  buf[MAX_PATH];

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    if ( !SetProcessShutdownParameters(0, 0) )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: SetProcessShutdownParameters failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err1;
    }

    dwRet = OpenMUdfFile();
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: OpenMUdfFile failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err1;
    }

    dwRet = GetMUdfName();
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: GetMUdfName failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err2;
    }

    dwRet = ntReadLine(USER_ID_FILE, gszUserId, sizeof(gszUserId));
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: get user id failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err2;
    }
    DBG(D_INFO, (L"%s: user id: %hs\n", FULL_FUNC_NAME, gszUserId));

    _snwprintf(buf, _countof(buf), IMG_VER_PATH, gszImgName);
    dwRet = ntReadLine(buf, gszImgVer, sizeof(gszImgVer));
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: get mudf ver failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err2;
    }
    DBG(D_INFO, (L"%s: psdata ver: %hs\n", FULL_FUNC_NAME, gszImgVer));

    return dwRet;

err2:
    CloseHandle(ghmuCtrl);
    CloseHandle(ghmuFile);

err1:
    return dwRet;
}

int GetLeftBlockNum(HANDLE hmuCtrl)
{
    DWORD  dwRet;
    DWORD  dwBytes;
    DWORD  dwLeftNum;

    DBG(D_VERB, (L"%s\n", FULL_FUNC_NAME));

    if ( !DeviceIoControl(hmuCtrl,
                    IOCTL_TC_GET_BLOCK_CNT_LEFT,
                    NULL,
                    0,
                    &dwLeftNum,
                    sizeof(dwLeftNum),
                    &dwBytes,
                    NULL) )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: IOCTL_TC_GET_BLOCK_CNT_LEFT failed: %d\n", FULL_FUNC_NAME, dwRet));
        return -1;
    }

    DBG(D_INFO, (L"%s: IOCTL_TC_GET_BLOCK_CNT_LEFT: bytes=%d, num=%d\n", FULL_FUNC_NAME, dwBytes, dwLeftNum));

    return (int)dwLeftNum;
}

#ifdef USE_ACTIVE_MODE
DWORD SendMergeReq()
{
    DWORD  dwRet;
    HINTERNET  hSession;
    HINTERNET  hConnect;
    HINTERNET  hRequest;
    char  szTmpBuf[800];
    DWORD  dwStatus, dwSize;
    const char * szHdrContentType = "contentType: 'application/json'";
    const char * szHdrDataType = "dataType: 'json'";

    DBG(D_INFO, (L"%s\n", FULL_FUNC_NAME));

    hSession = InternetOpenA("Mozilla/5.0", INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);
    if ( hSession == NULL )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: InternetOpen failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err1;
    }

    hConnect = InternetConnectA(hSession, gszServerIP, INTERNET_DEFAULT_HTTP_PORT, NULL, NULL, INTERNET_SERVICE_HTTP, 0, 0);
    if ( hConnect == NULL )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: InternetConnect failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err2;
    }

    _snprintf(szTmpBuf, _countof(szTmpBuf), "tc/rest/user.php/%s", gszUserId);
    hRequest = HttpOpenRequestA(hConnect, "POST", szTmpBuf, NULL, NULL, NULL, 0, 0);
    if ( hRequest == NULL )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: HttpOpenRequest failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err3;
    }

    if ( !HttpAddRequestHeadersA(hRequest, szHdrContentType, (DWORD)strlen(szHdrContentType), HTTP_ADDREQ_FLAG_ADD) )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: HttpAddRequestHeaders failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err4;
    }

    if ( !HttpAddRequestHeadersA(hRequest, szHdrDataType, (DWORD)strlen(szHdrDataType), HTTP_ADDREQ_FLAG_ADD) )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: HttpAddRequestHeaders failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err4;
    }

    _snprintf(szTmpBuf, _countof(szTmpBuf),
        "{\"uri\":\"head\",\"img\":\"%s\",\"udf\":\"%s\",\"mdf\":\"%s\",\"revision\":\"%s\"}",
        gszImgName, gszUdfName, gszMdfName, gszImgVer);
    if ( !HttpSendRequestA(hRequest, NULL, 0, (LPVOID)szTmpBuf, (DWORD)strlen(szTmpBuf)) )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: HttpSendRequest failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err4;
    }

    dwSize = sizeof(dwStatus);
    if ( !HttpQueryInfo(hRequest, HTTP_QUERY_STATUS_CODE|HTTP_QUERY_FLAG_NUMBER, &dwStatus, &dwSize, NULL) )
    {
        dwRet = GetLastError();
        DBG(D_WARN, (L"%s: HttpQueryInfo failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err4;
    }

    DBG(D_INFO, (L"%s: status code: %d\n", FULL_FUNC_NAME, dwStatus));

    dwRet = ERROR_SUCCESS;

err4:
    InternetCloseHandle(hRequest);

err3:
    InternetCloseHandle(hConnect);

err2:
    InternetCloseHandle(hSession);

err1:
    return dwRet;
}
#endif

DWORD SvcInstall()
{
    DWORD  dwRet;
    SC_HANDLE  hSCMgr;
    SC_HANDLE  hServ;
    TCHAR  szExePath[MAX_PATH];

    if( GetModuleFileName(NULL, szExePath, MAX_PATH) == 0 )
    {
        dwRet = GetLastError();
        log_wprintf(L"GetModuleFileName failed: %d\n", dwRet);
        goto err1;
    }

    hSCMgr = OpenSCManager(
        NULL,
        NULL,
        SC_MANAGER_ALL_ACCESS);
    if ( hSCMgr == NULL )
    {
        dwRet = GetLastError();
        log_wprintf(L"OpenSCManager failed: %d\n", dwRet);
        goto err1;
    }

    hServ = CreateService(
        hSCMgr,
        SVCNAME,
        SVCNAME,
        SERVICE_ALL_ACCESS,
        SERVICE_WIN32_OWN_PROCESS,
        SERVICE_DEMAND_START,
        SERVICE_ERROR_NORMAL,
        szExePath,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL);
    if ( hServ == NULL )
    {
        dwRet = GetLastError();
        log_wprintf(L"CreateService failed: %d\n", dwRet);
        goto err2;
    }

    log_wprintf(L"service installed successfully\n");

    dwRet = ERROR_SUCCESS;

    CloseServiceHandle(hServ); 

err2:
    CloseServiceHandle(hSCMgr);

err1:
    return dwRet;
}

DWORD SvcUninstall()
{
    DWORD  dwRet;
    SC_HANDLE  schSCManager;
    SC_HANDLE  schService;
    SERVICE_STATUS  status;

    // Get a handle to the SCM database.
    schSCManager = OpenSCManager(
        NULL,                    // local computer
        NULL,                    // ServicesActive database
        SC_MANAGER_ALL_ACCESS);  // full access rights
    if ( schSCManager == NULL )
    {
        dwRet = GetLastError();
        log_wprintf(L"OpenSCManager failed: %d\n", dwRet);
        goto err1;
    }

    // Create the service
    schService = OpenService(
        schSCManager,              // SCM database
        SVCNAME,                   // name of service
        SERVICE_STOP | DELETE);    // desired access
    if ( schService == NULL )
    {
        dwRet = GetLastError();
        log_wprintf(L"OpenService failed: %d\n", dwRet);
        goto err2;
    }

    if ( !ControlService(schService, SERVICE_CONTROL_STOP, &status) )
    {
        dwRet = GetLastError();
        log_wprintf(L"ControlService failed: %d\n", dwRet);
//        goto err3;
    }

    if ( !DeleteService(schService) )
    {
        dwRet = GetLastError();
        log_wprintf(L"DeleteService failed: %d\n", dwRet);
        goto err3;
    }

    log_wprintf(L"service uninstalled successfully\n");

    dwRet = ERROR_SUCCESS;

err3:
    CloseServiceHandle(schService); 

err2:
    CloseServiceHandle(schSCManager);

err1:
    return dwRet;
}


VOID SvcReportEvent(LPWSTR szFunction) 
{ 
    HANDLE  hEventSource;
    LPCWSTR  lpszStrings[2];
    WCHAR  Buffer[80];

    DBG(D_WARN, (L"%s: %s failed\n", FULL_FUNC_NAME, szFunction));

    hEventSource = RegisterEventSource(NULL, SVCNAME);

    if ( hEventSource != NULL )
    {
        _snwprintf(Buffer, _countof(Buffer), L"%s failed with %d", szFunction, GetLastError());

        lpszStrings[0] = SVCNAME;
        lpszStrings[1] = Buffer;

        ReportEvent(hEventSource,        // event log handle
                    EVENTLOG_ERROR_TYPE, // event type
                    0,                   // event category
                    SVC_ERROR,           // event identifier
                    NULL,                // no security identifier
                    2,                   // size of lpszStrings array
                    0,                   // no binary data
                    lpszStrings,         // array of strings
                    NULL);               // no binary data

        DeregisterEventSource(hEventSource);
    }
}

VOID ReportSvcStatus(DWORD dwCurState, DWORD dwExitCode, DWORD dwWaitHint)
{
    static DWORD  dwCheckPoint = 1;

    DBG(D_INFO, (L"%s: state=%d, code=%d\n", FULL_FUNC_NAME, dwCurState, dwExitCode));

    // Fill in the SERVICE_STATUS structure.
    gTcServStatus.dwCurrentState = dwCurState;
    gTcServStatus.dwWin32ExitCode = dwExitCode;
    gTcServStatus.dwWaitHint = dwWaitHint;

    if ( dwCurState == SERVICE_START_PENDING )
    {
        gTcServStatus.dwControlsAccepted = 0;
    }
    else
    {
        gTcServStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
    }

    if ( (dwCurState == SERVICE_RUNNING) || (dwCurState == SERVICE_STOPPED) )
    {
        gTcServStatus.dwCheckPoint = 0;
    }
    else
    {
        gTcServStatus.dwCheckPoint = dwCheckPoint++;
    }

    // Report the status of the service to the SCM.
    SetServiceStatus(gTcServStatusHandle, &gTcServStatus);
}

VOID WINAPI SvcCtrlHandler(DWORD dwCtrl)
{
#ifdef USE_ACTIVE_MODE
    int  nLeftNum = -1;
#endif

    DBG(D_INFO, (L"%s: ctrl=%d\n", FULL_FUNC_NAME, dwCtrl));

    // Handle the requested control code.
    switch ( dwCtrl )
    {
        // see issues_winsvc.txt
        case SERVICE_CONTROL_SHUTDOWN:
#ifdef USE_ACTIVE_MODE
            DBG(D_INFO, (L"%s: SERVICE_CONTROL_SHUTDOWN\n", FULL_FUNC_NAME));
            ReportSvcStatus(SERVICE_STOP_PENDING, NO_ERROR, SHUTDOWN_WAIT_TIME * 1000);

            DeleteVolumeMountPoint(TCC_DISK_MNTPOINT);

            for ( int i = 0; i < WAIT_FINISH_LIMIT; ++i )
            {
                nLeftNum = GetLeftBlockNum(ghmuCtrl);
                if ( nLeftNum <= 0 )
                {
                    DBG(D_WARN, (L"%s: no left or error: %d\n", FULL_FUNC_NAME, nLeftNum));
                    break;
                }

                DBG(D_INFO, (L"%s: %d blocks left, wait...\n", FULL_FUNC_NAME, nLeftNum));
                Sleep(WAIT_FINISH_INTERVAL * 1000);
            }
#endif

        case SERVICE_CONTROL_STOP:
#ifdef USE_ACTIVE_MODE
            gbMergeImg = (nLeftNum == 0);
#endif

            DBG(D_INFO, (L"%s: SERVICE_CONTROL_STOP\n", FULL_FUNC_NAME));
            ReportSvcStatus(SERVICE_STOP_PENDING, NO_ERROR, STOP_WAIT_TIME * 1000);

            if ( dwCtrl == SERVICE_CONTROL_STOP )
            {
                // invert wait@SvcInit() sequence
                SetEvent(ghTcServStopEvent);
                MUdfClientEp::Instance()->CancelUpload();
            }

            break;

        case SERVICE_CONTROL_INTERROGATE:
            break;

        default:
            break;
   }
}

VOID SvcInit(DWORD dwArgc, LPWSTR *lpszArgv)
{
    DWORD  dwRet;

    DBG(D_LOG, (L"%s\n", FULL_FUNC_NAME));

    // TO_DO: Declare and set any required variables.
    //   Be sure to periodically call ReportSvcStatus() with
    //   SERVICE_START_PENDING. If initialization fails, call
    //   ReportSvcStatus with SERVICE_STOPPED.

    // Create an event. The control handler function, SvcCtrlHandler,
    // signals this event when it receives the stop control code.
    ghTcServStopEvent = CreateEvent(
                         NULL,    // default security attributes
                         TRUE,    // manual reset event
                         FALSE,   // not signaled
                         NULL);   // no name

    if ( ghTcServStopEvent == NULL)
    {
        dwRet = GetLastError();
        DBG(D_ERROR, (L"%s: CreateEvent failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err1;
    }

    dwRet = PrepareRunEnv();
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: PrepareRunEnv failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err2;
    }

    WSADATA  WSAData;
    WSAStartup(MAKEWORD(2, 2), &WSAData);

#ifdef CONFIG_TLS_SUPPORT
    SecureCenter::Instance()->Load(NULL, NULL, NULL);
#endif
    ReactorSocket::Instance()->Load();
    ConnectCenter::Instance()->Load();
    dwRet = MUdfClientEp::Instance()->Load();
    if ( dwRet != ERROR_SUCCESS )
    {
        DBG(D_ERROR, (L"%s: init MUdfClientEp failed: %d\n", FULL_FUNC_NAME, dwRet));
        goto err3;
    }

    // Report running status when initialization is complete.
    ReportSvcStatus(SERVICE_RUNNING, NO_ERROR, 0);

#ifdef USE_ACTIVE_MODE
    EnableDebugPriv();
    SetCriticalProcess(TRUE);
#endif

    // TO_DO: Perform work until service stops.
    while ( true )
    {
        RefreshServerIP();

        DBG(D_INFO, (L"%s: start upload to %hs...\n", FULL_FUNC_NAME, gszServerIP));
        dwRet = MUdfClientEp::Instance()->StartUpload(gszServerIP, ghmuCtrl, ghmuFile, gszMdfName, gszUdfName);
        if ( dwRet == ERROR_SUCCESS )
        {
            dwRet = MUdfClientEp::Instance()->WaitForFinished(0);
            DBG(D_INFO, (L"%s: upload result: %d\n", FULL_FUNC_NAME, dwRet));
        }
        else
        {
            DBG(D_WARN, (L"%s: StartUpload failed: %d\n", FULL_FUNC_NAME, dwRet));
        }

        // Check whether to stop the service.
        DBG(D_INFO, (L"%s: sleep and then try again...\n", FULL_FUNC_NAME));
        dwRet = WaitForSingleObject(ghTcServStopEvent, RE_UPLOAD_WAIT_TIME);
        if ( dwRet != WAIT_TIMEOUT )
        {
            DBG(D_INFO, (L"%s: stop event signaled, exit...\n", FULL_FUNC_NAME));
            break;
        }
    }

    MUdfClientEp::Instance()->Unload();

#ifdef USE_ACTIVE_MODE
    if ( gbMergeImg )
    {
        ReportSvcStatus(SERVICE_STOP_PENDING, NO_ERROR, MERGE_REQ_TIME * 1000);
        SendMergeReq();
    }

    SetCriticalProcess(FALSE);
#endif

err3:
    ConnectCenter::Instance()->Unload();
    ReactorSocket::Instance()->Unload();
#ifdef CONFIG_TLS_SUPPORT
    SecureCenter::Instance()->Unload();
#endif

    MUdfClientEp::Destroy();
    ConnectCenter::Destroy();
    ReactorSocket::Destroy();
#ifdef CONFIG_TLS_SUPPORT
    SecureCenter::Destroy();
#endif

    WSACleanup();

    CloseHandle(ghmuFile);
    CloseHandle(ghmuCtrl);

err2:
    CloseHandle(ghTcServStopEvent);

err1:
    ReportSvcStatus(SERVICE_STOPPED, NO_ERROR, 0);
}

VOID WINAPI SvcMain(DWORD dwArgc, LPWSTR *lpszArgv)
{
    DBG(D_LOG, (L"%s: build at %hs %hs\n", FULL_FUNC_NAME, __DATE__, __TIME__));

    // Register the handler function for the service
    gTcServStatusHandle = RegisterServiceCtrlHandler(SVCNAME, SvcCtrlHandler);
    if ( gTcServStatusHandle == NULL )
    { 
        SvcReportEvent(L"RegisterServiceCtrlHandler");
        return;
    }

    // These SERVICE_STATUS members remain as set here
    gTcServStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
    gTcServStatus.dwServiceSpecificExitCode = 0;

    // Report initial status to the SCM
    ReportSvcStatus(SERVICE_START_PENDING, NO_ERROR, 3000);

    // Perform service-specific initialization and work.
    SvcInit(dwArgc, lpszArgv);
}

int __cdecl wmain(int argc, wchar_t *argv[])
{
    DBG(D_LOG, (L"%s: argc(%d) enter\n", FULL_FUNC_NAME, argc));

	// If command-line parameter is "install", install the service.
    // Otherwise, the service is probably being started by the SCM.
    if ( argc > 1 )
    {
        if ( _wcsicmp(argv[1], L"install") == 0 )
        {
            return SvcInstall();
        }
        else if ( _wcsicmp(argv[1], L"uninstall") == 0 )
        {
            return SvcUninstall();
        }
    }

    // TO_DO: Add any additional services for the process to this table.
    SERVICE_TABLE_ENTRY DispatchTable[] =
    {
        { SVCNAME, (LPSERVICE_MAIN_FUNCTION)SvcMain },
        { NULL, NULL }
    };

    // This call returns when the service has stopped.
    // The process should simply terminate when the call returns.
    if ( !StartServiceCtrlDispatcher(DispatchTable) )
    { 
        SvcReportEvent(L"StartServiceCtrlDispatcher"); 
    }

    DBG(D_LOG, (L"%s: leave\n", FULL_FUNC_NAME, argc));

    return 0;
}
