/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#if defined(_WIN32_) || defined(_WINCE_)
#include <vld.h>
#endif
#ifdef CONFIG_TLS_SUPPORT
#include "SecureCenter.h"
#endif
#include "ReactorSelect.h"
#include "ReactorEPoll.h"
#include "ConnectCenter.h"
#include "MUdfServerEp.h"


int main(int argc, char *argv[])
{
    DWORD  dwRet;
    UINT  nMaxConnNum;

    if ( argc < 5 )
    {
        log_wprintf(L"MUdfServer <max_conns> <psup_dir> <psdown_dir> <osroot_dir>\n");
        return ERROR_INVALID_PARAMETER;
    }

    nMaxConnNum = (UINT)atoi(argv[1]);

#if defined(_WIN32_) || defined(_WINCE_)
    WSADATA  WSAData;
    WSAStartup(MAKEWORD(2, 2), &WSAData);
#endif

#ifdef CONFIG_TLS_SUPPORT
    dwRet = SecureCenter::Instance()->Load(NULL, NULL, NULL);
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"SecureCenter init failed: %d\n", dwRet);
        goto err0;
    }
#endif
    ReactorSocket::Instance()->Load();
    ConnectCenter::Instance()->Load();
    dwRet = MUdfServerEp::Instance()->Load(nMaxConnNum, argv[2], argv[3], argv[4]);
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"MUdfServerEp init failed: %d\n", dwRet);
        goto err1;
    }

    getchar();

    MUdfServerEp::Instance()->Unload();
err1:
    ConnectCenter::Instance()->Unload();
    ReactorSocket::Instance()->Unload();
#ifdef CONFIG_TLS_SUPPORT
    SecureCenter::Instance()->Unload();
#endif

    MUdfServerEp::Destroy();
    ConnectCenter::Destroy();
    ReactorSocket::Destroy();
#ifdef CONFIG_TLS_SUPPORT
err0:
    SecureCenter::Destroy();
#endif

#if defined(_WIN32_) || defined(_WINCE_)
    WSACleanup();
#endif

    return ERROR_SUCCESS;
}
