/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef _MUDF_TARGET_H
#define _MUDF_TARGET_H


#include "FileMapping.h"


typedef enum _MUT_OPEN_MODE
{
    M_MUT_READONLY            = 1,
    M_MUT_OVERWRITE,
    M_MUT_UPDATE
}
MUT_OPEN_MODE;

typedef struct _MUT_RW_REC
{
    UINT                        nBlkIdx;
    QWORD                       qwBlkOffset;
    DWORD                       dwOffInBlk;
    DWORD                       dwLeftInBlk;

    QWORD                       qwCurOffset;
}
MUT_RW_REC, *PMUT_RW_REC;


class MUdfTarget
{
public:
    MUdfTarget();
    virtual ~MUdfTarget();

public:
    DWORD Init(const char *szMdfPath, const char *szUdfPath, MUT_OPEN_MODE OpenMode, DWORD dwMdfSize = 0);
    void Fini();

    DWORD GetMdfSize() const;
    UINT GetBlockCount() const;
    DWORD GetBlockSize() const;

    void GetStatInfo(UINT &rnDoneBlkCnt, QWORD &rqwReadBytes, QWORD &rqwWriteBytes) const;

    DWORD OpenBlock(UINT nBlkIdx, DWORD &rdwUdfOff, bool bAllocIf, DWORD &rdwBlkId);
    DWORD ReadBlock(DWORD dwBlkId, PBYTE pBuf, DWORD &rdwSize, bool &rbEndOfBlk);
    DWORD WriteBlock(DWORD dwBlkId, PBYTE pBuf, DWORD dwSize, bool &rbEndOfBlk);
    DWORD CloseBlock(DWORD dwBlkId);

private:    
    DWORD OpenMdFile(const char *szMdfPath, DWORD dwMdfSize, MUT_OPEN_MODE OpenMode, bool &rbReInit);
    DWORD OpenUdFile(const char *szUdfPath, MUT_OPEN_MODE OpenMode, bool bReInit);

    DWORD ReadUdFile(QWORD qwOffset, PBYTE pBuf, DWORD dwSize);
    DWORD WriteUdFile(QWORD qwOffset, PBYTE pBuf, DWORD dwSize);

private:
    MUT_OPEN_MODE               _OpenMode;
    FILE                      * _hMdFile;
    FILE                      * _hUdFile;

    DWORD                       _dwMdfSize;
    UINT                        _nMdfItemCnt;
    UINT                        _nUdfOffNext;

    FileMapping                 _MdfMapping;
    DWORD                     * _pMdfItems;

    MUT_RW_REC                  _BlkRWRec;

    UINT                        _nDoneBlkCnt;
    QWORD                       _qwReadBytes;
    QWORD                       _qwWriteBytes;
};


#endif
