/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <StdAfx.h>
#if defined(_WIN32_) || defined(_WINCE_)
#include <vld.h>
#endif
#include "SHA1Hash.h"
#include "ExecutorPool.h"
#include <SeedTarget.h>


int main(int argc, char *argv[])
{
    DWORD  dwRet;
    SeedTarget  *pSeedTarget;
    const BitArray  *pBitMask;
    UINT  nDegree;
    QWORD  qwMaskAnd, qwMaskOr;

    if ( argc < 3 )
    {
        log_wprintf(L"VerifyTarget <seed_file> <target_path>\n");
        return ERROR_INVALID_PARAMETER;
    }

    dwRet = SHA1Hash::Startup();
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"startup SHA1Hash failed: %d\n", dwRet);
        goto out1;
    }

    dwRet = ExecutorCenter::Instance()->Load(2);
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"load ExecutorCenter failed: %d\n", dwRet);
        goto out2;
    }

    pSeedTarget = new(std::nothrow) SeedTarget(1, 0);
    if ( !pSeedTarget )
    {
        dwRet = ERROR_OUTOFMEMORY;
        log_wprintf(L"alloc object failed\n");
        goto out3;
    }

//    dwRet = pSeedTarget->Init(argv[1], argv[2], true, false, NULL);
//    dwRet = pSeedTarget->Init(argv[1], argv[2], true, true, NULL);
//    dwRet = pSeedTarget->Init(argv[1], argv[2], false, false, NULL);
//    dwRet = pSeedTarget->Init(argv[1], argv[2], false, true, NULL);
//    dwRet = pSeedTarget->Init(argv[1], argv[2], false, false, "D:\\SeedTest\\dst\\XP_SP3.cache");
//    dwRet = pSeedTarget->Init(argv[1], argv[2], false, true, "D:\\SeedTest\\dst\\XP_SP3.cache");
//    dwRet = pSeedTarget->Init(argv[1], argv[2], true, false, "D:\\SeedTest\\dst\\XP_SP3.cache");
//    dwRet = pSeedTarget->Init(argv[1], argv[2], true, true, "D:\\SeedTest\\dst\\XP_SP3.cache");
    dwRet = pSeedTarget->Init(argv[1], argv[2], true, true);
    if ( dwRet != ERROR_SUCCESS )
    {
        log_wprintf(L"target(%hs%hs) invalid: %d\n", argv[2], argv[1], dwRet);
        goto out4;
    }

    pBitMask = pSeedTarget->GetProgressMask();
    nDegree = pBitMask->GetDegree();
    pBitMask->GetCompactMask(qwMaskAnd, qwMaskOr);
    log_wprintf(L"target file: degree: %d, mask: %#.16llx\n", nDegree, qwMaskAnd);

    dwRet = (pSeedTarget->IsFinished()) ? ERROR_SUCCESS : ERROR_NOT_READY;

    pSeedTarget->Fini();

out4:
    pSeedTarget->Release();

out3:
    ExecutorCenter::Instance()->Unload();

out2:
    ExecutorCenter::Destroy();
    SHA1Hash::Cleanup();

out1:
    return dwRet;
}
