/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <windows.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include "NtFileLib.h"


DWORD DoCatFile(HANDLE hFile)
{
    DWORD  dwRet = ERROR_SUCCESS;
    BYTE  buf[1024];
    ULONG  ulSize;

    _setmode(_fileno(stdout), _O_BINARY);

    while ( true )
    {
        ulSize = sizeof(buf);
        dwRet = ntReadFile(hFile, buf, &ulSize);
        if ( dwRet != ERROR_SUCCESS )
        {
            fwprintf(stderr, L"ReadFile failed: %d\n", dwRet);
            break;
        }

        if ( ulSize == 0 )
        {
            break;
        }

        // fwrite returns the number of full items actually written,
        // which may be less than count if an error occurs.
        if ( fwrite(buf, 1, ulSize, stdout) != ulSize )
        {
            dwRet = GetLastError();
            fwprintf(stderr, L"Output failed: %d\n", dwRet);
            break;
        }
    }

    return dwRet;
}

int __cdecl wmain(int argc, wchar_t *argv[])
{
    DWORD  dwRet;
    HANDLE  hFile;

    if ( argc < 2 )
    {
        fwprintf(stderr, L"NtFileCat.exe <file_name>\n");
        return ERROR_INVALID_PARAMETER;
    }

    dwRet = ntOpenFileW(&hFile, argv[1], "rb");
    if ( dwRet != ERROR_SUCCESS )
    {
        fwprintf(stderr, L"OpenFile(%s) failed: %d\n", argv[1], dwRet);
        goto err1;
    }

    dwRet = DoCatFile(hFile);
    if ( dwRet != ERROR_SUCCESS )
    {
        fwprintf(stderr, L"CatFile(%s) failed: %d\n", argv[1], dwRet);
        goto err2;
    }

err2:
    ntCloseFile(hFile);

err1:
    return dwRet;
}
