/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <windows.h>
#include <winternl.h>
#include <stdio.h>
#include "NtFileLib.h"


#ifdef __cplusplus
extern "C"
{
#endif
// from WinDDK\7600.16385.1\inc\api\ntstatus.h
#define STATUS_END_OF_FILE               ((NTSTATUS)0xC0000011L)


// from WinDDK\7600.16385.1\inc\ddk\wdm.h
#define FILE_OPEN                       0x00000001
#define FILE_OVERWRITE_IF               0x00000005

#define FILE_WRITE_THROUGH                      0x00000002
#define FILE_NO_INTERMEDIATE_BUFFERING          0x00000008
#define FILE_SYNCHRONOUS_IO_NONALERT            0x00000020
#define FILE_NON_DIRECTORY_FILE                 0x00000040

typedef enum _FILE_INFORMATION_CLASS_ {
    FileStandardInformation        = 5,
    FilePositionInformation        = 14,
    FileEndOfFileInformation       = 20
} FILE_INFORMATION_CLASS_, *PFILE_INFORMATION_CLASS_;

typedef struct _FILE_STANDARD_INFORMATION {
    LARGE_INTEGER AllocationSize;
    LARGE_INTEGER EndOfFile;
    ULONG NumberOfLinks;
    BOOLEAN DeletePending;
    BOOLEAN Directory;
} FILE_STANDARD_INFORMATION, *PFILE_STANDARD_INFORMATION;

typedef struct _FILE_POSITION_INFORMATION {
    LARGE_INTEGER CurrentByteOffset;
} FILE_POSITION_INFORMATION, *PFILE_POSITION_INFORMATION;

typedef enum _FSINFOCLASS {
    FileFsVolumeInformation       = 1,
    FileFsLabelInformation,      // 2
    FileFsSizeInformation,       // 3
    FileFsDeviceInformation,     // 4
    FileFsAttributeInformation,  // 5
    FileFsControlInformation,    // 6
    FileFsFullSizeInformation,   // 7
    FileFsObjectIdInformation,   // 8
    FileFsDriverPathInformation, // 9
    FileFsVolumeFlagsInformation,// 10
    FileFsMaximumInformation
} FS_INFORMATION_CLASS, *PFS_INFORMATION_CLASS;


// from WinDDK\7600.16385.1\inc\api\ntdef.h
#define NT_SUCCESS(Status) (((NTSTATUS)(Status)) >= 0)

#define OBJ_CASE_INSENSITIVE    0x00000040L

#define InitializeObjectAttributes( p, n, a, r, s ) { \
    (p)->Length = sizeof( OBJECT_ATTRIBUTES );          \
    (p)->RootDirectory = r;                             \
    (p)->Attributes = a;                                \
    (p)->ObjectName = n;                                \
    (p)->SecurityDescriptor = s;                        \
    (p)->SecurityQualityOfService = NULL;               \
    }


// from WinDDK\7600.16385.1\inc\ddk\ntddk.h
typedef struct _FILE_END_OF_FILE_INFORMATION {
    LARGE_INTEGER EndOfFile;
} FILE_END_OF_FILE_INFORMATION, *PFILE_END_OF_FILE_INFORMATION;

typedef struct _FILE_FS_SIZE_INFORMATION {
    LARGE_INTEGER TotalAllocationUnits;
    LARGE_INTEGER AvailableAllocationUnits;
    ULONG SectorsPerAllocationUnit;
    ULONG BytesPerSector;
} FILE_FS_SIZE_INFORMATION, *PFILE_FS_SIZE_INFORMATION;


// from WinDDK\7600.16385.1\inc\ddk\ntifs.h
NTSTATUS
NTAPI
NtReadFile (
    __in HANDLE FileHandle,
    __in_opt HANDLE Event,
    __in_opt PIO_APC_ROUTINE ApcRoutine,
    __in_opt PVOID ApcContext,
    __out PIO_STATUS_BLOCK IoStatusBlock,
    __out_bcount(Length) PVOID Buffer,
    __in ULONG Length,
    __in_opt PLARGE_INTEGER ByteOffset,
    __in_opt PULONG Key
    );

NTSTATUS
NTAPI
NtWriteFile (
    __in HANDLE FileHandle,
    __in_opt HANDLE Event,
    __in_opt PIO_APC_ROUTINE ApcRoutine,
    __in_opt PVOID ApcContext,
    __out PIO_STATUS_BLOCK IoStatusBlock,
    __in_bcount(Length) PVOID Buffer,
    __in ULONG Length,
    __in_opt PLARGE_INTEGER ByteOffset,
    __in_opt PULONG Key
    );

NTSTATUS
NTAPI
NtQueryInformationFile (
    __in HANDLE FileHandle,
    __out PIO_STATUS_BLOCK IoStatusBlock,
    __out_bcount(Length) PVOID FileInformation,
    __in ULONG Length,
    __in FILE_INFORMATION_CLASS FileInformationClass
    );

NTSTATUS
NTAPI
NtSetInformationFile (
    __in HANDLE FileHandle,
    __out PIO_STATUS_BLOCK IoStatusBlock,
    __in_bcount(Length) PVOID FileInformation,
    __in ULONG Length,
    __in FILE_INFORMATION_CLASS FileInformationClass
    );

NTSTATUS
NTAPI
NtQueryVolumeInformationFile (
    __in HANDLE FileHandle,
    __out PIO_STATUS_BLOCK IoStatusBlock,
    __out_bcount(Length) PVOID FsInformation,
    __in ULONG Length,
    __in FS_INFORMATION_CLASS FsInformationClass
    );
#ifdef __cplusplus
}
#endif


DWORD CALLCONV
ntOpenFileA
    (
        HANDLE                * phFile,
        const char            * lpFilePath,
        const char            * szOpenMode
    )
{
    DWORD  dwRet;
    size_t  bytes;
    wchar_t  wszFilePath[MAX_PATH];

    dwRet = mbstowcs_s(&bytes, wszFilePath, _countof(wszFilePath), lpFilePath, _TRUNCATE);
    if ( dwRet != 0 )
    {
        return dwRet;
    }

    return ntOpenFileW(phFile, wszFilePath, szOpenMode);
}

DWORD CALLCONV
ntOpenFileW
    (
        HANDLE                * phFile,
        const wchar_t         * lpFilePath,
        const char            * szOpenMode
    )
{
    NTSTATUS  ntStatus;
    UNICODE_STRING  uniFilePath;
    OBJECT_ATTRIBUTES ObjAttr;
    IO_STATUS_BLOCK  IoStatus;
    ACCESS_MASK  AccessMask;
    ULONG  CreateDisp;
    HANDLE  hFile;
    char  ch;

    *phFile = NULL;

    RtlInitUnicodeString(&uniFilePath, lpFilePath);
    InitializeObjectAttributes(&ObjAttr, &uniFilePath, OBJ_CASE_INSENSITIVE, NULL, NULL);

    AccessMask = 0;
    CreateDisp = 0;
    while ( (ch = *szOpenMode++) )
    {
        switch ( ch )
        {
            case 'r':
                AccessMask |= FILE_GENERIC_READ;
                CreateDisp |= FILE_OPEN;
                break;

            case 'w':
                AccessMask |= FILE_GENERIC_WRITE;
                CreateDisp |= FILE_OVERWRITE_IF;
                break;

            case '+':
                if ( AccessMask | FILE_GENERIC_READ )
                {
                    AccessMask |= FILE_GENERIC_WRITE;
                }
                else if ( AccessMask | FILE_GENERIC_WRITE )
                {
                    AccessMask |= FILE_GENERIC_READ;
                }
                break;

            default:
                break;
        }
    }

    // FILE_NO_INTERMEDIATE_BUFFERING is incompatible with the DesiredAccess's FILE_APPEND_DATA flag.
    // FILE_GENERIC_WRITE include FILE_APPEND_DATA
    ntStatus = NtCreateFile(&hFile, AccessMask, &ObjAttr, &IoStatus, NULL,
                            FILE_ATTRIBUTE_NORMAL,
                            FILE_SHARE_READ|FILE_SHARE_WRITE,
                            CreateDisp,
                            FILE_NON_DIRECTORY_FILE|FILE_SYNCHRONOUS_IO_NONALERT|((AccessMask & FILE_GENERIC_WRITE) ? (FILE_WRITE_THROUGH) : 0),
                            NULL,
                            0);
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtCreateFile(%s) failed: %x\n", lpFilePath, ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

    *phFile = hFile;

    return ERROR_SUCCESS;
}

void CALLCONV
ntCloseFile
    (
        HANDLE                  hFile
    )
{
    NtClose(hFile);
}

DWORD CALLCONV
ntReadFile
    (
        HANDLE                  hFile,
        BYTE                  * pBuffer,
        ULONG                 * pulSize
    )
{
    NTSTATUS  ntStatus;
    IO_STATUS_BLOCK  IoStatus;

    ntStatus = NtReadFile(hFile, NULL, NULL, NULL, &IoStatus, pBuffer, *pulSize, NULL, NULL);
    if ( ntStatus == STATUS_END_OF_FILE )
    {
        *pulSize = 0;
        return ERROR_SUCCESS;//ERROR_HANDLE_EOF;
    }
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtReadFile failed: %x\n", ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

    *pulSize = (ULONG)IoStatus.Information;

    return ERROR_SUCCESS;
}

DWORD CALLCONV
ntWriteFile
    (
        HANDLE                  hFile,
        BYTE                  * pBuffer,
        ULONG                   ulSize
    )
{
    NTSTATUS  ntStatus;
    IO_STATUS_BLOCK  IoStatus;

    ntStatus = NtWriteFile(hFile, NULL, NULL, NULL, &IoStatus, pBuffer, ulSize, NULL, NULL);
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtWriteFile failed: %x\n", ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

//    ulSize == (ULONG)IoStatus.Information;

    return ERROR_SUCCESS;
}

DWORD CALLCONV
ntSeekFile
    (
        HANDLE                  hFile,
        LONGLONG                llOffset
    )
{
    NTSTATUS  ntStatus;
    IO_STATUS_BLOCK  IoStatus;
    FILE_POSITION_INFORMATION  FilePosInfo;

    FilePosInfo.CurrentByteOffset.QuadPart = llOffset;
    ntStatus = NtSetInformationFile(hFile, &IoStatus, &FilePosInfo, sizeof(FilePosInfo), (FILE_INFORMATION_CLASS)FilePositionInformation);
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtSetInformationFile(FilePositionInformation) failed: %x\n", ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

    return ERROR_SUCCESS;
}

DWORD CALLCONV
ntGetFileLength
    (
        HANDLE                  hFile,
        LONGLONG              * pllLength
    )
{
    NTSTATUS  ntStatus;
    IO_STATUS_BLOCK  IoStatus;
    FILE_STANDARD_INFORMATION  FileStdInfo;

    ntStatus = NtQueryInformationFile(hFile, &IoStatus, &FileStdInfo, sizeof(FileStdInfo), (FILE_INFORMATION_CLASS)FileStandardInformation);
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtQueryInformationFile(FileStandardInformation) failed: %x\n", ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

    *pllLength = FileStdInfo.AllocationSize.QuadPart;

    return ERROR_SUCCESS;
}

DWORD CALLCONV
ntSetFileLength
    (
        HANDLE                  hFile,
        LONGLONG                llLength
    )
{
    NTSTATUS  ntStatus;
    IO_STATUS_BLOCK  IoStatus;
    FILE_END_OF_FILE_INFORMATION  FileEndInfo;

    FileEndInfo.EndOfFile.QuadPart = llLength;
    ntStatus = NtSetInformationFile(hFile, &IoStatus, &FileEndInfo, sizeof(FileEndInfo), (FILE_INFORMATION_CLASS)FileEndOfFileInformation);
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtSetInformationFile(FileEndOfFileInformation) failed: %x\n", ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

    return ERROR_SUCCESS;
}

DWORD CALLCONV
ntGetVolumeSizeInfo
    (
        HANDLE                  hVolume,
        LONGLONG              * pllTotalBytes,
        LONGLONG              * pllFreeBytes
    )
{
    NTSTATUS  ntStatus;
    IO_STATUS_BLOCK  IoStatus;
    FILE_FS_SIZE_INFORMATION  FsSizeInfo;
    ULONG  ulBytesPerUint;

    ntStatus = NtQueryVolumeInformationFile(hVolume, &IoStatus, &FsSizeInfo, sizeof(FsSizeInfo), (FS_INFORMATION_CLASS)FileFsSizeInformation);
    if ( !NT_SUCCESS(ntStatus) )
    {
        fwprintf(stderr, L"NtQueryVolumeInformationFile(FileFsSizeInformation) failed: %x\n", ntStatus);
        return ERROR_NOT_SUPPORTED;
    }

    ulBytesPerUint = FsSizeInfo.SectorsPerAllocationUnit * FsSizeInfo.BytesPerSector;
    *pllTotalBytes = FsSizeInfo.TotalAllocationUnits.QuadPart * ulBytesPerUint;
    *pllFreeBytes = FsSizeInfo.AvailableAllocationUnits.QuadPart * ulBytesPerUint;

    return ERROR_SUCCESS;
}
