/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/
// TCUpdateDlg.cpp : implementation file
//

#include "TCUpdate.h"
#include "TCUpdateDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define WM_UPDPS_MSG          (WM_USER + 100)
#define WM_UPDPS_END          (WM_USER + 102)

#define WM_NOTIFY_ICON        (WM_USER + 200)


CTCUpdateDlg::CTCUpdateDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTCUpdateDlg::IDD, pParent)
	, m_bStarted(false)
    , m_hQuitEvent(NULL)
    , m_hStartEvent(NULL)
    , m_hStopEvent(NULL)
    , m_hUpdQuitEvt(NULL)
    , m_hWorkThread(NULL)
    , m_hStdOutPipe(NULL)
    , m_hUpdProcess(NULL)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

    InitializeCriticalSection(&m_csOutput);
}

CTCUpdateDlg::~CTCUpdateDlg()
{
    DeleteCriticalSection(&m_csOutput);
}

void CTCUpdateDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_MSG, m_edtMsg);
	DDX_Control(pDX, IDC_BTN_START, m_btnStart);
}

BEGIN_MESSAGE_MAP(CTCUpdateDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
    ON_WM_CLOSE()
	ON_WM_SIZE()
	ON_WM_DESTROY()
    ON_WM_TIMER()
	ON_BN_CLICKED(IDC_BTN_START, OnClickedBtnStart)
    ON_MESSAGE(WM_NOTIFY_ICON, OnNotifyIcon)
    ON_MESSAGE(WM_UPDPS_MSG, OnUpdPsMsg)
    ON_MESSAGE(WM_UPDPS_END, OnUpdPsEnd)
	ON_COMMAND(ID_MENU_SHOW, &CTCUpdateDlg::OnMenuShow)
	ON_COMMAND(ID_MENU_QUIT, &CTCUpdateDlg::OnMenuQuit)
END_MESSAGE_MAP()


// CTCUpdateDlg message handlers

BOOL CTCUpdateDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetIcon(m_hIcon, TRUE);
	SetIcon(m_hIcon, FALSE);

    CString  tip;
    tip.LoadString(IDS_NID_TIP);
    m_nid.cbSize = sizeof(NOTIFYICONDATA);
	m_nid.hWnd = GetSafeHwnd();
    m_nid.uID = IDR_MAINFRAME;
    m_nid.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
    m_nid.uCallbackMessage = WM_NOTIFY_ICON;
    m_nid.hIcon = LoadIcon(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_MAINFRAME));
    strncpy_s(m_nid.szTip, 128, tip, _TRUNCATE);
    Shell_NotifyIcon(NIM_ADD, &m_nid);

    m_PopupMenu.LoadMenu(IDR_POPUPMENU);

    if ( InitUpdate() )
    {
        m_btnStart.EnableWindow(true);
        PostMessage(WM_SYSCOMMAND, SC_MINIMIZE, 0);
        PostMessage(WM_COMMAND, MAKEWPARAM(IDC_BTN_START, BN_CLICKED), NULL);
        SetTimer(1, 10*60*1000, NULL);
    }
    else
    {
        CString  msg;
        msg.LoadString(IDS_ERR_INIT);
        MessageBox(msg);
    }

	return TRUE;
}

void CTCUpdateDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if ( nType == SIZE_MINIMIZED)
    {
        ShowWindow(SW_HIDE);
    }
}

void CTCUpdateDlg::OnClose()
{
    ShowWindow(SW_HIDE);
}

void CTCUpdateDlg::OnMenuShow()
{
    ShowWindow(SW_SHOWNORMAL);
}

void CTCUpdateDlg::OnMenuQuit()
{
    // see issues_OnClose.txt
	EndDialog(0);//CDialog::OnClose();
}

void CTCUpdateDlg::OnDestroy()
{
    Shell_NotifyIcon(NIM_DELETE, &m_nid);

    KillTimer(1); 
    FiniUpdate();

    CDialog::OnDestroy();
}

void CTCUpdateDlg::OnTimer(UINT_PTR nIDEvent)
{
    if ( !m_bStarted )
    {
        PostMessage(WM_COMMAND, MAKEWPARAM(IDC_BTN_START, BN_CLICKED), NULL);
    }

    CDialog::OnTimer(nIDEvent);
}

void CTCUpdateDlg::OnPaint()
{
	if (IsIconic())
	{
		int smCxIcon;
		int smCyIcon;
		CRect clientRect;
		int screenX;
		int screenY;
			
		CPaintDC paintDC(this);
		
		SendMessage(WM_ICONERASEBKGND, 
			reinterpret_cast<WPARAM>(paintDC.GetSafeHdc()),
			0);

		smCxIcon = GetSystemMetrics(SM_CXICON);
		smCyIcon = GetSystemMetrics(SM_CYICON);
		
		GetClientRect(&clientRect);
		screenX = (clientRect.Width() - smCxIcon + 1) / 2;
		screenY = (clientRect.Height() - smCyIcon + 1) / 2;

		paintDC.DrawIcon(screenX, screenY, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

HCURSOR CTCUpdateDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}


void CTCUpdateDlg::OnClickedBtnStart()
{
    CString  str;

    if ( m_bStarted )
    {
        StopUpdate();
        str.LoadString(IDS_BTN_START);
    }
    else
    {
        StartUpdate();
        str.LoadString(IDS_BTN_STOP);
    }

    m_btnStart.SetWindowText(str);

    m_bStarted = !m_bStarted;
}

LRESULT CTCUpdateDlg::OnNotifyIcon(WPARAM wParam, LPARAM lParam)
{
    if ( lParam == WM_RBUTTONDOWN )
    {
        CMenu  *pSubMenu;
		pSubMenu = m_PopupMenu.GetSubMenu(0);
		CPoint ptCur;
		GetCursorPos(&ptCur);
        SetForegroundWindow();
		pSubMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, ptCur.x, ptCur.y, this);
    }
    else if ( lParam == WM_LBUTTONDBLCLK )
    {
        ShowWindow(SW_SHOWNORMAL);
        SetForegroundWindow();
    }

    return 1;
}

LRESULT CTCUpdateDlg::OnUpdPsMsg(WPARAM wParam, LPARAM lParam)
{
    EnterCriticalSection(&m_csOutput);
    m_edtMsg.SetWindowTextA(m_strOutput);
    LeaveCriticalSection(&m_csOutput);

    m_edtMsg.LineScroll(m_edtMsg.GetLineCount());

    return 0;
}

LRESULT CTCUpdateDlg::OnUpdPsEnd(WPARAM wParam, LPARAM lParam)
{
    CString  str;

    StopUpdate();
    m_bStarted = false;

    str.LoadString(IDS_BTN_START);
    m_btnStart.SetWindowText(str);

    return 0;
}

BOOL CTCUpdateDlg::InitUpdate()
{
    m_hQuitEvent = CreateEvent(NULL, TRUE, FALSE, NULL);
    if ( m_hQuitEvent == NULL )
    {
        goto err1;
    }

    m_hStartEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if ( m_hStartEvent == NULL )
    {
        goto err2;
    }

    m_hStopEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
    if ( m_hStopEvent == NULL )
    {
        goto err3;
    }

    m_hUpdQuitEvt = CreateEvent(NULL, TRUE, FALSE, "Global\\upd-process");
    if ( m_hUpdQuitEvt == NULL )
    {
        goto err4;
    }

    m_hWorkThread = CreateThread(NULL, 0, WorkThreadProc, (LPVOID)this, 0, NULL);
    if ( m_hWorkThread == NULL )
    {
        goto err5;
    }

    return TRUE;

err5:
    CloseHandle(m_hUpdQuitEvt);
    m_hUpdQuitEvt = NULL;

err4:
    CloseHandle(m_hStopEvent);
    m_hStopEvent = NULL;

err3:
    CloseHandle(m_hStartEvent);
    m_hStartEvent = NULL;

err2:
    CloseHandle(m_hQuitEvent);
    m_hQuitEvent = NULL;

err1:
    return FALSE;
}

VOID CTCUpdateDlg::FiniUpdate()
{
    StopUpdate();

    if ( m_hWorkThread != NULL )
    {
        SetEvent(m_hQuitEvent);
        WaitForSingleObject(m_hWorkThread, INFINITE);

        CloseHandle(m_hWorkThread);
        m_hWorkThread = NULL;
    }

    if ( m_hUpdQuitEvt != NULL )
    {
        CloseHandle(m_hUpdQuitEvt);
        m_hUpdQuitEvt = NULL;
    }

    if ( m_hStopEvent != NULL )
    {
        CloseHandle(m_hStopEvent);
        m_hStopEvent = NULL;
    }

    if ( m_hStartEvent != NULL )
    {
        CloseHandle(m_hStartEvent);
        m_hStartEvent = NULL;
    }

    if ( m_hQuitEvent != NULL )
    {
        CloseHandle(m_hQuitEvent);
        m_hQuitEvent = NULL;
    }
}

VOID CTCUpdateDlg::StartUpdate()
{
    if ( m_hStartEvent != NULL )
    {
        ResetEvent(m_hUpdQuitEvt);
        SetEvent(m_hStartEvent);
    }
}

VOID CTCUpdateDlg::StopUpdate()
{
    if ( m_hStopEvent != NULL )
    {
        SetEvent(m_hUpdQuitEvt);
        SetEvent(m_hStopEvent);
    }
}

DWORD CTCUpdateDlg::WorkThread()
{
    DWORD  dwRet;
    DWORD  dwWait;
    HANDLE  hEvents[] = {m_hQuitEvent, m_hStartEvent, m_hStopEvent};

    dwRet = PrepareUpdEnv();
    if ( dwRet != ERROR_SUCCESS )
    {
        goto err1;
    }

    while ( true )
    {
        dwWait = WaitForMultipleObjects(_countof(hEvents), hEvents, FALSE, INFINITE);

        if ( (dwWait == WAIT_OBJECT_0) || (dwWait == WAIT_FAILED) )
        {
            EndUpdProcess();
            break;
        }

        if ( dwWait == (WAIT_OBJECT_0 + 1) )
        {
            RunUpdProcess();
        }
        else
        {
            EndUpdProcess();
        }

        if ( m_hUpdProcess == NULL )
        {
            continue;
        }

        while ( ReadUpdProcess() );

        PostMessage(WM_UPDPS_END);
    }

err1:
    return dwRet;
}

DWORD WINAPI CTCUpdateDlg::WorkThreadProc(LPVOID lpParameter)
{
    CTCUpdateDlg  *pThis = (CTCUpdateDlg*)lpParameter;

    return pThis->WorkThread();
}

DWORD CTCUpdateDlg::PrepareUpdEnv()
{
    DWORD  dwRet;
    DWORD  dwSize;
    HRSRC  hPkgInfo;
    HGLOBAL  hPkgData;
    LPVOID  pPkgData;

    if ( GetTempPath(_countof(m_szWorkDir), m_szWorkDir) == 0 )
    {
        dwRet = GetLastError();
        goto err1;
    }

    hPkgInfo = FindResource(NULL, MAKEINTRESOURCE(IDR_UPPKG), "EXE");
    if ( hPkgInfo == NULL )
    {
        dwRet = GetLastError();
        goto err1;
    }

    dwSize = SizeofResource(NULL, hPkgInfo);
    if ( dwSize == 0 )
    {
        dwRet = GetLastError();
        goto err1;
    }

    hPkgData = LoadResource(NULL, hPkgInfo);
    if ( hPkgData == NULL )
    {
        dwRet = GetLastError();
        goto err1;
    }

    pPkgData = LockResource(hPkgData);
    if ( pPkgData == NULL )
    {
        dwRet = GetLastError();
        goto err2;
    }

    dwRet = ExportUpdPkg(pPkgData, dwSize);

err2:
    FreeResource(hPkgData);

err1:
    return dwRet;
}

DWORD CTCUpdateDlg::ExportUpdPkg(LPVOID pPkgData, DWORD dwPkgSize)
{
    DWORD  dwRet = ERROR_SUCCESS;
    char  szExeFile[MAX_PATH], szCmdLine[MAX_PATH];
    FILE  *hPkgFile;
    STARTUPINFO  si;
    PROCESS_INFORMATION  pi;

    _snprintf_s(szExeFile, _countof(szExeFile), _TRUNCATE, "%s\\%s", m_szWorkDir, "tcuppkg.exe");
    if ( fopen_s(&hPkgFile, szExeFile, "wb") != 0 )
    {
        dwRet = GetLastError();
        goto err1;
    }

    if ( fwrite(pPkgData, 1, dwPkgSize, hPkgFile) != dwPkgSize )
    {
        dwRet = GetLastError();
    }

    fclose(hPkgFile);

    if ( dwRet != ERROR_SUCCESS )
    {
        goto err1;
    }


    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    _snprintf_s(szCmdLine, _countof(szCmdLine), _TRUNCATE, "%s -y", szExeFile);
    if ( !CreateProcess(NULL, szCmdLine, NULL, NULL, FALSE, CREATE_NO_WINDOW, NULL, m_szWorkDir, &si, &pi) )
    {
        dwRet = GetLastError();
        goto err1;
    }

    WaitForSingleObject(pi.hProcess, INFINITE);

    CloseHandle(pi.hProcess);
    CloseHandle(pi.hThread);

err1:
    return dwRet;
}

DWORD CTCUpdateDlg::RunUpdProcess()
{
    DWORD  dwRet;
    HANDLE  hPipeWrite;
    char  szCmdLine[MAX_PATH];
    STARTUPINFO  si;
    PROCESS_INFORMATION  pi;

    if ( m_hUpdProcess )
    {
        return ERROR_SUCCESS;
    }

    if ( !CreatePipe(&m_hStdOutPipe, &hPipeWrite, NULL, 0) )
    {
        dwRet = GetLastError();
        goto err1;
    }
    SetHandleInformation(hPipeWrite, HANDLE_FLAG_INHERIT, HANDLE_FLAG_INHERIT);

    _snprintf_s(szCmdLine, _countof(szCmdLine), _TRUNCATE, "%s\\bash.exe update_os.sh", m_szWorkDir);
    memset(&si, 0, sizeof(si));
    si.cb = sizeof(si);
    si.hStdOutput = hPipeWrite;
    si.dwFlags = STARTF_USESTDHANDLES;
    if ( !CreateProcess(NULL, szCmdLine, NULL, NULL, TRUE, CREATE_NO_WINDOW, NULL, m_szWorkDir, &si, &pi) )
    {
        dwRet = GetLastError();
        goto err2;
    }

    CloseHandle(hPipeWrite);
    CloseHandle(pi.hThread);

    m_hUpdProcess = pi.hProcess;

    return ERROR_SUCCESS;

err2:
    CloseHandle(hPipeWrite);
    CloseHandle(m_hStdOutPipe);
    m_hStdOutPipe = NULL;

err1:
    return dwRet;
}

VOID CTCUpdateDlg::EndUpdProcess()
{
    if ( m_hUpdProcess == NULL )
    {
        return;
    }

    WaitForSingleObject(m_hUpdProcess, INFINITE);
    CloseHandle(m_hUpdProcess);
    m_hUpdProcess = NULL;

    CloseHandle(m_hStdOutPipe);
    m_hStdOutPipe = NULL;
}

BOOL CTCUpdateDlg::ReadUpdProcess()
{
    DWORD  dwBytes;
    char  buf[500];

    if ( !ReadFile(m_hStdOutPipe, buf, sizeof(buf) - 1, &dwBytes, NULL) )
    {
        return FALSE;
    }
    buf[dwBytes] = 0;

    EnterCriticalSection(&m_csOutput);
    m_strOutput += buf;
    LeaveCriticalSection(&m_csOutput);

    PostMessage(WM_UPDPS_MSG);

    return TRUE;
}

