/** @file **/
#ifndef _TCCTL_H
#define _TCCTL_H

#ifdef _WIN32
#include <winsock2.h>
#include <iphlpapi.h>
#include <ws2tcpip.h>
#include <time.h>
#include <WinIoCtl.h>
#include <Wincon.h>
#include <windows.h>
#else
#include <unistd.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/time.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <signal.h>
#include "TcConfig.h"
#include "TcDebugLib.h"

#if !defined _WIN32 && !defined ANDROID
#include "localsock.h"
#endif

#ifdef _WIN32
#define REBOOT_CMD_STR		"shutdown -f -r -t 0"
#define SHUTDOWN_CMD_STR	"shutdown -f -s -t 0"
#ifndef SIGUSR1
#define	SIGUSR1				10
#endif

#else
#define REBOOT_CMD_STR		"reboot"
#ifdef ANDROID
#define SHUTDOWN_CMD_STR	"reboot -p"
#else
 #ifdef LITE_LINUX
#define SHUTDOWN_CMD_STR	"/usr/sbin/tc/tc-shutdown.sh --poweroff"
 #else
#define SHUTDOWN_CMD_STR	"poweroff"
 #endif
#endif
#endif


#define SND_BUF_SZ 			          1024*1024
#define RCV_BUF_SZ 			          1024*1024

// the multicast ipaddr and port for data packet transfer. 
#define GROUP_IP 			              "226.1.1.1"
#define GROUP_PORT			              20000

// the default control port. 
#define CTRL_SRV_IP			          "192.168.0.1"
#define CTRL_PORT			              12345
#define CLT_PORT                        12345

#define INFO_SEP_CHAR		              ':'
#define INFO_SEP_STR		              ":"
#define UNAME_SZ			              256
#define OSNAME_SZ			              256
#define IP_BUF_SZ			              128
#define IMAGE_NAME_SZ			          128
#define USER_NAME_SZ			          128
#define CLIENT_NAME_SZ                  64
#define USER_TOKEN_SZ                   32

#define POST_REPORT_HWINFO              1
//#define	ENABLE_HB			              1
//in seconds
#define	HB_TIME_INTERVAL	              9

#define MAPSZ		     1
#define HWA_SZ		     6
#define INFO_SZ		 32
#define VERINFO_SZ	     64
#define FNAME_SZ	     256
#define IMAGENAME_SZ   256
#define SLOT_BYTES	    1460
#define CPU_STR_SZ	    64
#define STRIP_SZ	    16

// here the type definition for message exchanging between server and clients. 
typedef unsigned char uint8;
typedef unsigned short uint16;
typedef unsigned int uint32;
typedef unsigned long long uint64;


typedef void (*P_SIG_HANDLER)(int);


#ifdef _WIN32
struct timezone
{
    int tz_minuteswest;
    int tz_dsttime;
};
#endif

typedef struct _TH_PARMS {
	int argc;
	char ** argv;
}TH_PARMS;

typedef struct bulkmap_struct
{
	uint32	bits;

#define TOTALSZ		(MAPSZ<<6)
#define	LOGTOALSZ	(MAPSZ+5)
	uint64 map[MAPSZ];
}BULKMAP;

#include "_tcmsgcommon.h"


// define filed name for reserved status configuration file,
// which would be read while OS images are running.
#define ST_STATUS     "statusid"
#define ST_USERNAME   "username"
#define ST_OSID       "imageid"
#define ST_DISKSZ     "disksz"
#define ST_MEMSZ      "memsz"
#define ST_CPUMODEL   "cpumodel"

///////////////////////////////////////////////////////////////////////////////
// Here goes the global function decalartion
///////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
extern "C"
{
#endif//__cplusplus
#ifdef _WIN32
extern DWORD WINAPI ServiceWorkerThread(LPVOID pparms);
#else
extern int ServiceWorkerThread(void *pparms);
#endif // _WIN32

extern int actusr_send(int socket, struct sockaddr_in sain, char *uname, char *osname, int act);
extern void cleanup(int code);

#ifdef __cplusplus
}
#endif//__cplusplus

///////////////////////////////////////////////////////////////////////////////
// Here goes the global variables decalartion
///////////////////////////////////////////////////////////////////////////////

#ifdef _WIN32
extern HINSTANCE hInst;
extern HWND hWnd;
#endif // _WIN32

extern char *cifn;
extern char *localip;
extern unsigned char *localmac;
//extern int g_statusSock;

// data channel socket, UDP
extern int g_groupSock;
//extern struct sockaddr_in g_ctrl_sock_addr;

#ifdef __cplusplus
extern "C"
{
#endif//__cplusplus

extern unsigned char g_exit;
extern int g_statusSock;
extern struct sockaddr_in g_ctrl_sock_addr;

// parseip.c
/**
  Fetch ip addr from local hard drive

  @param ip        a buffer to store the result
  @param ip_len    size of the buffer
  @return          0 on success, negative value if error
**/
extern
int
TC_STDCALL
ParseIpConfig (
  char *ip,
  size_t ip_len
  );



#ifdef __cplusplus
}
#endif//__cplusplus

// create a socket to receive unicast messages
extern int g_clt_sock;
extern struct sockaddr_in g_clt_sock_addr;


/////////////////////////////////////////////////////
// macros defined for win32 service
/////////////////////////////////////////////////////

// the status configuration file path
#if defined(_WIN32)
#define STATUS_CONFIG_FILE     "\\Device\\Harddisk0\\Partition3\\tc-status.conf"
//#define STATUS_CONFIG_FILE     "\\Device\\Harddisk0\\Partition2\\tc-status.conf"
#else
#define STATUS_CONFIG_FILE     "tc-status.conf"
#endif // _WIN32

#ifdef _WIN32
#define TC_LOCAL_MSG_SHUTDOWN 13
#endif //_WIN32



#endif // _TCCTL_H
