/** @file
  debug header
**/
#ifndef _TC_DEBUG_LIB_H_
#define _TC_DEBUG_LIB_H_

#include "TcDebug/TcDebugIn.h"
#include "TcConfig.h"

//
// Debug internal implmenation
//
#define DEBUG1(x,fmt,...)                                                      \
  do {                                                                         \
    if ((x) & D_DEBUG_LEVEL){                                                  \
      TcDebugOutput(L"%ls" fmt, DBG_PREFIX(x), ##__VA_ARGS__);                 \
    }                                                                          \
  } while (0)

//
// Debug interface
//
#define DEBUG_INIT(debugType, debugFile)  TcDebugInit(debugType, debugFile)
#define DEBUG(expression)                 DEBUG1 expression
#define DEBUG_FINISH()                    TcDebugFinish()

//
// Debug Level
//
#if !defined(D_DEBUG_LEVEL)
#define D_DEBUG_LEVEL                     (D_INFO | D_ERROR | D_WARNING)
#endif


//
// Print interface
//
#define PRINT_INIT(printType, printFile)  TcPrintInit(printType, printFile)
#define PRINT(wfmt,...)                   TcPrintOutput(wfmt,##__VA_ARGS__)
#define PRINT_FINISH()                    TcPrintFinish()


////////////////////////////////////////////////////////////////////////////////
#include "wchar.h"

#if !defined(TC_STDCALL)
# if defined(_WIN32)
#  define TC_STDCALL __stdcall
# else
#  define TC_STDCALL
# endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
  define stream type
**/
typedef enum _OUTPUT_STREAM_ENUM
{
  TO_STD_CONSOLE = 0,   // stdout, stderr
  TO_FILE,              // a file
  TO_CONSOLE            // use windows outputdebugstring
} OUTPUT_STREAM_ENUM;

/**
  Debug initialization
  
  @param streamEnumType  one value of TO_STD_CONSOLE, TO_FILE and TO_CONSOLE
  @param filename        when streamEnumType is TO_FILE, filename specify a file name;
                         when streamEnumType has other value, this parameter is ignored.
  
  @return 0 on success
**/
extern
int
TC_STDCALL
TcDebugInit (
  OUTPUT_STREAM_ENUM streamEnumType,
  wchar_t *filename
  );

/**
  Print initialization
  
  @param streamEnumType  one value of TO_STD_CONSOLE, TO_FILE and TO_CONSOLE
  @param filename        when streamEnumType is TO_FILE, filename specify a file name;
                         when streamEnumType has other value, this parameter is ignored.
  
  @return 0 on success
**/
extern
int
TC_STDCALL
TcPrintInit (
  OUTPUT_STREAM_ENUM streamEnumType,
  wchar_t *filename
  );

/**
  output a format string to debug console
  
  @param fmt     string include specifiers
  @param ...
  @return number of characters written, or negative value if errors occurr
**/
extern
int
TC_STDCALL
TcDebugOutput (
  const wchar_t *fmt,
  ...
  );


/**
  output a format string to print console
  
  @param fmt     string include specifiers
  @param ...
  @return number of characters written, or negative value if errors occurr
**/
extern
int
TC_STDCALL
TcPrintOutput (
  const wchar_t *fmt,
  ...
  );
  
/**
  close the open deubg handle
  
  @return none
**/
extern
int
TC_STDCALL
TcDebugFinish (
  void
  );

/**
  close the open deubg handle
  
  @return none
**/
extern
int
TC_STDCALL
TcPrintFinish (
  void
  );

#ifdef __cplusplus
} // close extern "C"
#endif

//#ifdef TC_STDCALL
//#undef TC_STDCALL
//#endif

#endif // _TC_DEBUG_LIB_H_
