/** 
  @file
  parse command line options
**/
#ifndef PARSEOPT_LIB_H
#define PARSEOPT_LIB_H

#include <string.h>
#include <wchar.h>

//
// if necessary,
// define PARSEOPT_API to something like
// __declspec(dllexport)
// or
// __declspec(dllimport)
//
#define PARSEOPT_API

//
// define PARSEOPT_THROW to use with C++
//
#ifdef __cplusplus
#define PARSEOPT_THROW throw()
#else
#define PARSEOPT_THROW
#endif

//
// define to calling convention to __stdcall while compiling with Win32 code
// otherwise define it to default __cdecl
//
#if !defined(PARSEOPT_STDCALL)
# if defined(_WIN32)
#  define PARSEOPT_STDCALL __stdcall
# else
#  define PARSEOPT_STDCALL
# endif
#endif

#ifdef __cplusplus
  extern "C"{
#endif

//
// global variables in the library
//
extern PARSEOPT_API int     g_parseopt_ind;
extern PARSEOPT_API int     g_parseopt_err;
extern PARSEOPT_API int     g_parseopt_opt;

//
// data structes definitions for
// wide chars and strings
//
struct parse_option_struc_wide
{
	const wchar_t             *name;
//
// define possible values for the 'has_arg' variable
//
#define no_argument         0
#define required_argument   (no_argument + 1)
#define optional_argument   (no_argument + 2)
	int                       has_arg;

	int                       *flag;
	int                       val;
};
extern PARSEOPT_API wchar_t *g_parseopt_arg_wide;

//
// iterfaces definitions for
// wide chars and strings
//
extern PARSEOPT_API
int PARSEOPT_STDCALL
parseopt_wide
(
  int                                  listcnt,
  wchar_t *const                       *listw,
  const wchar_t                        *shortopts
) PARSEOPT_THROW;

extern PARSEOPT_API
int PARSEOPT_STDCALL
parseopt_long_wide
(
  int                                  listcnt,
  wchar_t *const                       *listw,
  const wchar_t                        *shortopts,
  const struct parse_option_struc_wide *longopts,
  int                                  *longopt_index
) PARSEOPT_THROW;

extern PARSEOPT_API
int PARSEOPT_STDCALL
parseopt_long_only_wide
(
  int                                  listcnt,
  wchar_t *const                       *listw,
  const wchar_t                        *shortopts,
  const struct parse_option_struc_wide *longopts,
  int                                  *longopt_index
) PARSEOPT_THROW;

#ifdef __cplusplus
  }
#endif

//
// compatible with glibc getopt.h
//
#define optind      g_parseopt_ind
#define opterr      g_parseopt_err
#define optopt      g_parseopt_opt
#define optarg      g_parseopt_arg_wide
#define option      parse_option_struc_wide
#define getopt      parseopt_wide
#define getopt_long parseopt_long_wide


#endif // PARSEOPT_LIB_H
