/** @file

**/
#ifndef _TC_KEY_VALUE_LIB_H
#define _TC_KEY_VALUE_LIB_H

#define CONF_KEYBUF_SZ   128
#define CONF_VALBUF_SZ   256
#define CONF_LINEBUF_SZ  (CONF_KEYBUF_SZ + CONF_VALBUF_SZ - 1)

#if !defined(TC_STDCALL)
# if defined(_WIN32)
#  define TC_STDCALL __stdcall
# else
#  define TC_STDCALL
# endif
#endif

#if defined(__cplusplus)
extern "C"
{
#endif

struct config_data
{
    char key[CONF_KEYBUF_SZ];
    char value[CONF_VALBUF_SZ];
    struct config_data *next;
};

// the trim functions have side effects.
extern char* TC_STDCALL trim_head(char *src);
extern char* TC_STDCALL trim_tail(char *src);
extern char* TC_STDCALL trim(char *src);

extern int TC_STDCALL get_a_key_val(char *key, int keysz, char *value, int valuesz, const char *str, const char *delim);
extern int TC_STDCALL get_key_val(const char *filename, const char *delim, struct config_data **pconf);
extern void TC_STDCALL free_key_val(struct config_data *pconf);
extern void TC_STDCALL print_key_val(struct config_data *pconf);

// load configs from a string, save legal key-val pairs,
// and return the number legal options
// Note: the string may contain one or mutiple CR-LF('\r''\n') or LF('\n').
// And make sure each key-val pair in a line.
// if pconf is not NULL, the all key-val pairs would be saved there.
// if find key-val pairs, return the number of legal key-val pairs, > 0
// if not find key-val pairs, return = 0
// if errors, return < 0
//
extern int TC_STDCALL sget_key_val(const char *input, int inputsz, const char *delim, struct config_data **pconf);

extern void TC_STDCALL test_get_key_val();

#if defined(__cplusplus)
}
#endif

#endif // _TC_KEY_VALUE_LIB_H
