// _tcmsgcommon.h
// define common message struct here
typedef struct tcdmsg_struct
{
	// type and size of the message head
	uint32 type;
	uint32 size;

	union uu
	{
// registration request, from client to server
#define TCDMSG_REGREQ	1
		// register request
		struct rr_s
		{
			uint32 rrdata;

			// whether the hard disk was mounted to the /mnt. yes = 1, no = 0
			uint32 checkmnt;

			unsigned char hwa[HWA_SZ];

			char tcdver[VERINFO_SZ];

			char sysname[INFO_SZ];	/* Operating system name (e.g., "Linux") */
			char nodename[INFO_SZ];	/* Name within "some implementation-defined network" */
			char release[INFO_SZ];	/* Operating system release (e.g., "2.6.28") */
			char version[INFO_SZ];	/* Operating system version */
			char machine[INFO_SZ];	/* Hardware identifier */

			unsigned long totalram;	/* Total usable main memory size */
			unsigned long freeram;	/* Available memory size */
			unsigned long sharedram;/* Amount of shared memory */
			unsigned long bufferram;/* Memory used by buffers */

			unsigned long totalswap;/* Total swap space size */
			unsigned long freeswap;	/* swap space still available */
			unsigned long procs;	/* Number of current processes */

		}rr;

// de-registration request, from server to client 
#define TCDMSG_DEREGREQ	2
		// de-register request
		struct dr_s
		{
			uint32 drdata;
		}dr;

// registration acknowledgement, from client to server
#define TCDMSG_REGACK	3
		// register acknowledgement
		struct ra_s
		{
			uint32 radata;
		}ra;


// run request, from server to client
#define TCDMSG_RUNREQ	4
		// run request
		struct ur_s
		{
// defined in tcd.h
// #define FNAME_SZ	64
			uint8 fname[FNAME_SZ];
			uint32 hasnofile;
			// uint64 tslots;
			uint32 u32tslots;

			// these parameters are used for client-server negotiation.
			uint32 pktack_delay_base;
			uint32 pktack_delay_coeff;

			uint32 pktack_delay_intpkt;

			uint32 dyna_mapsz;

			uint32 clt_count;
			uint32 clt_list[1];
		}ur;

// stop request, from server to request, not used at this time
#define TCDMSG_STOPREQ	5
		// stop request
		struct sr_s
		{
			uint32 srdata;
		}sr;

// run acknowledgement, from client to server
#define TCDMSG_RUNACK	6
		// run acknowledgement
		struct ua_s
		{
			uint32 uadata;
		}ua;

// inquiry packet, from server to client
#define TCDMSG_PKTINQ	7
		// packet inquiry
		struct pi_s
		{
			uint32 bulk;
			uint32 slot;
		}pi;

// packet acknowledgement, from client to server
#define TCDMSG_PKTACK	8
		// packet acknowledgement
		struct pa_s
		{
			uint32 bulk;
			uint32 full;
			BULKMAP bulkmap;			
		}pa;

// data packet, from server to client. 
#define TCDMSG_DATA	9
		// data packet
		struct d_s
		{
			uint32 bulk;
			uint32 slot;
			uint32 hasnext;
			uint32 bytes;
			unsigned char data[1];
		}d;

// hello broadcast, from server to client
#define TCDMSG_HELLO	10
		// hello packet
		struct h_s
		{
			uint32 srvip;
			uint32 srvport;
		}h;


// shutdown packet, from server to client
#define TCDMSG_SHUTDOWN	11
		// shutdown packet
		struct sd_s
		{
			uint32 srvip;
			uint32 srvport;
		}sd;

// reboot packet, from server to client
#define TCDMSG_REBOOT	12
		// reboot packet
		struct rb_s
		{
			uint32 srvip;
			uint32 srvport;
		}rb;

// restart app packet, from server to client
#define TCDMSG_RESTARTAPP	13
		// restart app packet
		struct rsa_s
		{
			uint32 srvip;
			uint32 srvport;
		}rsa;


// identify broadcast, from server to client
#define TCDMSG_IDENTIFY	14

		// identify packet
		struct id_s
		{
			uint32 srvip;
			uint32 srvport;
			uint32 sys_part_capacity; /* the partition size (Units: MiB) */

			uint32 clt_count;
			uint32 clt_list[1];
		}id;





// notify packet, from client to server
#define TCDMSG_NOTIFY	15
		// notify packet
		struct nf_s
		{
// type of notifications
#define NF_UNKNOWN		1
#define NF_WRITE_FAILURE	2
#define NF_RECV_DONE		3
#define NF_REBOOT		4
#define NF_SHUTDOWN		5
#define NF_CREATE_FAILURE	6
#define NF_LAST_FILE		7
#define NF_RESTARTAPP		8
#define NF_FORMAT		9
#define NF_INITIALIZE		10
#define NF_UPDATEFIRMWARE	11
#define NF_INITHD_FAILURE	12
#define NF_CGSVRIP		13
#define NF_REBOOT_IP		14
#define NF_SHUTDOWN_IP		15

			uint32 nf_code;
			uint32 nf_comment;
		}nf;


// client information request, from server to client
#define TCDMSG_CLTINFO_REQ	16
		// client info request
		struct cir_s
		{
			uint32 srvip;
			uint32 srvport;

			uint32 clt_count;
			uint32 clt_list[1];
		}cir;

// client information acknowledgement, from client to server
#define TCDMSG_CLTINFO_ACK	17
		// client info acknowledgement
		struct cia_s
		{
			uint32 cltip;
			uint64 cltmac;
			uint32 cltarch;
			uint32 cltos;
		}cia;

// to check the packet acknowledgement, broadcast from server to client
#define TCDMSG_PKTACK_CHECK	18
		// broadcast each client's cbulk#, so the client losing pktack should re-send the packet. 
		struct pktack_check_s
		{
			uint32 srvip;
			uint32 srvport;

			uint32 clt_count;
			uint32 clt_ipcbulk_list[1];	// client1 ip, client1 cbulk, client2 ip, client2 cbulk ...
		}pac;

#define TCDMSG_USERCMD	19
		// restart app packet
		struct uc_s
		{
			uint32 srvip;
			uint32 srvport;

			uint32 clt_count;
			uint32 clt_ipcbulk_list[1];	// client1 ip, client1 cbulk, client2 ip, client2 cbulk ...

		}uc;

// format broadcast, from server to client
#define TCDMSG_FORMAT	20
		// format packet
		struct fm_s
		{
			uint32 srvip;
			uint32 srvport;

		}fm;


// to start the deployment, from controller to server
#define TCDMSG_CMDSTART	21

		// command start packet
		struct cs_s
		{
			// char fname[FNAME_SZ];
			unsigned char fnbuf[1];
		}cs;

#define TCDMSG_CMDLIST	23
#define TCDMSG_CMDQUIT	24
#define TCDMSG_CMDREBOOT	25
#define TCDMSG_CMDSHUTDOWN	26
#define TCDMSG_CMDIDENTIFY	27
#define TCDMSG_CMDUSER	28
#define TCDMSG_CMDRESTARTAPP	29

#define TCDMSG_CMDFORMAT	30


#define TCDMSG_CMDCHKREGREQ	31
#define TCDMSG_CMDCHKREGACK	32
		struct cr_s
		{
			unsigned char hwa[HWA_SZ];
			uint32 retval;
		}cr;

#define TCDMSG_CMDREGREQ	33
#define TCDMSG_CMDREGACK	34
		struct rg_s
		{
			unsigned char hwa[HWA_SZ];
			uint8 hname[FNAME_SZ]; 
			uint32 retval;
			uint32 diskcapacity;     /* disk capacity (Units: MiB) */
			uint32 totalram;         /* total ram (Units: MiB) */
			char cpumodelname[CPU_STR_SZ];   /* cpu model name */
		}rg;


#define TCDMSG_CMDINITIALIZE		35
		struct ciz_s
		{
			uint32 sys_part_capacity; /* the partition size (Units: MiB) */
		}ciz;
#define TCDMSG_CMDUPDATEFIRMWARE	36

// initialize broadcast, from server to client
#define TCDMSG_INITIALIZE	37
		// initialize packet
		struct iz_s
		{
			uint32 srvip;
			uint32 srvport;

		}iz;

// updatefirmware broadcast, from server to client
#define TCDMSG_UPDATEFIRMWARE	38
		// updatefirmware packet
		struct uf_s
		{
			uint32 srvip;
			uint32 srvport;
		}uf;

#define TCDMSG_ACTUSR	39
		struct au_s
		{
			unsigned char hwa[HWA_SZ];
			uint8 uname[UNAME_SZ];
			uint8 osname[OSNAME_SZ];

#define ACT_POWERON		1	// nothing
#define ACT_AUTHENTICATED	2	// username
#define ACT_SYNCING		3	// username and osname
#define ACT_BOOTING		4	// username and osname
#define ACT_RUNNING		5	// nothing
#define ACT_SHUTTING_DOWN	6	// nothing
#define ACT_UPLOADING		7	// username and osname
#define ACT_INITIALIZING_DISK	8	// nothing
#define ACT_UPDATING_FIRMWARE	90	// nothing
#define ACT_UPDATING_FIRMWARE_CMD_SENT	91	// nothing
#define ACT_UPDATING_FIRMWARE_SUCCESS	92	// nothing
#define ACT_UPDATING_FIRMWARE_FAILURE	93	// nothing
#define ACT_DEPLOYING		10	// nothing
#define ACT_INTERRUPTED		11	// nothing
#define ACT_WAITING		12	// nothing

#define ACT_HEARTBEAT		17	// nothing
			uint32 act;

#ifdef POST_REPORT_HWINFO
            int disksz;
            int memsz;
            char strcpu[CPU_STR_SZ];
#endif
		}au;


// CMD change server, from controller to server
#define TCDMSG_CMDCGSVRIP	40
		// change server ip command packet
		struct ccgs_s
		{
			uint32 srvip;
		}ccgs;

// CMD change server ip, from server to client
#define TCDMSG_CGSVRIP	41
		// change server ip command packet
		struct cgs_s
		{
			uint32 srvip;
		}cgs;

// CMD update registered clients' info
#define	TCDMSG_CMDUPREGREQ 42
#define	TCDMSG_CMDUPREGACK 43
		struct cur_s
		{
			unsigned char hwa[HWA_SZ];
			uint8 hname[FNAME_SZ]; 
			uint32 retval;
			uint32 diskcapacity;     /* disk capacity (Units: MiB) */
			uint32 totalram;         /* total ram (Units: MiB) */
			char cpumodelname[CPU_STR_SZ];   /* cpu model name */
		}cur;

// reboot packet, from server to client, specifying a client
#define TCDMSG_REBOOT_IP	44
		struct rb_ip_s{
			uint32 srvip;
			uint32 srvport;
			char ipv4[STRIP_SZ];
		}rb_ip;
// shutdown packet, from server to client, specifying a client
#define TCDMSG_SHUTDOWN_IP	45
		struct sd_ip_s{
			uint32 srvip;
			uint32 srvport;
			char ipv4[STRIP_SZ];
		}sd_ip;

// CMD reboot client with specified IP addr
#define TCDMSG_CMDREBOOT_IP 46
		struct reboot_cmd_pl{
			char ipv4[STRIP_SZ];
		}rbt_cmd_payload;

// CMD shutdown client with specified IP addr
#define TCDMSG_CMDSHUTDOWN_IP 47
		struct shutdown_cmd_pl{
			char ipv4[STRIP_SZ];
		}shd_cmd_payload;

// given client IP,
// Image ID
// User ID
// Image Name
// boot a single TC client machine
#define TCDMSG_BOOT_IP  48
		struct boot_client{
			uint32 svrip;
			uint32 svrport;
			uint32 image_id;
			char image_org_name[IMAGE_NAME_SZ];
			char image_name[IMAGE_NAME_SZ];
			int image_per;
			//uint32 user_id;
			char user_token[USER_TOKEN_SZ];
			char user_name[USER_NAME_SZ];
			uint32 clt_ip;
		}bootinfo;

#define TCDMSG_CMDBOOT_IP  49
		struct cmd_boot_client{
			uint32 image_id;
			char image_org_name[IMAGE_NAME_SZ];
			char image_name[IMAGE_NAME_SZ];
			//int image_per;
			//uint32 user_id;
			char user_token[USER_TOKEN_SZ];
			char user_name[USER_NAME_SZ];
			uint32 clt_ip;
		}cmd_bootinfo;

// remove specified os images
#define TCDMSG_RM_IMAGE 50
		struct rm_image_pl{
			uint32 srvip;
			uint32 srvport;
			uint32 image_id;
			char image_org_name[IMAGENAME_SZ];
		}rm_image_payload;

// CMD remove specified os images
#define TCDMSG_CMDRM_IMAGE 51
		struct rm_image_cmd_pl{
			uint32 image_id;
			char image_org_name[IMAGENAME_SZ];
		}rm_image_cmd_payload;

// initialize client with specified IP
#define TCDMSG_INITIALIZE_IP 52
		struct initialize_pl{
			uint32 srvip;	
			uint32 srvport;	
			uint32 sys_part_capacity;
			char ipv4[STRIP_SZ];
		}init_payload;

// CMD initialize client with specified IP
#define TCDMSG_CMDINITIALIZE_IP 53
		struct initialize_cmd_pl{
			uint32 sys_part_capacity;
			char ipv4[STRIP_SZ];
		}init_cmd_payload;

// update firmware for client with specified IP
#define TCDMSG_UPDATEFIRMWARE_IP 54
		struct updatefirmware_pl{
			uint32 srvip;	
			uint32 srvport;	
			char ipv4[STRIP_SZ];
		}uf_payload;

// CMD update firmware for client with specified IP
#define TCDMSG_CMDUPDATEFIRMWARE_IP 55
		struct updatefirmware_cmd_pl{
			char ipv4[STRIP_SZ];
		}uf_cmd_payload;
    
// CMD change-server ip for client with specified IP
#define TCDMSG_CMDCGSVRIP_IP 56
		struct ccgsip_s
		{
			uint32 srvip;
      char ipv4[STRIP_SZ]; //client ip
		}ccgsip;

// change-server for client with specified IP
#define TCDMSG_CGSVRIP_IP 57
		struct cgsip_s
		{
			uint32 srvip;
		}cgsip;

// CMD remove-image for client with specified IP
#define TCDMSG_CMDRM_IMAGE_IP 58
		struct rm_image_ip_cmd_pl{
			uint32 image_id;
			char image_org_name[IMAGENAME_SZ];
            char ipv4[STRIP_SZ];
		}rm_image_ip_cmd_payload;

// remove-image for client with specified IP
#define TCDMSG_RM_IMAGE_IP 59
		struct rm_image_ip_pl{
			uint32 image_id;
			char image_org_name[IMAGENAME_SZ];
		}rm_image_ip_payload;

// set-resolution for client with specified IP
#define TCDMSG_CMDSET_RESOLUTION_IP 60
		struct set_resolution_ip_cmd_pl{
            int screen_width;
            int screen_height;
            char ipv4[STRIP_SZ];
		}set_resolution_ip_cmd_payload;

// set-resolution for client with specified IP
#define TCDMSG_SET_RESOLUTION_IP 61
		struct set_resolution_ip_pl{
            int screen_width;
            int screen_height;
		}set_resolution_ip_payload;

// enable-usb-storage for client with specified IP
#define TCDMSG_CMDUSB_STORAGE_CTRL_IP 62
		struct usb_storage_ctrl_ip_cmd_pl{
            int usb_enabled;
            char ipv4[STRIP_SZ];
		}usb_storage_ctrl_ip_cmd_payload;

// enable-usb-storage for client with specified IP
#define TCDMSG_USB_STORAGE_CTRL_IP 63
		struct usb_storage_ctrl_ip_pl{
            int usb_enabled;
		}usb_storage_ctrl_ip_payload;

	}u;

} /*__attribute__((packed))*/ TCDMSG;


#define BUFSZ (sizeof(TCDMSG)+SLOT_BYTES)	// 52+1460


// client control block: a bunch of data which controls the behaviour of the TCD client
typedef struct ccb_struct
{
	int clt_id;	// id# of local client, which is obtained from the server's runreq command

	// global variables used by clients
	int tcdclt_debugger;

	// only used by client part
	BULKMAP clt_bulkmap;
	uint32 clt_cbulk;
	uint64 clt_totalbytes;

	// total slots to be received from server
	uint64 clt_total_slots;

// state definition at client side, used in client's FSM
#define CLT_STATE_INIT      1
#define CLT_STATE_READY     2
#define CLT_STATE_RUN       3
#define CLT_STATE_STOP      4

#define CLT_STATE_STR(x)	\
	(((x) == CLT_STATE_INIT)? "CLT_STATE_INIT":	\
	(((x) == CLT_STATE_READY)? "CLT_STATE_READY":	\
	(((x) == CLT_STATE_RUN)? "CLT_STATE_RUN":	\
	(((x) == CLT_STATE_STOP)? "CLT_STATE_STOP" : "UNKNOWN_CLT_STATE"	\
	))))

#if defined(_WIN32) || defined(TC_META_OS)
 #define SET_CLT_STATE(p, x)	\
     { \
         (p)->clt_state = (x); 	\
         wprintf(L"client: change state to [%hs]\n", CLT_STATE_STR(x)); \
     }
#else
 #define SET_CLT_STATE(p, x)	\
    { \
        (p)->clt_state = (x); 	\
        printf("client: change state to [%hs]\n", CLT_STATE_STR(x)); \
    }
#endif

	uint32 clt_state;


#define CLT_MACHINE_STATUS_STR(x)	\
	(((x) == CLT_MACHINE_STATUS_UNKNOWN)? "CLT_MACHINE_STATUS_UNKNOWN":	\
	(((x) == CLT_MACHINE_STATUS_NORMAL)? "CLT_MACHINE_STATUS_NORMAL":	\
	(((x) == CLT_MACHINE_STATUS_UNMOUNT)? "CLT_MACHINE_STATUS_UNMOUNT":	\
	(((x) == CLT_MACHINE_STATUS_NOHD)? "CLT_MACHINE_STATUS_NOHD" : 		\
	(((x) == CLT_MACHINE_STATUS_INIT_ILLEGAL_SIZE)? "CLT_MACHINE_STATUS_INIT_ILLEGAL_SIZE" : 	\
	(((x) == CLT_MACHINE_STATUS_INIT_HD_FAILURE)? "CLT_MACHINE_STATUS_INIT_HD_FAILURE" : "CLT_MACHINE_STATUS_OTHER"	\
	))))))
#define CLT_MACHINE_STATUS_UNKNOWN	0
#define CLT_MACHINE_STATUS_NORMAL	1
#define CLT_MACHINE_STATUS_UNMOUNT	2	// for unmount client, we can not deploy, but can format. 
#define CLT_MACHINE_STATUS_NOHD		3	// for nohd client, we can not do anything related with disk
#define CLT_MACHINE_STATUS_INIT_ILLEGAL_SIZE 4    // initialization script exited with value
#define CLT_MACHINE_STATUS_INIT_HD_FAILURE 5    // initialization script exited with a non-zero value except 2
#define CLT_MACHINE_STATUS_OTHER	9

	uint32 clt_machine_status;

	// if setting this tag, client will quit.
	uint32 clt_hasnofile;


	struct timeval starttv, endtv;
	struct timeval currtv, lastcurrtv;
	struct timezone tz;

	// to record the time stamp of the packet acknowledgement time. 
	struct timeval pkacktv;

	char *clt_logfile; 

	uint32 pktack_delay_base;
	uint32 pktack_delay_coeff;
	uint32 pktack_delay_intpkt;
	uint32 dyna_mapsz;


}CCB;
