﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include "stdafx.h"
#include "TcClassFactory.h"
#include "TcProvider.h"
#include "TcProviderFilter.h"

ISSUnknown_REF_IMPL(TcClassFactory)
TcClassFactory::TcClassFactory()
{
  ISSUnknown_REF_INIT;
}

TcClassFactory::~TcClassFactory()
{

}

STDMETHODIMP TcClassFactory::QueryInterface(REFIID riid, void** ppv)
{
  if(ppv == NULL)
    return E_INVALIDARG;

  *ppv = NULL;

  if((riid != IID_IClassFactory) && 
  	(riid != IID_IUnknown))
    return E_NOINTERFACE;

  *ppv = static_cast<IUnknown*>(this);
  reinterpret_cast<IUnknown*>(*ppv)->AddRef();

  return S_OK;
}

STDMETHODIMP TcClassFactory::CreateInstance(
	IUnknown *pUnkOuter, 
	REFIID riid, 
	void **ppv)
{
  HRESULT ret;
  
  if(ppv == NULL)
    return E_INVALIDARG;

  *ppv = NULL;

  if(pUnkOuter != NULL)
    return CLASS_E_NOAGGREGATION;

  ret = TcProviderCreateInstance(riid, ppv);
  if(ret == S_OK)
    return ret;

  return TcProviderFilterCreateInstance(riid, ppv);
}

STDMETHODIMP TcClassFactory::LockServer(BOOL needLock)
{
  if(needLock)
  	DllAddRef();
  else
  	DllRelease();
  
  return S_OK;
}

HRESULT TcClassFactoryCreateInstance(
	REFCLSID rclsid, 
	REFIID riid, 
	void **ppv)
{
  HRESULT ret;
  TcClassFactory *pCredProvFactory;
  if (memcmp(&rclsid, &CLSID_SSCredProvider, sizeof(GUID)))
  {
	  return CLASS_E_CLASSNOTAVAILABLE;
  }
  /*if(rclsid != CLSID_SSCredProvider)
    return CLASS_E_CLASSNOTAVAILABLE;*/

  pCredProvFactory = new TcClassFactory;
  if(pCredProvFactory == NULL)
    return E_OUTOFMEMORY;

  ret = pCredProvFactory->QueryInterface(riid, ppv);
  pCredProvFactory->Release();

  return ret;
}
