﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include "stdafx.h"

#define SECURITY_WIN32
#include <security.h>
#include <secext.h>
#include <exception>

#include <Windows.h>
#include <WinReg.h>

#include <stdio.h>
#include <stdlib.h>
#include <string>

#include "TcCredential.h"
#include "TcHelpers.h"
#include "TcProvider.h"
#include "TcDebugLib.h"
#include "TcDll.h"

#pragma comment(lib,"Secur32.lib")

#define BUFSIZE 1024

extern int G_IS_FIRST_LOGON;
using namespace std;

#if defined __cplusplus
extern "C"{
#endif

extern 
int 
__stdcall 
wloadSecFile(
	wchar_t **pwcuname, 
	wchar_t **pwcpasswd, 
	int *lockscreen);

extern
DWORD
__stdcall
TcBufMd5sum (
  unsigned char *dest,
  const unsigned char *data,
  size_t nBytes
  );

/**
 * compute sha256sum of first nBytes of a buffer
 * @return 0 on success
 * @return -1 on failure
 */
extern 
DWORD
__stdcall
TcBufSha256Sum(
	unsigned char *dest,
	const unsigned char *data,
	size_t nBytes
);

#if defined __cplusplus
}
#endif

ISSUnknown_REF_IMPL(TcCredential)
  TcCredential::TcCredential()
{
   // load username and password for TC system
  int ret;
  int index;
  int itemNum;
  wchar_t *uname, *passwd;
  
  DllAddRef();

  ISSUnknown_REF_INIT;
  m_cpus = CPUS_INVALID;
  m_dwCredUIFlags = 0;
  m_pCredentialProvider = NULL;
  m_pcpce = NULL;
  m_logon_failed = 0;
  

  //Default username/password in windows
  ::SHStrDupW(L"tc", &m_win7_userName);
  ::SHStrDupW(L"", &m_win7_password);

  uname = NULL;
  passwd = NULL;
  ret = wloadSecFile(&uname, &passwd, &m_lockscreen);
  
  if (ret != 0)
  {
    DEBUG((D_ERROR, L"fail to load data, exit code: %d\n", ret));
    //DEBUG((D_INFO, L"will use default accout\n"));

    ::SHStrDupW(m_win7_userName, &m_tc_userName);
    ::SHStrDupW(m_win7_password, &m_tc_password);
  }
  else if (m_lockscreen == 0 )
  {
    //DEBUG((D_INFO, L"do not support lock screen\n"));
    ::SHStrDupW(m_win7_userName, &m_tc_userName);
    ::SHStrDupW(m_win7_password, &m_tc_password);
  }
  else
  {
    //DEBUG((D_INFO, L"load data successfully\n"));
    
    ::SHStrDupW(uname, &m_tc_userName);
    ::SHStrDupW(passwd, &m_tc_password);    
  }

  if (uname)
  {
	  free(uname);
	  uname = NULL;
  }
  if (passwd)
  {
	  free(passwd);
	  passwd = NULL;
  }
  //::SHStrDupW(L"111111", &m_tc_password);
  //::SHStrDupW(L"Tc_Preboot_User", &m_tc_userName);
  
  itemNum = _countof(m_pwszFieldStrings);
  for(index = 0; index < itemNum; index++)
  {
    m_pwszFieldStrings[index] = NULL;
    ::SHStrDupW(L"", &m_pwszFieldStrings[index]);
  }

  //在这里设置链接和登录的图标下面显示的字符
  ::CoTaskMemFree(m_pwszFieldStrings[TC_SFI_LARGE_TEXT]);
  ::SHStrDupW(m_tc_userName, &m_pwszFieldStrings[TC_SFI_LARGE_TEXT]);
  
  ::CoTaskMemFree(m_pwszFieldStrings[TC_SFI_COMMAND_LINK]);
  ::SHStrDupW(L"Help", &m_pwszFieldStrings[TC_SFI_COMMAND_LINK]);
  
  ::CoTaskMemFree(m_pwszFieldStrings[TC_SFI_SUBMIT_BUTTON]);
  ::SHStrDupW(L"Submit", &m_pwszFieldStrings[TC_SFI_SUBMIT_BUTTON]);
}

TcCredential::~TcCredential()
{
  int index;
  int itemNum;

  itemNum = _countof(m_pwszFieldStrings);
  for(index = 0; index < itemNum; index++)
  {
    ::CoTaskMemFree(m_pwszFieldStrings[index]);
    m_pwszFieldStrings[index] = NULL;
  }
  ::CoTaskMemFree(m_win7_userName);
  ::CoTaskMemFree(m_win7_password);
  ::CoTaskMemFree(m_tc_userName);
  ::CoTaskMemFree(m_tc_password);
  DllRelease();
}

STDMETHODIMP TcCredential::QueryInterface(REFIID riid, void** ppv)
{
  if(ppv == NULL)
    return E_INVALIDARG;

  *ppv = NULL;

  //if(riid != IID_IUnknown && riid != IID_ICredentialProviderCredential)
  if (memcmp(&riid, &IID_IUnknown, sizeof(GUID)) &&
	  memcmp(&riid, &IID_ICredentialProviderCredential, sizeof(GUID)))
    return E_NOINTERFACE;

  *ppv = static_cast<IUnknown*>(this);
  reinterpret_cast<IUnknown*>(*ppv)->AddRef();

  return S_OK;
}

STDMETHODIMP TcCredential::Advise(ICredentialProviderCredentialEvents *pcpce)
{
  if(m_pcpce != NULL)
  {
    m_pcpce->Release();
    m_pcpce = NULL;
  }

  m_pcpce = pcpce;
  m_pcpce->AddRef();

  return S_OK;
}

STDMETHODIMP TcCredential::UnAdvise()
{
  if(m_pcpce != NULL)
  {
    m_pcpce->Release();
    m_pcpce = NULL;
  }

  return S_OK;
}

STDMETHODIMP TcCredential::SetSelected(BOOL *pbAutoLogon)
{
  if(m_logon_failed == 1)
  {
   //::MessageBox(::GetForegroundWindow(), TEXT("请在输入框输入Windows密码"), TEXT("TC 密码错误"), MB_ICONQUESTION|MB_OK);
   //set TC_SFI_PASSWORD to visiable
   ;//m_pcpce->SetFieldState(this,TC_SFI_PASSWORD,CPFS_DISPLAY_IN_SELECTED_TILE);
  
  }

  if(StrCmpW(L"", m_tc_password) != 0) {  
    *pbAutoLogon = FALSE;
  } else {
    *pbAutoLogon = TRUE;
  }

  return S_OK;
}
STDMETHODIMP TcCredential::SetDeselected()
{  
  //::MessageBox(::GetForegroundWindow(), TEXT("deselected"), TEXT("Confirm"), MB_ICONQUESTION|MB_YESNO);
  return S_OK;
}

STDMETHODIMP TcCredential::GetFieldState(DWORD index,
	CREDENTIAL_PROVIDER_FIELD_STATE *pcpfs,
	CREDENTIAL_PROVIDER_FIELD_INTERACTIVE_STATE *pcpfis)
{  
  if((index < TC_SFI_BEGIN) || 
  	(index >= TC_SFI_NUM_FILEDS))
  {
    return E_INVALIDARG;
  }
  
  *pcpfis = CPFIS_NONE;
  *pcpfs = CPFS_DISPLAY_IN_SELECTED_TILE;  
  
  if(index == TC_SFI_PASSWORD)
  {
    *pcpfs = CPFS_HIDDEN;
  }

  if(index == TC_SFI_PIN)
  {
    *pcpfs = CPFS_DISPLAY_IN_SELECTED_TILE;
	*pcpfis = CPFIS_FOCUSED;
  }

  if(index == TC_SFI_USERNAME)
  {
    *pcpfs = CPFS_HIDDEN;
  }

  if(index == TC_SFI_LARGE_TEXT)
  {
    *pcpfs = CPFS_DISPLAY_IN_BOTH;
  }

  if(m_cpus != CPUS_CHANGE_PASSWORD)
  {
    if(index == TC_SFI_NEWPASSWORD || 
		index == TC_SFI_NEWPASSWORD_CONFIRM)
    {
      *pcpfs = CPFS_HIDDEN;
    }
    //在这里添加验证应该也行
    
    /*if(index==TC_SFI_SUBMIT_BUTTON)
    {
      *pcpfs = CPFS_DISPLAY_IN_BOTH;
    ::MessageBox(::GetForegroundWindow(), TEXT("GetButton"), TEXT("Confirm"), MB_ICONQUESTION|MB_YESNO);
    }*/
    
  }

  return S_OK;
}

STDMETHODIMP TcCredential::GetStringValue(DWORD index, PWSTR *ppwsz)
{  
  if(index < TC_SFI_BEGIN || 
  	index >= TC_SFI_NUM_FILEDS)
  {
    return E_INVALIDARG;
  }

  ::SHStrDupW(m_pwszFieldStrings[index], ppwsz);

  return S_OK;
}

STDMETHODIMP TcCredential::GetBitmapValue(DWORD index, HBITMAP *phbmp)
{  
  if(index < TC_SFI_BEGIN || 
  	index >= TC_SFI_NUM_FILEDS)
  {
    return E_INVALIDARG;
  }

  return E_NOTIMPL;
}

STDMETHODIMP TcCredential::GetCheckboxValue(DWORD index, 
	BOOL *pbChecked, 
	PWSTR *ppwszLabel)
{
  return E_NOTIMPL;
}

STDMETHODIMP TcCredential::GetComboBoxValueCount(DWORD index, 
	DWORD *pcItems,
	DWORD *pdwSelectedItem)
{  
  return E_NOTIMPL;
}

STDMETHODIMP TcCredential::GetComboBoxValueAt(DWORD index, 
	DWORD item, 
	PWSTR *ppwszItem)
{  
  return E_NOTIMPL;
}

STDMETHODIMP TcCredential::GetSubmitButtonValue(DWORD index, 
	DWORD *pdwAdjacentTo)
{  
  if(m_cpus == CPUS_CHANGE_PASSWORD)
  {
    *pdwAdjacentTo = TC_SFI_NEWPASSWORD_CONFIRM;
  } else {
    //set submit keyboard follow the TC_SFI_PIN
    //::MessageBox(::GetForegroundWindow(), TEXT("pin"), TEXT("submit"), MB_ICONQUESTION|MB_YESNO);
    *pdwAdjacentTo = TC_SFI_PIN;
  }
  return S_OK;
}

STDMETHODIMP TcCredential::SetStringValue(DWORD index, PCWSTR pwz)
{  
  if(index < TC_SFI_BEGIN || 
  	index >= TC_SFI_NUM_FILEDS)
  {
    return E_INVALIDARG;
  }
  
  ::CoTaskMemFree(m_pwszFieldStrings[index]);
  ::SHStrDupW(pwz, &m_pwszFieldStrings[index]);
  return S_OK;
}

STDMETHODIMP TcCredential::SetCheckboxValue(DWORD index, BOOL bChecked)
{  
  return E_NOTIMPL;
}

STDMETHODIMP TcCredential::SetComboBoxSelectedValue(DWORD index,
	DWORD selItem)
{  
  return E_NOTIMPL;
}
STDMETHODIMP TcCredential::CommandLinkClicked(DWORD index)
{
  ::MessageBox(::GetForegroundWindow(),
  	TEXT("请输入TC系统的用户名和密码！"),
  	TEXT("帮助"),
  	MB_ICONQUESTION|MB_OK);
  /*
  wchar_t buf[256]={0};
  wsprintf(buf, L"%u", index); 
  ::MessageBox(::GetForegroundWindow(), buf, TEXT("Confirm"), MB_ICONQUESTION|MB_YESNO);
  */
  return E_NOTIMPL;
}

HRESULT TcCredential::CheckTcPassword()
{
	HRESULT ret = S_OK;
	//BOOL isDomainUser;
	
	TryCreateFirstLogonKey();

	//WCHAR tmpBuf[64];
	//wsprintf(tmpBuf, L"%d_%s", G_IS_FIRST_LOGON, m_win7_userName);

	//::MessageBox(::GetForegroundWindow(), tmpBuf, TEXT(" Tc GETS  "), MB_ICONQUESTION|MB_OK);

	//DEBUG((D_INFO, 
	//	L"CredInfo : %s,%s", 
	//	m_pwszFieldStrings[TC_SFI_USERNAME], 
	//	m_pwszFieldStrings[TC_SFI_PASSWORD]));

	SetStringValue(TC_SFI_USERNAME, m_win7_userName);
	SetStringValue(TC_SFI_PASSWORD, L".@#Wrong pwd$");

	//isDomainUser = IsOS(OS_DOMAINMEMBER);

	if(G_IS_FIRST_LOGON == 1) {
		m_logon_failed = 0;
		SetStringValue(TC_SFI_PASSWORD, m_win7_password);
	} else {
		PWSTR pin_psw = NULL;
		PWSTR psw = NULL;
		PWSTR u_psw = NULL;

		GetStringValue(TC_SFI_PIN, &pin_psw);    
		GetStringValue(TC_SFI_USERNAME,&u_psw);
		GetStringValue(TC_SFI_PASSWORD,&psw);

		//TODO: may need to reget and verify win 7 username and password if it failed last time

		//::MessageBox(::GetForegroundWindow(), pin_psw, TEXT(" Tc password  "), MB_ICONQUESTION|MB_YESNO);  

		//hush pin_psw
		size_t requiredSize;
		size_t sz;
		unsigned char *mbpin_psw;
		wchar_t *newpin_psw;
		unsigned char buf1[128];
		int flag;
		int flag2;
		int tmp;

		flag = 0;
		flag2 = 0;
		mbpin_psw = NULL;
		newpin_psw = NULL;

		if (wcslen(pin_psw) > 0)
		{
		  flag = 1;
		  //DEBUG((D_INFO, L"flag: %d\n", flag));

		  requiredSize = wcstombs(NULL, pin_psw, 0);
		  mbpin_psw = (unsigned char *) calloc(requiredSize + 1, sizeof(char));
		  if (mbpin_psw == NULL)
		  {
		    flag = 0;
		    DEBUG((D_ERROR, L"memory allocation failure\n"));
		  }
		  else
		  {
		    sz = wcstombs((char *)mbpin_psw, pin_psw, requiredSize + 1);
		    if (sz == -1)
		    {
		      flag = 0;
		      DEBUG((D_ERROR, L"fail to convert wcstring\n"));
		    }
		  }

		  if (flag)
		  {
		    tmp = TcBufSha256Sum(buf1, mbpin_psw, strlen((char *)mbpin_psw));
		    if (tmp != 0)
		    {
		      flag = 0;
		      DEBUG((D_ERROR, L"fail to process %hs\n", mbpin_psw));
		    }
		    //else
		    //{
		      //DEBUG((D_INFO, L"buf1: %hs\n", buf1));
		    //}
		  }

		  if (flag)
		  {
		    requiredSize = mbstowcs(NULL, (char *)buf1, 0);
		    newpin_psw = (wchar_t *)calloc(requiredSize + 1, sizeof(wchar_t));
		    if (newpin_psw == NULL)
		    {
		      flag = 0;
		      DEBUG((D_ERROR, L"memory allocation failure\n"));
		    }
		  }

		  if (flag)
		  {
		    sz = mbstowcs(newpin_psw, (char *)buf1, requiredSize + 1);
		    if (sz == -1)
		    {
		      flag = 0;
		      DEBUG((D_ERROR, L"fail to convert mbstring\n"));
		    }
		    else
		    {
		      //wprintf(L"newpin_psw: %s\n", newpin_psw);
		    }
		  }
		}
		else{
		  flag2 = 1;
		  //DEBUG((D_INFO, L"flag2: %d\n", flag2));

		  newpin_psw = (wchar_t *)calloc(0 + 1, sizeof(wchar_t));
		  if (newpin_psw == NULL)
		  {
		    flag2 = 0;
		    DEBUG((D_ERROR, L"memory allocation failure\n"));
		  }

		  if (flag2)
		  {
		    wcscpy_s(newpin_psw, wcslen(L""), L"");
		  }
		}

		//DEBUG((D_INFO, L"flag: %d\n", flag));
		//DEBUG((D_INFO, L"flag2: %d\n", flag2));

		if (flag == 1 || flag2 == 1)
		{
		  if(StrCmpW(newpin_psw, m_tc_password) != 0) {
		  //if(StrCmpW(pin_psw, m_tc_password) != 0) {
		    m_logon_failed = 1;
		  } else {
		    m_logon_failed = 0;
		    SetStringValue(TC_SFI_PASSWORD, m_win7_password);
		  }
		}
		else {
		  //if(StrCmpW(newpin_psw, m_tc_password) != 0) {
		  if(StrCmpW(pin_psw, m_tc_password) != 0) {
		    m_logon_failed = 1;
		  } else {
		    m_logon_failed = 0;
		    SetStringValue(TC_SFI_PASSWORD, m_win7_password);
		  }
		}

		if (mbpin_psw)
		{
		  free(mbpin_psw);
		  mbpin_psw = NULL;
		}
		if (newpin_psw)
		{
		  free(newpin_psw);
		  newpin_psw= NULL;
		}
		CoTaskMemFree(pin_psw);
		CoTaskMemFree(u_psw);
		CoTaskMemFree(psw);
		
	}
	return ret;
}

STDMETHODIMP TcCredential::GetSerialization(
	CREDENTIAL_PROVIDER_GET_SERIALIZATION_RESPONSE *pcpgsr,
    CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION *pcpcs,
    PWSTR *ppwszOptionalStatusText, 
    CREDENTIAL_PROVIDER_STATUS_ICON *pcpsi)
{
  
  LPWSTR lpPasswordEncrypted = NULL;
  LPWSTR lpPasswordNewEncrypted = NULL;
  LPWSTR lpDomainUsername = NULL;

  BYTE *pRgb = NULL;
  DWORD cbRgb = 0;

  CheckTcPassword();
  
  /*
  PWSTR u_psw = NULL;
        
  GetStringValue(TC_SFI_USERNAME,&u_psw);
  ::MessageBox(::GetForegroundWindow(), u_psw, TEXT(" Win 7 usename"), MB_ICONQUESTION|MB_YESNO);
  CoTaskMemFree(u_psw);
  
    
  PWSTR psw = NULL;
  GetStringValue(TC_SFI_PASSWORD,&psw);
  ::MessageBox(::GetForegroundWindow(), psw, TEXT(" Win 7 password "), MB_ICONQUESTION|MB_YESNO);
  CoTaskMemFree(psw);
  */  
        
      
    //下面的是系统的，不用管

  check_HRESULT_begin
  {
    if(!m_CredentialInfos.fCredentialPrepared)
    {
      if(wcslen(m_pwszFieldStrings[TC_SFI_USERNAME]) > 0 && wcslen(m_pwszFieldStrings[TC_SFI_PASSWORD]) > 0)
      {
        check_HRESULT(TcSpliterDomainAndUsername(m_pwszFieldStrings[TC_SFI_USERNAME]
        , m_CredentialInfos.szDomain, SS_DIMOF(m_CredentialInfos.szDomain)
          , m_CredentialInfos.szUsername, SS_DIMOF(m_CredentialInfos.szUsername)));

        _tcsncpy_s(m_CredentialInfos.szPassword, ARRAYSIZE(m_CredentialInfos.szPassword),
			m_pwszFieldStrings[TC_SFI_PASSWORD], ARRAYSIZE(m_CredentialInfos.szPassword)-1);
        m_CredentialInfos.fCredentialPrepared = TRUE;
      }
    }

    SS_LOG_V(LL_TRACE, _T("CredInfoDetail : %s\\%s,%s,%d"), m_CredentialInfos.szDomain, m_CredentialInfos.szUsername
      , m_CredentialInfos.szPassword, m_CredentialInfos.fCredentialPrepared);

    check_HRESULT_bool(m_CredentialInfos.fCredentialPrepared, E_FAIL);

    DWORD cchProcted = 0;
    check_HRESULT(TcCopyProtectedStringIfNecessaryW(m_CredentialInfos.szPassword, m_cpus, NULL, &cchProcted));


    lpPasswordEncrypted = (LPWSTR)CoTaskMemAlloc(cchProcted * sizeof(WCHAR));
    check_HRESULT_bool(lpPasswordEncrypted != NULL, E_FAIL);
    check_HRESULT(TcCopyProtectedStringIfNecessaryW(m_CredentialInfos.szPassword, m_cpus, lpPasswordEncrypted, &cchProcted));


    if(m_cpus == CPUS_CHANGE_PASSWORD)
    {
      check_HRESULT_bool(wcslen(m_pwszFieldStrings[TC_SFI_NEWPASSWORD]) == wcslen(m_pwszFieldStrings[TC_SFI_NEWPASSWORD_CONFIRM])
        , E_FAIL);
      check_HRESULT_bool(wcscmp(m_pwszFieldStrings[TC_SFI_NEWPASSWORD], m_pwszFieldStrings[TC_SFI_NEWPASSWORD_CONFIRM]) == 0
        , E_FAIL);
      check_HRESULT(TcCopyProtectedStringIfNecessaryW(m_pwszFieldStrings[TC_SFI_NEWPASSWORD], m_cpus, NULL, &cchProcted));
      lpPasswordNewEncrypted = (LPWSTR)CoTaskMemAlloc(cchProcted * sizeof(WCHAR));
      check_HRESULT_bool(lpPasswordNewEncrypted != NULL, E_FAIL);
      check_HRESULT(TcCopyProtectedStringIfNecessaryW(m_pwszFieldStrings[TC_SFI_NEWPASSWORD], m_cpus, lpPasswordNewEncrypted, &cchProcted));

      KERB_CHANGEPASSWORD_REQUEST objKcr;
      check_HRESULT(TcKerbChangePasswordRequestInitW(m_CredentialInfos.szDomain, m_CredentialInfos.szUsername, lpPasswordEncrypted
        , lpPasswordNewEncrypted, &objKcr));
      check_HRESULT(TcKerbChangePasswordRequestPack(objKcr, &pcpcs->rgbSerialization, &pcpcs->cbSerialization));
    }else if(m_cpus == CPUS_CREDUI)
    {
      size_t cchLenDomain = 0, cchLenUser = 0,  cchDomainUser;
      check_HRESULT(StringCchLengthW(m_CredentialInfos.szDomain, STRSAFE_MAX_CCH, &cchLenDomain));
      check_HRESULT(StringCchLengthW(m_CredentialInfos.szUsername, STRSAFE_MAX_CCH, &cchLenUser));
      cchDomainUser = cchLenDomain + 1 + cchLenUser + 1;
      lpDomainUsername = (LPWSTR)CoTaskMemAlloc(cchDomainUser * sizeof(WCHAR));
      check_HRESULT_bool(lpDomainUsername != NULL, E_FAIL);
      swprintf_s(lpDomainUsername, cchDomainUser, L"%s\\%s", m_CredentialInfos.szDomain, m_CredentialInfos.szUsername);

      check_HRESULT_bool(!::CredPackAuthenticationBufferW((m_dwCredUIFlags & CREDUIWIN_PACK_32_WOW) ? CRED_PACK_WOW_BUFFER : 0
        , lpDomainUsername, lpPasswordEncrypted, pRgb, &cbRgb) && ::GetLastError() == ERROR_INSUFFICIENT_BUFFER, E_FAIL);
      pRgb = (BYTE*)HeapAlloc(::GetProcessHeap(), 0, cbRgb);
      check_HRESULT_bool(pRgb != NULL, E_FAIL);
      check_HRESULT_bool(::CredPackAuthenticationBufferW((m_dwCredUIFlags & CREDUIWIN_PACK_32_WOW) ? CRED_PACK_WOW_BUFFER : 0
        , lpDomainUsername, lpPasswordEncrypted, pRgb, &cbRgb), E_FAIL);
    }else
    {

      KERB_INTERACTIVE_UNLOCK_LOGON objKIUL;
      check_HRESULT(TcKerbInteractiveUnlockLogonInitW(m_CredentialInfos.szDomain
        , m_CredentialInfos.szUsername, lpPasswordEncrypted, m_cpus
        , &objKIUL));

      check_HRESULT(TcKerbInteractiveUnlockLogonPack(objKIUL, &pcpcs->rgbSerialization, &pcpcs->cbSerialization));
    }


    ULONG ulAuthPackage = 0;
    check_HRESULT(TcRetrieveNegotiateAuthPackage(&ulAuthPackage));

    pcpcs->ulAuthenticationPackage = ulAuthPackage;
    pcpcs->clsidCredentialProvider = CLSID_SSCredProvider;

    if(m_cpus == CPUS_CREDUI)
    {
      pcpcs->rgbSerialization = pRgb;
      pcpcs->cbSerialization = cbRgb;
      pRgb = NULL;
      cbRgb = 0;
    }

    *pcpgsr = CPGSR_RETURN_CREDENTIAL_FINISHED;
  }
  check_HRESULT_finally
  {

  }

  if(pRgb != NULL)
  {
    ::HeapFree(::GetProcessHeap(), 0, pRgb);
    pRgb = NULL;
  }

  if(lpPasswordNewEncrypted != NULL)
  {
    CoTaskMemFree(lpPasswordNewEncrypted);
    lpPasswordNewEncrypted = NULL;
  }

  if(lpPasswordEncrypted != NULL)
  {
    CoTaskMemFree(lpPasswordEncrypted);
    lpPasswordEncrypted = NULL;
  }

  m_CredentialInfos.Clear();

  check_HRESULT_return;
}

HRESULT TcCredential::Initialize(CREDENTIAL_PROVIDER_USAGE_SCENARIO cpus,
	DWORD cpusFlags, 
	ICredentialProvider *pCredentialProvider)
{
  m_cpus = cpus;
  m_dwCredUIFlags = cpusFlags;
  m_pCredentialProvider = pCredentialProvider;

  return S_OK;
}

STDMETHODIMP TcCredential::ReportResult(NTSTATUS status,
	NTSTATUS subStatus, 
	PWSTR *ppwszOptionalStatusText,
   	CREDENTIAL_PROVIDER_STATUS_ICON *pcpsi)
{
  if(status != S_OK)
  {
    m_logon_failed = 1;
  }

  return E_NOTIMPL;
}

