﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#ifndef _TC_CREDENTIAL_H_
#define _TC_CREDENTIAL_H_

#include "TcCredentialInterface.h"

enum TC_CRED_FIELD_ID
{
  TC_SFI_BEGIN				  = 0,
  TC_SFI_TITLEIMAGE           = 0,
  TC_SFI_PASSWORD             = 1,
  TC_SFI_PIN                  = 2,
  TC_SFI_USERNAME             = 3,
  TC_SFI_NEWPASSWORD          = 4,
  TC_SFI_NEWPASSWORD_CONFIRM  = 5,
  TC_SFI_LARGE_TEXT           = 6,
  TC_SFI_SMALL_TEXT           = 7,
  TC_SFI_SUBMIT_BUTTON        = 8,
  TC_SFI_COMMAND_LINK         = 9,  
  TC_SFI_NUM_FILEDS           = 10
};

class TcCredential : public TcCredentialProviderCredential
{
public:
  TcCredential();
  ~TcCredential();

public:
  
  ISSUnknown_REF_DEFINE;

  STDMETHOD(Advise)(ICredentialProviderCredentialEvents *pcpce);
  STDMETHOD(UnAdvise)();

  STDMETHOD(SetSelected)(BOOL *pbAutoLogon);
  STDMETHOD(SetDeselected)();

  STDMETHOD(GetFieldState)(DWORD index, 
  	CREDENTIAL_PROVIDER_FIELD_STATE *pcpfs,
    CREDENTIAL_PROVIDER_FIELD_INTERACTIVE_STATE *pcpfis);

  STDMETHOD(GetStringValue)(DWORD index, PWSTR *ppwsz);
  STDMETHOD(GetBitmapValue)(DWORD index, HBITMAP *phbmp);
  STDMETHOD(GetCheckboxValue)(DWORD index, 
  	BOOL *pbChecked, 
  	PWSTR *ppwszLabel);
  STDMETHOD(GetComboBoxValueCount)(DWORD index, 
  	DWORD *pcItems,
  	DWORD *pdwSelectedItem);
  STDMETHOD(GetComboBoxValueAt)(DWORD index, 
  	DWORD item, 
  	PWSTR *ppwszItem);
  STDMETHOD(GetSubmitButtonValue)(DWORD index, 
  	DWORD *pdwAdjacentTo);

  STDMETHOD(SetStringValue)(DWORD index, PCWSTR pwz);
  STDMETHOD(SetCheckboxValue)(DWORD index, BOOL bChecked);
  STDMETHOD(SetComboBoxSelectedValue)(DWORD index, DWORD selItem);

  STDMETHOD(CommandLinkClicked)(DWORD index);

  STDMETHOD(GetSerialization)(
  	CREDENTIAL_PROVIDER_GET_SERIALIZATION_RESPONSE *pcpgsr,
    CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION *pcpcs,
    PWSTR *ppwszOptionalStatusText, 
    CREDENTIAL_PROVIDER_STATUS_ICON *pcpsi);
  STDMETHOD(ReportResult)(NTSTATUS status, 
  	NTSTATUS subStatus, 
  	PWSTR *ppwszOptionalStatusText,
    CREDENTIAL_PROVIDER_STATUS_ICON *pcpsi);

public:
  virtual HRESULT Initialize(CREDENTIAL_PROVIDER_USAGE_SCENARIO cpus, 
  	DWORD cpusFlags, 
  	ICredentialProvider *pCredentialProvider);

private:
  HRESULT TcCredential::CheckTcPassword();
  
  CREDENTIAL_PROVIDER_USAGE_SCENARIO m_cpus;
  DWORD m_dwCredUIFlags;
  ICredentialProvider *m_pCredentialProvider;

  ICredentialProviderCredentialEvents *m_pcpce;

  LPWSTR m_pwszFieldStrings[TC_SFI_NUM_FILEDS];

  SSWACredProvCredentialInfos m_CredentialInfos;
  LPWSTR m_win7_userName;
  LPWSTR m_win7_password;
  LPWSTR m_tc_userName;
  LPWSTR m_tc_password;
  int m_logon_failed;
  int m_lockscreen;
};

#endif