﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include "stdafx.h"
#include "TcDebugLib.h"
#include "TcClassFactory.h"
#include "TcDll.h"

#ifdef _X86_
extern "C" { int _afxForceUSRDLL; }
#else
extern "C" { int __afxForceUSRDLL; }
#endif

#pragma data_seg("tc_data")
int G_IS_FIRST_LOGON = 1;
HINSTANCE G_HINST = NULL; // global dll hinstance
#pragma data_seg()
//#pragma comment(linker, "/SECTION:tc_data,rws")

static LONG g_cRef = 0;
extern "C" int __stdcall wloadSecFile(
	wchar_t **pwcuname,
	wchar_t **pwcpasswd,
	int *lockscreen);

VOID DllAddRef()
{
  InterlockedIncrement(&g_cRef);
}

VOID DllRelease()
{
  InterlockedDecrement(&g_cRef);
}

STDAPI DllCanUnloadNow()
{
  return g_cRef > 0 ? S_FALSE : S_OK;
}

/**
 check if the sec file exist and legal
 return:
 0 on success
 non-zero value otherwise
**/
BOOL __stdcall IsSecFileLegal()
{
	int ret;
	int lockscreen;
	wchar_t *uname;
	wchar_t *passwd;

	uname = NULL;
	passwd = NULL;

	ret = wloadSecFile(&uname, &passwd, &lockscreen);

	if (uname)
	{
		free(uname);
		uname = NULL;
	}
	if (passwd)
	{
		free(passwd);
		passwd = NULL;
	}
	return (ret == 0) ? TRUE : FALSE;
}

STDAPI_(BOOL) DllMain(__in HINSTANCE hinstDll, __in DWORD dwReason, __in void *)
{
	BOOL ret = TRUE;

	DEBUG_INIT(TO_CONSOLE, NULL);

    switch (dwReason)
    {
    case DLL_PROCESS_ATTACH:
        DisableThreadLibraryCalls(hinstDll);
		
		//DEBUG((D_INFO, L"checking domain\n"));
		// The dll should refuse to load if it's in domain
		if(IsOS(OS_DOMAINMEMBER)) {
			//DEBUG((D_INFO, L"Domain detected, will quit\n"));
			ret = FALSE;
		}
		
        break;
    case DLL_PROCESS_DETACH:
    case DLL_THREAD_ATTACH:
    case DLL_THREAD_DETACH:
        break;
    }

    G_HINST = hinstDll;

	//
	// is sec file does not exist or not legal, quit the dll
	//
	if (!IsSecFileLegal()) {
		ret = FALSE;
		return ret;
	}

    return ret;
}

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID FAR* ppv)
{
  //DEBUG_INIT(TO_CONSOLE, NULL);
  return TcClassFactoryCreateInstance(rclsid, riid, ppv);
}
