﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include "stdafx.h"
#include "TcProvider.h"
#include "TcCredential.h"

extern int G_IS_FIRST_LOGON;
#define TC_FIRST_LOGON_KEY  L".tc.logon."

static const CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR G_FIELDDESCS[] =
{
  { TC_SFI_TITLEIMAGE, CPFT_TILE_IMAGE, L"TitleImg" },
  { TC_SFI_PASSWORD, CPFT_PASSWORD_TEXT, L"Password" },
  { TC_SFI_PIN, CPFT_PASSWORD_TEXT, L"Tc Password" },
  { TC_SFI_USERNAME, CPFT_EDIT_TEXT, L"Username" },
  { TC_SFI_NEWPASSWORD, CPFT_PASSWORD_TEXT, L"New Password" },
  { TC_SFI_NEWPASSWORD_CONFIRM, CPFT_PASSWORD_TEXT, L"Confirm New Password" },
  { TC_SFI_LARGE_TEXT, CPFT_LARGE_TEXT, L"Large Text" },
  { TC_SFI_SMALL_TEXT, CPFT_SMALL_TEXT, L"Small Text" },
  { TC_SFI_SUBMIT_BUTTON, CPFT_SUBMIT_BUTTON, L"Submit" },
  { TC_SFI_COMMAND_LINK, CPFT_COMMAND_LINK, L"Command_link" },
};

HRESULT TcProviderCreateInstance(REFIID riid, void **ppv)
{
  HRESULT ret;
  TcProvider *pTcProvider;

  if(ppv == NULL)
    return E_INVALIDARG;

  *ppv = NULL;
  pTcProvider = new TcProvider;
  if(pTcProvider == NULL)
    return E_OUTOFMEMORY;

  ret = pTcProvider->QueryInterface(riid, ppv);
  pTcProvider->Release();

  return ret;
}

int TryCreateFirstLogonKey()
{
  HRESULT ret;
  HKEY logonKey = 0;
  DWORD disposition = 0;

  ret = RegCreateKeyExW(HKEY_CLASSES_ROOT, 
      TC_FIRST_LOGON_KEY,
      0,
      NULL,
      REG_OPTION_VOLATILE,
      KEY_ALL_ACCESS,
      NULL,
      &logonKey,
      &disposition);

  if(ret == ERROR_SUCCESS && 
    disposition == REG_CREATED_NEW_KEY) {
    G_IS_FIRST_LOGON = 1;
  } else {
    G_IS_FIRST_LOGON = 0;
  }
  if(logonKey != 0) {
    RegCloseKey(logonKey);
    logonKey = 0;
  }

  //WCHAR tmpBuf[32];
  //wsprintf(tmpBuf, L"%d", G_IS_FIRST_LOGON);
  //::MessageBox(::GetForegroundWindow(), tmpBuf, TEXT(" Tc  "), MB_ICONQUESTION|MB_OK);

  return G_IS_FIRST_LOGON;
}

int CheckFirstLogon()
{
  HRESULT ret;
  HKEY logonKey = 0;

  ret = RegOpenKeyExW(HKEY_CLASSES_ROOT, 
      TC_FIRST_LOGON_KEY,
      0, 
      KEY_READ, 
      &logonKey);

  if(ret != ERROR_SUCCESS) {
    G_IS_FIRST_LOGON = 1;
  } else {
    G_IS_FIRST_LOGON = 0;
  }
  if(logonKey != 0) {
    RegCloseKey(logonKey);
    logonKey = 0;
  }
  //RegCloseKey(logonKey);
  //logonKey = 0;
  //WCHAR tmpBuf[32];
  //wsprintf(tmpBuf, L"%d", G_IS_FIRST_LOGON);
  //::MessageBox(::GetForegroundWindow(), tmpBuf, TEXT(" Tc check  "), MB_ICONQUESTION|MB_OK);

  return G_IS_FIRST_LOGON;
}

ISSUnknown_REF_IMPL(TcProvider)
TcProvider::TcProvider()
{
  DllAddRef();

  ISSUnknown_REF_INIT;

  m_dwCredentials = 0;
  m_pCredentials[m_dwCredentials++] = new TcCredential();

  m_pcpe = NULL;
  m_uAdviseContext = 0;

  if(CheckFirstLogon() == 1) {
    m_dwAutoLogonCredentialIndex = 0;
  } else {
    m_dwAutoLogonCredentialIndex = MAXDWORD;
  }
}

TcProvider::~TcProvider()
{
  

  while(m_dwCredentials-- > 0)
  {
    if(m_pCredentials[m_dwCredentials] != NULL)
    {
      m_pCredentials[m_dwCredentials]->Release();
      m_pCredentials[m_dwCredentials] = 0;
    }
  }
  m_dwCredentials = 0;

  DllRelease();
}

STDMETHODIMP TcProvider::QueryInterface(REFIID riid, void** ppv)
{
  if(ppv == NULL)
    return E_INVALIDARG;

  *ppv = NULL;

  //if(riid != IID_IUnknown && riid != IID_ICredentialProvider)
  if (memcmp(&riid, &IID_IUnknown, sizeof(GUID)) &&
	  memcmp(&riid, &IID_ICredentialProvider, sizeof(GUID)))
    return E_NOINTERFACE;

  *ppv = static_cast<IUnknown*>(this);
  reinterpret_cast<IUnknown*>(*ppv)->AddRef();

  return S_OK;
}

STDMETHODIMP TcProvider::SetUsageScenario(
	CREDENTIAL_PROVIDER_USAGE_SCENARIO cpus, 
	DWORD flags)
{
  HRESULT ret = E_FAIL;

  switch (cpus)
  {
  case CPUS_UNLOCK_WORKSTATION:
  case CPUS_LOGON:
  case CPUS_CREDUI:
  case CPUS_CHANGE_PASSWORD:
    for(DWORD dwIdx = 0; dwIdx<m_dwCredentials; dwIdx++)
    {
      if(m_pCredentials[dwIdx] != NULL)
      {
        m_pCredentials[dwIdx]->Initialize(cpus, flags, this);
      }
    }
    ret = S_OK;
    break;
  case CPUS_PLAP:
  default:
    ret = E_NOTIMPL;
    break;
  }

  return ret;
}

STDMETHODIMP TcProvider::SetSerialization(
	CONST CREDENTIAL_PROVIDER_CREDENTIAL_SERIALIZATION*)
{  
  return E_INVALIDARG;
}

STDMETHODIMP TcProvider::Advise(ICredentialProviderEvents *pcpe, 
	UINT_PTR advContext)
{
  
  if(m_pcpe != NULL)
  {
    m_pcpe->Release();
    m_pcpe = NULL;
  }

  m_pcpe = pcpe;
  m_pcpe->AddRef();

  m_uAdviseContext = advContext;

  return S_OK;
}

STDMETHODIMP TcProvider::UnAdvise()
{  
  if(m_pcpe != NULL)
  {
    m_pcpe->Release();
    m_pcpe = NULL;
  }
  m_uAdviseContext = 0;

  return S_OK;
}

STDMETHODIMP TcProvider::GetFieldDescriptorCount(DWORD *pdwCount)
{
  *pdwCount = TC_SFI_NUM_FILEDS;

  return S_OK;
}

STDMETHODIMP TcProvider::GetFieldDescriptorAt(DWORD index, 
	CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR **ppcpfd)
{
  HRESULT ret = E_FAIL;
  CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR *pcpfd;
  CONST CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR *pcpfdSrc;
  
  if(index >= TC_SFI_NUM_FILEDS || 
  	ppcpfd == NULL)
  {
    return E_INVALIDARG;
  }

  *ppcpfd = NULL;

  pcpfd = (CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR*)CoTaskMemAlloc(
    		sizeof(CREDENTIAL_PROVIDER_FIELD_DESCRIPTOR));
  if(pcpfd == NULL)
  {
    return E_OUTOFMEMORY;
  }

  pcpfdSrc = &G_FIELDDESCS[index];

  pcpfd->dwFieldID = pcpfdSrc->dwFieldID;
  pcpfd->cpft = pcpfdSrc->cpft;
  if(pcpfdSrc->pszLabel != NULL)
  {
    ret = ::SHStrDupW(pcpfdSrc->pszLabel, &pcpfd->pszLabel);
  }else
  {
    pcpfd->pszLabel = NULL;
    ret = S_OK;
  }

  if(SUCCEEDED(ret))
  {
    *ppcpfd = pcpfd;
    pcpfd = NULL;
  }else
  {
    CoTaskMemFree(pcpfd);
    pcpfd = NULL;
  }

  return ret;
}

STDMETHODIMP TcProvider::GetCredentialCount(DWORD *pdwCount,
	DWORD *pdwDefault, 
	BOOL *pfAutoLogonWithDefault)
{  
  if(pdwCount != NULL)
  {
    *pdwCount = m_dwCredentials;
  }

  if(m_dwAutoLogonCredentialIndex == MAXDWORD)
  {
    *pdwDefault = MAXDWORD;
    *pfAutoLogonWithDefault = FALSE;
    //::MessageBox(::GetForegroundWindow(), L"FALSE", TEXT(" Auto  "), MB_ICONQUESTION|MB_YESNO);  
  }else
  {
    *pdwDefault = m_dwAutoLogonCredentialIndex;
    *pfAutoLogonWithDefault = TRUE;
    //::MessageBox(::GetForegroundWindow(),  L"TRUE", TEXT(" Auto  "), MB_ICONQUESTION|MB_YESNO);  
  }

  m_dwAutoLogonCredentialIndex = MAXDWORD;

  return S_OK;
}

STDMETHODIMP TcProvider::GetCredentialAt(DWORD index, 
	ICredentialProviderCredential **ppcpc)
{
  if(index >= m_dwCredentials || ppcpc == NULL)
    return E_INVALIDARG;

  if(m_pCredentials[index] == NULL)
    return E_POINTER;

  return m_pCredentials[index]->QueryInterface(IID_ICredentialProviderCredential, 
  			reinterpret_cast<void**>(ppcpc));
}

HRESULT TcProvider::ChangeCredentials(
	ICredentialProviderCredential *pCredential)
{  
  if(pCredential == NULL)
  {
    return E_INVALIDARG;
  }
  if(m_dwAutoLogonCredentialIndex != MAXDWORD)
  {
    return STG_E_INUSE;
  }

  if(m_pcpe == NULL)
  {
   return E_FAIL;
  }

  m_dwAutoLogonCredentialIndex = MAXDWORD;

  if(pCredential != NULL)
  {
    DWORD ind = 0;
    for (ind=0; ind<m_dwCredentials; ind++)
    {
      if(m_pCredentials[ind] == pCredential)
      {
        m_dwAutoLogonCredentialIndex =ind;
      }
    }
  }
  //ÇÐ»»ÓÃ»§
  return m_pcpe->CredentialsChanged(m_uAdviseContext);
}
