﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
/** @file

**/

#include "TcDebugLib.h"
#include "TcKeyValueLib.h"
#include "TcNtLib.h"

#if defined __cplusplus
extern "C"{
#endif

extern int __stdcall wloadSecFile(wchar_t **pwcuname, wchar_t **pwcpasswd, int *lockscreen);

#if defined __cplusplus
}
#endif

#define TC_SEC_FILE    "\\Device\\Harddisk0\\Partition1\\.tc.passwd"
#define TC_UNAME_LEN   255
#define TC_PASS_LEN    255

// on success, return 0;
// otherwise, return a nonzero value
int __stdcall loadSecFile(char *uname, int unamesz, char *passwd, int passwdsz, int *lockscreen)
{
	int ret;
	int rdlen;
	int nameflag;
	int passflag;
	int lockflag;
	int n;
	struct config_data *pcd, *pcursor, *qcursor;
	char buf[1024];
	const char *filepath = TC_SEC_FILE;
	
	//DEBUG((D_INFO, L"%hs", __FUNCTION__));

	if (uname == NULL || passwd == NULL || 
		lockscreen == NULL || unamesz < 1 || 
		passwdsz < 1)
	{
		DEBUG((D_ERROR, L"illegal arguments.\n"));
		return 1;
	}

	ret = 0;
	nameflag = 0;
	passflag = 0;
	lockflag = 0;

	rdlen = TcOneTimeNtReadFile((unsigned char *)buf, sizeof(buf), filepath);
	
	if (rdlen < 0)
	{
		DEBUG((D_ERROR, L"fail to load data, code = %d\n", rdlen));
		return 2;
	}
	else if (rdlen == 0)
	{
		DEBUG((D_ERROR, L"Data is empty\n"));
		return 3;
	}
	else
	{
		DEBUG((D_INFO, L"data was loaded successsfully\n"));
		//DEBUG((D_INFO, L"buf[%d] = \n", rdlen));
		//DEBUG((D_INFO, L"%hs\n", buf));
	}

	n = sget_key_val(buf, rdlen, "=", &pcd);
	if (n <= 0)
	{
		DEBUG((D_ERROR, L"Failed to process data\n"));
		return 4;
	}

	pcursor = pcd;
	while (pcursor != NULL)
	{
		if (!strcmp("username", pcursor->key))
		{
			//DEBUG((D_INFO, L"username[%d]: %hs\n", strlen(pcursor->value), pcursor->value));
			if (!nameflag)
			{
				strncpy(uname, pcursor->value, unamesz);
				uname[unamesz - 1] = '\0';
				nameflag = 1;
			}
		}
		else if (!strcmp("password", pcursor->key))
		{
			//DEBUG((D_INFO, L"password[%d]: %hs\n", strlen(pcursor->value), pcursor->value));
			if (!passflag)
			{
				strncpy(passwd, pcursor->value, passwdsz);
				passwd[passwdsz - 1] = '\0';
				passflag = 1;
			}
		}
		else if (!strcmp("lockscreen", pcursor->key))
		{
			//DEBUG((D_INFO, L"lockscreen[%d]: %hs\n", strlen(pcursor->value), pcursor->value));
			if (!lockflag)
			{
				if (!strncmp("yes", pcursor->value, 3))
				{
					*lockscreen = 1;
				}
				else
				{
					*lockscreen = 0;
				}
				lockflag = 1;
			}
		}
		else
		{
			DEBUG((D_ERROR, L"unknown: (%hs,%hs)\n", 
				pcursor->key, 
				pcursor->value));
		}

		if (nameflag && passflag && lockflag)
		{
			break;
		}
		
		pcursor = pcursor->next;
	}

	pcursor = pcd;
	while (pcursor != NULL)
	{
		qcursor = pcursor->next;
		free (pcursor);
		pcursor = qcursor;
	}

	if (!nameflag || !passflag || !lockflag)
	{
		ret = 5;
	}

    if (ret == 0)
    {
        DEBUG((D_INFO, L"successfully loaded the file.\n"));
    }
    else
    {
        DEBUG((D_ERROR, L"failed to load the file.\n"));
    }

	return ret;
}

// on success, return 0, otherwise return a non-zero value
// the caller should make sure to free the buffer allocated for the passed two pointers.
int __stdcall wloadSecFile(
	wchar_t **pwcuname, 
	wchar_t **pwcpasswd, 
	int *lockscreen)
{
	int t;
	char uname[TC_UNAME_LEN + 1];
	char passwd[TC_PASS_LEN + 1];
	size_t requiredSize;
	size_t sz;

	//DEBUG((D_INFO, L"%hs", __FUNCTION__));

	if (pwcuname == NULL || pwcpasswd == NULL)
	{
		DEBUG((D_ERROR, L"illegal arguments.\n"));
		return 1;
	}

	*pwcuname = NULL;
	*pwcpasswd = NULL;
	*lockscreen = 0;

	t = loadSecFile(uname, sizeof uname, passwd, sizeof passwd, lockscreen);

	if (t != 0)
	{
		DEBUG((D_ERROR, L"%hs returned with %d.\n", "loadSecFile()", t));
		return 2;
	}
	//DEBUG((D_INFO, L"uname = %hs, passwd = %hs, lockscreen = %d\n", uname, passwd, *lockscreen));

	requiredSize = mbstowcs(NULL, uname, 0);
	//DEBUG((D_INFO, L"requiredSize = %d\n", requiredSize));
	*pwcuname = (wchar_t *) calloc(requiredSize + 1, sizeof (wchar_t));
	if (*pwcuname == NULL)
	{
		DEBUG((D_ERROR, L"memory allocation failure\n"));
		return 3;
	}

	sz = mbstowcs(*pwcuname, uname, requiredSize + 1);
	if (sz == -1)
	{
		DEBUG((D_ERROR, L"cannot convet string -- invalid multibyte character.\n"));
		return 4;
	}
	//DEBUG((D_INFO, L"*pwcuname = %ls\n", *pwcuname));

	requiredSize = mbstowcs(NULL, passwd, 0);
	//DEBUG((D_INFO, L"requiredSize = %d\n", requiredSize));
	*pwcpasswd = (wchar_t *) calloc(requiredSize + 1, sizeof (wchar_t));
	if (*pwcpasswd == NULL)
	{
		DEBUG((D_ERROR, L"memory allocation failure\n"));
		return 5;
	}

	sz = mbstowcs(*pwcpasswd, passwd, requiredSize + 1);
	if (sz == -1)
	{
		DEBUG((D_ERROR, L"cannot convet string -- invalid multibyte character.\n"));
		return 6;
	}
	//DEBUG((D_INFO, L"*pwcpasswd = %ls\n", *pwcpasswd));

	return 0;
}