﻿/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include "stdafx.h"
#include <stdio.h>
#include <windows.h>
#include <Wincrypt.h>

#pragma warning(disable: 4995)

#define BUFSIZE      1024
#define MD5LEN       16
#define SHA256LEN    64
#define HASH_DIGITS  "0123456789abcdef"

#ifdef __cplusplus
extern "C" {
#endif
extern
DWORD
__stdcall
TcBufMd5sum (
  unsigned char *dest,
  const unsigned char *data,
  size_t nBytes
  );

extern
DWORD
__stdcall
TcFileMd5sum(
  unsigned char *dest,
  LPCWSTR wszFilename
  );

/**
 * compute sha256sum of first nBytes of a buffer
 * @return 0 on success
 * @return -1 on failure
 */
DWORD
__stdcall
TcBufSha256Sum(
	unsigned char *dest,
	const unsigned char *data,
	size_t nBytes
);

#ifdef __cplusplus
}
#endif

// compute md5sum of a file
DWORD
__stdcall
TcFileMd5sum(
  unsigned char *dest,
  LPCWSTR wszFilename
  )
{
  HANDLE hFile;
  HCRYPTPROV hCryptProv;
  HCRYPTHASH hCryptHash;
  BYTE rFileBuf[BUFSIZE];
  BYTE szHashBuf[MD5LEN];
  PCHAR szHashDigits;
  BOOL bTmpRet;
  DWORD i;
  DWORD dwRet;
  DWORD dwRdBufSz;
  DWORD dwHashBufSz;
  
  hFile = NULL;
  szHashDigits = HASH_DIGITS;
  hCryptProv = 0;
  hCryptHash = 0;
  dwRdBufSz = 0;
  dwHashBufSz = 0;
  bTmpRet = FALSE;
  dwRet = 0;
  
  hFile = CreateFile(
    wszFilename,
    GENERIC_READ,
    FILE_SHARE_READ,
    NULL,
    OPEN_EXISTING,
    FILE_FLAG_SEQUENTIAL_SCAN,
    NULL);

  if (INVALID_HANDLE_VALUE == hFile)
  {
    dwRet = GetLastError();
    printf("failed to call CreateFile, passed in %ls\n", wszFilename); 
    printf("retval: 0x%x\n", dwRet); 
    return dwRet;
  }

  bTmpRet = CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);

  if (!bTmpRet)
  {
    dwRet = GetLastError();
    printf("fail to get handle to CryptAcquireContext, retval: 0x%x\n", dwRet); 
    CloseHandle(hFile);
    return dwRet;
  }

  bTmpRet = CryptCreateHash(hCryptProv, CALG_MD5, 0, 0, &hCryptHash);

  if (!bTmpRet)
  {
    dwRet = GetLastError();
    printf("fail to call CryptCreateHash, retval: 0x%x\n", dwRet); 
    CloseHandle(hFile);
    CryptReleaseContext(hCryptProv, 0);
    return dwRet;
  }

  while (bTmpRet = ReadFile(hFile, rFileBuf, BUFSIZE, &dwRdBufSz, NULL))
  {
    if (0 == dwRdBufSz)
    {
      break;
    }

    if (!CryptHashData(hCryptHash, rFileBuf, dwRdBufSz, 0))
    {
      dwRet = GetLastError();
      printf("fail to call CryptHashData, retval: 0x%x\n", dwRet); 
      CryptReleaseContext(hCryptProv, 0);
      CryptDestroyHash(hCryptHash);
      CloseHandle(hFile);
      return dwRet;
    }
  }

  if (!bTmpRet)
  {
    dwRet = GetLastError();
    printf("ReadFile failed: 0x%x\n", dwRet); 
    CryptReleaseContext(hCryptProv, 0);
    CryptDestroyHash(hCryptHash);
    CloseHandle(hFile);
    return dwRet;
  }

  dwHashBufSz = MD5LEN;
  bTmpRet = CryptGetHashParam(hCryptHash, HP_HASHVAL, szHashBuf, &dwHashBufSz, 0);

  if (bTmpRet)
  {
    for (i = 0; i < dwHashBufSz; i++)
    {
      sprintf((char *) dest + 2 * i, "%c%c", szHashDigits[szHashBuf[i] >> 4], szHashDigits[szHashBuf[i] & 0xf]);
    }
  }
  else
  {
    dwRet = GetLastError();
    printf("fail to call CryptGetHashParam, retval: 0x%x\n", dwRet); 
  }

  CryptDestroyHash(hCryptHash);
  CryptReleaseContext(hCryptProv, 0);
  CloseHandle(hFile);

  return dwRet; 
} 


// compute the md5sum of a buf
DWORD
__stdcall
TcBufMd5sum (
  unsigned char *dest,
  const unsigned char *data,
  size_t nBytes
  )
{
  DWORD i;
  DWORD dwRet;
  BOOL bTmpRet;
  HCRYPTPROV hCryptProv;
  HCRYPTHASH hCryptHash;
  BYTE szHashBuf[MD5LEN];
  DWORD dwHashBufSz;
  PCHAR szHashDigits;
  
  szHashDigits = HASH_DIGITS;
    hCryptProv = 0;
    hCryptHash = 0;
    dwHashBufSz = 0;
    dwRet = 0;
    bTmpRet = FALSE;

  bTmpRet = CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT);
  if (!bTmpRet)
  {
    dwRet = GetLastError();
    printf("failed to call CryptAcquireContext, retval: 0x%x\n", dwRet); 
    return dwRet;
  }

  bTmpRet = CryptCreateHash(hCryptProv, CALG_MD5, 0, 0, &hCryptHash);
  if (!bTmpRet)
  {
    dwRet = GetLastError();
    printf("failed to call CryptCreateHash retval: %d\n", dwRet); 
    CryptReleaseContext(hCryptProv, 0);
    return dwRet;
  }

  bTmpRet = CryptHashData(hCryptHash, data, (DWORD)nBytes, 0);
  if (!bTmpRet)
  {
    dwRet = GetLastError();
    printf("failed to call CryptHashData retval: 0x%x\n", dwRet); 
    CryptReleaseContext(hCryptProv, 0);
    CryptDestroyHash(hCryptHash);
    return dwRet;
  }

  dwHashBufSz = MD5LEN;
  bTmpRet = CryptGetHashParam(hCryptHash, HP_HASHVAL, szHashBuf, &dwHashBufSz, 0);
  if (bTmpRet)
  {
    for (i = 0; i < dwHashBufSz; i++)
    {
      sprintf((char *)dest + 2 * i, "%c%c", szHashDigits[szHashBuf[i] >> 4], szHashDigits[szHashBuf[i] & 0xf]);
    }
  }
  else
  {
    dwRet = GetLastError();
    printf("failed to call CryptGetHashParam, retval: 0x%x\n", dwRet); 
  }

  CryptDestroyHash(hCryptHash);
  CryptReleaseContext(hCryptProv, 0);

  return dwRet; 
}

/**
 * compute sha256sum of first nBytes of a buffer
 * @return 0 on success
 * @return -1 on failure
 */
DWORD
__stdcall
TcBufSha256Sum(
	unsigned char *dest,
	const unsigned char *data,
	size_t nBytes
)
{
	DWORD i = 0;
	DWORD dwRet = -1;
	HCRYPTPROV hCryptProv = 0;
	HCRYPTHASH hCryptHash = 0;
	BYTE       szHashBuf[SHA256LEN] = { 0 };
	DWORD      dwHashBufSz = SHA256LEN;
	PCHAR      szHashDigits = HASH_DIGITS;

	if (CryptAcquireContext(&hCryptProv, NULL, NULL, PROV_RSA_AES, 0)) {
		wprintf(L"CryptAcquireContext complete. \n");
	}
	else {
		wprintf(L"Acquisition of context failed: 0x%x\n", GetLastError());
		goto exit1;
	}

	if (CryptCreateHash(hCryptProv, CALG_SHA_256, 0, 0, &hCryptHash)) {
		wprintf(L"An empty hash object has been created.\n");
	}
	else {
		wprintf(L"Error during CryptBeginHash!: 0x%x\n", GetLastError());
		goto exit2;
	}

	if (!CryptHashData(hCryptHash, data, (DWORD)nBytes, 0)) {
		wprintf(L"failed to call CryptHashData retval: 0x%x\n", GetLastError());
		goto exit3;
	}

	if (CryptGetHashParam(hCryptHash, HP_HASHVAL, szHashBuf, &dwHashBufSz, 0)) {
		for (i = 0; i < dwHashBufSz; i++)
		{
			sprintf((char *)dest + 2 * i, "%c%c", szHashDigits[szHashBuf[i] >> 4], szHashDigits[szHashBuf[i] & 0xf]);
		}
	}
	else {
		wprintf(L"failed to call CryptGetHashParam, retval: 0x%x\n", GetLastError());
		goto exit3;
	}

	dwRet = 0;
exit3:
	CryptDestroyHash(hCryptHash);
exit2:
	CryptReleaseContext(hCryptProv, 0);
exit1:
	return dwRet;
}
