/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
/** @file
**/
#define _CRT_SECURE_NO_WARNINGS
#ifdef _WIN32
#include <ntddk.h>
#else // _WIN32
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/fs.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#endif // _WIN32

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "TcNtLib.h"
#include "TcDebugLib.h"

#ifdef _WIN32

/*
* When building with Visual Studio 2015 + WDK7600.16385.1, need to define [W]ERR to empty
* Otherwise, the TcCredProvider.dll might not work the 1st time to boot a local windows.
*
* This issue does not reproduce when building with Visual Studio 2008SP1 + WDK7600.16385.1
*/
#define ERR(...)  /* fprintf(stderr, "Error: "##__VA_ARGS__) */
#define WERR(...) /* fwprintf(stderr, L"Error: "##__VA_ARGS__) */

#ifdef __cplusplus
extern "C"
{
#endif // __cplusplus

	NTSYSAPI
		NTSTATUS
		NTAPI
		NtReadFile(HANDLE FileHandle,
			HANDLE Event OPTIONAL,
			PIO_APC_ROUTINE ApcRoutine OPTIONAL,
			PVOID ApcContext OPTIONAL,
			PIO_STATUS_BLOCK IoStatusBlock,
			PVOID Buffer,
			ULONG Length,
			PLARGE_INTEGER ByteOffset OPTIONAL,
			PULONG Key OPTIONAL);

	NTSYSAPI
		NTSTATUS
		NTAPI
		NtWriteFile(HANDLE FileHandle,
			HANDLE Event OPTIONAL,
			PIO_APC_ROUTINE ApcRoutine OPTIONAL,
			PVOID ApcContext OPTIONAL,
			PIO_STATUS_BLOCK IoStatusBlock,
			PVOID Buffer,
			ULONG Length,
			PLARGE_INTEGER ByteOffset OPTIONAL,
			PULONG Key OPTIONAL);

	NTSYSAPI
		NTSTATUS
		NTAPI
		NtClose(HANDLE Handle);

	NTSYSAPI
		NTSTATUS
		NTAPI
		NtCreateFile(PHANDLE FileHandle,
			ACCESS_MASK DesiredAccess,
			POBJECT_ATTRIBUTES ObjectAttributes,
			PIO_STATUS_BLOCK IoStatusBlock,
			PLARGE_INTEGER AllocationSize OPTIONAL,
			ULONG FileAttributes,
			ULONG ShareAccess,
			ULONG CreateDisposition,
			ULONG CreateOptions,
			PVOID EaBuffer OPTIONAL,
			ULONG EaLength);
#ifdef __cplusplus
}
#endif // __cplusplus

#endif // _WIN32

// function:
// open a file with multibyte string name `pmbFilePath' and 
//try to read at most `nBytes' from the file, then save the
// content to address pointed by `buf'.
// return value:
// if encounter errors, return a negative integer
// otherwise, return the number of bytes returned
#ifdef _WIN32



int __stdcall TcOneTimeNtReadFileW(unsigned char *buf, size_t nBytes, const wchar_t *wszFilepath)
{
	HANDLE hFile;
	UNICODE_STRING usFilePath;
	OBJECT_ATTRIBUTES objAttr;
	IO_STATUS_BLOCK ioStatus;
	NTSTATUS ntStatus;
	int nBytesRead;
	unsigned __int64 offset;
	int ret;

	hFile = NULL;
	offset = 0;
	nBytesRead = -1;
	ret = -1;

	//
	// convert wide char string to unicode struct string
	//
	RtlInitUnicodeString(&usFilePath, wszFilepath);

	InitializeObjectAttributes(
		&objAttr,
		&usFilePath,
		OBJ_CASE_INSENSITIVE,
		NULL,
		NULL
	);

	ntStatus = NtCreateFile(
		&hFile,
		FILE_GENERIC_READ,
		&objAttr,
		&ioStatus,
		NULL,
		FILE_ATTRIBUTE_NORMAL,
		FILE_SHARE_READ,
		FILE_OPEN,
		FILE_NON_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT,
		NULL,
		0
	);

	if (!NT_SUCCESS(ntStatus))
	{
		WERR(L"TcOneTimeNtReadFile(): Failed to open file\n");
		WERR(L"TcOneTimeNtReadFile(): Error code is 0x%x\n", ntStatus);
		ret = -3;
		goto wTcOneTimeNtReadFile_Error_1;
	}

	ntStatus = NtReadFile(
		hFile,
		0,
		NULL,
		NULL,
		&ioStatus,
		buf,
		(ULONG)nBytes,
		(PLARGE_INTEGER)(&offset),
		NULL
	);

	if (ntStatus == STATUS_END_OF_FILE)
	{
		ret = 0;
		WERR(L"TcOneTimeNtReadFile(): Read reach end of file 0x%x\n", ntStatus);
		goto wTcOneTimeNtReadFile_Error_2;
	}

	if (!NT_SUCCESS(ntStatus))
	{
		WERR(L"TcOneTimeNtReadFile(): Failed to read security file\n");
		WERR(L"TcOneTimeNtReadFile(): Error code is 0x%x...\n", ntStatus);
		ret = -4;
		goto wTcOneTimeNtReadFile_Error_2;
	}

	nBytesRead = (int)ioStatus.Information;
	ret = nBytesRead;

wTcOneTimeNtReadFile_Error_2:
	NtClose(hFile);
	hFile = NULL;

wTcOneTimeNtReadFile_Error_1:
	return ret;
}

int __stdcall TcOneTimeNtReadFile(unsigned char *buf, size_t nBytes, const char *szFilepath)
{
	int ret;
	size_t requiredSize;
	size_t count;
	LPWSTR wszFilepath;

	ret = -1;
	
	//
	// convert file name from multibyte string to wide char string
	//
	requiredSize = mbstowcs(NULL, szFilepath, 0);
	wszFilepath = (LPWSTR)calloc(requiredSize + 1, sizeof(wchar_t));
	if (wszFilepath == NULL)
	{
		DEBUG((D_ERROR, L"TcOneTimeNtReadFile(): memory allocation failure.\n"));
		ret = -1;
		goto exit;
	}
	count = mbstowcs(wszFilepath, szFilepath, requiredSize + 1);
	if ((size_t)(-1) == count)
	{
		DEBUG((D_ERROR, L"TcOneTimeNtReadFile(): Couldn't convert string: %s\n", szFilepath));
		ret = -2;
		goto exit;
	}

	ret = TcOneTimeNtReadFileW(buf, nBytes, wszFilepath);

exit:
	if (wszFilepath != NULL)
	{
		free(wszFilepath);
		wszFilepath = NULL;
	}

	return ret;
}

/**
   write buffer to file
   @return                   0 on success
**/
int __stdcall TcOneTimeNtWriteFileW(unsigned char *buf, size_t nBytes, const wchar_t *wszFilepath)
{
	int ret;
	NTSTATUS  ntStatus;
	UNICODE_STRING  uniFilePath;
	OBJECT_ATTRIBUTES ObjAttr;
	IO_STATUS_BLOCK  IoStatus;
	HANDLE  hFile;

	ret = 1;

	RtlInitUnicodeString(&uniFilePath, wszFilepath);
	InitializeObjectAttributes(&ObjAttr, &uniFilePath, OBJ_CASE_INSENSITIVE, NULL, NULL);

	ntStatus = NtCreateFile(
		&hFile,
		FILE_GENERIC_WRITE,
		&ObjAttr,
		&IoStatus,
		NULL,
		FILE_ATTRIBUTE_NORMAL,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		FILE_OVERWRITE_IF,
		FILE_NON_DIRECTORY_FILE | FILE_SYNCHRONOUS_IO_NONALERT | FILE_WRITE_THROUGH,
		NULL,
		0
	);

	if (!NT_SUCCESS(ntStatus))
	{
		DEBUG((D_ERROR, L"%hs: NtCreateFile(%ls) failed: %x\n", __FUNCTION__, wszFilepath, ntStatus));
		ret = 1;
		goto exit;
	}

	ntStatus = NtWriteFile(hFile, NULL, NULL, NULL, &IoStatus, buf, (ULONG)nBytes, NULL, NULL);
	if (!NT_SUCCESS(ntStatus))
	{
		DEBUG((D_ERROR, L"%hs: NtWriteFile failed: 0x%X\n", __FUNCTION__, ntStatus));
		ret = 2;
		goto exit;
	}

	DEBUG((D_INFO, L"%hs: transfer bytes: 0x%X\n", __FUNCTION__, IoStatus.Information));

	NtClose(hFile);
	ret = 0;

exit:
	return ret;
}

/**
  write buffer to file
  @return 0 on success
**/
int __stdcall TcOneTimeNtWriteFile(unsigned char *buf, size_t nBytes, const char *szFilepath)
{
	int ret;
	size_t requiredSize;
	size_t count;
	LPWSTR wszFilepath;

	ret = -1;

	//
	// convert file name from multibyte string to wide char string
	//
	requiredSize = mbstowcs(NULL, szFilepath, 0);
	wszFilepath = (LPWSTR)calloc(requiredSize + 1, sizeof(wchar_t));
	if (wszFilepath == NULL)
	{
		DEBUG((D_ERROR, L"TcOneTimeNtWriteFile(): memory allocation failure.\n"));
		ret = -1;
		goto exit;
	}
	count = mbstowcs(wszFilepath, szFilepath, requiredSize + 1);
	if ((size_t)(-1) == count)
	{
		DEBUG((D_ERROR, L"TcOneTimeNtWriteFile(): Couldn't convert string: %s\n", szFilepath));
		ret = -2;
		goto exit;
	}

	ret = TcOneTimeNtWriteFileW(buf, nBytes, wszFilepath);

exit:
	if (wszFilepath != NULL)
	{
		free(wszFilepath);
		wszFilepath = NULL;
	}

	return ret;
}


#else // _WIN32

#ifdef ANDROID
# define MOUNT_POINT			"/system/tmpmnt4"
# define MOUNT_CACHE_DEV 	"mkdir -p " MOUNT_POINT " && mount -t ext3 /dev/block/sda4 " MOUNT_POINT
# define UMOUNT_CACHE_DEV	"umount " MOUNT_POINT
#else // ANDROID
# define MOUNT_POINT			".tcdisk_tmp"
# define MOUNT_CACHE_DEV 	"mkdir -p " MOUNT_POINT " && mount /dev/sda4 " MOUNT_POINT
# define UMOUNT_CACHE_DEV	"umount " MOUNT_POINT
#endif // ANDROID

# ifdef ANDROID
static int and_system_proxy(const char * cmdStr)
{
	pid_t childPid;
	int status;

	if (cmdStr == NULL)
	{
		return 1;
	}

	if ((childPid = fork())<0)
	{
		status = -1;
	}
	else if (childPid == 0)
	{
		execl("/system/bin/sh", "sh", "-c", cmdStr, (char *)0);
		_exit(127);
	}
	else
	{
		while (waitpid(childPid, &status, 0) < 0)
		{
			if (errno != EINTR)
			{
				status = -1;
				break;
			}
		}
	}

	return status;
}
# endif // ANDROID

int TcOneTimeNtReadFile(unsigned char *buf, size_t nBytes, const char *pmbFilePath)
{
	int fd;
	int realReadLen;
	int ret;
	int retry;
	char mntFilePath[512];
	char *p, *q;

	ret = -1;
	fd = -1;
	retry = 0;
	realReadLen = -1;
	bzero(mntFilePath, sizeof(mntFilePath));
	p = NULL;
	q = NULL;

	if (!buf || nBytes <= 0 || !pmbFilePath)
	{
		return ret;
	}
# ifdef ANDROID
	and_system_proxy(MOUNT_CACHE_DEV);
# else
	system(MOUNT_CACHE_DEV);
# endif

	p = strdup(pmbFilePath);
	q = strrchr(p, '/');
	if (q == NULL)
	{
		q = p;
	}
	strcpy(mntFilePath, MOUNT_POINT);
	strcat(mntFilePath, "/");
	strcat(mntFilePath, q);

try_open_file:
	fd = open(mntFilePath, O_RDONLY);
	if (fd <= 0)
	{
		DEBUG((D_ERROR, L"Error opening %hs: 0x%x\n", mntFilePath, errno));

#if !defined(ANDROID)
		if (retry == 0)
		{
			strcpy(mntFilePath, "/tmp");
			strcat(mntFilePath, "/");
			strcat(mntFilePath, q);

			retry++;
			goto try_open_file;
		}
#endif
		goto sys_exit;
	}

	realReadLen = read(fd, buf, nBytes);
	if (realReadLen == -1)
	{
		DEBUG((D_ERROR, L"Failed to read %d bytes (0x%x)\n", nBytes, errno));
		goto sys_exit;
	}

	ret = realReadLen;

sys_exit:

	if (p != NULL)
	{
		free(p);
		p = NULL;
	}

	if (fd > 0)
	{
		close(fd);
		fd = -1;
	}

# ifdef ANDROID
	and_system_proxy(UMOUNT_CACHE_DEV);
# else
	system(UMOUNT_CACHE_DEV);
# endif

	return ret;
}

#endif // _WIN32

