/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "TcConfig.h"
#include "TcDebugLib.h"
#include "TcCtl.h"

#ifdef _WIN32
#include "TcNtLib.h"
#else
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/fs.h>
#include <sys/stat.h>
#include <errno.h>
#endif

#ifdef _WIN32
#define IP_FILE_PATH4     "\\Device\\Harddisk0\\Partition3\\tc-serv.addr"
//#define IP_FILE_PATH4     "\\Device\\Harddisk0\\Partition2\\tc-serv.addr"
/**
Fetch ip addr from local hard drive

@param ip        a buffer to store the result
@param ip_len    size of the buffer
@return          0 on success, negative value if error
**/
int
TC_STDCALL
ParseIpConfig(
	char *ip,
	size_t ip_len
)
{
  int ret;

  ret = -1;

  if (ip == NULL || ip_len < 1) {
    return ret;
  }
  
  memset(ip, 0, ip_len);
  ret = TcOneTimeNtReadFile(ip, ip_len - 1, IP_FILE_PATH4);
  if (ret <= 0) {
    DEBUG((D_ERROR, L"fail to load config file: %d\n", ret));
    return ret;
  }
  else
  {
    ret = 0;
    DEBUG((D_INFO, L"read buf: %hs\n", ip));
  }

  return ret;
}

#else // _WIN32

#ifdef ANDROID
#define MOUNT_POINT      "/system/tmpmnt4"
#define MOUNT_CACHE_DEV4   "mount -t ext3 /dev/block/sda4 " MOUNT_POINT
#define UMOUNT_CACHE_DEV  "umount " MOUNT_POINT
#define IP_FILE_PATH     MOUNT_POINT "/tc-serv.addr"
#else // ANDROID
#define MOUNT_POINT      ".tcdisk_tmp"
#define MOUNT_CACHE_DEV4   "mkdir -p " MOUNT_POINT " && mount /dev/sda4 " MOUNT_POINT
#define UMOUNT_CACHE_DEV  "umount " MOUNT_POINT
#define IP_FILE_PATH     MOUNT_POINT "/tc-serv.addr"
#endif // ANDROID

#ifdef ANDROID

static int and_system_proxy(const char * cmdStr)
{
  pid_t childPid;
  int status;

  if (cmdStr == NULL)
  {
    return 1;
  }

  if ((childPid = fork()) < 0)
  {
    status = -1;
  }
  else if (childPid == 0)
  {
    execl("/system/bin/sh", "sh", "-c", cmdStr, (char *)0);
    _exit(127);
  }
  else
  {
    while (waitpid(childPid, &status, 0) < 0)
    {
      if (errno != EINTR)
      {
        status = -1;
        break;
      }
    }
  }

  return status;
}

#endif /* ANDROID */


int
TC_STDCALL
ParseIpConfig(
	char *ip,
	size_t ip_len
)
{
  int file_handle = -1;
  int realReadLen;
  int ret = -1;
  int retried = 0;

  if (!ip || (ip_len <= 0))
  {
    DEBUG((D_ERROR, L"Invalid parameter\n"));
    return -1;
  }

#ifdef ANDROID
  and_system_proxy(MOUNT_CACHE_DEV4);
#else
  system(MOUNT_CACHE_DEV4);
#endif

TryOpenIpFile_linux:
  file_handle = open(IP_FILE_PATH, O_RDONLY);
  if (file_handle <= 0)
  {
    DEBUG((D_ERROR, L"Open %hs: %d\n", IP_FILE_PATH, errno));
    //  if (retried == 0)
    //  {
    //#ifdef ANDROID
    //    and_system_proxy(UMOUNT_CACHE_DEV);
    //#else
    //    system(UMOUNT_CACHE_DEV);
    //#endif
    //#ifdef ANDROID
    //    and_system_proxy(MOUNT_CACHE_DEV4);
    //#else
    //    system(MOUNT_CACHE_DEV4);
    //#endif
    //    retried = 1;
    //    goto TryOpenIpFile_linux;    
    //  }
    goto sys_exit;
  }

  if ((realReadLen = read(file_handle, ip, ip_len)) == -1) {
    DEBUG((D_ERROR, L"Failed to read 0x%lx bytes (%d)\n", ip_len, errno));
    //close(file_handle);
    goto sys_exit;
  }

  if (realReadLen > 0 && (ip[realReadLen - 1] == '\n'))
  {
    ip[realReadLen - 1] = 0;
  }
  DEBUG((D_INFO, L"Ip parsed in file: %hs\n", ip));

  ret = 0;

sys_exit:

  if (file_handle > 0)
    close(file_handle);

#ifdef ANDROID
  and_system_proxy(UMOUNT_CACHE_DEV);
#else
  system(UMOUNT_CACHE_DEV);
#endif
  return ret;
}

#endif // _WIN32

