/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <locale.h>
#include <wchar.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include "TcDebugLib.h"
#include "localsock.h"


static int local_sock_entry (int option){
    int local_sock;
    struct sockaddr_un local_sockaddr;
    struct local_notify_msg msg;

    if ((local_sock = socket (AF_UNIX, SOCK_DGRAM, 0)) < 0){
        DEBUG((D_ERROR, L"socket(): %#x\n", errno));
        return -1;
    }

    memset (&local_sockaddr, 0, sizeof (local_sockaddr));
    local_sockaddr.sun_family = AF_UNIX;
    strcpy (local_sockaddr.sun_path, NOTIFY_SOCK_PATH);

    memset (&msg, 0, sizeof (msg));
    msg.type = option;
    msg.size = sizeof (msg);

    if ((sendto (local_sock, (unsigned char *)&msg, sizeof (msg), 0, (struct sockaddr *)&local_sockaddr, sizeof (local_sockaddr))) < 0){
        DEBUG((D_ERROR, L"sendto(): %#x\n", errno));
        close (local_sock);
        return -1;
    }

    close (local_sock);
    return 0;
}


int main (int argc, char *argv[]){

    setlocale(LC_CTYPE, "");
    DEBUG_INIT(TO_STD_CONSOLE, NULL);
    PRINT_INIT(TO_STD_CONSOLE, NULL);

    if (argc >= 2 && !strcmp(argv[1], "--poweroff")){
        return local_sock_entry (LOCAL_MSG_POWEROFF);
    }

    if (argc >= 2 && !strcmp(argv[1], "--reboot")){
        return local_sock_entry (LOCAL_MSG_REBOOT);
    }

    PRINT(L"SYSNOPSYS:\n");
    PRINT(L"    %hs --poweroff\n", argv[0]);
    PRINT(L"    %hs --reboot\n", argv[0]);

    DEBUG_FINISH();
    PRINT_FINISH();
    return 1;
}

