/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
/** @file
**/
#include "TcConfig.h"
#include "TcCtl.h"
#include "TcDebugLib.h"
#include "TcNtLib.h"
#include "TcKeyValueLib.h"
#ifdef _WIN32
#include "TcWin32Ctrl.h"
#endif

#ifdef _WIN32
#pragma warning(disable: 4101) // close unreferenced varialbe warnings
#pragma warning(disable: 4996) // close _WINSOCK_DEPRECATED_NO_WARNINGS
#endif

#ifdef _WIN32
HINSTANCE hInst;
HWND hWnd;
#endif // _WIN32

char *cifn = NULL;
char *localip = NULL;
unsigned char *localmac = NULL;
int g_statusSock = -1;

// data channel socket, UDP
int g_groupSock = -1;
struct sockaddr_in g_ctrl_sock_addr = { 0 };
unsigned char g_exit = 0;

// create a socket to receive unicast messages
int g_clt_sock = -1;
struct sockaddr_in g_clt_sock_addr = { 0 };

#ifdef POST_REPORT_HWINFO
static int g_disksz = 0;
static int g_memsz = 0;
static char g_strcpu[CPU_STR_SZ] = { 0 };
static char g_st_uname[UNAME_SZ] = { 0 };
static char g_st_imageid[OSNAME_SZ] = { 0 };
static int g_st_statusid = ACT_RUNNING;
#endif

// load status from a config file
int load_status_conf(const char *filePath)
{
  int ret = 0;
  int buflen;
  char buf[1024];
  struct config_data *pcd, *pcursor;
  int nKey;
  int j;
  int i;
  struct config_data sd[6];

  DEBUG((D_INFO, L"%hs:%d:\n", __FUNCTION__, __LINE__));

  // open status configuration file and read it
  buflen = TcOneTimeNtReadFile((unsigned char *)buf, sizeof(buf), filePath);
  DEBUG((D_INFO, L"%hs: TcOneTimeNtReadFile returned: %d\n", __FUNCTION__, buflen));

  if (buflen < 0)
  {
    DEBUG((D_ERROR, L"%hs: fail to open file %hs\n", __FUNCTION__, filePath));
    return -1;
  }
  if (buflen == 0)
  {
    DEBUG((D_ERROR, L"%hs: file %hs is empty\n", __FUNCTION__, filePath));
    return -1;
  }

  // initialize the struct
  strcpy(sd[0].key, ST_STATUS);
  strcpy(sd[1].key, ST_USERNAME);
  strcpy(sd[2].key, ST_OSID);
  strcpy(sd[3].key, ST_DISKSZ);
  strcpy(sd[4].key, ST_MEMSZ);
  strcpy(sd[5].key, ST_CPUMODEL);
  strcpy(sd[0].value, "5");
  strcpy(sd[1].value, "");
  strcpy(sd[2].value, "0");
  strcpy(sd[3].value, "0");
  strcpy(sd[4].value, "0");
  strcpy(sd[5].value, "");
  // for (j = 0; j < 6; j++)
  // {
    // sd[j].next = NULL;
    // strcpy(sd[j].value, "");
  // }

  nKey = sget_key_val(buf, buflen, "=", &pcd);
  if (nKey <= 0)
  {
    DEBUG((D_ERROR, L"%hs: can not find valid key-value pairs in file %hs\n", __FUNCTION__, filePath));
    // generate the command line string
    return -1;
  }
  // get key-value pairs from the buf
  // using "=" as delimiter between key and value
  else
  {
    // parse the key-value pair and store it into the struct.
    pcursor = pcd;
    while (pcursor != NULL)
    {
      for (j = 0; j < 6; j++)
      {
        if (!strcmp(sd[j].key, pcursor->key))
        {
          strcpy(sd[j].value, pcursor->value);

          DEBUG((D_INFO, L"%hs: find key-value pair (%hs,%hs)\n", __FUNCTION__, sd[j].key, sd[j].value));
          if (sd[j].next == NULL)
          {
            // just a simple mark indicating it is valid
            sd[j].next = &sd[j];
          }
          break;
        }
      }
      if (j == 6)
      {
        DEBUG((D_WARNING, L"%hs: unrecognized key-value pair (%hs,%hs)\n", __FUNCTION__, pcursor->key, pcursor->value));
      }

      pcursor = pcursor->next;
    }
  }

  // save to global vars
  g_disksz = atoi(sd[3].value);
  g_memsz = atoi(sd[4].value);
  strncpy(g_strcpu, sd[5].value, CPU_STR_SZ - 1);
  g_strcpu[CPU_STR_SZ - 1] = '\0';

  strncpy(g_st_uname, sd[1].value, sizeof(g_st_uname) - 1);
  g_st_uname[sizeof(g_st_uname) - 1] = '\0';

  strncpy(g_st_imageid, sd[2].value, sizeof(g_st_imageid) - 1);
  g_st_imageid[sizeof(g_st_imageid) - 1] = '\0';

  g_st_statusid = atoi(sd[0].value);

  // free the linked-list
  if (pcd != NULL)
  {
    free_key_val(pcd);
    pcd = NULL;
  }

  return 0;
}

int _parse_info(char *infoStr, char *data1, uint32 data1_offset,
  char *data2, uint32 data2_offset,
  char *data3, uint32 data3_offset)
{
  char *pos;
  char *tmp_str;
  char *output[3];
  uint32 output_offset[3];
  int i = 0, j = 0;
  int tmp_str_len;

  if (!infoStr)
    return -1;

  output[0] = data1;
  output[1] = data2;
  output[2] = data3;

  output_offset[0] = data1_offset;
  output_offset[1] = data2_offset;
  output_offset[2] = data3_offset;

  tmp_str = (char *)malloc(strlen(infoStr) + 1);
  if (!tmp_str)
    return -1;
  strcpy(tmp_str, infoStr);

  pos = tmp_str;

  tmp_str_len = (int)strlen(tmp_str);

  while ((pos) && j < 3) {
    if (tmp_str[i] == INFO_SEP_CHAR || (i == tmp_str_len)) {
      tmp_str[i] = '\0';
      if (strlen(pos) >= output_offset[j]) {
        strcpy(output[j], pos + output_offset[j]);
        j++;
      }
      else {
        DEBUG((D_ERROR, L"Target string(%hs)is shorter than expected: %d\n",
          pos, output_offset[j]));
        return -1;
      }
      if (i == tmp_str_len)
        pos = NULL;
      else
        pos = tmp_str + i + 1;
    }

    i++;
  }
  free(tmp_str);

  if (j > 0)
    j--;

  if (j < 2) {
    DEBUG((D_ERROR, L"Format error, only %d \"%hc\" found!\n", j, INFO_SEP_CHAR));
    return -1;
  }

  return 0;
}

void clear_ccb(CCB *ccbp)
{
  ccbp->clt_id = 999;
  ccbp->clt_cbulk = -1;
  ccbp->clt_totalbytes = 0;

  ccbp->clt_hasnofile = 0;

  memset(&(ccbp->pkacktv), 0, sizeof(ccbp->pkacktv));

  return;
}

void init_ccb(CCB *ccbp)
{
  ccbp->tcdclt_debugger = 0;

  DEBUG((D_INFO, L"client: initial state to [init]\n"));
  ccbp->clt_state = CLT_STATE_INIT;

  // default log file, defined in conf.h
  ccbp->clt_logfile = "";

  clear_ccb(ccbp);

  return;
}

#ifdef _WIN32

unsigned char * ifn2mac(char *ifn, char *lip)
{
  HRESULT hr;
  uint32  ipStr;
  unsigned char *hwa;
  unsigned long   ulLen;

  hwa = (unsigned char *)malloc(16);
  if (!hwa)
    return NULL;
  memset(hwa, 0xff, 16);

  ipStr = inet_addr(lip);
  ulLen = 6;
  hr = SendARP(ipStr, 0, hwa, &ulLen);
  if (hr != NO_ERROR)
    return NULL;

  return hwa;
}

char *ifn2ip_str(char *ifn)
{
  char *ip4addr = NULL;
  char hname[255];
  PHOSTENT hostinfo;

  if (gethostname(hname, sizeof(hname)) == 0)
  {
    if ((hostinfo = gethostbyname(hname)) != NULL)
    {
      ip4addr = (char *)calloc(65, sizeof(char));
	  if (ip4addr == NULL)
	  {
		  return -1;
	  }
	  memset(ip4addr, 0, 65);

      strncpy(ip4addr, inet_ntoa(*(struct in_addr *)*hostinfo->h_addr_list), 64);
	  ip4addr[63] = "\0";
      DEBUG((D_INFO, L"====> figure out local ip of interface <%hs>: <%hs>\n", ifn, ip4addr));
    }
    else
      DEBUG((D_ERROR, L"last error code: %d\n", GetLastError()));
  }
  else
    DEBUG((D_ERROR, L"last error code: %d\n", GetLastError()));

  return ip4addr;
}

#else

// get the mac address
unsigned char *ifn2mac(char *ifn, char *szIP)
{
  int sock_get_ip;
  unsigned char *hwa;

  struct sockaddr_in *sin;
  struct ifreq ifr_ip;

  if ((sock_get_ip = (int)socket(AF_INET, SOCK_STREAM, 0)) == -1)
  {
    DEBUG((D_ERROR, L"socket create failse...GetLocalIp!/n"));
    return NULL;
  }

  memset(&ifr_ip, 0, sizeof(ifr_ip));
  strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);

  if (ioctl(sock_get_ip, SIOCGIFHWADDR, &ifr_ip) < 0)
    return NULL;

  hwa = (unsigned char *)malloc(16);
  memcpy(hwa, ifr_ip.ifr_hwaddr.sa_data, 6);

  close(sock_get_ip);

  return hwa;
}

//auto search the nic with IP set, and get its name and mac, if any
int getFirstAvailableNicInfo(char *ethName, char *mac, char *ip)
{
  int fd;
  int nicNum = 0;
  struct ifreq nicBuf[16];
  struct ifconf ifc;
  struct ifreq ifrcopy;
  unsigned char found = 0;
  char *tmpIp;
  //char mac[16] = {0};  
  //char ip[32] = {0};

  if ((fd = (int)socket(AF_INET, SOCK_DGRAM, 0)) < 0)
  {
    DEBUG((D_ERROR, L"socket\n"));
    close(fd);
    return -1;
  }

  ifc.ifc_len = sizeof(nicBuf);
  ifc.ifc_buf = (caddr_t)nicBuf;
  if (!ioctl(fd, SIOCGIFCONF, (char *)&ifc))
  {
    nicNum = ifc.ifc_len / sizeof(struct ifreq);
    while (!found && nicNum-- > 0)
    {
      if (strncmp("virbr", nicBuf[nicNum].ifr_name, 5) == 0)
        continue;

      //ignore the interface that not up or not runing  
      ifrcopy = nicBuf[nicNum];
      if (ioctl(fd, SIOCGIFFLAGS, &ifrcopy))
      {
        DEBUG((D_ERROR, L"ioctl: %hs [%hs:%d]\n", strerror(errno), __FILE__, __LINE__));
        close(fd);
        return -1;
      }

      if (ip) {
        //get the IP of this interface  
        if (!ioctl(fd, SIOCGIFADDR, (char *)&nicBuf[nicNum]))
        {
          tmpIp = (char *)inet_ntoa(((struct sockaddr_in *)&(nicBuf[nicNum].ifr_addr))->sin_addr);

          if ((strcmp(tmpIp, "") == 0) || (strcmp(tmpIp, "127.0.0.1") == 0))
            continue;
          else {
            strcpy(ip, tmpIp);
            found = 1;
          }
        }
        else
        {
          DEBUG((D_ERROR, L"ioctl: %hs\n", strerror(errno)));
          close(fd);
          return -1;
        }
      }

      if (mac) {
        //get the mac of this interface  
        if (!ioctl(fd, SIOCGIFHWADDR, (char *)(&nicBuf[nicNum])))
        {
          memset(mac, 0, sizeof(mac));
          memcpy(mac, nicBuf[nicNum].ifr_hwaddr.sa_data, 6);
          if (!ip)
            found = 1;
        }
        else
        {
          DEBUG((D_ERROR, L"Failed to get mac: %hs\n", strerror(errno)));
          close(fd);
          return -1;
        }
      }

      if (found && ethName)
        strcpy(ethName, nicBuf[nicNum].ifr_name);
    }
  }
  else
  {
    DEBUG((D_ERROR, L"Failed to get nic info: %hs\n", strerror(errno)));
    close(fd);
    return -1;
  }

  close(fd);

  if (found)
    return 0;
  else
    return -1;
}

char *ifn2ip_str(char *ifn)
{
  int ret;
  char *ip4addr = NULL;

  ip4addr = (char *)calloc(64, sizeof(char));
  ret = getFirstAvailableNicInfo(ifn, NULL, ip4addr);
  if (ret == -1) {
    free(ip4addr);
    return NULL;
  }
  return ip4addr;
}


#endif


// command actusr request
int actusr_buf(char **buf, char *uname, char *osname, int act)
{
  TCDMSG *msg;

  if ((*buf = (char *)malloc(BUFSZ)) == NULL)
  {
    DEBUG((D_ERROR, L"malloc\n"));
    return -1;
  }
  msg = (TCDMSG *)*buf;
  msg->type = TCDMSG_ACTUSR;
  msg->size = BUFSZ;

#ifdef POST_REPORT_HWINFO
  msg->u.au.disksz = g_disksz;
  msg->u.au.memsz = g_memsz;
  strcpy(msg->u.au.strcpu, g_strcpu);
#endif // POST_REPORT_HWINFO


#if 0
  defined in tcdmsg.h ...

#define ACT_POWERON    1  // nothing
#define ACT_AUTHENTICATED  2  // username
#define ACT_SYNCING    3  // username and osname
#define ACT_BOOTING    4  // username and osname
#define ACT_RUNNING    5  // nothing
#define ACT_SHUTTING_DOWN  6  // nothing
#define ACT_UPLOADING    7  // username and osname
#endif


    msg->u.au.act = act;
  if (act == ACT_AUTHENTICATED)
  {
    memcpy(msg->u.au.hwa, localmac, HWA_SZ);
    strcpy((char *)msg->u.au.uname, uname);
    //strcpy((char *)msg->u.au.osname, "");
    strcpy((char *)msg->u.au.osname, osname);
  }
  else if (act == ACT_SYNCING || act == ACT_BOOTING || act == ACT_UPLOADING)
  {
    memcpy(msg->u.au.hwa, localmac, HWA_SZ);
    strcpy((char *)msg->u.au.uname, uname);
    strcpy((char *)msg->u.au.osname, osname);
  }
  else if (act == ACT_POWERON
    || act == ACT_RUNNING
    || act == ACT_SHUTTING_DOWN
    || act == ACT_HEARTBEAT
    || act == ACT_INITIALIZING_DISK
    || act == ACT_UPDATING_FIRMWARE
    || ACT_UPDATING_FIRMWARE_SUCCESS
    || ACT_UPDATING_FIRMWARE_FAILURE
    ) // in both OS runtime or shutdown, uname and osname are invalid
  {
    memcpy(msg->u.au.hwa, localmac, HWA_SZ);
    //strcpy((char *)msg->u.au.uname, "");
    strcpy((char *)msg->u.au.uname, uname);
    //strcpy((char *)msg->u.au.osname, "");
    strcpy((char *)msg->u.au.osname, osname);
  }
  else
  {
    DEBUG((D_ERROR, L"actusr_buf: unknown act %d\n", act));
    memcpy(msg->u.au.hwa, localmac, HWA_SZ);
    strcpy((char *)msg->u.au.uname, "");
    strcpy((char *)msg->u.au.osname, "");
    return -1;
  }

  return BUFSZ;
}

// to send a NOTIFY message. 
int actusr_send(int socket, struct sockaddr_in sain, char *uname, char *osname, int act)
{
  char *msgbuf;
  int msgbuf_len;
  int sain_len = sizeof(sain);

  msgbuf_len = actusr_buf(&msgbuf, uname, osname, act);
  //DEBUG((D_INFO, L"actusr_send: msgbuf_len=%d\n", msgbuf_len));

  if (msgbuf_len > 0)
  {
    sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
    free(msgbuf);
    return 0;
  }
  else
  {
    DEBUG((D_ERROR, L"actusr_send: request buffer failed\n"));
    return -1;
  }
}


#ifdef ANDROID

int and_system(const char * cmdStr)
{
  pid_t childPid;
  int status;

  if (cmdStr == NULL)
  {
    return 1;
  }

  if ((childPid = fork()) < 0)
  {
    status = -1;
  }
  else if (childPid == 0)
  {
    execl("/system/bin/sh", "sh", "-c", cmdStr, (char *)0);
    _exit(127);
  }
  else
  {
    while (waitpid(childPid, &status, 0) < 0)
    {
      if (errno != EINTR)
      {
        status = -1;
        break;
      }
    }
  }

  return status;
}

#endif


/*

Here list the brief description of the state machine at client side:

there are four states: init, ready, run and stop, same as the four states at server side.

1, init: the very beginning state of the client. At this state, the client can accept the broadcasted hello packet
   from the server, and acknowledged with the registration request message.
a) accept registration acknowledgement message from the server, go to state 'ready' and take notes accordingly.
b) when pressing 'q' key, will go to state 'stop'

2, ready: the client is ready to run. In this state the client is waiting for the runrequest message only
a) when getting the runreq message from server, it will send runack message back to server, go to state 'run'
   and start to send/receive data packets.
b) when pressing 'q' key, it will go to state 'stop' to terminate the data sending/receiving.

3, run: the running state of the client, where the massive inbounding and outbounding packet processing happens here.
a) when pressing 'q' key, will go to state 'stop'
b) when getting the last packet of the file, will go to state 'stop'

4, stop: All packet transfer was done.
a) to print the summary of this transfer and quit.

*/

// message processing in client part
int proc_tcdmsg_clt(int socket, char *pkt, struct sockaddr_in ctrl_sock_addr, struct sockaddr_in group_sock_addr, CCB *ccbp)
{
  TCDMSG *msg = (TCDMSG *)pkt;

  //struct in_addr ina;
  //ina.s_addr = dstip;
  //DEBUG((D_INFO, L"client: hello received, server[%hs:%d]\n", inet_ntoa(ina), dstport));

  //ctrl_sock_addr.sin_addr.s_addr = msg->u.h.srvip;
  //ctrl_sock_addr.sin_port = htons((u_short)msg->u.h.srvport);

  switch (msg->type)
  {
  case TCDMSG_SHUTDOWN:
    DEBUG((D_INFO, L"=======================>>>>> shutdown message\n"));
#ifdef ANDROID
    and_system(SHUTDOWN_CMD_STR);
#else
    system(SHUTDOWN_CMD_STR);
#endif
    break;

  case TCDMSG_SHUTDOWN_IP:
    DEBUG((D_INFO, L"=======================>>>>> shutdown(ip) message\n"));
#ifdef ANDROID
    //if (!strcmp(msg->u.sd_ip.ipv4, localip)) {
      and_system(SHUTDOWN_CMD_STR);
    //}
#else
    //if (!strcmp(msg->u.sd_ip.ipv4, localip)) {
      system(SHUTDOWN_CMD_STR);
    //}
#endif
    break;

  case TCDMSG_REBOOT:
    DEBUG((D_INFO, L"=======================>>>>> reboot message\n"));
#ifdef ANDROID
    and_system(REBOOT_CMD_STR);
#else
    system(REBOOT_CMD_STR);
#endif
    break;

  case TCDMSG_REBOOT_IP:
    DEBUG((D_INFO, L"=======================>>>>> reboot(ip) message\n"));
#ifdef ANDROID
    //if (!strcmp(msg->u.rb_ip.ipv4, localip)) {
      and_system(REBOOT_CMD_STR);
    //}
#else
    //if (!strcmp(msg->u.rb_ip.ipv4, localip)) {
      system(REBOOT_CMD_STR);
    //}
#endif
    break;

  case TCDMSG_RM_IMAGE:
  {
    char rmimagebuf[256];
    DEBUG((D_INFO, L"=======================>>>>> remove image message\n"));
#if !defined _WIN32 && !defined ANDROID
    sprintf(rmimagebuf, "/usr/sbin/tc/rm-image.sh %hs %d", msg->u.rm_image_payload.image_org_name, msg->u.rm_image_payload.image_id);
    system(rmimagebuf);
#endif
    break;
  }

  case TCDMSG_RM_IMAGE_IP:
  {
    char rmimagebuf[256];
    DEBUG((D_INFO, L"=======================>>>>> remove image message\n"));
#if !defined _WIN32 && !defined ANDROID
    sprintf(rmimagebuf, "/usr/sbin/tc/rm-image.sh %hs %d", msg->u.rm_image_ip_payload.image_org_name, msg->u.rm_image_ip_payload.image_id);
    system(rmimagebuf);
#endif
    break;
  }

#if !defined _WIN32 && !defined ANDROID
  case TCDMSG_INITIALIZE:
  {
    int k;
    char system_buf[512];
    DEBUG((D_INFO, L"=======================>>>>> initialize the local harddisk\n"));
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_INITIALIZE, 0);
    sprintf(system_buf, "/usr/sbin/tc/tc-initdisk.sh %u", msg->u.id.sys_part_capacity);
    k = system(system_buf);
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_INITIALIZE, 1 + WEXITSTATUS(k));
    break;
  }
#endif

#if !defined _WIN32 && !defined ANDROID
  case TCDMSG_INITIALIZE_IP:
  {
    int k;
    char system_buf[512];
    DEBUG((D_INFO, L"=======================>>>>> initialize the local harddisk[%hs]\n", msg->u.init_payload.ipv4));
    if (strcmp(msg->u.init_payload.ipv4, localip)) {
      break;
    }
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_INITIALIZE, 0);
    sprintf(system_buf, "/usr/sbin/tc/tc-initdisk.sh %u", msg->u.init_payload.sys_part_capacity);
    k = system(system_buf);
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_INITIALIZE, 1 + WEXITSTATUS(k));
    break;
  }

#endif

#if !defined _WIN32 && !defined ANDROID
  case TCDMSG_UPDATEFIRMWARE:
  {
    int k;
    DEBUG((D_INFO, L"=======================>>>>> firmware update for local harddisk\n"));
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_UPDATEFIRMWARE, 0);
    k = system("/usr/sbin/tc/tc-upfw.sh");
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_UPDATEFIRMWARE, 1 + WEXITSTATUS(k));
    break;
  }
#endif

#if !defined _WIN32 && !defined ANDROID
  case TCDMSG_UPDATEFIRMWARE_IP:
  {
    int k;
    DEBUG((D_INFO, L"=======================>>>>> firmware update for local harddisk[%hs]\n", msg->u.uf_payload.ipv4));
    if (strcmp(msg->u.uf_payload.ipv4, localip)) {
      break;
    }
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_UPDATEFIRMWARE, 0);
    k = system("/usr/sbin/tc/tc-upfw.sh");
    notify_send(g_statusSock, g_ctrl_sock_addr, NF_UPDATEFIRMWARE, 1 + WEXITSTATUS(k));
    break;
  }
#endif

#if !defined _WIN32 && !defined ANDROID
  case TCDMSG_BOOT_IP:
  {
    char client_ip_buf[16];
    strncpy(client_ip_buf, inet_ntoa(*(struct in_addr *)&msg->u.bootinfo.clt_ip), 16);
    client_ip_buf[15] = '\0';
    DEBUG((D_INFO, L"=======================>>>>> boot(ip) message\n"));
    DEBUG((D_INFO, L"=======================>>>>> boot(ip) message info server_ip[%hs], server_port[%u], image_id[%u], image_org_name[%hs], image_name[%hs], image_per[%d], user_name[%hs], user_token[%hs], client_ip[%hs]\n",
      inet_ntoa(*(struct in_addr *)&msg->u.bootinfo.svrip),
      msg->u.bootinfo.svrport,
      msg->u.bootinfo.image_id,
      msg->u.bootinfo.image_org_name,
      msg->u.bootinfo.image_name,
      msg->u.bootinfo.image_per,
      msg->u.bootinfo.user_name,
      msg->u.bootinfo.user_token,
      client_ip_buf/*inet_ntoa(*(struct in_addr *)&msg->u.bootinfo.clt_ip)*/
      ));

    //if (inet_addr(localip) == msg->u.bootinfo.clt_ip)
    {
      // TO DO
      int tmp;
      char cmdbuf[256];
      bzero(cmdbuf, sizeof(cmdbuf));
#define SCRIPT_LOGIN "/usr/sbin/tc/tc-login.sh"
      sprintf(cmdbuf, SCRIPT_LOGIN" %u %hs %hs %d %hs %hs",
        msg->u.bootinfo.image_id,
        msg->u.bootinfo.image_org_name,
        msg->u.bootinfo.image_name,
        msg->u.bootinfo.image_per,
        msg->u.bootinfo.user_name,
        msg->u.bootinfo.user_token
      );
      DEBUG((D_INFO, L"cmd is \"%hs\"\n", cmdbuf));
      tmp = system(cmdbuf);
      tmp = WEXITSTATUS(tmp);
      if (tmp != 0)
      {
        DEBUG((D_INFO, L"fail to run script \"SCRIPT_LOGIN\", exit code[%d]\n", tmp));
      }
    }
    break;
  }
#endif

  case TCDMSG_CGSVRIP:
  {
    char cgsbuf[256];
    uint32 ip = msg->u.cgs.srvip;
    DEBUG((D_INFO, L"=======================>>>>> change server IP cache in local client [%hs]\n", inet_ntoa(*(struct in_addr *)&ip)));
#if !defined _WIN32 && !defined ANDROID
    sprintf(cgsbuf, "/usr/sbin/tc/servip_trigger.sh %hs", inet_ntoa(*(struct in_addr *)&ip));
    system(cgsbuf);
#endif      
    break;
  }

  case TCDMSG_CGSVRIP_IP:
  {
    char cgsbuf[256];
    uint32 ip = msg->u.cgsip.srvip;
    DEBUG((D_INFO, L"=======================>>>>> change server IP cache in local client [%hs]\n", inet_ntoa(*(struct in_addr *)&ip)));
#if !defined _WIN32 && !defined ANDROID
    sprintf(cgsbuf, "/usr/sbin/tc/servip_trigger.sh %hs", inet_ntoa(*(struct in_addr *)&ip));
    system(cgsbuf);
#endif
    break;
  }

  case TCDMSG_SET_RESOLUTION_IP:
  {
    char resolbuf[256];
    DEBUG((D_INFO, L"=======================>>>>> set client screen resolution\n"));
#if defined(_WIN32)
    gnScreenWidth = msg->u.set_resolution_ip_payload.screen_width;
    gnScreenHeight = msg->u.set_resolution_ip_payload.screen_height; 
    gbCmdToSetResolution = 1;
#endif
#if !defined _WIN32 && !defined ANDROID
    sprintf(resolbuf, "echo -n \"%dx%d\" >/mnt/disk/resolution.config", msg->u.set_resolution_ip_payload.screen_width, msg->u.set_resolution_ip_payload.screen_height);
    system(resolbuf);
#endif
    break;
  }

  case TCDMSG_USB_STORAGE_CTRL_IP:
  {
    char usbbuf[256];
    DEBUG((D_INFO, L"=======================>>>>> set client screen resolution\n"));
#if defined(_WIN32)
    gnUsbAccessValue = msg->u.usb_storage_ctrl_ip_payload.usb_enabled; 
    gbCmdToSetUsbStorageAccess = 1;
#endif
#if !defined _WIN32 && !defined ANDROID
    sprintf(usbbuf, "echo -n \"%d\" >/mnt/disk/usbstorage.config", msg->u.usb_storage_ctrl_ip_payload.usb_enabled);
    system(usbbuf);
#endif
    break;
  }

  default:
    //DEBUG((D_ERROR, L"unknown tcdmsg type @ client: [%d]\n", msg->type));
    break;
  }

  return 0;
}

int join_skt_group()
{

  int sock;
  struct sockaddr_in localsock;
  struct ip_mreq group;
  int reuse = 1;
  int nSendBuf = SND_BUF_SZ;
  int nRecvBuf = RCV_BUF_SZ; // something like 64*1024, defined in conf.h

  // DEBUG((D_INFO, L"outbounding via [%hs] with local ip [%hs]\n", cifn, localip));

  // to create the data socket. the hello packet will also be from this socket. 
  if ((sock = (int)socket(AF_INET, SOCK_DGRAM, 0)) < 0)
  {
    DEBUG((D_ERROR, L"socket\n"));
	return -1;
  }
  DEBUG((D_INFO, L"tcdclt: socket successfully ...\n"));


  if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse, sizeof(reuse)) < 0)
  {
    DEBUG((D_ERROR, L"setting SO_REUSEADDR\n"));
#ifdef _WIN32
    closesocket(sock);
#else
    close(sock);
#endif
    return -1;
  }
  DEBUG((D_INFO, L"tcdclt: setsockopt SO_REUSEADDR successfully\n"));


  if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (const char *)&nSendBuf, sizeof(int)) < 0)
  {
    DEBUG((D_ERROR, L"setting socket buffer\n"));
#ifdef _WIN32
    closesocket(sock);
#else
    close(sock);
#endif
    return -1;
  }

  if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (const char *)&nRecvBuf, sizeof(int)) < 0)
  {
    DEBUG((D_ERROR, L"setting socket buffer\n"));
#ifdef _WIN32
    closesocket(sock);
#else
    close(sock);
#endif

    return -1;
  }
  DEBUG((D_INFO, L"tcdsrv: set socket buffer to 32K successfully ...\n"));

  memset((char *)&localsock, 0, sizeof(localsock));
  localsock.sin_family = AF_INET;
  localsock.sin_port = htons(GROUP_PORT);
  localsock.sin_addr.s_addr = INADDR_ANY;

  if (bind(sock, (struct sockaddr *)&localsock, sizeof(localsock)))
  {
    DEBUG((D_ERROR, L"bind()\n"));
#ifdef _WIN32
    closesocket(sock);
#else
    close(sock);
#endif

    return -1;
  }
  DEBUG((D_INFO, L"tcdclt: bind successfully\n"));


  // add the local interface to multicast group
  group.imr_multiaddr.s_addr = inet_addr(GROUP_IP);
  group.imr_interface.s_addr = inet_addr(localip);
  if (setsockopt(sock, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&group, sizeof(group)) < 0)
  {
    DEBUG((D_ERROR, L"adding multicast group error\n"));
#ifdef _WIN32
    closesocket(sock);
#else
    close(sock);
#endif

    return -1;
  }
  DEBUG((D_INFO, L"dskt: add to multicast group successfully\n"));
  return sock;
}


#if !defined _WIN32 && !defined ANDROID
int notify_send(int socket, struct sockaddr_in sain, unsigned int nf_code, unsigned int nf_comment) {
  char buf[BUFSZ] = { 0 };
  TCDMSG *msg = (TCDMSG *)buf;
  msg->type = TCDMSG_NOTIFY;
  msg->size = BUFSZ;
  msg->u.nf.nf_code = nf_code;
  msg->u.nf.nf_comment = nf_comment;

  if (sendto(socket, buf, BUFSZ, 0, (struct sockaddr *)&sain, sizeof(sain)) == -1) {
    DEBUG((D_ERROR, L"sendto(): notify_send\n"));
    return -1;
  }
  return 0;
}
#endif

/* client FSM main routine:
   cskt: the control socket. It's used to talk with server's control plane.
   dskt: the data socket. It's used to receive server's data plane packets.
   servadd: server's control plane information
 */
int clt_fsm_main(int groupSock, int statusSock, struct sockaddr_in ctrl_sock_addr, struct sockaddr_in group_sock_addr, CCB *ccbp)
{
  fd_set rdfds;
  struct timeval tv;
  struct sockaddr_in sain;
  int sain_len;
  int res;
  int max_sockfd;
  char ch;
  char buf[BUFSZ]; // buf can not be less than the packet size
  double     time_consumed;
#ifdef _WIN32
  time_t time_start, time_end;
#else
  struct timeval time_start, time_end;
#endif

  // send status info periodically
#if STATUS_CYCLICAL
//#ifdef _WIN32
  time_t st_tm_last;
  time_t st_tm_curr;
  int st_first_flag = 1;
  //#endif // _WIN32
#endif // STATUS_CYCLICAL

#if !defined _WIN32 && !defined ANDROID
  int local_notify_sock = get_sock_local();
#endif
  // struct termios new_settings, stored_settings;

    //DEBUG((D_INFO, L"BUFSZ = %d\n", BUFSZ));
  DEBUG((D_INFO, L"enter clt_fsm_main() loop ...\n"));


#if ENABLE_HB
#ifdef _WIN32
  time(&time_start);
#else
  gettimeofday(&time_start, 0);
#endif
#endif

#if STATUS_CYCLICAL
  //#ifdef _WIN32
  time(&st_tm_last);
  //#endif // _WIN32
#endif // STATUS_CYCLICAL

  // the while deadloop(), which can only be out if 
  //  clt_cmdline_proc() returns -1, or 
  //   proc_tcdmsg_clt() returns -1. 
  while (!g_exit)
  {    // add network socket and stdin to the FDSET
    FD_ZERO(&rdfds);
    FD_SET(groupSock, &rdfds);
    FD_SET(g_clt_sock, &rdfds);
    max_sockfd = groupSock > g_clt_sock ? groupSock : g_clt_sock;
#if !defined _WIN32 && !defined ANDROID
    if (local_notify_sock > 0) {
      FD_SET(local_notify_sock, &rdfds);
      max_sockfd = max_sockfd > local_notify_sock ? max_sockfd : local_notify_sock;
    }
#endif
    // and set the select timeout to 3s
    tv.tv_sec = 3;
    tv.tv_usec = 0; //DEFAULT_CLT_PKTACK_LOSS_TIMEOUT;

    res = select(max_sockfd + 1, &rdfds, NULL, NULL, &tv);

    if (res > 0)
    {

      memset(buf, 0, sizeof(buf));
      // socket events handling
      if (FD_ISSET(groupSock, &rdfds))
      {
        // receiving data packets until the last packet ...

        // change here for making it work in uclibc platform
        // if (recvfrom(sock, buf, sizeof(buf), 0, (struct sockaddr *)&sain, &sain_len) != -1)
        if (recv(groupSock, buf, sizeof(buf), 0) != -1)
        {
          if (proc_tcdmsg_clt(groupSock, buf, ctrl_sock_addr, group_sock_addr, ccbp) == -1)
            break;
        }
        else
        {
          DEBUG((D_ERROR, L"clt_fsm_main: recv failed!\n"));
#ifdef _WIN32
          DEBUG((D_ERROR, L"clt_fsm_main: exit code: 0x%x\n", GetLastError()));
#else
          DEBUG((D_ERROR, L"clt_fsm_main: exit code: 0x%x\n", errno));
#endif                    
        }
      }
      // socket events handling
      else if (FD_ISSET(g_clt_sock, &rdfds))
      {
        if (recv(g_clt_sock, buf, sizeof(buf), 0) != -1)
        {
          if (proc_tcdmsg_clt(groupSock, buf, ctrl_sock_addr, group_sock_addr, ccbp) == -1)
            break;
        }
        else
          DEBUG((D_ERROR, L"clt_fsm_main: recv from g_clt_sock failed!\n"));
      }
#if !defined _WIN32 && !defined ANDROID
      else if (FD_ISSET(local_notify_sock, &rdfds))
      {
        if (recv(local_notify_sock, buf, sizeof(buf), 0) != -1)
        {
          struct local_notify_msg *notify_msg = (struct local_notify_msg *) buf;
          switch (notify_msg->type) {
          case LOCAL_MSG_POWEROFF:
            DEBUG((D_INFO, L"received poweroff msg from local socket\n"));
            actusr_send(statusSock, ctrl_sock_addr, "", "", ACT_SHUTTING_DOWN);
            break;
          case LOCAL_MSG_REBOOT:
            DEBUG((D_INFO, L"received reboot msg from local socket\n"));
            actusr_send(statusSock, ctrl_sock_addr, "", "", ACT_SHUTTING_DOWN);
            break;
          default:
            DEBUG((D_INFO, L"received unknown msg from local socket\n"));
            break;
          }
        }
        else
          DEBUG((D_ERROR, L"clt_fsm_main: recv from local_notify_sock failed!\n"));
      }
#endif
    }
    else if (res == 0)
    {
      // DEBUG((D_INFO, L"clt_fsm_main: select timeout\n"));
    }
    else
    {
      DEBUG((D_ERROR, L"clt_fsm_main: select()\n"));
    }

#if ENABLE_HB
#ifdef _WIN32
    time(&time_end);
    //DEBUG((D_INFO, L"time start is %lld, time end is %lld\n", time_start, time_end));
    time_consumed = difftime(time_end, time_start);
#else
    gettimeofday(&time_end, 0);
    time_consumed = time_end.tv_sec - time_start.tv_sec;
    //DEBUG((D_INFO, L"time start is %lld, time end is %lld\n", time_start, time_end));
#endif
  //time_consumed = 1000000*(time_end.tv_sec - time_start.tv_sec) + \
    //          time_end.tv_usec - time_start.tv_usec;
  //if(time_consumed >= HB_TIME_INTERVAL * 1000000) {
    if (time_consumed >= HB_TIME_INTERVAL) {
      DEBUG((D_INFO, L"send heart beat at %f s\n", time_consumed));
      actusr_send(statusSock, ctrl_sock_addr, "", "", ACT_HEARTBEAT);
#ifdef _WIN32
      time(&time_start);
#else
      gettimeofday(&time_start, 0);
#endif
    }
#endif

#if STATUS_CYCLICAL
    //#ifdef _WIN32
    // report for the first time without checking interval
    //
    if (st_first_flag)
    {
      DEBUG((D_INFO, L"send status:      status[%d],uname[%hs],osid[%hs],disk[%d],mem[%d],cpu[%hs]\n", g_st_statusid, g_st_uname, g_st_imageid, g_disksz, g_memsz, g_strcpu));
      st_first_flag = 0;
    }
    time(&st_tm_curr);
    if (st_tm_curr - st_tm_last >= ST_INTERVAL)
    {
      //
      // comment the following statement to if you would not like the message be shown
      //
      //DEBUG((D_INFO, L"send status[%d s]: status[%d],uname[%hs],osid[%hs],disk[%d],mem[%d],cpu[%hs]\n", st_tm_curr - st_tm_last, g_st_statusid, g_st_uname, g_st_imageid, g_disksz, g_memsz, g_strcpu));
      actusr_send(statusSock, ctrl_sock_addr, g_st_uname, g_st_imageid, ACT_RUNNING);
      time(&st_tm_last);
    }
    //#endif // _WIN32
#endif // STATUS_CYCLICAL

  }
#ifdef _WIN32
  if (g_exit == TC_LOCAL_MSG_SHUTDOWN)
  {
    actusr_send(statusSock, ctrl_sock_addr, g_st_uname, g_st_imageid, ACT_SHUTTING_DOWN);
  }
#endif // _WIN32

#if !defined _WIN32 && !defined ANDROID
  close_sock_local();
  local_notify_sock = -1;
#endif
  return 0;
}

void tcdclt_usage(char *argv0)
{
  PRINT(L"\nUsage: ");

#ifdef POST_REPORT_HWINFO
  PRINT(L"\n\t%hs <statusSrvIp> <interf> <statusPort> [statusStr [diskSz [memSz [cpuModel]]]]\n\t\t--- from outbounding interface <interf>", argv0);
#else
  PRINT(L"\n\t%hs <statusSrvIp> <interf> <statusPort> [statusStr]\n\t\t--- from outbounding interface <interf>", argv0);
#endif //POST_REPORT_HWINFO
  PRINT(L"\n\t\t    example: %hs  192.168.0.1 eth0 12345", argv0);


  PRINT(L"\nArguments: ");
  PRINT(L"\n\t<statusSrvIp>:\tthe ip of the server managing client status, will use ip %hs if it was set to 0", CTRL_SRV_IP);
  PRINT(L"\n\t<interf>:\tinbounding interface like eth0, eth1, lo");
  PRINT(L"\n\t<statusPort>:  \tstatus server's listening UDP port");
  PRINT(L"\n\t[statusStr]:\toptional, client status string defined as \"status:userid:imageid\", such as \"1:2:3\"");
#ifdef POST_REPORT_HWINFO
  PRINT(L"\n\t[diskSz]:\toptional, disk size in MiB");
  PRINT(L"\n\t[memSz]:\toptional, memory size in MiB");
  PRINT(L"\n\t[cpuModel]:\toptional, a string for the CPU\n");
#endif
  return;
}


void cleanup(int code)
{
  if (code == SIGUSR1 ||
    code == SIGTERM ||
    code == SIGINT) {
    DEBUG((D_INFO, L"Clean up code received: %d\n", code));
    if (g_groupSock > 0 && g_statusSock > 0)
    {
      actusr_send(g_statusSock, g_ctrl_sock_addr, "", "", ACT_SHUTTING_DOWN);
      DEBUG((D_INFO, L"userid is , imageId is , statusId is ACT_SHUTTING_DOWN:%d\n", ACT_SHUTTING_DOWN));
    }
    g_exit = 1;
  }
}


#ifdef _WIN32
DWORD WINAPI ServiceWorkerThread(LPVOID pparms)
#else

// format: tcdclt <ip> <port>, ip and port are the target server's IP address and port#
int ServiceWorkerThread(void *pparms)
#endif
{
  // the entity of the client control block
  // the definition of CCB is in tcdmsg.h
  int i;
  int ret = 0;
  int argc;
  char **argv;
  CCB ccb;
  char *ctlStr = NULL;
  char *dstip_str = NULL;
  char dst_ip_in_file[IP_BUF_SZ] = { 0 };
  char statusStr[UNAME_SZ];
  char userId[UNAME_SZ];
  char imageId[OSNAME_SZ];
  int statusId;
  unsigned long nTryFetchIp;
  TH_PARMS * p_thparms;

  //struct sockaddr_in servadd;
  //char buf[2048];
  struct sockaddr_in group_sock_addr;

#ifdef POST_REPORT_HWINFO
  int disksz;
  int memsz;
  char strcpu[CPU_STR_SZ];
#endif

#ifdef _WIN32
  WORD wVersionRequested;
  WSADATA wsaData;
#endif

  int dstport = 0;
  uint32 dstip = 0;

  if (!pparms) {
    DEBUG((D_INFO, L"parms is NULL\n;"));
    ret = -1;
    goto sys_exit;
  }

  p_thparms = (TH_PARMS *)pparms;

  argc = p_thparms->argc;
  argv = p_thparms->argv;

  PRINT(L"%hs:%d: argc=%d\n", __FUNCTION__, __LINE__, argc);
  DEBUG((D_INFO, L"%hs:%d: argc=%d\n", __FUNCTION__, __LINE__, argc));
  for (i = 0; i < argc; i++)
  {
    PRINT(L"%hs:%d: argv[%d]=%hs\n", __FUNCTION__, __LINE__, i, argv[i]);
    DEBUG((D_INFO, L"%hs:%d: argv[%d]=%hs\n", __FUNCTION__, __LINE__, i, argv[i]));
  }

#ifdef _WIN32
  wVersionRequested = MAKEWORD(2, 0);
  if (WSAStartup(wVersionRequested, &wsaData) != 0)
  {
    DEBUG((D_ERROR, L"Failed to start up winsock: %d\n", GetLastError()));
    ret = -1;
    goto sys_exit;
  }
#endif

  // parameter handling
#ifdef POST_REPORT_HWINFO
  if (argc < 4 || argc > 8)
#else
  if (argc < 4 || argc > 5)
#endif
  {
    tcdclt_usage(argv[0]);
    ret = -1;
    goto sys_exit;
  }

  cifn = (char *)calloc(64, sizeof(char));
  if (!cifn) {
    DEBUG((D_ERROR, L"Failed to allocate mem for cifn\n"));
    ret = -1;
    goto sys_exit;
  }

  dstip_str = argv[1];

  strcpy(cifn, argv[2]);
  //cifn = argv[2];
  dstport = atoi(argv[3]);

  // if run as a message receiver and message sender
  // no matter standalone application or a daemon
  // get the status configuration file
  if (argc <= 4)
  {
    int t;
	DEBUG((D_INFO, L"to load status file...\n"));
    t = load_status_conf(STATUS_CONFIG_FILE);
    DEBUG((D_INFO, L"load_status_conf return %d\n", t));
  }
#ifdef POST_REPORT_HWINFO
  // initialization
  disksz = 0;
  memsz = 0;
  memset(strcpu, 0, sizeof(strcpu));
#endif
  if (argc > 4) {
    ctlStr = argv[4];
    if (_parse_info(ctlStr, statusStr, 0, userId, 0, imageId, 0) != 0) {
      DEBUG((D_ERROR, L"status string should be \"status:userid:imageid\", such as \"1:2:3\"\n"));
      ret = -1;
      goto sys_exit;
    }
    statusId = atoi(statusStr);
  }
#ifdef POST_REPORT_HWINFO
  if (argc > 5)  // disk size, in MiB
  {
    disksz = atoi(argv[5]);
    g_disksz = disksz;
  }
  if (argc > 6)  // memory size, in MiB
  {
    memsz = atoi(argv[6]);
    g_memsz = memsz;
  }
  if (argc > 7)  // CPU Model string
  {
    strncpy(strcpu, argv[7], sizeof(strcpu));
    strcpu[sizeof(strcpu) - 1] = '\0';

    strcpy(g_strcpu, strcpu);
  }
#endif // REPORT_HWINFO

  if (strcmp(dstip_str, "0") == 0) {
    //#ifdef _WIN32
    if (ParseIpConfig(dst_ip_in_file, IP_BUF_SZ) == 0) {
      dstip_str = dst_ip_in_file;
      DEBUG((D_INFO, L"successfully get ip from local disk: %hs\n", dstip_str));
    }
    else {
      dstip_str = CTRL_SRV_IP;
      DEBUG((D_ERROR, L"fail to get ip from local disk, will use %hs instead\n", dstip_str));
    }
    //#else
    //    dstip_str = CTRL_SRV_IP;
    //#endif
  }


#ifndef _WIN32
  setup_sighandler(cleanup);
#endif
  //#ifdef _WIN32
    //SetConsoleCtrlHandler(console_ctrl_handler, TRUE);
  //#endif

    //DEBUG((D_INFO, L"flw_dir=<%hs>\n", flw_dir));

  //
  // try to fetch ip
  //
  nTryFetchIp = 0;
  while (((localip = ifn2ip_str(cifn)) == NULL) ||
    (strcmp(localip, "") == 0) ||
    (strcmp(localip, "127.0.0.1") == 0) ||
    (strncmp(localip, "169.254.", 8) == 0))
  {
    if (localip == NULL)
      DEBUG((D_ERROR, L"ifn2ip_str\n"));
    else
    {
      free(localip);
      localip = NULL;
    }

    if (g_exit) {
      ret = -1;
      goto sys_exit;
    }

    //
    // print the information for how many times, 5?
    //
	if (nTryFetchIp < 5)
    {
      DEBUG((D_ERROR, L"Failed getting local ip of %hs, will try again in 3s later\n", cifn));
    }
#ifdef _WIN32
    Sleep(3000);
#else
    sleep(3);
#endif

	nTryFetchIp++;
  }

  PRINT(L"TC Control Client\n");
  PRINT(L"==========>>>>>>> Interface: %hs\n", cifn);
  PRINT(L"==========>>>>>>> local ip: %hs\n", localip);
  PRINT(L"==========>>>>>>> status server ip: %hs[%d]\n", dstip_str, dstport);
  PRINT(L"==========>>>>>>> group server ip: %hs[%d]\n", GROUP_IP, GROUP_PORT);

  DEBUG((D_INFO, L"TC Control Client\n"));
  DEBUG((D_INFO, L"==========>>>>>>> Interface: %hs\n", cifn));
  DEBUG((D_INFO, L"==========>>>>>>> local ip: %hs\n", localip));
  DEBUG((D_INFO, L"==========>>>>>>> status server ip: %hs[%d]\n", dstip_str, dstport));
  DEBUG((D_INFO, L"==========>>>>>>> group server ip: %hs[%d]\n", GROUP_IP, GROUP_PORT));

  if ((localmac = (unsigned char *)ifn2mac(cifn, localip)) == NULL)
  {
    DEBUG((D_ERROR, L"ifn2mac\n"));
    ret = -1;
    goto sys_exit;
  }


  PRINT(L"==========>>>>>>> eth mac: %02x-%02x-%02x-%02x-%02x-%02x\n", localmac[0], localmac[1], localmac[2], localmac[3], localmac[4], localmac[5]);
  DEBUG((D_INFO, L"==========>>>>>>> eth mac: %02x-%02x-%02x-%02x-%02x-%02x\n", localmac[0], localmac[1], localmac[2], localmac[3], localmac[4], localmac[5]));


  dstip = inet_addr(dstip_str);

  // to create the data socket. the hello packet will also be from this socket. 
  if ((g_statusSock = (int)socket(AF_INET, SOCK_DGRAM, 0)) < 0)
  {
    DEBUG((D_ERROR, L"Status socket\n"));
    ret = -1;
    goto sys_exit;
  }

  memset((char *)&g_ctrl_sock_addr, 0, sizeof(g_ctrl_sock_addr));
  g_ctrl_sock_addr.sin_family = AF_INET;
  g_ctrl_sock_addr.sin_addr.s_addr = inet_addr(dstip_str);
  g_ctrl_sock_addr.sin_port = htons((u_short)dstport);

  // PRINT(L"outbounding via [%hs] with local ip [%hs]\n", cifn, localip);

  if (ctlStr == NULL) {
    // to create the data socket. the hello packet will also be from this socket. 
    if ((g_groupSock = join_skt_group()) < 0)
    {
      DEBUG((D_ERROR, L"socket\n"));
#ifdef _WIN32
      closesocket(g_statusSock);
#else
      close(g_statusSock);
#endif
      ret = -1;
      goto sys_exit;
    }

    // to create the group_sock_addr and ctrl_sock_addr, used for multicast and unicast packet sending. 
    // setup the multicast group socket

    memset((char *)&group_sock_addr, 0, sizeof(group_sock_addr));
    group_sock_addr.sin_family = AF_INET;
    group_sock_addr.sin_addr.s_addr = inet_addr(GROUP_IP);
    group_sock_addr.sin_port = htons(GROUP_PORT);


    // create a socket to receive unicast messages
    if ((g_clt_sock = (int)socket(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
      DEBUG((D_ERROR, L"Failed creating g_clt_sock\n"));
      ret = -1;
      goto sys_exit;
    }

    memset((char *)&g_clt_sock_addr, 0, sizeof(g_clt_sock_addr));
    g_clt_sock_addr.sin_family = AF_INET;
    g_clt_sock_addr.sin_port = htons(CLT_PORT);
    //g_clt_sock_addr.sin_addr.s_addr = inet_addr(localip);
    g_clt_sock_addr.sin_addr.s_addr = INADDR_ANY;

    if (bind(g_clt_sock, (struct sockaddr *)&g_clt_sock_addr, sizeof(g_clt_sock_addr)) < 0) {
      DEBUG((D_ERROR, L"bind() g_clt_sock\n"));
#ifdef _WIN32            
      DEBUG((D_ERROR, L"bind() error code : 0x%x\n", WSAGetLastError()));
#else
      DEBUG((D_ERROR, L"bind() error code : 0x%x\n", errno));
      DEBUG((D_ERROR, L"bind() error string: %hs\n", strerror(errno)));
#endif

#ifdef _WIN32
      closesocket(g_clt_sock);
#else
      close(g_clt_sock);
#endif
      g_clt_sock = -1;
      ret = -1;
      goto sys_exit;
    }

    // the initialize the client control block
    init_ccb(&ccb);

    // to start the client FSM
    clt_fsm_main(g_groupSock, g_statusSock, g_ctrl_sock_addr, group_sock_addr, &ccb);
#ifdef _WIN32
    closesocket(g_groupSock);
    closesocket(g_clt_sock);
#else
    close(g_groupSock);
    close(g_clt_sock);
#endif
  }
  else {
    actusr_send(g_statusSock, g_ctrl_sock_addr, userId, imageId, statusId);
    DEBUG((D_INFO, L"userid is %hs, imageId is %hs, statusId is %d\n", userId, imageId, statusId));
  }

  SET_CLT_STATE((&ccb), CLT_STATE_STOP);

sys_exit:
  if (localmac)
    free(localmac);

  if (cifn)
    free(cifn);

  // to close the control socket  
  DEBUG((D_INFO, L"tcdclt: close the socket\n"));
#ifdef _WIN32
  if (g_statusSock > 0)
    closesocket(g_statusSock);
  WSACleanup();
#else
  if (g_statusSock > 0)
    close(g_statusSock);
#endif

  g_exit = 1;


  DEBUG((D_INFO, L"Quiting\n"));

#ifdef _WIN32
  DEBUG((D_INFO, L"sending wm_close\n"));
  SendMessage(hWnd, WM_CLOSE, 0, 0);
#endif

  return ret;
}

