/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include <locale.h>
#include <wchar.h>
#include <stdio.h>
#include <stdlib.h>
#include "TcConfig.h"
#include "TcDebugLib.h"
#include "TcCtl.h"

int setup_sighandler(P_SIG_HANDLER psh)
{
	if(signal(SIGINT, psh) == SIG_ERR) {
		PRINT(L"signal error (%hs)\n", strerror(errno));
		return -1;
	}

	if(signal(SIGTERM, psh) == SIG_ERR) {
		PRINT(L"signal error (%hs)\n", strerror(errno));
		return -1;
	}

	if(signal(SIGUSR1, psh) == SIG_ERR) {
		PRINT(L"signal error (%hs)\n", strerror(errno));
		return -1;
	}

	return 0;
}


#ifdef WIN32
static BOOL WINAPI Console_ctrl_handler(DWORD dwCtrlType)
{
	DEBUG((D_INFO, L"console code : %d received\n", dwCtrlType));
	if(dwCtrlType == CTRL_C_EVENT ||
	    dwCtrlType == CTRL_BREAK_EVENT ||
	    dwCtrlType == CTRL_CLOSE_EVENT )
	{
	    return TRUE;
	}
  
  	return FALSE;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    PAINTSTRUCT ps;
    HDC hdc;
    switch(message)
    {
        case WM_QUERYENDSESSION:
			DEBUG((D_INFO, L"going to log off\n"));
			cleanup(SIGUSR1);
            return FALSE;
        case WM_PAINT:
            hdc = BeginPaint(hWnd, &ps);
            EndPaint(hWnd, &ps);
            break;
        case WM_CREATE:
            ShutdownBlockReasonCreate(hWnd, L"Sending messge");
            break;
        case WM_DESTROY:
            ShutdownBlockReasonDestroy(hWnd);
            PostQuitMessage(0);
            break;
        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
    }
    return 0;
}

ATOM RegisterTcClass(HINSTANCE hInstance)
{
    WNDCLASSEX tcex;
    tcex.cbSize = sizeof(WNDCLASSEX);
    tcex.style = CS_HREDRAW | CS_VREDRAW;
    tcex.lpfnWndProc = WndProc;
    tcex.cbClsExtra = 0;
    tcex.cbWndExtra = 0;
    tcex.hInstance = hInstance;
    tcex.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    tcex.hCursor = LoadCursor(NULL, IDC_ARROW);
    tcex.hbrBackground = (HBRUSH)(COLOR_WINDOW + 1);
    tcex.lpszMenuName = NULL;
    tcex.lpszClassName = "TcClass";
    tcex.hIconSm = NULL;
    return RegisterClassEx(&tcex);
}

BOOL CreateTcInstance(HINSTANCE hInstance, int cmdShow)
{
    RECT sz = {0, 0, 512, 512};
	
	hInst = hInstance;
    AdjustWindowRect(&sz, WS_OVERLAPPEDWINDOW, TRUE);
    hWnd = CreateWindow("TcClass", "TC ctrl window", WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, sz.right - sz.left, sz.bottom - sz.top,
        NULL, NULL, hInstance, NULL);
    if(!hWnd)
    {
        return FALSE;
    }

    ShowWindow(hWnd, cmdShow);
    UpdateWindow(hWnd);
    return TRUE;
}

#endif

#ifdef WIN32
int main(int argc, char **argv)
{
	HANDLE hThread;
	TH_PARMS thparms;
	MSG msg = {0};

    HINSTANCE hInstance = GetModuleHandle(NULL);

	SetConsoleCtrlHandler(Console_ctrl_handler, TRUE);

    RegisterTcClass(hInstance);
    if(!CreateTcInstance(hInstance, SW_HIDE))
    {
        return FALSE;
    }

	thparms.argc = argc;
	thparms.argv = argv;
	
	hThread = CreateThread (NULL, 0, ServiceWorkerThread, &thparms, 0, NULL);

	if(hThread == NULL) 
	{
		DEBUG((D_ERROR, L"Failed to create thread\n"));
		return FALSE;
	}
	
    while(GetMessage(&msg, NULL, 0, 0))
    {
    	//TC_DBG("get message: %d\n",(int)msg.wParam);
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

	CloseHandle(hThread);
/*
	while(!g_exit)
    {
        Sleep(1000);
    }
  */  
    return (int)msg.wParam;
}
#else
int main(int argc, char **argv)
{
    int ret;
	TH_PARMS thparms;

    setlocale(LC_CTYPE, "");
    DEBUG_INIT(TO_STD_CONSOLE, NULL);
    PRINT_INIT(TO_STD_CONSOLE, NULL);

	thparms.argc = argc;
	thparms.argv = argv;
	ret = ServiceWorkerThread((void *)&thparms);

    DEBUG_FINISH();
    PRINT_FINISH();
    return ret;
}

#endif // WIN32

