/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
/** @file
  Test program
    A simple test program to demonstrate the use of tcdebug.h
  
  Build environment:
    Microsoft Windows 10 Ultimate 64-bit
    WinDDK, C:\WINDDK\7600.16385.1
    Microsoft Visual Studio 2015 update 3
  
  Build C Flags:
    _UNICODE
    _DEBUG
**/

#if 0
#define _CRT_SECURE_NO_WARNINGS
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <LmCons.h>

#include "TcConfig.h"    // this goes first
#include "TcDebugLib.h"
#include "TcNtLib.h"
#include "TcNtLib2.h"

#define DEBUG_FILE_PATH  L"C:\\TcDebug.txt"
#define PRINT_FILE_PATH  L"C:\\TcPrint.txt"

#pragma comment(lib, "AdvApi32.lib")

void test_debug_file()
{  
  DEBUG_INIT(TO_FILE, DEBUG_FILE_PATH);
  
  DEBUG((D_INFO, L"Hello, world\n"));
  DEBUG((D_WARNING, L"Hello, world\n"));
  DEBUG((D_ERROR, L"Hello, world\n"));
  
  DEBUG_FINISH();
}

void test_debug_std()
{
  DEBUG_INIT(TO_STD_CONSOLE, 0);
  
  DEBUG((D_INFO, L"Hello, world\n"));
  DEBUG((D_WARNING, L"Hello, world\n"));
  DEBUG((D_ERROR, L"Hello, world\n"));
  
  DEBUG_FINISH();
}

void test_debug_console()
{
  DEBUG_INIT(TO_CONSOLE, 0);
  
  DEBUG((D_INFO, L"Hello, world\n"));
  DEBUG((D_WARNING, L"Hello, world\n"));
  DEBUG((D_ERROR, L"Hello, world\n"));
  
  DEBUG_FINISH();
}

void test_print_file()
{  
  PRINT_INIT(TO_FILE, PRINT_FILE_PATH);
  PRINT(L"Hello, world\n");
  PRINT_FINISH();
}

void test_print_std()
{
  PRINT_INIT(TO_STD_CONSOLE, 0);
  PRINT(L"%hs\n", "Hello, world");
  PRINT(L"%ls\n", L"w Hello, world");
  PRINT_FINISH();
}

void test_print_console()
{
  PRINT_INIT(TO_CONSOLE, 0);
  PRINT(L"Hello, world\n");
  PRINT_FINISH();
}

/**
  test ntlib
  need to run as administrator to run this program
**/
void test_tcntlib(const wchar_t *wstrFile)
{
	char buf[1024];
	char strFile[256];
	int ret;

	ret = 0;

	PRINT_INIT(TO_STD_CONSOLE, 0);
	DEBUG_INIT(TO_STD_CONSOLE, 0);

	if (wstrFile == NULL || *wstrFile == L'\0')
	{
		DEBUG((D_ERROR, L"illegal arguments\n"));
	}
	PRINT(L"%hs\n", __FUNCTION__);

	PRINT(L"Usage:\n");
    PRINT(L"  ThisApp.exe \\Device\\Harddisk0\\Partition1\\TcReadTest.txt\n"); // ok
    PRINT(L"  ThisApp.exe \\Device\\HarddiskVolume1\\TcReadTest.txt\n"); // ok
    PRINT(L"  ThisApp.exe \\??\\C:\\TcReadTest.txt\n"); // ok

	memset(buf, 0, sizeof(buf));
	memset(strFile, 0, sizeof(strFile));
	wcstombs(strFile, wstrFile, sizeof(strFile) / sizeof(strFile[0]));

	DEBUG((D_INFO, L"wcs: %ls\n", wstrFile));
	DEBUG((D_INFO, L"mbs: %hs\n", strFile));
	ret = TcOneTimeNtReadFile(buf, sizeof(buf), strFile);
	DEBUG((D_ERROR, L"Don't worry, this is an test error message :)\n"));

	PRINT(L"ret = %d\n", ret);
	PRINT(L"buf:\n%hs\n", buf);

	PRINT_FINISH();
	DEBUG_FINISH();
}

void test_ntread2(LPCWSTR filepath)
{
	HANDLE hFile;
	DWORD  dwResult;
	BYTE   pBuffer[256];
	DWORD  dwCount;

	DEBUG_INIT(TO_CONSOLE, NULL);
	PRINT_INIT(TO_CONSOLE, NULL);

	ZeroMemory(pBuffer, _countof(pBuffer));

	dwResult = ntOpenFileW(&hFile, filepath, L"r");
	if (dwResult != ERROR_SUCCESS)
	{
		DEBUG((D_ERROR, L"%hs: ntOpenFileW\n", __FUNCTION__));
		goto exit1;
	}

	dwCount = _countof(pBuffer) - 1;
	PRINT(L"try to read at most %d bytes\n", dwCount);
	dwResult = ntReadFile(hFile, pBuffer, &dwCount);
	if (dwResult != ERROR_SUCCESS)
	{
		DEBUG((D_ERROR, L"%hs: ntReadFile\n", __FUNCTION__));
		goto exit2;
	}

	PRINT(L"buffer returned: %d bytes\n", dwCount);
	PRINT(L"%hs\n", pBuffer);

exit2:
	ntCloseFile(hFile);

exit1:
	DEBUG_FINISH();
	PRINT_FINISH();
}

void test_ntwrite2(LPCWSTR filepath)
{
	HANDLE hFile;
	DWORD  dwResult;
	BYTE   pBuffer[256] = "Hello, NtWriteFile";
	DWORD  dwCount;

	DEBUG_INIT(TO_CONSOLE, NULL);
	PRINT_INIT(TO_CONSOLE, NULL);

	dwResult = ntOpenFileW(&hFile, filepath, L"w");
	if (dwResult != ERROR_SUCCESS)
	{
		DEBUG((D_ERROR, L"%hs: ntOpenFileW\n", __FUNCTION__));
		goto exit1;
	}

	dwCount = (DWORD) strlen(pBuffer);
	PRINT(L"try to write %d bytes\n", dwCount);
	dwResult = ntWriteFile(hFile, pBuffer, dwCount);
	if (dwResult != ERROR_SUCCESS)
	{
		DEBUG((D_ERROR, L"%hs: ntWriteFile\n", __FUNCTION__));
		goto exit2;
	}

exit2:
	ntCloseFile(hFile);

exit1:
	DEBUG_FINISH();
	PRINT_FINISH();
}

int test_getusername()
{
	WCHAR wszUserName[UNLEN + 1];
	DWORD dwNameSize = _countof(wszUserName);
	DWORD ret = 1;

	DEBUG_INIT(TO_STD_CONSOLE, L"C:\\Test.log");
	DEBUG((D_INFO, L"%hs: dwNameSize = 0x%X\n", __FUNCTION__, wszUserName, dwNameSize));

	if (!GetUserName(wszUserName, &dwNameSize))
	{
		DEBUG((D_ERROR, L"%hs: GetUserName error: 0x%X\n", __FUNCTION__, GetLastError()));
		ret = 1;
		goto exit;
	}
	else
	{
		DEBUG((D_INFO, L"%hs: %ls\n", __FUNCTION__, wszUserName));
		DEBUG((D_INFO, L"%hs: dwNameSize = 0x%X\n", __FUNCTION__, wszUserName, dwNameSize));
		ret = 0;
		goto exit;
	}
exit:
	DEBUG_FINISH();
	return ret;
}

int __cdecl wmain(int argc, wchar_t *argv[])
{  
  /*DWORD a = MAX_PATH;
  test_debug_file();
  test_debug_std();  
  test_debug_console();
  test_print_file();
  test_print_std();
  test_print_console();
  test_tcntlib(L"\\Device\\Harddisk0\\Partition1\\TcReadTest.txt");
  test_ntread2(L"\\Device\\Harddisk0\\Partition1\\TcReadTest2.txt");
  test_ntwrite2(L"\\Device\\Harddisk0\\Partition1\\TcReadTest3.txt");
*/
	test_getusername();
  return 0;
}

#endif /* #if 0*/


#include <Windows.h>
#include <stdio.h>
#include "TcGetoptLib.h"
int __cdecl wmain(int argc, wchar_t *argv[])
{
	int a;


	
	scanf("%d", &a);
	return 0;
}
