/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "TcConfig.h"    // this goes first
#include "TcDebugLib.h"
#include "TcKeyValueLib.h"
//#include "TcNtLib.h"
//#include "TcNtLib2.h"

//#define DEBUG_FILE_PATH  L"C:\\TcDebug.txt"
//#define PRINT_FILE_PATH  L"C:\\TcPrint.txt"
#define DEBUG_FILE_PATH  L"/TcDebug.txt"
#define PRINT_FILE_PATH  L"/TcPrint.txt"

//#pragma comment(lib, "AdvApi32.lib")
extern
int
TC_DEBUG_LIB_API
TcDebugInit (
  OUTPUT_STREAM_ENUM streamEnumType,
  wchar_t *filename
  );

extern
int
TC_DEBUG_LIB_API
TcDebugOutput (
  const wchar_t *fmt,
  ...
  );

extern
int
TC_DEBUG_LIB_API
TcDebugFinish (
  void
  );

void test_debug_file()
{  
  DEBUG_INIT(TO_FILE, DEBUG_FILE_PATH);
  
  DEBUG((D_INFO, L"%ls", L"w Hello, world\n"));
  DEBUG((D_WARNING, L"Hello, world\n"));
  DEBUG((D_ERROR, L"Hello, world\n"));
  
  DEBUG_FINISH();
}

void test_debug_std()
{
  DEBUG_INIT(TO_STD_CONSOLE, 0);
  
  DEBUG((D_INFO, L"Hello, world\n"));
  DEBUG((D_WARNING, L"%hs", "h Hello, world\n"));
  DEBUG((D_ERROR, L"Hello, world\n"));
  
  DEBUG_FINISH();
}


void test_debug_console()
{
  DEBUG_INIT(TO_CONSOLE, 0);
  
  DEBUG((D_INFO, L"Hello, world\n"));
  DEBUG((D_WARNING, L"Hello, world\n"));
  DEBUG((D_ERROR, L"Hello, world\n"));
  
  DEBUG_FINISH();
}


void test_print_file()
{  
  PRINT_INIT(TO_FILE, PRINT_FILE_PATH);
  PRINT(L"%hs", "h Hello, world\n");
  PRINT(L"w Hello, world\n");
  //PRINT(L"%ls", L"Hello, world\n");
  PRINT_FINISH();
}

void test_print_std()
{
  PRINT_INIT(TO_STD_CONSOLE, 0);
  PRINT(L"%hs\n", "Hello, world, 你好，世界！");
  PRINT(L"%ls\n", L"w Hello, world");
  PRINT_FINISH();
}


void test_print_console()
{
  PRINT_INIT(TO_CONSOLE, 0);
  PRINT(L"Hello, world\n");
  PRINT_FINISH();
}

void
mytest_get_key_val();

void
test_get_key_val_sample_file(const char *filename, const char *delim);

void
test_get_key_val_from_file(const char *filename, const char *delim);


//int main(int argc, char *argv[])
int main(int argc, char *argv[])
{  
  setlocale(LC_ALL, "en_US.UTF-8");
  //setlocale(LC_ALL, "zh_CN.UTF-8");
  test_debug_file();
  test_debug_std();  
  test_debug_console();

  test_print_file();
  test_print_std();
  test_print_console();

  PRINT_INIT(TO_STD_CONSOLE, 0);

  PRINT(L"argv0 = %hs\n", argv[0]);
  //PRINT(L"argv1 = %hs\n", argv[1]);
  //mytest_get_key_val();
  PRINT(L"%ls\n", L"中文，你好。");

  PRINT_FINISH();

  return 0;
}

void
test_get_key_val_sample_file(const char *filename, const char *delim)
{
    FILE *fp;

    if (NULL == (fp = fopen(filename, "w")))
    {
        fprintf(stderr, "fail to open file \"%s\"\n", filename);
        exit(-1);
    }

    fprintf(fp, "\n");
    fprintf(fp, "\r\n");
    fprintf(fp, "key1\n");
    fprintf(fp, " key2\n");
    fprintf(fp, " key3 \n");
    fprintf(fp, " key 4 \n");
    fprintf(fp, "%s\n", delim);
    fprintf(fp, " %s\n", delim);
    fprintf(fp, "%s \n", delim);
    fprintf(fp, " %s \n", delim);
    fprintf(fp, "key5 %s \n", delim);
    fprintf(fp, " key5 %s \n", delim);
    fprintf(fp, " key 5 %s \n", delim);
    fprintf(fp, " %svalue6\n", delim);
    fprintf(fp, " %s value6\n", delim);
    fprintf(fp, " %s value6 \n", delim);
    fprintf(fp, " %s value 6 \n", delim);
    fprintf(fp, "key7%svalue7\n", delim);
    fprintf(fp, " key8%svalue8\n", delim);
    fprintf(fp, " key9 %svalue9\n", delim);
    fprintf(fp, " key 10 %svalue10\n", delim);
    fprintf(fp, " key 11 %s value11\n", delim);
    fprintf(fp, " key 12 %s value12 \n", delim);
    fprintf(fp, " key 13 %s value 13 \n", delim);
    fprintf(fp, " key 14 %s value 14 \r\n", delim);
    fprintf(fp, " key 15 %s value 15 ", delim);
    fprintf(fp, " key 16 %s value 16 ", delim);
    fflush(fp);

    if (fp != NULL)
    {
        fclose(fp);
        fp = NULL;
    }
}

void
mytest_get_key_val()
{
    const char buf[] = " adks al - = sl  ";
    const char *filename = "get_key_val_sample_file.conf";
    const char *delim = "=";

    //test_get_a_key_val(buf, "=");

    test_get_key_val_sample_file(filename, delim);
    test_get_key_val_from_file(filename, delim);
}

void
test_get_key_val_from_file(const char *filename, const char *delim)
{
    int n;
    struct config_data *pconf;

    pconf = NULL;
    n = get_key_val(filename, delim, &pconf);
    if (n <= 0)
    {
        return;
    }

    wprintf(L"n = %d\n", n);
    print_key_val(pconf);

    if (pconf != NULL)
    {
        free_key_val(pconf);
        pconf = NULL;
    }
}

