/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>

int do_read(int fd, int offset, char *buf, int size)
{
    ssize_t  bytes;

    if ( lseek(fd, offset, SEEK_SET) < 0 )
    {
        return errno;
    }

    bytes = read(fd, buf, size);
    if ( bytes != 4 )
    {
        return (bytes < 0) ? errno : EINVAL;
    }

    return 0;
}

int main(int argc, char *argv[])
{
    int  ret;
    int  fd, fd2;
    unsigned char  sig[4];
    int32_t  sectors;
    int64_t  offset;

    if ( argc < 3 )
    {
        printf("gendtag </dev/sda> </mnt/disk/pagefile.disk>\n");
        return EINVAL;
    }

    fd = open(argv[1], O_RDONLY);
    if ( fd < 0 )
    {
        ret = errno;
        printf("open %s failed: %d\n", argv[1], ret);
        goto out;
    }

    ret = do_read(fd, 0x1b8, sig, sizeof(sig));
    if ( ret != 0 )
    {
        printf("read hd sig failed: %d\n", ret);
        goto out;
    }

    ret = do_read(fd, (0x1be + 16*1 + 8), (char*)&sectors, sizeof(sectors));
    if ( ret != 0 )
    {
        printf("read p2 offset failed: %d\n", ret);
        goto out;
    }
    offset = (int64_t)sectors * 512;

    printf("sig=%02x%02x%02x%02x, offset=%016llx\n", sig[0], sig[1], sig[2], sig[3], offset);

    fd = open(argv[2], O_CREAT|O_TRUNC|O_WRONLY, 0444);
    if ( fd < 0 )
    {
        ret = errno;
        printf("open %s failed: %d\n", argv[2], ret);
        goto out;
    }

    write(fd, sig, sizeof(sig));
    write(fd, &offset, sizeof(offset));

    close(fd2);

out:
    close(fd);

    return ret;
}
