/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <linux/fs.h>
#include <errno.h>
#include <string.h>
#include <sys/sysinfo.h>
#include <sys/wait.h>

#include "conf.h"
#include "cltinfo.h"

#define SCRIPT_CPU_MODEL_NAME TCDCLT_SCRIPT_PATH"/cpumodelname.sh"

unsigned long int disk_capacity_mib(const char *dev_node){
    unsigned long int nblks = 0;
    int fd = open (dev_node, O_RDONLY);

    if (fd == -1){
        perror(dev_node);
        return 0;
    }

    ioctl (fd, BLKGETSIZE, &nblks);
    close (fd);

    return nblks / (2*1024);
}

unsigned long int phy_memory (){
    struct sysinfo sinfo;
    int ret;

    memset (&sinfo, 0, sizeof (char));
    ret = sysinfo (&sinfo);

    if (0 != ret){
        perror ("sysinfo()");
        return ret = 0;
    }

    ret = sinfo.totalram / 1024 / 1024 * sinfo.mem_unit;
    return ret;
}

/* on success, return 1, otherwise return 0 */
int cpu_model_name (char *buf, int sz){
    int ret;
    FILE *fp;
    char cmd[256];
    char cpufn[]="/tmp/cpumodelname.txt";

    sprintf (cmd, "%s %s", SCRIPT_CPU_MODEL_NAME, cpufn);

    ret = system (cmd);
    if (WEXITSTATUS (ret) != 0){
        fprintf (stderr, "fail to get cpu model name");
        return 0;
    }

    if ((fp = fopen (cpufn, "r")) == NULL ){
        perror (cpufn);
        return 0;
    }

    fgets (buf, sz, fp);

    fclose (fp);

    return 1;
}

/*
int main (int argc, char **argv){
    int ret;
    unsigned long int sz_mib = 0;
    unsigned long int sz_mem = 0;
    char buf[256];

    sz_mib = disk_capacity_mib (argv[1]);

    printf ("%s: %lu MiB (%lu GiB)\n", argv[1], sz_mib, sz_mib/1024);

    sz_mem = phy_memory ();
    printf ("%s: %lu MiB\n", "memory", sz_mem);

    ret = cpu_model_name (buf, 256);
    if (ret)
        printf ("CPU: %s\n", buf);

    return 0;
}
*/

