/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


// the configurable variables for the system

#ifndef _CONF_H_
#define _CONF_H_

// Check machine bits in windows
#if _WIN32 || _WIN64
 #if _WIN64
  #define MACHINE_X64
 #else
  #define MACHINE_X32
 #endif
#endif

// Check machine bits in Linux/GCC
#if __GNUC__
 #if __x86_64__ || __ppc64__
  #define MACHINE_X64
 #else
  #define MACHINE_X32
 #endif
#endif


// used for logo/slogan
#define AUTHOR "Wu Ming"
#define MACHINE	((sizeof(long) == 4)?"32-Bit System":"64-Bit System")
#define VERSION	"2.0 ALPHA"

// the multicast ipaddr and port for data packet transfer. 
#define GROUP_IP 	"226.1.1.1"
#define GROUP_PORT	20000

// the default control port. 
#define CTRL_PORT	12345

// usend/urecv port
#define USERVER_PORT	54321

// default port used by tcctl
#define CLT_PORT        12345

// to enable the system debugger by unmasking this macro
// #define _DEBUG_

// to use the system malloc()/free() by unmasking this macro. by default we prefer to use the vos_malloc()/vos_free() defined in vos_mem.c
#define USE_OS_MALLOC	1

// to return the brief packet acknowledgement message from client to server. 
// the advantage of using brief message is: save the network BW from client to server. 
// the disadvantage of using brief message is, server cannot selectively re-transmit the 
// missing slot, otherwise has to send all slots in the bulk. 
#define USE_PKTACK_WITH_BMAP 1

// whether to switch on/off the auto-adjustment of the inter-packet delay time 
// #define USE_PKTSND_AUTO_ADJUST	1


// this variable will be used in bmap.h
// # of slots for each bulk will be MAPSZ*64, since each uint64 word will stand for 64 slots, and there will be MAPSZ uint64
#define MAPSZ		1
// #define MAPSZ		2
// #define MAPSZ		4

// be used in vos_mem.c, how many memory blocks to be allocated
#define MEMBLKSZ	32





////////////////////////////////////////////////////////////////////////////////////////
// timeout configuration

// tick as the basic period ...
#define TIMEOUT_INTERVAL 100000	// alarm is the basic time period, about 0.1 second

// server side timeout ...
#define DEFAULT_SVR_CBULK_TIMEOUT 		(TIMEOUT_INTERVAL*5)	// CBULK timeout is about 0.5 second
#define DEFAULT_SVR_PKTACK_CHECK_CBULK_TIMEOUT 	(TIMEOUT_INTERVAL*12)	// PKTACK check timeout is about 1.28 second
#define DEFAULT_SVR_MAX_CBULK_TIMEOUT 		(TIMEOUT_INTERVAL*24)		
#define DEFAULT_SVR_HELLO_TIMEOUT 		(TIMEOUT_INTERVAL*20)	// hello broadcast peroid, set to ~2 seconds.

#define DEFAULT_SVR_ACTUSR_TIMEOUT 		(TIMEOUT_INTERVAL*50)	// hello broadcast peroid, set to ~5 seconds.

#define DEFAULT_SVR_USRSTATUS_CHECK	30	

// newly added for the bug found by wensheng
#define DEFAULT_SVR_READY_TIMEOUT		(TIMEOUT_INTERVAL*1200) // (TIMEOUT_INTERVAL*1200)

// client side timeout ...
#define DEFAULT_CLT_PKTACK_LOSS_TIMEOUT (TIMEOUT_INTERVAL*15)

// reg side timeout ...
#define DEFAULT_REG_TIMEOUT	(TIMEOUT_INTERVAL*30)		// 3 second

// end of timeout configuration
////////////////////////////////////////////////////////////////////////////////////////


// to print the instant sending/receiving rate
#define PRINT_INSTANT_SPEED 1

// to sample the sending/receiving every SAMPLE_FREQ bulks
#define SAMPLE_FREQ	8

// the buffer size of the sending/receiving buffer in multi-cast socket. 
//#define SND_BUF_SZ 128*1024
//#define RCV_BUF_SZ 128*1024

#define SND_BUF_SZ 1024*1024
#define RCV_BUF_SZ 1024*1024

#define SLOT_BYTES	1460	// 1024


// in order to limit the packet sending rate, we add a small delay between the sending of two intersecutive packets. 
// the variable intpkt_delay is used to measure the # of usecs for the inter-packet delay. 
// according to my experiment during the testing, 4~12 usec will be the effective value, so we use the 8 as the default value
// and set the range of this value between 1 to 24

// special for HCML
#define DEFAULT_INTPKT_DELAY	12

//#define DEFAULT_INTPKT_DELAY	4

// obsoleted ...
//#define MAX_INTPKT_DELAY 36
//#define MIN_INTPKT_DELAY 1

// parameter for different rate in 3F lab 26 clients: 
// base 100, coeff 5, intpkt delay 8 for ~330Mbps
// base 500, coeff 8, intpkt delay 8 for ~280Mbps
// base 500, coeff 12, intpkt delay 12 for ~250Mbps

// special for packet loss issue of HCML
#define RANDOM_DELAY_BASE	500
#define RANDOM_DELAY_COEFF	12

//#define RANDOM_DELAY_BASE	30
//#define RANDOM_DELAY_COEFF	2

// just for testing purpose, obsolete now...
// #define DEFAULT_TEST_REMAIN_SLOTS	237

#define FNAME_SZ	256
#define UNAME_SZ	256
#define OSNAME_SZ	256
#define IMAGENAME_SZ    256
#define USER_TOKEN_SZ   32 

#define TCDLOGFN	"./tcd.log"
// obsoleted ... it's dynamically got from ini file. 
// #define TCDRUNPIDFN	"/var/run/tcd.pid"
// #define TCDRUNPIDFN	"/tc/var/tcd.pid"
#define INIFN	"/home/tc/etc/tc.ini"
#define DB_INI_FILE "/home/tc/etc/local.ini"

// script path and command used by TCDCLT
#define TCDCLT_SCRIPT_PATH	"/tcdscript"
#define TCDCLT_SCRIPT_POWEROFF	"/bin/umount /mnt/disk; /bin/umount /mnt/rsvd; /sbin/poweroff"
#define TCDCLT_SCRIPT_REBOOT	"/bin/umount /mnt/disk; /bin/umount /mnt/rsvd; /sbin/reboot"


// whether update the percentage into database
#define USE_MYSQL	1


// if defined, tcdsvr will run as an application. 
// if not defined, it will run as daemon w/o stdin/stdout.
//#define STANDALONE_MODE	1

#define MNTDEV	"/dev/sda1"
#define RSVDDEV	"/dev/sda4"


#define REFRESH_STATUS_INTERVAL         4
#define REFRESH_STATUS_EXPIRE_INTERVAL  30

#endif

