/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


#define _LARGEFILE64_SOURCE
#define _FILE_OFFSET_BITS	64

#include "conf.h"
#include "tcd.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

#include "fl.h"
#include "bmap.h"
#include "log.h"

// parms used for control the tcdsvr's timeout...
int         g_tm_base;
int         g_tm_coeff;
int         g_tm_intpktdelay;

// global variable for file descriptor of the data file. 
int flr_fd, flw_fd;

int fl_open(char *name)
{
	int fd; 

	if ((fd = open(name, O_RDONLY)) == -1)
	{
#ifdef STANDALONE_MODE
		perror("fl_open");
#endif
		return -1;
	}
	DBG("fl_open: <%s> successfully\n", name);
	return fd;
}

int fl_stat(const char *name)
{
	int retval;
	struct stat flstat; 

	if ((retval = stat(name, &flstat)) == -1)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "fl_stat: <%s> failed\n", name);
#endif
		return -1;
	}
	DBG("fl_stat: <%s> successfully\n", name);
	return 0;
}


int fl_creat(char *name)
{
	int fd; 

	if ((fd = creat(name, 0666)) == -1)
	{
#ifdef STANDALONE_MODE
		perror("fl_creat");
#endif
		return -1;
	}
	DBG("fl_creat: <%s> successfully\n", name);
	return fd;
}

int fl_unlink(char *name)
{
	return unlink(name);
}

uint64 fl_slots(char *name)
{
	struct stat st;
	uint64 size;

	if (stat(name, &st) == -1)
	{
#ifdef STANDALONE_MODE
		perror("stat");
#endif
		return -1;
	}

	size = st.st_size;
	// printf("fl_slots: size=%lld %lld\n", size, (size - 1)/SLOT_BYTES + 1);

	return (size - 1)/SLOT_BYTES + 1;
}

void print_slot(char *s)
{
	int k;

#ifdef STANDALONE_MODE
	printf("----------------------------------------\n");
	k=-1;
	while(++k < SLOT_BYTES)
	{
		if (k % 16 == 0)
			printf("\n%3d ", k);
		printf("0x%x[%c] ", *(s+k), isprint(*(s+k))?*(s+k):'.');
	}
	printf("\n----------------------------------------\n");
#endif

	return;
}

int fl_count = 0;

int fl_read_slot(int fd, uint32 slot, char *buf)
{
	int retval;

#ifdef _DEBUG_
#ifdef STANDALONE_MODE
	printf("fl_read_slot: slot#=%ld\n", slot);
#endif
	// print_slot(buf);
#endif

	//if (++fl_count % 30000 == 29999)
	//	printf(">>> fl_read_slot: current slot: [%d] SLOT_BYTES: [%d] fl_offset: [%lld]\n", slot, SLOT_BYTES, (uint64)slot*SLOT_BYTES);

	if ((retval = lseek(fd, (off64_t)((uint64)slot*SLOT_BYTES), SEEK_SET)) == -1)
	{
#ifdef STANDALONE_MODE
		perror("lseek_read");
#endif
		return -1;
	}

	if ((retval = read(fd, buf, SLOT_BYTES)) < 0)
	{
#ifdef STANDALONE_MODE
		perror("fl_read");
#endif
		return -1;
	}

	return retval;
}

int fl_write_slot(int fd, uint32 slot, char *buf, uint32 bytes)
{
	int retval;

#ifdef _DEBUG_
#ifdef STANDALONE_MODE
	printf("fl_write_slot: slot#=%ld\n", slot);
#endif
	// print_slot(buf);
#endif

	if ((retval = lseek(fd, (off64_t)((uint64)slot*SLOT_BYTES), SEEK_SET)) == -1)
	{
#ifdef STANDALONE_MODE
		perror("lseek_write");
#endif
		return -1;
	}

	if ((retval = write(fd, buf, (bytes<SLOT_BYTES?bytes:SLOT_BYTES))) < 0)
	{
#ifdef STANDALONE_MODE
		perror("fl_write");
#endif
		return -1;
	}

	return retval;
}



int fl_close(int fd)
{
	return close(fd);
}



/**************************************************************************
 * to get a list of filenames from the buffer, and put to the string array. 
 * buf: the filename pool
 * **fn: the string array as the output. 
 * return: # of file names in **fn
 * #define SEGCHAR ';'	which is defined in fl.h
 **************************************************************************/
int getfn(char *buf, char **fn)
{
	char *chp, *head;
	int count=0;

	if (buf == NULL)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "NULL buf\n");
#endif
		return -1;
	}

	head = chp = buf;
	while(1)
	{
		while(*chp != SEGCHAR && *chp != '\0')
			chp++;

		fn[count++] = head;
		
		
		if (*chp == '\0')
			break;
		else
			*chp = '\0';

		head = ++chp;
	}

	return count;
}


/**************************************************************************
 * **fn: the string array as the output. 
 * count: # of filenames in **fn
 * buf: the filename pool
 * return: # of file names in **fn
 **************************************************************************/
int putfn(char **fn, int count, char *buf)
{
	int k;
	char *chp;

	if (buf == NULL || fn == NULL)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "NULL buf");
#endif
		return -1;
	}

	chp = buf;
	for (k=0; k<count; k++)
	{
		memcpy(chp, fn[k], strlen(fn[k]));
		chp += strlen(fn[k]);
		*chp++ = SEGCHAR;
	}

	*(--chp) = '\0';

	return 0;
}



//#define FNAME_SZ	128
#define SEG_OSIDVER '_'

//#if 0
// get the filename from <fn>_<osid>_<ver> string
char *fl_fname(char *fnidver)
{
	char *buf = (char *)malloc(FNAME_SZ);
	char *chp;
	if (buf == NULL)
	{
		/* code */
		return buf;
	}
	strcpy(buf, fnidver);
	chp = strrchr(buf, SEG_OSIDVER);
	if (chp != NULL)
	{
		*chp = '\0';

		chp = strrchr(buf, SEG_OSIDVER);
		if (chp != NULL)
			*chp = '\0';
	}

	return buf;
}
//#endif 

char *fl_fname_ex(char *dest_filename, int dest_filename_sz,const char *fnidver)
{
	char *buf = dest_filename;
	char *chp;

    if (dest_filename == NULL || fnidver == NULL) {
        tcd_syslog("fl_fname_ex(): illegal arguments\n");
        return NULL;
    } else if (dest_filename_sz < FNAME_SZ || *fnidver == '\0') {
        tcd_syslog("fl_fname_ex(): illegal arguments\n");
        return NULL;
    }

	strcpy(buf, fnidver);
	chp = strchr(buf, SEG_OSIDVER);
	if (chp != NULL)
		*chp = '\0';

	strcpy(buf, fnidver);
	chp = strrchr(buf, SEG_OSIDVER);
	if (chp != NULL)
	{
		*chp = '\0';

		chp = strrchr(buf, SEG_OSIDVER);
		if (chp != NULL)
			*chp = '\0';
	}

	return dest_filename;
}


#if 0
// get the osid from <fn>_<osid>_<ver> string
int fl_osid(char *fnidver)
{
	char *buf = (char *)malloc(FNAME_SZ);
	char *chp;

//	printf("fl_osid: fnidver=[%s]\n", fnidver);

	strcpy(buf, fnidver);
	chp = strrchr(buf, SEG_OSIDVER);
	if (chp == NULL)
	{
#ifdef STANDALONE_MODE
		printf("fl_osid: [1]\n");
#endif
		return 1;
	}

	*chp = '\0';
	chp = strrchr(buf, SEG_OSIDVER);
	if (chp == NULL)
	{
#ifdef STANDALONE_MODE
		printf("fl_osid: [1]\n");
#endif
		return 1;
	}
	else
	{
#ifdef STANDALONE_MODE
		printf("fl_osid: [%d]\n", atoi(chp+1));
#endif
		return atoi(chp+1);
	}
}
#endif


// get the osid from <fn>_<osid>_<ver> string
int fl_osid(char *fnidver)
{
    int ret = -1;
	char *buf = (char *)malloc(FNAME_SZ);
	char *chp;

//	printf("fl_osid: fnidver=[%s]\n", fnidver);
	if (buf == NULL){
        free(buf);
        goto fl_osid_try_quit;
    }
    memset(buf, 0, sizeof(buf));
	strncpy(buf, fnidver, sizeof(buf)-1);
	chp = strrchr(buf, SEG_OSIDVER);
	
	if (chp == NULL)
	{
#ifdef STANDALONE_MODE
		printf("fl_osid: [1]\n");
#endif
		ret = 1;
        goto fl_osid_try_quit;
	}

	*chp = '\0';
	chp = strrchr(buf, SEG_OSIDVER);
	if (chp == NULL)
	{
#ifdef STANDALONE_MODE
		printf("fl_osid: [1]\n");
#endif
		ret = 1;
        goto fl_osid_try_quit;
	}
	else
	{
#ifdef STANDALONE_MODE
		printf("fl_osid: [%d]\n", atoi(chp+1));
#endif
		ret = atoi(chp+1);
        goto fl_osid_try_quit;
	}

fl_osid_try_quit:

    
	free(buf);
    return ret;
}


// get the ver from <fn>_<osid>_<ver> string
int fl_ver(char *fnidver)
{
	char *chp = strrchr(fnidver, SEG_OSIDVER);

	if (chp != NULL)
	{
#ifdef STANDALONE_MODE
		printf("fl_ver: [%d]\n", atoi(chp+1));
#endif
		return atoi(chp+1);
	}
	else
	{
#ifdef STANDALONE_MODE
		printf("fl_ver: [1]\n");
#endif
		return 1;
	}
}



#if 0
char *get_tcdpid_fn(char *inifn)
{
	
	FILE *fp;
	char *sp;
	char buf[1024], t1[128], t2[128], t3[128];
	char *tcdpid_fn = (char *)malloc(256);

//	tcd_syslog("get_tcdpid_fn: inifn: %s\n", inifn);

	if (tcdpid_fn == NULL)
	{
		fprintf(stderr, "tcdpid_fn: buffer malloc failed\n");
		return NULL;
	}
	if (inifn == NULL)
	{
		fprintf(stderr, "tcdpid_fn: inifn NULL\n");
		return NULL;
	}

	if ((fp = fopen(inifn, "r")) == NULL)
	{
		fprintf(stderr, "fopen %s failed\n", inifn);
		exit(-1);
	}

	sp = fgets(buf, sizeof(buf), fp);
	while(sp != NULL)
	{
		// printf("read line: [%s]\n", buf);

		sscanf(buf, "%s %s %s", t1, t2, t3);
		// printf("t1=%s t2=%s t3=%s\n", t1, t2, t3);

		if (!strcmp(t1, "home"))
		{	
			printf("home=[%s]\n", t3);
			strcpy(tcdpid_fn, t3);
		}
		else if (!strcmp(t1, "home_tcd_pid"))
		{
			printf("home_tcd_pid=[%s]\n", t3);
			strcat(tcdpid_fn, t3);
		}
		sp = fgets(buf, sizeof(buf), fp);
	}

	fclose(fp);

	return tcdpid_fn;
}
#endif

char *get_tcdpid_fn_ex(char *dest_pid_filename, int dest_pid_filename_sz, const char *config_filename)
{
	
	FILE *fp;
	char *sp;
	char buf[1024], t1[128], t2[128], t3[128];
	//char *tcdpid_fn = (char *)malloc(256);

    if (dest_pid_filename == NULL || config_filename == NULL) {
		tcd_syslog("get_tcdpid_fn_ex(): NULL pointer, illegal arguments\n");
        return NULL;
    } else if (*config_filename == '\0' || dest_pid_filename_sz < 256) {
		tcd_syslog("get_tcdpid_fn_ex(): incorrect data, illegal arguments\n");
        return NULL;
    }

	if ((fp = fopen(config_filename, "r")) == NULL)
	{
		tcd_syslog("fopen %s failed\n", config_filename);
		return NULL;
	}

	sp = fgets(buf, sizeof(buf), fp);
	while(sp != NULL)
	{
		// printf("read line: [%s]\n", buf);

		sscanf(buf, "%127s %127s %127s", t1, t2, t3);
		// printf("t1=%s t2=%s t3=%s\n", t1, t2, t3);

		if (!strcmp(t1, "home"))
		{	
			printf("home=[%s]\n", t3);
			strcpy(dest_pid_filename, t3);
		}
		else if (!strcmp(t1, "home_tcd_pid"))
		{
			printf("home_tcd_pid=[%s]\n", t3);
			strcat(dest_pid_filename, t3);
		}
		sp = fgets(buf, sizeof(buf), fp);
	}

	fclose(fp);

	return dest_pid_filename;
}



int get_tcdsvr_parms(char *inifn)
{
	FILE *fp;
	char *sp;
	char buf[1024], t1[128], t2[128], t3[128];
	int count=0;

	tcd_syslog("get_tcdsvr_parms: inifn: %s\n", inifn);

	if (inifn == NULL)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "get_tcdsvr_parms: inifn NULL\n");
#else
		tcd_syslog("get_tcdsvr_parms: inifn NULL");
#endif
		return -1;
	}

	if ((fp = fopen(inifn, "r")) == NULL)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "get_tcdsvr_parms: fopen %s failed\n", inifn);
#else
		tcd_syslog("get_tcdsvr_parms: fopen %s failed\n", inifn);
#endif
		return -1;
	}

	sp = fgets(buf, sizeof(buf), fp);
	while(sp != NULL)
	{
#ifdef STANDALONE_MODE
		printf("read line: [%s]\n", buf);
#else
#endif

		sscanf(buf, "%127s %127s %127s", t1, t2, t3);
#ifdef STANDALONE_MODE
		printf("t1=%s t2=%s t3=%s\n", t1, t2, t3);
#else
#endif

		if (!strcmp(t1, "base"))
		{
			g_tm_base = atoi(t3);
			count++;
#ifdef STANDALONE_MODE
			printf("g_tm_base=%d\n", g_tm_base);
#else
			tcd_syslog("g_tm_base=%d\n", g_tm_base);
#endif

		}
		else if (!strcmp(t1, "coeff"))
		{
			g_tm_coeff = atoi(t3);
			count++;
#ifdef STANDALONE_MODE
			printf("g_tm_coeff=%d\n", g_tm_coeff);
#else
			tcd_syslog("g_tm_coeff=%d\n", g_tm_coeff);
#endif
		}
		else if (!strcmp(t1, "intpktdelay"))
		{
			g_tm_intpktdelay = atoi(t3);
			count++;
#ifdef STANDALONE_MODE
			printf("g_tm_intpktdelay=%d\n", g_tm_intpktdelay);
#else
			tcd_syslog("g_tm_intpktdelay=%d\n", g_tm_intpktdelay);
#endif
		}

		sp = fgets(buf, sizeof(buf), fp);
	}

	fclose(fp);

	return count;
}

// open file TCD_WLOCK_FILE with wr-lock
// return:
// -1, if fail to get wr-lock
// 0, if get wr-lock successfully

int fl_tcd_wlock()
{
    int ret;
    int fd;
    struct flock fl;

    ret = 0;

    fd = open(TCD_WLOCK_FILE, O_RDWR | O_CREAT, 0600);
    if (fd < 0){
        ret = -1;
        goto label_fl_tcd_wlock_exit;
    }
    fl.l_start = 0;
    fl.l_len = 0;
    fl.l_type = F_WRLCK;
    fl.l_whence = SEEK_SET;

    if (fcntl(fd, F_SETLK, &fl) < 0){
        ret = -1;
        goto label_fl_tcd_wlock_exit;
    }

label_fl_tcd_wlock_exit:
	close(fd);
    return ret;
}


#if 0

/**************************************************************************
 * to get a list of filename/id/ver-s from the buffer, and put to the string array. 
 * buf: the filename pool
 * **fn: the string array as the output. 
 * return: # of file names in **fn
 * #define SEGCHAR ';'	which is defined in fl.h
 **************************************************************************/
int getfnidver(char *buf, char **fnidver)
{
	char *chp, *head;
	int count=0;

	if (buf == NULL)
	{
		fprintf(stderr, "NULL buf\n");
		return -1;
	}

	head = chp = buf;
	while(1)
	{
		while(*chp != SEGCHAR && *chp != '\0')
			chp++;

		fnidver[count++] = head;
		
		
		if (*chp == '\0')
			break;
		else
			*chp = '\0';

		head = ++chp;
	}

	return count;
}


/**************************************************************************
 * **fn: the string array as the output. 
 * count: # of filenames in **fn
 * buf: the filename pool
 * return: # of file names in **fn
 **************************************************************************/
int putfnidver(char **fnidver, int count, char *buf)
{
	int k;
	char *chp;

	if (buf == NULL || fn == NULL)
	{
		fprintf(stderr, "NULL buf");
		return -1;
	}

	chp = buf;
	for (k=0; k<count; k++)
	{
		memcpy(chp, fnidver[k], strlen(fnidver[k]));
		chp += strlen(fnidver[k]);
		*chp++ = SEGCHAR;
	}

	*(--chp) = '\0';

	return 0;
}

#endif





/*
main()
{
	char buf[2048];
	int fd, retval;
	uint32 slot=0;
	int bytes=0;

	slot = fl_slots(FLR_NAME);
	printf("inf name: <%s> total slots: %ld outf name: <%s>\n", FLR_NAME, slot, FLW_NAME);

	slot=0;
	flr_fd = fl_open(FLR_NAME);
	flw_fd = fl_creat(FLW_NAME);

	retval = fl_read_slot(flr_fd, slot, buf);
	while(retval > 0)
	{
		printf("read %d bytes\n", retval);
		bytes += retval;
		fl_write_slot(flw_fd, slot, buf);
		// printf("read %d bytes total %d bytes: \n\n", retval, bytes);

		slot ++;
		retval = fl_read_slot(flr_fd, slot, buf);
	}
	printf("read total %d bytes: \n\n", bytes);
	fl_close(flr_fd);
	fl_close(flw_fd);

	exit(0);
}
*/

/*
main()
{
	int svr_remain_slots;

	svr_remain_slots = fl_slots(FLR_NAME); // DEFAULT_TEST_REMAIN_SLOTS;
	printf("read file slots: %d\n", svr_remain_slots);

}


main()
{
	char buf[2048];
	int fd, retval;
	uint32 slot=0;
	int bytes=0;

	fd = fl_open();

	retval = fl_read_slot(fd, slot++, buf);
	while(retval > 0)
	{
		bytes += retval;
		// printf("read %d bytes total %d bytes: \n\n", retval, bytes);
		retval = fl_read_slot(fd, slot++, buf);
	}
	printf("read total %d bytes: \n\n", bytes);
	fl_close(fd);

	exit(0);
}
*/


