/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/

/* ifname to ip address */

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <net/if_arp.h>
#include <netinet/in.h>

#include <ifaddrs.h>
#include <linux/if.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>

#include "tcd.h"

extern char *ifn2ipadd(char *ifn);
extern uint32 ifn2ip(char *ifn);
extern int is_valid_ipv4 (const char *ipaddr);
extern int is_valid_ipv4ex(const char* ipaddr);

#if 1
// this function is used for uclibc ... 
char *ifn2ip_str(char *ifn)
{
	int sock_get_ip;
	// char ipaddr[32]; 
	char *localip;

	struct sockaddr_in *sin;  
	struct ifreq ifr_ip;     

	if ((sock_get_ip=socket(AF_INET, SOCK_STREAM, 0)) == -1)  
	{
		printf("socket create failse...GetLocalIp!/n");  
		return NULL;  
	}  

	memset(&ifr_ip, 0, sizeof(ifr_ip));     
	strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);     

	if (ioctl(sock_get_ip, SIOCGIFADDR, &ifr_ip) < 0){
		close(sock_get_ip);  
		return NULL;
	}     

	sin = (struct sockaddr_in *)&ifr_ip.ifr_addr;     
	localip = (char *)malloc(32);
	if (localip != NULL)
	{
		/* code */
		memset(localip, 0, sizeof(localip));
		strncpy(localip,inet_ntoa(sin->sin_addr), sizeof(localip)-1);
	}
	         

	DBG("====> figure out local ip of interface <%s>: <%s>\n", ifn, localip);

	close(sock_get_ip);  

	return localip;
}
#endif

// this function is used for uclibc ... 
char *ifn2ip_str_ex(char *dest_ip, int dest_ip_sz, const char *ifn)
{
    if (dest_ip == NULL || ifn == NULL || dest_ip_sz < 16)
    {
        return NULL;
    }
    else if (*ifn == '\0')
    {
        return NULL;
    }

	int sock_get_ip;

	struct sockaddr_in *sin;  
	struct ifreq ifr_ip;     

	if ((sock_get_ip=socket(AF_INET, SOCK_STREAM, 0)) == -1)  
	{
		printf("socket create failse...GetLocalIp!/n");  
		return NULL;  
	}  

	memset(&ifr_ip, 0, sizeof(ifr_ip));     
	strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);     

	if (ioctl(sock_get_ip, SIOCGIFADDR, &ifr_ip) < 0){
		close(sock_get_ip);  
		return NULL;
	}     

		

	sin = (struct sockaddr_in *)&ifr_ip.ifr_addr;     
	strcpy(dest_ip, inet_ntoa(sin->sin_addr));         

	DBG("====> figure out local ip of interface <%s>: <%s>\n", ifn, dest_ip);

	close(sock_get_ip);  

	return dest_ip;
}


// get the mac address string
char *ifn2mac_str(char *ifn)
{
	int sock_get_ip;
	// char ipaddr[32]; 
	unsigned char hwa[6], *hwa_str;

	struct sockaddr_in *sin;  
	struct ifreq ifr_ip;     

	if ((sock_get_ip=socket(AF_INET, SOCK_STREAM, 0)) == -1)  
	{
		printf("socket create failse...GetLocalIp!/n");  
		return NULL;  
	}  

	memset(&ifr_ip, 0, sizeof(ifr_ip));     
	strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);     

	if (ioctl(sock_get_ip, SIOCGIFHWADDR, &ifr_ip) < 0){
		close(sock_get_ip);  
		return NULL;
	}     
		     

	memcpy(hwa, ifr_ip.ifr_hwaddr.sa_data, sizeof(hwa));
	hwa_str = (unsigned char *)malloc(32);
	if (hwa_str == NULL)
	{
		/* code */
		close(sock_get_ip);  
		return hwa_str;
	}
	sprintf(hwa_str, "%02X-%02X-%02X-%02X-%02X-%02X", hwa[0], hwa[1], hwa[2], hwa[3], hwa[4], hwa[5]);

	DBG("====> figure out mac address of interface <%s>: <%s>\n", ifn, hwa_str);

	close(sock_get_ip);  

	return hwa_str;
}


// get the mac address string
char *ifn2mac_str_ex(char *dest_hwa, int dest_hwa_sz, const char *ifn)
{
	int sock_get_ip;
	//unsigned char hwa[6], *hwa_str;
	unsigned char hwa[6];

	struct sockaddr_in *sin;  
	struct ifreq ifr_ip;     

    if (dest_hwa == NULL || ifn == NULL) {
        return NULL;
    } else if (*ifn == '\0' || dest_hwa_sz < 18) {
        return NULL;
    }

	if ((sock_get_ip=socket(AF_INET, SOCK_STREAM, 0)) == -1)  
	{
		printf("socket create failse...GetLocalIp!/n");  
		return NULL;  
	}  

	memset(&ifr_ip, 0, sizeof(ifr_ip));     
	strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);     

	if (ioctl(sock_get_ip, SIOCGIFHWADDR, &ifr_ip) < 0) {
		close(sock_get_ip);
		return NULL;
	}    
		     

	memcpy(hwa, ifr_ip.ifr_hwaddr.sa_data, sizeof(hwa));
	sprintf(dest_hwa, "%02X-%02X-%02X-%02X-%02X-%02X", hwa[0], hwa[1], hwa[2], hwa[3], hwa[4], hwa[5]);
	DBG("====> figure out mac address of interface <%s>: <%s>\n", ifn, dest_hwa);


	close(sock_get_ip);  

	return dest_hwa;
}


// get the IP address
uint32 ifn2ip(char *ifn)
{
	int sock_get_ip;
	// char ipaddr[32]; 
	char *localip;

	struct sockaddr_in *sin;  
	struct ifreq ifr_ip;     

	if ((sock_get_ip=socket(AF_INET, SOCK_STREAM, 0)) == -1)  
	{
		printf("socket create failse...GetLocalIp!/n");  
		return -1;  
	}  

	memset(&ifr_ip, 0, sizeof(ifr_ip));     
	strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);     

	if (ioctl(sock_get_ip, SIOCGIFADDR, &ifr_ip) < 0) {
		close(sock_get_ip);  
		return -1;
	}    
		     

	sin = (struct sockaddr_in *)&ifr_ip.ifr_addr;     


	close(sock_get_ip);  

	return sin->sin_addr.s_addr;
}



// get the mac address
unsigned char *ifn2mac(char *ifn)
{
	int sock_get_ip;
	// char ipaddr[32]; 
	unsigned char *hwa;

	struct sockaddr_in *sin;  
	struct ifreq ifr_ip;     

	if ((sock_get_ip=socket(AF_INET, SOCK_STREAM, 0)) == -1)  
	{
		printf("socket create failse...GetLocalIp!/n");  
		return NULL;  
	}  

	memset(&ifr_ip, 0, sizeof(ifr_ip));     
	strncpy(ifr_ip.ifr_name, ifn, sizeof(ifr_ip.ifr_name) - 1);     

	if (ioctl(sock_get_ip, SIOCGIFHWADDR, &ifr_ip) < 0){
		close(sock_get_ip); 
		return NULL;
	}  
		     

	hwa = (unsigned char *)malloc(16);
	if (hwa != NULL)
	{
		/* code */
		memcpy(hwa, ifr_ip.ifr_hwaddr.sa_data, 6);
	}
	
//	sprintf(hwa_str, "%02X-%02X-%02X-%02X-%02X-%02X", hwa[0], hwa[1], hwa[2], hwa[3], hwa[4], hwa[5]);

	// DBG("====> figure out mac address of interface <%s>: <%s>\n", ifn, hwa_str);

	close(sock_get_ip);  

	return hwa;
}




#if 0
char *ifn2ipadd(char *ifn)
{
	struct ifaddrs *ifaddrstruct=NULL;
	void *tmpaddrptr = NULL;
	char *localip=NULL;

	if (getifaddrs(&ifaddrstruct) < 0)
	{
		perror("getifaddrs");
		return NULL;
	}
	DBG("ifn2ipadd: getifaddrs successfully ...\n");

	while(ifaddrstruct != NULL)
	{
		if (ifaddrstruct->ifa_addr->sa_family == AF_INET)
		{
			tmpaddrptr = &((struct sockaddr_in *)ifaddrstruct->ifa_addr)->sin_addr;
			char addressbuffer[INET_ADDRSTRLEN];
			inet_ntop(AF_INET, tmpaddrptr, addressbuffer, INET_ADDRSTRLEN);
			DBG("%s ip add: %s\n", ifaddrstruct->ifa_name, addressbuffer);
			if (!strcmp(ifaddrstruct->ifa_name, ifn))
			{
				localip = (char *)malloc(32);
				strcpy(localip, addressbuffer);
			}
		}
		else if (ifaddrstruct->ifa_addr->sa_family == AF_INET6)
		{
			// NOT SUPPORT HERE
		}

		ifaddrstruct = ifaddrstruct->ifa_next;
	}

	DBG("====> figure out local ip of interface <%s>: <%s>\n", ifn, localip);
	return localip;
}
#endif


/* 
 * return 1 if legal ip
 * return 0 if illegal ip
 * return -1 if error
 */
int is_valid_ipv4 (const char *ipaddr){
    int num;
    int flag = 1;
    int counter = 0;
    int ndots;
    char ip[16];
    char *q;
    char *p;

    if (strlen(ipaddr) > 15)
    {
        return -1;
    }

    strncpy (ip, ipaddr, 16);
    ip[15] = '\0';
    p = ip;
    for (ndots = 0; *p; p++){
        if (*p == '.'){
            ndots++;
        }
    }

    p = strtok(ip, ".");
    while (p && flag){
        q = p;
        while (*q){
            if (*q < '0' || *q > '9'){
                flag = 0;
                break;
            }
            q++;
        }

        if (!flag){
            break;
        }

        if (*p == '0' && strlen (p) > 1){
            flag = 0;
            break;
        }

        num = atoi (p);
        if (num >= 0 && num <= 255 && (counter++ < 4)){
            flag = 1;
            p = strtok (NULL, ".");
        }
        else{
            flag = 0;
            break;
        }
    }
    return (ndots == 3) && flag && (counter == 4);
}

//
// return 1 if legal ip
// return 0 if illegal ip
// return -1 if error
//
int is_valid_ipv4ex(const char* ipaddr)
{
    struct sockaddr_in sa;

    return 0 != inet_pton(AF_INET, ipaddr, &(sa.sin_addr));
}
