/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


/*------------------------------------------------------*/
/* term.c: main routine 				*/
/* text-based UI for Prof. Zhou's medical terminal 	*/
/*------------------------------------------------------*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <string.h>

#include <newt.h>

#include "term.h"

#define PATH_APP_TCDREG      "/usr/local/bin/tcdreg"
#define PATH_FILE_LOG        "/tmp/newtreg.log"
#define PATH_SCRIPT_SHUTDOWN "/usr/sbin/tc/tc-shutdown.sh --poweroff"
#define PATH_SCRIPT_REBOOT   "/usr/sbin/tc/tc-shutdown.sh --reboot"

// comment this macro to disable the system shutdown/reboot feature
//#define NEWTREG_EXIT_OPTION

static int reg_ui(const char *svrip);

static int tc_newt_window (unsigned int timeout, char *title, char *btnok_text, char *text, const char * helpline1, const char * helpline2);
static int tc_newt_choice (unsigned int timeout, int default_btn, char *title, char *btnok_text, char *btncancel_text, char *text, const char * helpline1, const char *helpline2);

// global variable
int screen_cols, screen_rows; 

char *check_path_safe(char *path)
{
    char *retStr;
    size_t pathLen;
    int i;
    
    if(!path)
        return NULL;

    retStr = path;
    if (strlen(retStr) > 256)
    {
        /* code */
        return NULL;
    }
    // pathLen = strlen(path);
    // for(i=0; i<pathLen; i++) {
    //     if(path[i] == '.') {
    //         path[i] = '.';
    //         break;
    //     }
    // }

    return retStr;
} 


int main(int argc, char* argv[])
{
	int retval = -1;	
	if (argc < 2)
	{
		printf ("Usage:\n");
		printf ("    %s <svrip>\n", argv[0]);
		printf ("    e.g., %s 192.168.0.1\n", argv[0]);
		exit (retval);
	}
    if (check_path_safe(argv[1]))
    {
        /* code */
        return -1;
    }
	retval = reg_ui(argv[1]);
	return retval;
}


static int reg_ui(const char *svrip)
{
	int t;
	int loop_flag;
	int retval;
	newtComponent form, label, button, entry;
	struct newtExitStruct es;
	char *ev, cltname[128];
	char buf[512];

	newtInit();
	newtGetScreenSize(&screen_cols, &screen_rows);

	label = newtLabel(screen_cols/30, 1, "Input Client Name: ");
	entry = newtEntry(screen_cols/30, 3, NULL, 20, (const char **)&ev, /*NEWT_FLAG_SCROLL*/0);
	button = newtButton(screen_cols/5, 7, "OK");
	
	form = newtForm(NULL, NULL, NEWT_FLAG_NOF12);
	newtFormAddComponents(form, entry, label, button, NULL);
#ifdef NEWTREG_EXIT_OPTION
	newtFormAddHotKey(form, NEWT_KEY_ESCAPE);
#endif
	
	do {
		loop_flag = 1;

		newtCls();
		newtRefresh();
		newtDrawRootText(0, 0, "Transparent Computing");
		newtDrawRootText(0, 1, "Copyright from 2007~2017 by Intel");
#ifdef NEWTREG_EXIT_OPTION
		newtPushHelpLine("<Esc> Exit");
#endif

		newtCenteredWindow(screen_cols/2, screen_rows/2, "Client Registration");

		newtFormSetCurrent(form, entry);
		newtFormRun(form, &es);
		strcpy(cltname, ev);

		if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == button){
			if (!*ev){
				tc_newt_window(0, "Warning", "OK", "Client name cannot be empty", NULL, NULL);
			}
			else {
				sprintf (buf, "%s %256s --reg %s >%s 2>&1", PATH_APP_TCDREG, svrip, cltname, PATH_FILE_LOG);
				t = system(buf);
				t = WEXITSTATUS(t);
				retval = t;

				if (0 == t)
				{
					tc_newt_window(0, "Message", "OK", "Client is registered successfully", NULL, NULL);
					loop_flag = 0;
				}
				else if (254 == t) /* app exited with -2 */
				{
					tc_newt_window(0, "Warning", "OK", "Duplicate name, please try other names", NULL, NULL);
				}
				else{
					sprintf(buf, "Unknown error, please refer to log in file %s", PATH_FILE_LOG);
					tc_newt_window(0, "Warning", "OK", buf, NULL, NULL);
				}
			}
		}
#ifdef NEWTREG_EXIT_OPTION
		if (es.reason == NEWT_EXIT_HOTKEY && es.u.key == NEWT_KEY_ESCAPE){
			int cho;
			cho = tc_newt_choice(0, 0, "Message", "Poweroff", "Reboot", "Do you want to shutdown the system?", "<Esc> Cancel | <Poweroff> Power Off | <Reboot> Reboot", NULL);
			if (cho == 0){
				;
			} else if (cho == 1) {
				system(PATH_SCRIPT_SHUTDOWN);

			} else if (cho == 2) {
				system(PATH_SCRIPT_REBOOT);

			}
		}
#endif
#ifdef NEWTREG_EXIT_OPTION
		newtPopHelpLine();
#endif
		newtPopWindow();
	} while (loop_flag);

	newtFormDestroy(form);
	newtFinished();

	fprintf(stderr, "%s", cltname);
	return retval;	

}


// If timeout > 0;
//     If an Escape within timeout, stopping timing, and wait for keystrokes;
//     If no keystrokes within timeout, or a keystroke on 'Ok' within timeout, or an Escape within timeout followed, then a keystroke on 'Ok',  return 1;
//     If an Escape followed by another Escape, return 0; 
// If timeout = 0;
//     
// text length: not more than 512
static int tc_newt_window (unsigned int timeout, char *title, char *btnok_text, char *text, const char * helpline1, const char * helpline2){
    int ret = -1;
    newtComponent form, btnok, textbox, timelabel = NULL;
    char labeltext[64];
    char text_act[512], *p;
    int text_width, text_height;
    int wnd_width_local, wnd_height_local;
    unsigned int timeout_bak = timeout;
    struct newtExitStruct es;

    p = newtReflowText (text, 60, 5, 5, &text_width, &text_height);
    strncpy (text_act, p, 511);
    free (p);
    p = NULL;

    wnd_width_local = text_width + 4;
    wnd_height_local = text_height + 7;

    textbox = newtTextbox ((wnd_width_local - text_width) / 2, 2, text_width, text_height, 0);
    newtTextboxSetText (textbox, text_act);

    if (timeout > 0){
        sprintf (labeltext, "window will disappear in %u ", timeout);
        strcat (labeltext, timeout == 1 ? "second" : "seconds");
        timelabel = newtLabel (0, 0, labeltext);
    }
 
    btnok = newtButton ((wnd_width_local - strlen (btnok_text) - 4)/2, wnd_height_local - 4, btnok_text);

    form = newtForm (NULL, NULL, NEWT_FLAG_NOF12);
    if (timeout > 0){
        newtFormAddComponents (form, timelabel, NULL);
        newtFormSetTimer (form, 1000); 
    }
    newtFormAddComponents (form, textbox, btnok, NULL);
    newtFormSetCurrent (form, btnok); 
    newtFormAddHotKey (form, NEWT_KEY_ESCAPE); 

    if (helpline1 != NULL){
        newtPushHelpLine (helpline1);
    }

    newtClearKeyBuffer ();
    newtCursorOff ();
    newtCenteredWindow (wnd_width_local, wnd_height_local, title);

    do{
        newtFormRun (form, &es);
        if (es.reason == NEWT_EXIT_TIMER){
            timeout--;
            if (timeout <= 0){
                ret = 1;
                break;
            }
            sprintf (labeltext, "window will disappear in %u ", timeout);
            //sprintf (labeltext, "run with default option in %u ", timeout);
            strcat (labeltext, timeout == 1 ? "second" : "seconds");
            newtLabelSetText (timelabel, labeltext);
        }
        if (es.reason == NEWT_EXIT_HOTKEY && es.u.key == NEWT_KEY_ESCAPE){
            ret = 0;
            break;
        }
        if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == btnok){
            ret = 1;
            break;
        }
    } while (es.reason != NEWT_EXIT_COMPONENT);

    if (ret == 0 && timeout_bak > 0){
        if (helpline2 != NULL){
            newtPushHelpLine (helpline2);
        }
        newtClearKeyBuffer ();
        newtFormSetTimer (form, 0);
        newtLabelSetText (timelabel, "");
        newtFormRun (form, &es);
        if (es.reason == NEWT_EXIT_HOTKEY && es.u.key == NEWT_KEY_ESCAPE){
            ret = 0;
        }
        if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == btnok){
            ret = 1;
        }
        if (helpline2 != NULL){
            newtPopHelpLine ();
        }
    }

    if (helpline1 != NULL){
        newtPopHelpLine ();
    }
    newtPopWindow ();
    newtFormDestroy (form);
    return ret;
}



/* If timeout > 0:
 *    if no strokes within timeout, if default_btn == 0 then return 1; otherwise return 2;
 *    if Escape within timeout, stopping timing and is followed by:
 *        if another Escape, return 0;
 *        if a keystroke on 'Ok' button, return 1;
 *        if a keystroke on 'Cancel' button, return 2;
 *    if a keystroke on 'Ok' button within timeout, return 1;
 *    if a keystroke on 'Cancel' button within timeout, return 2;
 * If timeout = 0:
 *    if a keystroke on 'Ok' button, return 1;
 *    if a keystroke on 'Cancel' button, return 2;
 *    if Escape happened, return 0;
 */
// text length: not more than 512
static int tc_newt_choice (unsigned int timeout, int default_btn, char *title, char *btnok_text, char *btncancel_text, char *text, const char * helpline1, const char *helpline2){
    int ret = -1;
    newtComponent form, btnok, btncancel, textbox, timelabel = NULL;
    char labeltext[64];
    char text_act[512], *p;
    int text_width, text_height;
    int wnd_width_local, wnd_height_local;
    int btnbar_width, btnbar_height;
    unsigned int timeout_bak = timeout;
    struct newtExitStruct es;

    p = newtReflowText (text, 60, 5, 5, &text_width, &text_height);
    strncpy (text_act, p, 511);
    free (p);
    p = NULL;

    wnd_width_local = text_width + 4;
    wnd_height_local = text_height + 7;

    textbox = newtTextbox ((wnd_width_local - text_width) / 2, 2, text_width, text_height, 0);
    newtTextboxSetText (textbox, text_act);

    if (timeout > 0){
        sprintf (labeltext, "run with default option in %u ", timeout);
        strcat (labeltext, timeout >= 2 ? "seconds" : "second");
        timelabel = newtLabel (0, 0, labeltext);
    }
 
    btnbar_width = strlen (btnok_text) + strlen (btncancel_text) + 12;
    btnbar_height = wnd_height_local - 4;

    btnok = newtButton ((wnd_width_local - btnbar_width) / 2, btnbar_height, btnok_text);
    btncancel = newtButton (wnd_width_local - (wnd_width_local - btnbar_width) / 2 - strlen (btncancel_text), btnbar_height, btncancel_text);

    form = newtForm (NULL, NULL, NEWT_FLAG_NOF12);

    newtFormAddComponents (form, textbox, btnok, btncancel, NULL);
    if (timeout > 0){
        newtFormAddComponents (form, timelabel, NULL);
        newtFormSetTimer (form, 1000); 
    }

    newtFormSetCurrent (form, 0 == default_btn ? btnok : btncancel); 
    newtFormAddHotKey (form, NEWT_KEY_ESCAPE); 

    if (helpline1 != NULL){
        newtPushHelpLine (helpline1);
    }

    newtClearKeyBuffer ();
    newtCursorOff ();
    newtCenteredWindow (wnd_width_local, wnd_height_local, title);

    do{
        newtFormRun (form, &es);
        if (es.reason == NEWT_EXIT_TIMER){
            timeout--;
            if (timeout <= 0){
                ret = default_btn == 0 ? 1 : 2;
                break;
            }
            sprintf (labeltext, "run with default option in %u ", timeout);
            strcat (labeltext, timeout > 1 ? "seconds" : "second");
            newtLabelSetText (timelabel, labeltext);
        }

        if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == btnok){
            ret = 1;
            break;
        }

        if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == btncancel){
            ret = 2;
            break;
        }
        if (es.reason == NEWT_EXIT_HOTKEY && es.u.key == NEWT_KEY_ESCAPE){
            ret = 0;
            break;
        }
    } while (es.reason != NEWT_EXIT_COMPONENT);

    if (ret == 0 && timeout_bak > 0){
        if (helpline2 != NULL){
            newtPushHelpLine (helpline2);
        }
        newtClearKeyBuffer ();
        newtLabelSetText (timelabel, "");
        newtFormSetTimer (form, 0);
        newtFormRun (form, &es);
        if (es.reason == NEWT_EXIT_HOTKEY && es.u.key == NEWT_KEY_ESCAPE){
            ret = 0;
        }
        if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == btncancel){
            ret = 2;
        }
        if (es.reason == NEWT_EXIT_COMPONENT && es.u.co == btnok){
            ret = 1;
        }
        if (helpline2 != NULL){
            newtPopHelpLine ();
        }
    }

    if (helpline1 != NULL){
        newtPopHelpLine ();
    }

    newtPopWindow ();
    newtFormDestroy (form);

    return ret;
}

