/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


/* TC Deployment active user test code */

#include "conf.h"
#include "tcd.h"

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <termios.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <signal.h>

#include "tcd.h"
#include "cltdb.h"
#include "tcdmsg.h"
#include "term.h"

#include "bmap.h"
#include "fl.h"

#include "tm.h"
#include "vos_mem.h"

// defined in ifn2ipadd.c
extern char *ifn2ip_str(char *ifn);
extern char *ifn2mac_str(char *ifn);
extern unsigned char *ifn2mac(char *ifn);

unsigned char *localmac, *localip;
int sock;

void tcdactusr_usage(char *argv0)
{
	printf("\nUsage: ");
	printf("\n\t%s <svrip> <username> <osname> ...\n\t\t--- register the user with username and osname\n", argv0);

	return;
}


// command actusr request
int actusr_buf(char **buf, char *uname, char *osname, int act)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_ACTUSR;
	msg->size = BUFSZ;

#if 0
defined in tcdmsg.h ...

#define ACT_POWERON		1	// nothing
#define ACT_AUTHENTICATED	2	// username
#define ACT_SYNCING		3	// username and osname
#define ACT_BOOTING		4	// username and osname
#define ACT_RUNNING		5	// nothing
#define ACT_SHUTTING_DOWN	6	// nothing
#define ACT_UPLOADING		7	// username and osname
#endif


	msg->u.au.act = act;
	if (act == ACT_AUTHENTICATED)
	{
		memcpy(msg->u.au.hwa, localmac, HWA_SZ);
		strcpy(msg->u.au.uname, uname);
		strcpy(msg->u.au.osname, "");
	}
	else if (act == ACT_SYNCING || act == ACT_BOOTING || act == ACT_UPLOADING)
	{
		memcpy(msg->u.au.hwa, localmac, HWA_SZ);
		strcpy(msg->u.au.uname, uname);
		strcpy(msg->u.au.osname, osname);
	}
	else if (act == ACT_POWERON || act == ACT_RUNNING || act == ACT_SHUTTING_DOWN)	// in both OS runtime or shutdown, uname and osname are invalid
	{
		memcpy(msg->u.au.hwa, localmac, HWA_SZ);
		strcpy(msg->u.au.uname, "");
		strcpy(msg->u.au.osname, "");
	}
	else
	{
		printf("actusr_buf: unknown act %d\n", act);
		memcpy(msg->u.au.hwa, localmac, HWA_SZ);
		strcpy(msg->u.au.uname, "");
		strcpy(msg->u.au.osname, "");
		return -1;
	}

	return BUFSZ;
}

// to send a actusr message. 
int actusr_send(int socket, struct sockaddr_in sain, char *uname, char *osname, int act)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = actusr_buf(&msgbuf, uname, osname, act);
	DBG("actusr_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "actusr_send: request buffer failed\n");
		return -1;
	}
}


typedef void (*signal_handler)(int);
void signal_handler_proc_tcdactusr(int signo)
{
	printf("catch signal #%d\n", signo);

	printf("exit tcdactusr by interrupt...");

	// to close the control socket	
	DBG("tcdactusr: close the socket\n");
	close(sock);

	exit(0);
	return;
}


// tcdactusr <svrip> <uname> <osname>
int main(int argc, char **argv)
{
	struct sockaddr_in servadd;
	char *cifn;

	int dstport;
	uint32 dstip;
	char *dstip_str=NULL;
	char *uname, *osname;
	signal_handler psignal = signal_handler_proc_tcdactusr;
	int count=0;
	int act=0;

	// parameter handling
	if (argc != 4)
	{
		tcdactusr_usage(argv[0]);
		exit(0);
	}


	printf("tcdactuser: install signal handler ...\n");
	signal(SIGINT, psignal);
	signal(SIGTERM, psignal);

	
	dstip_str = argv[1];
	uname = argv[2];
	osname = argv[3];
	// act = atoi(argv[4]);


	printf("uname: [%s] osname: [%s]\n", uname, osname);

	cifn = "eth0";
//	dstip_str = "192.168.0.1";
	dstip = inet_addr(dstip_str);
	dstport = 12345; // atoi(argv[2]);


	// get the local ipadd of the interface
	if ((localip = ifn2ip_str(cifn)) == NULL)
	{
		perror("ifn2ip_str");	
		exit(-1);
	}

	if ((localmac = (unsigned char *)ifn2mac(cifn)) == NULL)
	{
		perror("ifn2mac");
		exit(-1);
	}

	printf("\ntcd active user test code\n");
	printf("Interface [%s] LocalIP: [%s] Network [0x%x:%d] \n", cifn, localip, dstip, dstport);
	printf("======>>>>> mac address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", localmac[0], localmac[1], localmac[2], localmac[3], localmac[4], localmac[5]);

		
	// to create the data socket. the hello packet will also be from this socket. 
	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	{
		perror("socket");
		exit(-1);
	}
	DBG("tcdclt: socket successfully ...\n");


	struct sockaddr_in singlesock;
	bzero(&singlesock, sizeof(singlesock));
	singlesock.sin_family = AF_INET;
	singlesock.sin_addr.s_addr = inet_addr(dstip_str);
	singlesock.sin_port = htons(dstport);

	// initialize the memory ...
	if (vos_meminit() == -1)
	{
		fprintf(stderr, "vos_meminit: failed\n");
		close(sock);
		exit(-1);
	}

	printf("preboot: start active user message ...\n");
	actusr_send(sock, singlesock, uname, osname, ACT_BOOTING);


	printf("runtime: start active user message ...\n");
	count = 0;
	while(++count < 16)
	{
		printf("sending %d message ...\n", count);
		actusr_send(sock, singlesock, uname, osname, ACT_RUNNING);
		sleep(2);
	}

	printf("shutdown: start active user message ...\n");
	actusr_send(sock, singlesock, uname, osname, ACT_SHUTTING_DOWN);

	// to close the control socket	
	DBG("tcdclt: close the socket\n");
	close(sock);

	exit(0);
}



