/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/

#ifndef _TCDMSG_H_
#define _TCDMSG_H_

#include <sys/time.h>
#include "tcd.h"
#include "cltdb.h"
#include "bmap.h"

#include "_tcmsgcommon.h"

// server control block: a bunch of data which controls the behaviour of the TCD server
typedef struct scb_struct
{
	// global variables of sending total file bulks, just for debugging
	uint32 svr_cbulk;		// current bulk #

	struct timeval svr_cbulk_ts; 	// timestamp of current bulk. If there are no ACKs after sending this bulk, it will expire and resend the packet. 
	// used by packet retransmission: if the diff between svr_cbulk_ts and current time is bigger than svr_cbulk_timeout, we'll retransmit the bulk.
	uint32 svr_cbulk_timeout; 

	struct timeval svr_hello_ts;
	uint32 svr_hello_timeout;
	
	uint64 svr_remain_slots;	// remain # of slots
	uint64 svr_totalbytes;		// total # of bytes which have been received
	BULKMAP svr_cbulkmap;		// current bulkmap

	// used to check the abnormal of sending ...
	uint32 signal_started;		// switch to control on/off of the signal
	// uint32 last_svr_cbulk; 		// , last_svr_cbulk_recvd = 0;

	int tcdsrv_debugger;		// debugger

// state definition at server side, used in server's FSM
#define SRV_STATE_INIT		1
#define SRV_STATE_READY		2
#define SRV_STATE_RUN		3
#define SRV_STATE_STOP		4

#define SRV_STATE_STR(x)	\
	(((x) == SRV_STATE_INIT)? "SRV_STATE_INIT":	\
	(((x) == SRV_STATE_READY)? "SRV_STATE_READY":	\
	(((x) == SRV_STATE_RUN)? "SRV_STATE_RUN":	\
	(((x) == SRV_STATE_STOP)? "SRV_STATE_STOP" : "UNKNOWN_SRV_STATE"	\
	))))

#ifdef STANDALONE_MODE
#define SET_SVR_STATE(p, x)	\
	{	\
		(p)->srv_state = (x); 	\
		printf("server: change state to [%s]\n", SRV_STATE_STR(x)); 	\
	}
#else
#define SET_SVR_STATE(p, x)	\
	{	\
		(p)->srv_state = (x); 	\
		tcd_syslog("server: change state to [%s]\n", SRV_STATE_STR(x)); 	\
	}
#endif

	uint32 srv_state;	// state of the server

	struct timeval starttv, endtv;
	struct timeval currtv, lastcurrtv;

	// newly added for periodically sending out hello packet in state init
	struct timeval init_state_tv;

	// added for checking runack timeout ... some client may not response during ready-run switching period.
	struct timeval ready_state_tv;

	uint32 svr_runack_rcvd; 

	CLT clts[MAXCLTS];
	int clts_count;		// total # of clients, including normal and clients w/o local hard disk, etc.
	int clts_normal_count;	// total # of normal clients. 

	ACTUSR actusrs[MAXACTUSRS];
	int actusrs_count;


// default value was defined in conf.h
	int intpkt_delay;

	// counter used for adjust the value of inter-packet delay when auto-control mechanism is enabled. 
	// if the # of consecutive packet sending rate reached MAX_SIGROUTE_SUCC, we will decrease the intpkt_delay by one to see if 
	// we can send the packet quicker, and vice versa, when the failure # of consecutive packet sending reached sigroute_fail_count, 
	// we will try to increase the intpkt_delay to reduce the packet sending rate.  
#define MAX_SIGROUTE_SUCC	128
	int sigroute_succ_count;
#define MAX_SIGROUTE_FAIL	32
	int sigroute_fail_count;

	// according to my experience, the optimized value of intpkt_delay (usec) for 6 Intel C1037 clients with 4G memory as TC client
	// will be 8~9 usec for non-packet loss, and 4-6 for packet loss but better performance. 
	// noted: the sending/receiving socket packet buffer will be 128KBytes

	char *svr_logfile; 

#define MAXF	32
	char *fn[MAXF];
	int fn_count;
	char fnbuf[1600];

	int fn_idx;

	// newly added to count all broadcast acknowledgements
	int brcack_count;
	// added by bright to count clients interrupted during initialization
	int interrupted_clts;

}SCB;

extern char tcdver[];


extern int regreq_buf(char **buf, char *localmac, uint32 machine_status);
extern int deregreq_buf(char **buf);
extern int regack_buf(char **buf);

extern int runreq_buf(char **buf, char *fname, uint32 u32tslots, uint32 hasnofile, uint32 pktack_delay_base, uint32 pktack_delay_coeff, uint32 pktack_delay_intpkt, uint32 dyna_mapsz, SCB *scbp);
extern int stopreq_buf(char **buf);
extern int runack_buf(char **buf, uint32 value);

extern int pktinq_buf(char **buf, uint32 bulk, uint32 slot);
extern int pktack_buf(char **buf, uint32 bulk, uint32 full, BULKMAP *bmp);
extern int data_buf(char **buf, uint32 bulk, uint32 slot, uint32 hasnext, SCB *scbp);
extern int hello_buf(char **buf, uint32 ip, uint16 port);

extern int proc_tcdmsg_clt(int socket, char *pkt, struct sockaddr_in sain, struct sockaddr_in groupsock, CCB *ccbp);
extern int proc_tcdmsg_srv(int socket, char *pkt, struct sockaddr_in sain, struct sockaddr_in groupsock, SCB *scbp);

extern int regreq_send(int socket, struct sockaddr_in sain, char *localmac, uint32 machine_status);
extern int deregreq_send(int socket, struct sockaddr_in sain);
extern int regack_send(int socket, struct sockaddr_in sain);
extern int runreq_send(int socket, struct sockaddr_in sain, char *fname, uint32 u32tslots, uint32 hasnofile, uint32 pktack_delay_base, uint32 pktack_delay_coeff, uint32 pktack_delay_intpkt, uint32 dyna_mapsz, SCB *scbp);
extern int stopreq_send(int socket, struct sockaddr_in sain);
extern int runack_send(int socket, struct sockaddr_in sain, uint32 value);
extern int pktack_send(int socket, struct sockaddr_in sain, uint32 bulk, uint32 full, BULKMAP *bmp);
extern int data_send(int socket, struct sockaddr_in sain, uint32 bulk, uint32 slot, uint32 hasnext, SCB *scbp);
// extern int hello_send(int socket, struct sockaddr_in sain, uint32 ip, uint16 port);
// extern int shutdown_send(int socket, struct sockaddr_in sain, uint32 ip, uint16 port);
// extern int reboot_send(int socket, struct sockaddr_in sain, uint32 ip, uint16 port);
extern int cmdrmimage_send(int socket, struct sockaddr_in sain, const char *image_org_name, uint32 image_id);
extern int hello_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port);
extern int shutdown_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port);
extern int reboot_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port);

extern int shutdown_ip_unicast(int socket, uint32 ip, uint16 port, const char *ipv4);
extern int reboot_ip_unicast(int socket, uint32 ip, uint16 port, const char* ipv4);
extern int boot_ip_unicast(int socket, uint32 svr_ip, uint16 svr_port, uint32 image_id, const char *image_org_name, const char *image_name, int image_per, /*uint32 user_id,*/ const char *user_token, const char *user_name, uint32 clt_ip);
extern int rm_image_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp, const char *image_org_name, uint32 image_id);
extern int set_resolution_unicast(int socket, const char *ipv4, int width, int height);
extern int usb_ctrl_unicast(int socket, const char *ipv4, int usb_enabled);

// defined in tcdmsgctl.c
extern int cmdstart_send(int socket, struct sockaddr_in sain, char **fn, int count);
extern int cmdrun_send(int socket, struct sockaddr_in sain);
extern int cmdlist_send(int socket, struct sockaddr_in sain);
extern int cmdquit_send(int socket, struct sockaddr_in sain);
extern int cmdreboot_send(int socket, struct sockaddr_in sain);
extern int cmdshutdown_send(int socket, struct sockaddr_in sain);

extern int cmdidentify_send(int socket, struct sockaddr_in sain);
extern int cmdboot_ip_send(int socket, struct sockaddr_in sain, uint32 image_id, const char *image_org_name, const char *image_name, const char *user_token, const char *user_name, uint32 clt_ip);
extern int cmdresol_ip_send(int socket, struct sockaddr_in sain, const char *ipv4, int width, int height);
extern int cmdusbctrl_ip_send(int socket, struct sockaddr_in sain, const char *ipv4, int usb_mode);

// defined in tcdmsgreg.c

extern int cmdchkregreq_send(int socket, struct sockaddr_in sain, char *localmac);
extern int cmdchkregack_send(int socket, struct sockaddr_in sain, char *localmac, int retval);
extern int cmdregreq_send(int socket, struct sockaddr_in sain, char *localmac, char *hostname);
extern int cmdregack_send(int socket, struct sockaddr_in sain, char *localmac, char *hostname, int retval);


#endif

