/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <sys/utsname.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <sys/sysinfo.h>

#include "conf.h"
#include "tcd.h"
#include "bmap.h"
#include "tcdmsg.h"
#include "cltdb.h"
#include "fl.h"
#include "vos_mem.h"


// defined in tcdclt.c
extern char *cifn, *localip, *localmac;
unsigned char *ifn2mac(char *ifn);


// routines to generate the message buffer
// register request
int regreq_buf(char **buf, char *localmac, uint32 machine_status)
{
	TCDMSG *msg;
	struct utsname utemp;
	struct sysinfo sinfo;

	if (uname(&utemp) == -1)
	{
		perror("uname");
		return -1;
	}

	if (sysinfo(&sinfo) == -1)
	{
		perror("sysinfo");
		return -1;
	}



	printf("memunit: [%d]\n", sinfo.mem_unit);
	printf("totalram: [%ld] freeram: [%ld] sharedram: [%ld] bufferram: [%ld]\n", \
		sinfo.totalram/KBYTES*sinfo.mem_unit, sinfo.freeram/KBYTES*sinfo.mem_unit, \
		sinfo.sharedram/KBYTES*sinfo.mem_unit, sinfo.bufferram/KBYTES*sinfo.mem_unit);
	printf("totalswap: [%ld] freeswap: [%ld] procs: [%d]\n", sinfo.totalswap/KBYTES*sinfo.mem_unit, sinfo.freeswap/KBYTES*sinfo.mem_unit, sinfo.procs);



	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_REGREQ;
	msg->size = BUFSZ;
	
	msg->u.rr.checkmnt = machine_status;

	memcpy(msg->u.rr.hwa, localmac, HWA_SZ);


	sprintf(tcdver, "BUILD on %s at %s %s by %s", MACHINE, __DATE__, __TIME__, AUTHOR);
	strcpy(msg->u.rr.tcdver, tcdver);

	strncpy(msg->u.rr.sysname, utemp.sysname, INFO_SZ);
	msg->u.rr.sysname[INFO_SZ-1]=0;

	strncpy(msg->u.rr.nodename, utemp.nodename, INFO_SZ);
	msg->u.rr.nodename[INFO_SZ-1]=0;

	strncpy(msg->u.rr.release, utemp.release, INFO_SZ);
	msg->u.rr.release[INFO_SZ-1]=0;

	strncpy(msg->u.rr.version, utemp.version, INFO_SZ);
	msg->u.rr.version[INFO_SZ-1]=0;

	strncpy(msg->u.rr.machine, utemp.machine, INFO_SZ);
	msg->u.rr.machine[INFO_SZ-1]=0;


	msg->u.rr.totalram = sinfo.totalram/KBYTES*sinfo.mem_unit;
	msg->u.rr.freeram = sinfo.freeram/KBYTES*sinfo.mem_unit;
	msg->u.rr.sharedram = sinfo.sharedram/KBYTES*sinfo.mem_unit;
	msg->u.rr.bufferram = sinfo.bufferram/KBYTES*sinfo.mem_unit;

	msg->u.rr.totalswap = sinfo.totalswap/KBYTES*sinfo.mem_unit;
	msg->u.rr.freeswap = sinfo.freeswap/KBYTES*sinfo.mem_unit;
	msg->u.rr.procs = sinfo.procs;

	return BUFSZ;
}

// de-register request
int deregreq_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_DEREGREQ;
	msg->size = BUFSZ;
	return BUFSZ;
}



// stop request
int stopreq_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_STOPREQ;
	msg->size = BUFSZ;
	return BUFSZ;
}


// packet acknowledgement
int pktack_buf(char **buf, uint32 bulk, uint32 full, BULKMAP *bmp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}

	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_PKTACK;
	msg->size = BUFSZ;

	msg->u.pa.bulk = bulk;
	msg->u.pa.full = full;

#ifdef USE_PKTACK_WITH_BMAP
	memcpy((unsigned char *)&(msg->u.pa.bulkmap), (unsigned char *)bmp, sizeof(BULKMAP));
#endif

	return BUFSZ;
}




// packet acknowledgement
int pktack_full_buf(char **buf, uint32 bulk, uint32 full)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_PKTACK;
	msg->size = BUFSZ;

	msg->u.pa.bulk = bulk;
	msg->u.pa.full = full;


	return BUFSZ;
}



// run acknowledgement
int runack_buf(char **buf, uint32 value)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_RUNACK;
	msg->size = BUFSZ;

	msg->u.ua.uadata = value;
	return BUFSZ;
}


// notify messages
int notify_buf(char **buf, uint32 nf_code, uint32 nf_comment)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_NOTIFY;
	msg->size = BUFSZ;

	msg->u.nf.nf_code = nf_code;
	msg->u.nf.nf_comment = nf_comment;

	return BUFSZ;
}


/***********************************************************************/
// routines to send the messages
// to send a register request message. 
int regreq_send(int socket, struct sockaddr_in sain, char *localmac, uint32 machine_status)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = regreq_buf(&msgbuf, localmac, machine_status);
	DBG("regreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: regreq_send\n");
		}
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "regreq_send: request buffer failed\n");
		return -1;
	}
}

// to send a de-registration request message. 
int deregreq_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = deregreq_buf(&msgbuf);
	DBG("deregreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: deregreq_send\n");
		}

		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "deregreq_send: request buffer failed\n");
		return -1;
	}
}


// to send a stop request message. 
int stopreq_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = stopreq_buf(&msgbuf);
	DBG("stopreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: stopreq_send\n");
		}

		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "stopreq_send: request buffer failed\n");
		return -1;
	}
}

// to send a packet acknowledgement message. 
int pktack_send(int socket, struct sockaddr_in sain, uint32 bulk, uint32 full, BULKMAP *bmp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = pktack_buf(&msgbuf, bulk, full, bmp);
	DBG("pktack_send: msgbuf_len=%d\n", msgbuf_len);
	// printf("pktack_send: bulk#=%d\n", bulk);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: in pktack_send\n");
		}
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "pktack_send: request buffer failed\n");
		return -1;
	}
}


// to send a packet acknowledgement message. 
int pktack_full_send(int socket, struct sockaddr_in sain, uint32 bulk, uint32 full)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = pktack_full_buf(&msgbuf, bulk, full);
	DBG("pktack_full_send: msgbuf_len=%d\n", msgbuf_len);
	// printf("pktack_send: bulk#=%d\n", bulk);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: pktack_full_send\n");
		};
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "pktack_full_send: request buffer failed\n");
		return -1;
	}
}


// to send a run acknowledgement message. 
int runack_send(int socket, struct sockaddr_in sain, uint32 value)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = runack_buf(&msgbuf, value);
	DBG("runack_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: runack_send\n");
		};
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "runack_send: request buffer failed\n");
		return -1;
	}
}


// to send a NOTIFY message. 
int notify_send(int socket, struct sockaddr_in sain, uint32 nf_code, uint32 nf_comment)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = notify_buf(&msgbuf, nf_code, nf_comment);
	DBG("notify_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto: notify_send\n");
		}
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "notify_send: request buffer failed\n");
		return -1;
	}
}








