/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "conf.h"
#include "tcd.h"
#include "bmap.h"
#include "tcdmsg.h"
#include "cltdb.h"
#include "fl.h"
#include "vos_mem.h"


// routines to generate the message buffer
// command start
int cmdstart_buf(char **buf, char **fn, int count)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDSTART;
	msg->size = BUFSZ;

	if (putfn(fn, count, msg->u.cs.fnbuf) == -1)
	{
		fprintf(stderr, "putfn: failed\n");
		return -1;
	}
	
	return BUFSZ;
}

// command startver
int cmdstartver_buf(char **buf, char **fnidver, int count)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDSTART;
	msg->size = BUFSZ;

	if (putfn(fnidver, count, msg->u.cs.fnbuf) == -1)
	{
		fprintf(stderr, "putfn: failed\n");
		return -1;
	}
	
	return BUFSZ;
}


// command list request
int cmdlist_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDLIST;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command quit request
int cmdquit_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDQUIT;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command reboot request
int cmdreboot_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDREBOOT;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command reboot request with a interal IPv4 payload
int cmdreboot_ip_buf(char **buf, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDREBOOT_IP;
	msg->size = BUFSZ;
	strcpy(msg->u.rbt_cmd_payload.ipv4, ipv4);
	return BUFSZ;
}

// command shutdown request
int cmdshutdown_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDSHUTDOWN;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command shutdown request
int cmdshutdown_ip_buf(char **buf, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDSHUTDOWN_IP;
	msg->size = BUFSZ;
	strcpy(msg->u.shd_cmd_payload.ipv4, ipv4);
	return BUFSZ;
}

//int cmdboot_ip_buf(char **buf, uint32 image_id, const char *image_org_name, const char *image_name, int image_per, uint32 user_id, const char *user_name, uint32 clt_ip)
int cmdboot_ip_buf(char **buf, uint32 image_id, const char *image_org_name, const char *image_name, const char *user_token, const char *user_name, uint32 clt_ip)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDBOOT_IP;
	msg->size = BUFSZ;
	msg->u.cmd_bootinfo.image_id = image_id;
	strncpy(msg->u.cmd_bootinfo.image_org_name, image_org_name, IMAGE_NAME_SZ - 1);
	msg->u.cmd_bootinfo.image_org_name[IMAGE_NAME_SZ - 1] = '\0';
	strncpy(msg->u.cmd_bootinfo.image_name, image_name, IMAGE_NAME_SZ - 1);
	msg->u.cmd_bootinfo.image_name[IMAGE_NAME_SZ - 1] = '\0';
	//msg->u.cmd_bootinfo.image_per = image_per;
	//msg->u.cmd_bootinfo.user_id = user_id;
	strncpy(msg->u.cmd_bootinfo.user_token, user_token, USER_TOKEN_SZ - 1);
	msg->u.cmd_bootinfo.user_token[USER_TOKEN_SZ - 1] = '\0';
	strncpy(msg->u.cmd_bootinfo.user_name, user_name, USER_NAME_SZ - 1);
	msg->u.cmd_bootinfo.user_name[USER_NAME_SZ - 1] = '\0';
	msg->u.cmd_bootinfo.clt_ip = clt_ip;
	return BUFSZ;
}


// command restartapp request
int cmdrestartapp_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDRESTARTAPP;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command identify request
int cmdidentify_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDIDENTIFY;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command format request
int cmdformat_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDFORMAT;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command initialize request
int cmdinitialize_buf(char **buf, uint32 sz)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDINITIALIZE;
	msg->size = BUFSZ;
	msg->u.ciz.sys_part_capacity = sz;
	return BUFSZ;
}

// command send initialized unicast message
int cmdinitialize_ip_buf(char **buf, uint32 sz, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDINITIALIZE_IP;
	msg->size = BUFSZ;
	msg->u.init_cmd_payload.sys_part_capacity = sz;
	strcpy(msg->u.init_cmd_payload.ipv4, ipv4);
	return BUFSZ;
}

// command updatefirmware request
int cmdupdatefirmware_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDUPDATEFIRMWARE;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command updatefirmware request
int cmdupdatefirmware_ip_buf(char **buf, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDUPDATEFIRMWARE_IP;
	msg->size = BUFSZ;
	strcpy(msg->u.uf_cmd_payload.ipv4, ipv4);
	return BUFSZ;
}


// command user request
int cmduser_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDUSER;
	msg->size = BUFSZ;
	return BUFSZ;
}

// command change server IP
int cmdcgsvrip_buf(char **buf, uint32 srvip)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDCGSVRIP;
	msg->size = BUFSZ;

	msg->u.ccgs.srvip = srvip;

	return BUFSZ;
}

int cmdrmimage_buf(char **buf, const char *image_org_name, uint32 image_id)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDRM_IMAGE;
	msg->size = BUFSZ;
        msg->u.rm_image_cmd_payload.image_id = image_id;
	strncpy (msg->u.rm_image_cmd_payload.image_org_name, image_org_name, IMAGENAME_SZ);
	msg->u.rm_image_cmd_payload.image_org_name[IMAGENAME_SZ - 1] = '\0';
	return BUFSZ;

}

/***********************************************************************/
// routines to send the messages
// to send a cmdstart request message. 
int cmdstart_send(int socket, struct sockaddr_in sain, char **fn, int count)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdstart_buf(&msgbuf, fn, count);
	DBG("cmdstart_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdstart_send: request buffer failed\n");
		return -1;
	}
}


// to send a cmdstartver request message. 
// fnidver is something like "<filename>:<osid>:<ver>"
int cmdstartver_send(int socket, struct sockaddr_in sain, char **fnidver, int count)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdstartver_buf(&msgbuf, fnidver, count);
	DBG("cmdstartver_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdstartver_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdlist message. 
int cmdlist_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdlist_buf(&msgbuf);
	DBG("cmdlist_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdlist_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdquit message. 
int cmdquit_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdquit_buf(&msgbuf);
	DBG("cmdquit_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdquit_send: request buffer failed\n");
		return -1;
	}
}


// to send a cmdreboot message. 
int cmdreboot_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdreboot_buf(&msgbuf);
	DBG("cmdreboot_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdreboot_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdreboot message, with a internal IPv4 payload. 
int cmdreboot_ip_send(int socket, struct sockaddr_in sain, const char* ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdreboot_ip_buf(&msgbuf, ipv4);
	DBG("cmdreboot_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdreboot_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdshutdown message. 
int cmdshutdown_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdshutdown_buf(&msgbuf);
	DBG("cmdshutdown_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdshutdown_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdshutdown message, with a internal IPv4 payload 
int cmdshutdown_ip_send(int socket, struct sockaddr_in sain, const char* ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdshutdown_ip_buf(&msgbuf, ipv4);
	DBG("cmdshutdown_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdshutdown_send: request buffer failed\n");
		return -1;
	}
}

// to send a boot message
//int cmdboot_ip_send(int socket, struct sockaddr_in sain, uint32 image_id, const char *image_org_name, const char *image_name, int image_per, uint32 user_id, const char *user_name, uint32 clt_ip)
int cmdboot_ip_send(int socket, struct sockaddr_in sain, uint32 image_id, const char *image_org_name, const char *image_name, const char *user_token, const char *user_name, uint32 clt_ip)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//msgbuf_len = cmdboot_ip_buf(&msgbuf, image_id, image_org_name, image_name, image_per, user_id, user_name, clt_ip);
	msgbuf_len = cmdboot_ip_buf(&msgbuf, image_id, image_org_name, image_name, user_token, user_name, clt_ip);
	DBG("cmdboot_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdshutdown_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdrestartapp message. 
int cmdrestartapp_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdrestartapp_buf(&msgbuf);
	DBG("cmdrestartapp_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdshutdown_send: request buffer failed\n");
		return -1;
	}
}




// to send a cmdidentify message
int cmdidentify_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdidentify_buf(&msgbuf);
	DBG("cmdidentify_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdidentify_send: request buffer failed\n");
		return -1;
	}
}


// to send a cmdformat message
int cmdformat_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdformat_buf(&msgbuf);
	DBG("cmdformat_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdformat_send: request buffer failed\n");
		return -1;
	}
}


// to send a cmdinitialize message
int cmdinitialize_send(int socket, struct sockaddr_in sain, uint32 partsz)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdinitialize_buf(&msgbuf, partsz);
	DBG("cmdinitialize_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdinitialize_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmd initialize command
int cmdinitialize_ip_send(int socket, struct sockaddr_in sain, uint32 partsz, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdinitialize_ip_buf(&msgbuf, partsz, ipv4);
	DBG("cmdinitialize_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdinitialize_ip_send: request buffer failed\n");
		return -1;
	}

}

// to send a cmdupdatefirmware message
int cmdupdatefirmware_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdupdatefirmware_buf(&msgbuf);
	DBG("cmdupdatefirmware_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdupdatefirmware_send: request buffer failed\n");
		return -1;
	}
}

// to send a cmdupdatefirmware message to a client with specified IPv4
int cmdupdatefirmware_ip_send(int socket, struct sockaddr_in sain, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdupdatefirmware_ip_buf(&msgbuf, ipv4);
	DBG("cmdupdatefirmware_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdupdatefirmware_ip_send: request buffer failed\n");
		return -1;
	}
}



// to send a cmduser message
int cmduser_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//printf("entering cmduser_send ...\n");
	msgbuf_len = cmduser_buf(&msgbuf);
	DBG("cmduser_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmduser_send: request buffer failed\n");
		return -1;
	}
}


// to send a cmd change server ip message
int cmdcgsvrip_send(int socket, struct sockaddr_in sain, uint32 svrip)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//printf("entering cmdcgsvrip_send ...\n");
	msgbuf_len = cmdcgsvrip_buf(&msgbuf, svrip);
	DBG("cmdcgsvrip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdcgsvrip_send: request buffer failed\n");
		return -1;
	}
}


int cmdrmimage_send(int socket, struct sockaddr_in sain, const char *image_org_name, uint32 image_id)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdrmimage_buf(&msgbuf, image_org_name, image_id);
	DBG("cmdrmimage_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdinitialize_send: request buffer failed\n");
		return -1;
	}
}


// command send change-server unicast message
int cmdcgsvrip_ip_buf(char **buf, uint32 svrip, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDCGSVRIP_IP;
	msg->size = BUFSZ;
	msg->u.ccgsip.srvip = svrip;
	strcpy(msg->u.ccgsip.ipv4, ipv4);
	return BUFSZ;
}


// to send a cmd change-server ip command to a specified client
int cmdcgsvrip_ip_send(int socket, struct sockaddr_in sain, uint32 svrip, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//printf("entering cmdcgsvrip_ip_send ...\n");
	msgbuf_len = cmdcgsvrip_ip_buf(&msgbuf, svrip, ipv4);
	DBG("cmdcgsvrip_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdcgsvrip_ip_send: request buffer failed\n");
		return -1;
	}  

}


// command send remove-image unicast message
int cmdrmimage_ip_buf(char **buf, const char *image_org_name, uint32 image_id, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDRM_IMAGE_IP;
	msg->size = BUFSZ;
	strncpy(msg->u.rm_image_ip_cmd_payload.image_org_name, image_org_name, IMAGENAME_SZ);
    msg->u.rm_image_ip_cmd_payload.image_org_name[IMAGENAME_SZ - 1] = '\0';
	strcpy(msg->u.rm_image_ip_cmd_payload.ipv4, ipv4);
    msg->u.rm_image_ip_cmd_payload.ipv4[STRIP_SZ - 1] = '\0';
    msg->u.rm_image_ip_cmd_payload.image_id = image_id;
	return BUFSZ;
}


// to send a cmd remove-image ip command to a specified client
int cmdrmimage_ip_send(int socket, struct sockaddr_in sain, const char *image_org_name, uint32 image_id, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//printf("entering cmdrmimage_ip_send ...\n");
	msgbuf_len = cmdrmimage_ip_buf(&msgbuf, image_org_name, image_id, ipv4);
	DBG("cmdrmimage_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdrmimage_ip_send: request buffer failed\n");
		return -1;
	}  

}

// command set-resolution request with a interal IPv4 payload
int cmdresol_ip_buf(char **buf, const char *ipv4, int width, int height)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDSET_RESOLUTION_IP;
	msg->size = BUFSZ;
    msg->u.set_resolution_ip_cmd_payload.screen_width = width;
    msg->u.set_resolution_ip_cmd_payload.screen_height = height;
	strcpy(msg->u.set_resolution_ip_cmd_payload.ipv4, ipv4);
	return BUFSZ;
}

// to send a cmd set-resolution ip command to a specified client
int cmdresol_ip_send(int socket, struct sockaddr_in sain, const char *ipv4, int width, int height)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//printf("entering cmdresol_ip_send ...\n");
	msgbuf_len = cmdresol_ip_buf(&msgbuf, ipv4, width, height);
	DBG("cmdresol_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdresol_ip_send: request buffer failed\n");
		return -1;
	}  

}

// command enable-usb-storage request with a interal IPv4 payload
int cmdusbctrl_ip_buf(char **buf, const char *ipv4, int usb_mode)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDUSB_STORAGE_CTRL_IP;
	msg->size = BUFSZ;
    msg->u.usb_storage_ctrl_ip_cmd_payload.usb_enabled = usb_mode;
	strcpy(msg->u.usb_storage_ctrl_ip_cmd_payload.ipv4, ipv4);
	return BUFSZ;
}

// to send an enable-usb-storage command to a specified client
int cmdusbctrl_ip_send(int socket, struct sockaddr_in sain, const char *ipv4, int usb_mode)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	//printf("entering cmdusbctrl_ip_send ...\n");
	msgbuf_len = cmdusbctrl_ip_buf(&msgbuf, ipv4, usb_mode);
	DBG("cmdusbctrl_ip_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdresol_ip_send: request buffer failed\n");
		return -1;
	}  

}

