/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "conf.h"
#include "tcd.h"
#include "bmap.h"
#include "tcdmsg.h"
#include "cltdb.h"
#include "fl.h"
#include "vos_mem.h"
#include "cltinfo.h"

// routines to generate the message buffer
// command chkregreq
int cmdchkregreq_buf(char **buf, unsigned char *localmac)
{
	TCDMSG *msg;

//	printf("===>> in cmdchkreg_buf: mac address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", localmac[0], localmac[1], localmac[2], localmac[3], localmac[4], localmac[5]);


	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDCHKREGREQ;
	msg->size = BUFSZ;

	memcpy(msg->u.cr.hwa, localmac, HWA_SZ);
	msg->u.cr.retval = 0;
	
	return BUFSZ;
}


// command chkregack
int cmdchkregack_buf(char **buf, unsigned char *localmac, int retval)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDCHKREGACK;
	msg->size = BUFSZ;

	memcpy(msg->u.cr.hwa, localmac, HWA_SZ);
	msg->u.cr.retval = retval;
	
	return BUFSZ;
}

// command regreq
int cmdregreq_buf(char **buf, char *localmac, char *hostname)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDREGREQ;
	msg->size = BUFSZ;

	memcpy(msg->u.rg.hwa, localmac, HWA_SZ);
	if (hostname!=0)
	{
		/* code */
		strcpy(msg->u.rg.hname, hostname);
	}
	
	msg->u.rg.retval = 0;

	msg->u.rg.diskcapacity = disk_capacity_mib ("/dev/sda");
	msg->u.rg.totalram = phy_memory ();
	cpu_model_name (msg->u.rg.cpumodelname, sizeof (msg->u.rg.cpumodelname));

	return BUFSZ;
}






// command regack
int cmdregack_buf(char **buf, char *localmac, char *hostname, int retval)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDREGACK;
	msg->size = BUFSZ;

	memcpy(msg->u.rg.hwa, localmac, HWA_SZ);
	strcpy(msg->u.rg.hname, hostname);
	msg->u.rg.retval = retval;

	return BUFSZ;
}


// command upregreq
int cmdupregreq_buf(char **buf, char *localmac)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDUPREGREQ;
	msg->size = BUFSZ;

	memcpy(msg->u.cur.hwa, localmac, HWA_SZ);
	msg->u.cur.retval = 0;

	msg->u.cur.diskcapacity = disk_capacity_mib ("/dev/sda");
	msg->u.cur.totalram = phy_memory ();
	cpu_model_name (msg->u.cur.cpumodelname, sizeof (msg->u.cur.cpumodelname));

	return BUFSZ;
}


// command update-reg-ack
int cmdupregack_buf(char **buf, char *localmac, char *hostname, int retval)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CMDUPREGACK;
	msg->size = BUFSZ;

	memcpy(msg->u.cur.hwa, localmac, HWA_SZ);
	strcpy(msg->u.cur.hname, hostname);
	msg->u.cur.retval = retval;

	return BUFSZ;
}








/***********************************************************************/
// routines to send the messages
// to send a cmd chkregreq message. 
int cmdchkregreq_send(int socket, struct sockaddr_in sain, char *localmac)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdchkregreq_buf(&msgbuf, localmac);
	DBG("cmdchkregreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdchkreg_send: request buffer failed\n");
		return -1;
	}
}


// to send a cmd chkregack message. 
int cmdchkregack_send(int socket, struct sockaddr_in sain, char *localmac, int retval)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdchkregack_buf(&msgbuf, localmac, retval);
	DBG("cmdchkregack_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdchkregack_send: request buffer failed\n");
		return -1;
	}
}



// to send a regreq message. 
int cmdregreq_send(int socket, struct sockaddr_in sain, char *localmac, char *hostname)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdregreq_buf(&msgbuf, localmac, hostname);
	DBG("cmdregreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdregreq_send: request buffer failed\n");
		return -1;
	}
}




// to send a regack message. 
int cmdregack_send(int socket, struct sockaddr_in sain, char *localmac, char *hostname, int retval)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdregack_buf(&msgbuf, localmac, hostname, retval);
	DBG("cmdregack_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdregack_send: request buffer failed\n");
		return -1;
	}
}



// to send a upregreq message. 
int cmdupregreq_send(int socket, struct sockaddr_in sain, char *localmac)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdupregreq_buf(&msgbuf, localmac);
	DBG("cmdregreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdregreq_send: request buffer failed\n");
		return -1;
	}
}



// to send a upregack message. 
int cmdupregack_send(int socket, struct sockaddr_in sain, char *localmac, char *hostname, int retval)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cmdupregack_buf(&msgbuf, localmac, hostname, retval);
	DBG("cmdupregack_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cmdregreq_send: request buffer failed\n");
		return -1;
	}
}


