/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "conf.h"
#include "tcd.h"
#include "bmap.h"
#include "tcdmsg.h"
#include "cltdb.h"
#include "fl.h"
#include "vos_mem.h"


extern SCB scb; 

// register acknowledgement
int regack_buf(char **buf)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_REGACK;
	msg->size = BUFSZ;
	return BUFSZ;
}




// hello acknowledgement
int hello_buf(char **buf, uint32 ip, unsigned short port)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_HELLO;
	msg->size = BUFSZ;

	msg->u.h.srvip = ip;
	msg->u.h.srvport = port;
	return BUFSZ;
}


// shutdown acknowledgement
int shutdown_buf(char **buf, uint32 ip, unsigned short port)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_SHUTDOWN;
	msg->size = BUFSZ;

	msg->u.sd.srvip = ip;
	msg->u.sd.srvport = port;
	return BUFSZ;
}


// shutdown a client
int shutdown_ip_buf(char **buf, uint32 ip, unsigned short port, const char* ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_SHUTDOWN_IP;
	msg->size = BUFSZ;

	msg->u.sd_ip.srvip = ip;
	msg->u.sd_ip.srvport = port;
	strcpy (msg->u.sd_ip.ipv4, ipv4);
	return BUFSZ;
}


// reboot acknowledgement
int reboot_buf(char **buf, uint32 ip, unsigned short port)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_REBOOT;
	msg->size = BUFSZ;

	msg->u.rb.srvip = ip;
	msg->u.rb.srvport = port;
	return BUFSZ;
}

// reboot a client
int reboot_ip_buf(char **buf, uint32 ip, unsigned short port, const char* ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_REBOOT_IP;
	msg->size = BUFSZ;

	msg->u.rb_ip.srvip = ip;
	msg->u.rb_ip.srvport = port;
	strcpy (msg->u.rb_ip.ipv4, ipv4);
	return BUFSZ;
}

// boot a client
int boot_ip_buf(char **buf, uint32 svr_ip, uint16 svr_port, uint32 image_id, const char *image_org_name, const char *image_name, int image_per, /*uint32 user_id,*/ const char *user_token, const char *user_name, uint32 clt_ip)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_BOOT_IP;
	msg->size = BUFSZ;

	msg->u.bootinfo.svrip = svr_ip;
	msg->u.bootinfo.svrport = svr_port;
	msg->u.bootinfo.image_id = image_id;
	strncpy(msg->u.bootinfo.image_name, image_name, IMAGE_NAME_SZ - 1);
	msg->u.bootinfo.image_name[IMAGE_NAME_SZ - 1] = '\0';
	strncpy(msg->u.bootinfo.image_org_name, image_org_name, IMAGE_NAME_SZ - 1);
	msg->u.bootinfo.image_org_name[IMAGE_NAME_SZ - 1] = '\0';
	msg->u.bootinfo.image_per = image_per;
	//msg->u.bootinfo.user_id = user_id;
	strncpy(msg->u.bootinfo.user_token, user_token, USER_TOKEN_SZ - 1);
	msg->u.bootinfo.user_token[USER_TOKEN_SZ - 1] = '\0';
	strncpy(msg->u.bootinfo.user_name, user_name, USER_NAME_SZ - 1);
	msg->u.bootinfo.user_name[USER_NAME_SZ - 1] = '\0';
	msg->u.bootinfo.clt_ip = clt_ip;
	return BUFSZ;
}

// restart app acknowledgement
int restart_app_buf(char **buf, uint32 ip, unsigned short port)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_RESTARTAPP;
	msg->size = BUFSZ;

	msg->u.rsa.srvip = ip;
	msg->u.rsa.srvport = port;
	return BUFSZ;
}

// identify broadcast buffer
int identify_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_IDENTIFY;
	msg->size = BUFSZ;

	msg->u.id.srvip = ip;
	msg->u.id.srvport = port;

//	msg->u.id.clt_count = put_clts(scbp, &(msg->u.id.clt_list[0]));

	return BUFSZ;
}


// format broadcast buffer
int format_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_FORMAT;
	msg->size = BUFSZ;

	msg->u.id.srvip = ip;
	msg->u.id.srvport = port;

//	msg->u.id.clt_count = put_clts(scbp, &(msg->u.id.clt_list[0]));

	return BUFSZ;
}

// initialize broadcast buffer
int initialize_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp, uint32 partsz)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_INITIALIZE;
	msg->size = BUFSZ;

	msg->u.id.srvip = ip;
	msg->u.id.srvport = port;
	msg->u.id.sys_part_capacity = partsz;

	return BUFSZ;
}

// initialize broadcast buffer
int initialize_ip_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp, uint32 partsz, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_INITIALIZE_IP;
	msg->size = BUFSZ;

	msg->u.init_payload.srvip = ip;
	msg->u.init_payload.srvport = port;
	msg->u.init_payload.sys_part_capacity = partsz;
	strcpy(msg->u.init_payload.ipv4, ipv4);

	return BUFSZ;

}


// updatefirmware broadcast buffer
int updatefirmware_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_UPDATEFIRMWARE;
	msg->size = BUFSZ;

	msg->u.id.srvip = ip;
	msg->u.id.srvport = port;

	return BUFSZ;
}

// updatefirmware unicast buffer
int updatefirmware_ip_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp, const char *ipv4)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_UPDATEFIRMWARE_IP;
	msg->size = BUFSZ;

	msg->u.uf_payload.srvip = ip;
	msg->u.uf_payload.srvport = port;
	strcpy(msg->u.uf_payload.ipv4, ipv4);

	return BUFSZ;
}

// usercmd broadcast buffer
int usercmd_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_USERCMD;
	msg->size = BUFSZ;

	msg->u.uc.srvip = ip;
	msg->u.uc.srvport = port;

	msg->u.uc.clt_count = put_clts_ipcbulk(scbp, &(msg->u.uc.clt_ipcbulk_list[0]));

	return BUFSZ;
}


// pktack check broadcast buffer
int pktack_check_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_PKTACK_CHECK;
	msg->size = BUFSZ;

	msg->u.pac.srvip = ip;
	msg->u.pac.srvport = port;

	msg->u.pac.clt_count = put_clts_ipcbulk(scbp, &(msg->u.pac.clt_ipcbulk_list[0]));

	return BUFSZ;
}


// packet inquiry
int pktinq_buf(char **buf, uint32 bulk, uint32 slot)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_PKTINQ;
	msg->size = BUFSZ;

	msg->u.pi.bulk = bulk;
	msg->u.pi.slot = slot;

	return BUFSZ;
}


// data buffer
int data_buf(char **buf, uint32 bulk, uint32 slot, uint32 hasnext, SCB *scbp)
{
	TCDMSG *msg;
	int retval;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}

	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_DATA;
	msg->size = BUFSZ;

	msg->u.d.bulk = bulk;
	msg->u.d.slot = slot;
	msg->u.d.hasnext = hasnext;

	retval = fl_read_slot(flr_fd, (bulk*TOTALSZ+slot), msg->u.d.data);
	msg->u.d.bytes = retval;
	scbp->svr_totalbytes += retval;

	return BUFSZ;
}



// run request
int runreq_buf(char **buf, char *fname, uint32 u32tslots, uint32 hasnofile, uint32 pktack_delay_base, uint32 pktack_delay_coeff, uint32 pktack_delay_intpkt, uint32 dyna_mapsz, SCB *scbp)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_RUNREQ;
	msg->size = BUFSZ;

	if (fname != NULL)
		strcpy(msg->u.ur.fname, fname);
	else
		strcpy(msg->u.ur.fname, "\0");

	msg->u.ur.u32tslots = u32tslots;
	msg->u.ur.hasnofile = hasnofile;

	msg->u.ur.pktack_delay_base = pktack_delay_base;
	msg->u.ur.pktack_delay_coeff = pktack_delay_coeff;
	msg->u.ur.pktack_delay_intpkt = pktack_delay_intpkt;
	msg->u.ur.dyna_mapsz = dyna_mapsz;

	msg->u.ur.clt_count = put_clts(scbp, &(msg->u.ur.clt_list[0]));

	return BUFSZ;
}


// cgsvrip broadcast buffer
int cgsvrip_buf(char **buf, uint32 svrip)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CGSVRIP;
	msg->size = BUFSZ;

	msg->u.cgs.srvip = svrip;
	return BUFSZ;
}

// cgsvrip unicast buffer
int cgsvrip_ip_buf(char **buf, uint32 svrip)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_CGSVRIP_IP;
	msg->size = BUFSZ;

	msg->u.cgsip.srvip = svrip;
	return BUFSZ;
}

// remove image broadcast buffer
int rm_image_buf(char **buf, uint32 ip, unsigned short port, SCB *scbp, const char *image_org_name, uint32 image_id)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_RM_IMAGE;
	msg->size = BUFSZ;

	msg->u.rm_image_payload.srvip = ip;
	msg->u.rm_image_payload.srvport = port;
	msg->u.rm_image_payload.image_id = image_id;
	strcpy(msg->u.rm_image_payload.image_org_name, image_org_name);

	return BUFSZ;
}

// remove image unicast buffer
int rm_image_ip_buf(char **buf, SCB *scbp, const char *image_org_name, uint32 image_id)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_RM_IMAGE_IP;
	msg->size = BUFSZ;

	msg->u.rm_image_ip_payload.image_id = image_id;
	strncpy(msg->u.rm_image_ip_payload.image_org_name, image_org_name, IMAGENAME_SZ);
	msg->u.rm_image_ip_payload.image_org_name[IMAGENAME_SZ - 1 ] = '\0';

	return BUFSZ;
}

// set-resolution unicast buffer
int set_resolution_ip_buf(char **buf, int width, int height)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_SET_RESOLUTION_IP;
	msg->size = BUFSZ;

	msg->u.set_resolution_ip_payload.screen_width = width;
	msg->u.set_resolution_ip_payload.screen_height = height;

	return BUFSZ;
}

// usb_ctrl unicast buffer
int usb_ctrl_ip_buf(char **buf, int usb_enabled)
{
	TCDMSG *msg;

	if ((*buf = (char *)vos_malloc())==NULL)
	{	
		perror("malloc");
		return -1;
	}
	msg = (TCDMSG *)*buf;
	msg->type = TCDMSG_USB_STORAGE_CTRL_IP;
	msg->size = BUFSZ;

	msg->u.usb_storage_ctrl_ip_payload.usb_enabled = usb_enabled;

	return BUFSZ;
}




/***********************************************************************/
// routines to send the messages


// to send a register acknowledgement message. 
int regack_send(int socket, struct sockaddr_in sain)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = regack_buf(&msgbuf);
	DBG("regack_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "regack_send: request buffer failed\n");
		return -1;
	}
}



// to send a slot (bulk#, slot#), and if it's the last slot of the file, hasnext will be 0, or will be 1. 
int data_send(int socket, struct sockaddr_in sain, uint32 bulk, uint32 slot, uint32 hasnext, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = data_buf(&msgbuf, bulk, slot, hasnext, scbp);
	DBG("data_send: msgbuf_len=%d bulk: %ld, slot: %ld, hasnext: %ld\n", msgbuf_len, bulk, slot, hasnext);

	if (msgbuf_len > 0)
	{
		if (sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len) == -1)
		{
			perror("sendto");
		}
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "data_send: request buffer failed\n");
		return -1;
	}
}

// to broadcast the hello packet from server to client. 
// mostly the sain will be multicast or broadcast address
int hello_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = hello_buf(&msgbuf, ip, port);
	DBG("hello_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "hello_broadcast: request buffer failed\n");
		return -1;
	}
}

// to broadcast the shutdown packet from server to client. 
// mostly the sain will be multicast or broadcast address
int shutdown_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = shutdown_buf(&msgbuf, ip, port);
	DBG("shutdown_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "shutdown_broadcast: request buffer failed\n");
		return -1;
	}
}

// to send the shutdown packet from server to client with specified ip. 
int shutdown_ip_unicast(int socket, uint32 ip, uint16 port, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;

	msgbuf_len = shutdown_ip_buf(&msgbuf, ip, port, ipv4);
	DBG("shutdown_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		// unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "shutdown_unicast: request buffer failed\n");
		return -1;
	}
}

// to send the shutdown packet from server to client with specified ip. 
int boot_ip_unicast(int socket, uint32 svr_ip, uint16 svr_port, uint32 image_id, const char *image_org_name, const char *image_name, int image_per, /*uint32 user_id,*/ const char *user_token, const char *user_name, uint32 clt_ip)
{
	char *msgbuf;
	int msgbuf_len;

	msgbuf_len = boot_ip_buf(&msgbuf, svr_ip, svr_port, image_id, image_org_name, image_name, image_per, /*user_id,*/ user_token, user_name, clt_ip);
	DBG("boot_ip_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		// unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = clt_ip;
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "boot_ip_unicast: request buffer failed\n");
		return -1;
	}

}

// to broadcast the reboot packet from server to client. 
// mostly the sain will be multicast or broadcast address
int reboot_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = reboot_buf(&msgbuf, ip, port);
	DBG("reboot_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "reboot_broadcast: request buffer failed\n");
		return -1;
	}
}

// to send the reboot packet from server to client with specified IP. 
int reboot_ip_unicast(int socket, uint32 ip, uint16 port, const char* ipv4)
{
	char *msgbuf;
	int msgbuf_len;

	msgbuf_len = reboot_ip_buf(&msgbuf, ip, port, ipv4);
	DBG("reboot_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		// unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "reboot_unicast: request buffer failed\n");
		return -1;
	}
}


// to broadcast the restart application packet from server to client. 
// mostly the sain will be multicast or broadcast address
int restart_app_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = restart_app_buf(&msgbuf, ip, port);
	DBG("restart_app_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "reboot_broadcast: request buffer failed\n");
		return -1;
	}
}




// to broadcast the identify packet from server to client. 
int identify_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = identify_buf(&msgbuf, ip, port, scbp);
	DBG("identify_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "identify_broadcast: request buffer failed\n");
		return -1;
	}
}


// to broadcast the format packet from server to client. 
int format_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = format_buf(&msgbuf, ip, port, scbp);
	DBG("format_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "format_broadcast: request buffer failed\n");
		return -1;
	}
}


// to broadcast the initialize command packet from server to client. 
int initialize_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp, uint32 sz)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = initialize_buf(&msgbuf, ip, port, scbp, sz);
	DBG("initialize_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "initialize_broadcast: request buffer failed\n");
		return -1;
	}
}

// to send the initialize command packet from server to client with specified IPv4. 
int initialize_unicast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp, uint32 sz, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;

	msgbuf_len = initialize_ip_buf(&msgbuf, ip, port, scbp, sz, ipv4);
	DBG("initialize_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		// unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "initialize_unicast: request buffer failed\n");
		return -1;
	}
}

// to broadcast the updatefirmware packet from server to client. 
int updatefirmware_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = updatefirmware_buf(&msgbuf, ip, port, scbp);
	DBG("updatefirmware_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "updatefirmware_broadcast: request buffer failed\n");
		return -1;
	}
}

// to unicast the updatefirmware packet from server to client with IPv4
int updatefirmware_unicast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;

	msgbuf_len = updatefirmware_ip_buf(&msgbuf, ip, port, scbp, ipv4);
	DBG("updatefirmware_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		// unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "updatefirmware_unicast: request buffer failed\n");
		return -1;
	}
}


// to broadcast the user-customized command packet from server to client. 
int usercmd_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = usercmd_buf(&msgbuf, ip, port, scbp);
	DBG("usercmd_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "usercmd_broadcast: request buffer failed\n");
		return -1;
	}
}

// to broadcast the pktack check packet from server to client. 
int pktack_check_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = pktack_check_buf(&msgbuf, ip, port, scbp);
	DBG("pktack_check_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "pktack_check_broadcast: request buffer failed\n");
		return -1;
	}
}

// to send a run request message. 
int runreq_send(int socket, struct sockaddr_in sain, char *fname, uint32 u32tslots, uint32 hasnofile, uint32 pktack_delay_base, uint32 pktack_delay_coeff, uint32 pktack_delay_intpkt, uint32 dyna_mapsz, SCB *scbp)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = runreq_buf(&msgbuf, fname, u32tslots, hasnofile, pktack_delay_base, pktack_delay_coeff, pktack_delay_intpkt, dyna_mapsz, scbp);
	DBG("runreq_send: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "runreq_send: request buffer failed\n");
		return -1;
	}
}


// to broadcast the cgsvrip packet from server to client. 
// mostly the sain will be multicast or broadcast address
int cgsvrip_broadcast(int socket, struct sockaddr_in sain, uint32 svrip)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = cgsvrip_buf(&msgbuf, svrip);
	DBG("cgsvrip_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cgsvrip_broadcast: request buffer failed\n");
		return -1;
	}
}

// to broadcast the rm-image command packet from server to client. 
int rm_image_broadcast(int socket, struct sockaddr_in sain, uint32 ip, uint16 port, SCB *scbp, const char *image_org_name, uint32 image_id)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = rm_image_buf(&msgbuf, ip, port, scbp, image_org_name, image_id);
	DBG("rm_image_broadcast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&sain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "rm_image_broadcast: request buffer failed\n");
		return -1;
	}
}


// to unicast the change-server ip packet from server to client with IPv4
int cgsvrip_unicast(int socket, struct sockaddr_in sain, uint32 svrip, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;

	msgbuf_len = cgsvrip_ip_buf(&msgbuf, svrip);
	DBG("cgsvrip_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
		// unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "cgsvrip_unicast: request buffer failed\n");
		return -1;
	}
}

// to unicast the rm-image command packet from server to client. 
int rm_image_unicast(int socket, struct sockaddr_in sain, SCB *scbp, const char *image_org_name, uint32 image_id, const char *ipv4)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len = sizeof(sain);

	msgbuf_len = rm_image_ip_buf(&msgbuf, scbp, image_org_name, image_id);
	DBG("rm_image_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
    // unicast
		struct sockaddr_in unisain;
		int sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "rm_image_unicast: request buffer failed\n");
		return -1;
	}
}

// to unicast the set-resolution command packet from server to client. 
int set_resolution_unicast(int socket, const char *ipv4, int width, int height)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len;
    struct sockaddr_in unisain;

	msgbuf_len = set_resolution_ip_buf(&msgbuf, width, height);
	DBG("set_resolution_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
        // unicast
		sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "set_resolution_unicast: request buffer failed\n");
		return -1;
	}
}

// to unicast the enable-usb-storage command packet from server to client. 
int usb_ctrl_unicast(int socket, const char *ipv4, int usb_enabled)
{
	char *msgbuf;
	int msgbuf_len;
	int sain_len;
    struct sockaddr_in unisain;

	msgbuf_len = usb_ctrl_ip_buf(&msgbuf, usb_enabled);
	DBG("usb_ctrl_unicast: msgbuf_len=%d\n", msgbuf_len);

	if (msgbuf_len > 0)
	{
        // unicast
		sain_len = sizeof(unisain);
		unisain.sin_family = AF_INET;
		unisain.sin_addr.s_addr = inet_addr(ipv4);
		unisain.sin_port = htons(CLT_PORT);

		sendto(socket, msgbuf, msgbuf_len, 0, (struct sockaddr *)&unisain, sain_len);
		vos_free(msgbuf);
		return 0;
	}
	else
	{
		fprintf(stderr, "set_resolution_unicast: request buffer failed\n");
		return -1;
	}
}
