/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


/* TC Deployment: a small tool to check if the local client has already been registered */

#include "conf.h"

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <termios.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>

#include "tcd.h"
#include "cltdb.h"
#include "tcdmsg.h"
#include "term.h"

#include "bmap.h"
#include "fl.h"

#include "tm.h"
#include "vos_mem.h"


unsigned char *ifn2mac(char *ifn);


//uint64 diff;

//char *flw_name=NULL, flw_dir[256];

// defined in ifn2ipadd.c
extern char *ifn2ip_str(char *ifn);
extern char *ifn2mac_str(char *ifn);


void tcdreg_usage(char *argv0)
{
	printf("\nUsage: ");

	printf("\n\t%s <svrip> --chkreg \n\t\tto check whether this client has been registered\n", argv0);
	printf("\t\t    example: %s 192.168.0.100 --chkreg", argv0);
	printf("\n\t%s <svrip> --reg <hostname>\n\t\tto register this client with host name <hostname>\n", argv0);
	printf("\t\t    example: %s 192.168.0.100 --reg host168", argv0);
	printf("\n\t%s <svrip> --update \n\t\tto update entries of a registered client to server\n", argv0);
	printf("\t\t    example: %s 192.168.0.100 --update", argv0);
	printf("\n");

	return;
}



// message processing in client part
int proc_tcdmsg_reg(int socket, char *pkt, struct sockaddr_in singlesock)
{
	TCDMSG *msg = (TCDMSG *)pkt;
	char *msgbuf;
	int msgbuf_len;
	CLT *cp;
	uint32 localipadd;
	uint32 maxcbulk=0;
	int retval;

	switch(msg->type)
	{
		case TCDMSG_CMDCHKREGACK:
			printf("proc_tcdmsg_reg: chkreg ack\n");
			retval = msg->u.cr.retval;
			// -1 means unregistered.
			if (retval == -1)
				retval = 1;
			break;

		case TCDMSG_CMDREGACK:
			printf("proc_tcdmsg_reg: reg ack\n");
			retval = msg->u.rg.retval;
			break;

		case TCDMSG_CMDUPREGACK:
			printf("proc_tcdmsg_reg: reg-update ack \n");
			retval = msg->u.cur.retval;
			break;

		default:
			printf("unknown tcdmsg type @ client: [%d]\n", msg->type);
			retval = -1;
			break;
	}

	return retval;
}






// reg command line processing
/*
int reg_cmdline_proc(char ch)
{
	switch(ch)
	{
		case 'h':
			tcdreg_usage("tcdreg");
			break;

		case 'q':
			printf("client: change state to stop\n");
			return -1;
			break;

		default: 
			DBG("clt_cmdline_proc: unknown char\n");
			break;
	}

	return 0;
}
*/


/* client FSM main routine: 
   cskt: the control socket. It's used to talk with server's control plane. 
   dskt: the data socket. It's used to receive server's data plane packets. 
   servadd: server's control plane information
 */
int reg_fsm_main(int sock, struct sockaddr_in singlesock)
{
	fd_set rdfds;
	struct timeval tv;
	int res, retval;
	char ch;
	char buf[2048];
	// struct termios new_settings, stored_settings;

	printf("enter reg_fsm_main() loop ...\n");

	// to enable the raw terminal mode so we can use hotkey ...
	// term_raw();


	// the while deadloop(), which can only be out if 
	//	clt_cmdline_proc() returns -1, or 
	// 	proc_tcdmsg_clt() returns -1. 
	while(1)
	{
		// add network socket and stdin to the FDSET
		FD_ZERO(&rdfds);
		// FD_SET(0, &rdfds);
		FD_SET(sock, &rdfds);

		// and set the select timeout to forever
		tv.tv_sec = 0;
		tv.tv_usec = DEFAULT_REG_TIMEOUT; 

		res = select(sock+1, &rdfds, NULL, NULL, &tv);
		// DBG("res=%d\n", res);
		
		if (res > 0)
		{
			// stdin events
			/*
			if (FD_ISSET(0, &rdfds))
			{
				ch = getchar();
				printf("0x%x [%c]\n", ch, ch);
				if (reg_cmdline_proc(ch) == -1)
				{
					retval = -1;
					break;
				}
			}
			else 
			*/
			// socket events handling
			if (FD_ISSET(sock, &rdfds))
			{
				// receiving data packets until the last packet ...
				printf("start receiving messages ...\n");

				// change here for making it work in uclibc platform
				// if (recvfrom(sock, buf, sizeof(buf), 0, (struct sockaddr *)&sain, &sain_len) != -1)
				if (recv(sock, buf, sizeof(buf), 0) != -1)
				{
					// printf("0x%x : 0x%x : 0x%x\n", sain.sin_family, sain.sin_addr.s_addr, sain.sin_port);
					printf("Receive from [%s:%d]: \n", inet_ntoa(singlesock.sin_addr),  htons(singlesock.sin_port));
					//if (proc_tcdmsg_reg(sock, buf, singlesock) == -1)
					//	break;
					retval = proc_tcdmsg_reg(sock, buf, singlesock);
					break;
				}
				else
					printf("reg_fsm_main: recvfrom failed!\n");
			}
		}
		else if (res == 0)
		{
			printf("reg_fsm_main: select timeout\n");
			retval = 2;
			break;
		}
		else
		{
			perror("reg_fsm_main: select()");
			retval = -1;
		}


	}

	// to restore the terminal mode
	// term_restore();	

	return retval;
}

// format: tcdreg <svrip> --chkreg , to check whether this client has been registered
//	   tcdreg <svrip> --reg <hostname>, to register this client with host name <hostname> 

/* for --chkreg: the return value will be extremely important
	0: already registered
	1: have not been registered yet
	2: timeout, most likely server did not start or network was disconnected
	-1: something wrong
*/
int main(int argc, char **argv)
{
	char *cifn, *localip;
	unsigned char *localmac;

	int sock;
	char buf[2048];
	char *dstip_str=NULL, *hostname_str=NULL;
	int dstport;
	int retval;

	cifn = "eth0";

	// parameter handling
	if (argc == 3 && !strcmp(argv[2], "--chkreg"))	// tcdreg <svrip> --chkreg
	{
		dstip_str = argv[1];
		dstport = 12345;
	}
	else if (argc == 3 && !strcmp (argv[2], "--update")) // tcdreg <svrip> --update
	{
		dstip_str = argv[1];
		dstport = 12345;
	}
	else if (argc == 4 && !strcmp(argv[2], "--reg"))	// tcdreg <svrip> --reg <hostname>
	{
		dstip_str = argv[1];
		dstport = 12345;
		hostname_str = argv[3];
	}
	else
	{
		int k;

		printf("original command: ");
		for (k=0; k<argc; k++)
			printf("%s ", argv[k]);
		printf("\n");
		tcdreg_usage(argv[0]);
		exit(-1);
	}

	if ((localmac = (unsigned char *)ifn2mac(cifn)) == NULL)
	{
		perror("ifn2mac");
		exit(-1);
	}
	printf("======>>>>> mac address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", localmac[0], localmac[1], localmac[2], localmac[3], localmac[4], localmac[5]);


	// to create the data socket. the hello packet will also be from this socket. 
	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
	{
		perror("socket");
		exit(-1);
	}
	printf("tcdreg: socket successfully ...\n");

	// initialize the memory ...
	if (vos_meminit() == -1)
	{
		fprintf(stderr, "vos_meminit: failed\n");
		close(sock);
		exit(-1);
	}


	struct sockaddr_in singlesock;
	bzero(&singlesock, sizeof(singlesock));
	singlesock.sin_family = AF_INET;
	singlesock.sin_addr.s_addr = inet_addr(dstip_str);
	singlesock.sin_port = htons(dstport);

	// newly added for register ...
	if (!strcmp(argv[2], "--chkreg"))
	{
		printf("check if the client is registered already\n");
		cmdchkregreq_send(sock, singlesock, localmac);
	}
	else if (!strcmp(argv[2], "--update"))
	{
		printf("update a client\n");
		cmdupregreq_send(sock, singlesock, localmac);
	}
	else if (!strcmp(argv[2], "--reg"))
	{
		printf("register a client\n");
		cmdregreq_send(sock, singlesock, localmac, hostname_str);
	}


	retval = reg_fsm_main(sock, singlesock);

	// to close the control socket	
	printf("tcdclt: close the socket\n");
	close(sock);

	printf("retval=%d\n", retval);

	exit(retval);
}


