/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/


/* TC Deployment Server (the Sender) */

#include "conf.h"
#include "tcd.h"

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <termios.h>

#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "cltdb.h"
#include "tcdmsg.h"
#include "term.h"
#include "fl.h"
#include "bmap.h"

#include "tm.h"
#include "vos_mem.h"

#ifdef USE_MYSQL
#include "db.h"
#include "cfgsmalllib.h"
#endif

// have to declare as the global variable for sigroutine(). will try to move to main()
// int sock;
// struct sockaddr_in groupsock;
//int sport;
//char *sifn, *localip;
//char *tcdrunpidfn;
//char *flr_name_idver=NULL;

// the entity of the server control block
// the definition of SCB is in tcdmsg.h
SCB scb; 

#define IFSTR_SZ        32
#define HWASTR_SZ       18
#define HWADATA_LEN      6
#define IPSTR_SZ        16
#define PIDFN_SZ        256
int                g_sockfd                        = -1;
struct sockaddr_in g_group_sockaddr_in             = {0};
char               g_if_name_str[IFSTR_SZ]         = {0};
static char        g_hwa_str[HWASTR_SZ]            = {0};
static char        g_hwa_data[HWADATA_LEN]         = {0};
char               g_ip_str[IPSTR_SZ]              = {0};
in_addr_t          g_ip_ineta                      = 0;
static char        g_pid_filename_str[PIDFN_SZ]    = {0};
int                g_port                          = 0;
char              *g_image_triple_info             = NULL;

extern int         g_tm_base;
extern int         g_tm_coeff;
extern int         g_tm_intpktdelay;


extern CLT *find_clts(SCB *scbp, uint32 ip, uint16 port);
extern void clearcbulk_clts(SCB *scbp);

// defined in ifn2ipadd.c
//extern char *ifn2ip_str(char *ifn);
extern char *ifn2ip_str_ex(char *dest_ip, int dest_ip_sz, const char *ifn);
extern char *ifn2mac_str_ex(char *dest_hwa, int dest_hwa_sz, const char *ifn);
extern char *get_tcdpid_fn_ex(char *dest_pid_filename, int dest_pid_filename_sz, const char *config_filename);

void tcd_cleanup_resource(){
    tcd_syslog("try removing file %s\n", g_pid_filename_str);
    fl_unlink(g_pid_filename_str);

    tcd_syslog("try removing file %s\n", TCD_WLOCK_FILE);
    fl_unlink(TCD_WLOCK_FILE);
}

// variable which should be set for each file transfer
void clear_scb(SCB *scbp)
{
	scbp->svr_cbulk = 0;
	scbp->svr_remain_slots = 0;
	scbp->svr_totalbytes = 0;

	// used to check the abnormal of sending ...
	// scbp->last_svr_cbulk = 0; // , last_svr_cbulk_recvd = 0;

	scbp->svr_runack_rcvd = 0;
	scbp->brcack_count = 0;

	// newly added
	clear_cbulk_clts(scbp);

	return;
}


// to initialize the server control block
void init_scb(SCB *scbp)
{
	int k;

// variables for each file transfer. 
	// the default value ...
	scbp->svr_cbulk_timeout = DEFAULT_SVR_CBULK_TIMEOUT; 
	scbp->svr_hello_timeout = DEFAULT_SVR_HELLO_TIMEOUT;
	scbp->svr_hello_ts = getts();

	// disable the server debugger
	scbp->tcdsrv_debugger = 0;

	// default log file, defined in conf.h
	scbp->svr_logfile = TCDLOGFN;	

	for (k=0; k<MAXF; k++)
		scbp->fn[k] = NULL;
	scbp->fn_count = 0;
	scbp->fn_idx = 0;


	// # of clients
	scbp->clts_count = 0;
	init_clts(scbp);

	// # of actusrs, used for OS runtime ... 
	scbp->actusrs_count = 0;
	init_actusrs(scbp);
	

	scbp->intpkt_delay = g_tm_intpktdelay; //DEFAULT_INTPKT_DELAY;

	// set the initial state
	scbp->srv_state = SRV_STATE_INIT;

	scbp->sigroute_succ_count = 0;
	scbp->sigroute_fail_count = 0;

	clear_scb(scbp);

	return;
}


// help for the server
void srv_help()
{
#ifdef STANDALONE_MODE

	printf("\n");
	printf("==============================================\n");
	printf("tcd server help: \n\n");
	printf("h: print the help information\n");
	printf("l: list the client informations\n");
	printf("d: turn on/off the debugger\n");
	printf("c: check the current status\n");
	printf("p: pause the machine, and press p again for reset\n");
	printf("b: reboot the machine\n");
	printf("0: poweroff all clients\n");
	printf("f: format and initialize the hard disks of clients and make it TC-bootable\n");
	printf("1: restart all ciients' tcdclt application\n");
	printf("v: print the build and version info\n");
	printf("q: quit\n");
	printf("==============================================\n");
	printf("\n");
#endif

}


char *vos_path2fname(char *path)
{
	char *p;

	p = strrchr(path, '/');
	return p+1;
}


void print_svr_status(SCB *scbp)
{
	// bit per second, and percent of bulks which have been sent
	double bps, bulk_pcnt;
	struct timeval tv;
    uint64 diff;


#ifdef STANDALONE_MODE
	printf("==============================================\n");

	bulk_pcnt = (double)scbp->svr_cbulk * (double)TOTALSZ / (double)(scbp->svr_cbulk*TOTALSZ + scbp->svr_remain_slots) * 100.0;
	printf("current bulk#: %d\nremain bulks: %lld\nsend bytes: %lld\n", scbp->svr_cbulk, scbp->svr_remain_slots/TOTALSZ, scbp->svr_totalbytes);
	printf("%8.2f%% bulks have been sent\n", bulk_pcnt);

	// to print the sending speed: from the first slot to now. 
	tv = getts();
	diff = diff_tv(scbp->starttv, tv);
	bps = (double)scbp->svr_totalbytes*8*ONEMILLION/(double)diff;

	if (bps >= ONEMILLION)
		printf("Speed: %8.2fMbps\n\n", bps/ONEMILLION);
	else if (bps >= ONETHOUSAND)
		printf("Speed: %8.2fKbps\n\n", bps/ONETHOUSAND);
	else
		printf("Speed: %8.2fbps\n\n", bps);

	// to print the server parameter list
	printf("server parameter list: ");
	printf("default intpacket delay [%d]\n", g_tm_intpktdelay /*DEFAULT_INTPKT_DELAY*/);

	#ifdef USE_PKTSND_AUTO_ADJUST
	printf("packet send rate-limit [enabled]\n");
	printf("sigroute success/fail range [%d-%d]\n", MAX_SIGROUTE_SUCC, MAX_SIGROUTE_FAIL);
	#else
	printf("packet send rate-limit [disabled]\n");
	#endif

	#ifdef USE_PKTACK_WITH_BMAP
	printf("slot-based packet retransmission [enabled]\n");
	#else
	printf("slot-based packet retransmission [disabled]\n");
	#endif

	#ifdef USE_OS_MALLOC
	printf("use system memory allocation mechanism\n");
	#else
	printf("use self-made memory allocation mechanism\n");
	#endif

	printf("sigroute interval [%d] usec\n", TIMEOUT_INTERVAL);
	printf("current bulk loss checking timeout [%d] usec, default [%d] usec\n", scbp->svr_cbulk_timeout, DEFAULT_SVR_CBULK_TIMEOUT);


	#ifdef PRINT_INSTANT_SPEED
	printf("instant speed display: enabled\n");
	#else
	printf("instant speed display: disabled\n");
	#endif

	printf("sending buffer size [%d] receiving buffer size [%d]\n", SND_BUF_SZ, RCV_BUF_SZ);
	printf("slot bytes [%d]\n", SLOT_BYTES);

	printf("==============================================\n");
#endif

	return;

}

/* used for data plane. send a bulk via the group socket. 
   socket, gsain and gsain_len: the network socket and group socket
   bulk: the bulk # 
   slots: # of slots in the bulk. mostly it will be TOTALSZ. The last bulk will be smaller than that. 
*/
void send_bulk(int socket, struct sockaddr_in gsain, int gsain_len, uint32 bulk, uint32 slots, uint32 hasnext, SCB *scbp)
{
	uint32 slot;

	log_printf("send_bulk: bulk#: [%d] slots: [%d] hasnext: [%d]\n", bulk, slots, hasnext);

	slot = -1;

	// all bulk will not be the last one, will append every slot with "has next"
	if (hasnext > 0)
	{
		while (++ slot < slots)
		{
			// to send the data packet
			data_send(socket, gsain, bulk, slot, 1, scbp);
			vos_usleep(scbp->intpkt_delay);
		}
	}
	else
	// the last slot's next will be 0, so (slots-1) slots will be "has next" but the last slot will be "no next"
	{
		while (++ slot < slots-1)
		{
			// to send the data packet
			data_send(socket, gsain, bulk, slot, 1, scbp);
			vos_usleep(scbp->intpkt_delay);
		}
		// last slot will be no-next.
		data_send(socket, gsain, bulk, slot, 0, scbp);
		vos_usleep(scbp->intpkt_delay);
	}

	return;
}


void send_bulk_according2bulkmap(int socket, struct sockaddr_in gsain, int gsain_len, uint32 bulk, uint32 slots, uint32 hasnext, BULKMAP *bmp, SCB *scbp)
{
	uint32 slot, count=0;

	log_printf("send_bulk_according2bulkmap: bulk#: [%ld] slots: [%ld] hasnext: [%ld]\n", bulk, slots, hasnext);

	slot = -1;

	// all bulk will not be the last one, will append every slot with "has next"
	if (hasnext > 0)
	{
		while (++ slot < slots)
		{
			if (isclear_bulkmap(bmp, slot) || slot == (TOTALSZ-1) )
			{
				// this bit is 0 (slot missing), have to send the data packet
				// printf("send slot #%ld\n", slot);
				data_send(socket, gsain, bulk, slot, 1, scbp);
				vos_usleep(scbp->intpkt_delay);
				count++;
			}
		}
	}
	else
	// the last slot's next will be 0, so (slots-1) slots will be "has next" but the last slot will be "no next"
	{
		while (++ slot < slots-1)
		{
			if (isclear_bulkmap(bmp, slot))
			{
				// printf("send slot #%ld\n", slot);
				// this bit is 0 (slot missing), have to send the data packet
				data_send(socket, gsain, bulk, slot, 1, scbp);
				vos_usleep(scbp->intpkt_delay);
				count++;
			}
		}

		// last slot will be no-next.
		data_send(socket, gsain, bulk, slot, 0, scbp);
		vos_usleep(scbp->intpkt_delay);

		count++;
	}
#ifdef _DEBUG_
	printf("send_bulk_according2bulkmap: append-send %d slots\n", count);
#endif

	return;
}


// print the instant transmit info ...
void print_svr_instant_speed(SCB *scbp, char *fn)
{
    char imagename[FNAME_SZ] = {0};
	uint64 diff;
	double bps, bulk_pcnt;

	scbp->lastcurrtv = scbp->currtv; 
	scbp->currtv = getts();							
	diff = (uint64)diff_tv(scbp->lastcurrtv, scbp->currtv);
	
	bps = (double)(TOTALSZ * SLOT_BYTES * SAMPLE_FREQ * 8)/(double)diff  * ONEMILLION;

#ifdef STANDALONE_MODE
	fflush(stdout);
	if (bps >= ONEMILLION)
		printf("\rInstant Sending Speed [%s]: %8.2fMbps", fn, bps/ONEMILLION);
	else if (bps >= ONETHOUSAND)
		printf("\rInstant Sending Speed [%s]: %8.2fKbps", fn, bps/ONETHOUSAND);
	else
		printf("\rInstant Sending Speed [%s]: %8.2fbps", fn, bps);
#endif

	bulk_pcnt = (double)scbp->svr_cbulk * (double)TOTALSZ / (double)(scbp->svr_cbulk*TOTALSZ + scbp->svr_remain_slots) * 100.0;
#ifdef STANDALONE_MODE
	printf("   %8.2f%% ", bulk_pcnt);
#endif


#ifdef USE_MYSQL
    fl_fname_ex(imagename, sizeof(imagename), g_image_triple_info);
	db_update_pcnt_info(imagename, (int)bulk_pcnt, bps);
#endif

	return;
}

// print the summary transmit statistics info ...
void print_svr_summary_speed(SCB *scbp, char *fn)
{
    char imagename[FNAME_SZ] = {0};
	uint64 diff;
	double bps;

#ifdef STANDALONE_MODE
	scbp->endtv = getts();							
	diff = diff_tv(scbp->starttv, scbp->endtv);
	bps = (double)scbp->svr_totalbytes*8*ONEMILLION/(double)diff;
	printf("\n\nSummary for [%s]: \ntotal bytes: %lld total seconds: %8.2f \n", fl_fname_ex(imagename, sizeof(imagename), g_image_triple_info), scbp->svr_totalbytes, (double)diff/1000000.0);

	if (bps >= ONEMILLION)
		printf("transmit speed: %8.2fMbps\n\n", bps/ONEMILLION);
	else if (bps >= ONETHOUSAND)
		printf("transmit speed: %8.2fKbps\n\n", bps/ONETHOUSAND);
	else
		printf("transmit speed: %8.2fbps\n\n", bps);
#endif


#ifdef USE_MYSQL
    fl_fname_ex(imagename, sizeof(imagename), g_image_triple_info);
	db_update_pcnt_info(imagename, (int)100, bps);
#endif

	return;
}

// to parse the file list from the incoming message to the SCB data structure
// return 0 for successful and -1 for failure
int parse_filelist(SCB *scbp, TCDMSG *msg)
{
	int k;

	if (scbp == NULL || msg == NULL)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "parse_filelist: NULL scbp or msg\n");
#endif
		return -1;
	}

	memcpy(scbp->fnbuf, msg->u.cs.fnbuf, 1460);

	scbp->fn_count = getfn(scbp->fnbuf, scbp->fn);
	if (scbp->fn_count <= 0)
	{
		// parse failed, so break switch-case and return 0 normally
		// and wait for the next CMDSTART

#ifdef STANDALONE_MODE
		printf("parse failed or empty file list: %d\n", scbp->fn_count);
#endif
		return -1; 
	}


#ifdef STANDALONE_MODE
	// to print the file list
	printf("========================\n");
	printf("total [%d] files\n", scbp->fn_count);
	for (k=0; k<scbp->fn_count; k++)
		printf("%d --- %s\n", k, scbp->fn[k]);
	printf("========================\n");
#endif
	// initialize the index #
	scbp->fn_idx = 0;

	return 0;
}


// get the next valid file name from the scb list.
char *getnextfn_from_filelist(SCB *scbp)
{
	char *fn;
	char imagename[FNAME_SZ] = {0};

	// find next file to be sent, and check whether it exists ...
	while (scbp->fn_idx < scbp->fn_count)
	{
		fn = scbp->fn[scbp->fn_idx];
		if (fl_stat(fl_fname_ex(imagename, sizeof(imagename), fn)) == -1)
		{
#ifdef STANDALONE_MODE
			fprintf(stderr, "proc_tcdmsg_srv: file [%s] not found\n", fn);
#endif
			tcd_syslog("proc_tcdmsg_srv: file [%s] not found\n", fn);
			scbp->fn_idx ++;
		}
		else
			break;	// find one seemingly
	}

	// have file to send
	if (scbp->fn_idx < scbp->fn_count)
	{
		fn = scbp->fn[scbp->fn_idx];
		scbp->fn_idx ++;
		return fn;
	}
	else
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "getfn_from_filelist: no additional file\n");
#endif
		tcd_syslog("getfn_from_filelist: no additional file\n");
		return NULL;
	}
}


// to open the file and send out the run-req message to kick off the file sending
void kickoff_filesending(SCB *scbp, int socket, char *fn_idver)
{
    char imagename[FNAME_SZ] = {0};
    fl_fname_ex(imagename, sizeof(imagename), fn_idver);

	// transmit first file ...
	scbp->svr_runack_rcvd = 0;

	// the multicast way of sending a file. trigger the first bulk sending via IPC message
	// get the # of slots of this file
	scbp->svr_remain_slots = fl_slots(imagename); // DEFAULT_TEST_REMAIN_SLOTS;

#ifdef STANDALONE_MODE
	printf("read file slots: %lld [0x%llx]\n", scbp->svr_remain_slots, scbp->svr_remain_slots);
#endif

	// open the requested file
#ifdef STANDALONE_MODE
	printf("fl_open: <%s>\n", );
#endif
	flr_fd = fl_open(imagename);

#ifdef STANDALONE_MODE
	printf("kickoff_sendfile: runreq broadcast ...\n");
#endif


	runreq_send(socket, g_group_sockaddr_in, vos_path2fname(fn_idver), scbp->svr_remain_slots, 0, g_tm_base /*RANDOM_DELAY_BASE*/, g_tm_coeff, g_tm_intpktdelay, MAPSZ, scbp);

#ifdef USE_MYSQL
	db_update_pcnt_info(imagename, (int)0, 0.0);

	char evt_text[EVT_TEXT_SZ];
	sprintf(evt_text, "start send file [%s] to [%d] clients", imagename, scbp->clts_normal_count);				
	db_insert_deploy_history("info", evt_text);

#endif


	return;
}


// packet acknowledgement handling for all clients
int handle_pktack_from_all_client(int sockfd, struct sockaddr_in grp_sockaddr, SCB *scbp)
{
    char imagename[FNAME_SZ] = {0};
    fl_fname_ex(imagename, sizeof(imagename), g_image_triple_info);

	// all pktack recvd, and all full
	if (all_client_rcvd_and_full_clts(scbp))
	{
#ifdef PRINT_INSTANT_SPEED
		// to print the instant sending speed
		if (scbp->svr_cbulk % SAMPLE_FREQ == SAMPLE_FREQ-1)
			print_svr_instant_speed(scbp, imagename);
#endif

		DBG("=====>>>>> BULK #[%d] FULL AT CLIENTS\n", scbp->svr_cbulk);
		log_printf("=====>>>>> BULK #[%d] FULL AT CLIENTS\n", scbp->svr_cbulk);

		// for debugging by logging the cbulk# list
		log_clts(scbp);

		// increase the cbulk counter and reset counters like recvd, full and notfull. 
		// also clear the server's bulkmap ...
		scbp->svr_cbulk ++;
		scbp->svr_cbulk_timeout = DEFAULT_SVR_CBULK_TIMEOUT; 

		// clear clt's cbulkfull status bit
		clear_cbulkfull_status_clts(scbp);

#ifdef USE_PKTACK_WITH_BMAP
		init_bulkmap(&scbp->svr_cbulkmap);
#endif

		scbp->svr_remain_slots -= TOTALSZ;
		// printf("svr_remain_slots=%d\n", svr_remain_slots);
		if ((int)scbp->svr_remain_slots <= 0)
		{
			DBG("have send all slots, close the file and go out\n");
			fl_close(flr_fd);
			tcd_syslog("file [%s] sending done\n", imagename);


			// not needed						
			// scbp->signal_started = 0;

			print_svr_summary_speed(scbp, imagename);

#ifdef USE_MYSQL
			char evt_text[EVT_TEXT_SZ];
			sprintf(evt_text, "finish send file [%s] to [%d] clients", imagename, scbp->clts_count);				
			db_insert_deploy_history("info", evt_text);

			// reset the status of each client to waiting.
			db_update_deploy_client_all_waiting();
			db_update_user_status_all_waiting();
#endif


// wming
			g_image_triple_info = getnextfn_from_filelist(scbp);
			if (g_image_triple_info != NULL) // have file to send
			{
#ifdef STANDALONE_MODE
				printf("still have file to send ...\n");
#endif

				// clear scb
				clear_scb(scbp);

				SET_SVR_STATE(scbp, SRV_STATE_READY);
				scbp->ready_state_tv = getts();

				tcd_syslog("send a file [%s] with version [%d]\n", imagename, fl_ver(g_image_triple_info));

				kickoff_filesending(scbp, sockfd, g_image_triple_info);
			}
			// all files have been transferred. 
			else
			{
#ifdef STANDALONE_MODE
				printf("all file transmitted, quit ...\n");
#endif
				// send a packet with hasnofile bit set, so client will quit.
				runreq_send(sockfd, grp_sockaddr, NULL, 0, 1, g_tm_base /*RANDOM_DELAY_BASE*/, g_tm_coeff, g_tm_intpktdelay, MAPSZ, scbp);
				SET_SVR_STATE(scbp, SRV_STATE_STOP);

				return -1;
			}
		}
		// This bug blocked me a couple of days... :( 
		// While completing sending out a file and the next file is a small one, I met a strange scenario that
		// when server sent out runreq, it will accidently receive pktack packet. It's caused by when server detected
		// the end of sending one file, it does not break out. Root cause is the missing of the below else {} statement. 
		else
		{
			uint32 slotstobesend = (scbp->svr_remain_slots <= TOTALSZ)?scbp->svr_remain_slots:TOTALSZ;
			send_bulk(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), scbp);
			scbp->svr_cbulk_ts = getts();
		}
	}

	// all clients have received the bulk, but some missed several slots
	// else if (scbp->svr_cbulk_recvd == scbp->clts_count)
	else if (all_client_rcvd_pktack_clts(scbp))
	{
		log_printf("=====>>>>> CLTS NOT ALL FULL for BULK #[%d], resend ...\n", scbp->svr_cbulk);

		tcd_syslog("clts not all full for bulk# [%d]\n", scbp->svr_cbulk);

#ifdef _DEBUG_
		print_bulkmap(&scbp->svr_cbulkmap);
#ifdef STANDALONE_MODE
		printf("recvd: %d notfull: %d\n", scbp->svr_cbulk_recvd, scbp->svr_cbulk_notfull);
#endif
#endif
		// for debugging by logging the cbulk# list
		log_clts(scbp);

		// wming: while not all bulks were received, we'd re-send the bulk and adjust the counter.
		// vos_msleep(100);	// sleep 0.5 second
		// vos_random_msleep();	// random sleep 0.01~0.99s

		// printf("resend bulk #[%d] due to slot loss\n", scbp->svr_cbulk);
		log_printf("resend bulk #[%d] due to slot loss\n", scbp->svr_cbulk);

		// to resend the whole bulk again --- can be optimized by selectively sending the missing slots
		uint32 slotstobesend = (scbp->svr_remain_slots <= TOTALSZ)?scbp->svr_remain_slots:TOTALSZ;
#ifdef USE_PKTACK_WITH_BMAP
		send_bulk_according2bulkmap(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), &scbp->svr_cbulkmap, scbp);
#else
		send_bulk(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), scbp);
#endif

		scbp->svr_cbulk_ts = getts();
	}

	return 0;
}


/*

Here list the brief description of the state machine at server side: 

There are four states: init, ready, run and stop. 

1, init: the very beginning state of the server. At this state, the server can accept the registration request from clients. 
a) accept registration request from clients, and take notes accordingly. 
b) when pressing 's' key, will go to state ready. 
c) when pressing 'q' key, will go to state 'stop' 

2, ready: the server is ready to run. In this state the server cannot accept the new incoming registration request from clients. 
a) when pressing 'r' key, will send out a runreq message to all clients. 
b) when getting the runack message from clients, it will go to state 'run' and start to send/receive data packets. 
c) when pressing 'q' key, it will go to state 'stop' to terminate the data sending/receiving. 

3, run: the running state of the server, where the massive inbounding and outbounding packet processing happens here. 
a) when pressing 'q' key, will go to state 'stop' 
b) when getting de-registration request message, will go to state 'stop'

4, stop: All packet transfer was done. Also the server can accept the de-registration requests from clients. 
a) when pressing 's' key, will go to state 'ready' 

*/


// message processing in server part
int proc_tcdmsg_srv(int sockfd, char *pkt, struct sockaddr_in sain, struct sockaddr_in grp_sockaddr, SCB *scbp)
{
	TCDMSG *msg = (TCDMSG *)pkt;
	CLT *clt;
	double bps, bulk_pcnt;
	char *fn[32], *hn;
	unsigned char *mac;
	int count, retval;

#ifdef USE_MYSQL
	char evt_text[EVT_TEXT_SZ];
#endif

	// printf("enter proc_tcdmsg_srv...\n");

	switch(msg->type)
	{
		// received the incoming registration requests from client. 
		case TCDMSG_REGREQ:
			if (scbp->srv_state != SRV_STATE_INIT)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: receive REGREQ in state [%s], should be [INIT]\n", SRV_STATE_STR(scbp->srv_state));
#endif
				break;
			}
#ifdef STANDALONE_MODE
			printf("=====> Regreq from client [%s:%d] with hwa [%02X:%02X:%02X:%02X:%02X:%02X]\n", \
				inet_ntoa(sain.sin_addr), htons(sain.sin_port), \
				msg->u.rr.hwa[0], msg->u.rr.hwa[1], msg->u.rr.hwa[2], msg->u.rr.hwa[3], msg->u.rr.hwa[4], msg->u.rr.hwa[5]);
			printf("tcdver: [%s]\nsysname: [%s] nodename: [%s] release: [%s] version: [%s] machine: [%s]\n", \
				msg->u.rr.tcdver, msg->u.rr.sysname, msg->u.rr.nodename, msg->u.rr.release, msg->u.rr.version, msg->u.rr.machine);

			printf("totalram: [%ld] freeram: [%ld] sharedram: [%ld] bufferram: [%ld]\n", \
				msg->u.rr.totalram, msg->u.rr.freeram, msg->u.rr.sharedram, msg->u.rr.bufferram);
			printf("totalswap: [%ld] freeswap: [%ld] procs: [%ld]\n", msg->u.rr.totalswap, msg->u.rr.freeswap, msg->u.rr.procs);
#endif

			DBG("register request\n");
			// in = inet_addr(src_ipadd);
			DBG("add: 0x%x\n", htonl(sain.sin_addr.s_addr));
			log_printf("register request %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
			tcd_syslog("register request %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));

			if (msg->u.rr.checkmnt == 0)
			{
#ifdef STANDALONE_MODE
				printf("current clients: [%d] with mounted local hard drive\n", count_clts(scbp));
#else
				tcd_syslog("current clients: [%d] with mounted local hard drive\n", count_clts(scbp));
#endif

				// successful
				insert_clts(scbp, sain.sin_addr.s_addr, sain.sin_port, msg->u.rr.hwa, CLT_MACHINE_STATUS_NORMAL);
				#ifdef USE_MYSQL
				db_update_user_status(msg->u.rr.hwa, sain.sin_addr.s_addr, "0", "0", ACT_WAITING);
				#endif
			}
			else
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "register request %s:%d: mount local hard drive failed, kicked out\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#else
				tcd_syslog("register request %s:%d: mount local hard drive failed, kicked out\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif

				insert_clts(scbp, sain.sin_addr.s_addr, sain.sin_port, msg->u.rr.hwa, CLT_MACHINE_STATUS_UNMOUNT);

				#ifdef USE_MYSQL
				// will change when the correct status was defined by jumper. 
				db_update_user_status(msg->u.rr.hwa, sain.sin_addr.s_addr, "0", "0", ACT_WAITING);
				#endif

			}

			// send back the regack message to the incoming client
			regack_send(sockfd, sain);

			break;


		// received the command start message from controller.
		case TCDMSG_CMDSTART:

			if (scbp->srv_state != SRV_STATE_INIT)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: receive CMDSTART in state [%s], should be [INIT]\n", SRV_STATE_STR(scbp->srv_state));
#endif
				break;
			}

#ifdef STANDALONE_MODE
			printf("### command start from controller %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif
			log_printf("### command start from controller %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));

			// get the list of files to be deployed
			if (parse_filelist(scbp, msg) == -1)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: parse file list failed\n");
#endif
				return -1;
			}

			g_image_triple_info = getnextfn_from_filelist(scbp);
			if (g_image_triple_info != NULL) // have file to send
			{
                char tmp_imagename[FNAME_SZ] = {0};
				tcd_syslog("send a file [%s] with version [%d]\n", fl_fname_ex(tmp_imagename, sizeof(tmp_imagename), g_image_triple_info), fl_ver(g_image_triple_info));

// wming, to align with Jumper's grey-button policy
#ifdef USE_MYSQL
				db_update_deploy_client_all_preparing();
#endif

				kickoff_filesending(scbp, sockfd, g_image_triple_info);
			}
			else // no additional file to send, just tell client to quit. 
			{
                char tmp_imagename[FNAME_SZ] = {0};
#ifdef STANDALONE_MODE
				printf("srv_cmdline_proc: no additional legal files which could be sent, goto STOP and quit ...\n");
#endif
				tcd_syslog("file [%s] sending done\n", fl_fname_ex(tmp_imagename, sizeof(tmp_imagename), g_image_triple_info));

				// send a packet with hasnofile bit set, so client will quit.
				runreq_send(sockfd, grp_sockaddr, NULL, 0, 1, g_tm_base /*RANDOM_DELAY_BASE*/, g_tm_coeff, g_tm_intpktdelay, MAPSZ, scbp);

				SET_SVR_STATE(scbp, SRV_STATE_STOP);

				return -1;
			}

			SET_SVR_STATE(scbp, SRV_STATE_READY);
			scbp->ready_state_tv = getts();

			break;

		// received the run acknowledgement from clients. 
		case TCDMSG_RUNACK:
			if (scbp->srv_state != SRV_STATE_READY)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: receive RUNACK in state [%s], should be [SRV_STATE_READY]\n", SRV_STATE_STR(scbp->srv_state));
#endif
				break;
			}

#ifdef STANDALONE_MODE
			printf("=====> incoming client runack %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
			printf("[debug] total clients: %d --- recvd: %d\n", scbp->clts_count, scbp->svr_runack_rcvd+1);
#endif

			log_printf("=====> incoming client runack %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
			log_printf("[debug] total clients: %d --- recvd: %d\n", scbp->clts_count, scbp->svr_runack_rcvd+1);

			clt = (CLT *)find_clts(scbp, sain.sin_addr.s_addr, sain.sin_port);
			if (clt == NULL)
			{
				perror("find_clts# in RUNACK");
				break;
			}

			clt->runacked = 1;

			if (msg->u.ua.uadata == 0)
			{
				tcd_syslog("WARNING: MOUNT FAILED FOR THE INCOMING CLIENT: %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));

				tcd_syslog("drop client %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
				drop_clts2(scbp, clt);

				break;
			}
			else
			{
				tcd_syslog("=====> incoming client runack %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
			}



#ifdef USE_MYSQL

			db_update_deploy_client(clt->srcipadd, "deploying");
			db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_DEPLOYING);
#endif

			// when all clients have replied runack, we can go to state RUN and start.
			if (++scbp->svr_runack_rcvd == scbp->clts_normal_count/*scbp->clts_count*/)
			{
				// note the initial timestamp
				scbp->lastcurrtv = scbp->currtv = scbp->starttv = getts();
				/*scbp->last_svr_cbulk = */scbp->svr_cbulk = 0;

				scbp->svr_cbulk_timeout = DEFAULT_SVR_CBULK_TIMEOUT; 

				init_bulkmap(&scbp->svr_cbulkmap);

				SET_SVR_STATE(scbp, SRV_STATE_RUN);

				uint32 slotstobesend = (scbp->svr_remain_slots <= TOTALSZ)?scbp->svr_remain_slots:TOTALSZ;
				send_bulk(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), scbp);
				scbp->svr_cbulk_ts = getts();
			}

			break;

		// received the packet acknowledgements from clients. 
		case TCDMSG_PKTACK:
			// acknowledgement packet can only be transferred in state RUN
			if (scbp->srv_state != SRV_STATE_RUN)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: receive PKTACK in state [%s], should be [SRV_STATE_RUN]\n", SRV_STATE_STR(scbp->srv_state));
#endif
				// printf("bulk: %d, full: %d\n", msg->u.pa.bulk, msg->u.pa.full);
				// print_bulkmap(&(msg->u.pa.bulkmap));
				break;
			}

			clt = (CLT *)find_clts(scbp, sain.sin_addr.s_addr, sain.sin_port);
			if (clt == NULL)
			{
				// packet acknowledgement from an unknown client ... 
				// should be something wrong, just quit silently
				// perror("find_clts#");
				break;
			}

			log_printf("=====> pakack %s:%d bulk# [%d] %s\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port), msg->u.pa.bulk, (msg->u.pa.full)?"full":"notfull");

			// if the client is full, it will not accept the packet acknowledge with same CBULK#
			if (clt->cbulkfull == CLT_CBULK_FULL)
			{
				// this packet is full again, duplicated and ignored
				tcd_syslog("duplicated full packet, bulk#[%d], ignore\n", msg->u.pa.bulk);
				break;
			}


#ifdef USE_PKTACK_WITH_BMAP
			#ifdef _DEBUG_
			#ifdef STANDALONE_MODE
			printf("bulkmap in incoming messages\n");
			#endif
			print_bulkmap(&(msg->u.pa.bulkmap));
			#endif

			// for the first packet acknowledgement, just dump the incoming bulkmap to the server bulkmap. 
			// for the second and later packet acknowledgement, we'd and the incoming bulkmap to the 
			// server's bulkmap. so finally if any of the client did not receive the slot, the correspondent
			// bit in the final server's bulkmap will be zero. 
			// if (scbp->svr_cbulk_recvd == 0)
			if (no_client_rcvd_pktack_clts(scbp))
				or_bulkmap(&scbp->svr_cbulkmap, &(msg->u.pa.bulkmap));
			else
				and_bulkmap(&scbp->svr_cbulkmap, &(msg->u.pa.bulkmap));

			#ifdef _DEBUG_
			print_bulkmap(&scbp->svr_cbulkmap);
			#endif
#endif

			// if clt->cbulkfull is CLT_CBULK_FULL, this message is duplicated ... just ignore. 
			if (msg->u.pa.full)
			{
				clt->cbulkfull = CLT_CBULK_FULL;
				clt->cbulk++;
				// clt->cbulk = msg->u.pa.bulk + 1;
			}
			else
			{
				clt->cbulkfull = CLT_CBULK_PARTFULL;
			}


			if (handle_pktack_from_all_client(sockfd, grp_sockaddr, scbp) == -1)
				return -1;

			break;


		// received the de-registration request from clients.
		case TCDMSG_DEREGREQ:
			if (scbp->srv_state != SRV_STATE_RUN && scbp->srv_state != SRV_STATE_STOP)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: receive DEREGREQ in state [%s], should be [SRV_STATE_RUN]\n", SRV_STATE_STR(scbp->srv_state));
#endif
				break;
			}

#ifdef STANDALONE_MODE
			printf("deregister request\n");
			printf("add: 0x%x\n", htonl(sain.sin_addr.s_addr));
#endif
			delete_clts(scbp, sain.sin_addr.s_addr, sain.sin_port);

			regack_send(sockfd, sain);

			SET_SVR_STATE(scbp, SRV_STATE_STOP);

			break;

		// received the notification message from clients.
		case TCDMSG_NOTIFY:
#ifdef STANDALONE_MODE
			printf("!!!!!! notification from client %s:%d", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#else
			tcd_syslog("!!!!!! notification from client %s:%d", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif

			switch(msg->u.nf.nf_code)
			{
				case NF_WRITE_FAILURE:
#ifdef STANDALONE_MODE
					printf("write failure\n");
#else
					tcd_syslog("write failure\n");
#endif

					break;
				case NF_UNKNOWN:
#ifdef STANDALONE_MODE
					printf("unknown\n");
#else
					// tcd_syslog("unknown\n");
#endif
					break;
				case NF_RECV_DONE:
#ifdef STANDALONE_MODE
					printf("all packets received\n");
#else
					//tcd_syslog("all packets received\n");
#endif
					break;

				case NF_RESTARTAPP:
#ifdef STANDALONE_MODE
					printf("client restart app now\n");
#else
					// tcd_syslog("client restart app now\n");
#endif
					break;

				case NF_REBOOT:
#ifdef STANDALONE_MODE
					printf("client reboot now\n");
#else
					// tcd_syslog("client reboot now\n");
#endif
					db_delete_from_user_status_by_ip(inet_ntoa(sain.sin_addr));
					break;

				case NF_REBOOT_IP:
#ifdef STANDALONE_MODE
					printf("client(ip) reboot now\n");
#else
					// tcd_syslog("client reboot now\n");
#endif
					db_delete_from_user_status_by_ip(inet_ntoa(sain.sin_addr));
					break;

				case NF_SHUTDOWN:
#ifdef STANDALONE_MODE
					printf("client shutdown now\n");
#else
					// tcd_syslog("client shutdown now\n");
#endif
					db_delete_from_user_status_by_ip(inet_ntoa(sain.sin_addr));
					break;

				case NF_SHUTDOWN_IP:
#ifdef STANDALONE_MODE
					printf("client(ip) shutdown now\n");
#else
					// tcd_syslog("client shutdown now\n");
#endif
					db_delete_from_user_status_by_ip(inet_ntoa(sain.sin_addr));
					break;

				case NF_FORMAT:
#ifdef STANDALONE_MODE
					printf("client format now\n");
#else
					// tcd_syslog("client format now\n");
#endif
					#ifdef USE_MYSQL
					clt = (CLT *)find_clts(scbp, sain.sin_addr.s_addr, sain.sin_port);
					if (clt == NULL)
					{
#ifdef STANDALONE_MODE
						perror("find_clts# in NOTIFY");
#endif
						break;
					}
					// db_update_deploy_client(uint32 ipadd, char *mac, char *status);
					if (msg->u.nf.nf_comment == 0)
					{
#ifdef STANDALONE_MODE
						printf("format start\n");
#endif

#ifdef USE_MYSQL
						db_update_deploy_client(clt->srcipadd, "formatting");
						db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_INITIALIZING_DISK);
#endif
					}
					else if (msg->u.nf.nf_comment == 1)
					{
#ifdef STANDALONE_MODE
						printf("format done\n");
#endif

#ifdef USE_MYSQL
						db_update_deploy_client(clt->srcipadd, "waiting");
						db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_WAITING);
#endif
					}
					else
					{
#ifdef STANDALONE_MODE
						fprintf(stderr, "NF_FORMAT: illegal nf_comment %d\n", msg->u.nf.nf_comment);					
#endif
					}
					#endif

					break;

				case NF_INITIALIZE:
#ifdef STANDALONE_MODE
					printf("initialize the file system\n");
#else
					//tcd_syslog("initialize the file system\n");
#endif

					#ifdef USE_MYSQL
					clt = (CLT *)find_clts(scbp, sain.sin_addr.s_addr, sain.sin_port);
					if (clt == NULL)
					{
						perror("find_clts# in NOTIFY");
						break;
					}
					// db_update_deploy_client(uint32 ipadd, char *mac, char *status);
					if (msg->u.nf.nf_comment == 0)
					{
#ifdef STANDALONE_MODE
						printf("initialize start\n");
#else
						tcd_syslog("initialize start\n");
#endif

#ifdef USE_MYSQL
						// to align with webconsole ...
						db_update_deploy_client(clt->srcipadd, "formatting");
						// db_update_deploy_client(clt->srcipadd, "initializing_disk");
						db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_INITIALIZING_DISK);
#endif
					}
					else // msg->u.nf.nf_comment >= 1
					{
						char buf[512];

						if (msg->u.nf.nf_comment == 1) // initialization script exited with value '0'
						{
#ifdef STANDALONE_MODE
							printf("initialize done\n");
#endif
							// db_update_deploy_client_with_machine_status(clt->srcipadd, "waiting", CLT_MACHINE_STATUS_NORMAL);
#ifdef USE_MYSQL
							db_update_deploy_client(clt->srcipadd, "waiting");
							db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_WAITING);
#endif
							scbp->brcack_count ++;
							tcd_syslog("brcack count: %d\n", scbp->brcack_count);
							if (scbp->brcack_count == scbp->clts_count)
							{
#ifdef USE_MYSQL
								// clear the database records of all clients. 
								// db_clear_deploy_client();
								// already included in clear_clts()

								// clear the connected clients
								// masked on Dec 6 2016 for Xiaowei'd tracker
								//clear_clts(&scb);

								// sprintf(evt_text, "reboot [%d] clients", scbp->clts_count);
								count = db_count_deploy_client_with_status ("interrupted");
								if (count > 0)
									sprintf(evt_text, "initialization: %d success, %d fail.", scbp->brcack_count - count, count);
								else
									sprintf(evt_text, "All clients completed their initialization");
								db_insert_deploy_history("info", evt_text);
#endif

								// tcd_syslog("reboot all connected clients after initialization");
								if (scbp->interrupted_clts > 0)
								{
									sprintf(buf, "initialization: %d success, %d fail.", scbp->brcack_count - scbp->interrupted_clts, scbp->interrupted_clts);
									tcd_syslog(buf);
								}
								else
								{
									sprintf(buf, "All clients completed their initialization");
									tcd_syslog(buf);
								}

								// clear the value
								scbp->brcack_count = 0;
								// restart the clients ...

								//sleep(10);

								// sock, grp_sockaddr, inet_addr(localip), sport
								// unmasked per NCN's request ... system will not reboot
								// reboot_broadcast(sock, grp_sockaddr, inet_addr(localip), sport);
								scbp->interrupted_clts = 0;
							}


						}
						else // msg->u.nf.nf_comment > 1, initialization script exited with a non-zero value.
						{
#ifdef STANDALONE_MODE
							printf("initialization error[%u]. \n", msg->u.nf.nf_comment - 1);
#endif
#ifdef USE_MYSQL
							if (msg->u.nf.nf_comment - 1 == 2) // 2 means script exited due to partition size issue
							{
								db_update_deploy_client_with_machine_status(clt->srcipadd, "interrupted", CLT_MACHINE_STATUS_INIT_ILLEGAL_SIZE);
							}
							else // for other exit code, just set a general flag
							{
								db_update_deploy_client_with_machine_status(clt->srcipadd, "interrupted", CLT_MACHINE_STATUS_INIT_HD_FAILURE);
							}
#endif
							scbp->brcack_count ++;
							scbp->interrupted_clts++;
							tcd_syslog("brcack count: %d\n", scbp->brcack_count);
							tcd_syslog("interrupted count: %d\n", scbp->interrupted_clts);
							if (scbp->brcack_count == scbp->clts_count)
							{
#ifdef USE_MYSQL
								count = db_count_deploy_client_with_status ("interrupted");
								sprintf(evt_text, "initialization: %d success, %d fail.", scbp->brcack_count - count, count);
								db_insert_deploy_history("error", evt_text);
#endif

								sprintf(buf, "initialization: %d success, %d fail.", scbp->brcack_count - scbp->interrupted_clts, scbp->interrupted_clts);
								tcd_syslog(buf);

								// clear the value
								scbp->brcack_count = 0;
								// restart the clients ...

								//sleep(10);

								// sock, grp_sockaddr, inet_addr(localip), sport
								// unmasked per NCN's request ... system will not reboot
								// reboot_broadcast(sock, grp_sockadd, inet_addr(localip), sport);
								scbp->interrupted_clts = 0;
							}

#ifdef STANDALONE_MODE
//							fprintf(stderr, "NF_INITIALIZE: illegal nf_comment %d\n", msg->u.nf.nf_comment);					
#endif
						}
					}
					#endif

					break;

				case NF_UPDATEFIRMWARE:
#ifdef STANDALONE_MODE
					printf("update the firmware of the client hard drive\n");
#else
					tcd_syslog("[%s] update the firmware of the client hard drive\n",
					inet_ntoa(*((struct in_addr *) &msg->u.cmd_bootinfo.clt_ip)));
#endif
					#ifdef USE_MYSQL
					clt = (CLT *)find_clts(scbp, sain.sin_addr.s_addr, sain.sin_port);
					if (clt == NULL)
					{
#ifdef STANDALONE_MODE
						perror("find_clts# in NOTIFY");
#endif
						break;
					}
					// db_update_deploy_client(uint32 ipadd, char *mac, char *status);
    				tcd_syslog("[%s] update firmware comment code: %d\n", 
     				inet_ntoa(*((struct in_addr *) &msg->u.cmd_bootinfo.clt_ip)),
                    msg->u.nf.nf_comment);

					if (msg->u.nf.nf_comment == 0)
					{
#ifdef STANDALONE_MODE
						printf("update firmware start\n");
#else
	    				tcd_syslog("[%s] update firmware start\n", 
	    				inet_ntoa(*((struct in_addr *) &msg->u.cmd_bootinfo.clt_ip)));
#endif

#ifdef USE_MYSQL
						// to align with webconsole ...
						db_update_deploy_client(clt->srcipadd, "formatting");
						// db_update_deploy_client(clt->srcipadd, "updating_firmware");
						db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_UPDATING_FIRMWARE);
#endif
					}
					else /* msg->u.nf.nf_comment >= 1 */
					{
                        // the number of clients who finsihed updating firmware covering both successful and unsuccesssful ones
						scbp->brcack_count++;
						if (msg->u.nf.nf_comment != 1 + ACT_UPDATING_FIRMWARE_SUCCESS)
						{
                            // the number of clients who failed to update firmware
							scbp->interrupted_clts++;
						}

						tcd_syslog("brcack count: %d\n", scbp->brcack_count);
						tcd_syslog("interrupted count: %d\n", scbp->interrupted_clts);

						if (msg->u.nf.nf_comment == 1 + ACT_UPDATING_FIRMWARE_SUCCESS)
						{
#ifdef STANDALONE_MODE
							printf("update firmware done\n");
#endif
							tcd_syslog("update firmware done\n");

#ifdef USE_MYSQL
							db_update_deploy_client(clt->srcipadd, "waiting");
                            // script exit with 0 means success
							db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_UPDATING_FIRMWARE_SUCCESS);

#endif

						}
						else /* msg->u.nf.nf_comment > 1 */
						{
#ifdef STANDALONE_MODE
							fprintf(stderr, "NF_UPDATEFIRMWARE: illegal nf_comment %d\n", msg->u.nf.nf_comment - 1);					
#endif
#ifdef USE_MYSQL
							// for other exit code, just set a general flag, one should check the detailed error info on the screen of clients
							db_update_deploy_client_with_machine_status(clt->srcipadd, "interrupted", CLT_MACHINE_STATUS_INIT_HD_FAILURE);
                            // script exit with non 0 value means updating failure 
							db_update_user_status(clt->hwa, clt->srcipadd, "0", "0", ACT_UPDATING_FIRMWARE_FAILURE);
#endif
						}

						if (scbp->brcack_count == scbp->clts_count) // received all resposes from all clients performing the task
						{
#ifdef USE_MYSQL
							count = db_count_deploy_client_with_status ("interrupted");
							if (count > 0)
								sprintf(evt_text, "update firmware: %d success, %d fail.", scbp->brcack_count - count, count);
							else
								sprintf(evt_text, "All clients completed updating firmwares");
							db_insert_deploy_history("info", evt_text);
#endif

#ifdef STANDALONE_MODE
							if (scbp->interrupted_clts > 0)
							{
								sprintf(buf, "update firmware: %d success, %d fail.", scbp->brcack_count - scbp->interrupted_clts, scbp->interrupted_clts);
								tcd_syslog(buf);
							}
							else
							{
								sprintf(buf, "All clients completed updating firmwares");
								tcd_syslog(buf);
							}
#endif
							scbp->brcack_count = 0;
							scbp->interrupted_clts = 0;
						}

					}
					#endif

					break;

				case NF_CREATE_FAILURE:
#ifdef STANDALONE_MODE
					printf("create file failed\n");
#else
					//tcd_syslog("create file failed\n");
#endif
					break;
				case NF_LAST_FILE:
#ifdef STANDALONE_MODE
					printf("last file, quit gracefully\n");
#else
					//tcd_syslog("last file, quit gracefully\n");
#endif
					break;

				default: 
#ifdef STANDALONE_MODE
					printf("unknown nf_code: %d\n", msg->u.nf.nf_code);
#endif
					break;
			}

			break;


		// to list all tcdclts
		case TCDMSG_CMDLIST:
			print_clts(scbp);
			break;

		// to quit the tcd server
		case TCDMSG_CMDQUIT:
			// quit ...

#ifdef USE_MYSQL
			sprintf(evt_text, "server exit");		
			db_insert_deploy_history("info", evt_text);
#endif
			SET_SVR_STATE(scbp, SRV_STATE_STOP);

			tcd_syslog("quit tcd server manually");

			return -1;
			break;






		// to reboot all tcd clients
		case TCDMSG_CMDREBOOT:
			// sock, grp_sockadd, inet_addr(localip), sport
			reboot_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
#ifdef USE_MYSQL
//			db_clear_deploy_client();
#endif

#ifdef USE_MYSQL
			// sprintf(evt_text, "reboot [%d] clients", scbp->clts_count);
			sprintf(evt_text, "reboot detected clients");
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog("reboot all connected clients");

			return -1;

			break;

		// to reboot a client
		case TCDMSG_CMDREBOOT_IP:
			reboot_ip_unicast(sockfd, g_ip_ineta, g_port, msg->u.rbt_cmd_payload.ipv4);
#ifdef USE_MYSQL
//			db_clear_deploy_client();
#endif

#ifdef USE_MYSQL
			// sprintf(evt_text, "reboot [%d] clients", scbp->clts_count);
			sprintf(evt_text, "reboot client with IP %s", msg->u.rbt_cmd_payload.ipv4);
			db_insert_deploy_history("info", evt_text);
			// remove thie piece of info about the client in db
			db_delete_from_deploy_client_by_ip (msg->u.rbt_cmd_payload.ipv4);
			db_delete_from_user_status_by_ip (msg->u.rbt_cmd_payload.ipv4);
#endif

			tcd_syslog("reboot client with specified IP");

			//return -1;

			break;

		// to shutdown all tcd clients
		case TCDMSG_CMDSHUTDOWN:
			shutdown_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
#ifdef USE_MYSQL
//			db_clear_deploy_client();
#endif
#ifdef USE_MYSQL
			sprintf(evt_text, "shutdown [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog("shutdown all connected clients");

			return -1;
			break;

		// to shutdown all tcd clients
		case TCDMSG_CMDSHUTDOWN_IP:
			shutdown_ip_unicast(sockfd, g_ip_ineta, g_port, msg->u.shd_cmd_payload.ipv4);
#ifdef USE_MYSQL
//			db_clear_deploy_client();
#endif
#ifdef USE_MYSQL
			sprintf(evt_text, "shutdown client with IP %s", msg->u.shd_cmd_payload.ipv4);		
			db_insert_deploy_history("info", evt_text);
			// remove thie piece of info about the client in db
			db_delete_from_deploy_client_by_ip(msg->u.shd_cmd_payload.ipv4);
			db_delete_from_user_status_by_ip(msg->u.shd_cmd_payload.ipv4);
#endif

			tcd_syslog("shutdown client with specified IP");

			//return -1;
			break;

		// to boot a client with given ip
		case TCDMSG_CMDBOOT_IP:
		{
			IMAGE_ACCESS image_per;
			char str_image_id[64];
			char str_user_name[128];
			bzero(str_image_id, sizeof(str_image_id));
			bzero(str_user_name, sizeof(str_user_name));
			sprintf(str_image_id, "%u", msg->u.cmd_bootinfo.image_id);
			strcpy(str_user_name, msg->u.cmd_bootinfo.user_name);

			if (0 != get_imageaccess(str_image_id, str_user_name, &image_per))
			{
				image_per = IMAGE_NO_RIGHTS;
			}
			//sprintf(evt_text, "try to boot client image_id[%u], image_org_name[%s], image_name[%s], image_per[%d], user_id[%u], user_name[%s], ip[%s]\n", msg->u.cmd_bootinfo.image_id, msg->u.cmd_bootinfo.image_org_name, msg->u.cmd_bootinfo.image_name, msg->u.cmd_bootinfo.image_per, msg->u.cmd_bootinfo.user_id, msg->u.cmd_bootinfo.user_name, inet_ntoa(*((struct in_addr *) &msg->u.cmd_bootinfo.clt_ip)));
			sprintf(evt_text, "try to boot client image_id[%u], image_org_name[%s], image_name[%s], image_per[%d], user_token[%s], user_name[%s], ip[%s]\n", msg->u.cmd_bootinfo.image_id, msg->u.cmd_bootinfo.image_org_name, msg->u.cmd_bootinfo.image_name, image_per, msg->u.cmd_bootinfo.user_token, msg->u.cmd_bootinfo.user_name, inet_ntoa(*((struct in_addr *) &msg->u.cmd_bootinfo.clt_ip)));
			tcd_syslog(evt_text);
			boot_ip_unicast(sockfd, g_ip_ineta, g_port, msg->u.cmd_bootinfo.image_id, msg->u.cmd_bootinfo.image_org_name, msg->u.cmd_bootinfo.image_name, image_per, msg->u.cmd_bootinfo.user_token, msg->u.cmd_bootinfo.user_name, msg->u.cmd_bootinfo.clt_ip);
			break;
		}

		// to restart applications for all clients
		case TCDMSG_CMDRESTARTAPP:
			restart_app_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
#ifdef USE_MYSQL
//			db_clear_deploy_client();
#endif
#ifdef USE_MYSQL
			sprintf(evt_text, "restart app of [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);
#endif
			tcd_syslog("restart tcdapp on all connected clients");

			return -1;
			break;

		// to tell each client print its ID
		case TCDMSG_CMDIDENTIFY:
			identify_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp);
#ifdef USE_MYSQL
			sprintf(evt_text, "try to identify [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog("identify all connected clients");

			break;

		// to tell each client format the local hard disk
		case TCDMSG_CMDFORMAT:
			format_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp);
#ifdef USE_MYSQL
			sprintf(evt_text, "format [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog("format local drives of all connected clients");

			break;


		// some user-defined commands
		case TCDMSG_CMDUSER:
			printf("=====> incoming cmduser %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));

			usercmd_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp);
			break;


// added for weiyu's login
		case TCDMSG_CMDCHKREGREQ:
#ifdef STANDALONE_MODE
			printf("=====> incoming cmdchkregreq %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif
			mac = &(msg->u.cr.hwa[0]); 
#ifdef STANDALONE_MODE
			printf("======>>>>> mac address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
#endif
			//usercmd_broadcast(sockfd, grp_sockaddr, inet_addr(localip), sport, scbp);

#ifdef USE_MYSQL
			retval = db_chkreg_client(mac);
#ifdef STANDALONE_MODE
			printf("chkregack send ...\n");
#endif
			cmdchkregack_send(sockfd, sain, mac, retval);
#endif

			break;

		case TCDMSG_CMDREGREQ:
#ifdef STANDALONE_MODE
			printf("=====> incoming cmdregreq %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif
			mac = &(msg->u.rg.hwa[0]); 
			hn = &(msg->u.rg.hname[0]);

#ifdef STANDALONE_MODE
			printf("======>>>>> mac address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
			printf("======>>>>> hostname: [%s]\n", hn);
#endif
			//usercmd_broadcast(sockfd, grp_sockaddr, inet_addr(localip), sport, scbp);

#ifdef USE_MYSQL
			retval = db_reg_client(mac, hn, msg);

#ifdef STANDALONE_MODE
			printf("regack send ...\n");
#endif
			cmdregack_send(sockfd, sain, mac, hn, retval);
#endif

			break;

		case TCDMSG_CMDUPREGREQ:
#ifdef STANDALONE_MODE
			printf("=====> incoming cmdregreq %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif
			mac = &(msg->u.cur.hwa[0]); 
			hn = &(msg->u.cur.hname[0]);

#ifdef STANDALONE_MODE
			printf("======>>>>> mac address: [%02X:%02X:%02X:%02X:%02X:%02X]\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
			printf("======>>>>> hostname: [%s]\n", hn);
#endif

#ifdef USE_MYSQL
			retval = db_update_client(mac, hn, msg);

#ifdef STANDALONE_MODE
			printf("update-reg-ack send ...\n");
#endif
			cmdupregack_send(sockfd, sain, mac, hn, retval);
#endif

			break;

		// to tell each client initialize the local hard disk
		case TCDMSG_CMDINITIALIZE:

			// added by wming: when start initialization, the count will be zero. It will be increased gradually for each notification...
			scbp->brcack_count = 0;
			scbp->interrupted_clts = 0;

			initialize_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp, msg->u.ciz.sys_part_capacity);
#ifdef USE_MYSQL
			sprintf(evt_text, "initialize [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog("initialize local drives of all connected clients");

			break;

		// to tell a client with specified IPv4 to initialize the local hard disk
		case TCDMSG_CMDINITIALIZE_IP:

			// added by wming: when start initialization, the count will be zero. It will be increased gradually for each notification...
			//scbp->brcack_count = 0;
			//scbp->interrupted_clts = 0;

			initialize_unicast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp, msg->u.init_cmd_payload.sys_part_capacity, msg->u.init_cmd_payload.ipv4);
#ifdef USE_MYSQL
			sprintf(evt_text, "initialize client with IPv4[%s]", msg->u.init_cmd_payload.ipv4);
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog(evt_text);

			break;

		// to tell each client update the firmware
		case TCDMSG_CMDUPDATEFIRMWARE:
			// similar reason to add the code as initialization
			scbp->brcack_count = 0;
			scbp->interrupted_clts = 0;
			updatefirmware_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp);
#ifdef USE_MYSQL
			sprintf(evt_text, "updatefirmware [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);

            // change all clients status
            db_update_all_existing_user_status(ACT_UPDATING_FIRMWARE_CMD_SENT);
#endif

			tcd_syslog("update firmware of all connected clients");

			break;

		// to tell client with IPv4 to update firmware
		case TCDMSG_CMDUPDATEFIRMWARE_IP:
			// similar reason to add the code as initialization
			//scbp->brcack_count = 0;
			//scbp->interrupted_clts = 0;
			updatefirmware_unicast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp, msg->u.uf_cmd_payload.ipv4);
#ifdef USE_MYSQL
			sprintf(evt_text, "updatefirmware for client[%s]", msg->u.uf_cmd_payload.ipv4);
			db_insert_deploy_history("info", evt_text);

            // change client status
            db_update_existing_user_status_by_ip(msg->u.uf_cmd_payload.ipv4, ACT_UPDATING_FIRMWARE_CMD_SENT);
#endif

			tcd_syslog(evt_text);

			break;

		// to tell each client to remove an image
		case TCDMSG_CMDRM_IMAGE:
			rm_image_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp, msg->u.rm_image_cmd_payload.image_org_name, msg->u.rm_image_cmd_payload.image_id);
#ifdef USE_MYSQL
			sprintf(evt_text, "remove image %s[%d] for all clients", msg->u.rm_image_cmd_payload.image_org_name, msg->u.rm_image_cmd_payload.image_id);
			db_insert_deploy_history("info", evt_text);
#endif

			tcd_syslog(evt_text);
			break;

		// to tell each client to remove an image
		case TCDMSG_CMDRM_IMAGE_IP:
        {
          rm_image_unicast(sockfd, grp_sockaddr, scbp, msg->u.rm_image_ip_cmd_payload.image_org_name, msg->u.rm_image_ip_cmd_payload.image_id, msg->u.rm_image_ip_cmd_payload.ipv4);
#ifdef USE_MYSQL
          sprintf(evt_text, "remove image %s[%d] for client[%s]", msg->u.rm_image_ip_cmd_payload.image_org_name, msg->u.rm_image_cmd_payload.image_id, msg->u.rm_image_ip_cmd_payload.ipv4);
          db_insert_deploy_history("info", evt_text);
#endif
          tcd_syslog(evt_text);
          break;
        }

		case TCDMSG_CMDSET_RESOLUTION_IP:
        {
          set_resolution_unicast(sockfd, msg->u.set_resolution_ip_cmd_payload.ipv4, msg->u.set_resolution_ip_cmd_payload.screen_width, msg->u.set_resolution_ip_cmd_payload.screen_height);

          sprintf(evt_text, "set-resolution %dx%d for client[%s]", msg->u.set_resolution_ip_cmd_payload.screen_width, msg->u.set_resolution_ip_cmd_payload.screen_height, msg->u.set_resolution_ip_cmd_payload.ipv4);
          tcd_syslog(evt_text);
          break;
        }

		case TCDMSG_CMDUSB_STORAGE_CTRL_IP:
        {
          usb_ctrl_unicast(sockfd, msg->u.usb_storage_ctrl_ip_cmd_payload.ipv4, msg->u.usb_storage_ctrl_ip_cmd_payload.usb_enabled);

          sprintf(evt_text, "usb-ctrl client[%s]: %d", msg->u.usb_storage_ctrl_ip_cmd_payload.ipv4, msg->u.usb_storage_ctrl_ip_cmd_payload.usb_enabled);
          tcd_syslog(evt_text);
          break;
        }

		// received the runtime update from clients, and they will tell server their OS, username and mac address, etc. 
		case TCDMSG_ACTUSR:
			//tcd_syslog(evt_text);
			if (scbp->srv_state != SRV_STATE_INIT)
			{
#ifdef STANDALONE_MODE
				fprintf(stderr, "proc_tcdmsg_srv: receive TCDMSG_ACTUSR in state [%s], should be [SRV_STATE_INIT]\n", SRV_STATE_STR(scbp->srv_state));
#endif
				break;
			}

#ifdef STANDALONE_MODE
			printf("=====> incoming actusr from %s:%d\n", inet_ntoa(sain.sin_addr), htons(sain.sin_port));
#endif

			printf("uname: %s osname: %s act: %d\n", msg->u.au.uname, msg->u.au.osname, msg->u.au.act);

#ifdef POST_REPORT_HWINFO
			insert_or_touch_actusrs(scbp, sain.sin_addr.s_addr, /*sain.sin_port, */msg->u.au.hwa, msg->u.au.uname, msg->u.au.osname, msg->u.au.act, msg->u.au.disksz, msg->u.au.memsz, msg->u.au.strcpu);
#else
			insert_or_touch_actusrs(scbp, sain.sin_addr.s_addr, /*sain.sin_port, */msg->u.au.hwa, msg->u.au.uname, msg->u.au.osname, msg->u.au.act);
#endif //POST_REPORT_HWINFO

#ifdef USE_MYSQL
//			db_update_deploy_client(clt->srcipadd, "deploying");
#ifdef POST_REPORT_HWINFO
			db_update_user_status(msg->u.au.hwa, sain.sin_addr.s_addr, msg->u.au.uname, msg->u.au.osname, msg->u.au.act, msg->u.au.disksz, msg->u.au.memsz, msg->u.au.strcpu);
#else
			db_update_user_status(msg->u.au.hwa, sain.sin_addr.s_addr, msg->u.au.uname, msg->u.au.osname, msg->u.au.act);
#endif //POST_REPORT_HWINFO

			// if the act (or status is to reboot or poweroff the client,
			// then delete the related table entry from the table `userstatus' also
			if (msg->u.au.act == ACT_SHUTTING_DOWN)
			{
				db_delete_from_user_status_by_ip(inet_ntoa(sain.sin_addr));
			}
#endif

			break;


		// to tell each client format the local hard disk
		case TCDMSG_CMDCGSVRIP:
			cgsvrip_broadcast(sockfd, grp_sockaddr, msg->u.ccgs.srvip);
#ifdef USE_MYSQL
			sprintf(evt_text, "change server ip cache in [%d] clients", scbp->clts_count);		
			db_insert_deploy_history("info", evt_text);
#endif

			uint32 ip = msg->u.ccgs.srvip;

			tcd_syslog("change server IP cache to [%s] in all connected clients", inet_ntoa(*(struct in_addr *)&ip));



			break;
      
        case TCDMSG_CMDCGSVRIP_IP:
            cgsvrip_unicast(sockfd, grp_sockaddr, msg->u.ccgsip.srvip, msg->u.ccgsip.ipv4);
			tcd_syslog("change server IP [%s] to %s", inet_ntoa(*(struct in_addr *)&msg->u.ccgsip.srvip), msg->u.ccgsip.ipv4);
            break;


		// otherwise ...
		default:
			DBG("unknown tcdmsg type @ server: [%ld]\n", msg->type);
			break;
	}

	return 0;
}

// server command line processing
int srv_cmdline_proc(char ch, int sockfd, struct sockaddr_in grp_sockaddr, SCB *scbp)
{
	switch(ch)
	{

		// command "list": to print the accepted clients' information
		case 'l':
			print_clts(scbp);
			print_actusrs(scbp);
			break;

		// command "help": to print the help information
		case 'h':
			srv_help();
			break;

		// command "quit": to quit the server immediately
		case 'q':
			// quit ...
			SET_SVR_STATE(scbp, SRV_STATE_STOP);
			return -1;
			break;

		// command "check": to check the current status
		case 'c':
			print_svr_status(scbp);
			break;

		// command "debug": to enable or disable the debugger
		case 'd':
			if (scbp->tcdsrv_debugger)
			{
#ifdef STANDALONE_MODE
				printf("turn off the debugger\n");
#endif
				scbp->tcdsrv_debugger = 0;
			}
			else
			{
#ifdef STANDALONE_MODE
				printf("turn on the debugger\n");
#endif
				scbp->tcdsrv_debugger = 1;
			}
			break;

		// command "reboot": to reboot the machine
		case 'b':
			// sock, grp_sockaddr, inet_addr(localip), sport
			reboot_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
			break;

		// command "pause": to pause or resume the machine
		case 'p':
			// sock, grp_sockaddr, inet_addr(localip), sport
			if (scbp->signal_started)
			{
#ifdef STANDALONE_MODE
				printf("paused!!!\n");
#endif
				scbp->signal_started = 0;
			}
			else
			{
#ifdef STANDALONE_MODE
				printf("pause resumed!!!\n");
#endif
				scbp->signal_started = 1;
			}
			break;

		// command "shutdown": to shutdown the machine
		case '0':
			shutdown_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
			break;

		// command "reset tcdclt": to reset the tcdclt application
		case '1':
			restart_app_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
			break;

		// command "format tcdclt": to format the client local hard disk
		case 'f':
			format_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);
			break;


		// command "version": to print the build info.
		case 'v':
			print_buildinfo();
			break;

		// otherwise the command will not be recognized. 
		default: 
			DBG("srv_cmdline_proc: unknown char\n");
			break;
	}
	
	return 0;
}


static uint32 timeout_counter=0, timeout_first=1, timeout_prev=0;

// timeout handling for the TCDSVR
void proc_tcdsvr_timeout(SCB *scbp, int sockfd, struct sockaddr_in grp_sockaddr, int port)
{
	uint64 diff;
	struct timeval ts;
	int retval;

	timeout_counter++;
	// get the current timestamp ...
	ts = getts();


	// in state init, just periodically send out hello packet
	if (scbp->srv_state == SRV_STATE_INIT)
	{
		diff = diff_tv(scbp->init_state_tv, ts);

		if (diff > DEFAULT_SVR_HELLO_TIMEOUT)
		{
			if (timeout_first == 1)
			{
				timeout_first = 0;
#ifdef STANDALONE_MODE
				fflush(stdout);
				printf("hello broadcasting.");
#endif
			}
			else
			{
#ifdef STANDALONE_MODE
				fflush(stdout);
				printf(".");
#endif
			}

			// broadcast the hello packet to initiate the registration. 
			hello_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port);

			// adjust the init_state timestamp to now.
			scbp->init_state_tv = ts;
		}

		// handle actusr data structure during OS runtime. 
		handle_actusers_timeout(scbp);
	}

	// this paragraph is added by wming to handle wensheng's new issue. 
	// when registered but suddenly disconnected, the whole systme will hang forever. 
	// we'll add a new timeout mechanism to handle. 

	// in state ready, check if there are clients which should be dropped due to loss of acknowledgements. 
	else if (scbp->srv_state == SRV_STATE_READY)
	{
		diff = diff_tv(scbp->ready_state_tv, ts);

		if (diff > DEFAULT_SVR_READY_TIMEOUT)
		{
#ifdef STANDALONE_MODE
			printf("SVR READY state timeout\n");
#endif
			tcd_syslog("SVR READY state timeout\n");

			check_and_drop_non_runack_clients(scbp);

			// no matter whether all clients have acked, we'll reset the ready state timestamp to latest time
			// or it will expire everytime too frequently
			scbp->ready_state_tv = ts;

			// when all clients have replied runack, we can go to state RUN and start.
			if (scbp->svr_runack_rcvd == scbp->clts_normal_count/*scbp->clts_count*/)
			{
				// note the initial timestamp
				scbp->lastcurrtv = scbp->currtv = scbp->starttv = ts;
				/*scbp->last_svr_cbulk = */scbp->svr_cbulk = 0;

				scbp->svr_cbulk_timeout = DEFAULT_SVR_CBULK_TIMEOUT; 

				init_bulkmap(&scbp->svr_cbulkmap);

				SET_SVR_STATE(scbp, SRV_STATE_RUN);

				uint32 slotstobesend = (scbp->svr_remain_slots <= TOTALSZ)?scbp->svr_remain_slots:TOTALSZ;
				send_bulk(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), scbp);
				scbp->svr_cbulk_ts = ts;
			}
		}

	}
	// end of wensheng's new issue. 

	else if (scbp->srv_state == SRV_STATE_RUN)
	{
		// below stage is RUN, otherwise it's illegal
#ifdef _DEBUG_
#ifdef STANDALONE_MODE
		printf("in proc_tcdsvr_timeout [%d]\n", timeout_counter);
#endif
		print_svr_status(scbp);
#endif

		// ts = getts();
		diff = diff_tv(scbp->svr_cbulk_ts, ts);

		// both resend the bulk and pktack
		if (diff > DEFAULT_SVR_MAX_CBULK_TIMEOUT)
		{
			// check which client may be dropped?
			retval = check_offline_client(scbp);

			#ifdef USE_MYSQL
			char evt_text[EVT_TEXT_SZ];
			CLT *cp;
			uint32 ip;
			// struct in_addr in;

			if (retval != -1)
			{
                char temp_imagename[FNAME_SZ] = {0};
				cp = &(scbp->clts[retval]);
				ip = cp->srcipadd;
				#ifdef STANDALONE_MODE
				printf("interrupted: idx=%d ip=0x%x\n", retval, ip);
				#endif
				sprintf(evt_text, "interrupted send file [%s] to %s:%d [%02X:%02X:%02X:%02X:%02X:%02X]", \
					fl_fname_ex(temp_imagename, sizeof(temp_imagename), g_image_triple_info), inet_ntoa(*(struct in_addr *)&ip), cp->srcport, \
					cp->hwa[0], cp->hwa[1], cp->hwa[2], cp->hwa[3], cp->hwa[4], cp->hwa[5]);
				db_insert_deploy_history("info", evt_text);
			}
			#endif	// USE MYSQL

			if (retval != -1)
			{
				tcd_syslog("drop client #%d\n", retval);
				drop_clts(scbp, retval);
			}

			if (handle_pktack_from_all_client(sockfd, grp_sockaddr, scbp) == -1)
				return;

		}
		else if (diff > DEFAULT_SVR_PKTACK_CHECK_CBULK_TIMEOUT)
		{
			// most likely it's caused by pktack loss, let's urge the clients to check ...
#ifdef STANDALONE_MODE
			printf("send pktack_check for bulk #[%d]...in proc_tcdsvr_timeout timeout [%d]\n", scbp->svr_cbulk, scbp->svr_cbulk_timeout);
#endif

			tcd_syslog("send pktack_check for bulk #[%d]...in proc_tcdsvr_timeout timeout [%d]\n", scbp->svr_cbulk, scbp->svr_cbulk_timeout);

			pktack_check_broadcast(sockfd, grp_sockaddr, g_ip_ineta, g_port, scbp);
		}
		else
		{
			log_printf("timeout [%d] resend bulk #[%d]\n", scbp->svr_cbulk_timeout, scbp->svr_cbulk);
			tcd_syslog("send pktack_check for bulk #[%d]...in proc_tcdsvr_timeout timeout [%d]\n", scbp->svr_cbulk, scbp->svr_cbulk_timeout);


#ifdef STANDALONE_MODE
			printf("\ntimeout [%d] resend bulk #[%d]\n", scbp->svr_cbulk_timeout, scbp->svr_cbulk);
#endif

			uint32 slotstobesend = (scbp->svr_remain_slots <= TOTALSZ)?scbp->svr_remain_slots:TOTALSZ;
#ifdef USE_PKTACK_WITH_BMAP
			send_bulk_according2bulkmap(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), &scbp->svr_cbulkmap, scbp);
#else
			send_bulk(sockfd, grp_sockaddr, sizeof(grp_sockaddr), scbp->svr_cbulk, slotstobesend, (scbp->svr_remain_slots > TOTALSZ), scbp);
#endif
		}


		// update the timestamp of server
		// scbp->svr_cbulk_ts = ts; //getts();

	}
	// otherwise, just ignore ...

	return;
}


int srv_fsm_main(int sockfd, struct sockaddr_in grp_sockaddr, SCB *scbp)
{
	fd_set rdfds;
	int res;
	char ch;
	struct timeval tv;
	char buf[2048];

#ifdef USE_MYSQL
    // time for refresh invalid entry for table userstatus
    static time_t old_chktm = 0;
    static time_t new_chktm = 0;
#endif

#ifdef STANDALONE_MODE
	printf("enter srv_fsm_main() loop ...\n");
#endif
	// get the initial timestamp before going to init state
	scbp->init_state_tv = getts();

	// to set the global receiver state
	SET_SVR_STATE(scbp, SRV_STATE_INIT);

	// to enable the raw terminal mode so we can use hotkey ...

#ifdef STANDALONE_MODE
	term_raw();
#endif

	// the while deadloop(), which can only be out if 
	//	srv_cmdline_proc() returns -1, or 
	// 	proc_tcdmsg_srv() returns -1. 
	while(1)
	{
		// add network sockfd and stdin into the FDSET, 
		FD_ZERO(&rdfds);

#ifdef STANDALONE_MODE
		FD_SET(0, &rdfds);
#endif

		FD_SET(sockfd, &rdfds);

		// and set the select timeout to forever
		tv.tv_sec = 0;
		tv.tv_usec = DEFAULT_SVR_CBULK_TIMEOUT; //ALARM_INTERVAL;

		res = select(sockfd + 1, &rdfds, NULL, NULL, &tv);
		// DBG("res=%d\n", res);

		if (res > 0)
		{
			// stdin events
			if (FD_ISSET(0, &rdfds))
			{
				ch = getchar();
				// DBG("0x%x [%c]\n", ch, ch);
				// returning -1 means it's time to go out
				if (srv_cmdline_proc(ch, sockfd, grp_sockaddr, scbp) == -1)
					break;
			}

			// cskt events
			else if (FD_ISSET(sockfd, &rdfds))
			{
				struct sockaddr_in sain;
				int sain_len=sizeof(sain);

				if (recvfrom(sockfd, buf, sizeof(buf), 0, (struct sockaddr *)&sain, &sain_len) != -1)
				{
					DBG("srv_fsm_main: receive from [%s:%d]: \n", inet_ntoa(sain.sin_addr),  htons(sain.sin_port));
					// returning -1 means all slots have been sent out, or some exceptions happened, so go out.  
					if (proc_tcdmsg_srv(sockfd, buf, sain, grp_sockaddr, scbp) == -1)
						break;
				}
				else
				{
#ifdef STANDALONE_MODE
					printf("srv_fsm_main: recvfrom failed!\n"); 
#endif
				}
			}
		}
		else if (res == 0)
		{
			proc_tcdsvr_timeout(scbp, sockfd, grp_sockaddr, g_port);
		}
		else
		{
			perror("srv_fsm_main: select()");
		}
		// if (res>0)

        // refresh the userstatus table periodically
#ifdef USE_MYSQL
        new_chktm = time(NULL);

        if (old_chktm == 0)
        {
            old_chktm = new_chktm;
        }

        if (old_chktm + REFRESH_STATUS_INTERVAL < new_chktm)
        {
            old_chktm = new_chktm;
            // do not refresh user status
            //db_refresh_user_status();
        }
#endif
	}

#ifdef STANDALONE_MODE
	// to restore the terminal mode
	term_restore();
#endif

	return 0;
}


int server_main_routine()
{
	int retval;
	TCHAR credential[128];
	TCHAR dummy_credential[128] = "dummy_credential";
	
	// to get the ipadd of the local interface
	//if ((localip = ifn2ip_str(sifn)) == NULL)
	if (NULL == (ifn2ip_str_ex(g_ip_str, IPSTR_SZ, g_if_name_str)))
	{
#ifdef STANDALONE_MODE
		perror("ifn2ip_str_ex");
#else
		tcd_syslog("server_main_routine: cannot get ip address via [%s]\n", g_if_name_str);
#endif

		return -1;
	}
    g_ip_ineta = inet_addr(g_ip_str);

#ifdef STANDALONE_MODE
	printf("\nMulticast Deployment Server\n");
	printf("Interface [%s] Network [%s:%d]\n", g_if_name_str, g_ip_str, g_port);
#endif

// added by wming, to initialize the db connection
#ifdef USE_MYSQL
	retval = cfg_getstring("", "tcctl_pass", dummy_credential, credential, sizeof(credential), DB_INI_FILE);
        if (retval == strlen(dummy_credential) && strcmp(credential, dummy_credential) == 0 ) {
		tcd_syslog("failed to load database credentials\n");
		return -1;
	}
	retval = start_db("tcctl", credential);
	if (retval != 0)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "failed to start the database connection\n");
#endif
		tcd_syslog("failed to start the database connection\n");

		return -1;
	}
	DBG("database connection initialized\n");
#endif

	// server socket ...
	if ((g_sockfd = socket(AF_INET, SOCK_DGRAM, 0))<0)
	{
#ifdef STANDALONE_MODE
		perror("socket");
#else
		tcd_syslog("server_main_routine: socket failed\n");
#endif

#ifdef USE_MYSQL
		end_db();
#endif
		return -1;
	}
	DBG("tcdsrv: socket successfully ...\n");

	// setup the listen socket
	struct sockaddr_in sain;
	bzero(&sain, sizeof(sain));
	sain.sin_family = AF_INET;
	sain.sin_addr.s_addr = htonl(INADDR_ANY);
	sain.sin_port = htons(g_port);

	if (bind(g_sockfd, (struct sockaddr *)&sain, sizeof(sain))<0)
	{

#ifdef STANDALONE_MODE
		perror("bind");
#else
		tcd_syslog("server_main_routine: bind failed\n");
#endif

		close(g_sockfd);

#ifdef USE_MYSQL
		end_db();
#endif
		return -1;
	}
	DBG("tcdsrv: bind successfully ...\n");

	// setup the multicast group socket
	memset((char *)&g_group_sockaddr_in, 0, sizeof(g_group_sockaddr_in));
	g_group_sockaddr_in.sin_family = AF_INET;
	g_group_sockaddr_in.sin_addr.s_addr = inet_addr(GROUP_IP);
	g_group_sockaddr_in.sin_port = htons(GROUP_PORT);

	struct in_addr localif;
	localif.s_addr = g_ip_ineta;
	if (setsockopt(g_sockfd, IPPROTO_IP, IP_MULTICAST_IF, (char *)&localif, sizeof(localif))<0)
	{
#ifdef STANDALONE_MODE
		perror("setting local interface");
#else
		tcd_syslog("server_main_routine: set local interface failed\n");
#endif

		close(g_sockfd);

#ifdef USE_MYSQL
		end_db();
#endif

		return -1;
	}
	DBG("tcdsrv: set local interface to multicast successfully ...\n");

	int nSendBuf = SND_BUF_SZ; // something like 64*1024, defined in conf.h
	if (setsockopt(g_sockfd, SOL_SOCKET, SO_SNDBUF, (const char *)&nSendBuf, sizeof(int)) < 0)
	{
#ifdef STANDALONE_MODE
		perror("set sending socket buffer");
#else
		tcd_syslog("server_main_routine: set sending socket buffer failed\n");
#endif

		close(g_sockfd);

#ifdef USE_MYSQL
		end_db();
#endif

		return -1;
	}
	int nRecvBuf = RCV_BUF_SZ; // something like 64*1024, defined in conf.h
	if (setsockopt(g_sockfd, SOL_SOCKET, SO_RCVBUF, (const char *)&nRecvBuf, sizeof(int)) < 0)
	{
#ifdef STANDALONE_MODE
		perror("set receiving socket buffer");
#else
		tcd_syslog("server_main_routine: set receiving socket buffer failed\n");
#endif


		close(g_sockfd);

#ifdef USE_MYSQL
		end_db();
#endif

		return -1;
	}
	DBG("tcdsrv: set socket buffer to 32K successfully ...\n");

    
    if (NULL == (ifn2mac_str_ex(g_hwa_str, sizeof(g_hwa_str), g_if_name_str)))
    {
       tcd_syslog("fail to get hw addr via %s\n", g_if_name_str); 
       return -1;
    }

#ifdef STANDALONE_MODE
	printf("==========>>>>>>>>>> mac address: [%s]\n", g_hwa_str);
#else
	tcd_syslog("==========>>>>>>>>>> mac address: [%s]\n", g_hwa_str);
#endif

	// initialize the memory ...
	if (vos_meminit() == -1)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "vos_meminit: failed\n");
#else
		tcd_syslog("vos_meminit: failed\n");
#endif

		close(g_sockfd);
		// exit(-1);
#ifdef USE_MYSQL
		end_db();
#endif

		return -1;
	}

	// obsolete, the data structure was initialized in init_scb()
	// initialize the client data structures ...
	// init_clts(&scb);

	// initialize the server control block data structure ...
	init_scb(&scb);

	// init the logfile
	log_init(scb.svr_logfile);

	// obsoleted ... put to timeout_proc()
	// broadcast the hello packet to initiate the registration. 
	// hello_broadcast(g_sockfd, g_group_sockaddr_in, inet_addr(localip), sport);

	// enter the finite state machine for server, and the return value will be returned in this procedure. 
	retval = srv_fsm_main(g_sockfd, g_group_sockaddr_in, &scb);

	// clear the connected clients
	clear_clts(&scb);

	// clear the active users
	clear_actusrs(&scb);

	// to close the server control socket
	DBG("tcdsrv: close the socket\n");
	close(g_sockfd);

	// close the logfile
	log_close();

#ifdef USE_MYSQL
	end_db();
#endif

	return retval;
}



typedef void (*signal_handler)(int);
void signal_handler_proc(int signo)
{
#ifdef STANDALONE_MODE
	printf("catch signal #%d\n", signo);
#else
	tcd_syslog("catch signal #%d\n", signo);
#endif

	fl_unlink(g_pid_filename_str);

#ifdef USE_MYSQL
	end_db();
#endif


	// to restore the terminal mode
	term_restore();

	tcd_syslog("exit tcdsrv by interrupt...");

	exit(20);
	return;
}

void tcdsrv_usage(char *argv0)
{
	printf("\nUsage: ");

	printf("\n\t%s <port> \n\t\t--- from default outbounding interface eth0", argv0);
	printf("\n\t\t    example: %s 12345", argv0);
	printf("\n\t%s <interf> <port> \n\t\t--- designate outbounding interface", argv0);
	printf("\n\t\t    example: %s eth1 12345", argv0);
	printf("\n\t%s --daemon \n\t\t--- run tcdsvr as backend service on eth0 and port 12345", argv0);
	printf("\n\t%s --daemon <interf> \n\t\t--- run tcdsvr as backend service on designated interface", argv0);
	printf("\n\t%s --daemon <interf> <port>\n\t\t--- run tcdsvr as backend service on designated interface and port", argv0);
	printf("\n\t%s --version \n\t\t--- print the version and build info", argv0);


	printf("\n\nArguments: ");
	printf("\n\t<interf>:\tinbounding interface like eth0, eth1, lo");
	printf("\n\t<port>:\tserver listening UDP port");
	printf("\n\n");

	return;
}

/*****************************************************************************************
// format: 
//	tcdsrv <port>
//	tcdsrv <interf> <port>
//	tcdsrv --daemon
//	tcdsrv --daemon <interf>
//	tcdsrv --daemon <interf> <port>
//	tcdsrv --version
// "port" is the listening UDP port of the receiver and "interface" is something like eth0. 
******************************************************************************************/
int main(int argc, char **argv)
{
	int runpid_fd;
	signal_handler psignal = signal_handler_proc;
	char pidinfo_buf[128];
	int retval;

	// parameter handling
	if (argc != 2 && argc != 3 && argc != 4)
	{
		tcdsrv_usage(argv[0]);
		tcd_syslog("illegal input parameter: argc count [%d]", argc);
		exit(2);
	}

	
	if (!strcmp(argv[1], "--version") || !strcmp(argv[1], "version"))
	{
		print_buildinfo();
		exit(0);
	}
	else if (!strcmp(argv[1], "--daemon") || !strcmp(argv[1], "daemon"))
	{
		//	tcdsrv --daemon
		if (argc == 2)
		{
			strncpy(g_if_name_str, "eth0", sizeof(g_if_name_str) - 1);
			g_port = 12345;

			// in daemon mode, just initialize it
			#ifdef STANDALONE_MODE
			fprintf(stderr, "compiled with standalone mode, cannot use this parameter\n");
			exit(3);
			#else
			daemon(0, 0);
			tcd_syslog("tcdsrv start: --daemon\n");
			#endif
		}

		//	tcdsrv --daemon <interf> <port>
		else if (argc == 4)
		{
			strncpy(g_if_name_str, argv[2], sizeof(g_if_name_str) - 1);
			g_port = atoi(argv[3]);

			// in daemon mode, just initialize it
			#ifdef STANDALONE_MODE
			fprintf(stderr, "compiled with standalone mode, cannot use this parameter\n");
			exit(3);
			#else
			daemon(0, 0);
			tcd_syslog("tcdsrv start: --daemon %s %s\n", argv[2], argv[3]);
			#endif
		}

		// 	tcdsrv --daemon <interface>
		else // argc == 3
		{
			strncpy(g_if_name_str, argv[2], sizeof(g_if_name_str) - 1);
			g_port = 12345;

			// in daemon mode, just initialize it
			#ifdef STANDALONE_MODE
			fprintf(stderr, "compiled with standalone mode, cannot use this parameter\n");
			exit(3);
			#else
			daemon(0, 0);
			tcd_syslog("tcdsrv start: --daemon %s\n", argv[2]);
			#endif
		}
	}
	else
	{
		//	tcdsrv <port>
		if (argc == 2)
		{
			#ifdef STANDALONE_MODE
			strncpy(g_if_name_str, "eth0", sizeof(g_if_name_str) - 1);
			g_port = atoi(argv[1]);
			tcd_syslog("tcdsrv %s\n", argv[1]);
			#else
			fprintf(stderr, "compiled with daemon mode, cannot use this parameter\n");
			exit(3);
			#endif
		}

		//	tcdsrv <interf> <port>
		else if (argc == 3)
		{
			#ifdef STANDALONE_MODE
			// %s <interf> <port>
			strncpy(g_if_name_str, argv[1], sizeof(g_if_name_str) - 1);
			g_port = atoi(argv[2]);
			tcd_syslog("tcdsrv %s %s\n", argv[1], argv[2]);
			#else
			fprintf(stderr, "compiled with daemon mode, cannot use this parameter\n");
			exit(3);
			#endif
		}

		else
		{
			fprintf(stderr, "incorrect argc: %d\n", argc);
			exit(2);
		}
	}

#ifdef STANDALONE_MODE
	printf("install signal handler ...\n");
#else
	tcd_syslog("install signal handler ...\n");
#endif
	signal(SIGINT, psignal);
	signal(SIGTERM, psignal);
	
	//tcdrunpidfn = get_tcdpid_fn(INIFN);
	if ( NULL == (get_tcdpid_fn_ex(g_pid_filename_str, sizeof(g_pid_filename_str), INIFN))){
        tcd_syslog("fail to get pid filename via %s\n", INIFN);
        exit (4);
    }

	if (g_pid_filename_str[0] == '\0')
	{
		#ifdef STANDALONE_MODE
		printf("find tcdpidfn failed, please check your init file\n");
		#else
		tcd_syslog("find tcdpidfn failed, please check your ini file");
		#endif

		exit(4);
	}
	else
	{
		#ifdef STANDALONE_MODE
		printf("tcdpidfn [%s]\n", g_pid_filename_str);
		#else
		tcd_syslog("tcdpidfn [%s]\n", g_pid_filename_str);
		#endif
	}

	if (get_tcdsvr_parms(INIFN) == 0)
	{
		g_tm_base = RANDOM_DELAY_BASE;
		g_tm_coeff = RANDOM_DELAY_COEFF;
		g_tm_intpktdelay = DEFAULT_INTPKT_DELAY;

		#ifdef STANDALONE_MODE
		printf("STD: get_tcdsvr_parms() will set with the default value\n");
		printf("STD: g_tm_base: %d g_tm_coeff: %d g_tm_intpktdelay: %d\n", g_tm_base, g_tm_coeff, g_tm_intpktdelay);
		#else
		tcd_syslog("get_tcdsvr_parms() will set with the default value");
		tcd_syslog("g_tm_base: %d g_tm_coeff: %d g_tm_intpktdelay: %d\n", g_tm_base, g_tm_coeff, g_tm_intpktdelay);
		#endif
	}
	else
	{
		#ifdef STANDALONE_MODE
		printf("g_tm_base: %d g_tm_coeff: %d g_tm_intpktdelay: %d\n", g_tm_base, g_tm_coeff, g_tm_intpktdelay);
		#else
		tcd_syslog("g_tm_base: %d g_tm_coeff: %d g_tm_intpktdelay: %d\n", g_tm_base, g_tm_coeff, g_tm_intpktdelay);
		#endif
	}

    // add wr-lock
    // to run only an instance
    // D_TCD_WLOCK defined in tcd.h
#ifdef D_TCD_WLOCK
    if (0 > fl_tcd_wlock()){
        tcd_syslog("there is another instance running, quit!\n");
        exit(11);
    }
    else{
        tcd_syslog("get wrlock successfully, the daemon is about to be started....\n");
    }
#endif

	if (fl_stat(g_pid_filename_str) == 0)
	{
		// another process exists, quit
#ifdef STANDALONE_MODE
		fprintf(stderr, "file %s exists already\n", g_pid_filename_str);
#else
		tcd_syslog("file %s exists already", g_pid_filename_str);
#endif

#ifdef D_TCD_WLOCK
//		exit(1);
#else
		exit(1);
#endif
	}

	else if ((runpid_fd = fl_creat(g_pid_filename_str)) == -1)
	{
#ifdef STANDALONE_MODE
		fprintf(stderr, "create runtime pid file [%s] failed\n", g_pid_filename_str);
#else
		tcd_syslog("create runtime pid file [%s] failed", g_pid_filename_str);
#endif
		exit(10);
	}

#ifdef D_TCD_WLOCK
    atexit(tcd_cleanup_resource);
#endif
	
	sprintf(pidinfo_buf, "%d", getpid());
	write(runpid_fd, pidinfo_buf, strlen(pidinfo_buf));

	fl_close(runpid_fd);

#if 1
	if (!strcmp(argv[1], "--daemon") || !strcmp(argv[1], "daemon"))
	{
		tcd_syslog("server start in daemon mode");
		retval = server_main_routine();
		while(retval != -1)
		{
			tcd_syslog("server exits from daemon mode normally");

			tcd_syslog("server start in daemon mode");
			retval = server_main_routine();
		}
	}
	else
	{
		printf("server start in standalone mode");
		retval = server_main_routine();
		while(retval != -1)
		{
			printf("server exits from standalone mode normally");

			printf("server start in standalone mode");
			retval = server_main_routine();
		}
	}
#endif


	fl_unlink(g_pid_filename_str);

	tcd_syslog("exit tcdsrv ...");

	exit(0);

}





