/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "tm.h"

// to get the current time stamp value
struct timeval getts()
{
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv, &tz);
	return tv;
}

void printts(struct timeval tv)
{
	printf("%ld:%ld", tv.tv_sec, tv.tv_usec);
	return;
}

uint64 diff_tv(struct timeval starttv, struct timeval endtv)
{
	struct timeval result;
	uint64 diff;

	if (endtv.tv_sec < starttv.tv_sec)
		return -1;
	
	if (endtv.tv_sec == starttv.tv_sec && endtv.tv_usec < starttv.tv_usec)
		return -1;

	result.tv_sec = endtv.tv_sec - starttv.tv_sec; 
	result.tv_usec = endtv.tv_usec - starttv.tv_usec; 

	if (result.tv_usec < 0)
	{
		result.tv_sec --;
		result.tv_usec += 1000000;
	}

	diff = (uint64)(result.tv_sec) * 1000000 + (uint64)(result.tv_usec);

	return diff;
}

void vos_msleep(uint32 msec)
{
	usleep(msec*1000);
}

// obsoleted ...
/*
void vos_usleep(uint32 usec)
{
	if (usec == 0)
		return;
	else
		usleep(usec);
	return;
}

void vos_usleep1(uint32 usec)
{
	volatile int i, j, k;

	i=j=k=0;
	while (k++ < usec)
		while (j++<30000)
		{
			__asm__ __volatile__("nop\nnop\nnop\nnop\nnop\nnop\nnop\nnop"::);
		}
	
}
*/

void vos_usleep(uint32 usec)
{
	struct timeval tv1, tv2;
	uint64 diff;

	if (usec > 0)
	{
		tv1 = getts();
		while(1)
		{
			// __asm__ __volatile__("nop\nnop\nnop\nnop\nnop\nnop\nnop\nnop");
			__asm__ __volatile__("nop\n");

			tv2 = getts();
			diff = diff_tv(tv1, tv2);
			// printf("diff=%lld\n", diff);
			if (diff > usec)
				break;
		}
	}
	return;
}


// to sleep 0.01~0.99 s
void vos_random_msleep()
{
	uint32 r = rand() % 100;

	printf("vos_random_msleep: %d ms\n", r*10);
	vos_msleep(r * 10);
}


// to sleep 0.01~0.99 s
void vos_random_usleep()
{
	uint32 r = rand() % 100;

	// printf("vos_random_usleep: %d us\n", r*10);
	vos_usleep(r * 10);
}


// time string to seconds elapsed since Epoch time
// "2018-06-08 19:09:19" -> 1528456159
time_t str2timet(const char *buf, const char *fmt)
{
    struct tm ts;
    strptime(buf, fmt, &ts);
    return mktime(&ts);
    //return timelocal(&ts);
}

// seconds elapsed since Epoch time to time string
// 1528456159 -> "2018-06-08 19:09:19"
char *timet2str(char *buf, int sz, const char *fmt, time_t *t)
{
    struct tm ts;
    localtime_r(t, &ts);
    strftime(buf, sz, fmt, &ts);
    return buf;
}

// get time string
// link with -lrt
//char *clktmstr(char *buf, int sz, const char *fmt)
//{
//    struct timespec ts = {0};
//    struct tm tm = {0};
//
//    if (!buf || (sz < 20) ||  !fmt)
//    {
//        return NULL;
//    }
//    if (0 != clock_gettime(CLOCK_REALTIME, &ts))
//        return NULL;
//    localtime_r(&ts.tv_sec, &tm);
//    strftime(buf, sz, fmt, &tm);
//    return buf;
//}
//
