/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
this software module.
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "conf.h"
#include "tcd.h"
#include "vos_mem.h"
#include "tcdmsg.h"


typedef struct memblk_struc
{
	char memblk[BUFSZ+128];
	int used;
}MEMBLK;

// MEMBLKSZ is defined in conf.h
MEMBLK *memblk[MEMBLKSZ];

int vos_meminit()
{
#ifdef USE_OS_MALLOC
	return 0;
#else
	int k;

	// to allocate a large block of memory ...
	for (k=0; k<MEMBLKSZ; k++)
	{	
		if ((memblk[k] = (MEMBLK *)malloc(sizeof(MEMBLK))) == NULL)
		{
			fprintf(stderr, "vos_meminit: malloc failed\n");
			return -1;
		}
		memblk[k]->used = 0;
	}

	return 0;

#endif
}

void *vos_malloc()
{

#ifdef USE_OS_MALLOC
	return malloc(BUFSZ);
#else

	MEMBLK *mp;
	int k;

	k = -1;
	while(++k < MEMBLKSZ)
	{
		mp = memblk[k];
		if (mp->used == 0)
		{
			mp->used = 1;
			bzero(mp->memblk, BUFSZ+128);
			return (void *)(mp->memblk);
		}
	}
	fprintf(stderr, "vos_malloc: empty\n");
	return NULL;
#endif
}

void vos_free(void *p)
{
#ifdef USE_OS_MALLOC

	return free(p);
#else

	if (p != NULL)
	{
		MEMBLK *mp = CR(p, MEMBLK, memblk);
		mp->used = 0;
	}
	else
		printf("vos_free: empty\n");

	return;

#endif

}

void vos_memprint()
{
#ifdef USE_OS_MALLOC

	printf("using OS memory management\n");
	return;

#else

	int k;

	printf("list of memory blocks ...\n");
	// to allocate a large block of memory ...
	for (k=0; k<MEMBLKSZ; k++)
	{	
		if (memblk[k]->used == 0)
			printf("%d --- %s\n", k, "notused");
		else
			printf("%d --- %s\n", k, "used");

	}

	return;
#endif

}


void print_buf(unsigned char *chp, int length)
{
	int i=0;
	
	for (i=0; i<length; i++)
		printf("0x%02X[%1c] ", *(chp+i), isprint(*(chp+i))?*(chp+i):'.');
	
	return;
}
/*
main()
{
	char *chp1, *chp2, *chp3, *chp4, *chp5;

	vos_meminit();
	vos_memprint();

	chp1 = vos_malloc();
	vos_memprint();
	
	chp2 = vos_malloc();
	vos_memprint();

	chp3 = vos_malloc();
	vos_memprint();

	chp4 = vos_malloc();
	vos_memprint();

	chp5 = vos_malloc();
	vos_memprint();

	vos_free(chp1);
	vos_free(chp3);
	vos_memprint();
	
}

*/




