#!/bin/sh
# create a virtual disk drive
#
source /usr/sbin/tc/global_def.sh

SCRIPT_PATH=$0
SCRIPT_NAME=${SCRIPT_PATH##*/}

# the lower bound
VDSZ_LOW=$((1 * 1024))

usage()
{
    cat <<EOT
SYNSNOPSIS
    $SCRIPT_NAME <virtualDiskSize>

DESCRIPTION
    Create a file used as a hard drive.
    <virtualDiskSize> is an integer which means total storage (unit: MiB) for the
    virtual hard drive. The legal value should be not smaller than ${VDSZ_LOW}.

EXAMPLE
    "$SCRIPT_NAME 6144" create a virtual hard drive with 6 GiB storage. 
EOT
}

# check argument
if [ $# -ne 1 ] ; then
    usage
    exit 1
else
    vdsz=$1
    if [ $vdsz -lt $VDSZ_LOW ] ; then
        exit 2
    fi
fi

# check mounting status
mount | grep -q $DISKDEV
if [ ! $? -eq 0 ] ; then
    exit 3
fi

# check available storage
DISKSZ=`df -m $DISKDEV | sed -n '2p' | awk '{print $4}'`
if [ $vdsz -ge $DISKSZ ] ; then
    exit 4
fi

TMPIMG=$DISKDIR/$TMPDISK

if [ ! -f $TMPIMG ] ; then
  dd if=/dev/zero of=$TMPIMG seek=$(($vdsz - 1)) bs=1M count=1
  losetup /dev/loop0 $TMPIMG
fdisk /dev/loop0 <<EOT
n
p
1


t
7
w
EOT
  mkfs.ntfs -f /dev/loop0p1
  #mkfs.ntfs --fast --force --label USER --cluster-size 4096 --sector-size 512 \
  #          --partition-start 2048 --sectors-per-track 63 /dev/loop0p1
  losetup -d /dev/loop0
fi

$TCDIR/createMdfUdf $TMPIMG tmp $DISKDIR

echo "BASEIMG:\\Device\\Harddisk0\\Partition1\\$TMPDISK" > $BOOT_FILE1
echo "UDFIMG:\\Device\\Harddisk0\\Partition1\\$TMPDISK-tmp.udf" >> $BOOT_FILE1
echo "MDFIMG:\\Device\\Harddisk0\\Partition1\\$TMPDISK-tmp.mdf" >> $BOOT_FILE1
