#!/bin/sh

. /usr/sbin/tc/global_def.sh

if [ -z "$1" -o -z "$2" -o -z "$3" -o -z "$4" ] ; then
  echo "Usage: _get_data.sh <curimg> <username> <osid> <orgimg>"
  exit 1
fi

if [ -n "$5" ] ; then
  SERVER=$5
  echo SERVER=$5
fi

UID=`cat $UID_FILE`
echo "user id: $UID"

DATFILE=data-$2.img
MDFFILE=$DATFILE-$2.mdf
UDFFILE=$DATFILE-$2.udf
CDFFILE=$DATFILE-$2.cdf

VERFILE=$DISKDIR/$DATFILE.ver


curl $CURL_OPTS -X GET --retry 3 --retry-delay 2 --output $PDATA_JSON $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID
if [ $? -ne 0 ] ; then
  echo "curl user data info failed"
  exit 3
fi
jret=`jq -r '.success' $PDATA_JSON`
if [ "$jret"x != "true"x ] ; then
  echo "query user data info failed"
  exit 51
fi
gp_size=`jq -r '.result[0].storage[0].group_size' $PDATA_JSON`
pd_size=`jq -r '.result[0].storage[0].user_size' $PDATA_JSON`
pd_rver=`jq -r '.result[0].storage[0].revision' $PDATA_JSON`
pfrozen=`jq -r '.result[0].storage_frozen' $PDATA_JSON`
if [ $pd_size -eq 0 ] ; then
  echo "zero size, call web to create it"
  curl $CURL_OPTS -X POST --output $PDATA_JSON -H "Content-Type: application/json" -d '{"uri":"head","size":"'$gp_size'"}' $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID
  if [ $? -ne 0 ] ; then
    echo "communicate with server timeout"
    exit 3
  fi
  jret=`jq -r '.success' $PDATA_JSON`
  if [ "$jret"x != "true"x ] ; then
    echo "call web to create user data failed"
    exit 52
  fi
fi

if [ -f $VERFILE ] ; then
  CURVER=`cat $VERFILE`
  echo "data's ver: c=$CURVER, s=$pd_rver"

  if [ $CURVER -gt $pd_rver ] ; then
    echo "delete user data"
    rm -f $DISKDIR/data-$2.*
  fi
fi

if [ ! -f $VERFILE ] ; then
  echo "data img not exist"

  $AUTH_APP -i $SERVER -p $SERVERPORT -q $UID:0 -f $DATASYNC_FILE -t $TOKEN_FILE
  if [ $? -ne 0 ] ; then
    echo "sync user data info(v=0) failed"
    exit 53
  fi

  filesize=`cat $DATASYNC_FILE | cut -d : -f 5`
  if [ -z "$filesize" ] ; then 
    echo "invalid user data file length" 
    exit 54
  fi

  echo "create data file: ${filesize}MB"
  filesize=$((filesize-1))
  dd if=/dev/zero of=$DISKDIR/$DATFILE seek=$filesize bs=1M count=1
  if [ $? -ne 0 ] ; then
    echo "init user data file failed"
    exit 55
  fi

  echo "1" >$VERFILE
fi


#$TCDCLT_APP $SERVER eth0 12345 "3:$2:$3" >/tmp/clt.log 2>&1
echo "3:$2:$3" >$STATUS_CONF_FILE


CURVER=`cat $VERFILE`
echo "data's ver: c=$CURVER, s=$pd_rver"

rm -f $DATASYNC_FILE
$AUTH_APP -i $SERVER -p $SERVERPORT -q $UID:$CURVER -f $DATASYNC_FILE -t $TOKEN_FILE
if [ $? -ne 0 ] ; then
  echo "sync user data info(v=$CURVER) failed"
  exit 56
fi

if [ -f $DATASYNC_FILE ] ; then
  echo "delete $DISKDIR/$DATFILE-*"
  rm -f $DISKDIR/$DATFILE-*

  $MERGEDATA_APP $1 $2 $3 $4 $SERVER
  ret=$?
  if [ $ret -ne 0 ] ; then
    echo "upgrade user data file failed: $ret"
    exit $ret
  fi
fi

if [ "x$pfrozen" == "xtrue" ] ; then
  echo "user data is frozen"
  rm -f $DISKDIR/$MDFFILE $DISKDIR/$UDFFILE
else
  echo "user data not frozen"
fi

if [ ! -f $DISKDIR/$MDFFILE -o ! -f $DISKDIR/$UDFFILE ] ; then
  $TCDIR/createMdfUdf $DISKDIR/$DATFILE $2 $DISKDIR
  if [ $? -ne 0 ] ; then
    echo "create mdf/udf failed"
    exit 57
  fi
fi

echo "BASEIMG:\\Device\\Harddisk0\\Partition1\\$DATFILE" >$BOOT_FILE2
echo "UDFIMG:\\Device\\Harddisk0\\Partition1\\$UDFFILE" >>$BOOT_FILE2
echo "MDFIMG:\\Device\\Harddisk0\\Partition1\\$MDFFILE" >>$BOOT_FILE2


if [ "x$pfrozen" != "xtrue" ] ; then
  autoupload=0
  curl $CURL_OPTS -X GET --retry 3 --retry-delay 2 --output $SYSCONF_JSON $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID
  if [ $? -eq 0 ] ; then
    jret=`jq -r '.success' $SYSCONF_JSON`
    if [ "$jret"x == "true"x ] ; then
      autoupload=`jq -r '.result[0].storage_background_upload' $SYSCONF_JSON`
      if [ $autoupload -eq -1 ] ; then
        echo "use global autoupload setting"
        autoupload=`jq -r '.result[0].storage_background_upload_global' $SYSCONF_JSON`
      fi
      echo "autoupload=$autoupload"
    fi
  fi

  if [ "x$autoupload" == "x1" ] ; then
    mac=`$GET_HWADDR_SCRIPT $DEFAULT_IFACE -`
    curl $CURL_OPTS -X POST --output $PDATA_JSON -H "Content-Type: application/json" -d '{"status":"lock","token":"'$mac'"}' $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID
    if [ $? -ne 0 ] ; then
      echo "communicate with server timeout"
      exit 3
    fi
    jret=`jq -r '.success' $PDATA_JSON`
    if [ "$jret"x != "true"x ] ; then
      echo "use $mac to lock PS failed"
      exit 45
    fi
    echo "use $mac to lock PS successfully"

    if [ ! -f $DISKDIR/$CDFFILE ] ; then
      echo "$CDFFILE not exist, copy $MDFFILE as"
      cp $DISKDIR/$MDFFILE $DISKDIR/$CDFFILE
    fi
    echo "AUTOUP:\\Device\\Harddisk0\\Partition1\\$CDFFILE" >>$BOOT_FILE2
  else
    echo "auto upload disabled, delete $CDFFILE"
    rm -f $DISKDIR/$CDFFILE
  fi
fi


echo "-drive if=virtio,format=tcmudf,file=$DISKDIR/$DATFILE,mdf=$DISKDIR/$MDFFILE,udf=$DISKDIR/$UDFFILE" >$VMDRIVE_FILE2

# for offline mode
cp $BOOT_FILE2 $DISKDIR/$2-$4.osenv2
