#!/bin/sh

. /usr/sbin/tc/global_def.sh

if [ $# -lt 4 ] ; then
  echo "Usage: create_cache.sh <osimg> <img_size> <cache_file> <blk_size>"
  exit 1
fi

exec=$TCDIR/CreateCache

export osimg=$1
export img_size=$2
export cache_file=$3
export blk_size=$4


build_bitmap_ntfs()
{
  cl_size=`ntfsinfo -m $1 | awk -F : '/Cluster\ Size/ {print $2}'`
  if [ -z $cl_size ] ; then
    echo "  get cluster size failed"
    return 5
  fi

  ntfscat $1 \$Bitmap >$3
  if [ $? -ne 0 ] ; then
    echo "  dump mft bitmap failed"
    return 6
  fi

  return 0
}

process_partition()
{
  echo "process partition[$1]: off=$2, len=$3"

  LOOPDEV=`losetup -f`
  losetup -o $2 $LOOPDEV "$osimg"
  if [ $? -ne 0 ] ; then
    echo "  losetup on $LOOPDEV failed"
    exit 4
  fi
  echo "  losetup on $LOOPDEV"

  blk_id=`blkid $LOOPDEV`
  fs_type=`echo ${blk_id##*TYPE=}`
  echo "  fs type=$fs_type"

  case "$fs_type" in
    \"ntfs\")
      SUFFIX=ntfs 
      ;;

    *)
      SUFFIX=
      ;;
  esac

  if [ -z $SUFFIX ] ; then
    echo "  skip it"
  else
    bmfile=/tmp/$1.bitmap
    build_bitmap_$SUFFIX $LOOPDEV $3 $bmfile
    if [ $? -eq 0 ] ; then
      echo "  bitmap OK, cluster size=$cl_size"

      $exec m $cache_file $img_size $blk_size $bmfile $2 $3 $cl_size
      if [ $? -ne 0 ] ; then
        echo "merge cache file failed"
      fi
    fi
  fi

  losetup -d $LOOPDEV

  echo
}


$exec c $cache_file $img_size $blk_size
if [ $? -ne 0 ] ; then
  echo "create cache file failed"
  exit 2
fi

parts=`fdisk -l -u "$osimg" | grep -w "$osimg[1-9]"`
if [ -z "$parts" ] ; then
  echo "$osimg not exist or no partitions"
  exit 3
fi

# "\n" will missing if use "echo $parts"
echo "$parts" | awk -F ' ' '
  BEGIN {
    sec_size=512
  }

  {
    if($2 ~ /^[0-9]+$/)
    {
      sec_start = $2;
      sec_end = $3;
    }
    else
    {
      sec_start = $3;
      sec_end = $4;
    }

    part_idx = substr($1, length($1), 1);
    part_off = sec_start * sec_size;
    part_len = (sec_end - sec_start + 1) * sec_size;
    print part_idx, part_off, part_len;
  }

  END {
  }
  ' | while read argv
  do
    process_partition $argv  # not use "$argv"
  done

