#/bin/sh
# get-disk-size.sh
# DESCRIPTION
#     compute the size for a block device
# PARAMETERS
#     optional, use the block node `/dev/sda' by default
# RETURN VALUE
#     0 on success, and display a number for disk size (Unit: MiB) on stdout,
#     otherwise, return a non-zero value,  and display error message on stderr.
#
SCRIPT_PATH=$0
SCRIPT_NAME=${SCRIPT_PATH##*/}

ERROR_SUCCESS=0
ERROR_NO_DEVICE=1
ERROR_NO_BLOCK_INFO=2
ERROR_RUNTIME=3

DISKDEV='/dev/sda'
if [ -n "$1" ] ; then
    DISKDEV=$1
fi

DISK_SIZE_FILE='/sys/block/'${DISKDEV##*/}'/size'
DISK_SIZE_UNIT_FILE='/sys/block/'${DISKDEV##*/}'/queue/hw_sector_size'

tcerr(){
    echo [ERROR] $@ >&2
}

if ! [ -b $DISKDEV ] ; then
    tcerr "$DISKDEV : no such file"
    exit $ERROR_NO_DEVICE
fi

if ! [ -f $DISK_SIZE_FILE ] ; then
    tcerr "$DISK_SIZE_FILE : no such file"
    exit $ERROR_NO_BLOCK_INFO
fi

if ! [ -f $DISK_SIZE_UNIT_FILE ] ; then
    tcerr "$DISK_SIZE_UNIT_FILE : no such file"
    exit $ERROR_NO_BLOCK_INFO
fi

DISK_SIZE=`cat ${DISK_SIZE_FILE}`
DISK_SIZE_UNIT=`cat ${DISK_SIZE_UNIT_FILE}`
DISK_SIZE_MIB=`echo "${DISK_SIZE} * ${DISK_SIZE_UNIT} / 1024 / 1024" | bc`
retval=$?

if ! [ $retval -eq 0 ] ; then
    tcerr "fail to get disk size for $DISKDEV"
    exit $ERROR_RUNTIME
fi

#echo "Units of sectors: $DISK_SIZE"
#echo "Sector size: $DISK_SIZE_UNIT"
echo $DISK_SIZE_MIB
exit $ERROR_SUCCESS

