#!/bin/sh
# get-hwaddr.sh
# get Ethernet HWaddr
# on success, exit 0
# otherwise, exit with a non-zero value
SCRIPT_PATH=$0

case $# in
    1)
        NIF=$1
        SEP=""
        ;;
    2)
        NIF=$1
        SEP=$2
        ;;
    *)
        cat <<EOT
SYNOPSIS
    $SCRIPT_PATH <Interface> [Separator]

DESCRIPTION
    To get the hardware addess for network card with name <Interface>, return
a string separated by <Separator>.

EXAMPLE
    $SCRIPT_PATH eth0
    The output may be:
    90E2BA8E3D28

    $SCRIPT_PATH eth0 :
    The output may be:
    90:E2:BA:8E:3D:28

    $SCRIPT_PATH eth0 -
    The output may be:
    90-E2-BA-8E-3D-28
EOT
        exit 1
        ;;
esac

ifconfig | grep -q "^${NIF}"
if [ $? -ne 0 ] ; then
    exit 2
fi

ifconfig $NIF | grep -q -E "^${NIF}.*HWaddr"
if [ $? -ne 0 ] ; then
    exit 3
fi

HWADDR_COLON=`ifconfig $NIF | grep -E "^${NIF}.*HWaddr" | awk '{print $NF}'`
HWADDR_MINUS=`echo $HWADDR_COLON | tr ':' '-'`
HWADDR_NOSEP=`echo $HWADDR_COLON | tr -d ':'`

case $SEP in
    ":")
        HWADDR=$HWADDR_COLON
        ;;
    "-")
        HWADDR=$HWADDR_MINUS
        ;;
    *)
        HWADDR=$HWADDR_NOSEP
        ;;
esac

echo $HWADDR
exit 0

