#!/bin/sh
# get-qrcode-url.sh
# To get the URL for the current TC client. One can access the URL to download
# QR code file after that.
# return 0 on success, otherwise return a non-zero value.

SCRIPT_NAME=$0
. /usr/sbin/tc/global_def.sh

SCRIPT_GET_HWADDR=$TCDIR/get-hwaddr.sh
NIF=$DEFAULT_IFACE
if [ -f $IFACE_FILE ] ; then
  NIF=`cat $IFACE_FILE`
fi
SEPARATOR="-"

if [ $# -eq 0 ] ; then
    serverip=${SERVER}
    mount | grep -q -E "${DISKRSVD}.*${RSVDDIR}"
    if [ $? -eq 0 ] ; then
        mounted=YES
        mount_action=NO
    else
        mount_action=YES

        mount ${DISKRSVD} ${RSVDDIR}
        if [ $? -eq 0 ] ; then
            mounted=YES
        else
            mounted=NO
        fi
    fi

    if [ "x${mounted}" == "xYES" -a  -f ${SERVADDR_FILE} ] ; then
        serverip=`cat ${SERVADDR_FILE}`
    fi

    if [ "x${mounted}" == "xYES" -a "x${mount_action}" == "xYES" ] ; then
        umount ${RSVDDIR}
    fi
elif [ $# -eq 1 ] ; then
    serverip=$1
else
    cat <<EOT
SYNOPSIS
    $SCRIPT_NAME [ServerIP]

DESCRIPTION
    Query from \`ServerIP' the QR code download URL for a client

EXAMPLE
    $SCRIPT_NAME 192.168.0.1
EOT
    exit 1
fi

sh $SCRIPT_GET_HWADDR $NIF $SEPARATOR >/dev/null 2>&1
if [ $? -ne 0 ] ; then
    exit 2
fi

HWADDR=`sh $SCRIPT_GET_HWADDR $NIF $SEPARATOR`
URL_CLIENT=$HTTP_PROT://$DOMAINNAME/tc/rest/client.php
JSON_TMP=/tmp/get-qrcode-url.json
URL_DST=

if [ -f $JSON_TMP ] ; then
    rm -f $JSON_TMP
fi

curl $CURL_OPTS -s -X POST --retry 3 --retry-delay 2 --output $JSON_TMP --data "{\"mac\":\"${HWADDR}\"}" --header "Content-Type: application/json" $URL_CLIENT

if [ $? -eq 0 ] ; then
    if [ -f $JSON_TMP ] ; then
        t=`jq -r '.success' $JSON_TMP`
        if [ "x$t" == "xtrue" ] ; then
            ret=`jq -r '.result.qrcode' $JSON_TMP`
            URL_DST="$HTTP_PROT://${DOMAINNAME}${ret}"
        else
            exit 2
        fi
    else
        exit 3
    fi
else
   exit 4
fi

echo "$URL_DST"
exit 0

