#!/bin/sh
# post-hw.sh
# DESCRIPTION
#     post hardware information to TC server
# PARAMETER
#     none
# RETURN VALUE
#     0 on success
#     otherwise, return a non-zero value
# 
#
SCRIPT_PATH=$0
SCRIPT_NAME=${SCRIPT_PATH##*/}
. /usr/sbin/tc/global_def.sh

ERROR_SUCCESS=0
ERROR_RUNTIME=3

tcerr(){
    echo [ERROR] $@ >&2
}

if [ $# -eq 0 ] ; then
    serverip=${SERVER}
    mount | grep -q -E "${DISKRSVD}.*${RSVDDIR}"
    if [ $? -eq 0 ] ; then
        mounted=YES
        mount_action=NO
    else
        mount_action=YES

        mount ${DISKRSVD} ${RSVDDIR}
        if [ $? -eq 0 ] ; then
            mounted=YES
        else
            mounted=NO
        fi
    fi

    if [ "x${mounted}" == "xYES" -a  -f ${SERVADDR_FILE} ] ; then
        serverip=`cat ${SERVADDR_FILE}`
    fi

    if [ "x${mounted}" == "xYES" -a "x${mount_action}" == "xYES" ] ; then
        umount ${RSVDDIR}
    fi
elif [ $# -eq 1 ] ; then
    serverip=$1
else
    cat <<EOT
SYNOPSIS
    $SCRIPT_NAME [ServerIP]

DESCRIPTION
    post hardware information to \`ServerIP' for the client

EXAMPLE
    $SCRIPT_NAME 192.168.0.1
EOT
    exit 1
fi

URL_POST_HW=$HTTP_PROT://$DOMAINNAME/tc/rest/client.php
NIF=$DEFAULT_IFACE
if [ -f $IFACE_FILE ] ; then
  NIF=`cat $IFACE_FILE`
fi
SEPARATOR="-"

# cpu information
if ! [ -f $GET_CPU_SCRIPT ] ; then
    tcerr "$GET_CPU_SCRIPT : no such file"
    exit $ERROR_RUNTIME
fi
str_cpu=`sh $GET_CPU_SCRIPT`
retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "$GET_CPU_SCRIPT : run"
    exit $ERROR_RUNTIME
fi

# hwaddr information
if ! [ -f $GET_HWADDR_SCRIPT ] ; then
    tcerr "$GET_HWADDR_SCRIPT : no such file"
    exit $ERROR_RUNTIME
fi
str_hwaddr=`sh $GET_HWADDR_SCRIPT $NIF $SEPARATOR`
retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "$GET_HWADDR_SCRIPT : run"
    exit $ERROR_RUNTIME
fi

# memory information
if ! [ -f $GET_MEMORY_SCRIPT ] ; then
    tcerr "$GET_MEMORY_SCRIPT : no such file"
    exit $ERROR_RUNTIME
fi
str_memory=`sh $GET_MEMORY_SCRIPT`
retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "$GET_MEMORY_SCRIPT : run"
    exit $ERROR_RUNTIME
fi

# disk information
if ! [ -f $GET_DISK_SIZE_SCRIPT ] ; then
    tcerr "$GET_DISK_SIZE_SCRIPT : no such file"
    exit $ERROR_RUNTIME
fi
str_disk=`sh $GET_DISK_SIZE_SCRIPT`
retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "$GET_DISK_SIZE_SCRIPT : run"
    exit $ERROR_RUNTIME
fi
# convert to GiB
# save 2 digit after the dot
str_disk=`echo "scale=2; $str_disk / 1024" | bc -l`
retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "fail to run the bc ultility"
    exit $ERROR_RUNTIME
fi

# firmware information
str_firmware=`cat /usr/sbin/tc/VERSION`
retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "fail to get firmware version information"
    exit $ERROR_RUNTIME
fi

# post the data
JSON_RETURN=/tmp/post-1.json
if [ -f $JSON_RETURN ] ; then
    rm -rf $JSON_RETURN
fi

# JSON format
# {
#     "mac":"00-00-00-00-00-00",
#     "cpu_model":"Intel(R) Xeon(R) CPU E5-2650 0 @ 2.00GHz",
#     "memory_size":"32844",
#     "disk_size":"58.00"
#     "firmware_version":"1902.8864"
# }
curl $CURL_OPTS -s -X POST --retry 3 --retry-delay 2 --output $JSON_RETURN \
--header "Content-Type: application/json" $URL_POST_HW \
--data "{\"mac\":\"${str_hwaddr}\",\"cpu_model\":\"${str_cpu}\",\"memory_size\":\"${str_memory}\",\"disk_size\":\"${str_disk}\",\"firmware_version\":\"${str_firmware}\"}"
#--data '{"mac":"'${str_hwaddr}'","cpu_model":"'${str_cpu}'","memory_size":"'${str_memory}'","disk_size":"'${str_disk}'"}'

retval=$?
if ! [ $retval -eq 0 ] ; then
    tcerr "curl: fail to post hardware information"
    exit $ERROR_RUNTIME
fi

if ! [ -f $JSON_RETURN ] ; then
    tcerr "$JSON_RETURN : no such file"
    exit $ERROR_RUNTIME
else
    t=`jq -r '.success' $JSON_RETURN`
    if [ -z "$t" ] ; then
        tcerr "$JSON_RETURN : invalid respose"
        exit $ERROR_RUNTIME
    else
        exit $ERROR_SUCCESS
    fi
fi
