#!/bin/sh

. /usr/sbin/tc/global_def.sh

# rsvd & disk error define
#-----------------------------------
# 0: ready, 1: fschking, 2: mounted but full, 3: not mounted
# 11: partition not exist, 12: fschk failed, 13: fschk ok but need reboot, 14: mount failed, 15: invalid TC partition
#-----------------------------------

# network error define
#-----------------------------------
# 0: ready, 1: line down, 2: no ip, 3: tc server down, 4: tc service down
#-----------------------------------


# prepare dbus
#-----------------------------------
# because we use "openvt" to launch startx,
# so put dbus-launch here to avoid conflict.
if [ ! -f $DBUSENV ] ; then
  echo "#!/bin/sh" > $DBUSENV
  dbus-launch --sh-syntax >> $DBUSENV
  chmod +x $DBUSENV
fi
. $DBUSENV
#-----------------------------------


# mount system partition
#-----------------------------------
cnt=0
until [ $cnt -ge 5 ] ; do
  if [ -b $HDDEV ] ; then
    break;
  fi

  sleep 1
  cnt=$((cnt+1))
done

if [ $cnt -lt 5 ] ; then
  mount $DISKRSVD $RSVDDIR
  if [ $? -ne 0 ] ; then
    rsvd_err=14
  else
    rsvd_err=0
  fi
else
  rsvd_err=11
fi
#-----------------------------------


# run preboot UI
#-----------------------------------
if [ -n "$1" ] ; then
  SERVER=$1
  echo SERVER=$SERVER
  echo $SERVER >$SERVADDR_FILE
fi

if [ $rsvd_err -eq 0 ] ; then
  if [ -z "$1" -a -f $SERVADDR_FILE ] ; then
    SERVER=`cat $SERVADDR_FILE`
    echo SERVER=$SERVER
  fi

  if [ -f $RSVDDIR/$ADMINPWD_FILE ] ; then
    cp $RSVDDIR/$ADMINPWD_FILE $TCDIR/
  fi

  if [ -f $PBBG_FILE ] ; then
    cp $PBBG_FILE $PBBG_THEMEP
  fi

  # speedup the UI launch time
  mkdir -p $RSVDDIR/mtcache
  ln -s $RSVDDIR/mtcache /var/cache
fi

if [ ! -f $SERVADDR_FILE ] ; then
  echo $SERVER >$SERVADDR_FILE
fi

echo "$SERVER $DOMAINNAME" >/etc/hosts

echo "$LINESPEED: $NOLIMIT" >$TC_CALCFLOW_OUT

openvt -c $TTY_NR_GUI -s startx $TCDIR/tcentry
# ----------------------------------


# re-config the network if RSVD mounted
#-----------------------------------
IFACE=$DEFAULT_IFACE
if [ -f $IFACE_FILE ] ; then
  IFACE=`cat $IFACE_FILE`
fi

if [ "x$IFACE" == "x$WLAN_IFACE" ] ; then
  if [ -f $WLANCFG_FILE ] ; then
    $TCDIR/tc-cfgwifi.sh
  fi
else
  if [ -f $NETCFG_FILE ] ; then
    # carrier_changes will change
    ifconfig $IFACE down
  fi
  ifconfig $IFACE up
fi
#-----------------------------------


# run heart-beat script
#-----------------------------------
echo "1: : " >$STATUS_CONF_FILE
$STATUS_SCRIPT &
#-----------------------------------


# check notify service
# ----------------------------------
while [ true ] ; do
  echo "check notify service..."
  $NOTIFY_APP 3 0 0
  if [ $? -eq 0 ] ; then
    break;
  fi

  echo "wait service online..."
  sleep 1
done
#-----------------------------------


# notify RSVD partition
echo "report rsvd status: $rsvd_err"
$NOTIFY_APP 3 3 $rsvd_err
#-----------------------------------


# mount TC partition
#-----------------------------------
if [ -b $DISKDEV ] ; then
  $NOTIFY_APP 3 1 1
  e2fsck -y $DISKDEV
  ret=$?

  if [ $ret -gt 2 ] ; then
    # failed?
    disk_err=12
  elif [ $ret -eq 2 ] ; then
    # errors corrected, system should reboot
    disk_err=13
  else
    # no errors or corrected
    mount $DISKDEV $DISKDIR
    if [ $? -ne 0 ] ; then
      disk_err=14
    else
      disk_err=0
    fi
  fi
else
  disk_err=11
fi

echo "report disk status: $disk_err"
$NOTIFY_APP 3 1 $disk_err
#-----------------------------------


# main loop
#-----------------------------------
phw_rdy=0
chk_boot=0
chk_fwv=0
try_pbbg=0
try_qrcode=5

ORGSERVER=0.0.0.0

ORGPCNAME=

while [ true ] ; do
  # check network
  #-----------------------------------
  IFACE=$DEFAULT_IFACE
  if [ -f $IFACE_FILE ] ; then
    IFACE=`cat $IFACE_FILE`
  fi

  if [ -f $SERVADDR_FILE ] ; then
    SERVER=`cat $SERVADDR_FILE`
    echo SERVER=$SERVER
  fi

  if [ $SERVER != $ORGSERVER ] ; then
    echo "server IP $ORGSERVER -> $SERVER"
    ORGSERVER=$SERVER

    killall tcctl_daemon.sh
    openvt -c $TTY_NR_TCD $TCDIR/tcctl_daemon.sh $SERVER
  fi

  ifconfig $IFACE >/dev/null 2>&1
  if [ $? -ne 0 ] ; then
    echo "net status: no device"
  else
    ret=`cat /sys/class/net/$IFACE/carrier`
    if [ "x$ret" == "x0" ] ; then
      tcnet_err=1  # line down
      echo "net status: line down"
    else
      MYIP=`ifconfig $IFACE | awk '/inet addr/{print substr($2,6)}'`
      if [ "x$MYIP" == "x" ] ; then
        tcnet_err=2  # no ip
        echo "net status: no ip"
      else
        ping $SERVER -c 1 -W 1
        if [ $? -ne 0 ] ; then
          tcnet_err=3  # server down
          echo "net status: tc server down"
        else
          $TC_QUERY_SERVER $SERVER
          if [ $? -ne 0 ] ; then
            tcnet_err=4
            echo "net status: tc service down"
          else
            tcnet_err=0
            echo "net status: ready"
          fi
        fi
      fi
    fi
  fi

  $NOTIFY_APP 3 2 $tcnet_err
  #-----------------------------------


  # check disk
  #-----------------------------------
  disk_usage=`$GET_DISK_USAGE`
  disk_err=$?
  if [ $disk_err -eq 0 ] ; then
    disk_free=`echo $disk_usage | awk -F ':' '{print $2}'`
    echo "disk_free=$disk_free MB"

    if [ $disk_free -lt $DISK_FULL_THRESHOLD ] ; then
      echo "tc partition full"
      disk_err=2
    else
      echo "tc partition ready"
      disk_err=0
    fi
  fi

  $NOTIFY_APP 3 1 $disk_err
  #-----------------------------------


  # jobs if tcnet is ready
  #-----------------------------------
  if [ $tcnet_err -eq 0 ] ; then
    if [ $phw_rdy -eq 0 ] ; then
      echo "post HW status..."
      $POST_HW_SCRIPT $SERVER
      if [ $? -eq 0 ] ; then
        phw_rdy=1
      fi
    fi

    if [ $try_qrcode -gt 0 ] ; then
      echo "download the qrcode file..."
      $QRCODE_SCRIPT $SERVER
      if [ $? -eq 0 ] ; then
        $NOTIFY_APP 4 $QRCODE_FILE
        try_qrcode=0
      fi
      try_qrcode=$((try_qrcode-1))
    fi
  fi
  #-----------------------------------

  # jobs if disk is ready
  #-----------------------------------
  if [ $disk_err -eq 0 ] ; then
  fi
  #-----------------------------------

  # jobs if tcnet & disk are ready
  #-----------------------------------
  if [ $tcnet_err -eq 0 -a $disk_err -eq 0 ] ; then
    if [ $try_pbbg -eq 0 ] ; then
      echo "download the preboot BG file..."
      wget $WGET_OPTS --tries=3 --timeout=1 --retry-connrefused -N $HTTP_PROT://$DOMAINNAME/$PBBG_URL -O $PBBG_FILE || rm -f $PBBG_FILE
      if [ $? -eq 0 ] ; then
        try_pbbg=1
      fi
    fi

    $GET_CLIENT_ENTRY_SCRIPT $SERVER
    tmpret=$?
    if [ $tmpret -eq 2 ] ; then
      echo "machine not registered"
      autocname=`cat $TC_NEW_CLIENT_NAME_FILE`
      if [ -z "$autocname" ] ; then
        $NOTIFY_APP 7
      else
        $NOTIFY_APP 8 $autocname
      fi
      $POST_HW_SCRIPT $SERVER
    elif [ $tmpret -eq 0 ] ; then
      PCNAME=`cat $MACHINE_NAME_FILE`
      if [ x$PCNAME != x$ORGPCNAME ] ; then
        echo "pc name: $ORGPCNAME -> $PCNAME"
        ORGPCNAME=$PCNAME
        $NOTIFY_APP 9 $PCNAME
      fi

      # jobs if machine is registered
      #-----------------------------------
      if [ $chk_fwv -eq 0 ] ; then
        $TCDIR/chk_fwver.sh $SERVER
        chk_fwv=1
      fi

      if [ $chk_boot -eq 0 ] ; then
        $TCDIR/chk_autoboot.sh $SERVER
        chk_boot=1
      fi
      #-----------------------------------
    fi
  fi

  sleep 5
done
#-----------------------------------


exit 0
