#!/bin/sh
# query-client.sh
# 
# query if a client has already been registered into the TC system.
#
# Ret Value:
# 0 if this client is Not in the server DB;
# 2 if this client is in the server DB;
# value greater than 2 on other conditions.
ERR_QC_UNREGISTERED=0
ERR_QC_REGISTERED=2
ERR_OTHER=11
ERR_PARA=12
ERR_CURL=13
ERR_HWADDR=14

SCRIPT_NAME=$0
. /usr/sbin/tc/global_def.sh

SCRIPT_GET_HWADDR=$TCDIR/get-hwaddr.sh
NIF=$DEFAULT_IFACE
if [ -f $IFACE_FILE ] ; then
  NIF=`cat $IFACE_FILE`
fi
SEPARATOR="-"

if [ $# -eq 0 ] ; then
    serverip=${SERVER}
    if ! [ -b ${DISKRSVD} ] ; then
        echo "${DISKRSVD}: no such block node"
        exit $ERR_OTHER
    fi

    mount | grep -q -E "${DISKRSVD}.*${RSVDDIR}"
    if [ $? -eq 0 ] ; then
        mounted=YES
        mount_action=NO
    else
        mount_action=YES

        TRY_MAX=5
        try_count=0
        until [ $try_count -ge $TRY_MAX ] || mount | grep -q -E "${DISKRSVD}.*${RSVDDIR}" ; do
            mount ${DISKRSVD} ${RSVDDIR}
            try_count=$(($try_count + 1))
            sleep 1
        done

        mount | grep -q -E "${DISKRSVD}.*${RSVDDIR}"
        if [ $? -eq 0 ] ; then
            mounted=YES
        else
            mounted=NO
        fi
    fi

    if [ "x${mounted}" == "xYES" -a -f ${SERVADDR_FILE} ] ; then
        serverip=`cat ${SERVADDR_FILE}`
    fi

    if [ "x${mounted}" == "xYES" -a "x${mount_action}" == "xYES" ] ; then
        umount ${RSVDDIR}
    fi
elif [ $# -eq 1 ] ; then
    serverip=$1
else
    cat <<EOT
SYNOPSIS
    $SCRIPT_NAME [ServerIP]

DESCRIPTION
    Query if a client is in DB resided on server <ServerIP>

EXAMPLE
    $SCRIPT_NAME 192.168.0.1
EOT
    exit $ERR_PARA
fi

sh $SCRIPT_GET_HWADDR $NIF $SEPARATOR >/dev/null 2>&1
if [ $? -ne 0 ] ; then
    exit $ERR_HWADDR
fi

HWADDR=`sh $SCRIPT_GET_HWADDR $NIF $SEPARATOR`
URL_QC="${HTTP_PROT}://$DOMAINNAME/tc/rest/client.php"
JSON_QC_TMP=/tmp/query-client.json
URL_DST=

if [ -f $JSON_QC_TMP ] ; then
    rm -f $JSON_QC_TMP
fi

curl $CURL_OPTS -s -X POST --retry 3 --retry-delay 2 --output $JSON_QC_TMP --data "{\"mac\":\"${HWADDR}\"}" --header "Content-Type: application/json" $URL_QC

if [ $? -eq 0 ] ; then
    if [ -f $JSON_QC_TMP ] ; then
        t=`jq -r '.success' $JSON_QC_TMP`
        if ! [ $? -eq 0 ] ; then
            exit $ERR_OTHER
        fi

        if [ "x$t" == "xtrue" ] ; then
            # for registered client
            exit $ERR_QC_REGISTERED
        elif [ "x$t" == "xfalse" ] ; then
            # for unregistered client
            exit $ERR_QC_UNREGISTERED
        else
            # for non-available registeration service
            exit $ERR_OTHER
        fi
    else
        exit $ERR_OTHER
    fi
else
    exit $ERR_CURL
fi


