#!/bin/sh
# query-reg-service.sh
# 
# query whether reg service is ON or OFF
#
# Ret Value:
# 0 if reg service is ON
# 1 if reg service is OFF
# value greater than 1 on other conditions.
ERR_SERVICE_ON=0
ERR_SERVICE_OFF=1
ERR_OTHER=11
ERR_PARA=12
ERR_CURL=13

SCRIPT_NAME=$0
. /usr/sbin/tc/global_def.sh

SCRIPT_GET_HWADDR=$TCDIR/get-hwaddr.sh
NIF=$DEFAULT_IFACE
if [ -f $IFACE_FILE ] ; then
  NIF=`cat $IFACE_FILE`
fi
SEPARATOR="-"

if [ $# -eq 0 ] ; then
    if [ -f ${SERVADDR_FILE} ] ; then
        serverip=`cat ${SERVADDR_FILE}`
    else
        serverip=${SERVER}
    fi
elif [ $# -eq 1 ] ; then
    serverip=$1
else
    cat <<EOT
SYNOPSIS
    $SCRIPT_NAME [ServerIP]

DESCRIPTION
    Query whether registration service resided in <ServerIP> is ON or OFF

EXAMPLE
    $SCRIPT_NAME 192.168.0.1
EOT
    exit $ERR_PARA
fi

URL_QES=$HTTP_PROT://$DOMAINNAME/tc/rest/system.php/settings
JSON_QES_TMP=/tmp/query-reg-service.json
URL_DST=

if [ -f $JSON_QES_TMP ] ; then
    rm -f $JSON_QES_TMP
fi

curl $CURL_OPTS -s -X GET --retry 3 --retry-delay 2 --output $JSON_QES_TMP $URL_QES

if [ $? -eq 0 ] ; then
    if [ -f $JSON_QES_TMP ] ; then
        t=`jq -r '.result.client_open_registration' $JSON_QES_TMP`
        if ! [ $? -eq 0 ] ; then
            exit $ERR_OTHER
        fi

        #if [ "x$t" == "xtrue" ] ; then
        if [ "x$t" == "xtrue" -o "x$t" == "x1" ] ; then
            # ON
            exit $ERR_SERVICE_ON
        #elif [ "x$t" == "xfalse" ] ; then
        elif [ "x$t" == "xfalse" -o -z "$t" ] ; then
            # OFF
            exit $ERR_SERVICE_OFF
        else
            # unknown
            exit $ERR_OTHER
        fi
    else
        exit $ERR_OTHER
    fi
else
    exit $ERR_CURL
fi

