#!/bin/sh

. /usr/sbin/tc/global_def.sh

if [ -z "$1" -o -z "$2" -o -z "$3" -o -z "$4" ] ; then
  echo "Usage: sync_data.sh <curimg> <username> <osid> <orgimg>"
  exit 1
fi

if [ -n "$5" ] ; then
  SERVER=$5
  echo SERVER=$5
fi

LOG_FILE=/tmp/psmudf-dbg.log

UID=`cat $UID_FILE`
echo "user id: $UID"

UCURPER=0
CURPER=0
PERFILE=$TCDIR/$3.per
if [ -f $PERFILE ] ; then
  CURPER=`cat $PERFILE`
fi
if [ ${#CURPER} -eq 2 ] ; then
  UCURPER=${CURPER:0:1}
fi
if [ "x$2" == "xroot" ] ; then
  UCURPER=0
fi
echo UCURPER=$UCURPER
if [ $UCURPER -ne 1 ] ; then
  echo permission denied
  exit 2
fi


DATFILE=data-$2.img
MDFFILE=$DATFILE-$2.mdf
UDFFILE=$DATFILE-$2.udf
CDFFILE=$DATFILE-$2.cdf

VERFILE=$DISKDIR/$DATFILE.ver
if [ ! -f $VERFILE ] ; then
  echo "no version file"
  exit 41
fi
CURVER=`cat $VERFILE`
echo "data's ver=$CURVER"

if [ ! -f $DISKDIR/$MDFFILE -o ! -f $DISKDIR/$UDFFILE ] ; then
  echo "no mdf/udf files"
  exit 42
fi


curl $CURL_OPTS -X GET --retry 3 --retry-delay 2 --output $PDATA_JSON $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID
if [ $? -ne 0 ] ; then
  echo "communicate with server timeout"
  exit 3
fi
jret=`jq -r '.success' $PDATA_JSON`
if [ "$jret"x != "true"x ] ; then
  echo "query user data info failed"
  exit 51
fi

pfrozen=`jq -r '.result[0].storage_frozen' $PDATA_JSON`

if [ "x$pfrozen" == "xtrue" ] ; then
  echo "user data is frozen"
  exit 43
fi


#$TCDCLT_APP $SERVER eth0 12345 "7:$2:$3"
echo "7:$2:$3" >$STATUS_CONF_FILE

ret=0
while [ true ] ; do  # break is only meaningful in loop
  $AUTH_APP -i $SERVER -p $SERVERPORT -v $3:$CURVER -f /dev/null -t $TOKEN_FILE
  ret=$?
  if [ $ret -ne 0 ] ; then
    echo "check failed: server refused to accept the data"
    ret=44
    break
  fi

  mac=`$GET_HWADDR_SCRIPT $DEFAULT_IFACE -`
  curl $CURL_OPTS -X POST --output $PDATA_JSON -H "Content-Type: application/json" -d '{"status":"lock","token":"'$mac'"}' $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID
  if [ $? -ne 0 ] ; then
    echo "communicate with server timeout"
    exit 3
  fi
  jret=`jq -r '.success' $PDATA_JSON`
  if [ "$jret"x != "true"x ] ; then
    echo "use $mac to lock PS failed"
    exit 45
  fi
  echo "use $mac to lock PS successfully"

  echo "<$UPLOADUSRDATA>"
  if [ -f $DISKDIR/$CDFFILE ] ; then
    echo "use $CDFFILE to upload $UDFFILE"
    $UPLOAD_MUDF_APP $SERVER $MDFFILE $UDFFILE $DISKDIR/$CDFFILE $DISKDIR/$UDFFILE 1 1 2>$LOG_FILE
  else
    echo "use $MDFFILE to upload $UDFFILE"
    $UPLOAD_MUDF_APP $SERVER $MDFFILE $UDFFILE $DISKDIR/$MDFFILE $DISKDIR/$UDFFILE 1 0 2>$LOG_FILE
  fi
  if [ $ret -ne 0 ] ; then
    echo "upload ps data failed"
    ret=46
    break
  fi

  echo "auto merge mdf/udf"
  curl $CURL_OPTS -X POST --output $PDATA_JSON -H "Content-Type: application/json" -d '{"uri":"head","img":"'$DATFILE'","udf":"'$UDFFILE'","mdf":"'$MDFFILE'","revision":"'$CURVER'"}' ''$HTTP_PROT'://'$DOMAINNAME'/'$MERGEDAT_URL'/'$UID''
  if [ $? -ne 0 ] ; then
    echo "communicate with server timeout"
    ret=3
    break
  fi
  jret=`jq -r '.success' $PDATA_JSON`
  if [ "$jret"x != "true"x ] ; then
    echo "auto merge mdf/udf failed"
    ret=47
    break
  fi

  rm -f $DISKDIR/$CDFFILE
  curl $CURL_OPTS -X POST --output $PDATA_JSON -H "Content-Type: application/json" -d '{"status":"unlock","token":"'$mac'"}' $HTTP_PROT://$DOMAINNAME/$PDATA_URL/$UID

  $AUTH_APP -i $SERVER -p $SERVERPORT -d -f /dev/null -t $TOKEN_FILE

  break
done

#$TCDCLT_APP $SERVER eth0 12345 "1:$2:$3"
echo "1:$2:$3" >$STATUS_CONF_FILE

#echo "sleeping for debuging"
#sleep 30

exit $ret

