#!/bin/sh

. /usr/sbin/tc/global_def.sh

if [ -z "$1" -o -z "$2" -o -z "$3" -o -z "$4" ] ; then
  echo "Usage: sync_img.sh <curimg> <username> <osid> <orgimg>"
  exit 1
fi

if [ -n "$5" ] ; then
  SERVER=$5
  echo SERVER=$5
fi

LOG_FILE=/tmp/osmudf-dbg.log

IDFILE=$DISKDIR/$4.id
CURID=`cat $IDFILE`
echo "$4: id_c=$CURID, id_s=$3"
if [ "x$CURID" != "x$3" ] ; then
  echo "OS img id not match"
  exit 63
fi

UID=`cat $UID_FILE`
echo "user id: $UID"

CURPER=0
PERFILE=$TCDIR/$3.per
if [ -f $PERFILE ] ; then
  CURPER=`cat $PERFILE`
fi
if [ ${#CURPER} -eq 2 ] ; then
  CURPER=${CURPER:1:1}
fi
echo CURPER=$CURPER
if [ $CURPER -lt 2 ] ; then
  echo permission denied
  exit 2
fi


CURVER=0
VERFILE=$DISKDIR/$4.ver
if [ -f $VERFILE ] ; then
  CURVER=`cat $VERFILE`
fi
echo CURVER=$CURVER

#$TCDCLT_APP $SERVER eth0 12345 "7:$2:$3"
echo "7:$2:$3" >$STATUS_CONF_FILE

ret=0
while [ $CURVER -gt 0 ] ; do  # break is only meaningful in loop
  $AUTH_APP -i $SERVER -p $SERVERPORT -c $3:$CURVER -f /dev/null -t $TOKEN_FILE
  if [ $? -ne 0 ] ; then
    echo "check failed: server refused to accept the cache"
    ret=31
    break
  fi

  wget $WGET_OPTS --tries=3 --timeout=1 --retry-connrefused $HTTP_PROT://$DOMAINNAME/$MUDFVER_URL/$3?uid=$UID -O $MUDFVER_JSON
  if [ $? -ne 0 ] ; then
    echo "communicate with server timeout"
    ret=3
    break
  fi
  jret=`jq -r '.success' $MUDFVER_JSON`
  if [ "$jret"x != "true"x ] ; then
    echo "get mdf/udf version failed"
    ret=32
    break
  fi

  mudf_sver=`jq -r '.result.private_revision' $MUDFVER_JSON`
  mudf_cver=`cat $DISKDIR/$4-$2.ver`
  echo "mudf version: s=$mudf_sver, c=$mudf_cver"
  if [ $mudf_cver -ne $mudf_sver ] ; then
    echo "mdf/udf version not match: s=$mudf_sver, c=$mudf_cver"
    ret=33
    break
  fi

  DFNAME="$3"_"$CURVER"_"$2"
  echo "sync ($4-$2.mdf/udf -> $DFNAME.mdf/udf)..."
  echo "<$UPLOADSYSDATA>"
  $UPLOAD_MUDF_APP $SERVER $DFNAME.mdf $DFNAME.udf $DISKDIR/$4-$2.mdf $DISKDIR/$4-$2.udf 2 0 2>$LOG_FILE
  if [ $? -ne 0 ] ; then
    echo "upload mdf/udf files failed"
    ret=34
    break
  fi

  mudf_cver=$((mudf_cver+1))
  echo "update mdf/udf version: $mudf_sver -> $mudf_cver"
  curl $CURL_OPTS -X POST --output $MUDFVER_JSON -H "Content-Type: application/json" -d '{"uid":"'$UID'","private_revision":"'$mudf_cver'","desc":"upload"}' ''$HTTP_PROT'://'$DOMAINNAME'/'$MUDFVER_URL'/'$3''
  if [ $? -ne 0 ] ; then
    echo "communicate with server timeout"
    ret=3
    break
  fi
  jret=`jq -r '.success' $MUDFVER_JSON`
  if [ "$jret"x != "true"x ] ; then
    echo "update mdf/udf version failed"
    ret=35
    break
  fi

  echo $mudf_cver >$DISKDIR/$4-$2.ver

  $AUTH_APP -i $SERVER -p $SERVERPORT -u $3:$CURVER -f /dev/null -t $TOKEN_FILE

  $AUTH_APP -i $SERVER -p $SERVERPORT -d -f /dev/null -t $TOKEN_FILE

  break
done

#$TCDCLT_APP $SERVER eth0 12345 "1:$2:$3"
echo "1:$2:$3" >$STATUS_CONF_FILE

#echo "sleeping for debuging"
#sleep 30

exit $ret

