#!/bin/sh

. /usr/sbin/tc/global_def.sh

VFACE=ifb0

if [ -z "$1" -o -z "$2" ] ; then
  echo "Usage: tc-cfgflow.sh outrate(KB/s) inrate(KB/s)"
  exit 1
fi

IFACE=$DEFAULT_IFACE
if [ -f $IFACE_FILE ] ; then
  IFACE=`cat $IFACE_FILE`
fi

MYIP=`ifconfig $IFACE | awk '/inet addr/{print substr($2,6)}'`

OUTRATE="$1"kbps
INRATE="$2"kbps
OUTR2Q=$(($1/25))
INR2Q=$(($2/25))

echo "tc on $IFACE($MYIP), out=$OUTRATE(r2q=$OUTR2Q), in=$INRATE(r2q=$INR2Q)"


tc qdisc del dev $IFACE root

if [ $1 -ne 0 ] ; then
  echo "limit the upload rate..."
  tc qdisc add dev $IFACE root handle 1: htb default 10 r2q $OUTR2Q

  tc class add dev $IFACE parent 1: classid 1:1 htb rate $OUTRATE ceil $OUTRATE

  tc class add dev $IFACE parent 1:1 classid 1:10 htb rate $OUTRATE ceil $OUTRATE
  tc qdisc add dev $IFACE parent 1:10 handle 10: sfq perturb 10

  tc filter add dev $IFACE parent 1: protocol ip u32 match ip src $MYIP flowid 1:10
fi


tc qdisc del dev $IFACE handle ffff: ingress
tc qdisc del dev $VFACE root

if [ $2 -ne 0 ] ; then
  echo "limit the download rate..."

  ip link set dev $VFACE up

  tc qdisc add dev $IFACE handle ffff: ingress
  tc filter add dev $IFACE parent ffff: protocol ip u32 match u32 0 0 action mirred egress redirect dev $VFACE

  tc qdisc add dev $VFACE root handle 2: htb default 20 r2q $INR2Q

  tc class add dev $VFACE parent 2: classid 2:1 htb rate $INRATE ceil $INRATE

  tc class add dev $VFACE parent 2:1 classid 2:10 htb rate $INRATE ceil $INRATE
  tc qdisc add dev $VFACE parent 2:10 handle 10: sfq perturb 10

  tc class add dev $VFACE parent 2:1 classid 2:20 htb rate $INRATE ceil $INRATE
  tc qdisc add dev $VFACE parent 2:20 handle 20: sfq perturb 10

  tc filter add dev $VFACE parent 2: protocol ip prio 1 u32 match ip sport 4620 0xffff flowid 2:10
  tc filter add dev $VFACE parent 2: protocol ip prio 1 u32 match ip dst $MYIP flowid 2:20
fi

killall tc-calcflow.sh
rm -f $TC_CALCFLOW_OUT
if [ $1 -ne 0 -o $2 -ne 0 ] ; then
  $TCDIR/tc-calcflow.sh &
fi

