#!/bin/sh

. /usr/sbin/tc/global_def.sh

exec >>/var/log/tc-cfgnet.log
exec 2>&1

if [ -n "$1" ] ; then
  IFACE=$1
elif [ -f $IFACE_FILE ] ; then
  IFACE=`cat $IFACE_FILE`
else
  IFACE=$DEFAULT_IFACE
fi
echo " $0 IFACE=$IFACE, CHGNET=$CHGNET"

RESOLV_CONF="/etc/resolv.conf"
LOG_FILE=/var/log/udhcpc.log

killall udhcpc

SKIPIFS=lo:ifb0:tap0:br0:$IFACE
for iface in $(ls -1 /sys/class/net) ; do
  if [ -z "$iface" ] ; then
    continue
  fi
  echo $SKIPIFS | grep $iface
  if [ $? -ne 0 ] ; then
    echo "down $iface"
    ifconfig $iface down
  else
    echo "skip $iface"
  fi
done

while route del default gw 0.0.0.0 dev $IFACE ; do
done

echo -n >$RESOLV_CONF


cnt=0
tries=5
until [ $cnt -ge $tries ] ; do
  if [ -d /sys/class/net/$IFACE ] ; then
    break;
  fi

  echo "wait 1s for $IFACE"
  sleep 1
  cnt=$((cnt+1))
done
if [ $cnt -eq $tries ] ; then
  echo "$IFACE not exit"
  exit 3
fi


dhcpoff=0

if [ -f $NETCFG_FILE ] ; then
  echo "$NETCFG_FILE exist"
  argv=`awk -F ':' '{printf("dhcpoff=%s;ip=%s;mask=%s;gw=%s;dns1=%s;dns2=%s;",$1,$2,$3,$4,$5,$6)}' $NETCFG_FILE`
  eval $argv
fi


if [ "x$CHGNET" == "x1" ] ; then
  OSNET_CONF=$DISKDIR/tcnet.conf
  mac=`$GET_HWADDR_SCRIPT $IFACE -`
  if [ $dhcpoff -eq 0 ] ; then
    echo "HWADDR = $mac" >$OSNET_CONF
    echo "BOOTPROTO = dynamic" >>$OSNET_CONF
  else
    echo "HWADDR = $mac" >$OSNET_CONF
    echo "BOOTPROTO = static" >>$OSNET_CONF
    echo "IPADDR = $ip" >>$OSNET_CONF
    echo "NETMASK = $mask" >>$OSNET_CONF
    echo "GATEWAY = $gw" >>$OSNET_CONF
    echo "DNS = $dns1" >>$OSNET_CONF
    echo "DNS2 = $dns2" >>$OSNET_CONF
  fi
fi


if [ $dhcpoff -eq 0 ] ; then
  echo "use dhcpc on $IFACE"
  udhcpc -i $IFACE -f >>$LOG_FILE 2>&1 &
  exit 0
fi


echo "set $IFACE: $ip/$mask"
ifconfig $IFACE $ip netmask $mask
if [ $? -ne 0 ] ; then
  echo "ifconfig failed"
  exit 4
fi

echo "set gateway: $gw"
route add default gw $gw dev $IFACE
if [ $? -ne 0 ] ; then
  echo "route add failed"
  exit 5
fi

if [ -n "$dns1" ] ; then
  echo "add dns: $dns1"
  echo nameserver $dns1 >>$RESOLV_CONF
fi

if [ -n "$dns2" ] ; then
  echo "add dns: $dns2"
  echo nameserver $dns2 >>$RESOLV_CONF
fi

exit 0
