#!/bin/sh
# tc-query-server.sh
#
# query whether TC server is ON or OFF
#
# Ret Value:
# 0 if reg service is ON
# a non-zero value if reg service is OFF
# value greater than 1 on other conditions.
ERR_SERVER_ON=0
ERR_SERVER_OFF=1
ERR_OTHER=11
ERR_PARA=12
ERR_CURL=13

SCRIPT_NAME=$0
. /usr/sbin/tc/global_def.sh

if [ $# -ne 1 ] ; then
    cat <<EOT
SYNOPSIS
    $SCRIPT_NAME <ServerIP>

DESCRIPTION
    Query whether TC server with the given <ServerIP> is ON or OFF

EXAMPLE
    $SCRIPT_NAME 192.168.0.1
EOT
    exit $ERR_PARA
else
#    serverip=$SERVER
    serverip=$1
fi

URL="$HTTP_PROT://${DOMAINNAME}/${TC_QUERY_SERVER_URL}"
JSON_TMP=/tmp/tc-query-server.json

if [ -f $JSON_TMP ] ; then
    rm -f $JSON_TMP
fi

curl $CURL_OPTS -s -X GET --retry 3 --retry-delay 2 --output $JSON_TMP $URL

if [ $? -eq 0 ] ; then
    if [ -f $JSON_TMP ] ; then
        t=`jq -r '.result.disk.status' $JSON_TMP`
        if ! [ $? -eq 0 ] ; then
            exit $ERR_OTHER
        fi

        if [ "x$t" == "xon" ] ; then
            # ON
            exit $ERR_SERVER_ON
        elif [ "x$t" == "xoff" ] ; then
            # OFF
            exit $ERR_SERVER_OFF
        else
            # unknown
            exit $ERR_OTHER
        fi
    else
        exit $ERR_OTHER
    fi
else
    exit $ERR_CURL
fi

