#!/bin/sh
# enable network by DHCP or manually set ip address,
# test network connection is available or not at last.
source import_debug.sh
LOG=/var/log/tcsh.log
SH_PATH=$0
SH_NAME=${SH_PATH#**/}

if [ "x$HOST_DEBUG" == "x" ] ; then
    ROOTDIR=''
else
    ROOTDIR='/home/bright/tc/tc-project/ramdisk/imgexport_diff'
fi


exec 5>&1 6>&2
exec 1>>$LOG 2>&1
echo "$SH_NAME:"

help_msg (){
    echo "Wrong parameters format"
    echo "Usage:"
    echo "$0 <ServerIPv4> [<LocalIPv4> [<LocalNetmask>]]"
    echo "for example: $0 192.168.0.1"
    echo "             $0 192.168.0.1 192.168.0.100"
    echo "             $0 192.168.0.1 192.168.0.100 255.255.255.0"
}



if [ $# -eq 1 ] ; then
    SIP=$1
elif [ $# -eq 2 ] ; then
    SIP=$1
    IP=$2
    NETMASK='255.255.255.0'
elif [ $# -eq 3 ] ; then
    SIP=$1
    IP=$2
    NETMASK=$3
else
    help_msg
    exit 1
fi



# check ethernet interface
echo "Try to detect availabe ethernet card..."
linenum=$(cat /proc/net/dev | wc -l) 
nets=$(cat /proc/net/dev | tail -n $(expr $linenum - 2)| awk -F: '{print $1}')
for net in $nets ; do
# by default, we use the first NIC with name starting with "eth"
# if no ethernet cards the name of which begin with 'eth', then report no ethernet cards are detected.
    if [ "xlo" != "x$net" ] ; then
	phynets="$net $phynets"
#        IFNAME=$net
#        break
    fi
done

# check the number of phynets
nPhynets=0
for pn in $phynets ; do
	nPhynets=$(expr $nPhynets + 1)
done
if [ $nPhynets -ge 1 ] ; then
	for pn in $phynets ; do
		if echo $pn | grep -q "^eth" ; then
			IFNAME=$pn
			break
		fi
	done
fi

if [ "x" == "x${IFNAME}" ] ; then
    echo "No ethernet card detected."
    exit 2
fi

echo "Try to bring up ${IFNAME}..."
ifconfig $IFNAME up
if [ $? -ne 0 ] ; then
    echo "Failed to bring up ${IFNAME}"
    exit 3
fi

echo "Try to check if ${IFNAME} is alive?"
ifconfig | grep -q "^${IFNAME}"
if [ $? -ne 0 ] ; then
    echo "Network interface ${IFNAME} is not alive."
    exit 4
fi



###############  DHCP ##################
if [ $# -eq 1 ] ; then
    echo "Try to get IP from DHCP server..."
    udhcpc -t 5 -n -i $IFNAME

    if [ $? -ne 0 ] ; then
        echo "Unable to get an IP address."
        exit 5
    fi

    echo "Try to get local IP address..."
    ifconfig $IFNAME | grep -q 'inet addr'
    if [ $? -ne 0 ] ; then
        echo "Unable to get local IP address."
        exit 6
    else
        curIP=$(ifconfig $IFNAME | grep 'inet addr' | awk '{print $2}' | awk -F: '{print $2}')
    fi
fi # end of [ $# -eq 1 ]
############### end DHCP  ##################


############### Manual IP  ##################
if [ $# -eq 2 -o $# -eq 3 ] ; then
    echo "Try to manually set IP addresss..."
    ifconfig $IFNAME $IP netmask $NETMASK

    if [ $? -ne 0 ] ; then
        echo "Failed to set ip for ${IFNAME} manually."
        exit 7
    else
        curIP=$(ifconfig $IFNAME | grep 'inet addr' | awk '{print $2}' | awk -F: '{print $2}')
    fi
fi # end of [ $# -eq 3 ]
########### end Manual IP  ##################



echo "Try to ping server $SIP"
ping -c 2 -I ${IFNAME} $SIP
if [ $? -ne 0 ] ; then
    echo "Client IP: $curIP, failed to ping server $SIP"
    exit 8
fi

exec 1>&5 2>&6

exit 0

