#!/bin/sh

source import_debug.sh

if [ "x$HOST_DEBUG" == "x" ] ; then
    ROOTDIR=''
else
    ROOTDIR='/home/bright/tc/tc-project/ramdisk/imgexport_diff'
fi

LOG=${ROOTDIR}'/var/log/tcsh.log'
SH_PATH=$0
SH_NAME=${SH_PATH#**/}

exec 5>&1 6>&2
exec 1>>$LOG 2>&1
echo "$SH_NAME:"

help_msg (){
    echo "Usage:"
    echo "    $SH_NAME <ServerIP>"
    echo "    for example: $SH_NAME 192.168.0.1"
}

if [ $# -ne 1 ] ; then
    help_msg
    exit 1
else
    SIP=$1
fi

# SIP='192.168.0.1'
SHARENAME='tcimage'
AUTH_CONF='/etc/samba/smb-tc-auth.conf'
REMOTEDIR="//${SIP}/${SHARENAME}"
MOUNTDIR=${ROOTDIR}'/mnt/usb'
CFG_DIR=${ROOTDIR}'/etc/tc.conf.d'




echo "Try to communicate with samba service $REMOTEDIR"
smbclient $REMOTEDIR -c "ls" -A ${AUTH_CONF}
if [ $? -ne 0 ] ; then
    echo "Samba service $REMOTEDIR is not available"
    exit 2
fi

echo "Try to mount samba service $REMOTEDIR"
mount.cifs $REMOTEDIR $MOUNTDIR -o credentials=${AUTH_CONF}
#if [ $? -ne 0 ] ; then
#    echo "Failed to mount samba service $REMOTEDIR" 
#    exit 3
#fi

echo "Try to detect if mounted?"
df | grep -q "${SIP}" 
if [ $? -ne 0 ] ; then
    echo "It is NOT mounted currently."
    exit 4
else
    echo "It is mounted currently."
fi

echo "test if we have write access to samba service $REMOTEDIR"
echo "hello" > $MOUNTDIR/hello.msg
if [ $? -ne 0 ] ; then
    echo "No write permission to samba service $REMOTEDIR"
    umount $MOUNTDIR
    exit 5
else
    rm -f $MOUNTDIR/hello.msg
fi

exit 0

exec 1>&5 2>&6
