#!/bin/sh
# clone disks.
# RET VAL: normally return 0;
# otherwise, return a non-zero value.
source import_debug.sh
if [ "x$HOST_DEBUG" == "x" ] ; then
    ROOTDIR=''
else
    ROOTDIR='/home/bright/tc/tc-project/ramdisk/imgexport_diff'
fi

LOG=${ROOTDIR}'/var/log/tcsh.log'
SH_PATH=$0
SH_NAME=${SH_PATH##*/}

exec 5>&1 6>&2
exec 1>>$LOG 2>&1
echo "$SH_NAME:"

help_msg (){
    echo "Usage:"
    echo "    $SH_NAME <ServerIP> <Disk> <LBABegin> <LBAEnd> <FILE>"
    echo "    for example: $SH_NAME 192.168.0.1 /dev/sda 0 20480 win7.img"
}

if [ $# -ne 5 ] ; then
    help_msg
    exit 1
else
    SIP=$1
    LOCAL_DISK=$2
    LBA_BEGIN=$3
    LBA_END=$4
    FILENAME=$5
fi

SHARENAME='tcimage'
USER='tcuser'
REMOTEDIR="//${SIP}:/${SHARENAME}"
MOUNTDIR=${ROOTDIR}'/mnt/usb'
CFG_DIR=${ROOTDIR}'/etc/tc.conf.d'



rm -f $MOUNTDIR/$FILENAME
lba_count=$(($LBA_END + 1 - $LBA_BEGIN))

echo "Try to export disk image to ${MOUNTDIR}..."
ret=1
if [ $LBA_BEGIN -eq 0 ] ; then
  FSIZE=$(($lba_count * 512))
  BSIZE=4096
  CFILE=/tmp/clone.cache
  /usr/local/bin/script/create_cache.sh $LOCAL_DISK $FSIZE $CFILE $BSIZE
  if [ $? -eq 0 ] ; then
    /usr/sbin/tc/FastCopy $LOCAL_DISK $MOUNTDIR/$FILENAME $FSIZE $CFILE $BSIZE
    ret=$?
  fi
fi
if [ $ret -ne 0 ] ; then
  dd if=$LOCAL_DISK of=$MOUNTDIR/$FILENAME skip=$LBA_BEGIN bs=512 count=$lba_count
fi

if [ $? -ne 0 ] ; then
    echo "Error encounted during creating disk images."
    retval=2
    exit $retval
fi


echo "Try to sync disk image file ${MOUNTDIR}/${FILENAME}..."
sync

if [ $? -ne 0 ] ; then
    echo "Error encounted during syncing disk images."
    retval=3
    exit $retval
fi

echo "Try to check file size..."
#outsize=$(ls -l $MOUNTDIR/$FILENAME | awk '{print $5}')
outsize=$(ls -l $MOUNTDIR/$FILENAME | awk '{print $3}')
#outsectors=$(expr $outsize / 512)
outsectors=$(echo "$outsize / 512" | bc)

if [ "x$lba_count" == "x${outsectors}" ] ; then
    retval=0
else
    echo "Filesize not matched!"
    retval=4
    exit $retval
fi

umount -f $MOUNTDIR
if [ $? -ne 0 ] ; then
    echo "Failed to umount samba service"
    retval=5
    exit $retval
fi

exit $retval

exec 1>&5 2>&6

