#!/bin/sh

alias cp='/bin/cp -f'
alias mv='/bin/mv -f'
alias rm='/bin/rm -f'

TOPDIR=$(pwd)
BASE=$TOPDIR/base
BIN=$TOPDIR/bin
WORKDIR=$TOPDIR/tmpdir
DLDIR=$TOPDIR/3rdParty
MEEGOTOUCH_THEME_PKG=$DLDIR/meegotouchtheme_0.21.14-1+0m6.tar.gz
MEEGOTOUCH_DIR_NAME='meegotouchtheme-0.21.14'

# use firmware instaed of bzImage & rootfs.cpio.gz
FW_VERSION_MAJOR=${TC_MAJOR}
FW_VERSION_MINOR=${TC_MINOR}

if [ -z "$FW_VERSION_MAJOR" ] ; then
    FW_VERSION_MAJOR="major"
fi

if [ -z "$FW_VERSION_MINOR" ] ; then
    FW_VERSION_MINOR="minor"
fi


rm -r $WORKDIR

cp -a $BASE $WORKDIR

if [ -f $MEEGOTOUCH_THEME_PKG ] ; then
    tar zxf $MEEGOTOUCH_THEME_PKG -C $WORKDIR
fi

cd $WORKDIR

if [ -d $MEEGOTOUCH_DIR_NAME ] ; then
    echo "update meegotouch-themes..."
    cp -a $MEEGOTOUCH_DIR_NAME/base/* mtf_diff/usr/share/themes/base/
    #rm mtf_diff/usr/share/themes/base/meegotouch/svg/meegotouch-navigationbar.svg
    rm -r $MEEGOTOUCH_DIR_NAME
    #
    # update hotfixNB243914.svg
    #
    sed -i '/meegotouch-button-checkbox-background-selected-disabled/,+2d' mtf_diff/usr/share/themes/base/meegotouch/svg/hotfixNB243914.svg
    sed -i '/meegotouch-button-checkbox-background-selected/,+2d' mtf_diff/usr/share/themes/base/meegotouch/svg/hotfixNB243914.svg
    sed -i '/meegotouch-button-checkbox-background-disabled/,+2d' mtf_diff/usr/share/themes/base/meegotouch/svg/hotfixNB243914.svg
else
    echo "missing $MEEGOTOUCH_THEME_PKG"
    echo "you may get a rootfs that does NOT work."
fi

echo "update binaries..."
cp -a $BIN/usr/local/bin/* $WORKDIR/deploy_diff/usr/local/bin/
cp -a $BIN/usr/sbin/tc/* $WORKDIR/tc_diff/usr/sbin/tc/
cp -a $BIN/usr/share/l10n/meegotouch/* $WORKDIR/mtf_diff/usr/share/l10n/meegotouch/

if [ "x$SDL_BUILD_ENABLE" == "x1" ] ; then
  echo "write build options to ramdisk"
  cat >$WORKDIR/tc_diff/usr/sbin/tc/bldopt.sh <<EOT
#!/bin/sh
# @file
export SDL_BUILD_ENABLE=1
EOT
  chmod +x $WORKDIR/tc_diff/usr/sbin/tc/bldopt.sh
fi

echo "create rootfs..."
export FW_VERSION_MAJOR
export FW_VERSION_MINOR
sh ./mkiso
sh ./mk-initializer.sh

cat > rootfs_log <<EOT
${FW_VERSION_MAJOR}.${FW_VERSION_MINOR}
ramdisk image for TC clients
EOT

unset FW_VERSION_MAJOR
unset FW_VERSION_MINOR

tar cf tc_firmware rootfs_log rootfs.cpio.gz
gpg2 -c --batch --no-tty --cipher-algo AES256 --symmetric --homedir "$home/var" --passphrase "tc-firmware" -o "tc_firmware.tmp" "tc_firmware"
mv "tc_firmware.tmp" "tc_firmware"

rm rootfs.cpio.gz rootfs_log

mv tc_firmware tcboot.iso ../out/
mv tc-initializer.iso ../out/
cd -

rm -r $WORKDIR

echo "done"

