/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef __COMMON_H__
#define __COMMON_H__
#include <stdint.h>
#include "bldopt.h"
#ifdef SDL_BUILD_ENABLE
#include <openssl/ssl.h>
#include <openssl/err.h>
#endif

#define	SRV_PORT			54321
#define	SRV_IP				"127.0.0.1"
#define NIC_NAME			"eth0"

#define MAX_STRING_LEN  	256
#define MAX_U_NAME_LEN		MAX_STRING_LEN
#define MAX_PWD_LEN			MAX_STRING_LEN
#define MAX_OS_NAME_LEN		MAX_STRING_LEN
#define MAX_VER_LEN			128

#define INFO_SEP_CHAR		':'
#define INFO_SEP_STR		":"

#define DFT_CERT_PREFIX		"tc_client"
#define DFT_CA_FILE			DFT_CERT_PREFIX "_ca.pem"
#define DFT_KEY_FILE		DFT_CERT_PREFIX "_key.pem"
#define DFT_CERT_FILE		DFT_CERT_PREFIX "_cert.pem"

typedef enum {
	AUTHENTICATION = 1,
	INUSE,
	SYNC,
	UPLOAD_CHECK,
	UPLOAD,
	DISCONNECT,	
	SYNC_DATA,
	UPLOAD_CHECK_DATA,
	UPLOAD_DATA,
	GET_USER_ID
}STATUS;

typedef struct {
	char 		username[MAX_OS_NAME_LEN + 1];  /* client's authenticate string to be verified by server **/
	char		password[MAX_PWD_LEN + 1];
	char		osname[MAX_OS_NAME_LEN + 1];	/* os names to be boot/sync */
	char 		curVer[MAX_VER_LEN + 1];
	char		nicName[MAX_STRING_LEN];	/* nic used for license check */
	char 		ip[MAX_STRING_LEN];   		/* The IP address we're listening on */

#ifdef SDL_BUILD_ENABLE
#ifdef ENABLE_TC_MUTUAL_TLS
	char		*ca_file;
	char		*key_file;
	char		*cert_file;
#endif
	SSL_CTX		*p_tc_ssl_ctx;
	SSL 		*p_ssl;
#endif

	uint16_t 	port;
	uint16_t	timeout;		/**< how long a connection may be idle
			       	   	   	   (0=forever) */
	int 		socket;	     	/* The socket of this client */
	int			status;
    char        filename[MAX_STRING_LEN];
    char        tokenfile[MAX_STRING_LEN];
} SERVER_CONFIG;

SERVER_CONFIG* parse_cmdline(int argc, char *argv[]) ;

#endif
