/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "debug.h"
#include "session.h"
#include "protocol.h"

int max_pkt_size = 1024*5;

int tc_send_req(tc_pkt_t *pkt, uint len)
{
  int head_len = sizeof(tc_hdr_t);
  int buf_len = len - head_len;
  uchar *buf = (((uchar *)pkt) + head_len); 
  int ret;

#ifdef SDL_BUILD_ENABLE
  ret = SSL_write(tc_hd[0].s_config->p_ssl, (uchar *)pkt, len);
#else
  ret = write(tc_hd[0].s_config->socket, (uchar *)pkt, len);
#endif
  if(ret != len) {
    //ERR("transmit failure: %s \r\n", strerror(errno));
    return TC_ERR_SEND;
  }
  
  return 0;
}

int tc_read_rsp(tc_pkt_t *pkt, uint *len)
{
	int ret;
	int head_len;
	int buf_len;
    int left;
	uchar *buf;
    uchar *p;

  
	buf_len = *len;
	head_len = sizeof(tc_hdr_t); 
	buf = (uchar *)pkt;
	
#ifdef SDL_BUILD_ENABLE
	ret = SSL_read(tc_hd[0].s_config->p_ssl, (uchar *)pkt,  head_len);
#else
    ret = read(tc_hd[0].s_config->socket, (uchar *)pkt,  head_len);
#endif
	if(ret <= 0) {
		if(ret == 0) {
			//ERR("Connection has been closed by server\n");
			return TC_ERR_CONN_CLOSED;
		}
		else {
	  		//ERR("Receive failure :%s\r\n", strerror(errno));
	  		return TC_ERR_RECV;
		}
	} 
    //PRT("hdr bytes: %d\n", head_len);

    left = pkt->hdr.len - head_len;
    p = (uchar *)(&pkt->tc_cmd);
	if(left > buf_len) {
		ERR("Error: package overflow\n");
		return TC_ERR_RECV;
	}
	
    while ( left > 0 )
    {
        //PRT("left bytes: %d\n", left);
#ifdef SDL_BUILD_ENABLE
		ret = SSL_read(tc_hd[0].s_config->p_ssl, p, left);
#else
     	ret = read(tc_hd[0].s_config->socket, p, left);
#endif
     	if(ret <= 0) {
     		if(ret == 0) {
     			//ERR("Connection has been closed by server\n");
     			return TC_ERR_CONN_CLOSED;
     		}
     		else {
     	  		//ERR("Receive failure :%s\r\n", strerror(errno));
     	  		return TC_ERR_RECV;
     		}
	    } 

        p += ret;
        left -= ret;
    }
      
	*len = (p - (uchar *)pkt);
    //PRT("total bytes: %d\n", *len);
	if(((pkt->hdr.ver_flags & 0x0f) & (1<<2)) != 0) {
		//ERR("Server set the response error bit: %d\r\n", pkt->hdr.error);
		if(pkt->hdr.error == TC_ERR_PWD_UNAME) {
			if(pkt->hdr.cmd > 0) {
				ret = pkt->hdr.error;
				ret |= (pkt->hdr.cmd << 8);
				return ret;
			}
		}
		return pkt->hdr.error;
	}

  	return 0;
}

int init_tc_hdr(tc_hdr_t *hdr)
{
  if(hdr == NULL) {
    ERR("(%s) hdr is null\r\n",__FUNCTION__);
	return -1;
  }

  memset(hdr, 0x0, sizeof(tc_hdr_t));
  hdr->ver_flags = 0x10;
  return 0;
}

int tc_cmd(tc_pkt_t *snd_pkt, uint snd_len, tc_pkt_t *rcv_pkt, uint *rcv_len) 
{
	int     i;
	uint    len;
	int     ret = -1; 
      
	ret = tc_send_req(snd_pkt, snd_len);
	if(ret != 0) {
		//ERR("tc_send_req error\r\n");
		return ret;
	}

	len = max_pkt_size;
	memset(rcv_pkt, 0x0, len);

	ret = tc_read_rsp(rcv_pkt, &len);
	if(ret != 0){
		//ERR("TC read rsp error 0x%x\r\n", ret);
		return ret;
	}

	*rcv_len = len;        
	return 0;
}
