/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef __PREDEFINE_H__
#define __PREDEFINE_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <syslog.h>
#include <stdint.h>
#include "bldopt.h"
#ifdef SDL_BUILD_ENABLE
#include <openssl/ssl.h>
#include <openssl/err.h>
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif


#define _LARGEFILE64_SOURCE
#define _FILE_OFFSET_BITS 		64

//Whether to verify the license file on server
#define VERIFY_LICENSE			1
// Whether to use mysql database to store server/client setttings
#define USE_MYSQL 				1
//Whether to collect and print the performance data
#define SYS_PERF				0
// Define whether to run the server in daemon mode 
#define RUN_IN_DAEMON_MODE		1
//Whether to update server status in db
#define UPDATE_SRV_STATUS_IN_DB	0
//Whether to update user status in db
#define UPDATE_USR_STATUS_IN_DB	0
//Whether to enable watch dog for tc user
#define ENABLE_TC_TIMER			0


#define	SRV_PORT				54321
#define	SRV_IP					"0.0.0.0"
#define NIC_NAME				"eth0"
#define LICENSE_FILE_NAME  		"license"

#define	DFT_CFG_DIR				"/etc/tc"
#define DFT_LIC_FILE			DFT_CFG_DIR "/" LICENSE_FILE_NAME
#define DFT_HOME_TC				"/home/tc"
#define DFT_HOME_VAR			DFT_HOME_TC	"/var"
#define DFT_DB_UNAME			"tcsync"
#define DFT_DB_PWD				"sa"
#define DFT_CERT_PREFIX			"syncSrv"
#define DFT_CA_FILE				DFT_CERT_PREFIX "_ca.pem"
#define DFT_KEY_FILE			DFT_CERT_PREFIX "_key.pem"
#define DFT_CERT_FILE			DFT_CERT_PREFIX "_cert.pem"

#define DFT_APP_NAME			"tc-sync-server"
#define SINGLE_INST_LOCK_FORMAT	"%s/%s-%s-%ld.pid"

#define MAX_UNAME_LEN  			64
#define MAX_PWD_LEN  			MAX_UNAME_LEN
#define MAX_IP_LEN			64
#define MAX_NIC_NAME_LEN		64

#define MAX_STRING_LEN  		256
#define DB_MAX_STRING_LEN		MAX_STRING_LEN*2
#define MAX_SI_FILE_LEN			(MAX_NIC_NAME_LEN + 64)	
#define DIFF_PAGE_SIZE    		(2*1024*1024)
#define MAX_BUF_SIZE			1024*5
// 4096 = 2 ^ 12
#define DIFF_PAGE_INDEX    		12
#define DFT_UDF_DIR				DFT_HOME_TC "/userdata"
#define MAX_CLIENTS     		1024
#define DFT_MAX_VER_KEPT		20
#define DFT_MAX_DATA_VER_KEPT	80

#define MIN_HB_COUNT    		1
#define CHECK_TIMER_INTERVAL  	10
// 4 mins
#define HB_CHECK_TIMER  		CHECK_TIMER_INTERVAL*2
//if we hadn't received client's HB for longer than HB_LOST_TIMEOUT (8 mins) second,
//then we'll destory the client. 
#define HB_LOST_TIMEOUT  		HB_CHECK_TIMER*4 
// 4 mins
#define RESET_CONF_INTERVAL   	CHECK_TIMER_INTERVAL
#define RW_BUF_CNT      		0x1000 
#define TC_SERVER_ID   			4
// 1 * HB_CHECK_TIMER
#define BOOT_TIMEOUT			0

#define IN_USE(a)				((a)->usedIdx >= 0)
#define NOT_IN_USE(a)			!IN_USE(a)

#define INFO_SEP_CHAR			':'
#define INFO_SEP_STR			":"

#define AUTO_BOOT_USERNAME		"autoboot"
#define AUTO_BOOT_GROUPNAME		"system"
#define AUTO_BOOT_GROUPID		"4"

#define ERR_DUP_ENTRY_IN_DB		1062
#define DFT_LOGIN_FAILURE_INTERVAL		5
#define DFT_TC_CA_DIR			"/home/tc/etc/cert/"


typedef enum {
  POWERED_OFF = 0x0,
  POWERED_ON = 0x1,
  AUTHENTICATED = 0x02,
  SYNCING = 0x03,
  BOOTING= 0x04,
  RUNNING = 0x05,  
  UPLOADING = 0x06,
  SHUTTING_DOWN = 0x07,
  INVALID = 0x08,  
  INITING_DISK = 0x09,
  UPDATING_FIRMWARE = 0x0A,
  DEPLOYING = 0x0B  
} CLIENT_STATUS;

typedef enum {
  SERVER_STATUS_OFFLINE = 0x0,
  SERVER_STATUS_ONLINE = 0x1,
  SERVER_STATUS_OVERLOAD = 0x2,
  SERVER_STATUS_NORMAL = 0x3
} SERVER_STATUS;

typedef enum {
	IMAGE_NO_RIGHTS=0x0,
	IMAGE_READONLY=0x1,
	IMAGE_DIFF_READWRITE=0x2,	
	IMAGE_RAW_READWRITE=0x3,
	IMAGE_INSTALL_RWRITE=0x4	
}IMAGE_ACCESS;

typedef enum {
	IMAGE_DISABLE_PERS=0x0,
	IMAGE_ENABLE_PERS=0x1
}IMAGE_PERSONALIZATION;

/**
 * Variables associated with a server.
 **/
typedef struct {
	char 		*cur_dir;
	char 		single_inst_file[MAX_SI_FILE_LEN];
	char		*license_file;
#ifdef SDL_BUILD_ENABLE
#ifdef ENABLE_TC_MUTUAL_TLS
	char		*ca_file;
#endif
	char		*key_file;
	char		*cert_file;
	SSL_CTX		*p_tc_ssl_ctx;
#endif
	uint8_t	  	eth_mac[6];
	char		ethName[64];	/** nic used for license check >  **/
	char 		ip[64];   		/** The IP address we're listening on */
	long	 	port;   	/**< port we're exporting this server at */
	uint16_t	timeout;		/** how long a connection may be idle
			       	   	   	   (0=forever) */
	int 		socket;	     	/** The socket of this server. */
	int			status;
	int			login_failure_interval;
	uint64_t	runTime;
} SERVER_CONFIG;


#endif

