/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef __TC_PROTOCOL_H__
#define __TC_PROTOCOL_H__

typedef struct _tc_cmd_t {	
	__uint16_t  bufcnt;
	__uint8_t	scnt;
	__uint8_t	vercmd;
	__uint16_t	len;
	__uint8_t   data[0];
} __attribute__ (( packed )) tc_cmd_t;

typedef struct _tc_hdr_t {
	__uint8_t	  ver_flags;
	__uint8_t	  error;
	__uint8_t	  cmd;
	__uint16_t	  len;
	__uint8_t	  tag[4];
} __attribute__ (( packed )) tc_hdr_t;

typedef struct _tc_pkt_t {
  tc_hdr_t hdr; 
  tc_cmd_t  tc_cmd;
	
} __attribute__ (( packed )) tc_pkt_t;

#define TC_MAIN_ERR_LICENSE			-1
#define TC_MAIN_ERR_DB				-2
#define TC_MAIN_ERR_BAD_PARAMETER	-3
#define TC_MAIN_ERR_INST_RUNNING	-4 
#define TC_MAIN_ERR_SOCK			-5 
#define TC_MAIN_ERR_MAC			  	-6
#define TC_MAIN_ERR_COMMON		  	-7
#define TC_MAIN_ERR_SSL			  	-8


#define TC_ERR_BAD_VERSION	  	0x05 

#define TC_ERR_BAD_COMMAND	  	0x01
#define TC_ERR_BAD_PARAMETER	0x02
#define TC_ERR_UNAVAILABLE	  	0x03 
#define TC_ERR_CONFIG_ERR	  	0x04 
#define TC_ERR_BAD_VERSION	  	0x05 
#define TC_ERR_TIMEOUT       	0x06
#define TC_ERR_ONLINE		  	0x07
#define TC_ERR_NO_RECORD	  	0x08
#define TC_ERR_INVALID_MAC	  	0x09
#define TC_ERR_INVALID_TAG	  	0x0A

#define TC_ERR_PWD_UNAME	  	0x0B
#define TC_ERR_GET_VERSION	  	0x0C
#define TC_ERR_CONN_CLOSED	  	0x0D
#define TC_ERR_RECV			  	0x0E
#define TC_ERR_SEND			  	0x0F
#define TC_ERR_DATABASE		  	0x20
#define TC_ERR_NOT_LOGINED	  	0x21
#define TC_ERR_NO_IMAGE		  	0x22
#define TC_ERR_IMAGE_SETUP	  	0x23
#define TC_ERR_PERMISSION	  	0x24
#define TC_ERR_IMAGE_LOCKED		0x25
#define TC_ERR_IMAGE_NOT_READY 	0x26
#define TC_ERR_USER_BLOCKED	  	0x27
#define TC_ERR_SSL_FAILURE	  	0x28
#define TC_ERR_MAXIUM_FAILURE  	0x29

#define TC_AFLAGS_WRITE	        0x01
#define TC_AFLAGS_ASYNC	        0x02
#define TC_AFLAGS_DEV_HEAD	    0x10
#define TC_AFLAGS_EXTENDED	    0x40

#define TC_VERSION		        0x10
#define TC_VERSION_MASK 	    0xf0

#define TC_FL_ERROR	            0x04
#define TC_FL_RESPONSE	        0x08

#define TC_MAJOR_BROADCAST      0xffff
#define TC_MINOR_BROADCAST      0xff

#define TC_CMD_ATA	          	0x00
#define TC_CMD_CONFIG	      	0x01

#define TC_STATUS_ERR_MASK	  	0x0f
#define TC_STATUS_PENDING	  	0x80

#define TC_TAG_MAGIC	        0xebeb0000

#define TC_PROTO_TYPE           0x88a2

#define TC_MAX_SECTORS         	2

#define TC_TAG_MAGIC	        0xebeb0000

#define QCMD(x) (((x)->vercmd & 0xf0)>>4)
#define SUB_CMD(x)  (((x)->tc_cmd.vercmd & 0xf0)>>4)
#define TC_TAG16(high_8,low_8)			(high_8<<8|low_8)
#define TC_TAG32(high_16,low_16)		(high_16<<16|low_16)
#define SET_TAG(tag, val)     			do{int newtag=val+1;memcpy(tag, &newtag, 4);}while(0)
#define GET_TAG(tag, val)     			do{memcpy(&val, tag, 4);val--;}while(0)

typedef enum {
  	READ_CONF = 0,
	HB_CONF,
	AUTH_CONF,
	SET_OS_CONF,
	RESET_CONF,
	UPLOAD_CHECK_CONF,
	UPLOAD_CONF,
	//this conf cmd is for image syncing
	SYNC_CONF,
	SYNC_DATA_CONF,
	UPLOAD_CHECK_DATA_CONF,
	UPLOAD_DATA_CONF,
	GET_USER_ID_CONF,
} CONF_CMD;

int tc_main(int argc, char *argv[]);

#endif
