/******************************************************************************
Copyright 2012 - 2020 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include "predefine.h"
#include "tcprotocol.h"
#include "client.h"
#include "tctimer.h"
#include "debug.h"
#include "db.h"

TC_timer 	g_tc_timers[MAX_CLIENTS + 1] = {0};
extern Client g_running_clients[MAX_CLIENTS];

int _get_timer_index(int timer_id)
{
	int i;
	
	for(i=0; i< MAX_CLIENTS; i++) {
		if(IN_USE(g_tc_timers+i) && 
			g_tc_timers[i].usedIdx == timer_id)
			return i;
	}

	return -1;		
}
int set_timer(uint32_t timer_id, uint32_t time, timer_callback fn, 
		int context, uint8_t periodic)
{
	int i;

	if(!fn)
		return 1;

	for(i=0; i< MAX_CLIENTS; i++) {
		if(NOT_IN_USE(g_tc_timers+i))
			break;
	}
	if(i == MAX_CLIENTS) {
		ERR("Error: no timer slot available, max is: %d\n", MAX_CLIENTS);
		return 1;
	}

	//g_tc_timers[i].timer_id = timer_id;
	g_tc_timers[i].usedIdx = timer_id;
	g_tc_timers[i].time_total = time;
	g_tc_timers[i].time_left = time;
	g_tc_timers[i].function = fn;
	g_tc_timers[i].fn_context = context;
	g_tc_timers[i].periodic = periodic;

	return 0;
		
}

void _check_timer(int signumber)
{
	int i;	
	uint32_t time_old;
	uint32_t index;
	
	for(i=0; i< MAX_CLIENTS; i++) {
		if(NOT_IN_USE(g_tc_timers+i))
			continue;
		
		if(g_tc_timers[i].time_left >= CHECK_TIMER_INTERVAL)
			g_tc_timers[i].time_left -= CHECK_TIMER_INTERVAL;
		else
			g_tc_timers[i].time_left = 0;
		
		if(g_tc_timers[i].time_left == 0){
			time_old = g_tc_timers[i].time_total;

			if(g_tc_timers[i].function != NULL)
				g_tc_timers[i].function(g_tc_timers[i].fn_context);

			MSG("Timer %d fired!\n", g_tc_timers[i].usedIdx);
			g_tc_timers[i].time_total = 0;
			index = g_tc_timers[i].usedIdx;
			g_tc_timers[i].usedIdx = -1;

			if(g_tc_timers[i].periodic){
				set_timer(index, time_old, g_tc_timers[i].function, g_tc_timers[i].fn_context, g_tc_timers[i].periodic);
			}else{
				g_tc_timers[i].function = NULL;
			}
		}
	}
	alarm(CHECK_TIMER_INTERVAL);
}

int remove_timer(int timer_id)
{
	int i;
	
	i = _get_timer_index(timer_id);
	//DEBUG("timer_id: %d, and timer index: %d\n", timer_id, i);
	if(i < 0)
		return -1;
	
	memset(&g_tc_timers[i], 0x0, sizeof(g_tc_timers[i]));
	g_tc_timers[i].usedIdx = -1;

	return i;
}


void _watchdog_func(int dunno) 
{ 
  Client *client;
  int i;
  
  for(i = 0; i < MAX_CLIENTS; i++) {
    client = g_running_clients + i;
    if(IN_USE(client)) {

      if(client->boot_timeout != 0) {
        client->boot_timeout--;
        continue;
      }
      //MSG("Client %d:%s hb is %d\n",i, client->username, client->hb_count); 

      if(client->hb_count < MIN_HB_COUNT) {
	  	client->hb_lost_timeout += HB_CHECK_TIMER;
		//Have we reached the timeout to destory lost client?
		if(client->hb_lost_timeout < HB_LOST_TIMEOUT) {
			//not yet
			if(client->status == RUNNING) {
				MSG("no hb received, set user %s to invalid\n", client->username);
				mark_client_invalid(client);
			}
		//yes, go ahead...
		}else{
	      		MSG("update_hb:user %s maybe lost\n", client->username);
				if(client->username && client->username[0] != 0)
	      			delete_user_status(client->username, client->image_id);
	        	free_client(g_running_clients+i);
		}
      } else {
        client->hb_count = 0;
		client->hb_lost_timeout = 0;
      }
    
    }
  }
}

void timer_array_init(TC_timer *timer_array, int item_cnt)
{
	int i;
	
	if(!timer_array || item_cnt < 0)
		return;

	for(i = 0; i < item_cnt; i++)
		timer_array[i].usedIdx = -1;
}

void setup_watchdog()
{
	timer_array_init(g_tc_timers, sizeof(g_tc_timers)/sizeof(TC_timer));
	signal(SIGALRM, _check_timer);
	set_timer(MAX_CLIENTS, HB_CHECK_TIMER, _watchdog_func, SIGALRM, 1);
	alarm(CHECK_TIMER_INTERVAL);
}



