@Rem
@Rem @file
@Rem install TC applications to specified destinations
@Rem
@echo off

setlocal EnableDelayedExpansion

@Rem
@Rem define installation target directories
@Rem
set dstAppDir=c:\
set dstDllDir=%SYSTEMROOT%\system32

@Rem
@Rem %~dp0 has tailing '\'
@Rem
set srcAppDir=%~dp0
if /i "%srcAppDir:~-1%" == "\" set srcAppDir=%srcAppDir:~0,-1%

@Rem
@Rem undefine enableRdp to disable Rdp
@Rem
set /A enableRdp=1

@Rem
@Rem detect system bitness
@Rem
call :FooGetOsBitness platform
echo Operating System platform: %platform%

@Rem
@Rem define executables
@Rem
set ifSetupBat=%srcAppDir%\ifsetup2.bat
set delDosDevBat=%srcAppDir%\delDosDev.bat
set md5sumApp=%srcAppDir%\md5sum\%platform%\md5sum.exe
set getUsernameApp=%srcAppDir%\getusername\%platform%\getusername.exe
set getNetCfgApp=%srcAppDir%\getnetcfg\%platform%\getnetcfg.exe
set chgResApp=%srcAppDir%\tcchgres\%platform%\tcchgres.exe
set credProviderDll=%srcAppDir%\cred\%platform%\TcCredProvider.dll
set credProviderReg=%srcAppDir%\cred\TcCredProvider.reg
set monitorService=%srcAppDir%\TcSvcMonitor\%platform%\TcSvcMonitor.exe
set monitorServiceName=TcSvcMonitor
set usbCtrlBat=%srcAppDir%\TcUsbCtrl.bat
set setConApp=%srcAppDir%\setcon\%platform%\setcon.exe
set tcUpdateApp=%srcAppDir%\TCUpdate.exe
set udfClientServiceName=MUdfClientService
set udfClientService=%srcAppDir%\TcSvcPsUpload\%platform%\MUdfClientSvc.exe
set libLz4Dll=%srcAppDir%\TcSvcPsUpload\%platform%\liblz4.dll
set pThreadVcDll=%srcAppDir%\TcSvcPsUpload\%platform%\pthreadVC2.dll

@Rem ===========================================================================
@Rem installation goes here...
@Rem

@Rem
@Rem install services, applications, dynamically-linking libraries
@Rem

@Rem monitorService
call :FooInstallService "%monitorServiceName%" "%dstAppDir%" "%monitorService%"
call :FooChkRetVal "%monitorService%"
if ErrorLevel 1 goto ExitLabel

@Rem MUdfClientSvc.exe
call :FooInstallService "%udfClientServiceName%" "%dstAppDir%" "%udfClientService%"
call :FooChkRetVal "%udfClientService%"
if ErrorLevel 1 goto ExitLabel

call :FooInstallFile "%dstAppDir%" "%libLz4Dll%"
call :FooChkRetVal "%libLz4Dll%"
if ErrorLevel 1 goto ExitLabel

call :FooInstallFile "%dstAppDir%" "%pThreadVcDll%"
call :FooChkRetVal "%pThreadVcDll%"
if ErrorLevel 1 goto ExitLabel

@Rem md5sumApp
call :FooInstallFile "%dstAppDir%" "%md5sumApp%"
call :FooChkRetVal "%md5sumApp%"
if ErrorLevel 1 goto ExitLabel

@Rem getUsernameApp
call :FooInstallFile "%dstAppDir%" "%getUsernameApp%"
call :FooChkRetVal "%getUsernameApp%"
if ErrorLevel 1 goto ExitLabel

@Rem getNetCfgApp
call :FooInstallFile "%dstAppDir%" "%getNetCfgApp%"
call :FooChkRetVal "%getNetCfgApp%"
if ErrorLevel 1 goto ExitLabel

@Rem chgResApp
call :FooInstallFile "%dstAppDir%" "%chgResApp%"
call :FooChkRetVal "%chgResApp%"
if ErrorLevel 1 goto ExitLabel

@Rem ifSetupBat
call :FooInstallFile "%dstAppDir%" "%ifSetupBat%"
call :FooChkRetVal "%ifSetupBat%"
if ErrorLevel 1 goto ExitLabel

@Rem delDosDevBat
call :FooInstallFile "%dstAppDir%" "%delDosDevBat%"
call :FooChkRetVal "%delDosDevBat%"
if ErrorLevel 1 goto ExitLabel

@Rem credProviderDll
call :FooInstallFile "%dstDllDir%" "%credProviderDll%"
call :FooChkRetVal "%credProviderDll%"
if ErrorLevel 1 goto ExitLabel

if exist "%credProviderReg%" ( 
  reg import "%credProviderReg%"
  if "%errorlevel%" NEQ "0" ( 
    echo Failed to import %credProviderReg%
    set exitCode=1
    goto ExitLabel
  )
) else (
  echo "%credProviderReg%": no such file
  set exitCode=1
  goto ExitLabel
)

@Rem usb ctrl
call :FooInstallFile "%dstAppDir%" "%usbCtrlBat%"
call :FooChkRetVal "%usbCtrlBat%"
if ErrorLevel 1 goto ExitLabel

@Rem setcon
call :FooInstallFile "%dstAppDir%" "%setConApp%"
call :FooChkRetVal "%setConApp%"
if ErrorLevel 1 goto ExitLabel

@Rem TCUpdate.exe
call :FooInstallFile "%dstAppDir%" "%tcUpdateApp%"
call :FooChkRetVal "%tcUpdateApp%"
if ErrorLevel 1 goto ExitLabel

call :FooGetFilename "%tcUpdateApp%" tmpUpdateAppFileName
reg add "HKLM\SOFTWARE\Microsoft\Windows\CurrentVersion\Run" /v TcUpdateApp /t REG_SZ /d "%dstAppDir%%tmpUpdateAppFileName%" /f >nul
set ntp2pgetApp=ntp2pget.exe
netsh advfirewall firewall add rule name="%ntp2pgetApp%" dir=in action=allow program="%LocalAppData%\temp\%ntp2pgetApp%" enable=yes profile=domain,public,private

@Rem
@Rem enable Rdp
@Rem
if /i not "%enableRdp%" == "0" (
  call :FooEnableRdp
  call :FooChkRetVal Rdp
  if ErrorLevel 1 goto ExitLabel
)

set exitCode=0
goto ExitLabel


:FooGetOsBitness
@Rem ===========================================================================
@Rem
@Rem get OS bitness
@Rem %1, a variable name to save the result: x86 or amd64
@Rem
if /i "%~1" == "" exit /b 1
set %~1=amd64
if not defined PROCESSOR_ARCHITECTURE (
  echo fail to detect os bitness
  echo please explicitly specify platform when invoking this script
  set exitCode=1
  goto ExitLabel
) else (
  if /i %PROCESSOR_ARCHITECTURE% == x86 (
    if not defined PROCESSOR_ARCHITEW6432 set %~1=x86
  )
)
exit /b 0


:FooChkRetVal
@Rem ===========================================================================
@Rem
@Rem check return value
@Rem %1, module name
@Rem
set retVal=%ErrorLevel%
set modName=%~1
if /i not "%retVal%" == "0" (
  echo fail to install "%modName%".
  set exitCode=1
@Rem  goto ExitLabel
) else (
  set exitCode=0
  echo "%modName%" was installed successfully.
)
exit /b !exitCode!


:FooGetFilename
@Rem ===========================================================================
@Rem return file name from a given filepath
@Rem %1, a file path
@Rem %2, a variable name
@Rem example
@Rem   call :FooGetFilename "a long path/t o/filename.ext" varfile
@Rem   result: echo %varfile% would be filename.txt
if /i "%~1" == "" exit /b 1
if /i "%~2" == "" exit /b 1
for /f %%a in ("%~1") do set %~2=%%~nxa
exit /b 0

:FooInstallFile
@Rem ===========================================================================
@Rem install file to destination
@Rem %1, destination directory
@Rem %2, source executable file path
if /i "%~1" == "" exit /b 1
if /i "%~2" == "" exit /b 1
set tmpDstDir=%~1
set tmpSrcFileAbsPath=%~2

@Rem add tailing "\" if none
if /i not "%tmpDstDir:~-1%" == "\" set tmpDstDir=%tmpDstDir%\
if not exist "%tmpDstDir%" (
  echo "%tmpDstDir%": no such directory
  echo create directory "%tmpDstDir%"
  md "%tmpDstDir%"
)

call :FooGetFilename "%tmpSrcFileAbsPath%" tmpSrcFile
set tmpDstFile=%tmpDstDir%%tmpSrcFile%

if exist "%tmpDstFile%" (attrib -h -s "%tmpDstFile%")
if exist "%tmpSrcFileAbsPath%" (
  copy /y "%tmpSrcFileAbsPath%" "%tmpDstDir%" >nul
) else (
  echo "%tmpSrcFileAbsPath%" : no such file
  exit /b 1
)
if exist "%tmpDstFile%" (
  attrib +h +s "%tmpDstFile%"
  exit /b 0
) else (
  exit /b 1
)


:FooInstallService
@Rem ===========================================================================
@Rem install a win32 service
@Rem %1, Service Name
@Rem %2, destination directory name
@Rem %3, source file path
if /i "%~1" == "" exit /b 1
if /i "%~2" == "" exit /b 1
if /i "%~3" == "" exit /b 1
set tmpSvcName=%~1
set tmpDstDir=%~2
set tmpSrcFileAbsPath=%~3

@Rem add tailing "\" if none
if /i not "%tmpDstDir:~-1%" == "\" set tmpDstDir=%tmpDstDir%\
if not exist "%tmpDstDir%" (
  echo "%tmpDstDir%": no such directory
  echo create directory "%tmpDstDir%"
  md "%tmpDstDir%"
)

call :FooGetFilename "%tmpSrcFileAbsPath%" tmpSrcFile
set tmpDstFile=%tmpDstDir%%tmpSrcFile%

@Rem kill service, if any
sc query %tmpSvcName% >nul
if "%errorlevel%" EQU "0" ( goto queryServiceStatus ) else ( goto installService )

:queryServiceStatus
echo query service status ...
for /f "skip=3 tokens=4" %%i in ('sc query %tmpSvcName%') do set "svcStatus=%%i" & goto checkServiceStatusValue

:checkServiceStatusValue
echo service status value = %svcStatus%
if /i "%svcStatus%"=="STOPPED" ( goto installService ) else ( goto stopService )

:stopService
echo stop service ...
sc stop %tmpSvcName% > nul
choice /t 1 /n /d y >nul
goto queryServiceStatus

:installService
if exist "%tmpDstFile%" ( attrib -h -s "%tmpDstFile%" )
if exist "%tmpSrcFileAbsPath%" (
  copy /y "%tmpSrcFileAbsPath%" "%tmpDstFile%"
) else (
  echo "%tmpSrcFileAbsPath%" : no such file
  exit /b 1
)

sc query %tmpSvcName% >nul
if "%errorlevel%" EQU "0" (
  if exist "%tmpDstFile%" (sc config %tmpSvcName% start= auto binPath= "%tmpDstFile%")
) else (
  if exist "%tmpDstFile%" (sc create %tmpSvcName% start= auto binPath= "%tmpDstFile%")
)
netsh advfirewall firewall delete rule name="%tmpSvcName%" program="%tmpDstFile%" profile=domain,public,private >NUL
if exist "%tmpDstFile%" (netsh advfirewall firewall add rule name="%tmpSvcName%" dir=in action=allow program="%tmpDstFile%" enable=yes profile=domain,public,private)
if exist "%tmpDstFile%" (
  attrib +h +s "%tmpDstFile%"
  exit /b 0
) else (
  exit /b 1
)

:FooGetLanguageCode
@Rem ===========================================================================
@Rem get installed language code
@Rem 
@Rem %1, variable name to save the result
if /i "%~1" == "" exit /b 1
for /f "tokens=3 delims= " %%c in ('reg query "hklm\system\controlset001\control\nls\language" /v InstallLanguage') do (
  set %~1=%%c
)
exit /b 0


:FooEnableRdp
@Rem ===========================================================================
@Rem enable Rdp
call :FooGetLanguageCode langCode
echo installed language: %langCode%

echo try to enable remote desktop
@Rem enable remote desktop
reg add "HKLM\SYSTEM\CurrentControlSet\Control\Terminal Server" /v fDenyTSConnections /t REG_DWORD /d 0 /f >nul
if %ERRORLEVEL% NEQ 0 (echo fail to enable remote desktop & exit /b 1)

@Rem enable remote desktop to pass-through windows firewall
netsh advfirewall firewall add rule name="TcAllowTcpPort3389" dir=in action=allow enable=yes protocol=tcp localport=3389 profile=any
netsh advfirewall firewall add rule name="TcAllowUdpPort3389" dir=in action=allow enable=yes protocol=udp localport=3389 profile=any

@Rem enable windows account with an empty password can be logged on via remote desktop 
reg add "HKLM\SYSTEM\CurrentControlSet\Control\Lsa" /v LimitBlankPasswordUse /t REG_DWORD /d 0 /f >nul
if %ERRORLEVEL% NEQ 0 (echo fail to enable remote desktop for windows account with empty password & goto exit /b 1)
exit /b 0


@Rem ===========================================================================
@Rem exit
:ExitLabel
(
  echo.
  echo -- %date% %time%
  echo -- batch exited code: !exitCode!
  echo.

  if /i "!exitCode!" == "0" (
    echo -- App installed successfully!
  ) else (
    echo -- Failed to install app
  )
  
  endlocal
  exit /b !exitCode!
)
