:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: ifsetup2.bat
::     Set network configuration with command-line options,
::     or from options read from an external configuration file.
:: synopsis
::     see the Usage section.
:: params
::     optional
:: return
::     0 on success,
::     non-zero value otherwise.
:: example
::     try ifsetup2.bat /help
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
@set ExtraBat=c:\delDosDev.bat
@if exist "%ExtraBat%" ( 
  @call %ExtraBat%
)

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Save another copy of stdout and stderr for the batch to a file
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
@set batchLogFile=%TEMP%\ifsetup2.log
@set batchLogFile=C:\ifsetup2.log

@if /i not "%1" == "DUMMY" (
  @echo log to file "%batchLogFile%"
  @echo.
  @%0 DUMMY %* > %batchLogFile% 2>&1
  @REM choice /t 1 /n /d y > nul
  @REM type %batchLogFile%
)
@shift /1


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: real batch below
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
@echo off
setlocal enableExtensions enableDelayedExpansion

set thisScript=%0
set /a exitCode=0
@REM time (seconds) to wait for nic to be active 
set /a nWaitNicActiveTime=10 * 60

set targetBootProto=
set targetHwAddr=
set targetIpAddr=
set targetNetMask=
set targetGateway=
set targetIfName=
set targetDnsPrimary=
set targetDnsSecondary=

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: batch entry
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

if /i "%~1" == "" (
  goto GGetFileConfigLabel
) else (
  goto GGetCmdLineConfigLabel
)

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GGetCmdLineConfigLabel
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
set dBootProto=
set dHwAddr=
set dIpAddr=
set dNetMask=
set dGateway=
set dPrimaryDns=
set dSecondaryDns=

:LabelParseArgs

if /i "%~1" == "/?" shift & goto Usage
if /i "%~1" == "/h" shift & goto Usage
if /i "%~1" == "/help" shift & goto Usage
if /i "%~1" == "-h" shift & goto Usage
if /i "%~1" == "--help" shift & goto Usage

if /i "%~1" == "/mac" (
  shift
  call set val=%%~1
  if not defined val goto MissingArgument
  if /i "!val:~0,1!" == "/" goto MissingArgument
  set dHwAddr=!val!
  shift
  goto LabelParseArgs
)

if /i "%~1" == "/ip" (
  shift
  call set val=%%~1
  if not defined val goto MissingArgument
  if /i "!val:~0,1!" == "/" goto MissingArgument
  set dIpAddr=!val!
  shift
  goto LabelParseArgs
)

if /i "%~1" == "/mask" (
  shift
  call set val=%%~1
  if not defined val goto MissingArgument
  if /i "!val:~0,1!" == "/" goto MissingArgument
  set dNetMask=!val!
  shift
  goto LabelParseArgs
)

if /i "%~1" == "/gw" (
  shift
  call set val=%%~1
  if not defined val goto MissingArgument
  if /i "!val:~0,1!" == "/" goto MissingArgument
  set dGateway=!val!
  shift
  goto LabelParseArgs
)

if /i "%~1" == "/dns" (
  shift
  call set val=%%~1
  if not defined val goto MissingArgument
  if /i "!val:~0,1!" == "/" goto MissingArgument
  set dPrimaryDns=!val!
  shift
  goto LabelParseArgs
)

if /i "%~1" == "/dns2" (
  shift
  call set val=%%~1
  if not defined val goto MissingArgument
  if /i "!val:~0,1!" == "/" goto MissingArgument
  set dSecondaryDns=!val!
  shift
  goto LabelParseArgs
)

if /i not "%~1" == "" (
  echo -- unknown argument: %1
  shift
  goto Usage
)

if /i "%dIpAddr%" == "" (
  set dBootProto=dhcp
) else (
  set dBootProto=static
)
echo option parsed from command-line
echo -- dBootProto=%dBootProto%
echo -- dHwAddr=%dHwAddr%
echo -- dIpAddr=%dIpAddr%
echo -- dNetMask=%dNetMask%
echo -- dGateway=%dGateway%
echo -- dPrimaryDns=%dPrimaryDns%
echo -- dSecondaryDns=%dSecondaryDns%

@REM copy value from (key,value) pairs
set targetBootProto=%dBootProto%
set targetHwAddr=%dHwAddr%
set targetIpAddr=%dIpAddr%
set targetNetMask=%dNetMask%
set targetGateway=%dGateway%
set targetIfName=
set targetDnsPrimary=%dPrimaryDns%
set targetDnsSecondary=%dSecondaryDns%

goto GTaskEntry


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GGetFileConfigLabel
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
set TCNET_APP=C:\getnetcfg.exe
set TCNET_ERROR_FILE=%TEMP%\tcnet.err.txt
set TCNET_CFG_FILE=%TEMP%\tcneft.cfg
set TCNET_CMD=%TCNET_APP% --all
set KEY_BOOTPROTO=BOOTPROTO
set KEY_HWADDR=HWADDR
set KEY_IPADDR=IPADDR
set KEY_NETMASK=NETMASK
set KEY_GATEWAY=GATEWAY
set KEY_DNS=DNS
set KEY_DNS2=DNS2
set VAL_BOOTPROTO=
set VAL_HWADDR=
set VAL_IPADDR=
set VAL_NETMASK=
set VAL_GATEWAY=
set VAL_DNS=
set VAL_DNS2=
@REM remove cache file, if any
if exist %TCNET_CFG_FILE% (
  echo remove cache file %TCNET_CFG_FILE%
  del /q /f %TCNET_CFG_FILE%
  if errorlevel 1 (
    echo -- Fail to remove cache file
    set /a exitCode=4
    goto ExitLabel
  ) else (
    echo -- successfully remove file %TCNET_CFG_FILE%
  )
)

@REM get desirable network configurations
echo get desirable network config
%TCNET_CMD% >%TCNET_CFG_FILE%
@REM %TCNET_CMD% >%TCNET_CFG_FILE% 2>%TCNET_ERROR_FILE%
set /a exitCode=%errorlevel%
echo -- %TCNET_APP% exited with %exitCode%
if /i "%exitCode%" == "0" (
  for /f "tokens=1,2 delims== " %%i in (%TCNET_CFG_FILE%) do (
    @REM echo %%i %%j
    if /i [%%i]==[%KEY_BOOTPROTO%] set VAL_BOOTPROTO=%%j
    if /i [%%i]==[%KEY_HWADDR%] set VAL_HWADDR=%%j
    if /i [%%i]==[%KEY_IPADDR%] set VAL_IPADDR=%%j
    if /i [%%i]==[%KEY_NETMASK%] set VAL_NETMASK=%%j
    if /i [%%i]==[%KEY_GATEWAY%] set VAL_GATEWAY=%%j
    if /i [%%i]==[%KEY_DNS%] set VAL_DNS=%%j
    if /i [%%i]==[%KEY_DNS2%] set VAL_DNS2=%%j
  )
) else (
  set /a exitCode=5
  goto ExitLabel
)

echo -- %KEY_BOOTPROTO% = %VAL_BOOTPROTO%
echo -- %KEY_HWADDR% = %VAL_HWADDR%
echo -- %KEY_IPADDR% = %VAL_IPADDR%
echo -- %KEY_NETMASK% = %VAL_NETMASK%
echo -- %KEY_GATEWAY% = %VAL_GATEWAY%
echo -- %KEY_DNS% = %VAL_DNS%
echo -- %KEY_DNS2% = %VAL_DNS2%

@REM copy value from (key,value) pairs
set targetBootProto=%VAL_BOOTPROTO%
set targetHwAddr=%VAL_HWADDR%
set targetIpAddr=%VAL_IPADDR%
set targetNetMask=%VAL_NETMASK%
set targetGateway=%VAL_GATEWAY%
set targetIfName=
set targetDnsPrimary=%VAL_DNS%
set targetDnsSecondary=%VAL_DNS2%

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GTaskEntry
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

@REM check install language
echo check installed language
for /f "tokens=3" %%i in ('reg query HKLM\SYSTEM\CurrentControlSet\Control\nls\language /v Installlanguage') do (
  echo -- Windows installed language, code [0x%%i]
)

@REM check system language
echo check system language
for /f "tokens=3" %%i in ('reg query "HKEY_USERS\S-1-5-18\Control Panel\Desktop\MuiCached" /v MachinePreferredUILanguages') do (
  echo -- Windows system language, code [%%i]
)

@REM check windows version
@REM only win7 and win10 are supported
echo check Windows version
set WINDOWS7=6.1
set WINDOWS10=10.0
for /f "tokens=4,5 delims=,. " %%i in ('ver') do set "build_n=%%i.%%j"
if /i [%build_n%]==[%WINDOWS7%] (
  set OSVER=Windows7
  echo -- system version: !OSVER!
) else (
  if /i [%build_n%]==[%WINDOWS10%] (
    set OSVER=Windows10
    echo -- system version: !OSVER!
  ) else (
    echo -- unsupported system version[%build_n%]
    set /a exitCode=3
    goto ExitLabel
  )
)

@REM check hwaddr value 
echo check if hwaddr value is set
if defined targetHwAddr (
  echo -- hwaddr value: %targetHwAddr%
) else (
  echo -- illegal hwaddr value
  set /a exitCode=5
  goto ExitLabel
)

@REM get interface name by hwaddr
echo try to get interface name
:GCheckNicLoop
set targetIfName=
call :GetIfNameByHwAddrFunc %targetHwAddr% targetIfName
if defined targetIfName (
  echo -- target network interface name: %targetIfName%
) else (
  set /a exitCode=6
  @REM the nic interface might not be active
  echo -- [!nWaitNicActiveTime!] try to get interface name with given hwaddr: %targetHwAddr%
  set /a nWaitNicActiveTime="!nWaitNicActiveTime! - 1"
  @REM sleep for 1 second
  choice /t 1 /n /d y >nul
  if !nWaitNicActiveTime! GTR 0 (
    goto GCheckNicLoop
  )
)
if not defined targetIfName (
  set /a exitCode=6
  echo -- fail to get interface name with given hwaddr: %targetHwAddr%
  goto ExitLabel
)

@REM if not 'static', then 'dhcp'
echo check use static or dhcp configuration
if /i not "%targetBootProto%" == "static" (
  set targetBootProto=dhcp
  echo -- boot protocol is about to be set to dhcp
) else (
  echo -- boot protocol is about to be set to static
)

@REM check the parameters and call different sub routines
echo check boot protocol
if /i "%targetBootProto%" == "static" (
  echo -- static

  echo call GStaticFunc to set up network
  call :GStaticFunc
  set /a exitCode=!errorlevel!
  echo -- GStaticFunc exited with !errorlevel!
  if /i not "!exitCode!" == "0" (
    echo -- fail to setup network by calling GStaticFunc
    set /a exitCode=9
  ) else (
    echo -- successfully setup network by calling GStaticFunc
    set /a exitCode=0
  )
) else (
  echo -- dhcp

  echo call GDhcpFunc to set up network
  call :GDhcpFunc
  set /a exitCode=!errorlevel!
  echo -- GDhcpFunc exited with !errorlevel!
  if /i not "!exitCode!" == "0" (
    echo -- fail to setup network by calling GDhcpFunc
    set /a exitCode=9
  ) else (
    echo -- successfully setup network by calling GDhcpFunc
    set /a exitCode=0
  )
)


@Rem
@Rem batch exit
@Rem
goto ExitLabel



:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GStaticFunc
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
echo call GStaticIpFunc to set static ip
call :GStaticIpFunc
set /a exitCode=!errorlevel!
echo -- GStaticIpFunc exited with !exitCode!
if /i not "!exitCode!" == "0" (
  set /a exitCode=7
  exit /b !exitCode!
)

if not defined targetDnsPrimary (
  set /a exitCode=0
  exit /b !exitCode!
)

echo call GStaticDnsFunc to set static dns
call :GStaticDnsFunc
set /a exitCode=!errorlevel!
echo -- GStaticDnsFunc exited with !exitCode!
if /i not "!exitCode!" == "0" (
  set /a exitCode=8
  exit /b !exitCode!
)
exit /b 0

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GDhcpFunc
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
echo call GDhcpIpFunc to set dhcp ip
call :GDhcpIpFunc
set /a exitCode=!errorlevel!
echo -- GDhcpIpFunc exited with !exitCode!
echo -- The exitCode is ommitted intentionally 
@REM if /i not "!exitCode!" == "0" (
@REM    set /a exitCode=7
@REM    exit /b !exitCode!
@REM )

@REM check if dns is static or dhcp
@REM if the targetDnsPrimary is defined then we set it manually
@REM otherwise set it to dhcp
echo check primary dns value to determine whether set dns to static or dhcp
if not defined targetDnsPrimary (
  echo -- primary dns is null, will set dns to dhcp
  goto GDhcpFunc_DhcpLabel
) else (
  echo -- primary dns exists, will set dns manually
  goto GDhcpFunc_StaticLabel
)

@REM set dns to dhcp
:GDhcpFunc_DhcpLabel
echo call GDhcpDnsFunc to set dns to dhcp
call :GDhcpDnsFunc
set /a exitCode=!errorlevel!
echo -- GDhcpDnsFunc exited with !exitCode!
if /i not "!exitCode!" == "0" (
  set /a exitCode=9
  exit /b !exitCode!
)
goto GDhcpFunc_EndLabel

@REM set dns to static
:GDhcpFunc_StaticLabel
echo call GStaticDnsFunc to set dns to dhcp
call :GStaticDnsFunc
set /a exitCode=!errorlevel!
echo -- GStaticDnsFunc exited with !exitCode!
if /i not "!exitCode!" == "0" (
  set /a exitCode=8
  exit /b !exitCode!
)

:GDhcpFunc_EndLabel
exit /b 0


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GStaticIpFunc
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
@REM echo !targetIfName! !targetIpAddr! !targetNetMask! !targetGateway!
echo call SetIpAddrByIfNameFunc to set static ip addr
if defined targetNetMask (
  set tmpNetMaskPlaceHolder=%targetNetMask%
) else (
  set tmpNetMaskPlaceHolder=0
)

if defined targetGateway (
  set tmpGatewayPlaceHolder=%targetGateway%
) else (
  set tmpGatewayPlaceHolder=0
)

@REM call :SetIpAddrByIfNameFunc %targetIfName% %targetIpAddr% %targetNetMask% %targetGateway%
call :SetIpAddrByIfNameFunc %targetIfName% %targetIpAddr% %tmpNetMaskPlaceHolder% %tmpGatewayPlaceHolder%
set /a exitCode=!errorlevel!
echo -- SetIpAddrByIfNameFunc exited with !exitCode!
if /i not "!exitCode!" == "0" (
  set /a exitCode=7
  exit /b !exitCode!
)
exit /b 0


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GDhcpIpFunc
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
call :SetIpToDhcpByIfNameFunc %targetIfName%
set /a exitCode=!errorlevel!
exit /b !exitCode!


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GStaticDnsFunc
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
call :SetDnsToStaticFunc %targetIfName% %targetDnsPrimary% %targetDnsSecondary%
set /a exitCode=!errorlevel!
exit /b !exitCode!


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GDhcpDnsFunc
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
call :SetDnsToDhcpFunc %targetIfName%
set /a exitCode=!errorlevel!
exit /b !exitCode!


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Do not edit lines below
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function SetIpAddrByHwAddr
::     set ip address to a given physical address
:: params
::     %1 [in]  hwaddr
::     %2 [in]  ip address
::     %3 [in]  netmask
::     %4 [in]  gateway
:: return
::     0, on success
::     1, otherwise
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:SetIpAddrByHwAddr
set localDestHwAddr=%1
set localDestIpAddr=%2
set localDestMask=%3
set localDestGateway=%4
set localDestIfName=
set /a nLocalFetchIpLimit=0
set localTmpResultIpAddr=

@REM check if the hwaddr can be found in the host system
echo check if the hwaddr %localDestHwAddr% exists in host system
call :IsHwaddrEnabledFunc %localDestHwAddr%
set /a exitCode=!errorlevel!
if /i "!exitCode!" == "0" (
  echo -- find the hwaddr in host system
) else (
  echo -- fail to find the hwaddr in host system
  exit /b 1
)

@REM get if name by hwaddr
echo get if name with given hwaddr %localDestHwAddr%
call :GetIfNameByHwAddrFunc %localDestHwAddr% localDestIfName
echo -- interface name: %localDestIfName%


@REM set ip addr to an interface
echo try to setup ip addr onto interface %localDestIfName%

if defined localDestMask (
  set tmpLocalNetMaskPlaceHolder=%localDestMask%
) else (
  set tmpLocalNetMaskPlaceHolder=0
)

if defined localDestGateway (
  set tmpLocalGatewayPlaceHolder=%localDestGateway%
) else (
  set tmpLocalGatewayPlaceHolder=0
)

@REM call :SetIpAddrByIfNameFunc %localDestIfName% %localDestIpAddr% %localDestMask% %localDestGateway%
call :SetIpAddrByIfNameFunc %localDestIfName% %localDestIpAddr% %tmpLocalNetMaskPlaceHolder% %tmpLocalGatewayPlaceHolder%
set /a exitCode=%errorlevel%
if /i "!exitCode!" == "0" (
  echo -- successfully set ip addr onto interface: %localDestIfName%
) else (
  echo -- fail to set ip addr onto interface: %localDestIfName%
  exit /b 1
)

@REM check the result ip address again
set /a nLocalFetchIpLimit=30
set localTmpResultIpAddr=
echo get ip addr for interface: %localDestIfName%
:LocalTryFetchIpLoop
echo -- try to fetch ip, %nLocalFetchIpLimit% seconds left
call :GetIpAddrByIfNameFunc "%localDestIfName%" localTmpResultIpAddr
if /i "%localTmpResultIpAddr%" == "" (
  choice /t 1 /n /d y > nul
  set /a nLocalFetchIpLimit="%nLocalFetchIpLimit% - 1"
  if /i not "!nLocalFetchIpLimit!" == "1" (
    goto LocalTryFetchIpLoop
  )
)
echo -- ip addr: %localTmpResultIpAddr%

@REM check if the ip is what we set in last step
echo check if current ip is what we set
if /i "%localTmpResultIpAddr%" == "%localDestIpAddr%" (
  echo -- yes
  exit /b 0
) else (
  echo -- no
  exit /b 1
)


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function IsHwaddrEnabledFunc
::     check if the given hwaddr exists in the host system
:: params
::     %1 [in]  hwaddr
:: return
::     0, if the the hwaddr is found in the host system,
::     1, otherwise
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:IsHwaddrEnabledFunc
set localHwAddr=%1
for /f "delims=" %%a in ('getmac /fo csv /nh /v') do (
  set line=%%a
  set line=!line:"=,!
  for /f "delims=, tokens=1,3" %%b in ("!line!") do (
    set tmpHwAddr=%%c
    if /i "!localHwAddr!" == "!tmpHwAddr!" (
      exit /b 0
    )
  )
)
exit /b 1


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function GetIfNameByHwAddrFunc
::     return interface name with given hwaddr
:: params
::     %1 [in]  hwaddr
::     %2 [out] interface name
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GetIfNameByHwAddrFunc
set localHwAddr=%1
for /f "delims=" %%a in ('getmac /fo csv /nh /v') do (
  set line=%%a
  set line=!line:"=,!
  for /f "delims=, tokens=1,3" %%b in ("!line!") do (
    set tmpIfName=%%b
    set tmpHwAddr=%%c
    if /i "!localHwAddr!" == "!tmpHwAddr!" (
      set candIfName=!tmpIfName!
    )
    @REM echo tmpIfName       = !tmpIfName!
    @REM echo tmpHwAddr       = !tmpHwAddr!
  )
)
if defined candIfName (
  set %~2="%candIfName%"
)

goto :eof


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function GetIpAddrByIfNameFunc
::     get ip with given interface name
:: params
::     %1 [in]  interface name
::     %2 [out] ipv4 address
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:GetIpAddrByIfNameFunc
set localIfName=%1
REM for /f "tokens=1,2*" %%i in ('netsh interface ipv4 show address name^=!localIfName! ^| findstr /C:"IP Address:"') do (
for /f "tokens=1,2*" %%i in ('netsh interface ipv4 show address name^=!localIfName! ^| findstr /C:"IP ַ:"') do (
  set %~2=%%k
)
goto :eof


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function SetIpAddrByIfNameFunc
::     set ip addr onto given interface name
:: parms
::     %1 [in]  interface name
::     %2 [in]  ip addr
::     %3 [in]  net mask, optional, can use 0 to set a placeholder
::     %4 [in]  default gateway, optional, can use 0 to set a placeholder
:: return
::     0, on success
::     1, otherwise
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:SetIpAddrByIfNameFunc
set localIfName=%~1
set localIpAddr=%2
set localMask=%3
set localGateway=%4
echo SetIpAddrByIfNameFunc: 
if /i "%localIfName%" == "" (
  echo -- illegal interface name
  exit /b 1
)
if not defined localIpAddr (
  echo -- illegal ip addr
  exit /b 1
)

if defined localMask (
  if /i "%localMask%" == "0" (
    set "localMask="
  )
)

if defined localGateway (
  if /i "%localGateway%" == "0" (
    set "localGateway="
  )
)

if not defined localGateway (
  if not defined localMask (
    goto SetIpAddrByIfNameFunc_Label
  ) else (
    goto SetIpAddrByIfNameFunc_Label_Mask
  )
) else (
  if not defined localMask (
    goto SetIpAddrByIfNameFunc_Label_Gateway
  ) else (
    goto SetIpAddrByIfNameFunc_Label_Mask_Gateway
  )
)

:SetIpAddrByIfNameFunc_Label
netsh interface ipv4 set address name="%localIfName%" ^
                                 source=static ^
                                 address=%localIpAddr%
set /a exitCode=!errorlevel!
goto SetIpAddrByIfNameFunc_Label_End

:SetIpAddrByIfNameFunc_Label_Mask
netsh interface ipv4 set address name="%localIfName%" ^
                                 source=static ^
                                 address=%localIpAddr% ^
                                 mask=%localMask%
set /a exitCode=!errorlevel!
goto SetIpAddrByIfNameFunc_Label_End

:SetIpAddrByIfNameFunc_Label_Gateway
netsh interface ipv4 set address name="%localIfName%" ^
                                 source=static ^
                                 address=%localIpAddr% ^
                                 gateway=%localGateway%
set /a exitCode=!errorlevel!
goto SetIpAddrByIfNameFunc_Label_End

:SetIpAddrByIfNameFunc_Label_Mask_Gateway
netsh interface ipv4 set address name="%localIfName%" ^
                                 source=static ^
                                 address=%localIpAddr% ^
                                 mask=%localMask% ^
                                 gateway=%localGateway%
set /a exitCode=!errorlevel!
goto SetIpAddrByIfNameFunc_Label_End

:SetIpAddrByIfNameFunc_Label_End
exit /b !exitCode!


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function SetIpToDhcpByIfNameFunc
::     set ip source to dhcp for the given interface
:: parms
::     %1 [in]  interface name
:: return
::     0, on success
::     1, otherwise
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:SetIpToDhcpByIfNameFunc
set localIfName=%~1
netsh interface ipv4 set address name="%localIfName%" source=dhcp
set /a exitCode=!errorlevel!
exit /b !exitCode!


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function SetDnsToStaticFunc
::     set statc dns servers for given interface
:: parms
::     %1 [in]  interface name
::     %2 [in]  primary dns
::     %3 [in]  optional, secondary dns
:: return
::     0, on success
::     1, otherwise
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:SetDnsToStaticFunc
set localIfName=%~1
set localDnsPrimary=%2
set localDnsSecondary=%3
echo SetDnsToStaticFunc
if /i "%localIfName%" == "" (
  echo -- localIfName: illegal argument
  exit /b 1
)
if /i "%localDnsPrimary%" == "" (
  echo -- localDnsPrimary: illegal argument
  exit /b 1
)

@REM remove dns list
netsh interface ipv4 set dnsservers name="%localIfName%" source=static address=none validate=no
if errorlevel 1 (
  echo -- fail to remove dns list
  exit /b 1
)

@REM set primary dns
netsh interface ipv4 set dnsservers name="%localIfName%" source=static address=%localDnsPrimary% register=primary validate=no
if errorlevel 1 (
  echo -- fail to set up primary dns
  exit /b 1
)

if /i "%localDnsSecondary%" == "" (
  exit /b 0
)

netsh interface ipv4 add dnsservers name="%localIfName%" address=%localDnsSecondary% index=2 validate=no
if errorlevel 1 (
  echo -- fail to set up secondary dns
  exit /b 1
)

exit /b 0


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Function SetDnsToDhcpFunc
::     set dns servers to be dhcp for given interface
:: parms
::     %1 [in]  interface name
:: return
::     0, on success
::     1, otherwise
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:SetDnsToDhcpFunc
set localIfName=%~1
echo SetDnsToDhcpFunc
if /i "%localIfName%" == "" (
  echo -- localIfName: illegal argument
  exit /b 1
)

@REM set dns to dhcp
netsh interface ipv4 set dnsservers name="%localIfName%" source=dhcp
if errorlevel 1 (
  echo -- fail to set dns to dhcp
  exit /b 1
)
exit /b 0


:MissingArgument
echo -- missing argument
echo.
goto Usage


:Usage
echo Setup network with specified configuration
echo.
echo Usage: %thisScript% /mac ^<HwAddr^> [/ip ^<IpAddr^> [/mask ^<NetMask^>] [/gw ^<Gateway^>] [/dns ^<PrimaryDns^>] [/dns2 ^<SecondaryDns^>]]
echo.
echo set up ip and dns to use dhcp
echo   %thisScript% /mac 74-D4-35-05-26-58
echo.
echo set up static ip and leave dns alone
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /mask 255.255.255.0
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /gw 192.168.0.1
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /mask 255.255.255.0 /gw 192.168.0.1
echo.
echo set up static ip and static dns
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /dns 192.168.0.1
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /mask 255.255.255.0 /dns 192.168.0.1
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /gw 192.168.0.1 /dns 192.168.0.1
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /mask 255.255.255.0 /gw 192.168.0.1 /dns 192.168.0.1
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /dns 192.168.0.1 /dns2 192.168.0.2
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /mask 255.255.255.0 /dns 192.168.0.1 /dns2 192.168.0.2
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /gw 192.168.0.1 /dns 192.168.0.1 /dns2 192.168.0.2
echo   %thisScript% /mac 74-D4-35-05-26-58 /ip 192.168.0.33 /mask 255.255.255.0 /gw 192.168.0.1 /dns 192.168.0.1 /dns2 192.168.0.2
echo.
set /a exitCode=1


:ExitLabel
(
  echo.
  echo -- %date% %time%
  echo.
  echo -- batch exited code: !exitCode!
  endlocal
  exit /b !exitCode!
)


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Sample I
:: Set static ip and dns
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
:: C:\Users\z>ifsetup3.bat /mac 08-00-27-6C-35-69 /ip 192.168.0.35 /dns 192.168.0.1 /dns2 192.168.0.5 /gw 192.168.0.6 /mask 255.255.255.0
:: option parsed from command-line
:: -- dBootProto=static
:: -- dHwAddr=08-00-27-6C-35-69
:: -- dIpAddr=192.168.0.35
:: -- dNetMask=255.255.255.0
:: -- dGateway=192.168.0.6
:: -- dPrimaryDns=192.168.0.1
:: -- dSecondaryDns=192.168.0.5
:: check installed language
:: -- installed language, simplified Chinese[0x0804]
:: check system language
:: -- system language, simplified Chinese[zh-CN]
:: check Windows version
:: -- system version: Windows7
:: check if hwaddr value is set
:: -- hwaddr value: 08-00-27-6C-35-69
:: try to get interface name
:: -- target network interface name: " 2"
:: check use static or dhcp configuration
:: -- boot protocol is about to be set to static
:: check boot protocol
:: -- static
:: call GStaticFunc to set up network
:: call GStaticIpFunc to set static ip
:: call SetIpAddrByIfNameFunc to set static ip addr
:: SetIpAddrByIfNameFunc:
:: 
:: -- SetIpAddrByIfNameFunc exited with 0
:: -- GStaticIpFunc exited with 0
:: call GStaticDnsFunc to set static dns
:: SetDnsToStaticFunc
:: 
::
::
:: -- GStaticDnsFunc exited with 0
:: -- GStaticFunc exited with 0
:: -- successfully setup network by calling GStaticFunc
:: 
:: Thu 12/27/2018 15:58:56.32
::
:: -- batch exited code: 0
::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Sample II
:: Set ip and dns to dhcp
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: 
:: C:\Users\z>ifsetup3.bat /mac 08-00-27-6C-35-69
:: option parsed from command-line
:: -- dBootProto=dhcp
:: -- dHwAddr=08-00-27-6C-35-69
:: -- dIpAddr=
:: -- dNetMask=
:: -- dGateway=
:: -- dPrimaryDns=
:: -- dSecondaryDns=
:: check installed language
:: -- installed language, simplified Chinese[0x0804]
:: check system language
:: -- system language, simplified Chinese[zh-CN]
:: check Windows version
:: -- system version: Windows7
:: check if hwaddr value is set
:: -- hwaddr value: 08-00-27-6C-35-69
:: try to get interface name
:: -- target network interface name: " 2"
:: check use static or dhcp configuration
:: -- boot protocol is about to be set to dhcp
:: check boot protocol
:: -- dhcp
:: call GDhcpFunc to set up network
:: call GDhcpIpFunc to set dhcp ip
:: 
:: -- GDhcpIpFunc exited with 0
:: -- The exitCode is ommitted intentionally
:: check primary dns value to determine whether set dns to static or dhcp
:: -- primary dns is null, will set dns to dhcp
:: call GDhcpDnsFunc to set dns to dhcp
:: SetDnsToDhcpFunc
:: 
:: -- GDhcpDnsFunc exited with 0
:: -- GDhcpFunc exited with 0
:: -- successfully setup network by calling GDhcpFunc
:: 
:: Thu 12/27/2018 15:58:56.32
::
:: -- batch exited code: 0
:: 


:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:::: End
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::