/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QGraphicsLinearLayout>
#include <QSwipeGesture>
#include <MLabel>
#include <MButton>
#include <MDebug>

#include "misc.h"
#include "cleandiskpage.h"


#define DISKUSAGE_PROG          "/usr/sbin/tc/get_disk_usage.sh"
#define CLEANDISK_PROG          "/usr/sbin/tc/clean_disk.sh"


CleanDiskPage::CleanDiskPage()
{
    mDebug("CleanDiskPage::CleanDiskPage()");

    setObjectName("cleandisk_page");
}

CleanDiskPage::~CleanDiskPage()
{
    mDebug("CleanDiskPage::~CleanDiskPage");
}

void CleanDiskPage::createContent()
{
    mDebug("CleanDiskPage::createContent()");

//    MApplicationPage::createContent();
//    setStyleName("CommonApplicationPage");

    setComponentsDisplayMode(NavigationBar, MApplicationPageModel::Hide);
    setComponentsDisplayMode(HomeButton, MApplicationPageModel::Hide);
    setComponentsDisplayMode(EscapeButton, MApplicationPageModel::Hide);

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, centralWidget());
    QGraphicsLinearLayout *lyClean = new QGraphicsLinearLayout(Qt::Horizontal);

    layout->addStretch();

    lbUsage = new MLabel("");
    lbUsage->setObjectName("cleandisk_lb_usage");
    layout->addItem(lbUsage);

    btnDeep = new MButton();
    btnDeep->setObjectName("cleandisk_btn_deep");
    btnDeep->setViewType(MButton::checkboxType);
    btnDeep->setCheckable(true);
    btnDeep->setChecked(false);
    lyClean->addItem(btnDeep);
    lyClean->setAlignment(btnDeep, Qt::AlignVCenter);

    MLabel *lbDeep = new MLabel(qtTrId("deep"));
    lbDeep->setObjectName("cleandisk_lb_deep");
    lyClean->addItem(lbDeep);
    lyClean->setAlignment(lbDeep, Qt::AlignVCenter);

    btnClean = new MButton(qtTrId("Clean up"));
    btnClean->setObjectName("cleandisk_btn_clean");
    lyClean->addItem(btnClean);
    lyClean->setAlignment(btnClean, Qt::AlignVCenter);

    layout->addItem(lyClean);

    layout->addStretch();

    connect(btnClean, SIGNAL(clicked()), this, SLOT(onBtnCleanClicked()));

    ShowDiskUsage();
}

void CleanDiskPage::swipeGestureEvent(QGestureEvent *event, QSwipeGesture *gesture)
{
    mDebug("CleanDiskPage::swipeGestureEvent()") << "State:" << gesture->state() << "HDir:" << gesture->horizontalDirection() << "VDir:" << gesture->verticalDirection();

    event->accept(gesture);

    if (gesture->state() == Qt::GestureFinished)
    {
        if (gesture->horizontalDirection() == QSwipeGesture::Right)
        {
            disappear();
        }
    }
}

void CleanDiskPage::ShowDiskUsage()
{
    int exitCode;
    QProcess psUsage;
    QString cmd(DISKUSAGE_PROG);

    psUsage.start(cmd);
    psUsage.waitForFinished();
    exitCode = psUsage.exitCode();
    if ( exitCode != 0 )
    {
        lbUsage->setText(qtTrId("Can not get disk usage information."));
        return;
    }

    QString msg = psUsage.readAll();
    QStringList info = msg.split(":");
    QString usage;
    usage = qtTrId("disk usage info [ uesd: %1 MB, free: %2 MB ]").arg(info.at(0).toInt()).arg(info.at(1).toInt());
    lbUsage->setText(usage);
}

void CleanDiskPage::onBtnCleanClicked()
{
    int exitCode;
    bool bDeep;
    QString cmd(CLEANDISK_PROG);
    QStringList lstArgs;

    bDeep = btnDeep->isChecked();

    lstArgs.append(bDeep ? "1" : "0");
    exitCode = QProcess::execute(cmd, lstArgs);
    if ( exitCode != 0 )
    {
        displayBanner(this, qtTrId("Clean up disk failed."));
        return;
    }

    displayBanner(this, qtTrId("Clean up disk successfully."));

    ShowDiskUsage();
}

