/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QGraphicsLinearLayout>
#include <QPropertyAnimation>
#include <MLabel>
#include <MProgressIndicator>
#include <MDebug>

#include "indicatordialog.h"


IndicatorDialog::IndicatorDialog(const QString &title, const QString &text, int timeout)
  : MDialog(title, M::NoStandardButton)
{
    MLabel *label;
    MProgressIndicator *bar;
    QPropertyAnimation *barValueAnimation;

    mDebug("IndicatorDialog::IndicatorDialog");

    setObjectName("progressIndicatorDialog");

    label = new MLabel(text);
    label->setObjectName("progressIndicatorLabel");
    label->setStyleName("CommonBodyTextInverted");
    label->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);

    bar = new MProgressIndicator(0, MProgressIndicator::barType);
    bar->setObjectName("progressIndicatorBar");
    bar->setStyleName("CommonProgressBar");
    bar->setRange(0, 1000);
    bar->setValue(0);

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    layout->addItem(label);
    layout->addItem(bar);

    centralWidget()->setLayout(layout);

    barValueAnimation = new QPropertyAnimation(bar, "value", this);
    barValueAnimation->setEasingCurve(QEasingCurve::OutInQuad);
    barValueAnimation->setStartValue(bar->minimum());
    barValueAnimation->setEndValue(bar->maximum());
    barValueAnimation->setDuration(timeout);
    barValueAnimation->start();
    connect(barValueAnimation, SIGNAL(finished()), this, SLOT(Timedout()));
}

IndicatorDialog::~IndicatorDialog()
{
    mDebug("IndicatorDialog::~IndicatorDialog");
}


void IndicatorDialog::Timedout()
{
    mDebug("IndicatorDialog::Timedout");

    done(MDialog::Accepted);
}

