/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include "inputpanel.h"


static char  LetterMap[] =
{
    'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P',
    'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L',
    'Z', 'X', 'C', 'V', 'B', 'N', 'M',
    ',', ' ', '.',
    14, 15, 8, 10
};

static char  NumberMap[] =
{
    '1', '2', '3', '4', '5', '6', '7', '8', '9', '0',
    '*', '#', '+', '-', '=', '(', ')', '?', '!',
    '@', '~', '/', '\\', '\'', ';', ':',
    '$', ' ', '%',
    14, 15, 8, 10
};

InputPanel::InputPanel()
    : QWidget(0, Qt::Tool | Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint),
      lastFocusedWidget(0), capslock(false), numlock(false)
{
    form.setupUi(this);
    form.panelButton_capslock->setIcon(QIcon(":/res/icon-vkb-capslock.svg"));
    form.panelButton_bs->setIcon(QIcon(":/res/icon-vkb-backspace.svg"));
    form.panelButton_enter->setIcon(QIcon(":/res/icon-vkb-enter.svg"));

    pButtons[0] = form.panelButton_q;
    pButtons[1] = form.panelButton_w;
    pButtons[2] = form.panelButton_e;
    pButtons[3] = form.panelButton_r;
    pButtons[4] = form.panelButton_t;
    pButtons[5] = form.panelButton_y;
    pButtons[6] = form.panelButton_u;
    pButtons[7] = form.panelButton_i;
    pButtons[8] = form.panelButton_o;
    pButtons[9] = form.panelButton_p;
    pButtons[10] = form.panelButton_a;
    pButtons[11] = form.panelButton_s;
    pButtons[12] = form.panelButton_d;
    pButtons[13] = form.panelButton_f;
    pButtons[14] = form.panelButton_g;
    pButtons[15] = form.panelButton_h;
    pButtons[16] = form.panelButton_j;
    pButtons[17] = form.panelButton_k;
    pButtons[18] = form.panelButton_l;
    pButtons[19] = form.panelButton_z;
    pButtons[20] = form.panelButton_x;
    pButtons[21] = form.panelButton_c;
    pButtons[22] = form.panelButton_v;
    pButtons[23] = form.panelButton_b;
    pButtons[24] = form.panelButton_n;
    pButtons[25] = form.panelButton_m;
    pButtons[26] = form.panelButton_comma;
    pButtons[27] = form.panelButton_sp;
    pButtons[28] = form.panelButton_dot;
    pButtons[29] = form.panelButton_capslock;
    pButtons[30] = form.panelButton_num;
    pButtons[31] = form.panelButton_bs;

    for (int i = 0; i < 32; i++)
    {
        signalMapper.setMapping(pButtons[i], pButtons[i]);
        connect(pButtons[i], SIGNAL(clicked()), &signalMapper, SLOT(map()));
    }
    
    connect(qApp, SIGNAL(focusChanged(QWidget*,QWidget*)), this, SLOT(saveFocusWidget(QWidget*,QWidget*)));
    connect(&signalMapper, SIGNAL(mapped(QWidget*)), this, SLOT(buttonClicked(QWidget*)));
}


bool InputPanel::event(QEvent *e)
{
    switch (e->type()) {
    case QEvent::WindowActivate:
        if (lastFocusedWidget)
            lastFocusedWidget->activateWindow();
        break;
    default:
        break;
    }

    return QWidget::event(e);
}

void InputPanel::saveFocusWidget(QWidget * /*oldFocus*/, QWidget *newFocus)
{
    if (newFocus != 0 && !this->isAncestorOf(newFocus)) {
        lastFocusedWidget = newFocus;
    }
}

void InputPanel::buttonClicked(QWidget *w)
{
    QString str = qvariant_cast<QString>(w->property("buttonValue"));
    int keyId = str.toInt(NULL, 10);
    QChar  chr;

    if (keyId == 29 )
    {
        capslock = !capslock;
        if (!numlock)
        {
            for (int i = 0; i < 29; i++)
            {
                if (capslock || (i >=26))
                {
                    pButtons[i]->setText(QChar(LetterMap[i]));
                }
                else
                {
                    pButtons[i]->setText(QChar(LetterMap[i] + 32));
                }
            }
        }
        return;
    }

    if (keyId == 30)
    {
        numlock= !numlock;
        for (int i = 0; i < 29; i++)
        {
            if (numlock)
            {
                pButtons[i]->setText(QChar(NumberMap[i]));
            }
            else
            {
                if (capslock || (i >=26))
                {
                    pButtons[i]->setText(QChar(LetterMap[i]));
                }
                else
                {
                    pButtons[i]->setText(QChar(LetterMap[i] + 32));
                }
            }
        }
        return;
    }

    if (numlock)
    {
        chr = QChar(NumberMap[keyId]);
    }
    else
    {
        if (capslock || (keyId >=26))
        {
            chr = QChar(LetterMap[keyId]);
        }
        else
        {
            chr = QChar(LetterMap[keyId] + 32);
        }
    }
    emit characterGenerated(chr);
}

