/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QtCore>
#include <QGraphicsView>
#include <MDebug>
#include <mnamespace.h>

#include "inputpanelcontext.h"


InputPanelContext::InputPanelContext()
{
    inputPanel = new InputPanel;
    connect(inputPanel, SIGNAL(characterGenerated(QChar)), SLOT(sendCharacter(QChar)));
}

InputPanelContext::~InputPanelContext()
{
    delete inputPanel;
}


bool InputPanelContext::filterEvent(const QEvent* event)
{
    if (event->type() == QEvent::RequestSoftwareInputPanel) {
        mDebug("RequestSoftwareInputPanel");
        updatePosition();
        inputPanel->show();
        return true;
    } else if (event->type() == QEvent::CloseSoftwareInputPanel) {
        mDebug("CloseSoftwareInputPanel");
        inputPanel->hide();
        return true;
    }
    return false;
}

QString InputPanelContext::identifierName()
{
    return "InputPanelContext";
}

void InputPanelContext::reset()
{
}

bool InputPanelContext::isComposing() const
{
    return false;
}

QString InputPanelContext::language()
{
    return "en_US";
}

void InputPanelContext::sendCharacter(QChar character)
{
    QPointer<QWidget> widget = QInputContext::focusWidget();
    int key;

    if (!widget)
        return;

    key = character.unicode();
    if (key == 8)
    {
        key = Qt::Key_Backspace;
    }
    QKeyEvent keyPress(QEvent::KeyPress, key, Qt::NoModifier, QString(character));
    QApplication::sendEvent(widget, &keyPress);

//    QKeyEvent keyRelease(QEvent::KeyPress, key, Qt::NoModifier, QString());
//    QApplication::sendEvent(widget, &keyRelease);
}


void InputPanelContext::updatePosition()
{
    QWidget *focused = focusWidget();
    if (!focused)
    {
        return;
    }

    QRect deskRect = QApplication::desktop()->availableGeometry();

    Qt::InputMethodQuery query = static_cast<Qt::InputMethodQuery>(M::PreeditRectangleQuery);
    QRect preeditRect = focused->inputMethodQuery(query).toRect();

    int panelX, panelY;
    QRect panelRect = inputPanel->rect();

    panelX = preeditRect.x() - panelRect.width() / 2;
    if ( panelX < 0 )
    {
        panelX = 20;
    }
    else if ((panelX + panelRect.width()) > deskRect.width())
    {
        panelX = deskRect.width() - panelRect.width() - 20;
    }
    if (preeditRect.y() < (deskRect.height() / 2))
    {
        panelY = preeditRect.y() + 20;
    }
    else
    {
        panelY = preeditRect.y() - panelRect.height() - 20;
    }

    inputPanel->move(panelX, panelY);
}

