/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef LOGINPAGE_H
#define LOGINPAGE_H


#include <QProcess>
#include <QSettings>
#include <QPointer>
#include <MApplicationPage>

class MButton;
class MLabel;
class MTextEdit;
class MImageWidget;
class MProgressIndicator;
class IndicatorDialog;
class InitDiskPage;
class UpdateFwPage;
class RegMachPage;
class CleanDiskPage;
class ProfilePage;
class SettingPage;


class LoginPage : public MApplicationPage
{
    Q_OBJECT

public:
    LoginPage();
    virtual ~LoginPage();

    virtual void createContent();

protected:
    virtual bool event(QEvent *e);

private:
    void enableInput();
    void disableInput();

    void OnRsvdStatus(int nStatus);
    void OnDiskStatus(int nStatus);
    void OnNetStatus(int nStatus);

public slots:
    Q_SCRIPTABLE int AutoBootOS(const QString &uname, const QString &passwd, int delay, const QString &osname, int osid);
    Q_SCRIPTABLE int QuickBootOS(int osid, const QString &orgos, const QString &curos, const QString &uname);
    Q_SCRIPTABLE int HwStatus(int nHw, int nStatus);
    Q_SCRIPTABLE int ShowQRImg(const QString &imgfile);
    Q_SCRIPTABLE int InitDisk(int nImgPartMB);
    Q_SCRIPTABLE int UpdateFw();
    Q_SCRIPTABLE int RegMachine();
    Q_SCRIPTABLE int RegMachineAuto(const QString &cname);
    Q_SCRIPTABLE int MachineName(const QString &cname);

signals:
    void AutoBootEvent(const QString &uname, const QString &passwd, int delay, const QString &osname, int osid);
    void QuickBootEvent(int osid, const QString &orgos, const QString &curos, const QString &uname);
    void HwStatusEvent(int nHw, int nStatus);
    void ShowQRImgEvent(const QString &imgfile);
    void InitDiskEvent(int nImgPartMB);
    void UpdateFwEvent();
    void RegMachineEvent();
    void RegMachineAutoEvent(const QString &cname);
    void MachineNameEvent(const QString &cname);

private slots:
    void onTabPressed();
    void onBtnLoginClicked();
    void onBtnDiskClicked();
    void onBtnProfileClicked();
    void onBtnSettingClicked();
    void onBtnPowerClicked();
    void onSettingPageFinished();
    void onPsTcSyncError(QProcess::ProcessError errCode);
    void onPsTcSyncFinished(int exitCode, QProcess::ExitStatus exitStatus);
    void onAutoBootEvent(const QString &uname, const QString &passwd, int delay, const QString &osname, int osid);
    void onQuickBootEvent(int osid, const QString &orgos, const QString &curos, const QString &uname);
    void onHwStatusEvent(int nHw, int nStatus);
    void onShowQRImgEvent(const QString &imgfile);
    void onInitDiskEvent(int nImgPartMB);
    void onUpdateFwEvent();
    void onRegMachineEvent();
    void onRegMachineAutoEvent(const QString &cname);
    void onMachineNameEvent(const QString &cname);

    void onAppeared();

private:
    MLabel *lbClientVer;
    MLabel *lbMachineName;

    MImageWidget *imgQRCode;
    MButton *btnHdStatus;
    MButton *btnNetStatus;
    MButton *btnRemID;
    MButton *btnOffMode;
    MButton *btnLogin;
    MButton *btnProfile;
    MButton *btnSetting;
    MButton *btnPower;
    MTextEdit *teUser;
    MTextEdit *tePass;
    MProgressIndicator *spinner;
    QProcess psTcSync;

    int nDiskStatus;
    bool bDiskBusy;

    QPointer<InitDiskPage> pgInitDisk;
    QPointer<UpdateFwPage> pgUpdateFw;
    QPointer<RegMachPage> pgRegMach;

    QString clientVer;

    QString userName;
    QString servIP;
    QPointer<CleanDiskPage>  pgCleanDisk;
    QPointer<ProfilePage> pgProfile;
    QPointer<SettingPage> pgSetting;

    int nBootOsId;
};

#endif // LOGINPAGE_H
