/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <MApplication>
#include <MApplicationWindow>
#include <MSceneManager>
#include <QtDBus>

#include "inputpanelcontext.h"
#include "loginpage.h"


#define TC_SERV_NAME            "TC.NOTIFIER"

class MyProxyStyle : public QProxyStyle
{
public:
    int styleHint(StyleHint hint, const QStyleOption *opt, const QWidget *widget, QStyleHintReturn *returnData) const
    {
        if (hint == QStyle::SH_RequestSoftwareInputPanel)
        {
            return QStyle::RSIP_OnMouseClickAndAlreadyFocused;
        }
        return QProxyStyle::styleHint(hint, opt, widget, returnData);
    }
};


void custMsgHandler(QtMsgType type, const char *msg)
{
    QString txt;

    switch (type)
    {
        case QtDebugMsg:
            txt = QString("Debug: %1").arg(msg);
            break;

        case QtWarningMsg:
            txt = QString("Warning: %1").arg(msg);
            break;

        case QtCriticalMsg:
            txt = QString("Critical: %1").arg(msg);
            break;

        case QtFatalMsg:
            txt = QString("Fatal: %1").arg(msg);
            abort();
    }

    QFile outFile("/var/log/tcentry.log");
    outFile.open(QIODevice::WriteOnly | QIODevice::Append);
    QTextStream ts(&outFile);

    ts << QTime::currentTime().toString(Qt::ISODate) << txt << endl;
}

int main(int argc, char **argv)
{
    MApplication  app(argc, argv);

    qInstallMsgHandler(custMsgHandler);

    app.setStyle(new MyProxyStyle);

    if ( QDBusConnection::sessionBus().isConnected() )
    {
        QDBusConnection::sessionBus().registerService(TC_SERV_NAME);
    }

    // qt_cleanup () at kernel/qapplication_qws.cpp will do "delete QApplicationPrivate::inputContext"
    InputPanelContext *ic = new InputPanelContext;
    app.setInputContext(ic);

    MApplicationWindow  w;
    w.show();

    LoginPage pgLogin;
    pgLogin.appear(&w, MSceneWindow::DestroyWhenDismissed);

    return app.exec();
}
