/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#ifndef OSLISTPAGE_H
#define OSLISTPAGE_H


#include <QAbstractAnimation>
#include <MApplicationPage>
#include <MObjectMenu>

class MImageWidget;
class QState;
class QStateMachine;
class QParallelAnimationGroup;


typedef struct _OsListEntry
{
    QString id;
    QString desc;
    QString cur_img;
    QString org_img;
    QImage icon;
    QString latest_ver;
    int cur_per;
}
OsListEntry;

typedef struct _ActListEntry
{
    QString desc;
    QString cmd;
    int permission;
    MAction *action;
}
ActListEntry;


class OsListObjectMenu : public MObjectMenu
{
    Q_OBJECT

public:
    OsListObjectMenu();
    virtual ~OsListObjectMenu();

    bool Init(const QString &name);
    void checkPermission(int cur_per);
    void clickDefaultAct();

public slots:
    void actListItem();

signals:
    void onActClicked(const QString &desc, const QString &cmd);

private:
    bool LoadFromFile(const QString &name);

private:
    QVector<ActListEntry*> actListEntries;
};

class OSListPage : public MApplicationPage
{
    Q_OBJECT

public:
    OSListPage(const QString &strName, const QString &serv, int osid);
    virtual ~OSListPage();

    virtual void createContent();

protected:
    virtual void enterDisplayEvent();
    virtual void swipeGestureEvent(QGestureEvent *event, QSwipeGesture *gesture);
    virtual void tapGestureEvent(QGestureEvent *event, QTapGesture* gesture);
    virtual void tapAndHoldGestureEvent(QGestureEvent *event, QTapAndHoldGesture *gesture);

private:
    bool LoadFromFile(const QString &name);
    int GetTappedIndex(const QPointF &pos);
    void BootOsAni();

signals:
    void aniTriggerShow();
    void aniTriggerHide();

public slots:
    void delayAutoBoot();

private slots:
    void actClicked(const QString &desc, const QString &cmd);
    void aniFinished();
    void aniStateChanged(QAbstractAnimation::State newState, QAbstractAnimation::State oldState);

private:
    QString userName;
    QString server;

    QParallelAnimationGroup *aniGroupOs;
    QStateMachine *stMachOs;
    QState *stRootOs;
    QState *stShowOs;
    QState *stHideOs;

    int curIndex;
    QVector<OsListEntry*> osListEntries;
    QImage osDefaultIcon;

    QVector<MImageWidget*> lstOsWidget;

    OsListObjectMenu *objMenu;

    int nBootOsId;

    int aniState;
};


#endif // OSLISTPAGE_H
