/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QGraphicsLinearLayout>
#include <QSwipeGesture>
#include <MContainer>
#include <MTextEdit>
#include <MButton>
#include <MDebug>

#include "misc.h"
#include "profilepage.h"


#define TCCHGPWD_PROG           "/usr/sbin/tc/tc-chgpwd.sh"


ProfilePage::ProfilePage(const QString &serverIP)
    : servIP(serverIP)
{
    mDebug("ProfilePage::ProfilePage()");

    setObjectName("profile_page");
}

ProfilePage::~ProfilePage()
{
    mDebug("ProfilePage::~ProfilePage");
}

void ProfilePage::createContent()
{
    mDebug("ProfilePage::createContent()");

//    MApplicationPage::createContent();
//    setStyleName("CommonApplicationPage");

    setComponentsDisplayMode(NavigationBar, MApplicationPageModel::Hide);
    setComponentsDisplayMode(HomeButton, MApplicationPageModel::Hide);
    setComponentsDisplayMode(EscapeButton, MApplicationPageModel::Hide);

    MContainer *ctnChgPwd = new MContainer(this);
    ctnChgPwd->setStyleName("CommonContainer");
    ctnChgPwd->setTitle(qtTrId("change user's password"));

    QGraphicsLinearLayout *lyOuter = new QGraphicsLinearLayout(Qt::Vertical, centralWidget());
    QGraphicsLinearLayout *lyChgPwd = new QGraphicsLinearLayout(Qt::Horizontal, ctnChgPwd->centralWidget());


    teUser = new MTextEdit;
    teUser->setObjectName("chgpwd_te_user");
    teUser->setMaxLength(30);
    teUser->setPrompt(qtTrId("user name"));
    teUser->setPreferredWidth(200);
    teUser->setTextInteractionFlags(Qt::TextEditable);
    lyChgPwd->addItem(teUser);

    teOldPwd = new MTextEdit;
    teOldPwd->setObjectName("chgpwd_te_pwd");
    teOldPwd->setMaxLength(32);
    teOldPwd->setPrompt(qtTrId("old password"));
    teOldPwd->setEchoMode(MTextEditModel::Password);
    teOldPwd->setPreferredWidth(200);
    teOldPwd->setTextInteractionFlags(Qt::TextEditable);
    lyChgPwd->addItem(teOldPwd);

    teNewPwd1 = new MTextEdit;
    teNewPwd1->setObjectName("chgpwd_te_pwd");
    teNewPwd1->setMaxLength(32);
    teNewPwd1->setPrompt(qtTrId("new password"));
    teNewPwd1->setEchoMode(MTextEditModel::Password);
    teNewPwd1->setPreferredWidth(200);
    teNewPwd1->setTextInteractionFlags(Qt::TextEditable);
    lyChgPwd->addItem(teNewPwd1);

    teNewPwd2 = new MTextEdit;
    teNewPwd1->setObjectName("chgpwd_te_pwd");
    teNewPwd2->setMaxLength(32);
    teNewPwd2->setPrompt(qtTrId("type again"));
    teNewPwd2->setEchoMode(MTextEditModel::Password);
    teNewPwd2->setPreferredWidth(200);
    teNewPwd2->setTextInteractionFlags(Qt::TextEditable);
    lyChgPwd->addItem(teNewPwd2);

    btnChgPwd = new MButton(qtTrId("Change"));
    btnChgPwd->setObjectName("setting_btn");
    btnChgPwd->setPreferredWidth(200);
    lyChgPwd->addItem(btnChgPwd);


    lyOuter->addStretch();
    lyOuter->addItem(ctnChgPwd);


    scene()->setFocusItem(teUser, Qt::ActiveWindowFocusReason);

    connect(teUser, SIGNAL(tabPressed()), this, SLOT(onChgPwdTabPressed()));
    connect(teOldPwd, SIGNAL(tabPressed()), this, SLOT(onChgPwdTabPressed()));
    connect(teNewPwd1, SIGNAL(tabPressed()), this, SLOT(onChgPwdTabPressed()));
    connect(teNewPwd2, SIGNAL(tabPressed()), this, SLOT(onChgPwdTabPressed()));

    connect(teNewPwd2, SIGNAL(returnPressed()), btnChgPwd, SIGNAL(clicked()));
    connect(btnChgPwd, SIGNAL(clicked()), this, SLOT(onBtnChgPwdClicked()));
}

void ProfilePage::swipeGestureEvent(QGestureEvent *event, QSwipeGesture *gesture)
{
    mDebug("ProfilePage::swipeGestureEvent()") << "State:" << gesture->state() << "HDir:" << gesture->horizontalDirection() << "VDir:" << gesture->verticalDirection();

    event->accept(gesture);

    if (gesture->state() == Qt::GestureFinished)
    {
        if (gesture->horizontalDirection() == QSwipeGesture::Right)
        {
            disappear();
        }
    }
}

void ProfilePage::onChgPwdTabPressed()
{
    QGraphicsItem *item;

    mDebug("onChgPwdTabPressed");

    item = centralWidget()->focusItem();

    if (item == teUser)
    {
        scene()->setFocusItem(teOldPwd, Qt::TabFocusReason);
    }
    else if(item == teOldPwd)
    {
        scene()->setFocusItem(teNewPwd1, Qt::TabFocusReason);
    }
    else if(item == teNewPwd1)
    {
        scene()->setFocusItem(teNewPwd2, Qt::TabFocusReason);
    }
    else if(item == teNewPwd2)
    {
        scene()->setFocusItem(teUser, Qt::TabFocusReason);
    }
}

void ProfilePage::onBtnChgPwdClicked()
{
    int exitCode;
    QString strError;
    QStringList lstArgs;

    if (teUser->text().isEmpty())
    {
        strError = qtTrId("Please input user name.");
    }
    else if (teOldPwd->text().isEmpty())
    {
        strError = qtTrId("Please input old password.");
    }
    else if (teNewPwd1->text().isEmpty())
    {
        strError = qtTrId("Please input new password.");
    }
    else if (teNewPwd2->text().isEmpty())
    {
        strError = qtTrId("Please input new password again.");
    }
    else if (teNewPwd1->text().compare(teNewPwd2->text()))
    {
        strError = qtTrId("The passwords did not match.");
    }
    else if ((teNewPwd1->text().length() < 10) || (teNewPwd1->text().length() > 32))
    {
        strError = qtTrId("The password length must between 10~32.");
    }

    if (!strError.isEmpty())
    {
        displayBanner(this, strError);
        return;
    }

    lstArgs.append(teUser->text());
    lstArgs.append(teOldPwd->text());
    lstArgs.append(teNewPwd1->text());
    lstArgs.append(servIP);

    exitCode = QProcess::execute(TCCHGPWD_PROG, lstArgs);
    if ( exitCode != 0 )
    {
        displayBanner(this, qtTrId("Change user password failed."));
        return;
    }

    displayBanner(this, qtTrId("Change password successfully."));

    teUser->clear();
    teOldPwd->clear();
    teNewPwd1->clear();
    teNewPwd2->clear();
}
