/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QHostAddress>
#include <QGraphicsLinearLayout>
#include <QGraphicsGridLayout>
#include <QSwipeGesture>
#include <MMessageBox>
#include <MContainer>
#include <MComboBox>
#include <MTextEdit>
#include <MLabel>
#include <MButton>
#include <MDebug>

#include "misc.h"
#include "initdiskpage.h"
#include "settingpage.h"


#define WIFICFG_FILE            "/mnt/rsvd/tc-wifi.addr"
#define CFGWIFI_PROG            "/usr/sbin/tc/tc-cfgwifi.sh"

#define DEFAULT_IFACE           "eth0"
#define IFACE_FILE              "/mnt/rsvd/tc-iface.def"
#define ENUMIFACE_PROG          "/usr/sbin/tc/enum_iface.sh"

#define NETCFG_FILE             "/mnt/rsvd/tc-local.addr"
#define CFGNET_PROG             "/usr/sbin/tc/tc-cfgnet.sh"

#define HOSTIP_FILE             "/mnt/rsvd/tc-serv.addr"


SettingPage::SettingPage(const QString &serverIP)
	: bIPChanged(false)
	, servIP(serverIP)
{
    mDebug("SettingPage::SettingPage()");

    setObjectName("setting_page");
}

SettingPage::~SettingPage()
{
    mDebug("SettingPage::~SettingPage()");
}

void SettingPage::createContent()
{
    mDebug("SettingPage::createContent()");

//    MApplicationPage::createContent();
//    setStyleName("CommonApplicationPage");

    setComponentsDisplayMode(NavigationBar, MApplicationPageModel::Hide);
    setComponentsDisplayMode(HomeButton, MApplicationPageModel::Hide);
    setComponentsDisplayMode(EscapeButton, MApplicationPageModel::Hide);

    MContainer *ctnInitHd = new MContainer(this);
    ctnInitHd->setStyleName("CommonContainer");
    ctnInitHd->setTitle(qtTrId("initialize disk"));

//    MContainer *ctnChgWifi = new MContainer(this);
//    ctnChgWifi->setStyleName("CommonContainer");
//    ctnChgWifi->setTitle(qtTrId("change wireless's SSID/Key"));

    MContainer *ctnCfgNet = new MContainer(this);
    ctnCfgNet->setStyleName("CommonContainer");
    ctnCfgNet->setTitle(qtTrId("change the local network"));

    MContainer *ctnChgServIP = new MContainer(this);
    ctnChgServIP->setStyleName("CommonContainer");
    ctnChgServIP->setTitle(qtTrId("change the server IP address"));

    QGraphicsLinearLayout *lyOuter = new QGraphicsLinearLayout(Qt::Vertical, centralWidget());
    QGraphicsLinearLayout *lyInitHd = new QGraphicsLinearLayout(Qt::Horizontal, ctnInitHd->centralWidget());
//    QGraphicsLinearLayout *lyChgWifi = new QGraphicsLinearLayout(Qt::Horizontal, ctnChgWifi->centralWidget());
    QGraphicsGridLayout *lyCfgNet = new QGraphicsGridLayout(ctnCfgNet->centralWidget());
    QGraphicsLinearLayout *lyChgServIP = new QGraphicsLinearLayout(Qt::Horizontal, ctnChgServIP->centralWidget());


    teImgPart = new MTextEdit;
    teImgPart->setObjectName("setting_te_imgpart");
    teImgPart->setMaxLength(20);
    teImgPart->setPrompt(qtTrId("cache img size(MB) specified by system"));
    teImgPart->setEnabled(false);
    teImgPart->setPreferredWidth(600);
    teImgPart->setTextInteractionFlags(Qt::TextEditable);
    lyInitHd->addItem(teImgPart);

    btnInitHd = new MButton(qtTrId("Initialize"));
    btnInitHd->setObjectName("setting_btn");
    btnInitHd->setPreferredWidth(200);
    lyInitHd->addItem(btnInitHd);


    teWifiSSID = new MTextEdit;
    teWifiSSID->setObjectName("setting_te_wifissid");
    teWifiSSID->setMaxLength(20);
    teWifiSSID->setPrompt(qtTrId("SSID"));
    teWifiSSID->setPreferredWidth(300);
    teWifiSSID->setTextInteractionFlags(Qt::TextEditable);
//    lyChgWifi->addItem(teWifiSSID);

    teWifiPwd = new MTextEdit;
    teWifiPwd->setObjectName("setting_te_wifipwd");
    teWifiPwd->setMaxLength(20);
    teWifiPwd->setPrompt(qtTrId("password"));
    teWifiPwd->setEchoMode(MTextEditModel::Password);
    teWifiPwd->setPreferredWidth(300);
    teWifiPwd->setTextInteractionFlags(Qt::TextEditable);
//    lyChgWifi->addItem(teWifiPwd);

    btnChgWifi = new MButton(qtTrId("Change"));
    btnChgWifi->setObjectName("setting_btn");
    btnChgWifi->setPreferredWidth(200);
//    lyChgWifi->addItem(btnChgWifi);


    cbIfaceList = new MComboBox();
    cbIfaceList->setObjectName("comboBox");
    cbIfaceList->setStyleName("CommonComboBox");
    cbIfaceList->setIconID("icon-iface");
    cbIfaceList->setTitle(qtTrId("interface"));
    lyCfgNet->addItem(cbIfaceList, 0, 0);


    QGraphicsLinearLayout *lyDHCPOff = new QGraphicsLinearLayout(Qt::Horizontal);

    MLabel *lbDHCPOff = new MLabel(qtTrId("DHCP off"));
    lbDHCPOff->setObjectName("setting_lb");
    lyDHCPOff->addItem(lbDHCPOff);
    lyDHCPOff->setAlignment(lbDHCPOff, Qt::AlignVCenter);

    btnDHCPOff = new MButton();
    btnDHCPOff->setObjectName("setting_btn");
    btnDHCPOff->setViewType(MButton::checkboxType);
    btnDHCPOff->setCheckable(true);
    btnDHCPOff->setPreferredWidth(80);
    lyDHCPOff->addItem(btnDHCPOff);
    lyDHCPOff->setAlignment(btnDHCPOff, Qt::AlignVCenter);

    lyCfgNet->addItem(lyDHCPOff, 1, 0);

    teLocalIP = new MTextEdit;
    teLocalIP->setObjectName("setting_te_addr");
    teLocalIP->setMaxLength(20);
    teLocalIP->setPrompt(qtTrId("IP address"));
    teLocalIP->setPreferredWidth(200);
    teLocalIP->setTextInteractionFlags(Qt::TextEditable);
    lyCfgNet->addItem(teLocalIP, 1, 1);

    teNetmask = new MTextEdit;
    teNetmask->setObjectName("setting_te_addr");
    teNetmask->setMaxLength(20);
    teNetmask->setPrompt(qtTrId("subnet mask"));
    teNetmask->setPreferredWidth(200);
    teNetmask->setTextInteractionFlags(Qt::TextEditable);
    lyCfgNet->addItem(teNetmask, 1, 2);

    teGateway = new MTextEdit;
    teGateway->setObjectName("setting_te_addr");
    teGateway->setMaxLength(20);
    teGateway->setPrompt(qtTrId("default gateway"));
    teGateway->setPreferredWidth(200);
    teGateway->setTextInteractionFlags(Qt::TextEditable);
    lyCfgNet->addItem(teGateway, 2, 0);

    teDNS1 = new MTextEdit;
    teDNS1->setObjectName("setting_te_addr");
    teDNS1->setMaxLength(20);
    teDNS1->setPrompt(qtTrId("preferred DNS"));
    teDNS1->setPreferredWidth(200);
    teDNS1->setTextInteractionFlags(Qt::TextEditable);
    lyCfgNet->addItem(teDNS1, 2, 1);

    teDNS2 = new MTextEdit;
    teDNS2->setObjectName("setting_te_addr");
    teDNS2->setMaxLength(20);
    teDNS2->setPrompt(qtTrId("alternate DNS"));
    teDNS2->setPreferredWidth(200);
    teDNS2->setTextInteractionFlags(Qt::TextEditable);
    lyCfgNet->addItem(teDNS2, 2, 2);

    btnCfgNet = new MButton(qtTrId("Change"));
    btnCfgNet->setObjectName("setting_btn");
    btnCfgNet->setPreferredWidth(200);
    lyCfgNet->addItem(btnCfgNet, 2, 4);


    teServIP = new MTextEdit;
    teServIP->setObjectName("setting_te_addr");
    teServIP->setMaxLength(20);
    teServIP->setText(servIP);
    teServIP->setPrompt(qtTrId("host address"));
    teServIP->setPreferredWidth(600);
    teServIP->setTextInteractionFlags(Qt::TextEditable);
    lyChgServIP->addItem(teServIP);

    btnChgServIP = new MButton(qtTrId("Change"));
    btnChgServIP->setObjectName("setting_btn");
    btnChgServIP->setPreferredWidth(200);
    lyChgServIP->addItem(btnChgServIP);


    lyOuter->addStretch();
    lyOuter->addItem(ctnInitHd);
//    lyOuter->addItem(ctnChgWifi);
    lyOuter->addItem(ctnCfgNet);
    lyOuter->addItem(ctnChgServIP);

    connect(teLocalIP, SIGNAL(tabPressed()), this, SLOT(onCfgNetTabPressed()));
    connect(teNetmask, SIGNAL(tabPressed()), this, SLOT(onCfgNetTabPressed()));
    connect(teGateway, SIGNAL(tabPressed()), this, SLOT(onCfgNetTabPressed()));
    connect(teDNS1, SIGNAL(tabPressed()), this, SLOT(onCfgNetTabPressed()));
    connect(teDNS2, SIGNAL(tabPressed()), this, SLOT(onCfgNetTabPressed()));

    connect(btnInitHd, SIGNAL(clicked()), this, SLOT(onBtnInitHdClicked()));
    connect(btnChgWifi, SIGNAL(clicked()), this, SLOT(onBtnChgWifiClicked()));

    connect(btnDHCPOff, SIGNAL(clicked()), this, SLOT(onBtnDHCPOffClicked()));
    connect(teGateway, SIGNAL(returnPressed()), btnCfgNet, SIGNAL(clicked()));
    connect(teDNS1, SIGNAL(returnPressed()), btnCfgNet, SIGNAL(clicked()));
    connect(teDNS2, SIGNAL(returnPressed()), btnCfgNet, SIGNAL(clicked()));
    connect(btnCfgNet, SIGNAL(clicked()), this, SLOT(onBtnCfgNetClicked()));

    connect(teServIP, SIGNAL(returnPressed()), btnChgServIP, SIGNAL(clicked()));
    connect(btnChgServIP, SIGNAL(clicked()), this, SLOT(onBtnChgServIPClicked()));

    connect(this, SIGNAL(appeared()), this, SLOT(onAppeared()));
}

void SettingPage::swipeGestureEvent(QGestureEvent *event, QSwipeGesture *gesture)
{
    mDebug("SettingPage::swipeGestureEvent()") << "State:" << gesture->state() << "HDir:" << gesture->horizontalDirection() << "VDir:" << gesture->verticalDirection();

    event->accept(gesture);

    if (gesture->state() == Qt::GestureFinished)
    {
        if (gesture->horizontalDirection() == QSwipeGesture::Right)
        {
            disappear();
        }
    }
}

void SettingPage::readWifiCfg()
{
    QString wificfg;
    QStringList wifiparam;
    readLine(WIFICFG_FILE, wificfg);
    if (wificfg.isEmpty())
    {
        wificfg = ":";
    }
    wifiparam = wificfg.split(":");

    teWifiSSID->setText(wifiparam.at(0));
    teWifiPwd->setText(wifiparam.at(1));
}

void SettingPage::readIfaceList()
{
    int exitCode;
    QProcess psEnumIfaces;
    QString cmd(ENUMIFACE_PROG);

    psEnumIfaces.start(cmd);
    psEnumIfaces.waitForFinished();
    exitCode = psEnumIfaces.exitCode();
    if ( exitCode != 0 )
    {
        displayBanner(this, qtTrId("obtain interfaces info failed."));
        return;
    }

    QString msg = psEnumIfaces.readAll();
    QStringList ifacelist = msg.split(":");
    cbIfaceList->addItems(ifacelist);

    QString iface;
    readLine(IFACE_FILE, iface);
    if (iface.isEmpty())
    {
        iface = DEFAULT_IFACE;
    }

    int nIfIdx = ifacelist.indexOf(iface);
    if ( nIfIdx >= 0 )
    {
        cbIfaceList->setCurrentIndex(nIfIdx);
    }
}

void SettingPage::readNetCfg()
{
    QString netcfg;
    QStringList netparam;
    readLine(NETCFG_FILE, netcfg);
    if (netcfg.isEmpty())
    {
        netcfg = "0:::::";
    }
    netparam = netcfg.split(":");

    if (netparam.at(0) == "1")
    {
        btnDHCPOff->setChecked(true);
        teLocalIP->setText(netparam.at(1));
        teNetmask->setText(netparam.at(2));
        teGateway->setText(netparam.at(3));
        teDNS1->setText(netparam.at(4));
        teDNS2->setText(netparam.at(5));
    }
    else
    {
        btnDHCPOff->setChecked(false);
        teLocalIP->clear();
        teNetmask->clear();
        teGateway->clear();
        teDNS1->clear();
        teDNS2->clear();
    }

    emit onBtnDHCPOffClicked();
}

void SettingPage::onCfgNetTabPressed()
{
    QGraphicsItem *item;

    mDebug("onCfgNetTabPressed");

    item = centralWidget()->focusItem();

    if (item == teLocalIP)
    {
        scene()->setFocusItem(teNetmask, Qt::TabFocusReason);
    }
    else if(item == teNetmask)
    {
        scene()->setFocusItem(teGateway, Qt::TabFocusReason);
    }
    else if(item == teGateway)
    {
        scene()->setFocusItem(teDNS1, Qt::TabFocusReason);
    }
    else if(item == teDNS1)
    {
        scene()->setFocusItem(teDNS2, Qt::TabFocusReason);
    }
    else if(item == teDNS2)
    {
        scene()->setFocusItem(teLocalIP, Qt::TabFocusReason);
    }
}

void SettingPage::onBtnInitHdClicked()
{
    QString  size("0");

    if (!teImgPart->text().isEmpty())
    {
        size = teImgPart->text().simplified();
    }

    MMessageBox dlg(qtTrId("Confirm"), qtTrId("Really want to initialize harddisk?"), M::YesButton|M::NoButton);
    dlg.setObjectName("messageBox");
    if ( dlg.exec() == M::YesButton )
    {
        InitDiskPage *pgInitDisk = new InitDiskPage(size.toInt(NULL, 10));
        pgInitDisk->appear(scene(), MSceneWindow::DestroyWhenDismissed);
    }
}

void SettingPage::onBtnChgWifiClicked()
{
    int exitCode;
    QString ssid, pwd;
    QString cmd(CFGWIFI_PROG);
    QStringList wificfg;

    if (teWifiSSID->text().isEmpty())
    {
        displayBanner(this, qtTrId("Please input the SSID name."));
        return;
    }
    ssid = teWifiSSID->text().simplified();

    pwd = teWifiPwd->text().simplified();

    wificfg.append(ssid);
    wificfg.append(pwd);
    writeLine(WIFICFG_FILE, wificfg.join(":"));

    exitCode = QProcess::execute(cmd);
    if ( exitCode != 0 )
    {
        displayBanner(this, qtTrId("Setup wireless network failed."));
        return;
    }

    displayBanner(this, qtTrId("Setup wireless network successfully."));
}

void SettingPage::onBtnDHCPOffClicked()
{
    if (btnDHCPOff->isChecked())
    {
        teLocalIP->setEnabled(true);
        teNetmask->setEnabled(true);
        teGateway->setEnabled(true);
        teDNS1->setEnabled(true);
        teDNS2->setEnabled(true);
    }
    else
    {
        teLocalIP->setEnabled(false);
        teNetmask->setEnabled(false);
        teGateway->setEnabled(false);
        teDNS1->setEnabled(false);
        teDNS2->setEnabled(false);
    }
}

void SettingPage::onBtnCfgNetClicked()
{
    int exitCode;
    bool bDHCPOff;
    QHostAddress ipAddr;
    QString ip, mask, gw, dns1, dns2;
    QString cmd(CFGNET_PROG);
    QStringList netcfg;

    if (cbIfaceList->currentIndex() < 0)
    {
        displayBanner(this, qtTrId("Please specify the interface."));
        return;
    }
    writeLine(IFACE_FILE, cbIfaceList->currentText());


    bDHCPOff = btnDHCPOff->isChecked();

    if (bDHCPOff)
    {
        if (teLocalIP->text().isEmpty() || (!ipAddr.setAddress(teLocalIP->text().simplified())))
        {
            displayBanner(this, qtTrId("IP address is invalid."));
            return;
        }
        ip = ipAddr.toString();

        if (teNetmask->text().isEmpty() || (!ipAddr.setAddress(teNetmask->text().simplified())))
        {
            displayBanner(this, qtTrId("Subnet mask is invalid."));
            return;
        }
        mask = ipAddr.toString();

        if (teGateway->text().isEmpty() || (!ipAddr.setAddress(teGateway->text().simplified())))
        {
            displayBanner(this, qtTrId("Default gateway is invalid."));
            return;
        }
        gw = ipAddr.toString();

        if (!teDNS1->text().isEmpty())
        {
            if (!ipAddr.setAddress(teDNS1->text().simplified()))
            {
                displayBanner(this, qtTrId("Preferred DNS is invalid."));
                return;
            }
            dns1 = ipAddr.toString();
        }

        if (!teDNS2->text().isEmpty())
        {
            if (!ipAddr.setAddress(teDNS2->text().simplified()))
            {
                displayBanner(this, qtTrId("Alternate DNS is invalid."));
                return;
            }
            dns2 = ipAddr.toString();
        }
    }

    netcfg.append(bDHCPOff ? "1" : "0");
    netcfg.append(ip);
    netcfg.append(mask);
    netcfg.append(gw);
    netcfg.append(dns1);
    netcfg.append(dns2);
    writeLine(NETCFG_FILE, netcfg.join(":"));

    QProcess process;
    QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
    env.insert("CHGNET", "1");
    process.setProcessEnvironment(env);
    process.start(cmd);
    process.waitForFinished();
    exitCode = process.exitCode();
    if ( exitCode != 0 )
    {
        displayBanner(this, qtTrId("Setup local network failed."));
        return;
    }

    displayBanner(this, qtTrId("Setup local network successfully."));
}

void SettingPage::onBtnChgServIPClicked()
{
    QHostAddress servAddr;

    bIPChanged = false;

    if (teServIP->text().isEmpty())
    {
        displayBanner(this, qtTrId("Please input the server IP address."));
        return;
    }

    if (!servAddr.setAddress(teServIP->text().simplified()))
    {
        displayBanner(this, qtTrId("IP address is invalid."));
        return;
    }

    bIPChanged = true;
    servIP = servAddr.toString();
    writeLine(HOSTIP_FILE, servIP);
    displayBanner(this, qtTrId("Change server IP address successfully."));
}

void SettingPage::onAppeared()
{
    mDebug("SettingPage::onAppeared");

    readWifiCfg();
    readIfaceList();
    readNetCfg();
}

