/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QGraphicsLinearLayout>
#include <QTimer>
#include <MLabel>
#include <MProgressIndicator>
#include <MDebug>

#include "misc.h"
#include "updatefwpage.h"


#define UPDATEFW_PROG           "/tcdscript/updatefirmware.sh"


UpdateFwPage::UpdateFwPage()
{
    mDebug("UpdateFwPage::UpdateFwPage()");

    setObjectName("updatefw_page");
}

UpdateFwPage::~UpdateFwPage()
{
    mDebug("UpdateFwPage::~UpdateFwPage()");
}

void UpdateFwPage::createContent()
{
    mDebug("UpdateFwPage::createContent()");

//    MApplicationPage::createContent();
//    setStyleName("CommonApplicationPage");

    setComponentsDisplayMode(NavigationBar, MApplicationPageModel::Hide);
    setComponentsDisplayMode(HomeButton, MApplicationPageModel::Hide);
    setComponentsDisplayMode(EscapeButton, MApplicationPageModel::Hide);

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Vertical, centralWidget());

    layout->addStretch();

    spinner = new MProgressIndicator(this, MProgressIndicator::spinnerType);
    spinner->setObjectName("updatefw_spinner");
    spinner->setUnknownDuration(true);
    spinner->setVisible(true);
    layout->addItem(spinner);
    layout->setAlignment(spinner, Qt::AlignCenter);

    lbPrompt = new MLabel(qtTrId("Updating the firmware, please wait..."));
    lbPrompt->setObjectName("updatefw_lb_prompt");
    layout->addItem(lbPrompt);

    layout->addStretch();

    connect(&psTcSync, SIGNAL(error(QProcess::ProcessError)), this, SLOT(onPsError(QProcess::ProcessError)));
    connect(&psTcSync, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(onPsFinished(int,QProcess::ExitStatus)));


    QStringList lstArgs;

    psTcSync.start(UPDATEFW_PROG, lstArgs);
}

void UpdateFwPage::ShowMessage(const QString &msg)
{
    displayBanner(this, msg);
    QTimer::singleShot(1500, this, SLOT(dismiss()));
}

void UpdateFwPage::onPsError(QProcess::ProcessError errCode)
{
    QString msg;

    mDebug("UpdateFwPage::onPsError: ") << errCode;

    msg = qtTrId("Execute program failed: %1.").arg(errCode);
    ShowMessage(msg);
}

void UpdateFwPage::onPsFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    QString msg;

    mDebug("UpdateFwPage::onPsFinished: ") << exitCode << exitStatus;

    if ( exitCode != 0 )
    {
        msg = qtTrId("Update firmware failed: %1.").arg(exitCode);
    }
    else
    {
        msg = qtTrId("Update firmware successfully.");
    }
    ShowMessage(msg);
}
