/******************************************************************************
Copyright 2016 - 2019 Intel Corporation
 
For licensing information, see the file 'LICENSE' in the root folder of
  this software module. 
******************************************************************************/

#include <QCoreApplication>
#include <QtDBus>


#define TC_SERV_NAME            "TC.NOTIFIER"


void Usages()
{
    printf("\nUsages:\n");
    printf("\ttcnotifier 1 uname passwd delay osname osid\n");
    printf("\ttcnotifier 2 osid orgos curos uname\n");
    printf("\ttcnotifier 3 hw status\n");
    printf("\ttcnotifier 4 qrimg\n");
    printf("\ttcnotifier 5 imgpart(MB)\n");
    printf("\ttcnotifier 6(update fw)\n");
    printf("\ttcnotifier 7(reg mac)\n");
    printf("\ttcnotifier 8(reg mac) clientname\n");
    printf("\ttcnotifier 9 clientname\n");
    printf("\ttcnotifier 11(skip updates) seconds\n");
}

bool CheckArgs(int ch, int argc)
{
    if ( argc < 2 )
    {
        return false;
    }

    if ( (ch == 1) && (argc == 7) )
    {
        return true;
    }

    if ( (ch == 2) && (argc == 6) )
    {
        return true;
    }

    if ( (ch == 3) && (argc == 4) )
    {
        return true;
    }

    if ( (ch == 4) && (argc == 3) )
    {
        return true;
    }

    if ( (ch == 5) && (argc == 3) )
    {
        return true;
    }

    if ( (ch == 6) && (argc == 2) )
    {
        return true;
    }

    if ( (ch == 7) && (argc == 2) )
    {
        return true;
    }

    if ( (ch == 8) && (argc == 3) )
    {
        return true;
    }

    if ( (ch == 9) && (argc == 3) )
    {
        return true;
    }

    if ( (ch == 11) && (argc == 3) )
    {
        return true;
    }

    return false;
}

int main(int argc, char *argv[])
{
    QDBusError::ErrorType  errType;
    int  ch;

    ch = atoi(argv[1]);
    if ( !CheckArgs(ch, argc) )
    {
        Usages();
        return -1;
    }

    QCoreApplication  app(argc, argv);

    if ( !QDBusConnection::sessionBus().isConnected() )
    {
        fprintf(stderr, "Cannot connect to the session bus.\n");
        return QDBusError::Failed;
    }


    QString  path;
    if ( ch < 10 )
    {
        path = "/LoginPage";
    }
    else
    {
        path = "/ProcessPage";
    }
    QDBusInterface  iface(TC_SERV_NAME, path, "", QDBusConnection::sessionBus());
    if ( !iface.isValid() )
    {
        errType = QDBusConnection::sessionBus().lastError().type();
        fprintf(stderr, "iface failed: %s\n", qPrintable(QDBusError::errorString(errType)));
        return errType;
    }

    QDBusReply<int>  reply;
    switch ( ch )
    {
        case 1:
            reply = iface.call("AutoBootOS", argv[2], argv[3], atoi(argv[4]), argv[5], atoi(argv[6]));
            break;

        case 2:
            reply = iface.call("QuickBootOS", atoi(argv[2]), argv[3], argv[4], argv[5]);
            break;

        case 3:
            reply = iface.call("HwStatus", atoi(argv[2]), atoi(argv[3]));
            break;

        case 4:
            reply = iface.call("ShowQRImg", argv[2]);
            break;

        case 5:
            reply = iface.call("InitDisk", atoi(argv[2]));
            break;

        case 6:
            reply = iface.call("UpdateFw");
            break;

        case 7:
            reply = iface.call("RegMachine");
            break;

        case 8:
            reply = iface.call("RegMachineAuto", argv[2]);
            break;

        case 9:
            reply = iface.call("MachineName", argv[2]);
            break;

        case 11:
            reply = iface.call("SkipUpdates", atoi(argv[2]));
            break;

        default:
            break;

    }

    if ( !reply.isValid() )
    {
        errType = reply.error().type();
        fprintf(stderr, "call failed: %s\n", qPrintable(QDBusError::errorString(errType)));
        return errType;
    }

    printf("call reply: %d\n", reply.value());
    return reply.value();
}
