<?php

/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
                           this software module.
******************************************************************************/

error_reporting(E_ALL);

require("rest.inc.php");
require("../libs/libtc.php");

function handle_get($target, $params, $body) {
    if ($target) {
        $cid = array_shift($target);
        if (!is_null($cid)) {
            $cid = sugar_valid_int($cid);
            if (!$cid) {
                return rest_error("e_operation_unsupported");
            }
        }
        $client = client_read($cid);
        if (empty($client['client_token'])) {
            update_client_token($cid);
        }
        return rest_result_ok(client_read($cid));
    }
    return rest_error('e_operation_unsupported');
}

/**
 * /client.php      query if mac is registered, support lower case in
 *                  both of 00-00-00-00-00-00 and 00:00:00:00:00:00
 * /client.php/:id  command for this client, details in body
 * @param  [type]
 * @param  [type]
 * @param  [type]
 * @return [type]
 */
function handle_post($target, $params, $body) {
    if ($target) {
        $client_id = array_shift($target);
        $cid = $client_id;
        $cid = sugar_valid_int($cid);
        if (!$cid) {
            return rest_error("e_operation_unsupported");
        }
        $client = client_read($client_id);
        if (empty($client['client_token'])) {
            update_client_token($client_id);
        }
        $boot = safe_array_get('boot', $body);
        if ($boot) {
            $client_token = safe_array_get('client_token', $boot);
            $access_token = safe_array_get('access_token', $boot);
            if (empty($access_token)) {
                $auth = current_session_auth();
                $access_token = $auth["access_token"];
                log_info("load session access token, $access_token");
            }
            $image_id = safe_array_get('image_id', $boot);
            $error = remote_boot_client($client_token, $access_token, $image_id);
            return $error ? rest_error($error) : rest_result_ok();
        }

        $status = safe_array_get('client_status', $body);
        if ($status) {
            update_client_online_status($client_id, $status);
        }
        return rest_result_ok();
    }

    // client use this 'mac' to query the register status
    $mac = safe_array_get('mac', $body);
    if ($mac) {
        $mac = str_replace(':', '-', $mac);
        $mac = strtoupper($mac);

        // update hardware information
        $cpu_model = db_safe_array_get('cpu_model', $body);
        $disk_size = db_safe_array_get('disk_size', $body);
        $memory_size = db_safe_array_get('memory_size', $body);

        $client_id = db_select_first_value("SELECT id FROM machines WHERE mac='$mac'");
        if (is_null($client_id)) {
            // new client
            if (!is_null($cpu_model)) {
                update_client_online_hardware($mac, $cpu_model, $memory_size, $disk_size);
            }
            return rest_error('e_new_client');
        }

        // update token
        $client = client_read($client_id);
        if (empty($client['client_token'])) {
            update_client_token($client_id);
        }
        $client['qrcode'] = "/tc/rest/file.php/qrcode/$client_id";

        if (!is_null($cpu_model)) {
            $changes = array(
                'cpu_model' => $cpu_model,
                'memory_size' => $memory_size,
                'disk_size' => $disk_size,
            );
            db_update_table('machines', $changes, "id='$client_id'");
            db_update_table('userstatus', $changes, "mac='$mac'");            

            $firmware_version = db_safe_array_get('firmware_version', $body);
            $changes = array(
                'firmware' => $firmware_version,
            );
            db_update_table('machines', $changes, "id='$client_id'");
        }
        return rest_result_ok($client);
    }
    return rest_result_ok();
}


function handle_put($target, $params, $body) {

    $image_id = safe_array_get('image_id', $body);
    if (is_null($image_id)) {
        return rest_error("e_operation_unsupported");
    }
    $filepath = db_select_first_value("
        SELECT newPath FROM imageupdatehistory
        WHERE revision=1 AND imageId=$image_id"
    );

    $filename = basename($filepath);

    $result = send_cc_command_registered("--remove-image --image-org-name $filename --image-id $image_id");
    if (is_cmd_fail($result)) {
        return rest_error_shell($result);
    }
    return rest_result_ok($result['exit_value']);
}

$handlers = array(
    "GET" => array("handle_get", "log_info"),
    "POST" => array("handle_post", "log_info"),
    "PUT" => array("handle_put", "log_info"),
);
rest_start_loop($handlers);

?>
