<?php

/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
                           this software module.
******************************************************************************/

require('rest.inc.php');
require("../libs/libtc.php");


function handle_put($target, $params, $body) {
    $cgid = array_shift($target);
    if (is_null($cgid)) {
        return rest_error("e_operation_unsupported");
    }
    $cgid = sugar_valid_int($cgid);
    if (!$cgid) {
        return rest_error("e_operation_unsupported");
    }
    // name check
    $name = db_safe_array_get("name", $body);
    if (is_null($name) || $name === "") {
        return rest_error("e_client_group_name");
    }
    $desc = db_safe_array_get("desc", $body, "");
    $autoboot = db_safe_array_get("autoboot", $body);
    $fields = array(
        "cg_name" => $name,
        "cg_desc" => $desc,
        "cg_autoboot_ciid" => $autoboot,
    );

    // update clients in machine table
    $res = db_update_table("client_group", $fields, "cgid='$cgid'");
    if (is_null($res)) {
        return rest_error_mysql();
    }
    return rest_result_ok();
}

function handle_delete($target, $params, $body) {
    $cgid = array_shift($target);
    if (is_null($cgid)) {
        return rest_error("e_operation_unsupported");
    }
    $cgid = sugar_valid_int($cgid);
    if (!$cgid) {
        return rest_error("e_operation_unsupported");
    }
    // delete client group row
    $res = db_delete("client_group", "cgid='$cgid'");
    if (is_null($res)) {
        return rest_error_mysql();
    }
    // update clients in machine table
    $res = db_update_table("machines", array("client_group" => "0"), "client_group='$cgid'");
    if (is_null($res)) {
        log_error("Cannot clear clients client group for $cgid");
    }
    return rest_result_ok();
}

function handle_post($target, $params, $body) {
    // count check
    $max = constant("TC_CLIENT_GROUP_MAX");
    $count = db_select_first_value(db_sql_select("client_group", array("count(cgid)")));
    if (is_null($count)) {
        return rest_error_mysql();
    }
    if ($count >= $max) {
        log_error("Client group count $count, max $max");
        return rest_error("e_client_group_max");
    }
    // name check
    $name = db_safe_array_get("name", $body);
    if (is_null($name) || $name === "") {
        return rest_error("e_client_group_name");
    }
    $desc = db_safe_array_get("desc", $body, "");
    $autoboot = db_safe_array_get("autoboot", $body);
    $fields = array(
        "cg_name" => $name,
        "cg_desc" => $desc,
        "cg_autoboot_ciid" => $autoboot,
    );
    $result = db_insert_table("client_group", $fields);
    if (is_null($result)) {
        return rest_error_mysql();
    }
    return rest_result_ok($result);
}

function handle_get($target, $params, $body) {
    $cgid = array_shift($target);
    $fields = array(
        "cgid",
        "cg_name AS name",
        "cg_desc AS description",
        "cg_create_ts AS create_timestamp",
        "cg_autoboot_ciid AS autoboot_ciid",
    );
    if (is_null($cgid)) {
        $result = db_select(db_sql_select("client_group", $fields));
        if (is_null($result)) {
            return rest_error_mysql();
        }
        foreach ($result as &$cg) {
            $cg["client_members"] = client_group_members($cg["cgid"]);
            $cg["autoboot_image"] = "";
            if (intval($cg["autoboot_ciid"])) {
                $cg["autoboot_image"] = db_client_image_read($cg["autoboot_ciid"], array("imageName"));
            }
        }
        return rest_result_ok($result);
    }

    $cgid = sugar_valid_int($cgid);
    if (!$cgid) {
        return rest_error("e_operation_unsupported");
    }
    $where = "cgid=$cgid";
    $result = db_select_row(db_sql_select("client_group", $fields, $where));
    if (is_null($result)) {
        return rest_error_mysql();
    }
    $result['client_members'] = client_group_members($result["cgid"]);
    if (intval($result["autoboot_ciid"])) {
        $result["autoboot_image"] = db_client_image_read($result["autoboot_ciid"], array("imageName"));
    }
    return rest_result_ok($result);
}

auth_login_required();

$handlers = array(
    "GET" => array("handle_get"),
    "PUT" => array("handle_put", "log_info"),
    "POST" => array("handle_post", "log_info"),
    "DELETE" => array("handle_delete", "log_info"),
);
rest_start_loop($handlers);

?>
