<?php

/******************************************************************************
Copyright 2012 - 2020 Intel Corporation

For licensing information, see the file 'LICENSE' in the root folder of
                           this software module.
******************************************************************************/

    error_reporting(E_ALL);
    require('../libs/libtc.php');
    require("libpage.php");
    require("include/inc.bootstrap.php");

    standard_page_begin('client_groups');
?>

<div class="container-fluid">
<!-- row for toolbar -->
    <div class="row">
<?php
    $bar = array(
        "button_groups" => array(
            array(
                "buttons" => array(
                    array("i18n" => "s_client_group_add", "icon" => "glyphicon-plus"),
                    array("i18n" => "s_client_group_remove", "icon" => "glyphicon-minus"),
                    array("i18n" => "s_client_group_edit", "icon" => "glyphicon-edit"),
                ),
            ),
        ),
        "id" => "toolbar_control",
    );
    echo html_toolbar($bar);
?>
    </div>

    <div class="row">
        <div class="list-group col-xs-6" style="margin-top: 10px;" id="client_group_list">
        </div>
        <div id="client_group_member_list" readonly class="col-xs-6 well" style="margin-top: 10px; display: none;">
        </div>
    </div>
</div>

<div id="dialog_client_group_details" style="display: none;">
    <div class="form-group">
        <label><span i18n="s_client_group_name">Group Name</span></label>
        <input type="text" class="form-control" maxlength="20"></input>
    </div>
    <div class="form-group">
        <label><span i18n="s_client_group_desc">Description</span></label>
        <textarea class="form-control" maxlength="100"></textarea>
    </div>
    <div class="form-group">
        <label><span i18n="s_client_group_autoboot">Autoboot Image</span></label>
        <select class="form-control">
            <option value="0"></option>
            <option value="-1"></option>
        </select>
    </div>
</div>

<?php standard_page_mid(); ?>

<script language="javascript">

"use strict";
var create_group = function($dialog) {
    var name = $dialog.find('input').val(),
        desc = $dialog.find('textarea').val(),
        autoboot = $dialog.find('select').val(),
        data = {name: name, desc: desc, autoboot: autoboot};

    util_page.rest_post('/tc/rest/client_group.php', data, function() {
        load_groups();
    });
};

var delete_group = function(cgid) {
    util_page.dialog_confirm(find_i18n('c_client_group_delete'), function() {
        util_page.rest_delete('/tc/rest/client_group.php/' + cgid, {}, function() {
            load_groups();
            util_page.dialog_message(find_i18n('s_operation_complete'));
        });
    });
};

var edit_group = function(cgid) {
    create_empty_details_dialog(function($dialog) {
        var name = $dialog.find('input').val(),
            desc = $dialog.find('textarea').val(),
            autoboot = $dialog.find('select').val(),
            data = {name: name, desc: desc, autoboot: autoboot};
        util_page.rest_put('/tc/rest/client_group.php/' + cgid, data, function() {
            load_groups();
            util_page.dialog_message(find_i18n('s_operation_complete'));
        });
    }, function($dialog) {
        util_page.rest_get('/tc/rest/client_group.php/' + cgid, function(cg) {
            $dialog.find('input').val(cg.name);
            $dialog.find('textarea').val(cg.description);
            $dialog.find('select').val(cg.autoboot_ciid);
        });
    });
};

var html_list_item = function(cg) {
    var html = '<a href="#" class="list-group-item">';
    html += '<span class="badge">' + cg.client_members.length + '</span>';
    html += '<h4 id="list-group-item-name" class="list-group-item-heading"></h4>';
    html += '<p id="list-group-item-desc" class="list-group-item-text"></p>';
    if (cg.autoboot_image) {
        html += '<p class="list-group-item-text pull-right">' + cg.autoboot_image + '</p>';
    }
    html += '<p class="list-group-item-text">' + cg.create_timestamp + '</p>';
    html += '</a>';
    return html;
};

var load_groups = function() {
    util_page.rest_get('/tc/rest/client_group.php', function(cgs) {
        $('#client_group_list').empty();
        $('#client_group_member_list').text('').hide();

        $.map(cgs, function(cg) {
            $(html_list_item(cg))
                .find('#list-group-item-name').text(cg.name).end()
                .find('#list-group-item-desc').text(cg.description).end()
                .data('cgid', cg.cgid)
                .css('word-wrap', 'break-word')
                .click(function(e) {
                    e.preventDefault();

                    $('#client_group_list a').removeClass('active');
                    $(this).addClass('active');
                    $('#client_group_member_list').text('').hide();

                    if (cg.client_members.length) {
                        var name_string = '';
                        $.map(cg.client_members, function(m) {
                            name_string += m.name + ', ';
                        });
                        $('#client_group_member_list').show().text(name_string);
                    }
                })
                .appendTo($('#client_group_list'));
        });
    });
};

var create_empty_details_dialog = function(fn_done, fn_init) {
    var $dialog = util_page.dialog_confirm_builder(
        $('#dialog_client_group_details').html(),
        find_i18n('s_save'),
        function() {
            fn_done($dialog);
        }
    );
    util_page.rest_get('/tc/rest/bootimage2.php', function(result) {
        var options = $.map(result['images'], function(img) {
            return '<option value="' + img.id + '">' + img.name + '</option';
        });
        $dialog.find('select').append(options);
        fn_init($dialog);
    });
    $dialog.find('option:nth(1)').text(find_i18n('s_general_global'));
    return $dialog;
}

function page_load() {
    util_page.disable_cache();
    util_page.enable_locale();
    util_page.render_sidebar();

    load_groups();

    $('#toolbar_control button:nth(0)').click(function() {
        create_empty_details_dialog(create_group);
    });

    $('#toolbar_control button:nth(1)').click(function() {
        var $span = $('#client_group_list span');
        if ($span.hasClass('glyphicon-remove')) {
            load_groups();
            return;
        }
        if ($span.hasClass('glyphicon-edit')) {
            $('#client_group_list .badge').off('click');
        }
        $('#client_group_list .badge')
            .html('<span class="glyphicon glyphicon-remove"></span>')
            .click(function() {
                delete_group($(this).closest('a').data('cgid'));
            });
    });

    $('#toolbar_control button:nth(2)').click(function() {
        var $span = $('#client_group_list span');
        if ($span.hasClass('glyphicon-edit')) {
            load_groups();
            return;
        }
        if ($span.hasClass('glyphicon-remove')) {
            $('#client_group_list .badge').off('click');
        }
        $('#client_group_list .badge')
            .html('<span class="glyphicon glyphicon-edit"></span>')
            .click(function() {
                edit_group($(this).closest('a').data('cgid'));
            });
    });
}

$(document).ready(page_load);
</script>

<?php standard_page_end(); ?>