#!/bin/bash
# install tc server
thisscript=$0
thisscript=${thisscript##*/}
TOP_DIR=`pwd`
DL_DIR=$TOP_DIR/3rdParty
DL_TCSERVER_DIR=$DL_DIR/server
MODULE_DIR=$TOP_DIR/module
OUTPUT_DIR=$TOP_DIR/output
BUILD_DIR=$OUTPUT_DIR/build
TARGET_DIR=$OUTPUT_DIR/target
MODPS_DIR=$MODULE_DIR/ps
MOD_TCSERVER_DIR=$MODULE_DIR/tcserver
MOD_TCSERVER_DL_DIR=$MOD_TCSERVER_DIR/3rdparty
MOD_TCSERVER_REL_DIR=`basename $MODULE_DIR`/`basename $MOD_TCSERVER_DIR`

# commands
CP='/bin/cp -f'
RM='/bin/rm -f'
MV='/bin/mv -f'
MKDIR='/bin/mkdir -p'

if [ ! ${BASH_SOURCE[0]} -ef ./install.sh ] ; then
  echo ""
  echo "Run this script from the base of the installer directory."
  echo "For example:"
  echo "  cd /Path/To/Installer/Root"
  echo "  bash install.sh"
  exit 1
fi

###############################################################################
#
# address possible positional arguments
#
#"ab:c::"
shortopts="c"
#"a-long,b-long:,c-long::"
longopts="centos-update"
unset centos_update_flag

newargs=`getopt --options $shortopts --longoptions $longopts -n $thisscript -- "$@"`
if [ $? != 0 ] ; then
  echo "illegal arguments"
  exit 1
fi

eval set -- "$newargs"
while : ; do
  case "$1" in
    -c|--centos-update)
      # echo "centos-update"
      centos_update_flag=1
      shift
      ;;

    --)
      shift
      break;
      ;;

    *)
      echo "error parsing arguments"
      exit 1;
      ;;
  esac
done

###############################################################################
# get version string
TC_VERSION_FILE="$TOP_DIR/version"
TC_ISSUE_MAJOR=
TC_ISSUE_MINOR=

if [ ! -f "$TC_VERSION_FILE" ] ; then
  echo "$TC_VERSION_FILE: no such file"
  exit 2
else
  TC_ISSUE_MAJOR=`awk -F" *= *" '/major/{print $2}' $TC_VERSION_FILE | xargs`
  TC_ISSUE_MINOR=`awk -F" *= *" '/minor/{print $2}' $TC_VERSION_FILE | xargs`
fi

if [ -z "$TC_ISSUE_MAJOR" ] ; then
  echo "cannot check the version of current TC Installer"
  exit 2
fi

# set log info
LOG_FILE="${TOP_DIR}/install.${TC_ISSUE_MAJOR}.log"
LOG_MSG_TYPE='3'
LOG_STEP=17



###############################################################################
#source ./tcmsg.sh $LOG_FILE $LOG_MSG_TYPE #$LOG_STEP
#!/bin/sh
# =============================== tc log (begin) ===============================
# functions: control how messages are print on screen
# tcmsg_init()
# tcmsg_finish()
# tcmsg_exit()
# tcinfo()
# tcerr()
# tcwarn()
# step()
#
# a filename log would be written into
#TCMSG_LOG_FILE=$1
TCMSG_LOG_FILE=$LOG_FILE
# msg type
# 0: none
# 1: steps only
# 2: classified log messages only
# 3: both
#TCMSG_MSG_TYPE=$2
TCMSG_MSG_TYPE=$LOG_MSG_TYPE
# total steps of your task
#TCMSG_STEP=$3
TCMSG_STEP=

TCMSG_STEPN=0

tcmsg_init(){
  echo -e "\e[32m[INFO]\e[0m set log file to $TCMSG_LOG_FILE"
  echo -e "\e[32m[INFO]\e[0m set log file to $TCMSG_LOG_FILE" >$TCMSG_LOG_FILE
  exec 3>&1 4>&2
  exec >>$TCMSG_LOG_FILE 2>&1
}

tcmsg_finish(){
  echo -e "\e[32m[INFO]\e[0m save logs to file $TCMSG_LOG_FILE" >&3
  echo -e "\e[32m[INFO]\e[0m save logs to file $TCMSG_LOG_FILE"
  exec 1>&3 2>&4
  exec 3>&- 4>&-
}

#alias tcexit='tcmsg_finish && exit'
tcexit(){
  tcmsg_finish
  exit $1
}

tcinfo(){ # green
  if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -ge 2 ] ; then
    echo -e "\e[32m[INFO]\e[0m $*" >&3
  fi
  echo -e "\e[32m[INFO]\e[0m $*" 
}

tcerr(){ # red
  if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -ge 2 ] ; then
    echo -e "\e[31m[ERROR]\e[0m $*" >&3
  fi
  echo -e "\e[31m[ERROR]\e[0m $*"
}

tcwarn(){ # magenta
  if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -ge 2 ] ; then
    echo -e "\e[35m[WARNING]\e[0m $*" >&3
  fi
  echo -e "\e[35m[WARNING]\e[0m $*"
}

step(){
  local s
  TCMSG_STEPN=$(($TCMSG_STEPN + 1))
  if [ -z "${TCMSG_STEP}" ] || [ -n "${TCMSG_STEP//[0-9]/}" -a ! -z "$TCMSG_STEP" ] ; then
    s=$(printf "%02u" $TCMSG_STEPN)
    if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -eq 1 -o $TCMSG_MSG_TYPE -eq 3 ] ; then
      echo -e "[$s] $*" >&3
    fi
    echo -e "[$s] $*"
  else
    s=$(echo "$TCMSG_STEPN % (1 + $TCMSG_STEP)" | bc)
    s=$(printf "%0${#TCMSG_STEP}u" $s)
    if [ -n "$TCMSG_MSG_TYPE" ] && [ $TCMSG_MSG_TYPE -eq 1 -o $TCMSG_MSG_TYPE -eq 3 ] ; then
      echo -e "[$s/$TCMSG_STEP] $*" >&3
    fi
    echo -e "[$s/$TCMSG_STEP] $*"
  fi
}
# ================================ tc log (end) ================================
# tcmsg init
tcmsg_init

# Only support root user permissions
if [ "$(id -u)" -ne 0 ]; then
  tcerr "Please install TC server as root user"
  tcexit 1
fi

###############################################################################
$RM -r $MOD_TCSERVER_DL_DIR
$CP -a $DL_TCSERVER_DIR $MOD_TCSERVER_DL_DIR

###############################################################################
# check old version

TC_ISSUE_DIR='/etc/tc'
TC_ISSUE_FILE=$TC_ISSUE_DIR'/issue'
TC_ISSUE_FILE_EX='/home/tc/etc/issue'
TC_INI_FILE=/home/tc/etc/tc.ini
TC_ENABLE_8801=TRUE
TC_ISSUE_MAJOR_OLD=
TC_ISSUE_MINOR_OLD=
install_choice=

check_env(){
  if [ -f "$TC_ISSUE_FILE_EX" ] ; then
    TC_ISSUE_MAJOR_OLD=`awk -F" *= *" '/major/{print $2}' $TC_ISSUE_FILE_EX | xargs`
    TC_ISSUE_MINOR_OLD=`awk -F" *= *" '/minor/{print $2}' $TC_ISSUE_FILE_EX | xargs`
  elif [ -f "$TC_ISSUE_FILE" ] ; then
    TC_ISSUE_MAJOR_OLD=`awk -F" *= *" '/major/{print $2}' $TC_ISSUE_FILE | xargs`
    TC_ISSUE_MINOR_OLD=`awk -F" *= *" '/minor/{print $2}' $TC_ISSUE_FILE | xargs`
  fi
}

check_revision(){
  #
  # special case
  # support upgrade from specified revision
  #
  local tcPrevRev=$1
  if [ -z "$TC_ISSUE_MINOR_OLD" -a -f "$TC_INI_FILE" ] ; then
    TC_ISSUE_MINOR_OLD=`awk -F" *= *" '/build_revision/{print $2}' $TC_INI_FILE | xargs`
    if [ "x$TC_ISSUE_MINOR_OLD" == "x$tcPrevRev" ] ; then
      #
      # set both of the items
      #
      TC_ISSUE_MAJOR_OLD=1902
      TC_ISSUE_MINOR_OLD=$tcPrevRev
    else
      #
      # empty both of the items
      #
      TC_ISSUE_MAJOR_OLD=
      TC_ISSUE_MINOR_OLD=
    fi
  fi
}

tcinfo "check tc installation environment..."
check_env
if [ "x$TC_ENABLE_8801" == "xTRUE" ] ; then
  check_revision "8801"
fi

if [ -n "$TC_ISSUE_MAJOR_OLD" -a -n "$TC_ISSUE_MINOR_OLD" ] ; then
  tcinfo "Detected early installed version ${TC_ISSUE_MAJOR_OLD}.${TC_ISSUE_MINOR_OLD} in this system."
  install_by_choice=TRUE
fi

###############################################################################
INTERNAL_INST_SCRIPT=
install_op(){
  echo "" >&3
  echo "======================================================================" >&3 
  echo "Please select:" >&3 >&4
  select ch in "upgrade install" "clean install" ; do
    case $REPLY in
      1) 
        INTERNAL_INST_SCRIPT="server_upgrade.sh"
        break
        ;;
      2)
        INTERNAL_INST_SCRIPT="install"
        break
        ;;
      *)
        echo "$REPLY : no such choice."
        exit 3
        ;;
    esac
  done >&3 2>&4
}

#
# pop up menu and let use to make a choice
#
if [ "x$install_by_choice" == "xTRUE" ] ; then
  install_op
else
  if [ -z "$INTERNAL_INST_SCRIPT" ] ; then
    INTERNAL_INST_SCRIPT="install"
  fi
fi

#
# if the sub routine is not set, exit
#
if [ -z "$INTERNAL_INST_SCRIPT" ] ; then
  tcerr "script should not reach here."
  tcexit 4
fi

tcinfo "script: $INTERNAL_INST_SCRIPT"

MOD_TCSERVER_REL_INSTALL_SH=$MOD_TCSERVER_REL_DIR/${INTERNAL_INST_SCRIPT}

export INSTALL_LOGFILE=${LOG_FILE}

tcinfo "enter directory $MOD_TCSERVER_REL_DIR"
cd $MOD_TCSERVER_DIR

tcinfo "run script $MOD_TCSERVER_REL_INSTALL_SH"
chmod a+x $INTERNAL_INST_SCRIPT
./$INTERNAL_INST_SCRIPT >&3 2>&4
INSTALL_RET=$?

tcinfo "exitCode = $INSTALL_RET"
if [ $INSTALL_RET -eq 0 ] ; then
  tcinfo "successfully run script $MOD_TCSERVER_REL_INSTALL_SH"
  tcinfo "write version string to $TC_ISSUE_FILE"
  if ! [ -d $TC_ISSUE_DIR ] ; then
    $MKDIR $TC_ISSUE_DIR
    if ! [ $? -eq 0 ] ; then
      tcerr "$TC_ISSUE_DIR: fail to create directory"
      tcexit 1
    fi
  fi
  $CP $TC_VERSION_FILE $TC_ISSUE_FILE
  $CP $TC_VERSION_FILE $TC_ISSUE_FILE_EX

else
  tcerr "fail to run script $MOD_TCSERVER_REL_INSTALL_SH"
fi

cd $TOP_DIR


if ! [ $INSTALL_RET -eq 0 ] ; then
    tcexit 1
fi

###############################################################################
# loaddata
LOAD_DATA_SCRIPT=$MODPS_DIR/loaddata.sh
if [ -f $LOAD_DATA_SCRIPT ] ; then
  tcinfo "load data files for TC"
  cd $MODPS_DIR && bash $LOAD_DATA_SCRIPT
else
  tcwarn "fail to load data for TC"
fi

###############################################################################
# kernel updgrade
function centos_update(){
  tcinfo "enter directory $MOD_TCSERVER_REL_DIR"
  cd $MOD_TCSERVER_DIR
  KRN957_SCRIPT=centos_update.sh
  bash $KRN957_SCRIPT --interactive >&3 2>&4
  INSTALL_RET=$?
  tcinfo "script $KRN957_SCRIPT exit code: $INSTALL_RET"
  if [ $INSTALL_RET -eq 0 ] ; then
  tcinfo "Reboot system to make kernel update take effect"
  #reboot
  elif [ $INSTALL_RET -eq 1 ] ; then 
    :
  else
    tcerr "fail to run script $KRN957_SCRIPT: $INSTALL_RET" 
    tcexit 1
  fi
  cd $TOP_DIR
}

# make this option as a required one
centos_update_flag=1
if [ "x$centos_update_flag" == "x1" ] ; then
  centos_update
fi

###############################################################################
# reboot

tcwarn "waiting for system reboot to finish TC system installation" 
for pc in $(seq 10 -1 1) ; do
  echo -ne "\rReboot in $pc seconds, press any key to cancel ...\r" >&3
  read -t 1 -n 1 -s key
  if [ $? -eq 0 ] ; then
      break
  fi

  if [ $pc -eq 1 ] ; then
    tcmsg_finish
    reboot
    echo "reboot" >&3
  fi
done

###############################################################################

# tcmsg
tcmsg_finish

